var exports = (function (exports) {
  'use strict';

  var offerToDownloadPlugin = {
  	id: "jgclark.WindowTools",
  	minVersion: "1.0.0"
  };
  var commandMigrationMessage = "Note: the 'open ... note in new ...' commands have been migrated to the WindowTools plugin.";
  var pluginJson$4 = {
  	"noteplan.minAppVersion": "3.0.23",
  	"macOS.minVersion": "10.13.0",
  	"plugin.id": "jgclark.NoteHelpers",
  	"plugin.name": "📙 Note Helpers",
  	"plugin.description": "Commands to quickly jump around and manage notes",
  	"plugin.author": "Jonathan Clark & Eduard Metzger",
  	"plugin.url": "https://github.com/NotePlan/plugins/tree/main/jgclark.NoteHelpers/",
  	"plugin.changelog": "https://github.com/NotePlan/plugins/blob/main/jgclark.NoteHelpers/CHANGELOG.md",
  	"plugin.version": "1.0.0",
  	"plugin.lastUpdateInfo": "1.0.0: move '/new note from ...' commands here from Filer plugin, and revived '/new note' command.\n0.20.3: New 'printEditorDetailed' command to log all paragraph details as well.\n0.20.2: 'printNote' extended to cover backlinks.\n0.20.1: new command 'printNote' for debugging purposes.\n0.20.0: new commands \"unlinked note finder\" and \"delete note\". Bug fix to \"rename note filename\" command.\n0.19.2: fix edge cases with \"add trigger to note\".\n0.19.1: \"add trigger to note\" command can now be run from template. Added migration message about 'open note' commands.\n0.19.0: fix to '/rename inconsistent filename' command. Moved 'open note' functions to WindowTools plugin. Updated the display of the 'index folders' command. Removed 'Show month/quarter/year' commands as they are now in the main NP menus.",
  	"plugin.dependencies": [
  ],
  	"plugin.script": "script.js",
  	"plugin.commands": [
  	{
  		name: "add number of days to dates",
  		alias: [
  			"count",
  			"add",
  			"days"
  		],
  		description: "Look for bullets mentioning dates and add number of days till that date",
  		jsFunction: "countAndAddDays"
  	},
  	{
  		name: "add trigger to note",
  		alias: [
  			"trigger",
  			"add"
  		],
  		description: "Select from a list of available triggers to add to the current note",
  		jsFunction: "addTriggerToNote",
  		parameters: [
  			"trigger string (e.g. 'onEditorWillSave => jgclark.DashboardReact.decideWhetherToUpdateDashboard')"
  		]
  	},
  	{
  		name: "convert to frontmatter",
  		alias: [
  			"frontmatter"
  		],
  		description: "Convert the current note to use frontmatter syntax, including some default text that can be added in the Plugin's settings",
  		jsFunction: "addFrontmatterToNote"
  	},
  	{
  		name: "delete note",
  		alias: [
  			"dn"
  		],
  		description: "Delete the current note (moves to Trash)",
  		jsFunction: "trashNote",
  		parameters: [
  		]
  	},
  	{
  		name: "enable heading links",
  		alias: [
  			"local links",
  			"hash links",
  			"links to headings"
  		],
  		description: "Look for Links to headings and make them work by converting them to plugin command calls",
  		jsFunction: "convertLocalLinksToPluginLinks"
  	},
  	{
  		name: "find and link unlinked notes in current note",
  		alias: [
  			"unlinked"
  		],
  		description: "Find and create links to unlinked notes",
  		jsFunction: "findUnlinkedNotesInCurrentNote"
  	},
  	{
  		name: "find and link unlinked notes in all notes",
  		alias: [
  			"allunlinked"
  		],
  		description: "Find and create links to all unlinked notes",
  		jsFunction: "findUnlinkedNotesInAllNotes"
  	},
  	{
  		name: "index folders",
  		alias: [
  			"index"
  		],
  		description: "Make/Update indexes for all notes in a folder (and sub-folders if wanted)",
  		jsFunction: "indexFolders",
  		parameters: [
  			"folder",
  			"string: displayOrder=(key);dateDisplayType=(key);includeSubfolders=(key)"
  		]
  	},
  	{
  		name: "jump to heading",
  		alias: [
  			"jh",
  			"jump",
  			"heading"
  		],
  		description: "Jumps to the heading that the user selects. (Currently only works in main window)",
  		jsFunction: "jumpToHeading",
  		parameters: [
  			"heading"
  		]
  	},
  	{
  		name: "jump to note's heading",
  		alias: [
  			"jn",
  			"jump",
  			"note"
  		],
  		description: "Jump to a different note, and then selected heading. (Currently only works in main window)",
  		jsFunction: "jumpToNoteHeading"
  	},
  	{
  		name: "jump to done",
  		alias: [
  			"jd",
  			"jump",
  			"done"
  		],
  		description: "Jump to the '## Done' section. (Currently only works in main window)",
  		jsFunction: "jumpToDone"
  	},
  	{
  		name: "move note",
  		alias: [
  			"mn",
  			"move",
  			"note"
  		],
  		description: "Moves the currently opened (non-calendar) note to a folder you select",
  		jsFunction: "moveNote"
  	},
  	{
  		name: "new note",
  		alias: [
  			"nn"
  		],
  		description: "Make New note with choice of folder",
  		jsFunction: "newNote"
  	},
  	{
  		name: "new note from clipboard",
  		alias: [
  			"nnc",
  			"new"
  		],
  		description: "New note from clipboard",
  		jsFunction: "newNoteFromClipboard"
  	},
  	{
  		name: "new note from selection",
  		alias: [
  			"nns",
  			"new"
  		],
  		description: "New note from selection (and leave link to it in its place)",
  		jsFunction: "newNoteFromSelection"
  	},
  	{
  		name: "open URL from a note",
  		alias: [
  			"oun",
  			"open",
  			"URL"
  		],
  		description: "Open a chosen URL from a chosen note",
  		jsFunction: "openURLFromANote"
  	},
  	{
  		name: "reset caches",
  		alias: [
  			"reset",
  			"cache"
  		],
  		description: "Reset NotePlan caches",
  		jsFunction: "resetCaches"
  	},
  	{
  		name: "list inconsistent note filenames",
  		alias: [
  			"listInconsistentNames"
  		],
  		description: "Lists the names of notes whose filenames are inconsistent with their titles.",
  		jsFunction: "listInconsistentNames"
  	},
  	{
  		name: "rename filename to title",
  		alias: [
  			"titleToFilename"
  		],
  		description: "Renames the current filename to the title of the note.",
  		jsFunction: "titleToFilename"
  	},
  	{
  		name: "rename note filename",
  		alias: [
  			"rename"
  		],
  		description: "Rename the current note's filename to one you specify.",
  		jsFunction: "renameNoteFile"
  	},
  	{
  		name: "reset title to match filename",
  		alias: [
  			"filenameToTitle"
  		],
  		description: "Resets the current note title to match its filename.",
  		jsFunction: "filenameToTitle"
  	},
  	{
  		name: "rename inconsistent note filenames",
  		alias: [
  			"renameInconsistentNames"
  		],
  		description: "Renames the files of notes whose filenames are inconsistent with their titles.",
  		jsFunction: "renameInconsistentNames"
  	},
  	{
  		name: "update all indexes",
  		alias: [
  			"uai",
  			"index"
  		],
  		description: "Update all folder index notes",
  		jsFunction: "updateAllIndexes"
  	},
  	{
  		name: "NoteHelpers: update plugin settings",
  		description: "Settings interface (even for iOS)",
  		jsFunction: "updateSettings"
  	},
  	{
  		name: "log note details",
  		alias: [
  			"logNote",
  			"lnd"
  		],
  		description: "Log main details about note to console",
  		jsFunction: "printNote"
  	},
  	{
  		name: "logEditorNoteDetailed",
  		alias: [
  			"lend"
  		],
  		description: "Log full details about current Editor note to console",
  		jsFunction: "logEditorNoteDetailed"
  	},
  	{
  		name: "triggerFindUnlinkedNotes",
  		description: "onEditorWillSave",
  		jsFunction: "triggerFindUnlinkedNotes",
  		hidden: true
  	}
  ],
  	offerToDownloadPlugin: offerToDownloadPlugin,
  	commandMigrationMessage: commandMigrationMessage,
  	"plugin.disabled_commands": [
  	{
  		name: "Show This Month",
  		alias: [
  			"stm",
  			"month"
  		],
  		description: "Open the current Month note",
  		jsFunction: "showMonth"
  	},
  	{
  		name: "Show This Quarter",
  		alias: [
  			"stq",
  			"quarter"
  		],
  		description: "Open the current Quarter note",
  		jsFunction: "showQuarter"
  	},
  	{
  		name: "Show This Year",
  		alias: [
  			"sty",
  			"year"
  		],
  		description: "Open the current Year note",
  		jsFunction: "showYear"
  	}
  ],
  	"plugin.settings": [
  	{
  		type: "separator"
  	},
  	{
  		type: "heading",
  		title: "'convert to frontmatter' command setting"
  	},
  	{
  		key: "defaultFMText",
  		title: "Default Text to add to frontmatter",
  		description: "The standard text to add after the title in the frontmatter. Can include line breaks by typing return.",
  		type: "string",
  		"default": "",
  		required: false
  	},
  	{
  		type: "separator"
  	},
  	{
  		type: "heading",
  		title: "'indexFolders' command settings"
  	},
  	{
  		key: "displayOrder",
  		title: "Sort order for index items",
  		description: "Whether index entries are sorted alphabetically by title (the default), by created date, or by last updated date",
  		type: "string",
  		choices: [
  			"alphabetical",
  			"createdDate",
  			"updatedDate"
  		],
  		"default": "alphabetical",
  		required: true
  	},
  	{
  		key: "dateDisplayType",
  		title: "What type of date suffix to add?",
  		description: "What type of date/time period to add to the end of note links in the index. 'timeSince' is time since the note was last updated; 'updatedDate' shows the date the note was last updated.",
  		type: "string",
  		choices: [
  			"none",
  			"timeSince",
  			"updatedDate"
  		],
  		"default": "none",
  		required: true
  	},
  	{
  		key: "includeSubfolders",
  		title: "Include Subfolders when making an index?",
  		description: "If set, then all subfolders will be indexed in the same name as the folder.",
  		type: "bool",
  		"default": true,
  		required: true
  	},
  	{
  		key: "indexTitle",
  		title: "Title to use for Index notes",
  		description: "This can include a placeholder `{{folder}}` or `{{full_folder_path}}` which will be replaced by the folder's name or full path.",
  		type: "string",
  		"default": "_{{folder}} index",
  		required: true
  	},
  	{
  		type: "separator"
  	},
  	{
  		type: "heading",
  		title: "'open URL from a note' command setting"
  	},
  	{
  		key: "ignoreCompletedItems",
  		title: "Ignore URLs in completed items?",
  		description: "Whether to ignore URLs found in completed (or cancelled) tasks or checklists.",
  		type: "bool",
  		"default": true,
  		required: true
  	},
  	{
  		type: "separator"
  	},
  	{
  		type: "heading",
  		title: "Debugging"
  	},
  	{
  		key: "_logLevel",
  		title: "Log Level",
  		description: "Set how much output will be displayed for this plugin in the NotePlan > Help > Plugin Console. DEBUG is the most verbose; NONE is the least (silent).",
  		type: "string",
  		choices: [
  			"DEBUG",
  			"INFO",
  			"WARN",
  			"ERROR",
  			"none"
  		],
  		"default": "INFO",
  		required: true
  	}
  ]
  };

  var commonjsGlobal = typeof globalThis !== 'undefined' ? globalThis : typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

  function getDefaultExportFromCjs (x) {
  	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
  }

  function commonjsRequire(path) {
  	throw new Error('Could not dynamically require "' + path + '". Please configure the dynamicRequireTargets or/and ignoreDynamicRequires option of @rollup/plugin-commonjs appropriately for this require call to work.');
  }

  var moment$3 = {exports: {}};

  var moment$2 = moment$3.exports;

  var hasRequiredMoment;

  function requireMoment () {
  	if (hasRequiredMoment) return moment$3.exports;
  	hasRequiredMoment = 1;
  	(function (module, exports) {
  		(function (global, factory) {
  		  module.exports = factory() ;
  		})(moment$2, function () {

  		  var hookCallback;
  		  function hooks() {
  		    return hookCallback.apply(null, arguments);
  		  }

  		  // This is done to register the method called with moment()
  		  // without creating circular dependencies.
  		  function setHookCallback(callback) {
  		    hookCallback = callback;
  		  }
  		  function isArray(input) {
  		    return input instanceof Array || Object.prototype.toString.call(input) === '[object Array]';
  		  }
  		  function isObject(input) {
  		    // IE8 will treat undefined and null as object if it wasn't for
  		    // input != null
  		    return input != null && Object.prototype.toString.call(input) === '[object Object]';
  		  }
  		  function hasOwnProp(a, b) {
  		    return Object.prototype.hasOwnProperty.call(a, b);
  		  }
  		  function isObjectEmpty(obj) {
  		    if (Object.getOwnPropertyNames) {
  		      return Object.getOwnPropertyNames(obj).length === 0;
  		    } else {
  		      var k;
  		      for (k in obj) {
  		        if (hasOwnProp(obj, k)) {
  		          return false;
  		        }
  		      }
  		      return true;
  		    }
  		  }
  		  function isUndefined(input) {
  		    return input === void 0;
  		  }
  		  function isNumber(input) {
  		    return typeof input === 'number' || Object.prototype.toString.call(input) === '[object Number]';
  		  }
  		  function isDate(input) {
  		    return input instanceof Date || Object.prototype.toString.call(input) === '[object Date]';
  		  }
  		  function map(arr, fn) {
  		    var res = [],
  		      i,
  		      arrLen = arr.length;
  		    for (i = 0; i < arrLen; ++i) {
  		      res.push(fn(arr[i], i));
  		    }
  		    return res;
  		  }
  		  function extend(a, b) {
  		    for (var i in b) {
  		      if (hasOwnProp(b, i)) {
  		        a[i] = b[i];
  		      }
  		    }
  		    if (hasOwnProp(b, 'toString')) {
  		      a.toString = b.toString;
  		    }
  		    if (hasOwnProp(b, 'valueOf')) {
  		      a.valueOf = b.valueOf;
  		    }
  		    return a;
  		  }
  		  function createUTC(input, format, locale, strict) {
  		    return createLocalOrUTC(input, format, locale, strict, true).utc();
  		  }
  		  function defaultParsingFlags() {
  		    // We need to deep clone this object.
  		    return {
  		      empty: false,
  		      unusedTokens: [],
  		      unusedInput: [],
  		      overflow: -2,
  		      charsLeftOver: 0,
  		      nullInput: false,
  		      invalidEra: null,
  		      invalidMonth: null,
  		      invalidFormat: false,
  		      userInvalidated: false,
  		      iso: false,
  		      parsedDateParts: [],
  		      era: null,
  		      meridiem: null,
  		      rfc2822: false,
  		      weekdayMismatch: false
  		    };
  		  }
  		  function getParsingFlags(m) {
  		    if (m._pf == null) {
  		      m._pf = defaultParsingFlags();
  		    }
  		    return m._pf;
  		  }
  		  var some;
  		  if (Array.prototype.some) {
  		    some = Array.prototype.some;
  		  } else {
  		    some = function (fun) {
  		      var t = Object(this),
  		        len = t.length >>> 0,
  		        i;
  		      for (i = 0; i < len; i++) {
  		        if (i in t && fun.call(this, t[i], i, t)) {
  		          return true;
  		        }
  		      }
  		      return false;
  		    };
  		  }
  		  function isValid(m) {
  		    var flags = null,
  		      parsedParts = false,
  		      isNowValid = m._d && !isNaN(m._d.getTime());
  		    if (isNowValid) {
  		      flags = getParsingFlags(m);
  		      parsedParts = some.call(flags.parsedDateParts, function (i) {
  		        return i != null;
  		      });
  		      isNowValid = flags.overflow < 0 && !flags.empty && !flags.invalidEra && !flags.invalidMonth && !flags.invalidWeekday && !flags.weekdayMismatch && !flags.nullInput && !flags.invalidFormat && !flags.userInvalidated && (!flags.meridiem || flags.meridiem && parsedParts);
  		      if (m._strict) {
  		        isNowValid = isNowValid && flags.charsLeftOver === 0 && flags.unusedTokens.length === 0 && flags.bigHour === undefined;
  		      }
  		    }
  		    if (Object.isFrozen == null || !Object.isFrozen(m)) {
  		      m._isValid = isNowValid;
  		    } else {
  		      return isNowValid;
  		    }
  		    return m._isValid;
  		  }
  		  function createInvalid(flags) {
  		    var m = createUTC(NaN);
  		    if (flags != null) {
  		      extend(getParsingFlags(m), flags);
  		    } else {
  		      getParsingFlags(m).userInvalidated = true;
  		    }
  		    return m;
  		  }

  		  // Plugins that add properties should also add the key here (null value),
  		  // so we can properly clone ourselves.
  		  var momentProperties = hooks.momentProperties = [],
  		    updateInProgress = false;
  		  function copyConfig(to, from) {
  		    var i,
  		      prop,
  		      val,
  		      momentPropertiesLen = momentProperties.length;
  		    if (!isUndefined(from._isAMomentObject)) {
  		      to._isAMomentObject = from._isAMomentObject;
  		    }
  		    if (!isUndefined(from._i)) {
  		      to._i = from._i;
  		    }
  		    if (!isUndefined(from._f)) {
  		      to._f = from._f;
  		    }
  		    if (!isUndefined(from._l)) {
  		      to._l = from._l;
  		    }
  		    if (!isUndefined(from._strict)) {
  		      to._strict = from._strict;
  		    }
  		    if (!isUndefined(from._tzm)) {
  		      to._tzm = from._tzm;
  		    }
  		    if (!isUndefined(from._isUTC)) {
  		      to._isUTC = from._isUTC;
  		    }
  		    if (!isUndefined(from._offset)) {
  		      to._offset = from._offset;
  		    }
  		    if (!isUndefined(from._pf)) {
  		      to._pf = getParsingFlags(from);
  		    }
  		    if (!isUndefined(from._locale)) {
  		      to._locale = from._locale;
  		    }
  		    if (momentPropertiesLen > 0) {
  		      for (i = 0; i < momentPropertiesLen; i++) {
  		        prop = momentProperties[i];
  		        val = from[prop];
  		        if (!isUndefined(val)) {
  		          to[prop] = val;
  		        }
  		      }
  		    }
  		    return to;
  		  }

  		  // Moment prototype object
  		  function Moment(config) {
  		    copyConfig(this, config);
  		    this._d = new Date(config._d != null ? config._d.getTime() : NaN);
  		    if (!this.isValid()) {
  		      this._d = new Date(NaN);
  		    }
  		    // Prevent infinite loop in case updateOffset creates new moment
  		    // objects.
  		    if (updateInProgress === false) {
  		      updateInProgress = true;
  		      hooks.updateOffset(this);
  		      updateInProgress = false;
  		    }
  		  }
  		  function isMoment(obj) {
  		    return obj instanceof Moment || obj != null && obj._isAMomentObject != null;
  		  }
  		  function warn(msg) {
  		    if (hooks.suppressDeprecationWarnings === false && typeof console !== 'undefined' && console.warn) {
  		      console.warn('Deprecation warning: ' + msg);
  		    }
  		  }
  		  function deprecate(msg, fn) {
  		    var firstTime = true;
  		    return extend(function () {
  		      if (hooks.deprecationHandler != null) {
  		        hooks.deprecationHandler(null, msg);
  		      }
  		      if (firstTime) {
  		        var args = [],
  		          arg,
  		          i,
  		          key,
  		          argLen = arguments.length;
  		        for (i = 0; i < argLen; i++) {
  		          arg = '';
  		          if (typeof arguments[i] === 'object') {
  		            arg += '\n[' + i + '] ';
  		            for (key in arguments[0]) {
  		              if (hasOwnProp(arguments[0], key)) {
  		                arg += key + ': ' + arguments[0][key] + ', ';
  		              }
  		            }
  		            arg = arg.slice(0, -2); // Remove trailing comma and space
  		          } else {
  		            arg = arguments[i];
  		          }
  		          args.push(arg);
  		        }
  		        warn(msg + '\nArguments: ' + Array.prototype.slice.call(args).join('') + '\n' + new Error().stack);
  		        firstTime = false;
  		      }
  		      return fn.apply(this, arguments);
  		    }, fn);
  		  }
  		  var deprecations = {};
  		  function deprecateSimple(name, msg) {
  		    if (hooks.deprecationHandler != null) {
  		      hooks.deprecationHandler(name, msg);
  		    }
  		    if (!deprecations[name]) {
  		      warn(msg);
  		      deprecations[name] = true;
  		    }
  		  }
  		  hooks.suppressDeprecationWarnings = false;
  		  hooks.deprecationHandler = null;
  		  function isFunction(input) {
  		    return typeof Function !== 'undefined' && input instanceof Function || Object.prototype.toString.call(input) === '[object Function]';
  		  }
  		  function set(config) {
  		    var prop, i;
  		    for (i in config) {
  		      if (hasOwnProp(config, i)) {
  		        prop = config[i];
  		        if (isFunction(prop)) {
  		          this[i] = prop;
  		        } else {
  		          this['_' + i] = prop;
  		        }
  		      }
  		    }
  		    this._config = config;
  		    // Lenient ordinal parsing accepts just a number in addition to
  		    // number + (possibly) stuff coming from _dayOfMonthOrdinalParse.
  		    // TODO: Remove "ordinalParse" fallback in next major release.
  		    this._dayOfMonthOrdinalParseLenient = new RegExp((this._dayOfMonthOrdinalParse.source || this._ordinalParse.source) + '|' + /\d{1,2}/.source);
  		  }
  		  function mergeConfigs(parentConfig, childConfig) {
  		    var res = extend({}, parentConfig),
  		      prop;
  		    for (prop in childConfig) {
  		      if (hasOwnProp(childConfig, prop)) {
  		        if (isObject(parentConfig[prop]) && isObject(childConfig[prop])) {
  		          res[prop] = {};
  		          extend(res[prop], parentConfig[prop]);
  		          extend(res[prop], childConfig[prop]);
  		        } else if (childConfig[prop] != null) {
  		          res[prop] = childConfig[prop];
  		        } else {
  		          delete res[prop];
  		        }
  		      }
  		    }
  		    for (prop in parentConfig) {
  		      if (hasOwnProp(parentConfig, prop) && !hasOwnProp(childConfig, prop) && isObject(parentConfig[prop])) {
  		        // make sure changes to properties don't modify parent config
  		        res[prop] = extend({}, res[prop]);
  		      }
  		    }
  		    return res;
  		  }
  		  function Locale(config) {
  		    if (config != null) {
  		      this.set(config);
  		    }
  		  }
  		  var keys;
  		  if (Object.keys) {
  		    keys = Object.keys;
  		  } else {
  		    keys = function (obj) {
  		      var i,
  		        res = [];
  		      for (i in obj) {
  		        if (hasOwnProp(obj, i)) {
  		          res.push(i);
  		        }
  		      }
  		      return res;
  		    };
  		  }
  		  var defaultCalendar = {
  		    sameDay: '[Today at] LT',
  		    nextDay: '[Tomorrow at] LT',
  		    nextWeek: 'dddd [at] LT',
  		    lastDay: '[Yesterday at] LT',
  		    lastWeek: '[Last] dddd [at] LT',
  		    sameElse: 'L'
  		  };
  		  function calendar(key, mom, now) {
  		    var output = this._calendar[key] || this._calendar['sameElse'];
  		    return isFunction(output) ? output.call(mom, now) : output;
  		  }
  		  function zeroFill(number, targetLength, forceSign) {
  		    var absNumber = '' + Math.abs(number),
  		      zerosToFill = targetLength - absNumber.length,
  		      sign = number >= 0;
  		    return (sign ? forceSign ? '+' : '' : '-') + Math.pow(10, Math.max(0, zerosToFill)).toString().substr(1) + absNumber;
  		  }
  		  var formattingTokens = /(\[[^\[]*\])|(\\)?([Hh]mm(ss)?|Mo|MM?M?M?|Do|DDDo|DD?D?D?|ddd?d?|do?|w[o|w]?|W[o|W]?|Qo?|N{1,5}|YYYYYY|YYYYY|YYYY|YY|y{2,4}|yo?|gg(ggg?)?|GG(GGG?)?|e|E|a|A|hh?|HH?|kk?|mm?|ss?|S{1,9}|x|X|zz?|ZZ?|.)/g,
  		    localFormattingTokens = /(\[[^\[]*\])|(\\)?(LTS|LT|LL?L?L?|l{1,4})/g,
  		    formatFunctions = {},
  		    formatTokenFunctions = {};

  		  // token:    'M'
  		  // padded:   ['MM', 2]
  		  // ordinal:  'Mo'
  		  // callback: function () { this.month() + 1 }
  		  function addFormatToken(token, padded, ordinal, callback) {
  		    var func = callback;
  		    if (typeof callback === 'string') {
  		      func = function () {
  		        return this[callback]();
  		      };
  		    }
  		    if (token) {
  		      formatTokenFunctions[token] = func;
  		    }
  		    if (padded) {
  		      formatTokenFunctions[padded[0]] = function () {
  		        return zeroFill(func.apply(this, arguments), padded[1], padded[2]);
  		      };
  		    }
  		    if (ordinal) {
  		      formatTokenFunctions[ordinal] = function () {
  		        return this.localeData().ordinal(func.apply(this, arguments), token);
  		      };
  		    }
  		  }
  		  function removeFormattingTokens(input) {
  		    if (input.match(/\[[\s\S]/)) {
  		      return input.replace(/^\[|\]$/g, '');
  		    }
  		    return input.replace(/\\/g, '');
  		  }
  		  function makeFormatFunction(format) {
  		    var array = format.match(formattingTokens),
  		      i,
  		      length;
  		    for (i = 0, length = array.length; i < length; i++) {
  		      if (formatTokenFunctions[array[i]]) {
  		        array[i] = formatTokenFunctions[array[i]];
  		      } else {
  		        array[i] = removeFormattingTokens(array[i]);
  		      }
  		    }
  		    return function (mom) {
  		      var output = '',
  		        i;
  		      for (i = 0; i < length; i++) {
  		        output += isFunction(array[i]) ? array[i].call(mom, format) : array[i];
  		      }
  		      return output;
  		    };
  		  }

  		  // format date using native date object
  		  function formatMoment(m, format) {
  		    if (!m.isValid()) {
  		      return m.localeData().invalidDate();
  		    }
  		    format = expandFormat(format, m.localeData());
  		    formatFunctions[format] = formatFunctions[format] || makeFormatFunction(format);
  		    return formatFunctions[format](m);
  		  }
  		  function expandFormat(format, locale) {
  		    var i = 5;
  		    function replaceLongDateFormatTokens(input) {
  		      return locale.longDateFormat(input) || input;
  		    }
  		    localFormattingTokens.lastIndex = 0;
  		    while (i >= 0 && localFormattingTokens.test(format)) {
  		      format = format.replace(localFormattingTokens, replaceLongDateFormatTokens);
  		      localFormattingTokens.lastIndex = 0;
  		      i -= 1;
  		    }
  		    return format;
  		  }
  		  var defaultLongDateFormat = {
  		    LTS: 'h:mm:ss A',
  		    LT: 'h:mm A',
  		    L: 'MM/DD/YYYY',
  		    LL: 'MMMM D, YYYY',
  		    LLL: 'MMMM D, YYYY h:mm A',
  		    LLLL: 'dddd, MMMM D, YYYY h:mm A'
  		  };
  		  function longDateFormat(key) {
  		    var format = this._longDateFormat[key],
  		      formatUpper = this._longDateFormat[key.toUpperCase()];
  		    if (format || !formatUpper) {
  		      return format;
  		    }
  		    this._longDateFormat[key] = formatUpper.match(formattingTokens).map(function (tok) {
  		      if (tok === 'MMMM' || tok === 'MM' || tok === 'DD' || tok === 'dddd') {
  		        return tok.slice(1);
  		      }
  		      return tok;
  		    }).join('');
  		    return this._longDateFormat[key];
  		  }
  		  var defaultInvalidDate = 'Invalid date';
  		  function invalidDate() {
  		    return this._invalidDate;
  		  }
  		  var defaultOrdinal = '%d',
  		    defaultDayOfMonthOrdinalParse = /\d{1,2}/;
  		  function ordinal(number) {
  		    return this._ordinal.replace('%d', number);
  		  }
  		  var defaultRelativeTime = {
  		    future: 'in %s',
  		    past: '%s ago',
  		    s: 'a few seconds',
  		    ss: '%d seconds',
  		    m: 'a minute',
  		    mm: '%d minutes',
  		    h: 'an hour',
  		    hh: '%d hours',
  		    d: 'a day',
  		    dd: '%d days',
  		    w: 'a week',
  		    ww: '%d weeks',
  		    M: 'a month',
  		    MM: '%d months',
  		    y: 'a year',
  		    yy: '%d years'
  		  };
  		  function relativeTime(number, withoutSuffix, string, isFuture) {
  		    var output = this._relativeTime[string];
  		    return isFunction(output) ? output(number, withoutSuffix, string, isFuture) : output.replace(/%d/i, number);
  		  }
  		  function pastFuture(diff, output) {
  		    var format = this._relativeTime[diff > 0 ? 'future' : 'past'];
  		    return isFunction(format) ? format(output) : format.replace(/%s/i, output);
  		  }
  		  var aliases = {
  		    D: 'date',
  		    dates: 'date',
  		    date: 'date',
  		    d: 'day',
  		    days: 'day',
  		    day: 'day',
  		    e: 'weekday',
  		    weekdays: 'weekday',
  		    weekday: 'weekday',
  		    E: 'isoWeekday',
  		    isoweekdays: 'isoWeekday',
  		    isoweekday: 'isoWeekday',
  		    DDD: 'dayOfYear',
  		    dayofyears: 'dayOfYear',
  		    dayofyear: 'dayOfYear',
  		    h: 'hour',
  		    hours: 'hour',
  		    hour: 'hour',
  		    ms: 'millisecond',
  		    milliseconds: 'millisecond',
  		    millisecond: 'millisecond',
  		    m: 'minute',
  		    minutes: 'minute',
  		    minute: 'minute',
  		    M: 'month',
  		    months: 'month',
  		    month: 'month',
  		    Q: 'quarter',
  		    quarters: 'quarter',
  		    quarter: 'quarter',
  		    s: 'second',
  		    seconds: 'second',
  		    second: 'second',
  		    gg: 'weekYear',
  		    weekyears: 'weekYear',
  		    weekyear: 'weekYear',
  		    GG: 'isoWeekYear',
  		    isoweekyears: 'isoWeekYear',
  		    isoweekyear: 'isoWeekYear',
  		    w: 'week',
  		    weeks: 'week',
  		    week: 'week',
  		    W: 'isoWeek',
  		    isoweeks: 'isoWeek',
  		    isoweek: 'isoWeek',
  		    y: 'year',
  		    years: 'year',
  		    year: 'year'
  		  };
  		  function normalizeUnits(units) {
  		    return typeof units === 'string' ? aliases[units] || aliases[units.toLowerCase()] : undefined;
  		  }
  		  function normalizeObjectUnits(inputObject) {
  		    var normalizedInput = {},
  		      normalizedProp,
  		      prop;
  		    for (prop in inputObject) {
  		      if (hasOwnProp(inputObject, prop)) {
  		        normalizedProp = normalizeUnits(prop);
  		        if (normalizedProp) {
  		          normalizedInput[normalizedProp] = inputObject[prop];
  		        }
  		      }
  		    }
  		    return normalizedInput;
  		  }
  		  var priorities = {
  		    date: 9,
  		    day: 11,
  		    weekday: 11,
  		    isoWeekday: 11,
  		    dayOfYear: 4,
  		    hour: 13,
  		    millisecond: 16,
  		    minute: 14,
  		    month: 8,
  		    quarter: 7,
  		    second: 15,
  		    weekYear: 1,
  		    isoWeekYear: 1,
  		    week: 5,
  		    isoWeek: 5,
  		    year: 1
  		  };
  		  function getPrioritizedUnits(unitsObj) {
  		    var units = [],
  		      u;
  		    for (u in unitsObj) {
  		      if (hasOwnProp(unitsObj, u)) {
  		        units.push({
  		          unit: u,
  		          priority: priorities[u]
  		        });
  		      }
  		    }
  		    units.sort(function (a, b) {
  		      return a.priority - b.priority;
  		    });
  		    return units;
  		  }
  		  var match1 = /\d/,
  		    //       0 - 9
  		    match2 = /\d\d/,
  		    //      00 - 99
  		    match3 = /\d{3}/,
  		    //     000 - 999
  		    match4 = /\d{4}/,
  		    //    0000 - 9999
  		    match6 = /[+-]?\d{6}/,
  		    // -999999 - 999999
  		    match1to2 = /\d\d?/,
  		    //       0 - 99
  		    match3to4 = /\d\d\d\d?/,
  		    //     999 - 9999
  		    match5to6 = /\d\d\d\d\d\d?/,
  		    //   99999 - 999999
  		    match1to3 = /\d{1,3}/,
  		    //       0 - 999
  		    match1to4 = /\d{1,4}/,
  		    //       0 - 9999
  		    match1to6 = /[+-]?\d{1,6}/,
  		    // -999999 - 999999
  		    matchUnsigned = /\d+/,
  		    //       0 - inf
  		    matchSigned = /[+-]?\d+/,
  		    //    -inf - inf
  		    matchOffset = /Z|[+-]\d\d:?\d\d/gi,
  		    // +00:00 -00:00 +0000 -0000 or Z
  		    matchShortOffset = /Z|[+-]\d\d(?::?\d\d)?/gi,
  		    // +00 -00 +00:00 -00:00 +0000 -0000 or Z
  		    matchTimestamp = /[+-]?\d+(\.\d{1,3})?/,
  		    // 123456789 123456789.123
  		    // any word (or two) characters or numbers including two/three word month in arabic.
  		    // includes scottish gaelic two word and hyphenated months
  		    matchWord = /[0-9]{0,256}['a-z\u00A0-\u05FF\u0700-\uD7FF\uF900-\uFDCF\uFDF0-\uFF07\uFF10-\uFFEF]{1,256}|[\u0600-\u06FF\/]{1,256}(\s*?[\u0600-\u06FF]{1,256}){1,2}/i,
  		    match1to2NoLeadingZero = /^[1-9]\d?/,
  		    //         1-99
  		    match1to2HasZero = /^([1-9]\d|\d)/,
  		    //           0-99
  		    regexes;
  		  regexes = {};
  		  function addRegexToken(token, regex, strictRegex) {
  		    regexes[token] = isFunction(regex) ? regex : function (isStrict, localeData) {
  		      return isStrict && strictRegex ? strictRegex : regex;
  		    };
  		  }
  		  function getParseRegexForToken(token, config) {
  		    if (!hasOwnProp(regexes, token)) {
  		      return new RegExp(unescapeFormat(token));
  		    }
  		    return regexes[token](config._strict, config._locale);
  		  }

  		  // Code from http://stackoverflow.com/questions/3561493/is-there-a-regexp-escape-function-in-javascript
  		  function unescapeFormat(s) {
  		    return regexEscape(s.replace('\\', '').replace(/\\(\[)|\\(\])|\[([^\]\[]*)\]|\\(.)/g, function (matched, p1, p2, p3, p4) {
  		      return p1 || p2 || p3 || p4;
  		    }));
  		  }
  		  function regexEscape(s) {
  		    return s.replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&');
  		  }
  		  function absFloor(number) {
  		    if (number < 0) {
  		      // -0 -> 0
  		      return Math.ceil(number) || 0;
  		    } else {
  		      return Math.floor(number);
  		    }
  		  }
  		  function toInt(argumentForCoercion) {
  		    var coercedNumber = +argumentForCoercion,
  		      value = 0;
  		    if (coercedNumber !== 0 && isFinite(coercedNumber)) {
  		      value = absFloor(coercedNumber);
  		    }
  		    return value;
  		  }
  		  var tokens = {};
  		  function addParseToken(token, callback) {
  		    var i,
  		      func = callback,
  		      tokenLen;
  		    if (typeof token === 'string') {
  		      token = [token];
  		    }
  		    if (isNumber(callback)) {
  		      func = function (input, array) {
  		        array[callback] = toInt(input);
  		      };
  		    }
  		    tokenLen = token.length;
  		    for (i = 0; i < tokenLen; i++) {
  		      tokens[token[i]] = func;
  		    }
  		  }
  		  function addWeekParseToken(token, callback) {
  		    addParseToken(token, function (input, array, config, token) {
  		      config._w = config._w || {};
  		      callback(input, config._w, config, token);
  		    });
  		  }
  		  function addTimeToArrayFromToken(token, input, config) {
  		    if (input != null && hasOwnProp(tokens, token)) {
  		      tokens[token](input, config._a, config, token);
  		    }
  		  }
  		  function isLeapYear(year) {
  		    return year % 4 === 0 && year % 100 !== 0 || year % 400 === 0;
  		  }
  		  var YEAR = 0,
  		    MONTH = 1,
  		    DATE = 2,
  		    HOUR = 3,
  		    MINUTE = 4,
  		    SECOND = 5,
  		    MILLISECOND = 6,
  		    WEEK = 7,
  		    WEEKDAY = 8;

  		  // FORMATTING

  		  addFormatToken('Y', 0, 0, function () {
  		    var y = this.year();
  		    return y <= 9999 ? zeroFill(y, 4) : '+' + y;
  		  });
  		  addFormatToken(0, ['YY', 2], 0, function () {
  		    return this.year() % 100;
  		  });
  		  addFormatToken(0, ['YYYY', 4], 0, 'year');
  		  addFormatToken(0, ['YYYYY', 5], 0, 'year');
  		  addFormatToken(0, ['YYYYYY', 6, true], 0, 'year');

  		  // PARSING

  		  addRegexToken('Y', matchSigned);
  		  addRegexToken('YY', match1to2, match2);
  		  addRegexToken('YYYY', match1to4, match4);
  		  addRegexToken('YYYYY', match1to6, match6);
  		  addRegexToken('YYYYYY', match1to6, match6);
  		  addParseToken(['YYYYY', 'YYYYYY'], YEAR);
  		  addParseToken('YYYY', function (input, array) {
  		    array[YEAR] = input.length === 2 ? hooks.parseTwoDigitYear(input) : toInt(input);
  		  });
  		  addParseToken('YY', function (input, array) {
  		    array[YEAR] = hooks.parseTwoDigitYear(input);
  		  });
  		  addParseToken('Y', function (input, array) {
  		    array[YEAR] = parseInt(input, 10);
  		  });

  		  // HELPERS

  		  function daysInYear(year) {
  		    return isLeapYear(year) ? 366 : 365;
  		  }

  		  // HOOKS

  		  hooks.parseTwoDigitYear = function (input) {
  		    return toInt(input) + (toInt(input) > 68 ? 1900 : 2000);
  		  };

  		  // MOMENTS

  		  var getSetYear = makeGetSet('FullYear', true);
  		  function getIsLeapYear() {
  		    return isLeapYear(this.year());
  		  }
  		  function makeGetSet(unit, keepTime) {
  		    return function (value) {
  		      if (value != null) {
  		        set$1(this, unit, value);
  		        hooks.updateOffset(this, keepTime);
  		        return this;
  		      } else {
  		        return get(this, unit);
  		      }
  		    };
  		  }
  		  function get(mom, unit) {
  		    if (!mom.isValid()) {
  		      return NaN;
  		    }
  		    var d = mom._d,
  		      isUTC = mom._isUTC;
  		    switch (unit) {
  		      case 'Milliseconds':
  		        return isUTC ? d.getUTCMilliseconds() : d.getMilliseconds();
  		      case 'Seconds':
  		        return isUTC ? d.getUTCSeconds() : d.getSeconds();
  		      case 'Minutes':
  		        return isUTC ? d.getUTCMinutes() : d.getMinutes();
  		      case 'Hours':
  		        return isUTC ? d.getUTCHours() : d.getHours();
  		      case 'Date':
  		        return isUTC ? d.getUTCDate() : d.getDate();
  		      case 'Day':
  		        return isUTC ? d.getUTCDay() : d.getDay();
  		      case 'Month':
  		        return isUTC ? d.getUTCMonth() : d.getMonth();
  		      case 'FullYear':
  		        return isUTC ? d.getUTCFullYear() : d.getFullYear();
  		      default:
  		        return NaN;
  		      // Just in case
  		    }
  		  }
  		  function set$1(mom, unit, value) {
  		    var d, isUTC, year, month, date;
  		    if (!mom.isValid() || isNaN(value)) {
  		      return;
  		    }
  		    d = mom._d;
  		    isUTC = mom._isUTC;
  		    switch (unit) {
  		      case 'Milliseconds':
  		        return void (isUTC ? d.setUTCMilliseconds(value) : d.setMilliseconds(value));
  		      case 'Seconds':
  		        return void (isUTC ? d.setUTCSeconds(value) : d.setSeconds(value));
  		      case 'Minutes':
  		        return void (isUTC ? d.setUTCMinutes(value) : d.setMinutes(value));
  		      case 'Hours':
  		        return void (isUTC ? d.setUTCHours(value) : d.setHours(value));
  		      case 'Date':
  		        return void (isUTC ? d.setUTCDate(value) : d.setDate(value));
  		      // case 'Day': // Not real
  		      //    return void (isUTC ? d.setUTCDay(value) : d.setDay(value));
  		      // case 'Month': // Not used because we need to pass two variables
  		      //     return void (isUTC ? d.setUTCMonth(value) : d.setMonth(value));
  		      case 'FullYear':
  		        break;
  		      // See below ...
  		      default:
  		        return;
  		      // Just in case
  		    }
  		    year = value;
  		    month = mom.month();
  		    date = mom.date();
  		    date = date === 29 && month === 1 && !isLeapYear(year) ? 28 : date;
  		    void (isUTC ? d.setUTCFullYear(year, month, date) : d.setFullYear(year, month, date));
  		  }

  		  // MOMENTS

  		  function stringGet(units) {
  		    units = normalizeUnits(units);
  		    if (isFunction(this[units])) {
  		      return this[units]();
  		    }
  		    return this;
  		  }
  		  function stringSet(units, value) {
  		    if (typeof units === 'object') {
  		      units = normalizeObjectUnits(units);
  		      var prioritized = getPrioritizedUnits(units),
  		        i,
  		        prioritizedLen = prioritized.length;
  		      for (i = 0; i < prioritizedLen; i++) {
  		        this[prioritized[i].unit](units[prioritized[i].unit]);
  		      }
  		    } else {
  		      units = normalizeUnits(units);
  		      if (isFunction(this[units])) {
  		        return this[units](value);
  		      }
  		    }
  		    return this;
  		  }
  		  function mod(n, x) {
  		    return (n % x + x) % x;
  		  }
  		  var indexOf;
  		  if (Array.prototype.indexOf) {
  		    indexOf = Array.prototype.indexOf;
  		  } else {
  		    indexOf = function (o) {
  		      // I know
  		      var i;
  		      for (i = 0; i < this.length; ++i) {
  		        if (this[i] === o) {
  		          return i;
  		        }
  		      }
  		      return -1;
  		    };
  		  }
  		  function daysInMonth(year, month) {
  		    if (isNaN(year) || isNaN(month)) {
  		      return NaN;
  		    }
  		    var modMonth = mod(month, 12);
  		    year += (month - modMonth) / 12;
  		    return modMonth === 1 ? isLeapYear(year) ? 29 : 28 : 31 - modMonth % 7 % 2;
  		  }

  		  // FORMATTING

  		  addFormatToken('M', ['MM', 2], 'Mo', function () {
  		    return this.month() + 1;
  		  });
  		  addFormatToken('MMM', 0, 0, function (format) {
  		    return this.localeData().monthsShort(this, format);
  		  });
  		  addFormatToken('MMMM', 0, 0, function (format) {
  		    return this.localeData().months(this, format);
  		  });

  		  // PARSING

  		  addRegexToken('M', match1to2, match1to2NoLeadingZero);
  		  addRegexToken('MM', match1to2, match2);
  		  addRegexToken('MMM', function (isStrict, locale) {
  		    return locale.monthsShortRegex(isStrict);
  		  });
  		  addRegexToken('MMMM', function (isStrict, locale) {
  		    return locale.monthsRegex(isStrict);
  		  });
  		  addParseToken(['M', 'MM'], function (input, array) {
  		    array[MONTH] = toInt(input) - 1;
  		  });
  		  addParseToken(['MMM', 'MMMM'], function (input, array, config, token) {
  		    var month = config._locale.monthsParse(input, token, config._strict);
  		    // if we didn't find a month name, mark the date as invalid.
  		    if (month != null) {
  		      array[MONTH] = month;
  		    } else {
  		      getParsingFlags(config).invalidMonth = input;
  		    }
  		  });

  		  // LOCALES

  		  var defaultLocaleMonths = 'January_February_March_April_May_June_July_August_September_October_November_December'.split('_'),
  		    defaultLocaleMonthsShort = 'Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec'.split('_'),
  		    MONTHS_IN_FORMAT = /D[oD]?(\[[^\[\]]*\]|\s)+MMMM?/,
  		    defaultMonthsShortRegex = matchWord,
  		    defaultMonthsRegex = matchWord;
  		  function localeMonths(m, format) {
  		    if (!m) {
  		      return isArray(this._months) ? this._months : this._months['standalone'];
  		    }
  		    return isArray(this._months) ? this._months[m.month()] : this._months[(this._months.isFormat || MONTHS_IN_FORMAT).test(format) ? 'format' : 'standalone'][m.month()];
  		  }
  		  function localeMonthsShort(m, format) {
  		    if (!m) {
  		      return isArray(this._monthsShort) ? this._monthsShort : this._monthsShort['standalone'];
  		    }
  		    return isArray(this._monthsShort) ? this._monthsShort[m.month()] : this._monthsShort[MONTHS_IN_FORMAT.test(format) ? 'format' : 'standalone'][m.month()];
  		  }
  		  function handleStrictParse(monthName, format, strict) {
  		    var i,
  		      ii,
  		      mom,
  		      llc = monthName.toLocaleLowerCase();
  		    if (!this._monthsParse) {
  		      // this is not used
  		      this._monthsParse = [];
  		      this._longMonthsParse = [];
  		      this._shortMonthsParse = [];
  		      for (i = 0; i < 12; ++i) {
  		        mom = createUTC([2000, i]);
  		        this._shortMonthsParse[i] = this.monthsShort(mom, '').toLocaleLowerCase();
  		        this._longMonthsParse[i] = this.months(mom, '').toLocaleLowerCase();
  		      }
  		    }
  		    if (strict) {
  		      if (format === 'MMM') {
  		        ii = indexOf.call(this._shortMonthsParse, llc);
  		        return ii !== -1 ? ii : null;
  		      } else {
  		        ii = indexOf.call(this._longMonthsParse, llc);
  		        return ii !== -1 ? ii : null;
  		      }
  		    } else {
  		      if (format === 'MMM') {
  		        ii = indexOf.call(this._shortMonthsParse, llc);
  		        if (ii !== -1) {
  		          return ii;
  		        }
  		        ii = indexOf.call(this._longMonthsParse, llc);
  		        return ii !== -1 ? ii : null;
  		      } else {
  		        ii = indexOf.call(this._longMonthsParse, llc);
  		        if (ii !== -1) {
  		          return ii;
  		        }
  		        ii = indexOf.call(this._shortMonthsParse, llc);
  		        return ii !== -1 ? ii : null;
  		      }
  		    }
  		  }
  		  function localeMonthsParse(monthName, format, strict) {
  		    var i, mom, regex;
  		    if (this._monthsParseExact) {
  		      return handleStrictParse.call(this, monthName, format, strict);
  		    }
  		    if (!this._monthsParse) {
  		      this._monthsParse = [];
  		      this._longMonthsParse = [];
  		      this._shortMonthsParse = [];
  		    }

  		    // TODO: add sorting
  		    // Sorting makes sure if one month (or abbr) is a prefix of another
  		    // see sorting in computeMonthsParse
  		    for (i = 0; i < 12; i++) {
  		      // make the regex if we don't have it already
  		      mom = createUTC([2000, i]);
  		      if (strict && !this._longMonthsParse[i]) {
  		        this._longMonthsParse[i] = new RegExp('^' + this.months(mom, '').replace('.', '') + '$', 'i');
  		        this._shortMonthsParse[i] = new RegExp('^' + this.monthsShort(mom, '').replace('.', '') + '$', 'i');
  		      }
  		      if (!strict && !this._monthsParse[i]) {
  		        regex = '^' + this.months(mom, '') + '|^' + this.monthsShort(mom, '');
  		        this._monthsParse[i] = new RegExp(regex.replace('.', ''), 'i');
  		      }
  		      // test the regex
  		      if (strict && format === 'MMMM' && this._longMonthsParse[i].test(monthName)) {
  		        return i;
  		      } else if (strict && format === 'MMM' && this._shortMonthsParse[i].test(monthName)) {
  		        return i;
  		      } else if (!strict && this._monthsParse[i].test(monthName)) {
  		        return i;
  		      }
  		    }
  		  }

  		  // MOMENTS

  		  function setMonth(mom, value) {
  		    if (!mom.isValid()) {
  		      // No op
  		      return mom;
  		    }
  		    if (typeof value === 'string') {
  		      if (/^\d+$/.test(value)) {
  		        value = toInt(value);
  		      } else {
  		        value = mom.localeData().monthsParse(value);
  		        // TODO: Another silent failure?
  		        if (!isNumber(value)) {
  		          return mom;
  		        }
  		      }
  		    }
  		    var month = value,
  		      date = mom.date();
  		    date = date < 29 ? date : Math.min(date, daysInMonth(mom.year(), month));
  		    void (mom._isUTC ? mom._d.setUTCMonth(month, date) : mom._d.setMonth(month, date));
  		    return mom;
  		  }
  		  function getSetMonth(value) {
  		    if (value != null) {
  		      setMonth(this, value);
  		      hooks.updateOffset(this, true);
  		      return this;
  		    } else {
  		      return get(this, 'Month');
  		    }
  		  }
  		  function getDaysInMonth() {
  		    return daysInMonth(this.year(), this.month());
  		  }
  		  function monthsShortRegex(isStrict) {
  		    if (this._monthsParseExact) {
  		      if (!hasOwnProp(this, '_monthsRegex')) {
  		        computeMonthsParse.call(this);
  		      }
  		      if (isStrict) {
  		        return this._monthsShortStrictRegex;
  		      } else {
  		        return this._monthsShortRegex;
  		      }
  		    } else {
  		      if (!hasOwnProp(this, '_monthsShortRegex')) {
  		        this._monthsShortRegex = defaultMonthsShortRegex;
  		      }
  		      return this._monthsShortStrictRegex && isStrict ? this._monthsShortStrictRegex : this._monthsShortRegex;
  		    }
  		  }
  		  function monthsRegex(isStrict) {
  		    if (this._monthsParseExact) {
  		      if (!hasOwnProp(this, '_monthsRegex')) {
  		        computeMonthsParse.call(this);
  		      }
  		      if (isStrict) {
  		        return this._monthsStrictRegex;
  		      } else {
  		        return this._monthsRegex;
  		      }
  		    } else {
  		      if (!hasOwnProp(this, '_monthsRegex')) {
  		        this._monthsRegex = defaultMonthsRegex;
  		      }
  		      return this._monthsStrictRegex && isStrict ? this._monthsStrictRegex : this._monthsRegex;
  		    }
  		  }
  		  function computeMonthsParse() {
  		    function cmpLenRev(a, b) {
  		      return b.length - a.length;
  		    }
  		    var shortPieces = [],
  		      longPieces = [],
  		      mixedPieces = [],
  		      i,
  		      mom,
  		      shortP,
  		      longP;
  		    for (i = 0; i < 12; i++) {
  		      // make the regex if we don't have it already
  		      mom = createUTC([2000, i]);
  		      shortP = regexEscape(this.monthsShort(mom, ''));
  		      longP = regexEscape(this.months(mom, ''));
  		      shortPieces.push(shortP);
  		      longPieces.push(longP);
  		      mixedPieces.push(longP);
  		      mixedPieces.push(shortP);
  		    }
  		    // Sorting makes sure if one month (or abbr) is a prefix of another it
  		    // will match the longer piece.
  		    shortPieces.sort(cmpLenRev);
  		    longPieces.sort(cmpLenRev);
  		    mixedPieces.sort(cmpLenRev);
  		    this._monthsRegex = new RegExp('^(' + mixedPieces.join('|') + ')', 'i');
  		    this._monthsShortRegex = this._monthsRegex;
  		    this._monthsStrictRegex = new RegExp('^(' + longPieces.join('|') + ')', 'i');
  		    this._monthsShortStrictRegex = new RegExp('^(' + shortPieces.join('|') + ')', 'i');
  		  }
  		  function createDate(y, m, d, h, M, s, ms) {
  		    // can't just apply() to create a date:
  		    // https://stackoverflow.com/q/181348
  		    var date;
  		    // the date constructor remaps years 0-99 to 1900-1999
  		    if (y < 100 && y >= 0) {
  		      // preserve leap years using a full 400 year cycle, then reset
  		      date = new Date(y + 400, m, d, h, M, s, ms);
  		      if (isFinite(date.getFullYear())) {
  		        date.setFullYear(y);
  		      }
  		    } else {
  		      date = new Date(y, m, d, h, M, s, ms);
  		    }
  		    return date;
  		  }
  		  function createUTCDate(y) {
  		    var date, args;
  		    // the Date.UTC function remaps years 0-99 to 1900-1999
  		    if (y < 100 && y >= 0) {
  		      args = Array.prototype.slice.call(arguments);
  		      // preserve leap years using a full 400 year cycle, then reset
  		      args[0] = y + 400;
  		      date = new Date(Date.UTC.apply(null, args));
  		      if (isFinite(date.getUTCFullYear())) {
  		        date.setUTCFullYear(y);
  		      }
  		    } else {
  		      date = new Date(Date.UTC.apply(null, arguments));
  		    }
  		    return date;
  		  }

  		  // start-of-first-week - start-of-year
  		  function firstWeekOffset(year, dow, doy) {
  		    var
  		      // first-week day -- which january is always in the first week (4 for iso, 1 for other)
  		      fwd = 7 + dow - doy,
  		      // first-week day local weekday -- which local weekday is fwd
  		      fwdlw = (7 + createUTCDate(year, 0, fwd).getUTCDay() - dow) % 7;
  		    return -fwdlw + fwd - 1;
  		  }

  		  // https://en.wikipedia.org/wiki/ISO_week_date#Calculating_a_date_given_the_year.2C_week_number_and_weekday
  		  function dayOfYearFromWeeks(year, week, weekday, dow, doy) {
  		    var localWeekday = (7 + weekday - dow) % 7,
  		      weekOffset = firstWeekOffset(year, dow, doy),
  		      dayOfYear = 1 + 7 * (week - 1) + localWeekday + weekOffset,
  		      resYear,
  		      resDayOfYear;
  		    if (dayOfYear <= 0) {
  		      resYear = year - 1;
  		      resDayOfYear = daysInYear(resYear) + dayOfYear;
  		    } else if (dayOfYear > daysInYear(year)) {
  		      resYear = year + 1;
  		      resDayOfYear = dayOfYear - daysInYear(year);
  		    } else {
  		      resYear = year;
  		      resDayOfYear = dayOfYear;
  		    }
  		    return {
  		      year: resYear,
  		      dayOfYear: resDayOfYear
  		    };
  		  }
  		  function weekOfYear(mom, dow, doy) {
  		    var weekOffset = firstWeekOffset(mom.year(), dow, doy),
  		      week = Math.floor((mom.dayOfYear() - weekOffset - 1) / 7) + 1,
  		      resWeek,
  		      resYear;
  		    if (week < 1) {
  		      resYear = mom.year() - 1;
  		      resWeek = week + weeksInYear(resYear, dow, doy);
  		    } else if (week > weeksInYear(mom.year(), dow, doy)) {
  		      resWeek = week - weeksInYear(mom.year(), dow, doy);
  		      resYear = mom.year() + 1;
  		    } else {
  		      resYear = mom.year();
  		      resWeek = week;
  		    }
  		    return {
  		      week: resWeek,
  		      year: resYear
  		    };
  		  }
  		  function weeksInYear(year, dow, doy) {
  		    var weekOffset = firstWeekOffset(year, dow, doy),
  		      weekOffsetNext = firstWeekOffset(year + 1, dow, doy);
  		    return (daysInYear(year) - weekOffset + weekOffsetNext) / 7;
  		  }

  		  // FORMATTING

  		  addFormatToken('w', ['ww', 2], 'wo', 'week');
  		  addFormatToken('W', ['WW', 2], 'Wo', 'isoWeek');

  		  // PARSING

  		  addRegexToken('w', match1to2, match1to2NoLeadingZero);
  		  addRegexToken('ww', match1to2, match2);
  		  addRegexToken('W', match1to2, match1to2NoLeadingZero);
  		  addRegexToken('WW', match1to2, match2);
  		  addWeekParseToken(['w', 'ww', 'W', 'WW'], function (input, week, config, token) {
  		    week[token.substr(0, 1)] = toInt(input);
  		  });

  		  // HELPERS

  		  // LOCALES

  		  function localeWeek(mom) {
  		    return weekOfYear(mom, this._week.dow, this._week.doy).week;
  		  }
  		  var defaultLocaleWeek = {
  		    dow: 0,
  		    // Sunday is the first day of the week.
  		    doy: 6 // The week that contains Jan 6th is the first week of the year.
  		  };
  		  function localeFirstDayOfWeek() {
  		    return this._week.dow;
  		  }
  		  function localeFirstDayOfYear() {
  		    return this._week.doy;
  		  }

  		  // MOMENTS

  		  function getSetWeek(input) {
  		    var week = this.localeData().week(this);
  		    return input == null ? week : this.add((input - week) * 7, 'd');
  		  }
  		  function getSetISOWeek(input) {
  		    var week = weekOfYear(this, 1, 4).week;
  		    return input == null ? week : this.add((input - week) * 7, 'd');
  		  }

  		  // FORMATTING

  		  addFormatToken('d', 0, 'do', 'day');
  		  addFormatToken('dd', 0, 0, function (format) {
  		    return this.localeData().weekdaysMin(this, format);
  		  });
  		  addFormatToken('ddd', 0, 0, function (format) {
  		    return this.localeData().weekdaysShort(this, format);
  		  });
  		  addFormatToken('dddd', 0, 0, function (format) {
  		    return this.localeData().weekdays(this, format);
  		  });
  		  addFormatToken('e', 0, 0, 'weekday');
  		  addFormatToken('E', 0, 0, 'isoWeekday');

  		  // PARSING

  		  addRegexToken('d', match1to2);
  		  addRegexToken('e', match1to2);
  		  addRegexToken('E', match1to2);
  		  addRegexToken('dd', function (isStrict, locale) {
  		    return locale.weekdaysMinRegex(isStrict);
  		  });
  		  addRegexToken('ddd', function (isStrict, locale) {
  		    return locale.weekdaysShortRegex(isStrict);
  		  });
  		  addRegexToken('dddd', function (isStrict, locale) {
  		    return locale.weekdaysRegex(isStrict);
  		  });
  		  addWeekParseToken(['dd', 'ddd', 'dddd'], function (input, week, config, token) {
  		    var weekday = config._locale.weekdaysParse(input, token, config._strict);
  		    // if we didn't get a weekday name, mark the date as invalid
  		    if (weekday != null) {
  		      week.d = weekday;
  		    } else {
  		      getParsingFlags(config).invalidWeekday = input;
  		    }
  		  });
  		  addWeekParseToken(['d', 'e', 'E'], function (input, week, config, token) {
  		    week[token] = toInt(input);
  		  });

  		  // HELPERS

  		  function parseWeekday(input, locale) {
  		    if (typeof input !== 'string') {
  		      return input;
  		    }
  		    if (!isNaN(input)) {
  		      return parseInt(input, 10);
  		    }
  		    input = locale.weekdaysParse(input);
  		    if (typeof input === 'number') {
  		      return input;
  		    }
  		    return null;
  		  }
  		  function parseIsoWeekday(input, locale) {
  		    if (typeof input === 'string') {
  		      return locale.weekdaysParse(input) % 7 || 7;
  		    }
  		    return isNaN(input) ? null : input;
  		  }

  		  // LOCALES
  		  function shiftWeekdays(ws, n) {
  		    return ws.slice(n, 7).concat(ws.slice(0, n));
  		  }
  		  var defaultLocaleWeekdays = 'Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday'.split('_'),
  		    defaultLocaleWeekdaysShort = 'Sun_Mon_Tue_Wed_Thu_Fri_Sat'.split('_'),
  		    defaultLocaleWeekdaysMin = 'Su_Mo_Tu_We_Th_Fr_Sa'.split('_'),
  		    defaultWeekdaysRegex = matchWord,
  		    defaultWeekdaysShortRegex = matchWord,
  		    defaultWeekdaysMinRegex = matchWord;
  		  function localeWeekdays(m, format) {
  		    var weekdays = isArray(this._weekdays) ? this._weekdays : this._weekdays[m && m !== true && this._weekdays.isFormat.test(format) ? 'format' : 'standalone'];
  		    return m === true ? shiftWeekdays(weekdays, this._week.dow) : m ? weekdays[m.day()] : weekdays;
  		  }
  		  function localeWeekdaysShort(m) {
  		    return m === true ? shiftWeekdays(this._weekdaysShort, this._week.dow) : m ? this._weekdaysShort[m.day()] : this._weekdaysShort;
  		  }
  		  function localeWeekdaysMin(m) {
  		    return m === true ? shiftWeekdays(this._weekdaysMin, this._week.dow) : m ? this._weekdaysMin[m.day()] : this._weekdaysMin;
  		  }
  		  function handleStrictParse$1(weekdayName, format, strict) {
  		    var i,
  		      ii,
  		      mom,
  		      llc = weekdayName.toLocaleLowerCase();
  		    if (!this._weekdaysParse) {
  		      this._weekdaysParse = [];
  		      this._shortWeekdaysParse = [];
  		      this._minWeekdaysParse = [];
  		      for (i = 0; i < 7; ++i) {
  		        mom = createUTC([2000, 1]).day(i);
  		        this._minWeekdaysParse[i] = this.weekdaysMin(mom, '').toLocaleLowerCase();
  		        this._shortWeekdaysParse[i] = this.weekdaysShort(mom, '').toLocaleLowerCase();
  		        this._weekdaysParse[i] = this.weekdays(mom, '').toLocaleLowerCase();
  		      }
  		    }
  		    if (strict) {
  		      if (format === 'dddd') {
  		        ii = indexOf.call(this._weekdaysParse, llc);
  		        return ii !== -1 ? ii : null;
  		      } else if (format === 'ddd') {
  		        ii = indexOf.call(this._shortWeekdaysParse, llc);
  		        return ii !== -1 ? ii : null;
  		      } else {
  		        ii = indexOf.call(this._minWeekdaysParse, llc);
  		        return ii !== -1 ? ii : null;
  		      }
  		    } else {
  		      if (format === 'dddd') {
  		        ii = indexOf.call(this._weekdaysParse, llc);
  		        if (ii !== -1) {
  		          return ii;
  		        }
  		        ii = indexOf.call(this._shortWeekdaysParse, llc);
  		        if (ii !== -1) {
  		          return ii;
  		        }
  		        ii = indexOf.call(this._minWeekdaysParse, llc);
  		        return ii !== -1 ? ii : null;
  		      } else if (format === 'ddd') {
  		        ii = indexOf.call(this._shortWeekdaysParse, llc);
  		        if (ii !== -1) {
  		          return ii;
  		        }
  		        ii = indexOf.call(this._weekdaysParse, llc);
  		        if (ii !== -1) {
  		          return ii;
  		        }
  		        ii = indexOf.call(this._minWeekdaysParse, llc);
  		        return ii !== -1 ? ii : null;
  		      } else {
  		        ii = indexOf.call(this._minWeekdaysParse, llc);
  		        if (ii !== -1) {
  		          return ii;
  		        }
  		        ii = indexOf.call(this._weekdaysParse, llc);
  		        if (ii !== -1) {
  		          return ii;
  		        }
  		        ii = indexOf.call(this._shortWeekdaysParse, llc);
  		        return ii !== -1 ? ii : null;
  		      }
  		    }
  		  }
  		  function localeWeekdaysParse(weekdayName, format, strict) {
  		    var i, mom, regex;
  		    if (this._weekdaysParseExact) {
  		      return handleStrictParse$1.call(this, weekdayName, format, strict);
  		    }
  		    if (!this._weekdaysParse) {
  		      this._weekdaysParse = [];
  		      this._minWeekdaysParse = [];
  		      this._shortWeekdaysParse = [];
  		      this._fullWeekdaysParse = [];
  		    }
  		    for (i = 0; i < 7; i++) {
  		      // make the regex if we don't have it already

  		      mom = createUTC([2000, 1]).day(i);
  		      if (strict && !this._fullWeekdaysParse[i]) {
  		        this._fullWeekdaysParse[i] = new RegExp('^' + this.weekdays(mom, '').replace('.', '\\.?') + '$', 'i');
  		        this._shortWeekdaysParse[i] = new RegExp('^' + this.weekdaysShort(mom, '').replace('.', '\\.?') + '$', 'i');
  		        this._minWeekdaysParse[i] = new RegExp('^' + this.weekdaysMin(mom, '').replace('.', '\\.?') + '$', 'i');
  		      }
  		      if (!this._weekdaysParse[i]) {
  		        regex = '^' + this.weekdays(mom, '') + '|^' + this.weekdaysShort(mom, '') + '|^' + this.weekdaysMin(mom, '');
  		        this._weekdaysParse[i] = new RegExp(regex.replace('.', ''), 'i');
  		      }
  		      // test the regex
  		      if (strict && format === 'dddd' && this._fullWeekdaysParse[i].test(weekdayName)) {
  		        return i;
  		      } else if (strict && format === 'ddd' && this._shortWeekdaysParse[i].test(weekdayName)) {
  		        return i;
  		      } else if (strict && format === 'dd' && this._minWeekdaysParse[i].test(weekdayName)) {
  		        return i;
  		      } else if (!strict && this._weekdaysParse[i].test(weekdayName)) {
  		        return i;
  		      }
  		    }
  		  }

  		  // MOMENTS

  		  function getSetDayOfWeek(input) {
  		    if (!this.isValid()) {
  		      return input != null ? this : NaN;
  		    }
  		    var day = get(this, 'Day');
  		    if (input != null) {
  		      input = parseWeekday(input, this.localeData());
  		      return this.add(input - day, 'd');
  		    } else {
  		      return day;
  		    }
  		  }
  		  function getSetLocaleDayOfWeek(input) {
  		    if (!this.isValid()) {
  		      return input != null ? this : NaN;
  		    }
  		    var weekday = (this.day() + 7 - this.localeData()._week.dow) % 7;
  		    return input == null ? weekday : this.add(input - weekday, 'd');
  		  }
  		  function getSetISODayOfWeek(input) {
  		    if (!this.isValid()) {
  		      return input != null ? this : NaN;
  		    }

  		    // behaves the same as moment#day except
  		    // as a getter, returns 7 instead of 0 (1-7 range instead of 0-6)
  		    // as a setter, sunday should belong to the previous week.

  		    if (input != null) {
  		      var weekday = parseIsoWeekday(input, this.localeData());
  		      return this.day(this.day() % 7 ? weekday : weekday - 7);
  		    } else {
  		      return this.day() || 7;
  		    }
  		  }
  		  function weekdaysRegex(isStrict) {
  		    if (this._weekdaysParseExact) {
  		      if (!hasOwnProp(this, '_weekdaysRegex')) {
  		        computeWeekdaysParse.call(this);
  		      }
  		      if (isStrict) {
  		        return this._weekdaysStrictRegex;
  		      } else {
  		        return this._weekdaysRegex;
  		      }
  		    } else {
  		      if (!hasOwnProp(this, '_weekdaysRegex')) {
  		        this._weekdaysRegex = defaultWeekdaysRegex;
  		      }
  		      return this._weekdaysStrictRegex && isStrict ? this._weekdaysStrictRegex : this._weekdaysRegex;
  		    }
  		  }
  		  function weekdaysShortRegex(isStrict) {
  		    if (this._weekdaysParseExact) {
  		      if (!hasOwnProp(this, '_weekdaysRegex')) {
  		        computeWeekdaysParse.call(this);
  		      }
  		      if (isStrict) {
  		        return this._weekdaysShortStrictRegex;
  		      } else {
  		        return this._weekdaysShortRegex;
  		      }
  		    } else {
  		      if (!hasOwnProp(this, '_weekdaysShortRegex')) {
  		        this._weekdaysShortRegex = defaultWeekdaysShortRegex;
  		      }
  		      return this._weekdaysShortStrictRegex && isStrict ? this._weekdaysShortStrictRegex : this._weekdaysShortRegex;
  		    }
  		  }
  		  function weekdaysMinRegex(isStrict) {
  		    if (this._weekdaysParseExact) {
  		      if (!hasOwnProp(this, '_weekdaysRegex')) {
  		        computeWeekdaysParse.call(this);
  		      }
  		      if (isStrict) {
  		        return this._weekdaysMinStrictRegex;
  		      } else {
  		        return this._weekdaysMinRegex;
  		      }
  		    } else {
  		      if (!hasOwnProp(this, '_weekdaysMinRegex')) {
  		        this._weekdaysMinRegex = defaultWeekdaysMinRegex;
  		      }
  		      return this._weekdaysMinStrictRegex && isStrict ? this._weekdaysMinStrictRegex : this._weekdaysMinRegex;
  		    }
  		  }
  		  function computeWeekdaysParse() {
  		    function cmpLenRev(a, b) {
  		      return b.length - a.length;
  		    }
  		    var minPieces = [],
  		      shortPieces = [],
  		      longPieces = [],
  		      mixedPieces = [],
  		      i,
  		      mom,
  		      minp,
  		      shortp,
  		      longp;
  		    for (i = 0; i < 7; i++) {
  		      // make the regex if we don't have it already
  		      mom = createUTC([2000, 1]).day(i);
  		      minp = regexEscape(this.weekdaysMin(mom, ''));
  		      shortp = regexEscape(this.weekdaysShort(mom, ''));
  		      longp = regexEscape(this.weekdays(mom, ''));
  		      minPieces.push(minp);
  		      shortPieces.push(shortp);
  		      longPieces.push(longp);
  		      mixedPieces.push(minp);
  		      mixedPieces.push(shortp);
  		      mixedPieces.push(longp);
  		    }
  		    // Sorting makes sure if one weekday (or abbr) is a prefix of another it
  		    // will match the longer piece.
  		    minPieces.sort(cmpLenRev);
  		    shortPieces.sort(cmpLenRev);
  		    longPieces.sort(cmpLenRev);
  		    mixedPieces.sort(cmpLenRev);
  		    this._weekdaysRegex = new RegExp('^(' + mixedPieces.join('|') + ')', 'i');
  		    this._weekdaysShortRegex = this._weekdaysRegex;
  		    this._weekdaysMinRegex = this._weekdaysRegex;
  		    this._weekdaysStrictRegex = new RegExp('^(' + longPieces.join('|') + ')', 'i');
  		    this._weekdaysShortStrictRegex = new RegExp('^(' + shortPieces.join('|') + ')', 'i');
  		    this._weekdaysMinStrictRegex = new RegExp('^(' + minPieces.join('|') + ')', 'i');
  		  }

  		  // FORMATTING

  		  function hFormat() {
  		    return this.hours() % 12 || 12;
  		  }
  		  function kFormat() {
  		    return this.hours() || 24;
  		  }
  		  addFormatToken('H', ['HH', 2], 0, 'hour');
  		  addFormatToken('h', ['hh', 2], 0, hFormat);
  		  addFormatToken('k', ['kk', 2], 0, kFormat);
  		  addFormatToken('hmm', 0, 0, function () {
  		    return '' + hFormat.apply(this) + zeroFill(this.minutes(), 2);
  		  });
  		  addFormatToken('hmmss', 0, 0, function () {
  		    return '' + hFormat.apply(this) + zeroFill(this.minutes(), 2) + zeroFill(this.seconds(), 2);
  		  });
  		  addFormatToken('Hmm', 0, 0, function () {
  		    return '' + this.hours() + zeroFill(this.minutes(), 2);
  		  });
  		  addFormatToken('Hmmss', 0, 0, function () {
  		    return '' + this.hours() + zeroFill(this.minutes(), 2) + zeroFill(this.seconds(), 2);
  		  });
  		  function meridiem(token, lowercase) {
  		    addFormatToken(token, 0, 0, function () {
  		      return this.localeData().meridiem(this.hours(), this.minutes(), lowercase);
  		    });
  		  }
  		  meridiem('a', true);
  		  meridiem('A', false);

  		  // PARSING

  		  function matchMeridiem(isStrict, locale) {
  		    return locale._meridiemParse;
  		  }
  		  addRegexToken('a', matchMeridiem);
  		  addRegexToken('A', matchMeridiem);
  		  addRegexToken('H', match1to2, match1to2HasZero);
  		  addRegexToken('h', match1to2, match1to2NoLeadingZero);
  		  addRegexToken('k', match1to2, match1to2NoLeadingZero);
  		  addRegexToken('HH', match1to2, match2);
  		  addRegexToken('hh', match1to2, match2);
  		  addRegexToken('kk', match1to2, match2);
  		  addRegexToken('hmm', match3to4);
  		  addRegexToken('hmmss', match5to6);
  		  addRegexToken('Hmm', match3to4);
  		  addRegexToken('Hmmss', match5to6);
  		  addParseToken(['H', 'HH'], HOUR);
  		  addParseToken(['k', 'kk'], function (input, array, config) {
  		    var kInput = toInt(input);
  		    array[HOUR] = kInput === 24 ? 0 : kInput;
  		  });
  		  addParseToken(['a', 'A'], function (input, array, config) {
  		    config._isPm = config._locale.isPM(input);
  		    config._meridiem = input;
  		  });
  		  addParseToken(['h', 'hh'], function (input, array, config) {
  		    array[HOUR] = toInt(input);
  		    getParsingFlags(config).bigHour = true;
  		  });
  		  addParseToken('hmm', function (input, array, config) {
  		    var pos = input.length - 2;
  		    array[HOUR] = toInt(input.substr(0, pos));
  		    array[MINUTE] = toInt(input.substr(pos));
  		    getParsingFlags(config).bigHour = true;
  		  });
  		  addParseToken('hmmss', function (input, array, config) {
  		    var pos1 = input.length - 4,
  		      pos2 = input.length - 2;
  		    array[HOUR] = toInt(input.substr(0, pos1));
  		    array[MINUTE] = toInt(input.substr(pos1, 2));
  		    array[SECOND] = toInt(input.substr(pos2));
  		    getParsingFlags(config).bigHour = true;
  		  });
  		  addParseToken('Hmm', function (input, array, config) {
  		    var pos = input.length - 2;
  		    array[HOUR] = toInt(input.substr(0, pos));
  		    array[MINUTE] = toInt(input.substr(pos));
  		  });
  		  addParseToken('Hmmss', function (input, array, config) {
  		    var pos1 = input.length - 4,
  		      pos2 = input.length - 2;
  		    array[HOUR] = toInt(input.substr(0, pos1));
  		    array[MINUTE] = toInt(input.substr(pos1, 2));
  		    array[SECOND] = toInt(input.substr(pos2));
  		  });

  		  // LOCALES

  		  function localeIsPM(input) {
  		    // IE8 Quirks Mode & IE7 Standards Mode do not allow accessing strings like arrays
  		    // Using charAt should be more compatible.
  		    return (input + '').toLowerCase().charAt(0) === 'p';
  		  }
  		  var defaultLocaleMeridiemParse = /[ap]\.?m?\.?/i,
  		    // Setting the hour should keep the time, because the user explicitly
  		    // specified which hour they want. So trying to maintain the same hour (in
  		    // a new timezone) makes sense. Adding/subtracting hours does not follow
  		    // this rule.
  		    getSetHour = makeGetSet('Hours', true);
  		  function localeMeridiem(hours, minutes, isLower) {
  		    if (hours > 11) {
  		      return isLower ? 'pm' : 'PM';
  		    } else {
  		      return isLower ? 'am' : 'AM';
  		    }
  		  }
  		  var baseConfig = {
  		    calendar: defaultCalendar,
  		    longDateFormat: defaultLongDateFormat,
  		    invalidDate: defaultInvalidDate,
  		    ordinal: defaultOrdinal,
  		    dayOfMonthOrdinalParse: defaultDayOfMonthOrdinalParse,
  		    relativeTime: defaultRelativeTime,
  		    months: defaultLocaleMonths,
  		    monthsShort: defaultLocaleMonthsShort,
  		    week: defaultLocaleWeek,
  		    weekdays: defaultLocaleWeekdays,
  		    weekdaysMin: defaultLocaleWeekdaysMin,
  		    weekdaysShort: defaultLocaleWeekdaysShort,
  		    meridiemParse: defaultLocaleMeridiemParse
  		  };

  		  // internal storage for locale config files
  		  var locales = {},
  		    localeFamilies = {},
  		    globalLocale;
  		  function commonPrefix(arr1, arr2) {
  		    var i,
  		      minl = Math.min(arr1.length, arr2.length);
  		    for (i = 0; i < minl; i += 1) {
  		      if (arr1[i] !== arr2[i]) {
  		        return i;
  		      }
  		    }
  		    return minl;
  		  }
  		  function normalizeLocale(key) {
  		    return key ? key.toLowerCase().replace('_', '-') : key;
  		  }

  		  // pick the locale from the array
  		  // try ['en-au', 'en-gb'] as 'en-au', 'en-gb', 'en', as in move through the list trying each
  		  // substring from most specific to least, but move to the next array item if it's a more specific variant than the current root
  		  function chooseLocale(names) {
  		    var i = 0,
  		      j,
  		      next,
  		      locale,
  		      split;
  		    while (i < names.length) {
  		      split = normalizeLocale(names[i]).split('-');
  		      j = split.length;
  		      next = normalizeLocale(names[i + 1]);
  		      next = next ? next.split('-') : null;
  		      while (j > 0) {
  		        locale = loadLocale(split.slice(0, j).join('-'));
  		        if (locale) {
  		          return locale;
  		        }
  		        if (next && next.length >= j && commonPrefix(split, next) >= j - 1) {
  		          //the next array item is better than a shallower substring of this one
  		          break;
  		        }
  		        j--;
  		      }
  		      i++;
  		    }
  		    return globalLocale;
  		  }
  		  function isLocaleNameSane(name) {
  		    // Prevent names that look like filesystem paths, i.e contain '/' or '\'
  		    // Ensure name is available and function returns boolean
  		    return !!(name && name.match('^[^/\\\\]*$'));
  		  }
  		  function loadLocale(name) {
  		    var oldLocale = null,
  		      aliasedRequire;
  		    // TODO: Find a better way to register and load all the locales in Node
  		    if (locales[name] === undefined && 'object' !== 'undefined' && module && module.exports && isLocaleNameSane(name)) {
  		      try {
  		        oldLocale = globalLocale._abbr;
  		        aliasedRequire = commonjsRequire;
  		        aliasedRequire('./locale/' + name);
  		        getSetGlobalLocale(oldLocale);
  		      } catch (e) {
  		        // mark as not found to avoid repeating expensive file require call causing high CPU
  		        // when trying to find en-US, en_US, en-us for every format call
  		        locales[name] = null; // null means not found
  		      }
  		    }
  		    return locales[name];
  		  }

  		  // This function will load locale and then set the global locale.  If
  		  // no arguments are passed in, it will simply return the current global
  		  // locale key.
  		  function getSetGlobalLocale(key, values) {
  		    var data;
  		    if (key) {
  		      if (isUndefined(values)) {
  		        data = getLocale(key);
  		      } else {
  		        data = defineLocale(key, values);
  		      }
  		      if (data) {
  		        // moment.duration._locale = moment._locale = data;
  		        globalLocale = data;
  		      } else {
  		        if (typeof console !== 'undefined' && console.warn) {
  		          //warn user if arguments are passed but the locale could not be set
  		          console.warn('Locale ' + key + ' not found. Did you forget to load it?');
  		        }
  		      }
  		    }
  		    return globalLocale._abbr;
  		  }
  		  function defineLocale(name, config) {
  		    if (config !== null) {
  		      var locale,
  		        parentConfig = baseConfig;
  		      config.abbr = name;
  		      if (locales[name] != null) {
  		        deprecateSimple('defineLocaleOverride', 'use moment.updateLocale(localeName, config) to change ' + 'an existing locale. moment.defineLocale(localeName, ' + 'config) should only be used for creating a new locale ' + 'See http://momentjs.com/guides/#/warnings/define-locale/ for more info.');
  		        parentConfig = locales[name]._config;
  		      } else if (config.parentLocale != null) {
  		        if (locales[config.parentLocale] != null) {
  		          parentConfig = locales[config.parentLocale]._config;
  		        } else {
  		          locale = loadLocale(config.parentLocale);
  		          if (locale != null) {
  		            parentConfig = locale._config;
  		          } else {
  		            if (!localeFamilies[config.parentLocale]) {
  		              localeFamilies[config.parentLocale] = [];
  		            }
  		            localeFamilies[config.parentLocale].push({
  		              name: name,
  		              config: config
  		            });
  		            return null;
  		          }
  		        }
  		      }
  		      locales[name] = new Locale(mergeConfigs(parentConfig, config));
  		      if (localeFamilies[name]) {
  		        localeFamilies[name].forEach(function (x) {
  		          defineLocale(x.name, x.config);
  		        });
  		      }

  		      // backwards compat for now: also set the locale
  		      // make sure we set the locale AFTER all child locales have been
  		      // created, so we won't end up with the child locale set.
  		      getSetGlobalLocale(name);
  		      return locales[name];
  		    } else {
  		      // useful for testing
  		      delete locales[name];
  		      return null;
  		    }
  		  }
  		  function updateLocale(name, config) {
  		    if (config != null) {
  		      var locale,
  		        tmpLocale,
  		        parentConfig = baseConfig;
  		      if (locales[name] != null && locales[name].parentLocale != null) {
  		        // Update existing child locale in-place to avoid memory-leaks
  		        locales[name].set(mergeConfigs(locales[name]._config, config));
  		      } else {
  		        // MERGE
  		        tmpLocale = loadLocale(name);
  		        if (tmpLocale != null) {
  		          parentConfig = tmpLocale._config;
  		        }
  		        config = mergeConfigs(parentConfig, config);
  		        if (tmpLocale == null) {
  		          // updateLocale is called for creating a new locale
  		          // Set abbr so it will have a name (getters return
  		          // undefined otherwise).
  		          config.abbr = name;
  		        }
  		        locale = new Locale(config);
  		        locale.parentLocale = locales[name];
  		        locales[name] = locale;
  		      }

  		      // backwards compat for now: also set the locale
  		      getSetGlobalLocale(name);
  		    } else {
  		      // pass null for config to unupdate, useful for tests
  		      if (locales[name] != null) {
  		        if (locales[name].parentLocale != null) {
  		          locales[name] = locales[name].parentLocale;
  		          if (name === getSetGlobalLocale()) {
  		            getSetGlobalLocale(name);
  		          }
  		        } else if (locales[name] != null) {
  		          delete locales[name];
  		        }
  		      }
  		    }
  		    return locales[name];
  		  }

  		  // returns locale data
  		  function getLocale(key) {
  		    var locale;
  		    if (key && key._locale && key._locale._abbr) {
  		      key = key._locale._abbr;
  		    }
  		    if (!key) {
  		      return globalLocale;
  		    }
  		    if (!isArray(key)) {
  		      //short-circuit everything else
  		      locale = loadLocale(key);
  		      if (locale) {
  		        return locale;
  		      }
  		      key = [key];
  		    }
  		    return chooseLocale(key);
  		  }
  		  function listLocales() {
  		    return keys(locales);
  		  }
  		  function checkOverflow(m) {
  		    var overflow,
  		      a = m._a;
  		    if (a && getParsingFlags(m).overflow === -2) {
  		      overflow = a[MONTH] < 0 || a[MONTH] > 11 ? MONTH : a[DATE] < 1 || a[DATE] > daysInMonth(a[YEAR], a[MONTH]) ? DATE : a[HOUR] < 0 || a[HOUR] > 24 || a[HOUR] === 24 && (a[MINUTE] !== 0 || a[SECOND] !== 0 || a[MILLISECOND] !== 0) ? HOUR : a[MINUTE] < 0 || a[MINUTE] > 59 ? MINUTE : a[SECOND] < 0 || a[SECOND] > 59 ? SECOND : a[MILLISECOND] < 0 || a[MILLISECOND] > 999 ? MILLISECOND : -1;
  		      if (getParsingFlags(m)._overflowDayOfYear && (overflow < YEAR || overflow > DATE)) {
  		        overflow = DATE;
  		      }
  		      if (getParsingFlags(m)._overflowWeeks && overflow === -1) {
  		        overflow = WEEK;
  		      }
  		      if (getParsingFlags(m)._overflowWeekday && overflow === -1) {
  		        overflow = WEEKDAY;
  		      }
  		      getParsingFlags(m).overflow = overflow;
  		    }
  		    return m;
  		  }

  		  // iso 8601 regex
  		  // 0000-00-00 0000-W00 or 0000-W00-0 + T + 00 or 00:00 or 00:00:00 or 00:00:00.000 + +00:00 or +0000 or +00)
  		  var extendedIsoRegex = /^\s*((?:[+-]\d{6}|\d{4})-(?:\d\d-\d\d|W\d\d-\d|W\d\d|\d\d\d|\d\d))(?:(T| )(\d\d(?::\d\d(?::\d\d(?:[.,]\d+)?)?)?)([+-]\d\d(?::?\d\d)?|\s*Z)?)?$/,
  		    basicIsoRegex = /^\s*((?:[+-]\d{6}|\d{4})(?:\d\d\d\d|W\d\d\d|W\d\d|\d\d\d|\d\d|))(?:(T| )(\d\d(?:\d\d(?:\d\d(?:[.,]\d+)?)?)?)([+-]\d\d(?::?\d\d)?|\s*Z)?)?$/,
  		    tzRegex = /Z|[+-]\d\d(?::?\d\d)?/,
  		    isoDates = [['YYYYYY-MM-DD', /[+-]\d{6}-\d\d-\d\d/], ['YYYY-MM-DD', /\d{4}-\d\d-\d\d/], ['GGGG-[W]WW-E', /\d{4}-W\d\d-\d/], ['GGGG-[W]WW', /\d{4}-W\d\d/, false], ['YYYY-DDD', /\d{4}-\d{3}/], ['YYYY-MM', /\d{4}-\d\d/, false], ['YYYYYYMMDD', /[+-]\d{10}/], ['YYYYMMDD', /\d{8}/], ['GGGG[W]WWE', /\d{4}W\d{3}/], ['GGGG[W]WW', /\d{4}W\d{2}/, false], ['YYYYDDD', /\d{7}/], ['YYYYMM', /\d{6}/, false], ['YYYY', /\d{4}/, false]],
  		    // iso time formats and regexes
  		    isoTimes = [['HH:mm:ss.SSSS', /\d\d:\d\d:\d\d\.\d+/], ['HH:mm:ss,SSSS', /\d\d:\d\d:\d\d,\d+/], ['HH:mm:ss', /\d\d:\d\d:\d\d/], ['HH:mm', /\d\d:\d\d/], ['HHmmss.SSSS', /\d\d\d\d\d\d\.\d+/], ['HHmmss,SSSS', /\d\d\d\d\d\d,\d+/], ['HHmmss', /\d\d\d\d\d\d/], ['HHmm', /\d\d\d\d/], ['HH', /\d\d/]],
  		    aspNetJsonRegex = /^\/?Date\((-?\d+)/i,
  		    // RFC 2822 regex: For details see https://tools.ietf.org/html/rfc2822#section-3.3
  		    rfc2822 = /^(?:(Mon|Tue|Wed|Thu|Fri|Sat|Sun),?\s)?(\d{1,2})\s(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)\s(\d{2,4})\s(\d\d):(\d\d)(?::(\d\d))?\s(?:(UT|GMT|[ECMP][SD]T)|([Zz])|([+-]\d{4}))$/,
  		    obsOffsets = {
  		      UT: 0,
  		      GMT: 0,
  		      EDT: -4 * 60,
  		      EST: -5 * 60,
  		      CDT: -5 * 60,
  		      CST: -6 * 60,
  		      MDT: -6 * 60,
  		      MST: -7 * 60,
  		      PDT: -7 * 60,
  		      PST: -8 * 60
  		    };

  		  // date from iso format
  		  function configFromISO(config) {
  		    var i,
  		      l,
  		      string = config._i,
  		      match = extendedIsoRegex.exec(string) || basicIsoRegex.exec(string),
  		      allowTime,
  		      dateFormat,
  		      timeFormat,
  		      tzFormat,
  		      isoDatesLen = isoDates.length,
  		      isoTimesLen = isoTimes.length;
  		    if (match) {
  		      getParsingFlags(config).iso = true;
  		      for (i = 0, l = isoDatesLen; i < l; i++) {
  		        if (isoDates[i][1].exec(match[1])) {
  		          dateFormat = isoDates[i][0];
  		          allowTime = isoDates[i][2] !== false;
  		          break;
  		        }
  		      }
  		      if (dateFormat == null) {
  		        config._isValid = false;
  		        return;
  		      }
  		      if (match[3]) {
  		        for (i = 0, l = isoTimesLen; i < l; i++) {
  		          if (isoTimes[i][1].exec(match[3])) {
  		            // match[2] should be 'T' or space
  		            timeFormat = (match[2] || ' ') + isoTimes[i][0];
  		            break;
  		          }
  		        }
  		        if (timeFormat == null) {
  		          config._isValid = false;
  		          return;
  		        }
  		      }
  		      if (!allowTime && timeFormat != null) {
  		        config._isValid = false;
  		        return;
  		      }
  		      if (match[4]) {
  		        if (tzRegex.exec(match[4])) {
  		          tzFormat = 'Z';
  		        } else {
  		          config._isValid = false;
  		          return;
  		        }
  		      }
  		      config._f = dateFormat + (timeFormat || '') + (tzFormat || '');
  		      configFromStringAndFormat(config);
  		    } else {
  		      config._isValid = false;
  		    }
  		  }
  		  function extractFromRFC2822Strings(yearStr, monthStr, dayStr, hourStr, minuteStr, secondStr) {
  		    var result = [untruncateYear(yearStr), defaultLocaleMonthsShort.indexOf(monthStr), parseInt(dayStr, 10), parseInt(hourStr, 10), parseInt(minuteStr, 10)];
  		    if (secondStr) {
  		      result.push(parseInt(secondStr, 10));
  		    }
  		    return result;
  		  }
  		  function untruncateYear(yearStr) {
  		    var year = parseInt(yearStr, 10);
  		    if (year <= 49) {
  		      return 2000 + year;
  		    } else if (year <= 999) {
  		      return 1900 + year;
  		    }
  		    return year;
  		  }
  		  function preprocessRFC2822(s) {
  		    // Remove comments and folding whitespace and replace multiple-spaces with a single space
  		    return s.replace(/\([^()]*\)|[\n\t]/g, ' ').replace(/(\s\s+)/g, ' ').replace(/^\s\s*/, '').replace(/\s\s*$/, '');
  		  }
  		  function checkWeekday(weekdayStr, parsedInput, config) {
  		    if (weekdayStr) {
  		      // TODO: Replace the vanilla JS Date object with an independent day-of-week check.
  		      var weekdayProvided = defaultLocaleWeekdaysShort.indexOf(weekdayStr),
  		        weekdayActual = new Date(parsedInput[0], parsedInput[1], parsedInput[2]).getDay();
  		      if (weekdayProvided !== weekdayActual) {
  		        getParsingFlags(config).weekdayMismatch = true;
  		        config._isValid = false;
  		        return false;
  		      }
  		    }
  		    return true;
  		  }
  		  function calculateOffset(obsOffset, militaryOffset, numOffset) {
  		    if (obsOffset) {
  		      return obsOffsets[obsOffset];
  		    } else if (militaryOffset) {
  		      // the only allowed military tz is Z
  		      return 0;
  		    } else {
  		      var hm = parseInt(numOffset, 10),
  		        m = hm % 100,
  		        h = (hm - m) / 100;
  		      return h * 60 + m;
  		    }
  		  }

  		  // date and time from ref 2822 format
  		  function configFromRFC2822(config) {
  		    var match = rfc2822.exec(preprocessRFC2822(config._i)),
  		      parsedArray;
  		    if (match) {
  		      parsedArray = extractFromRFC2822Strings(match[4], match[3], match[2], match[5], match[6], match[7]);
  		      if (!checkWeekday(match[1], parsedArray, config)) {
  		        return;
  		      }
  		      config._a = parsedArray;
  		      config._tzm = calculateOffset(match[8], match[9], match[10]);
  		      config._d = createUTCDate.apply(null, config._a);
  		      config._d.setUTCMinutes(config._d.getUTCMinutes() - config._tzm);
  		      getParsingFlags(config).rfc2822 = true;
  		    } else {
  		      config._isValid = false;
  		    }
  		  }

  		  // date from 1) ASP.NET, 2) ISO, 3) RFC 2822 formats, or 4) optional fallback if parsing isn't strict
  		  function configFromString(config) {
  		    var matched = aspNetJsonRegex.exec(config._i);
  		    if (matched !== null) {
  		      config._d = new Date(+matched[1]);
  		      return;
  		    }
  		    configFromISO(config);
  		    if (config._isValid === false) {
  		      delete config._isValid;
  		    } else {
  		      return;
  		    }
  		    configFromRFC2822(config);
  		    if (config._isValid === false) {
  		      delete config._isValid;
  		    } else {
  		      return;
  		    }
  		    if (config._strict) {
  		      config._isValid = false;
  		    } else {
  		      // Final attempt, use Input Fallback
  		      hooks.createFromInputFallback(config);
  		    }
  		  }
  		  hooks.createFromInputFallback = deprecate('value provided is not in a recognized RFC2822 or ISO format. moment construction falls back to js Date(), ' + 'which is not reliable across all browsers and versions. Non RFC2822/ISO date formats are ' + 'discouraged. Please refer to http://momentjs.com/guides/#/warnings/js-date/ for more info.', function (config) {
  		    config._d = new Date(config._i + (config._useUTC ? ' UTC' : ''));
  		  });

  		  // Pick the first defined of two or three arguments.
  		  function defaults(a, b, c) {
  		    if (a != null) {
  		      return a;
  		    }
  		    if (b != null) {
  		      return b;
  		    }
  		    return c;
  		  }
  		  function currentDateArray(config) {
  		    // hooks is actually the exported moment object
  		    var nowValue = new Date(hooks.now());
  		    if (config._useUTC) {
  		      return [nowValue.getUTCFullYear(), nowValue.getUTCMonth(), nowValue.getUTCDate()];
  		    }
  		    return [nowValue.getFullYear(), nowValue.getMonth(), nowValue.getDate()];
  		  }

  		  // convert an array to a date.
  		  // the array should mirror the parameters below
  		  // note: all values past the year are optional and will default to the lowest possible value.
  		  // [year, month, day , hour, minute, second, millisecond]
  		  function configFromArray(config) {
  		    var i,
  		      date,
  		      input = [],
  		      currentDate,
  		      expectedWeekday,
  		      yearToUse;
  		    if (config._d) {
  		      return;
  		    }
  		    currentDate = currentDateArray(config);

  		    //compute day of the year from weeks and weekdays
  		    if (config._w && config._a[DATE] == null && config._a[MONTH] == null) {
  		      dayOfYearFromWeekInfo(config);
  		    }

  		    //if the day of the year is set, figure out what it is
  		    if (config._dayOfYear != null) {
  		      yearToUse = defaults(config._a[YEAR], currentDate[YEAR]);
  		      if (config._dayOfYear > daysInYear(yearToUse) || config._dayOfYear === 0) {
  		        getParsingFlags(config)._overflowDayOfYear = true;
  		      }
  		      date = createUTCDate(yearToUse, 0, config._dayOfYear);
  		      config._a[MONTH] = date.getUTCMonth();
  		      config._a[DATE] = date.getUTCDate();
  		    }

  		    // Default to current date.
  		    // * if no year, month, day of month are given, default to today
  		    // * if day of month is given, default month and year
  		    // * if month is given, default only year
  		    // * if year is given, don't default anything
  		    for (i = 0; i < 3 && config._a[i] == null; ++i) {
  		      config._a[i] = input[i] = currentDate[i];
  		    }

  		    // Zero out whatever was not defaulted, including time
  		    for (; i < 7; i++) {
  		      config._a[i] = input[i] = config._a[i] == null ? i === 2 ? 1 : 0 : config._a[i];
  		    }

  		    // Check for 24:00:00.000
  		    if (config._a[HOUR] === 24 && config._a[MINUTE] === 0 && config._a[SECOND] === 0 && config._a[MILLISECOND] === 0) {
  		      config._nextDay = true;
  		      config._a[HOUR] = 0;
  		    }
  		    config._d = (config._useUTC ? createUTCDate : createDate).apply(null, input);
  		    expectedWeekday = config._useUTC ? config._d.getUTCDay() : config._d.getDay();

  		    // Apply timezone offset from input. The actual utcOffset can be changed
  		    // with parseZone.
  		    if (config._tzm != null) {
  		      config._d.setUTCMinutes(config._d.getUTCMinutes() - config._tzm);
  		    }
  		    if (config._nextDay) {
  		      config._a[HOUR] = 24;
  		    }

  		    // check for mismatching day of week
  		    if (config._w && typeof config._w.d !== 'undefined' && config._w.d !== expectedWeekday) {
  		      getParsingFlags(config).weekdayMismatch = true;
  		    }
  		  }
  		  function dayOfYearFromWeekInfo(config) {
  		    var w, weekYear, week, weekday, dow, doy, temp, weekdayOverflow, curWeek;
  		    w = config._w;
  		    if (w.GG != null || w.W != null || w.E != null) {
  		      dow = 1;
  		      doy = 4;

  		      // TODO: We need to take the current isoWeekYear, but that depends on
  		      // how we interpret now (local, utc, fixed offset). So create
  		      // a now version of current config (take local/utc/offset flags, and
  		      // create now).
  		      weekYear = defaults(w.GG, config._a[YEAR], weekOfYear(createLocal(), 1, 4).year);
  		      week = defaults(w.W, 1);
  		      weekday = defaults(w.E, 1);
  		      if (weekday < 1 || weekday > 7) {
  		        weekdayOverflow = true;
  		      }
  		    } else {
  		      dow = config._locale._week.dow;
  		      doy = config._locale._week.doy;
  		      curWeek = weekOfYear(createLocal(), dow, doy);
  		      weekYear = defaults(w.gg, config._a[YEAR], curWeek.year);

  		      // Default to current week.
  		      week = defaults(w.w, curWeek.week);
  		      if (w.d != null) {
  		        // weekday -- low day numbers are considered next week
  		        weekday = w.d;
  		        if (weekday < 0 || weekday > 6) {
  		          weekdayOverflow = true;
  		        }
  		      } else if (w.e != null) {
  		        // local weekday -- counting starts from beginning of week
  		        weekday = w.e + dow;
  		        if (w.e < 0 || w.e > 6) {
  		          weekdayOverflow = true;
  		        }
  		      } else {
  		        // default to beginning of week
  		        weekday = dow;
  		      }
  		    }
  		    if (week < 1 || week > weeksInYear(weekYear, dow, doy)) {
  		      getParsingFlags(config)._overflowWeeks = true;
  		    } else if (weekdayOverflow != null) {
  		      getParsingFlags(config)._overflowWeekday = true;
  		    } else {
  		      temp = dayOfYearFromWeeks(weekYear, week, weekday, dow, doy);
  		      config._a[YEAR] = temp.year;
  		      config._dayOfYear = temp.dayOfYear;
  		    }
  		  }

  		  // constant that refers to the ISO standard
  		  hooks.ISO_8601 = function () {};

  		  // constant that refers to the RFC 2822 form
  		  hooks.RFC_2822 = function () {};

  		  // date from string and format string
  		  function configFromStringAndFormat(config) {
  		    // TODO: Move this to another part of the creation flow to prevent circular deps
  		    if (config._f === hooks.ISO_8601) {
  		      configFromISO(config);
  		      return;
  		    }
  		    if (config._f === hooks.RFC_2822) {
  		      configFromRFC2822(config);
  		      return;
  		    }
  		    config._a = [];
  		    getParsingFlags(config).empty = true;

  		    // This array is used to make a Date, either with `new Date` or `Date.UTC`
  		    var string = '' + config._i,
  		      i,
  		      parsedInput,
  		      tokens,
  		      token,
  		      skipped,
  		      stringLength = string.length,
  		      totalParsedInputLength = 0,
  		      era,
  		      tokenLen;
  		    tokens = expandFormat(config._f, config._locale).match(formattingTokens) || [];
  		    tokenLen = tokens.length;
  		    for (i = 0; i < tokenLen; i++) {
  		      token = tokens[i];
  		      parsedInput = (string.match(getParseRegexForToken(token, config)) || [])[0];
  		      if (parsedInput) {
  		        skipped = string.substr(0, string.indexOf(parsedInput));
  		        if (skipped.length > 0) {
  		          getParsingFlags(config).unusedInput.push(skipped);
  		        }
  		        string = string.slice(string.indexOf(parsedInput) + parsedInput.length);
  		        totalParsedInputLength += parsedInput.length;
  		      }
  		      // don't parse if it's not a known token
  		      if (formatTokenFunctions[token]) {
  		        if (parsedInput) {
  		          getParsingFlags(config).empty = false;
  		        } else {
  		          getParsingFlags(config).unusedTokens.push(token);
  		        }
  		        addTimeToArrayFromToken(token, parsedInput, config);
  		      } else if (config._strict && !parsedInput) {
  		        getParsingFlags(config).unusedTokens.push(token);
  		      }
  		    }

  		    // add remaining unparsed input length to the string
  		    getParsingFlags(config).charsLeftOver = stringLength - totalParsedInputLength;
  		    if (string.length > 0) {
  		      getParsingFlags(config).unusedInput.push(string);
  		    }

  		    // clear _12h flag if hour is <= 12
  		    if (config._a[HOUR] <= 12 && getParsingFlags(config).bigHour === true && config._a[HOUR] > 0) {
  		      getParsingFlags(config).bigHour = undefined;
  		    }
  		    getParsingFlags(config).parsedDateParts = config._a.slice(0);
  		    getParsingFlags(config).meridiem = config._meridiem;
  		    // handle meridiem
  		    config._a[HOUR] = meridiemFixWrap(config._locale, config._a[HOUR], config._meridiem);

  		    // handle era
  		    era = getParsingFlags(config).era;
  		    if (era !== null) {
  		      config._a[YEAR] = config._locale.erasConvertYear(era, config._a[YEAR]);
  		    }
  		    configFromArray(config);
  		    checkOverflow(config);
  		  }
  		  function meridiemFixWrap(locale, hour, meridiem) {
  		    var isPm;
  		    if (meridiem == null) {
  		      // nothing to do
  		      return hour;
  		    }
  		    if (locale.meridiemHour != null) {
  		      return locale.meridiemHour(hour, meridiem);
  		    } else if (locale.isPM != null) {
  		      // Fallback
  		      isPm = locale.isPM(meridiem);
  		      if (isPm && hour < 12) {
  		        hour += 12;
  		      }
  		      if (!isPm && hour === 12) {
  		        hour = 0;
  		      }
  		      return hour;
  		    } else {
  		      // this is not supposed to happen
  		      return hour;
  		    }
  		  }

  		  // date from string and array of format strings
  		  function configFromStringAndArray(config) {
  		    var tempConfig,
  		      bestMoment,
  		      scoreToBeat,
  		      i,
  		      currentScore,
  		      validFormatFound,
  		      bestFormatIsValid = false,
  		      configfLen = config._f.length;
  		    if (configfLen === 0) {
  		      getParsingFlags(config).invalidFormat = true;
  		      config._d = new Date(NaN);
  		      return;
  		    }
  		    for (i = 0; i < configfLen; i++) {
  		      currentScore = 0;
  		      validFormatFound = false;
  		      tempConfig = copyConfig({}, config);
  		      if (config._useUTC != null) {
  		        tempConfig._useUTC = config._useUTC;
  		      }
  		      tempConfig._f = config._f[i];
  		      configFromStringAndFormat(tempConfig);
  		      if (isValid(tempConfig)) {
  		        validFormatFound = true;
  		      }

  		      // if there is any input that was not parsed add a penalty for that format
  		      currentScore += getParsingFlags(tempConfig).charsLeftOver;

  		      //or tokens
  		      currentScore += getParsingFlags(tempConfig).unusedTokens.length * 10;
  		      getParsingFlags(tempConfig).score = currentScore;
  		      if (!bestFormatIsValid) {
  		        if (scoreToBeat == null || currentScore < scoreToBeat || validFormatFound) {
  		          scoreToBeat = currentScore;
  		          bestMoment = tempConfig;
  		          if (validFormatFound) {
  		            bestFormatIsValid = true;
  		          }
  		        }
  		      } else {
  		        if (currentScore < scoreToBeat) {
  		          scoreToBeat = currentScore;
  		          bestMoment = tempConfig;
  		        }
  		      }
  		    }
  		    extend(config, bestMoment || tempConfig);
  		  }
  		  function configFromObject(config) {
  		    if (config._d) {
  		      return;
  		    }
  		    var i = normalizeObjectUnits(config._i),
  		      dayOrDate = i.day === undefined ? i.date : i.day;
  		    config._a = map([i.year, i.month, dayOrDate, i.hour, i.minute, i.second, i.millisecond], function (obj) {
  		      return obj && parseInt(obj, 10);
  		    });
  		    configFromArray(config);
  		  }
  		  function createFromConfig(config) {
  		    var res = new Moment(checkOverflow(prepareConfig(config)));
  		    if (res._nextDay) {
  		      // Adding is smart enough around DST
  		      res.add(1, 'd');
  		      res._nextDay = undefined;
  		    }
  		    return res;
  		  }
  		  function prepareConfig(config) {
  		    var input = config._i,
  		      format = config._f;
  		    config._locale = config._locale || getLocale(config._l);
  		    if (input === null || format === undefined && input === '') {
  		      return createInvalid({
  		        nullInput: true
  		      });
  		    }
  		    if (typeof input === 'string') {
  		      config._i = input = config._locale.preparse(input);
  		    }
  		    if (isMoment(input)) {
  		      return new Moment(checkOverflow(input));
  		    } else if (isDate(input)) {
  		      config._d = input;
  		    } else if (isArray(format)) {
  		      configFromStringAndArray(config);
  		    } else if (format) {
  		      configFromStringAndFormat(config);
  		    } else {
  		      configFromInput(config);
  		    }
  		    if (!isValid(config)) {
  		      config._d = null;
  		    }
  		    return config;
  		  }
  		  function configFromInput(config) {
  		    var input = config._i;
  		    if (isUndefined(input)) {
  		      config._d = new Date(hooks.now());
  		    } else if (isDate(input)) {
  		      config._d = new Date(input.valueOf());
  		    } else if (typeof input === 'string') {
  		      configFromString(config);
  		    } else if (isArray(input)) {
  		      config._a = map(input.slice(0), function (obj) {
  		        return parseInt(obj, 10);
  		      });
  		      configFromArray(config);
  		    } else if (isObject(input)) {
  		      configFromObject(config);
  		    } else if (isNumber(input)) {
  		      // from milliseconds
  		      config._d = new Date(input);
  		    } else {
  		      hooks.createFromInputFallback(config);
  		    }
  		  }
  		  function createLocalOrUTC(input, format, locale, strict, isUTC) {
  		    var c = {};
  		    if (format === true || format === false) {
  		      strict = format;
  		      format = undefined;
  		    }
  		    if (locale === true || locale === false) {
  		      strict = locale;
  		      locale = undefined;
  		    }
  		    if (isObject(input) && isObjectEmpty(input) || isArray(input) && input.length === 0) {
  		      input = undefined;
  		    }
  		    // object construction must be done this way.
  		    // https://github.com/moment/moment/issues/1423
  		    c._isAMomentObject = true;
  		    c._useUTC = c._isUTC = isUTC;
  		    c._l = locale;
  		    c._i = input;
  		    c._f = format;
  		    c._strict = strict;
  		    return createFromConfig(c);
  		  }
  		  function createLocal(input, format, locale, strict) {
  		    return createLocalOrUTC(input, format, locale, strict, false);
  		  }
  		  var prototypeMin = deprecate('moment().min is deprecated, use moment.max instead. http://momentjs.com/guides/#/warnings/min-max/', function () {
  		      var other = createLocal.apply(null, arguments);
  		      if (this.isValid() && other.isValid()) {
  		        return other < this ? this : other;
  		      } else {
  		        return createInvalid();
  		      }
  		    }),
  		    prototypeMax = deprecate('moment().max is deprecated, use moment.min instead. http://momentjs.com/guides/#/warnings/min-max/', function () {
  		      var other = createLocal.apply(null, arguments);
  		      if (this.isValid() && other.isValid()) {
  		        return other > this ? this : other;
  		      } else {
  		        return createInvalid();
  		      }
  		    });

  		  // Pick a moment m from moments so that m[fn](other) is true for all
  		  // other. This relies on the function fn to be transitive.
  		  //
  		  // moments should either be an array of moment objects or an array, whose
  		  // first element is an array of moment objects.
  		  function pickBy(fn, moments) {
  		    var res, i;
  		    if (moments.length === 1 && isArray(moments[0])) {
  		      moments = moments[0];
  		    }
  		    if (!moments.length) {
  		      return createLocal();
  		    }
  		    res = moments[0];
  		    for (i = 1; i < moments.length; ++i) {
  		      if (!moments[i].isValid() || moments[i][fn](res)) {
  		        res = moments[i];
  		      }
  		    }
  		    return res;
  		  }

  		  // TODO: Use [].sort instead?
  		  function min() {
  		    var args = [].slice.call(arguments, 0);
  		    return pickBy('isBefore', args);
  		  }
  		  function max() {
  		    var args = [].slice.call(arguments, 0);
  		    return pickBy('isAfter', args);
  		  }
  		  var now = function () {
  		    return Date.now ? Date.now() : +new Date();
  		  };
  		  var ordering = ['year', 'quarter', 'month', 'week', 'day', 'hour', 'minute', 'second', 'millisecond'];
  		  function isDurationValid(m) {
  		    var key,
  		      unitHasDecimal = false,
  		      i,
  		      orderLen = ordering.length;
  		    for (key in m) {
  		      if (hasOwnProp(m, key) && !(indexOf.call(ordering, key) !== -1 && (m[key] == null || !isNaN(m[key])))) {
  		        return false;
  		      }
  		    }
  		    for (i = 0; i < orderLen; ++i) {
  		      if (m[ordering[i]]) {
  		        if (unitHasDecimal) {
  		          return false; // only allow non-integers for smallest unit
  		        }
  		        if (parseFloat(m[ordering[i]]) !== toInt(m[ordering[i]])) {
  		          unitHasDecimal = true;
  		        }
  		      }
  		    }
  		    return true;
  		  }
  		  function isValid$1() {
  		    return this._isValid;
  		  }
  		  function createInvalid$1() {
  		    return createDuration(NaN);
  		  }
  		  function Duration(duration) {
  		    var normalizedInput = normalizeObjectUnits(duration),
  		      years = normalizedInput.year || 0,
  		      quarters = normalizedInput.quarter || 0,
  		      months = normalizedInput.month || 0,
  		      weeks = normalizedInput.week || normalizedInput.isoWeek || 0,
  		      days = normalizedInput.day || 0,
  		      hours = normalizedInput.hour || 0,
  		      minutes = normalizedInput.minute || 0,
  		      seconds = normalizedInput.second || 0,
  		      milliseconds = normalizedInput.millisecond || 0;
  		    this._isValid = isDurationValid(normalizedInput);

  		    // representation for dateAddRemove
  		    this._milliseconds = +milliseconds + seconds * 1e3 +
  		    // 1000
  		    minutes * 6e4 +
  		    // 1000 * 60
  		    hours * 1000 * 60 * 60; //using 1000 * 60 * 60 instead of 36e5 to avoid floating point rounding errors https://github.com/moment/moment/issues/2978
  		    // Because of dateAddRemove treats 24 hours as different from a
  		    // day when working around DST, we need to store them separately
  		    this._days = +days + weeks * 7;
  		    // It is impossible to translate months into days without knowing
  		    // which months you are are talking about, so we have to store
  		    // it separately.
  		    this._months = +months + quarters * 3 + years * 12;
  		    this._data = {};
  		    this._locale = getLocale();
  		    this._bubble();
  		  }
  		  function isDuration(obj) {
  		    return obj instanceof Duration;
  		  }
  		  function absRound(number) {
  		    if (number < 0) {
  		      return Math.round(-1 * number) * -1;
  		    } else {
  		      return Math.round(number);
  		    }
  		  }

  		  // compare two arrays, return the number of differences
  		  function compareArrays(array1, array2, dontConvert) {
  		    var len = Math.min(array1.length, array2.length),
  		      lengthDiff = Math.abs(array1.length - array2.length),
  		      diffs = 0,
  		      i;
  		    for (i = 0; i < len; i++) {
  		      if (toInt(array1[i]) !== toInt(array2[i])) {
  		        diffs++;
  		      }
  		    }
  		    return diffs + lengthDiff;
  		  }

  		  // FORMATTING

  		  function offset(token, separator) {
  		    addFormatToken(token, 0, 0, function () {
  		      var offset = this.utcOffset(),
  		        sign = '+';
  		      if (offset < 0) {
  		        offset = -offset;
  		        sign = '-';
  		      }
  		      return sign + zeroFill(~~(offset / 60), 2) + separator + zeroFill(~~offset % 60, 2);
  		    });
  		  }
  		  offset('Z', ':');
  		  offset('ZZ', '');

  		  // PARSING

  		  addRegexToken('Z', matchShortOffset);
  		  addRegexToken('ZZ', matchShortOffset);
  		  addParseToken(['Z', 'ZZ'], function (input, array, config) {
  		    config._useUTC = true;
  		    config._tzm = offsetFromString(matchShortOffset, input);
  		  });

  		  // HELPERS

  		  // timezone chunker
  		  // '+10:00' > ['10',  '00']
  		  // '-1530'  > ['-15', '30']
  		  var chunkOffset = /([\+\-]|\d\d)/gi;
  		  function offsetFromString(matcher, string) {
  		    var matches = (string || '').match(matcher),
  		      chunk,
  		      parts,
  		      minutes;
  		    if (matches === null) {
  		      return null;
  		    }
  		    chunk = matches[matches.length - 1] || [];
  		    parts = (chunk + '').match(chunkOffset) || ['-', 0, 0];
  		    minutes = +(parts[1] * 60) + toInt(parts[2]);
  		    return minutes === 0 ? 0 : parts[0] === '+' ? minutes : -minutes;
  		  }

  		  // Return a moment from input, that is local/utc/zone equivalent to model.
  		  function cloneWithOffset(input, model) {
  		    var res, diff;
  		    if (model._isUTC) {
  		      res = model.clone();
  		      diff = (isMoment(input) || isDate(input) ? input.valueOf() : createLocal(input).valueOf()) - res.valueOf();
  		      // Use low-level api, because this fn is low-level api.
  		      res._d.setTime(res._d.valueOf() + diff);
  		      hooks.updateOffset(res, false);
  		      return res;
  		    } else {
  		      return createLocal(input).local();
  		    }
  		  }
  		  function getDateOffset(m) {
  		    // On Firefox.24 Date#getTimezoneOffset returns a floating point.
  		    // https://github.com/moment/moment/pull/1871
  		    return -Math.round(m._d.getTimezoneOffset());
  		  }

  		  // HOOKS

  		  // This function will be called whenever a moment is mutated.
  		  // It is intended to keep the offset in sync with the timezone.
  		  hooks.updateOffset = function () {};

  		  // MOMENTS

  		  // keepLocalTime = true means only change the timezone, without
  		  // affecting the local hour. So 5:31:26 +0300 --[utcOffset(2, true)]-->
  		  // 5:31:26 +0200 It is possible that 5:31:26 doesn't exist with offset
  		  // +0200, so we adjust the time as needed, to be valid.
  		  //
  		  // Keeping the time actually adds/subtracts (one hour)
  		  // from the actual represented time. That is why we call updateOffset
  		  // a second time. In case it wants us to change the offset again
  		  // _changeInProgress == true case, then we have to adjust, because
  		  // there is no such time in the given timezone.
  		  function getSetOffset(input, keepLocalTime, keepMinutes) {
  		    var offset = this._offset || 0,
  		      localAdjust;
  		    if (!this.isValid()) {
  		      return input != null ? this : NaN;
  		    }
  		    if (input != null) {
  		      if (typeof input === 'string') {
  		        input = offsetFromString(matchShortOffset, input);
  		        if (input === null) {
  		          return this;
  		        }
  		      } else if (Math.abs(input) < 16 && !keepMinutes) {
  		        input = input * 60;
  		      }
  		      if (!this._isUTC && keepLocalTime) {
  		        localAdjust = getDateOffset(this);
  		      }
  		      this._offset = input;
  		      this._isUTC = true;
  		      if (localAdjust != null) {
  		        this.add(localAdjust, 'm');
  		      }
  		      if (offset !== input) {
  		        if (!keepLocalTime || this._changeInProgress) {
  		          addSubtract(this, createDuration(input - offset, 'm'), 1, false);
  		        } else if (!this._changeInProgress) {
  		          this._changeInProgress = true;
  		          hooks.updateOffset(this, true);
  		          this._changeInProgress = null;
  		        }
  		      }
  		      return this;
  		    } else {
  		      return this._isUTC ? offset : getDateOffset(this);
  		    }
  		  }
  		  function getSetZone(input, keepLocalTime) {
  		    if (input != null) {
  		      if (typeof input !== 'string') {
  		        input = -input;
  		      }
  		      this.utcOffset(input, keepLocalTime);
  		      return this;
  		    } else {
  		      return -this.utcOffset();
  		    }
  		  }
  		  function setOffsetToUTC(keepLocalTime) {
  		    return this.utcOffset(0, keepLocalTime);
  		  }
  		  function setOffsetToLocal(keepLocalTime) {
  		    if (this._isUTC) {
  		      this.utcOffset(0, keepLocalTime);
  		      this._isUTC = false;
  		      if (keepLocalTime) {
  		        this.subtract(getDateOffset(this), 'm');
  		      }
  		    }
  		    return this;
  		  }
  		  function setOffsetToParsedOffset() {
  		    if (this._tzm != null) {
  		      this.utcOffset(this._tzm, false, true);
  		    } else if (typeof this._i === 'string') {
  		      var tZone = offsetFromString(matchOffset, this._i);
  		      if (tZone != null) {
  		        this.utcOffset(tZone);
  		      } else {
  		        this.utcOffset(0, true);
  		      }
  		    }
  		    return this;
  		  }
  		  function hasAlignedHourOffset(input) {
  		    if (!this.isValid()) {
  		      return false;
  		    }
  		    input = input ? createLocal(input).utcOffset() : 0;
  		    return (this.utcOffset() - input) % 60 === 0;
  		  }
  		  function isDaylightSavingTime() {
  		    return this.utcOffset() > this.clone().month(0).utcOffset() || this.utcOffset() > this.clone().month(5).utcOffset();
  		  }
  		  function isDaylightSavingTimeShifted() {
  		    if (!isUndefined(this._isDSTShifted)) {
  		      return this._isDSTShifted;
  		    }
  		    var c = {},
  		      other;
  		    copyConfig(c, this);
  		    c = prepareConfig(c);
  		    if (c._a) {
  		      other = c._isUTC ? createUTC(c._a) : createLocal(c._a);
  		      this._isDSTShifted = this.isValid() && compareArrays(c._a, other.toArray()) > 0;
  		    } else {
  		      this._isDSTShifted = false;
  		    }
  		    return this._isDSTShifted;
  		  }
  		  function isLocal() {
  		    return this.isValid() ? !this._isUTC : false;
  		  }
  		  function isUtcOffset() {
  		    return this.isValid() ? this._isUTC : false;
  		  }
  		  function isUtc() {
  		    return this.isValid() ? this._isUTC && this._offset === 0 : false;
  		  }

  		  // ASP.NET json date format regex
  		  var aspNetRegex = /^(-|\+)?(?:(\d*)[. ])?(\d+):(\d+)(?::(\d+)(\.\d*)?)?$/,
  		    // from http://docs.closure-library.googlecode.com/git/closure_goog_date_date.js.source.html
  		    // somewhat more in line with 4.4.3.2 2004 spec, but allows decimal anywhere
  		    // and further modified to allow for strings containing both week and day
  		    isoRegex = /^(-|\+)?P(?:([-+]?[0-9,.]*)Y)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)W)?(?:([-+]?[0-9,.]*)D)?(?:T(?:([-+]?[0-9,.]*)H)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)S)?)?$/;
  		  function createDuration(input, key) {
  		    var duration = input,
  		      // matching against regexp is expensive, do it on demand
  		      match = null,
  		      sign,
  		      ret,
  		      diffRes;
  		    if (isDuration(input)) {
  		      duration = {
  		        ms: input._milliseconds,
  		        d: input._days,
  		        M: input._months
  		      };
  		    } else if (isNumber(input) || !isNaN(+input)) {
  		      duration = {};
  		      if (key) {
  		        duration[key] = +input;
  		      } else {
  		        duration.milliseconds = +input;
  		      }
  		    } else if (match = aspNetRegex.exec(input)) {
  		      sign = match[1] === '-' ? -1 : 1;
  		      duration = {
  		        y: 0,
  		        d: toInt(match[DATE]) * sign,
  		        h: toInt(match[HOUR]) * sign,
  		        m: toInt(match[MINUTE]) * sign,
  		        s: toInt(match[SECOND]) * sign,
  		        ms: toInt(absRound(match[MILLISECOND] * 1000)) * sign // the millisecond decimal point is included in the match
  		      };
  		    } else if (match = isoRegex.exec(input)) {
  		      sign = match[1] === '-' ? -1 : 1;
  		      duration = {
  		        y: parseIso(match[2], sign),
  		        M: parseIso(match[3], sign),
  		        w: parseIso(match[4], sign),
  		        d: parseIso(match[5], sign),
  		        h: parseIso(match[6], sign),
  		        m: parseIso(match[7], sign),
  		        s: parseIso(match[8], sign)
  		      };
  		    } else if (duration == null) {
  		      // checks for null or undefined
  		      duration = {};
  		    } else if (typeof duration === 'object' && ('from' in duration || 'to' in duration)) {
  		      diffRes = momentsDifference(createLocal(duration.from), createLocal(duration.to));
  		      duration = {};
  		      duration.ms = diffRes.milliseconds;
  		      duration.M = diffRes.months;
  		    }
  		    ret = new Duration(duration);
  		    if (isDuration(input) && hasOwnProp(input, '_locale')) {
  		      ret._locale = input._locale;
  		    }
  		    if (isDuration(input) && hasOwnProp(input, '_isValid')) {
  		      ret._isValid = input._isValid;
  		    }
  		    return ret;
  		  }
  		  createDuration.fn = Duration.prototype;
  		  createDuration.invalid = createInvalid$1;
  		  function parseIso(inp, sign) {
  		    // We'd normally use ~~inp for this, but unfortunately it also
  		    // converts floats to ints.
  		    // inp may be undefined, so careful calling replace on it.
  		    var res = inp && parseFloat(inp.replace(',', '.'));
  		    // apply sign while we're at it
  		    return (isNaN(res) ? 0 : res) * sign;
  		  }
  		  function positiveMomentsDifference(base, other) {
  		    var res = {};
  		    res.months = other.month() - base.month() + (other.year() - base.year()) * 12;
  		    if (base.clone().add(res.months, 'M').isAfter(other)) {
  		      --res.months;
  		    }
  		    res.milliseconds = +other - +base.clone().add(res.months, 'M');
  		    return res;
  		  }
  		  function momentsDifference(base, other) {
  		    var res;
  		    if (!(base.isValid() && other.isValid())) {
  		      return {
  		        milliseconds: 0,
  		        months: 0
  		      };
  		    }
  		    other = cloneWithOffset(other, base);
  		    if (base.isBefore(other)) {
  		      res = positiveMomentsDifference(base, other);
  		    } else {
  		      res = positiveMomentsDifference(other, base);
  		      res.milliseconds = -res.milliseconds;
  		      res.months = -res.months;
  		    }
  		    return res;
  		  }

  		  // TODO: remove 'name' arg after deprecation is removed
  		  function createAdder(direction, name) {
  		    return function (val, period) {
  		      var dur, tmp;
  		      //invert the arguments, but complain about it
  		      if (period !== null && !isNaN(+period)) {
  		        deprecateSimple(name, 'moment().' + name + '(period, number) is deprecated. Please use moment().' + name + '(number, period). ' + 'See http://momentjs.com/guides/#/warnings/add-inverted-param/ for more info.');
  		        tmp = val;
  		        val = period;
  		        period = tmp;
  		      }
  		      dur = createDuration(val, period);
  		      addSubtract(this, dur, direction);
  		      return this;
  		    };
  		  }
  		  function addSubtract(mom, duration, isAdding, updateOffset) {
  		    var milliseconds = duration._milliseconds,
  		      days = absRound(duration._days),
  		      months = absRound(duration._months);
  		    if (!mom.isValid()) {
  		      // No op
  		      return;
  		    }
  		    updateOffset = updateOffset == null ? true : updateOffset;
  		    if (months) {
  		      setMonth(mom, get(mom, 'Month') + months * isAdding);
  		    }
  		    if (days) {
  		      set$1(mom, 'Date', get(mom, 'Date') + days * isAdding);
  		    }
  		    if (milliseconds) {
  		      mom._d.setTime(mom._d.valueOf() + milliseconds * isAdding);
  		    }
  		    if (updateOffset) {
  		      hooks.updateOffset(mom, days || months);
  		    }
  		  }
  		  var add = createAdder(1, 'add'),
  		    subtract = createAdder(-1, 'subtract');
  		  function isString(input) {
  		    return typeof input === 'string' || input instanceof String;
  		  }

  		  // type MomentInput = Moment | Date | string | number | (number | string)[] | MomentInputObject | void; // null | undefined
  		  function isMomentInput(input) {
  		    return isMoment(input) || isDate(input) || isString(input) || isNumber(input) || isNumberOrStringArray(input) || isMomentInputObject(input) || input === null || input === undefined;
  		  }
  		  function isMomentInputObject(input) {
  		    var objectTest = isObject(input) && !isObjectEmpty(input),
  		      propertyTest = false,
  		      properties = ['years', 'year', 'y', 'months', 'month', 'M', 'days', 'day', 'd', 'dates', 'date', 'D', 'hours', 'hour', 'h', 'minutes', 'minute', 'm', 'seconds', 'second', 's', 'milliseconds', 'millisecond', 'ms'],
  		      i,
  		      property,
  		      propertyLen = properties.length;
  		    for (i = 0; i < propertyLen; i += 1) {
  		      property = properties[i];
  		      propertyTest = propertyTest || hasOwnProp(input, property);
  		    }
  		    return objectTest && propertyTest;
  		  }
  		  function isNumberOrStringArray(input) {
  		    var arrayTest = isArray(input),
  		      dataTypeTest = false;
  		    if (arrayTest) {
  		      dataTypeTest = input.filter(function (item) {
  		        return !isNumber(item) && isString(input);
  		      }).length === 0;
  		    }
  		    return arrayTest && dataTypeTest;
  		  }
  		  function isCalendarSpec(input) {
  		    var objectTest = isObject(input) && !isObjectEmpty(input),
  		      propertyTest = false,
  		      properties = ['sameDay', 'nextDay', 'lastDay', 'nextWeek', 'lastWeek', 'sameElse'],
  		      i,
  		      property;
  		    for (i = 0; i < properties.length; i += 1) {
  		      property = properties[i];
  		      propertyTest = propertyTest || hasOwnProp(input, property);
  		    }
  		    return objectTest && propertyTest;
  		  }
  		  function getCalendarFormat(myMoment, now) {
  		    var diff = myMoment.diff(now, 'days', true);
  		    return diff < -6 ? 'sameElse' : diff < -1 ? 'lastWeek' : diff < 0 ? 'lastDay' : diff < 1 ? 'sameDay' : diff < 2 ? 'nextDay' : diff < 7 ? 'nextWeek' : 'sameElse';
  		  }
  		  function calendar$1(time, formats) {
  		    // Support for single parameter, formats only overload to the calendar function
  		    if (arguments.length === 1) {
  		      if (!arguments[0]) {
  		        time = undefined;
  		        formats = undefined;
  		      } else if (isMomentInput(arguments[0])) {
  		        time = arguments[0];
  		        formats = undefined;
  		      } else if (isCalendarSpec(arguments[0])) {
  		        formats = arguments[0];
  		        time = undefined;
  		      }
  		    }
  		    // We want to compare the start of today, vs this.
  		    // Getting start-of-today depends on whether we're local/utc/offset or not.
  		    var now = time || createLocal(),
  		      sod = cloneWithOffset(now, this).startOf('day'),
  		      format = hooks.calendarFormat(this, sod) || 'sameElse',
  		      output = formats && (isFunction(formats[format]) ? formats[format].call(this, now) : formats[format]);
  		    return this.format(output || this.localeData().calendar(format, this, createLocal(now)));
  		  }
  		  function clone() {
  		    return new Moment(this);
  		  }
  		  function isAfter(input, units) {
  		    var localInput = isMoment(input) ? input : createLocal(input);
  		    if (!(this.isValid() && localInput.isValid())) {
  		      return false;
  		    }
  		    units = normalizeUnits(units) || 'millisecond';
  		    if (units === 'millisecond') {
  		      return this.valueOf() > localInput.valueOf();
  		    } else {
  		      return localInput.valueOf() < this.clone().startOf(units).valueOf();
  		    }
  		  }
  		  function isBefore(input, units) {
  		    var localInput = isMoment(input) ? input : createLocal(input);
  		    if (!(this.isValid() && localInput.isValid())) {
  		      return false;
  		    }
  		    units = normalizeUnits(units) || 'millisecond';
  		    if (units === 'millisecond') {
  		      return this.valueOf() < localInput.valueOf();
  		    } else {
  		      return this.clone().endOf(units).valueOf() < localInput.valueOf();
  		    }
  		  }
  		  function isBetween(from, to, units, inclusivity) {
  		    var localFrom = isMoment(from) ? from : createLocal(from),
  		      localTo = isMoment(to) ? to : createLocal(to);
  		    if (!(this.isValid() && localFrom.isValid() && localTo.isValid())) {
  		      return false;
  		    }
  		    inclusivity = inclusivity || '()';
  		    return (inclusivity[0] === '(' ? this.isAfter(localFrom, units) : !this.isBefore(localFrom, units)) && (inclusivity[1] === ')' ? this.isBefore(localTo, units) : !this.isAfter(localTo, units));
  		  }
  		  function isSame(input, units) {
  		    var localInput = isMoment(input) ? input : createLocal(input),
  		      inputMs;
  		    if (!(this.isValid() && localInput.isValid())) {
  		      return false;
  		    }
  		    units = normalizeUnits(units) || 'millisecond';
  		    if (units === 'millisecond') {
  		      return this.valueOf() === localInput.valueOf();
  		    } else {
  		      inputMs = localInput.valueOf();
  		      return this.clone().startOf(units).valueOf() <= inputMs && inputMs <= this.clone().endOf(units).valueOf();
  		    }
  		  }
  		  function isSameOrAfter(input, units) {
  		    return this.isSame(input, units) || this.isAfter(input, units);
  		  }
  		  function isSameOrBefore(input, units) {
  		    return this.isSame(input, units) || this.isBefore(input, units);
  		  }
  		  function diff(input, units, asFloat) {
  		    var that, zoneDelta, output;
  		    if (!this.isValid()) {
  		      return NaN;
  		    }
  		    that = cloneWithOffset(input, this);
  		    if (!that.isValid()) {
  		      return NaN;
  		    }
  		    zoneDelta = (that.utcOffset() - this.utcOffset()) * 6e4;
  		    units = normalizeUnits(units);
  		    switch (units) {
  		      case 'year':
  		        output = monthDiff(this, that) / 12;
  		        break;
  		      case 'month':
  		        output = monthDiff(this, that);
  		        break;
  		      case 'quarter':
  		        output = monthDiff(this, that) / 3;
  		        break;
  		      case 'second':
  		        output = (this - that) / 1e3;
  		        break;
  		      // 1000
  		      case 'minute':
  		        output = (this - that) / 6e4;
  		        break;
  		      // 1000 * 60
  		      case 'hour':
  		        output = (this - that) / 36e5;
  		        break;
  		      // 1000 * 60 * 60
  		      case 'day':
  		        output = (this - that - zoneDelta) / 864e5;
  		        break;
  		      // 1000 * 60 * 60 * 24, negate dst
  		      case 'week':
  		        output = (this - that - zoneDelta) / 6048e5;
  		        break;
  		      // 1000 * 60 * 60 * 24 * 7, negate dst
  		      default:
  		        output = this - that;
  		    }
  		    return asFloat ? output : absFloor(output);
  		  }
  		  function monthDiff(a, b) {
  		    if (a.date() < b.date()) {
  		      // end-of-month calculations work correct when the start month has more
  		      // days than the end month.
  		      return -monthDiff(b, a);
  		    }
  		    // difference in months
  		    var wholeMonthDiff = (b.year() - a.year()) * 12 + (b.month() - a.month()),
  		      // b is in (anchor - 1 month, anchor + 1 month)
  		      anchor = a.clone().add(wholeMonthDiff, 'months'),
  		      anchor2,
  		      adjust;
  		    if (b - anchor < 0) {
  		      anchor2 = a.clone().add(wholeMonthDiff - 1, 'months');
  		      // linear across the month
  		      adjust = (b - anchor) / (anchor - anchor2);
  		    } else {
  		      anchor2 = a.clone().add(wholeMonthDiff + 1, 'months');
  		      // linear across the month
  		      adjust = (b - anchor) / (anchor2 - anchor);
  		    }

  		    //check for negative zero, return zero if negative zero
  		    return -(wholeMonthDiff + adjust) || 0;
  		  }
  		  hooks.defaultFormat = 'YYYY-MM-DDTHH:mm:ssZ';
  		  hooks.defaultFormatUtc = 'YYYY-MM-DDTHH:mm:ss[Z]';
  		  function toString() {
  		    return this.clone().locale('en').format('ddd MMM DD YYYY HH:mm:ss [GMT]ZZ');
  		  }
  		  function toISOString(keepOffset) {
  		    if (!this.isValid()) {
  		      return null;
  		    }
  		    var utc = keepOffset !== true,
  		      m = utc ? this.clone().utc() : this;
  		    if (m.year() < 0 || m.year() > 9999) {
  		      return formatMoment(m, utc ? 'YYYYYY-MM-DD[T]HH:mm:ss.SSS[Z]' : 'YYYYYY-MM-DD[T]HH:mm:ss.SSSZ');
  		    }
  		    if (isFunction(Date.prototype.toISOString)) {
  		      // native implementation is ~50x faster, use it when we can
  		      if (utc) {
  		        return this.toDate().toISOString();
  		      } else {
  		        return new Date(this.valueOf() + this.utcOffset() * 60 * 1000).toISOString().replace('Z', formatMoment(m, 'Z'));
  		      }
  		    }
  		    return formatMoment(m, utc ? 'YYYY-MM-DD[T]HH:mm:ss.SSS[Z]' : 'YYYY-MM-DD[T]HH:mm:ss.SSSZ');
  		  }

  		  /**
  		   * Return a human readable representation of a moment that can
  		   * also be evaluated to get a new moment which is the same
  		   *
  		   * @link https://nodejs.org/dist/latest/docs/api/util.html#util_custom_inspect_function_on_objects
  		   */
  		  function inspect() {
  		    if (!this.isValid()) {
  		      return 'moment.invalid(/* ' + this._i + ' */)';
  		    }
  		    var func = 'moment',
  		      zone = '',
  		      prefix,
  		      year,
  		      datetime,
  		      suffix;
  		    if (!this.isLocal()) {
  		      func = this.utcOffset() === 0 ? 'moment.utc' : 'moment.parseZone';
  		      zone = 'Z';
  		    }
  		    prefix = '[' + func + '("]';
  		    year = 0 <= this.year() && this.year() <= 9999 ? 'YYYY' : 'YYYYYY';
  		    datetime = '-MM-DD[T]HH:mm:ss.SSS';
  		    suffix = zone + '[")]';
  		    return this.format(prefix + year + datetime + suffix);
  		  }
  		  function format(inputString) {
  		    if (!inputString) {
  		      inputString = this.isUtc() ? hooks.defaultFormatUtc : hooks.defaultFormat;
  		    }
  		    var output = formatMoment(this, inputString);
  		    return this.localeData().postformat(output);
  		  }
  		  function from(time, withoutSuffix) {
  		    if (this.isValid() && (isMoment(time) && time.isValid() || createLocal(time).isValid())) {
  		      return createDuration({
  		        to: this,
  		        from: time
  		      }).locale(this.locale()).humanize(!withoutSuffix);
  		    } else {
  		      return this.localeData().invalidDate();
  		    }
  		  }
  		  function fromNow(withoutSuffix) {
  		    return this.from(createLocal(), withoutSuffix);
  		  }
  		  function to(time, withoutSuffix) {
  		    if (this.isValid() && (isMoment(time) && time.isValid() || createLocal(time).isValid())) {
  		      return createDuration({
  		        from: this,
  		        to: time
  		      }).locale(this.locale()).humanize(!withoutSuffix);
  		    } else {
  		      return this.localeData().invalidDate();
  		    }
  		  }
  		  function toNow(withoutSuffix) {
  		    return this.to(createLocal(), withoutSuffix);
  		  }

  		  // If passed a locale key, it will set the locale for this
  		  // instance.  Otherwise, it will return the locale configuration
  		  // variables for this instance.
  		  function locale(key) {
  		    var newLocaleData;
  		    if (key === undefined) {
  		      return this._locale._abbr;
  		    } else {
  		      newLocaleData = getLocale(key);
  		      if (newLocaleData != null) {
  		        this._locale = newLocaleData;
  		      }
  		      return this;
  		    }
  		  }
  		  var lang = deprecate('moment().lang() is deprecated. Instead, use moment().localeData() to get the language configuration. Use moment().locale() to change languages.', function (key) {
  		    if (key === undefined) {
  		      return this.localeData();
  		    } else {
  		      return this.locale(key);
  		    }
  		  });
  		  function localeData() {
  		    return this._locale;
  		  }
  		  var MS_PER_SECOND = 1000,
  		    MS_PER_MINUTE = 60 * MS_PER_SECOND,
  		    MS_PER_HOUR = 60 * MS_PER_MINUTE,
  		    MS_PER_400_YEARS = (365 * 400 + 97) * 24 * MS_PER_HOUR;

  		  // actual modulo - handles negative numbers (for dates before 1970):
  		  function mod$1(dividend, divisor) {
  		    return (dividend % divisor + divisor) % divisor;
  		  }
  		  function localStartOfDate(y, m, d) {
  		    // the date constructor remaps years 0-99 to 1900-1999
  		    if (y < 100 && y >= 0) {
  		      // preserve leap years using a full 400 year cycle, then reset
  		      return new Date(y + 400, m, d) - MS_PER_400_YEARS;
  		    } else {
  		      return new Date(y, m, d).valueOf();
  		    }
  		  }
  		  function utcStartOfDate(y, m, d) {
  		    // Date.UTC remaps years 0-99 to 1900-1999
  		    if (y < 100 && y >= 0) {
  		      // preserve leap years using a full 400 year cycle, then reset
  		      return Date.UTC(y + 400, m, d) - MS_PER_400_YEARS;
  		    } else {
  		      return Date.UTC(y, m, d);
  		    }
  		  }
  		  function startOf(units) {
  		    var time, startOfDate;
  		    units = normalizeUnits(units);
  		    if (units === undefined || units === 'millisecond' || !this.isValid()) {
  		      return this;
  		    }
  		    startOfDate = this._isUTC ? utcStartOfDate : localStartOfDate;
  		    switch (units) {
  		      case 'year':
  		        time = startOfDate(this.year(), 0, 1);
  		        break;
  		      case 'quarter':
  		        time = startOfDate(this.year(), this.month() - this.month() % 3, 1);
  		        break;
  		      case 'month':
  		        time = startOfDate(this.year(), this.month(), 1);
  		        break;
  		      case 'week':
  		        time = startOfDate(this.year(), this.month(), this.date() - this.weekday());
  		        break;
  		      case 'isoWeek':
  		        time = startOfDate(this.year(), this.month(), this.date() - (this.isoWeekday() - 1));
  		        break;
  		      case 'day':
  		      case 'date':
  		        time = startOfDate(this.year(), this.month(), this.date());
  		        break;
  		      case 'hour':
  		        time = this._d.valueOf();
  		        time -= mod$1(time + (this._isUTC ? 0 : this.utcOffset() * MS_PER_MINUTE), MS_PER_HOUR);
  		        break;
  		      case 'minute':
  		        time = this._d.valueOf();
  		        time -= mod$1(time, MS_PER_MINUTE);
  		        break;
  		      case 'second':
  		        time = this._d.valueOf();
  		        time -= mod$1(time, MS_PER_SECOND);
  		        break;
  		    }
  		    this._d.setTime(time);
  		    hooks.updateOffset(this, true);
  		    return this;
  		  }
  		  function endOf(units) {
  		    var time, startOfDate;
  		    units = normalizeUnits(units);
  		    if (units === undefined || units === 'millisecond' || !this.isValid()) {
  		      return this;
  		    }
  		    startOfDate = this._isUTC ? utcStartOfDate : localStartOfDate;
  		    switch (units) {
  		      case 'year':
  		        time = startOfDate(this.year() + 1, 0, 1) - 1;
  		        break;
  		      case 'quarter':
  		        time = startOfDate(this.year(), this.month() - this.month() % 3 + 3, 1) - 1;
  		        break;
  		      case 'month':
  		        time = startOfDate(this.year(), this.month() + 1, 1) - 1;
  		        break;
  		      case 'week':
  		        time = startOfDate(this.year(), this.month(), this.date() - this.weekday() + 7) - 1;
  		        break;
  		      case 'isoWeek':
  		        time = startOfDate(this.year(), this.month(), this.date() - (this.isoWeekday() - 1) + 7) - 1;
  		        break;
  		      case 'day':
  		      case 'date':
  		        time = startOfDate(this.year(), this.month(), this.date() + 1) - 1;
  		        break;
  		      case 'hour':
  		        time = this._d.valueOf();
  		        time += MS_PER_HOUR - mod$1(time + (this._isUTC ? 0 : this.utcOffset() * MS_PER_MINUTE), MS_PER_HOUR) - 1;
  		        break;
  		      case 'minute':
  		        time = this._d.valueOf();
  		        time += MS_PER_MINUTE - mod$1(time, MS_PER_MINUTE) - 1;
  		        break;
  		      case 'second':
  		        time = this._d.valueOf();
  		        time += MS_PER_SECOND - mod$1(time, MS_PER_SECOND) - 1;
  		        break;
  		    }
  		    this._d.setTime(time);
  		    hooks.updateOffset(this, true);
  		    return this;
  		  }
  		  function valueOf() {
  		    return this._d.valueOf() - (this._offset || 0) * 60000;
  		  }
  		  function unix() {
  		    return Math.floor(this.valueOf() / 1000);
  		  }
  		  function toDate() {
  		    return new Date(this.valueOf());
  		  }
  		  function toArray() {
  		    var m = this;
  		    return [m.year(), m.month(), m.date(), m.hour(), m.minute(), m.second(), m.millisecond()];
  		  }
  		  function toObject() {
  		    var m = this;
  		    return {
  		      years: m.year(),
  		      months: m.month(),
  		      date: m.date(),
  		      hours: m.hours(),
  		      minutes: m.minutes(),
  		      seconds: m.seconds(),
  		      milliseconds: m.milliseconds()
  		    };
  		  }
  		  function toJSON() {
  		    // new Date(NaN).toJSON() === null
  		    return this.isValid() ? this.toISOString() : null;
  		  }
  		  function isValid$2() {
  		    return isValid(this);
  		  }
  		  function parsingFlags() {
  		    return extend({}, getParsingFlags(this));
  		  }
  		  function invalidAt() {
  		    return getParsingFlags(this).overflow;
  		  }
  		  function creationData() {
  		    return {
  		      input: this._i,
  		      format: this._f,
  		      locale: this._locale,
  		      isUTC: this._isUTC,
  		      strict: this._strict
  		    };
  		  }
  		  addFormatToken('N', 0, 0, 'eraAbbr');
  		  addFormatToken('NN', 0, 0, 'eraAbbr');
  		  addFormatToken('NNN', 0, 0, 'eraAbbr');
  		  addFormatToken('NNNN', 0, 0, 'eraName');
  		  addFormatToken('NNNNN', 0, 0, 'eraNarrow');
  		  addFormatToken('y', ['y', 1], 'yo', 'eraYear');
  		  addFormatToken('y', ['yy', 2], 0, 'eraYear');
  		  addFormatToken('y', ['yyy', 3], 0, 'eraYear');
  		  addFormatToken('y', ['yyyy', 4], 0, 'eraYear');
  		  addRegexToken('N', matchEraAbbr);
  		  addRegexToken('NN', matchEraAbbr);
  		  addRegexToken('NNN', matchEraAbbr);
  		  addRegexToken('NNNN', matchEraName);
  		  addRegexToken('NNNNN', matchEraNarrow);
  		  addParseToken(['N', 'NN', 'NNN', 'NNNN', 'NNNNN'], function (input, array, config, token) {
  		    var era = config._locale.erasParse(input, token, config._strict);
  		    if (era) {
  		      getParsingFlags(config).era = era;
  		    } else {
  		      getParsingFlags(config).invalidEra = input;
  		    }
  		  });
  		  addRegexToken('y', matchUnsigned);
  		  addRegexToken('yy', matchUnsigned);
  		  addRegexToken('yyy', matchUnsigned);
  		  addRegexToken('yyyy', matchUnsigned);
  		  addRegexToken('yo', matchEraYearOrdinal);
  		  addParseToken(['y', 'yy', 'yyy', 'yyyy'], YEAR);
  		  addParseToken(['yo'], function (input, array, config, token) {
  		    var match;
  		    if (config._locale._eraYearOrdinalRegex) {
  		      match = input.match(config._locale._eraYearOrdinalRegex);
  		    }
  		    if (config._locale.eraYearOrdinalParse) {
  		      array[YEAR] = config._locale.eraYearOrdinalParse(input, match);
  		    } else {
  		      array[YEAR] = parseInt(input, 10);
  		    }
  		  });
  		  function localeEras(m, format) {
  		    var i,
  		      l,
  		      date,
  		      eras = this._eras || getLocale('en')._eras;
  		    for (i = 0, l = eras.length; i < l; ++i) {
  		      switch (typeof eras[i].since) {
  		        case 'string':
  		          // truncate time
  		          date = hooks(eras[i].since).startOf('day');
  		          eras[i].since = date.valueOf();
  		          break;
  		      }
  		      switch (typeof eras[i].until) {
  		        case 'undefined':
  		          eras[i].until = +Infinity;
  		          break;
  		        case 'string':
  		          // truncate time
  		          date = hooks(eras[i].until).startOf('day').valueOf();
  		          eras[i].until = date.valueOf();
  		          break;
  		      }
  		    }
  		    return eras;
  		  }
  		  function localeErasParse(eraName, format, strict) {
  		    var i,
  		      l,
  		      eras = this.eras(),
  		      name,
  		      abbr,
  		      narrow;
  		    eraName = eraName.toUpperCase();
  		    for (i = 0, l = eras.length; i < l; ++i) {
  		      name = eras[i].name.toUpperCase();
  		      abbr = eras[i].abbr.toUpperCase();
  		      narrow = eras[i].narrow.toUpperCase();
  		      if (strict) {
  		        switch (format) {
  		          case 'N':
  		          case 'NN':
  		          case 'NNN':
  		            if (abbr === eraName) {
  		              return eras[i];
  		            }
  		            break;
  		          case 'NNNN':
  		            if (name === eraName) {
  		              return eras[i];
  		            }
  		            break;
  		          case 'NNNNN':
  		            if (narrow === eraName) {
  		              return eras[i];
  		            }
  		            break;
  		        }
  		      } else if ([name, abbr, narrow].indexOf(eraName) >= 0) {
  		        return eras[i];
  		      }
  		    }
  		  }
  		  function localeErasConvertYear(era, year) {
  		    var dir = era.since <= era.until ? +1 : -1;
  		    if (year === undefined) {
  		      return hooks(era.since).year();
  		    } else {
  		      return hooks(era.since).year() + (year - era.offset) * dir;
  		    }
  		  }
  		  function getEraName() {
  		    var i,
  		      l,
  		      val,
  		      eras = this.localeData().eras();
  		    for (i = 0, l = eras.length; i < l; ++i) {
  		      // truncate time
  		      val = this.clone().startOf('day').valueOf();
  		      if (eras[i].since <= val && val <= eras[i].until) {
  		        return eras[i].name;
  		      }
  		      if (eras[i].until <= val && val <= eras[i].since) {
  		        return eras[i].name;
  		      }
  		    }
  		    return '';
  		  }
  		  function getEraNarrow() {
  		    var i,
  		      l,
  		      val,
  		      eras = this.localeData().eras();
  		    for (i = 0, l = eras.length; i < l; ++i) {
  		      // truncate time
  		      val = this.clone().startOf('day').valueOf();
  		      if (eras[i].since <= val && val <= eras[i].until) {
  		        return eras[i].narrow;
  		      }
  		      if (eras[i].until <= val && val <= eras[i].since) {
  		        return eras[i].narrow;
  		      }
  		    }
  		    return '';
  		  }
  		  function getEraAbbr() {
  		    var i,
  		      l,
  		      val,
  		      eras = this.localeData().eras();
  		    for (i = 0, l = eras.length; i < l; ++i) {
  		      // truncate time
  		      val = this.clone().startOf('day').valueOf();
  		      if (eras[i].since <= val && val <= eras[i].until) {
  		        return eras[i].abbr;
  		      }
  		      if (eras[i].until <= val && val <= eras[i].since) {
  		        return eras[i].abbr;
  		      }
  		    }
  		    return '';
  		  }
  		  function getEraYear() {
  		    var i,
  		      l,
  		      dir,
  		      val,
  		      eras = this.localeData().eras();
  		    for (i = 0, l = eras.length; i < l; ++i) {
  		      dir = eras[i].since <= eras[i].until ? +1 : -1;

  		      // truncate time
  		      val = this.clone().startOf('day').valueOf();
  		      if (eras[i].since <= val && val <= eras[i].until || eras[i].until <= val && val <= eras[i].since) {
  		        return (this.year() - hooks(eras[i].since).year()) * dir + eras[i].offset;
  		      }
  		    }
  		    return this.year();
  		  }
  		  function erasNameRegex(isStrict) {
  		    if (!hasOwnProp(this, '_erasNameRegex')) {
  		      computeErasParse.call(this);
  		    }
  		    return isStrict ? this._erasNameRegex : this._erasRegex;
  		  }
  		  function erasAbbrRegex(isStrict) {
  		    if (!hasOwnProp(this, '_erasAbbrRegex')) {
  		      computeErasParse.call(this);
  		    }
  		    return isStrict ? this._erasAbbrRegex : this._erasRegex;
  		  }
  		  function erasNarrowRegex(isStrict) {
  		    if (!hasOwnProp(this, '_erasNarrowRegex')) {
  		      computeErasParse.call(this);
  		    }
  		    return isStrict ? this._erasNarrowRegex : this._erasRegex;
  		  }
  		  function matchEraAbbr(isStrict, locale) {
  		    return locale.erasAbbrRegex(isStrict);
  		  }
  		  function matchEraName(isStrict, locale) {
  		    return locale.erasNameRegex(isStrict);
  		  }
  		  function matchEraNarrow(isStrict, locale) {
  		    return locale.erasNarrowRegex(isStrict);
  		  }
  		  function matchEraYearOrdinal(isStrict, locale) {
  		    return locale._eraYearOrdinalRegex || matchUnsigned;
  		  }
  		  function computeErasParse() {
  		    var abbrPieces = [],
  		      namePieces = [],
  		      narrowPieces = [],
  		      mixedPieces = [],
  		      i,
  		      l,
  		      erasName,
  		      erasAbbr,
  		      erasNarrow,
  		      eras = this.eras();
  		    for (i = 0, l = eras.length; i < l; ++i) {
  		      erasName = regexEscape(eras[i].name);
  		      erasAbbr = regexEscape(eras[i].abbr);
  		      erasNarrow = regexEscape(eras[i].narrow);
  		      namePieces.push(erasName);
  		      abbrPieces.push(erasAbbr);
  		      narrowPieces.push(erasNarrow);
  		      mixedPieces.push(erasName);
  		      mixedPieces.push(erasAbbr);
  		      mixedPieces.push(erasNarrow);
  		    }
  		    this._erasRegex = new RegExp('^(' + mixedPieces.join('|') + ')', 'i');
  		    this._erasNameRegex = new RegExp('^(' + namePieces.join('|') + ')', 'i');
  		    this._erasAbbrRegex = new RegExp('^(' + abbrPieces.join('|') + ')', 'i');
  		    this._erasNarrowRegex = new RegExp('^(' + narrowPieces.join('|') + ')', 'i');
  		  }

  		  // FORMATTING

  		  addFormatToken(0, ['gg', 2], 0, function () {
  		    return this.weekYear() % 100;
  		  });
  		  addFormatToken(0, ['GG', 2], 0, function () {
  		    return this.isoWeekYear() % 100;
  		  });
  		  function addWeekYearFormatToken(token, getter) {
  		    addFormatToken(0, [token, token.length], 0, getter);
  		  }
  		  addWeekYearFormatToken('gggg', 'weekYear');
  		  addWeekYearFormatToken('ggggg', 'weekYear');
  		  addWeekYearFormatToken('GGGG', 'isoWeekYear');
  		  addWeekYearFormatToken('GGGGG', 'isoWeekYear');

  		  // ALIASES

  		  // PARSING

  		  addRegexToken('G', matchSigned);
  		  addRegexToken('g', matchSigned);
  		  addRegexToken('GG', match1to2, match2);
  		  addRegexToken('gg', match1to2, match2);
  		  addRegexToken('GGGG', match1to4, match4);
  		  addRegexToken('gggg', match1to4, match4);
  		  addRegexToken('GGGGG', match1to6, match6);
  		  addRegexToken('ggggg', match1to6, match6);
  		  addWeekParseToken(['gggg', 'ggggg', 'GGGG', 'GGGGG'], function (input, week, config, token) {
  		    week[token.substr(0, 2)] = toInt(input);
  		  });
  		  addWeekParseToken(['gg', 'GG'], function (input, week, config, token) {
  		    week[token] = hooks.parseTwoDigitYear(input);
  		  });

  		  // MOMENTS

  		  function getSetWeekYear(input) {
  		    return getSetWeekYearHelper.call(this, input, this.week(), this.weekday() + this.localeData()._week.dow, this.localeData()._week.dow, this.localeData()._week.doy);
  		  }
  		  function getSetISOWeekYear(input) {
  		    return getSetWeekYearHelper.call(this, input, this.isoWeek(), this.isoWeekday(), 1, 4);
  		  }
  		  function getISOWeeksInYear() {
  		    return weeksInYear(this.year(), 1, 4);
  		  }
  		  function getISOWeeksInISOWeekYear() {
  		    return weeksInYear(this.isoWeekYear(), 1, 4);
  		  }
  		  function getWeeksInYear() {
  		    var weekInfo = this.localeData()._week;
  		    return weeksInYear(this.year(), weekInfo.dow, weekInfo.doy);
  		  }
  		  function getWeeksInWeekYear() {
  		    var weekInfo = this.localeData()._week;
  		    return weeksInYear(this.weekYear(), weekInfo.dow, weekInfo.doy);
  		  }
  		  function getSetWeekYearHelper(input, week, weekday, dow, doy) {
  		    var weeksTarget;
  		    if (input == null) {
  		      return weekOfYear(this, dow, doy).year;
  		    } else {
  		      weeksTarget = weeksInYear(input, dow, doy);
  		      if (week > weeksTarget) {
  		        week = weeksTarget;
  		      }
  		      return setWeekAll.call(this, input, week, weekday, dow, doy);
  		    }
  		  }
  		  function setWeekAll(weekYear, week, weekday, dow, doy) {
  		    var dayOfYearData = dayOfYearFromWeeks(weekYear, week, weekday, dow, doy),
  		      date = createUTCDate(dayOfYearData.year, 0, dayOfYearData.dayOfYear);
  		    this.year(date.getUTCFullYear());
  		    this.month(date.getUTCMonth());
  		    this.date(date.getUTCDate());
  		    return this;
  		  }

  		  // FORMATTING

  		  addFormatToken('Q', 0, 'Qo', 'quarter');

  		  // PARSING

  		  addRegexToken('Q', match1);
  		  addParseToken('Q', function (input, array) {
  		    array[MONTH] = (toInt(input) - 1) * 3;
  		  });

  		  // MOMENTS

  		  function getSetQuarter(input) {
  		    return input == null ? Math.ceil((this.month() + 1) / 3) : this.month((input - 1) * 3 + this.month() % 3);
  		  }

  		  // FORMATTING

  		  addFormatToken('D', ['DD', 2], 'Do', 'date');

  		  // PARSING

  		  addRegexToken('D', match1to2, match1to2NoLeadingZero);
  		  addRegexToken('DD', match1to2, match2);
  		  addRegexToken('Do', function (isStrict, locale) {
  		    // TODO: Remove "ordinalParse" fallback in next major release.
  		    return isStrict ? locale._dayOfMonthOrdinalParse || locale._ordinalParse : locale._dayOfMonthOrdinalParseLenient;
  		  });
  		  addParseToken(['D', 'DD'], DATE);
  		  addParseToken('Do', function (input, array) {
  		    array[DATE] = toInt(input.match(match1to2)[0]);
  		  });

  		  // MOMENTS

  		  var getSetDayOfMonth = makeGetSet('Date', true);

  		  // FORMATTING

  		  addFormatToken('DDD', ['DDDD', 3], 'DDDo', 'dayOfYear');

  		  // PARSING

  		  addRegexToken('DDD', match1to3);
  		  addRegexToken('DDDD', match3);
  		  addParseToken(['DDD', 'DDDD'], function (input, array, config) {
  		    config._dayOfYear = toInt(input);
  		  });

  		  // HELPERS

  		  // MOMENTS

  		  function getSetDayOfYear(input) {
  		    var dayOfYear = Math.round((this.clone().startOf('day') - this.clone().startOf('year')) / 864e5) + 1;
  		    return input == null ? dayOfYear : this.add(input - dayOfYear, 'd');
  		  }

  		  // FORMATTING

  		  addFormatToken('m', ['mm', 2], 0, 'minute');

  		  // PARSING

  		  addRegexToken('m', match1to2, match1to2HasZero);
  		  addRegexToken('mm', match1to2, match2);
  		  addParseToken(['m', 'mm'], MINUTE);

  		  // MOMENTS

  		  var getSetMinute = makeGetSet('Minutes', false);

  		  // FORMATTING

  		  addFormatToken('s', ['ss', 2], 0, 'second');

  		  // PARSING

  		  addRegexToken('s', match1to2, match1to2HasZero);
  		  addRegexToken('ss', match1to2, match2);
  		  addParseToken(['s', 'ss'], SECOND);

  		  // MOMENTS

  		  var getSetSecond = makeGetSet('Seconds', false);

  		  // FORMATTING

  		  addFormatToken('S', 0, 0, function () {
  		    return ~~(this.millisecond() / 100);
  		  });
  		  addFormatToken(0, ['SS', 2], 0, function () {
  		    return ~~(this.millisecond() / 10);
  		  });
  		  addFormatToken(0, ['SSS', 3], 0, 'millisecond');
  		  addFormatToken(0, ['SSSS', 4], 0, function () {
  		    return this.millisecond() * 10;
  		  });
  		  addFormatToken(0, ['SSSSS', 5], 0, function () {
  		    return this.millisecond() * 100;
  		  });
  		  addFormatToken(0, ['SSSSSS', 6], 0, function () {
  		    return this.millisecond() * 1000;
  		  });
  		  addFormatToken(0, ['SSSSSSS', 7], 0, function () {
  		    return this.millisecond() * 10000;
  		  });
  		  addFormatToken(0, ['SSSSSSSS', 8], 0, function () {
  		    return this.millisecond() * 100000;
  		  });
  		  addFormatToken(0, ['SSSSSSSSS', 9], 0, function () {
  		    return this.millisecond() * 1000000;
  		  });

  		  // PARSING

  		  addRegexToken('S', match1to3, match1);
  		  addRegexToken('SS', match1to3, match2);
  		  addRegexToken('SSS', match1to3, match3);
  		  var token, getSetMillisecond;
  		  for (token = 'SSSS'; token.length <= 9; token += 'S') {
  		    addRegexToken(token, matchUnsigned);
  		  }
  		  function parseMs(input, array) {
  		    array[MILLISECOND] = toInt(('0.' + input) * 1000);
  		  }
  		  for (token = 'S'; token.length <= 9; token += 'S') {
  		    addParseToken(token, parseMs);
  		  }
  		  getSetMillisecond = makeGetSet('Milliseconds', false);

  		  // FORMATTING

  		  addFormatToken('z', 0, 0, 'zoneAbbr');
  		  addFormatToken('zz', 0, 0, 'zoneName');

  		  // MOMENTS

  		  function getZoneAbbr() {
  		    return this._isUTC ? 'UTC' : '';
  		  }
  		  function getZoneName() {
  		    return this._isUTC ? 'Coordinated Universal Time' : '';
  		  }
  		  var proto = Moment.prototype;
  		  proto.add = add;
  		  proto.calendar = calendar$1;
  		  proto.clone = clone;
  		  proto.diff = diff;
  		  proto.endOf = endOf;
  		  proto.format = format;
  		  proto.from = from;
  		  proto.fromNow = fromNow;
  		  proto.to = to;
  		  proto.toNow = toNow;
  		  proto.get = stringGet;
  		  proto.invalidAt = invalidAt;
  		  proto.isAfter = isAfter;
  		  proto.isBefore = isBefore;
  		  proto.isBetween = isBetween;
  		  proto.isSame = isSame;
  		  proto.isSameOrAfter = isSameOrAfter;
  		  proto.isSameOrBefore = isSameOrBefore;
  		  proto.isValid = isValid$2;
  		  proto.lang = lang;
  		  proto.locale = locale;
  		  proto.localeData = localeData;
  		  proto.max = prototypeMax;
  		  proto.min = prototypeMin;
  		  proto.parsingFlags = parsingFlags;
  		  proto.set = stringSet;
  		  proto.startOf = startOf;
  		  proto.subtract = subtract;
  		  proto.toArray = toArray;
  		  proto.toObject = toObject;
  		  proto.toDate = toDate;
  		  proto.toISOString = toISOString;
  		  proto.inspect = inspect;
  		  if (typeof Symbol !== 'undefined' && Symbol.for != null) {
  		    proto[Symbol.for('nodejs.util.inspect.custom')] = function () {
  		      return 'Moment<' + this.format() + '>';
  		    };
  		  }
  		  proto.toJSON = toJSON;
  		  proto.toString = toString;
  		  proto.unix = unix;
  		  proto.valueOf = valueOf;
  		  proto.creationData = creationData;
  		  proto.eraName = getEraName;
  		  proto.eraNarrow = getEraNarrow;
  		  proto.eraAbbr = getEraAbbr;
  		  proto.eraYear = getEraYear;
  		  proto.year = getSetYear;
  		  proto.isLeapYear = getIsLeapYear;
  		  proto.weekYear = getSetWeekYear;
  		  proto.isoWeekYear = getSetISOWeekYear;
  		  proto.quarter = proto.quarters = getSetQuarter;
  		  proto.month = getSetMonth;
  		  proto.daysInMonth = getDaysInMonth;
  		  proto.week = proto.weeks = getSetWeek;
  		  proto.isoWeek = proto.isoWeeks = getSetISOWeek;
  		  proto.weeksInYear = getWeeksInYear;
  		  proto.weeksInWeekYear = getWeeksInWeekYear;
  		  proto.isoWeeksInYear = getISOWeeksInYear;
  		  proto.isoWeeksInISOWeekYear = getISOWeeksInISOWeekYear;
  		  proto.date = getSetDayOfMonth;
  		  proto.day = proto.days = getSetDayOfWeek;
  		  proto.weekday = getSetLocaleDayOfWeek;
  		  proto.isoWeekday = getSetISODayOfWeek;
  		  proto.dayOfYear = getSetDayOfYear;
  		  proto.hour = proto.hours = getSetHour;
  		  proto.minute = proto.minutes = getSetMinute;
  		  proto.second = proto.seconds = getSetSecond;
  		  proto.millisecond = proto.milliseconds = getSetMillisecond;
  		  proto.utcOffset = getSetOffset;
  		  proto.utc = setOffsetToUTC;
  		  proto.local = setOffsetToLocal;
  		  proto.parseZone = setOffsetToParsedOffset;
  		  proto.hasAlignedHourOffset = hasAlignedHourOffset;
  		  proto.isDST = isDaylightSavingTime;
  		  proto.isLocal = isLocal;
  		  proto.isUtcOffset = isUtcOffset;
  		  proto.isUtc = isUtc;
  		  proto.isUTC = isUtc;
  		  proto.zoneAbbr = getZoneAbbr;
  		  proto.zoneName = getZoneName;
  		  proto.dates = deprecate('dates accessor is deprecated. Use date instead.', getSetDayOfMonth);
  		  proto.months = deprecate('months accessor is deprecated. Use month instead', getSetMonth);
  		  proto.years = deprecate('years accessor is deprecated. Use year instead', getSetYear);
  		  proto.zone = deprecate('moment().zone is deprecated, use moment().utcOffset instead. http://momentjs.com/guides/#/warnings/zone/', getSetZone);
  		  proto.isDSTShifted = deprecate('isDSTShifted is deprecated. See http://momentjs.com/guides/#/warnings/dst-shifted/ for more information', isDaylightSavingTimeShifted);
  		  function createUnix(input) {
  		    return createLocal(input * 1000);
  		  }
  		  function createInZone() {
  		    return createLocal.apply(null, arguments).parseZone();
  		  }
  		  function preParsePostFormat(string) {
  		    return string;
  		  }
  		  var proto$1 = Locale.prototype;
  		  proto$1.calendar = calendar;
  		  proto$1.longDateFormat = longDateFormat;
  		  proto$1.invalidDate = invalidDate;
  		  proto$1.ordinal = ordinal;
  		  proto$1.preparse = preParsePostFormat;
  		  proto$1.postformat = preParsePostFormat;
  		  proto$1.relativeTime = relativeTime;
  		  proto$1.pastFuture = pastFuture;
  		  proto$1.set = set;
  		  proto$1.eras = localeEras;
  		  proto$1.erasParse = localeErasParse;
  		  proto$1.erasConvertYear = localeErasConvertYear;
  		  proto$1.erasAbbrRegex = erasAbbrRegex;
  		  proto$1.erasNameRegex = erasNameRegex;
  		  proto$1.erasNarrowRegex = erasNarrowRegex;
  		  proto$1.months = localeMonths;
  		  proto$1.monthsShort = localeMonthsShort;
  		  proto$1.monthsParse = localeMonthsParse;
  		  proto$1.monthsRegex = monthsRegex;
  		  proto$1.monthsShortRegex = monthsShortRegex;
  		  proto$1.week = localeWeek;
  		  proto$1.firstDayOfYear = localeFirstDayOfYear;
  		  proto$1.firstDayOfWeek = localeFirstDayOfWeek;
  		  proto$1.weekdays = localeWeekdays;
  		  proto$1.weekdaysMin = localeWeekdaysMin;
  		  proto$1.weekdaysShort = localeWeekdaysShort;
  		  proto$1.weekdaysParse = localeWeekdaysParse;
  		  proto$1.weekdaysRegex = weekdaysRegex;
  		  proto$1.weekdaysShortRegex = weekdaysShortRegex;
  		  proto$1.weekdaysMinRegex = weekdaysMinRegex;
  		  proto$1.isPM = localeIsPM;
  		  proto$1.meridiem = localeMeridiem;
  		  function get$1(format, index, field, setter) {
  		    var locale = getLocale(),
  		      utc = createUTC().set(setter, index);
  		    return locale[field](utc, format);
  		  }
  		  function listMonthsImpl(format, index, field) {
  		    if (isNumber(format)) {
  		      index = format;
  		      format = undefined;
  		    }
  		    format = format || '';
  		    if (index != null) {
  		      return get$1(format, index, field, 'month');
  		    }
  		    var i,
  		      out = [];
  		    for (i = 0; i < 12; i++) {
  		      out[i] = get$1(format, i, field, 'month');
  		    }
  		    return out;
  		  }

  		  // ()
  		  // (5)
  		  // (fmt, 5)
  		  // (fmt)
  		  // (true)
  		  // (true, 5)
  		  // (true, fmt, 5)
  		  // (true, fmt)
  		  function listWeekdaysImpl(localeSorted, format, index, field) {
  		    if (typeof localeSorted === 'boolean') {
  		      if (isNumber(format)) {
  		        index = format;
  		        format = undefined;
  		      }
  		      format = format || '';
  		    } else {
  		      format = localeSorted;
  		      index = format;
  		      localeSorted = false;
  		      if (isNumber(format)) {
  		        index = format;
  		        format = undefined;
  		      }
  		      format = format || '';
  		    }
  		    var locale = getLocale(),
  		      shift = localeSorted ? locale._week.dow : 0,
  		      i,
  		      out = [];
  		    if (index != null) {
  		      return get$1(format, (index + shift) % 7, field, 'day');
  		    }
  		    for (i = 0; i < 7; i++) {
  		      out[i] = get$1(format, (i + shift) % 7, field, 'day');
  		    }
  		    return out;
  		  }
  		  function listMonths(format, index) {
  		    return listMonthsImpl(format, index, 'months');
  		  }
  		  function listMonthsShort(format, index) {
  		    return listMonthsImpl(format, index, 'monthsShort');
  		  }
  		  function listWeekdays(localeSorted, format, index) {
  		    return listWeekdaysImpl(localeSorted, format, index, 'weekdays');
  		  }
  		  function listWeekdaysShort(localeSorted, format, index) {
  		    return listWeekdaysImpl(localeSorted, format, index, 'weekdaysShort');
  		  }
  		  function listWeekdaysMin(localeSorted, format, index) {
  		    return listWeekdaysImpl(localeSorted, format, index, 'weekdaysMin');
  		  }
  		  getSetGlobalLocale('en', {
  		    eras: [{
  		      since: '0001-01-01',
  		      until: +Infinity,
  		      offset: 1,
  		      name: 'Anno Domini',
  		      narrow: 'AD',
  		      abbr: 'AD'
  		    }, {
  		      since: '0000-12-31',
  		      until: -Infinity,
  		      offset: 1,
  		      name: 'Before Christ',
  		      narrow: 'BC',
  		      abbr: 'BC'
  		    }],
  		    dayOfMonthOrdinalParse: /\d{1,2}(th|st|nd|rd)/,
  		    ordinal: function (number) {
  		      var b = number % 10,
  		        output = toInt(number % 100 / 10) === 1 ? 'th' : b === 1 ? 'st' : b === 2 ? 'nd' : b === 3 ? 'rd' : 'th';
  		      return number + output;
  		    }
  		  });

  		  // Side effect imports

  		  hooks.lang = deprecate('moment.lang is deprecated. Use moment.locale instead.', getSetGlobalLocale);
  		  hooks.langData = deprecate('moment.langData is deprecated. Use moment.localeData instead.', getLocale);
  		  var mathAbs = Math.abs;
  		  function abs() {
  		    var data = this._data;
  		    this._milliseconds = mathAbs(this._milliseconds);
  		    this._days = mathAbs(this._days);
  		    this._months = mathAbs(this._months);
  		    data.milliseconds = mathAbs(data.milliseconds);
  		    data.seconds = mathAbs(data.seconds);
  		    data.minutes = mathAbs(data.minutes);
  		    data.hours = mathAbs(data.hours);
  		    data.months = mathAbs(data.months);
  		    data.years = mathAbs(data.years);
  		    return this;
  		  }
  		  function addSubtract$1(duration, input, value, direction) {
  		    var other = createDuration(input, value);
  		    duration._milliseconds += direction * other._milliseconds;
  		    duration._days += direction * other._days;
  		    duration._months += direction * other._months;
  		    return duration._bubble();
  		  }

  		  // supports only 2.0-style add(1, 's') or add(duration)
  		  function add$1(input, value) {
  		    return addSubtract$1(this, input, value, 1);
  		  }

  		  // supports only 2.0-style subtract(1, 's') or subtract(duration)
  		  function subtract$1(input, value) {
  		    return addSubtract$1(this, input, value, -1);
  		  }
  		  function absCeil(number) {
  		    if (number < 0) {
  		      return Math.floor(number);
  		    } else {
  		      return Math.ceil(number);
  		    }
  		  }
  		  function bubble() {
  		    var milliseconds = this._milliseconds,
  		      days = this._days,
  		      months = this._months,
  		      data = this._data,
  		      seconds,
  		      minutes,
  		      hours,
  		      years,
  		      monthsFromDays;

  		    // if we have a mix of positive and negative values, bubble down first
  		    // check: https://github.com/moment/moment/issues/2166
  		    if (!(milliseconds >= 0 && days >= 0 && months >= 0 || milliseconds <= 0 && days <= 0 && months <= 0)) {
  		      milliseconds += absCeil(monthsToDays(months) + days) * 864e5;
  		      days = 0;
  		      months = 0;
  		    }

  		    // The following code bubbles up values, see the tests for
  		    // examples of what that means.
  		    data.milliseconds = milliseconds % 1000;
  		    seconds = absFloor(milliseconds / 1000);
  		    data.seconds = seconds % 60;
  		    minutes = absFloor(seconds / 60);
  		    data.minutes = minutes % 60;
  		    hours = absFloor(minutes / 60);
  		    data.hours = hours % 24;
  		    days += absFloor(hours / 24);

  		    // convert days to months
  		    monthsFromDays = absFloor(daysToMonths(days));
  		    months += monthsFromDays;
  		    days -= absCeil(monthsToDays(monthsFromDays));

  		    // 12 months -> 1 year
  		    years = absFloor(months / 12);
  		    months %= 12;
  		    data.days = days;
  		    data.months = months;
  		    data.years = years;
  		    return this;
  		  }
  		  function daysToMonths(days) {
  		    // 400 years have 146097 days (taking into account leap year rules)
  		    // 400 years have 12 months === 4800
  		    return days * 4800 / 146097;
  		  }
  		  function monthsToDays(months) {
  		    // the reverse of daysToMonths
  		    return months * 146097 / 4800;
  		  }
  		  function as(units) {
  		    if (!this.isValid()) {
  		      return NaN;
  		    }
  		    var days,
  		      months,
  		      milliseconds = this._milliseconds;
  		    units = normalizeUnits(units);
  		    if (units === 'month' || units === 'quarter' || units === 'year') {
  		      days = this._days + milliseconds / 864e5;
  		      months = this._months + daysToMonths(days);
  		      switch (units) {
  		        case 'month':
  		          return months;
  		        case 'quarter':
  		          return months / 3;
  		        case 'year':
  		          return months / 12;
  		      }
  		    } else {
  		      // handle milliseconds separately because of floating point math errors (issue #1867)
  		      days = this._days + Math.round(monthsToDays(this._months));
  		      switch (units) {
  		        case 'week':
  		          return days / 7 + milliseconds / 6048e5;
  		        case 'day':
  		          return days + milliseconds / 864e5;
  		        case 'hour':
  		          return days * 24 + milliseconds / 36e5;
  		        case 'minute':
  		          return days * 1440 + milliseconds / 6e4;
  		        case 'second':
  		          return days * 86400 + milliseconds / 1000;
  		        // Math.floor prevents floating point math errors here
  		        case 'millisecond':
  		          return Math.floor(days * 864e5) + milliseconds;
  		        default:
  		          throw new Error('Unknown unit ' + units);
  		      }
  		    }
  		  }
  		  function makeAs(alias) {
  		    return function () {
  		      return this.as(alias);
  		    };
  		  }
  		  var asMilliseconds = makeAs('ms'),
  		    asSeconds = makeAs('s'),
  		    asMinutes = makeAs('m'),
  		    asHours = makeAs('h'),
  		    asDays = makeAs('d'),
  		    asWeeks = makeAs('w'),
  		    asMonths = makeAs('M'),
  		    asQuarters = makeAs('Q'),
  		    asYears = makeAs('y'),
  		    valueOf$1 = asMilliseconds;
  		  function clone$1() {
  		    return createDuration(this);
  		  }
  		  function get$2(units) {
  		    units = normalizeUnits(units);
  		    return this.isValid() ? this[units + 's']() : NaN;
  		  }
  		  function makeGetter(name) {
  		    return function () {
  		      return this.isValid() ? this._data[name] : NaN;
  		    };
  		  }
  		  var milliseconds = makeGetter('milliseconds'),
  		    seconds = makeGetter('seconds'),
  		    minutes = makeGetter('minutes'),
  		    hours = makeGetter('hours'),
  		    days = makeGetter('days'),
  		    months = makeGetter('months'),
  		    years = makeGetter('years');
  		  function weeks() {
  		    return absFloor(this.days() / 7);
  		  }
  		  var round = Math.round,
  		    thresholds = {
  		      ss: 44,
  		      // a few seconds to seconds
  		      s: 45,
  		      // seconds to minute
  		      m: 45,
  		      // minutes to hour
  		      h: 22,
  		      // hours to day
  		      d: 26,
  		      // days to month/week
  		      w: null,
  		      // weeks to month
  		      M: 11 // months to year
  		    };

  		  // helper function for moment.fn.from, moment.fn.fromNow, and moment.duration.fn.humanize
  		  function substituteTimeAgo(string, number, withoutSuffix, isFuture, locale) {
  		    return locale.relativeTime(number || 1, !!withoutSuffix, string, isFuture);
  		  }
  		  function relativeTime$1(posNegDuration, withoutSuffix, thresholds, locale) {
  		    var duration = createDuration(posNegDuration).abs(),
  		      seconds = round(duration.as('s')),
  		      minutes = round(duration.as('m')),
  		      hours = round(duration.as('h')),
  		      days = round(duration.as('d')),
  		      months = round(duration.as('M')),
  		      weeks = round(duration.as('w')),
  		      years = round(duration.as('y')),
  		      a = seconds <= thresholds.ss && ['s', seconds] || seconds < thresholds.s && ['ss', seconds] || minutes <= 1 && ['m'] || minutes < thresholds.m && ['mm', minutes] || hours <= 1 && ['h'] || hours < thresholds.h && ['hh', hours] || days <= 1 && ['d'] || days < thresholds.d && ['dd', days];
  		    if (thresholds.w != null) {
  		      a = a || weeks <= 1 && ['w'] || weeks < thresholds.w && ['ww', weeks];
  		    }
  		    a = a || months <= 1 && ['M'] || months < thresholds.M && ['MM', months] || years <= 1 && ['y'] || ['yy', years];
  		    a[2] = withoutSuffix;
  		    a[3] = +posNegDuration > 0;
  		    a[4] = locale;
  		    return substituteTimeAgo.apply(null, a);
  		  }

  		  // This function allows you to set the rounding function for relative time strings
  		  function getSetRelativeTimeRounding(roundingFunction) {
  		    if (roundingFunction === undefined) {
  		      return round;
  		    }
  		    if (typeof roundingFunction === 'function') {
  		      round = roundingFunction;
  		      return true;
  		    }
  		    return false;
  		  }

  		  // This function allows you to set a threshold for relative time strings
  		  function getSetRelativeTimeThreshold(threshold, limit) {
  		    if (thresholds[threshold] === undefined) {
  		      return false;
  		    }
  		    if (limit === undefined) {
  		      return thresholds[threshold];
  		    }
  		    thresholds[threshold] = limit;
  		    if (threshold === 's') {
  		      thresholds.ss = limit - 1;
  		    }
  		    return true;
  		  }
  		  function humanize(argWithSuffix, argThresholds) {
  		    if (!this.isValid()) {
  		      return this.localeData().invalidDate();
  		    }
  		    var withSuffix = false,
  		      th = thresholds,
  		      locale,
  		      output;
  		    if (typeof argWithSuffix === 'object') {
  		      argThresholds = argWithSuffix;
  		      argWithSuffix = false;
  		    }
  		    if (typeof argWithSuffix === 'boolean') {
  		      withSuffix = argWithSuffix;
  		    }
  		    if (typeof argThresholds === 'object') {
  		      th = Object.assign({}, thresholds, argThresholds);
  		      if (argThresholds.s != null && argThresholds.ss == null) {
  		        th.ss = argThresholds.s - 1;
  		      }
  		    }
  		    locale = this.localeData();
  		    output = relativeTime$1(this, !withSuffix, th, locale);
  		    if (withSuffix) {
  		      output = locale.pastFuture(+this, output);
  		    }
  		    return locale.postformat(output);
  		  }
  		  var abs$1 = Math.abs;
  		  function sign(x) {
  		    return (x > 0) - (x < 0) || +x;
  		  }
  		  function toISOString$1() {
  		    // for ISO strings we do not use the normal bubbling rules:
  		    //  * milliseconds bubble up until they become hours
  		    //  * days do not bubble at all
  		    //  * months bubble up until they become years
  		    // This is because there is no context-free conversion between hours and days
  		    // (think of clock changes)
  		    // and also not between days and months (28-31 days per month)
  		    if (!this.isValid()) {
  		      return this.localeData().invalidDate();
  		    }
  		    var seconds = abs$1(this._milliseconds) / 1000,
  		      days = abs$1(this._days),
  		      months = abs$1(this._months),
  		      minutes,
  		      hours,
  		      years,
  		      s,
  		      total = this.asSeconds(),
  		      totalSign,
  		      ymSign,
  		      daysSign,
  		      hmsSign;
  		    if (!total) {
  		      // this is the same as C#'s (Noda) and python (isodate)...
  		      // but not other JS (goog.date)
  		      return 'P0D';
  		    }

  		    // 3600 seconds -> 60 minutes -> 1 hour
  		    minutes = absFloor(seconds / 60);
  		    hours = absFloor(minutes / 60);
  		    seconds %= 60;
  		    minutes %= 60;

  		    // 12 months -> 1 year
  		    years = absFloor(months / 12);
  		    months %= 12;

  		    // inspired by https://github.com/dordille/moment-isoduration/blob/master/moment.isoduration.js
  		    s = seconds ? seconds.toFixed(3).replace(/\.?0+$/, '') : '';
  		    totalSign = total < 0 ? '-' : '';
  		    ymSign = sign(this._months) !== sign(total) ? '-' : '';
  		    daysSign = sign(this._days) !== sign(total) ? '-' : '';
  		    hmsSign = sign(this._milliseconds) !== sign(total) ? '-' : '';
  		    return totalSign + 'P' + (years ? ymSign + years + 'Y' : '') + (months ? ymSign + months + 'M' : '') + (days ? daysSign + days + 'D' : '') + (hours || minutes || seconds ? 'T' : '') + (hours ? hmsSign + hours + 'H' : '') + (minutes ? hmsSign + minutes + 'M' : '') + (seconds ? hmsSign + s + 'S' : '');
  		  }
  		  var proto$2 = Duration.prototype;
  		  proto$2.isValid = isValid$1;
  		  proto$2.abs = abs;
  		  proto$2.add = add$1;
  		  proto$2.subtract = subtract$1;
  		  proto$2.as = as;
  		  proto$2.asMilliseconds = asMilliseconds;
  		  proto$2.asSeconds = asSeconds;
  		  proto$2.asMinutes = asMinutes;
  		  proto$2.asHours = asHours;
  		  proto$2.asDays = asDays;
  		  proto$2.asWeeks = asWeeks;
  		  proto$2.asMonths = asMonths;
  		  proto$2.asQuarters = asQuarters;
  		  proto$2.asYears = asYears;
  		  proto$2.valueOf = valueOf$1;
  		  proto$2._bubble = bubble;
  		  proto$2.clone = clone$1;
  		  proto$2.get = get$2;
  		  proto$2.milliseconds = milliseconds;
  		  proto$2.seconds = seconds;
  		  proto$2.minutes = minutes;
  		  proto$2.hours = hours;
  		  proto$2.days = days;
  		  proto$2.weeks = weeks;
  		  proto$2.months = months;
  		  proto$2.years = years;
  		  proto$2.humanize = humanize;
  		  proto$2.toISOString = toISOString$1;
  		  proto$2.toString = toISOString$1;
  		  proto$2.toJSON = toISOString$1;
  		  proto$2.locale = locale;
  		  proto$2.localeData = localeData;
  		  proto$2.toIsoString = deprecate('toIsoString() is deprecated. Please use toISOString() instead (notice the capitals)', toISOString$1);
  		  proto$2.lang = lang;

  		  // FORMATTING

  		  addFormatToken('X', 0, 0, 'unix');
  		  addFormatToken('x', 0, 0, 'valueOf');

  		  // PARSING

  		  addRegexToken('x', matchSigned);
  		  addRegexToken('X', matchTimestamp);
  		  addParseToken('X', function (input, array, config) {
  		    config._d = new Date(parseFloat(input) * 1000);
  		  });
  		  addParseToken('x', function (input, array, config) {
  		    config._d = new Date(toInt(input));
  		  });

  		  //! moment.js

  		  hooks.version = '2.30.1';
  		  setHookCallback(createLocal);
  		  hooks.fn = proto;
  		  hooks.min = min;
  		  hooks.max = max;
  		  hooks.now = now;
  		  hooks.utc = createUTC;
  		  hooks.unix = createUnix;
  		  hooks.months = listMonths;
  		  hooks.isDate = isDate;
  		  hooks.locale = getSetGlobalLocale;
  		  hooks.invalid = createInvalid;
  		  hooks.duration = createDuration;
  		  hooks.isMoment = isMoment;
  		  hooks.weekdays = listWeekdays;
  		  hooks.parseZone = createInZone;
  		  hooks.localeData = getLocale;
  		  hooks.isDuration = isDuration;
  		  hooks.monthsShort = listMonthsShort;
  		  hooks.weekdaysMin = listWeekdaysMin;
  		  hooks.defineLocale = defineLocale;
  		  hooks.updateLocale = updateLocale;
  		  hooks.locales = listLocales;
  		  hooks.weekdaysShort = listWeekdaysShort;
  		  hooks.normalizeUnits = normalizeUnits;
  		  hooks.relativeTimeRounding = getSetRelativeTimeRounding;
  		  hooks.relativeTimeThreshold = getSetRelativeTimeThreshold;
  		  hooks.calendarFormat = getCalendarFormat;
  		  hooks.prototype = proto;

  		  // currently HTML5 input type only supports 24-hour formats
  		  hooks.HTML5_FMT = {
  		    DATETIME_LOCAL: 'YYYY-MM-DDTHH:mm',
  		    // <input type="datetime-local" />
  		    DATETIME_LOCAL_SECONDS: 'YYYY-MM-DDTHH:mm:ss',
  		    // <input type="datetime-local" step="1" />
  		    DATETIME_LOCAL_MS: 'YYYY-MM-DDTHH:mm:ss.SSS',
  		    // <input type="datetime-local" step="0.001" />
  		    DATE: 'YYYY-MM-DD',
  		    // <input type="date" />
  		    TIME: 'HH:mm',
  		    // <input type="time" />
  		    TIME_SECONDS: 'HH:mm:ss',
  		    // <input type="time" step="1" />
  		    TIME_MS: 'HH:mm:ss.SSS',
  		    // <input type="time" step="0.001" />
  		    WEEK: 'GGGG-[W]WW',
  		    // <input type="week" />
  		    MONTH: 'YYYY-MM' // <input type="month" />
  		  };
  		  return hooks;
  		}); 
  	} (moment$3));
  	return moment$3.exports;
  }

  var momentExports = requireMoment();
  var moment$1 = /*@__PURE__*/getDefaultExportFromCjs(momentExports);

  // Development-related helper functions

  /**
   * NotePlan API properties which should not be traversed when stringifying an object
   */

  const PARAM_BLACKLIST = ['note', 'referencedBlocks', 'availableThemes', 'currentTheme', 'linkedNoteTitles', 'linkedItems']; // fields not to be traversed (e.g. circular references)

  const dt = () => {
    const d = new Date();
    const pad = value => {
      return value < 10 ? `0${value}` : value.toString();
    };
    return `${d.getFullYear()}-${pad(d.getMonth() + 1)}-${pad(d.getDate())} ${d.toLocaleTimeString('en-GB')}`;
  };

  /**
   * JSON.stringify() with support for Prototype properties
   * @author @dwertheimer
   *
   * @param {object} obj
   * @param {string | number} space - A String or Number of spaces that's used to insert white space (including indentation, line break characters, etc.) into the output JSON string for readability purposes.
   * @returns {string} stringified object
   * @example console.log(JSP(obj, '\t')) // prints the full object with newlines and tabs for indentation
   */
  function JSP(obj, space = 2) {
    if (typeof obj !== 'object' || obj instanceof Date) {
      return String(obj);
    } else {
      if (Array.isArray(obj)) {
        const arrInfo = [];
        let isValues = false;
        obj.forEach((item, i) => {
          if (typeof item === 'object') {
            arrInfo.push(`[${i}] = ${JSP(item, space)}`);
          } else {
            isValues = true;
            arrInfo.push(`${item}`);
          }
        });
        return `${isValues ? '[' : ''}${arrInfo.join(isValues ? ', ' : ',\n')}${isValues ? ']' : ''}`;
      }
      const propNames = getFilteredProps(obj);
      const fullObj = propNames.reduce((acc, propName) => {
        if (!/^__/.test(propName)) {
          if (Array.isArray(obj[propName])) {
            try {
              if (PARAM_BLACKLIST.indexOf(propName) === -1) {
                acc[propName] = obj[propName].map(x => {
                  if (typeof x === 'object' && !(x instanceof Date)) {
                    return JSP(x, '');
                  } else {
                    return x;
                  }
                });
              } else {
                acc[propName] = obj[propName]; //do not traverse any further
              }
            } catch (error) {
              logDebug('helpers/dev', `Caught error in JSP for propname=${propName} : ${error} typeof obj[propName]=${typeof obj[propName]} isArray=${String(Array.isArray(obj[propName]))} len=${obj[propName]?.length} \n VALUE: ${JSON.stringify(obj[propName])}`);
            }
          } else {
            acc[propName] = obj[propName];
          }
        }
        return acc;
      }, {});
      // return cleanStringifiedResults(JSON.stringify(fullObj, null, space ?? null))
      return typeof fullObj === 'object' && !(fullObj instanceof Date) ? JSON.stringify(fullObj, null, space ?? null) : 'date';
    }
  }

  /**
   * Console.logs all property names/values of an object to console with text preamble
   * @author @dwertheimer
   *
   * @param {object} obj - array or object
   * @param {string} preamble - (optional) text to prepend to the output
   * @param {string | number} space - A String or Number of spaces that's used to insert white space (including indentation, line break characters, etc.) into the output JSON string for readability purposes.
   * @example clo(obj, 'myObj:')
   */
  function clo(obj, preamble = '', space = 2) {
    if (!obj) {
      logDebug(preamble, `null`);
      return;
    }
    if (typeof obj !== 'object') {
      logDebug(preamble, `${obj}`);
    } else {
      logDebug(preamble, JSP(obj, space));
    }
  }

  /**
   * Create a list of the properties of an object, including inherited properties (which are not typically visible in JSON.stringify)
   * Often includes a bunch of properties that are not useful for the user, e.g. constructor, __proto__
   * See getFilteredProps for a cleaner version
   * @author @dwertheimer (via StackOverflow)
   *
   * @param {object} inObj
   * @returns {Array<string>}
   * @reference https://stackoverflow.com/questions/59228638/console-log-an-object-does-not-log-the-method-added-via-prototype-in-node-js-c
   */
  function getAllPropertyNames(inObj) {
    let obj = inObj;
    const props = [];
    do {
      Object.getOwnPropertyNames(obj).forEach(function (prop) {
        if (props.indexOf(prop) === -1) {
          props.push(prop);
        }
      });
    } while (obj = Object.getPrototypeOf(obj));
    return props;
  }

  /**
   * Get the properties of interest (i.e. excluding all the ones added automatically)
   * @author @dwertheimer
   * @param {object} object
   * @returns {Array<string>} - an array of the interesting properties of the object
   */
  const getFilteredProps = object => {
    const ignore = ['toString', 'toLocaleString', 'valueOf', 'hasOwnProperty', 'propertyIsEnumerable', 'isPrototypeOf'];
    if (typeof object !== 'object' || Array.isArray(object)) {
      // console.log(`getFilteredProps improper type: ${typeof object}`)
      return [];
    }
    return getAllPropertyNames(object).filter(prop => !/(^__)|(constructor)/.test(prop) && !ignore.includes(prop));
  };

  /**
   * Converts any to message string
   * @author @codedungeon
   * @param {any} message
   * @returns {string}
   */
  const _message = message => {
    let logMessage = '';
    switch (typeof message) {
      case 'string':
        logMessage = message;
        break;
      case 'object':
        if (Array.isArray(message)) {
          logMessage = message.toString();
        } else {
          logMessage = message instanceof Date ? message.toString() : JSON.stringify(message);
        }
        break;
      default:
        logMessage = message.toString();
        break;
    }
    return logMessage;
  };
  const LOG_LEVELS = ['DEBUG', 'INFO', 'WARN', 'ERROR', 'none'];
  const LOG_LEVEL_STRINGS = ['| DEBUG |', '| INFO  |', '🥺 WARN 🥺', '❗️ ERROR ❗️', 'none'];

  /**
   * Test _logLevel against logType to decide whether to output
   * @param {string} logType
   * @returns {boolean}
   */
  const shouldOutputForLogLevel = logType => {
    let userLogLevel = 1;
    const thisMessageLevel = LOG_LEVELS.indexOf(logType.toUpperCase());
    const pluginSettings = typeof DataStore !== 'undefined' ? DataStore.settings : null;
    // Note: Performing a null change against a value that is `undefined` will be true
    // Sure wish NotePlan would not return `undefined` but instead null, then the previous implementataion would not have failed

    // se _logLevel to decide whether to output
    if (pluginSettings && pluginSettings.hasOwnProperty('_logLevel')) {
      userLogLevel = pluginSettings['_logLevel'];
    }
    const userLogLevelIndex = LOG_LEVELS.indexOf(userLogLevel);
    return thisMessageLevel >= userLogLevelIndex;
  };

  /**
   * Test if _logFunctionRE is set and matches the current log details.
   * Note: only works if DataStore is available.
   * @param {any} pluginInfo
   * @returns
   */
  const shouldOutputForFunctionName = pluginInfo => {
    const pluginSettings = typeof DataStore !== 'undefined' ? DataStore.settings : null;
    if (pluginSettings && pluginSettings.hasOwnProperty('_logFunctionRE')) {
      const functionRE = new RegExp(pluginSettings['_logFunctionRE'], 'i');
      const infoStr = pluginInfo === 'object' ? pluginInfo['plugin.id'] : String(pluginInfo);
      return functionRE.test(infoStr);
    }
    return false;
  };
  function getLogDateAndTypeString(type) {
    const thisMessageLevel = LOG_LEVELS.indexOf(type.toUpperCase());
    const thisIndicator = LOG_LEVEL_STRINGS[thisMessageLevel];
    return `${dt().padEnd(19)} ${thisIndicator}`;
  }

  /**
   * Formats log output to include timestamp pluginId, pluginVersion
   * @author @codedungeon extended by @jgclark
   * @param {any} pluginInfo
   * @param {any} message
   * @param {string} type
   * @returns {string}
   */
  function log(pluginInfo, message = '', type = 'INFO') {
    let msg = '';
    if (shouldOutputForLogLevel(type) || shouldOutputForFunctionName(pluginInfo)) {
      let pluginId = '';
      let pluginVersion = '';
      const isPluginJson = typeof pluginInfo === 'object' && pluginInfo.hasOwnProperty('plugin.id');
      const ldts = getLogDateAndTypeString(type);
      if (isPluginJson) {
        pluginId = pluginInfo.hasOwnProperty('plugin.id') ? pluginInfo['plugin.id'] : 'INVALID_PLUGIN_ID';
        pluginVersion = pluginInfo.hasOwnProperty('plugin.version') ? pluginInfo['plugin.version'] : 'INVALID_PLUGIN_VERSION';
        msg = `${ldts} ${pluginId} v${pluginVersion} :: ${_message(message)}`;
      } else {
        if (message.length > 0) {
          // msg = `${dt().padEnd(19)} | ${thisIndicator.padEnd(7)} | ${pluginInfo} :: ${_message(message)}`
          msg = `${ldts} ${pluginInfo} :: ${_message(message)}`;
        } else {
          // msg = `${dt().padEnd(19)} | ${thisIndicator.padEnd(7)} | ${_message(pluginInfo)}`
          msg = `${ldts} ${_message(pluginInfo)}`;
        }
      }
      console.log(msg);
    }
    return msg;
  }

  /**
   * Formats log output as ERROR to include timestamp pluginId, pluginVersion
   * @author @codedungeon
   * @param {any} pluginInfo
   * @param {any} message
   * @returns {string}
   */
  function logError(pluginInfo, error) {
    if (typeof error === 'object' && error != null) {
      const msg = `${error.filename ?? '<unknown file>'} ${error.lineNumber ?? '<unkonwn line>'}: ${error.message}`;
      return log(pluginInfo, msg, 'ERROR');
    }
    return log(pluginInfo, error, 'ERROR');
  }

  /**
   * Formats log output as WARN to include timestamp pluginId, pluginVersion
   * @author @codedungeon
   * @param {any} pluginInfo
   * @param {any} message
   * @returns {string}
   */
  function logWarn(pluginInfo, message = '') {
    return log(pluginInfo, message, 'WARN');
  }

  /**
   * Formats log output as INFO to include timestamp pluginId, pluginVersion
   * @author @codedungeon
   * @param {any} pluginInfo
   * @param {any} message
   * @returns {string}
   */
  function logInfo(pluginInfo, message = '') {
    return log(pluginInfo, message, 'INFO');
  }

  /**
   * Formats log output as DEBUG to include timestamp pluginId, pluginVersion
   * @author @dwertheimer
   * @param {any} pluginInfo
   * @param {any} message
   * @returns {string}
   */
  function logDebug(pluginInfo, message = '') {
    return log(pluginInfo, message, 'DEBUG');
  }

  /**
   * Time a function
   * @param {*} startTime - the date object from when timer started (using Date.now())
   * @returns {string} - the formatted elapsed time
   * @author @dwertheimer
   * @example
   * const startTime = Date.now()
   * ...some long-running stuff here...
   * const elapsedTime = timer(startTime)
   */
  function timer(startTime) {
    const timeStart = startTime ?? new Date();
    const timeEnd = new Date();
    // $FlowIgnore[unsafe-arithmetic]
    const difference = timeEnd - timeStart;
    // const d = new Date(difference)
    // const diffText = `${d.getMinutes()}m${d.getSeconds()}.${d.getMilliseconds()}s`
    const diffText = `${difference.toLocaleString()}ms`;
    return diffText;
  }

  /**
   * Add or override parameters from args to the supplied config object.
   * This is the **simple version** that treats all the passed arguments as strings, leaving some of the typing to the developer.
   * Tested with strings, ints, floats, boolean and simple array of strings.
   * Note: Different parameters are separated by ';' (not the more usual ',' to allow for comma-separated arrays)
   * Note: use the advanced version to pass more advanced quoted arrays, and items containing commas or semicolons.
   * Note: This can't tell the difference between single-element arrays and strings, so that processing needs to be done by the calling function.
   * @author @jgclark and @dwertheimer
   * @param {any} config object
   * @param {string} argsAsString e.g. 'field1=Bob Skinner;field2=false;field3=simple,little,array'
   * @returns {any} configOut
   */
  function overrideSettingsWithStringArgs(config, argsAsString) {
    try {
      // Parse argsAsJSON (if any) into argObj using JSON
      if (argsAsString) {
        const argObj = {};
        argsAsString.split(';').forEach(arg => arg.split('=').length === 2 ? argObj[arg.split('=')[0]] = arg.split('=')[1] : null);
        // use the built-in way to add (or override) from argObj into config
        const configOut = Object.assign(config);

        // Attempt to change arg values that are numerics or booleans to the right types, otherwise they will stay as strings
        for (const key in argObj) {
          let value = argObj[key];
          logDebug(`dev.js`, `overrideSettingsWithStringArgs key:${key} value:${argObj[key]} typeof:${typeof argObj[key]} !isNaN(${value}):${String(!isNaN(argObj[key]))}`);
          if (!isNaN(value) && value !== '') {
            // Change to number type
            value = Number(value);
          } else if (value === 'false') {
            // Change to boolean type
            value = false;
          } else if (value === 'true') {
            // Change to boolean type
            value = true;
          } else if (value.includes(',')) {
            // Split to make an array
            value = value.split(',');
          }
          configOut[key] = value;
          if (configOut[key] !== argObj[key]) {
            logDebug('overrideSettingsWithStringArgs', `- updated setting '${key}' -> value '${String(value)}'`);
          }
        }
        return configOut;
      } else {
        return config;
      }
    } catch (error) {
      logError('overrideSettingsWithStringArgs', JSP(error));
      console.log(JSP(error));
    }
  }

  var dist$1 = {exports: {}};

  var dist = dist$1.exports;

  var hasRequiredDist;

  function requireDist () {
  	if (hasRequiredDist) return dist$1.exports;
  	hasRequiredDist = 1;
  	(function (module, exports) {
  		(function (global, factory) {
  		  module.exports = factory() ;
  		})(dist, function () {

  		  function createCommonjsModule(fn, module) {
  		    return module = {
  		      exports: {}
  		    }, fn(module, module.exports), module.exports;
  		  }
  		  var _global = createCommonjsModule(function (module) {
  		    // https://github.com/zloirock/core-js/issues/86#issuecomment-115759028
  		    var global = module.exports = typeof window != 'undefined' && window.Math == Math ? window : typeof self != 'undefined' && self.Math == Math ? self
  		    // eslint-disable-next-line no-new-func
  		    : Function('return this')();
  		    if (typeof __g == 'number') {
  		      __g = global;
  		    } // eslint-disable-line no-undef
  		  });
  		  var _core = createCommonjsModule(function (module) {
  		    var core = module.exports = {
  		      version: '2.6.5'
  		    };
  		    if (typeof __e == 'number') {
  		      __e = core;
  		    } // eslint-disable-line no-undef
  		  });
  		  _core.version;
  		  var _isObject = function (it) {
  		    return typeof it === 'object' ? it !== null : typeof it === 'function';
  		  };
  		  var _anObject = function (it) {
  		    if (!_isObject(it)) {
  		      throw TypeError(it + ' is not an object!');
  		    }
  		    return it;
  		  };
  		  var _fails = function (exec) {
  		    try {
  		      return !!exec();
  		    } catch (e) {
  		      return true;
  		    }
  		  };

  		  // Thank's IE8 for his funny defineProperty
  		  var _descriptors = !_fails(function () {
  		    return Object.defineProperty({}, 'a', {
  		      get: function () {
  		        return 7;
  		      }
  		    }).a != 7;
  		  });
  		  var document = _global.document;
  		  // typeof document.createElement is 'object' in old IE
  		  var is = _isObject(document) && _isObject(document.createElement);
  		  var _domCreate = function (it) {
  		    return is ? document.createElement(it) : {};
  		  };
  		  var _ie8DomDefine = !_descriptors && !_fails(function () {
  		    return Object.defineProperty(_domCreate('div'), 'a', {
  		      get: function () {
  		        return 7;
  		      }
  		    }).a != 7;
  		  });

  		  // 7.1.1 ToPrimitive(input [, PreferredType])

  		  // instead of the ES6 spec version, we didn't implement @@toPrimitive case
  		  // and the second argument - flag - preferred type is a string
  		  var _toPrimitive = function (it, S) {
  		    if (!_isObject(it)) {
  		      return it;
  		    }
  		    var fn, val;
  		    if (typeof (fn = it.toString) == 'function' && !_isObject(val = fn.call(it))) {
  		      return val;
  		    }
  		    if (typeof (fn = it.valueOf) == 'function' && !_isObject(val = fn.call(it))) {
  		      return val;
  		    }
  		    throw TypeError("Can't convert object to primitive value");
  		  };
  		  var dP = Object.defineProperty;
  		  var f = _descriptors ? Object.defineProperty : function defineProperty(O, P, Attributes) {
  		    _anObject(O);
  		    P = _toPrimitive(P);
  		    _anObject(Attributes);
  		    if (_ie8DomDefine) {
  		      try {
  		        return dP(O, P, Attributes);
  		      } catch (e) {/* empty */}
  		    }
  		    if ('get' in Attributes || 'set' in Attributes) {
  		      throw TypeError('Accessors not supported!');
  		    }
  		    if ('value' in Attributes) {
  		      O[P] = Attributes.value;
  		    }
  		    return O;
  		  };
  		  var _objectDp = {
  		    f: f
  		  };
  		  var _propertyDesc = function (bitmap, value) {
  		    return {
  		      enumerable: !(bitmap & 1),
  		      configurable: !(bitmap & 2),
  		      writable: !(bitmap & 4),
  		      value: value
  		    };
  		  };
  		  var _hide = _descriptors ? function (object, key, value) {
  		    return _objectDp.f(object, key, _propertyDesc(1, value));
  		  } : function (object, key, value) {
  		    object[key] = value;
  		    return object;
  		  };
  		  var hasOwnProperty = {}.hasOwnProperty;
  		  var _has = function (it, key) {
  		    return hasOwnProperty.call(it, key);
  		  };
  		  var id = 0;
  		  var px = Math.random();
  		  var _uid = function (key) {
  		    return 'Symbol('.concat(key, ')_', (++id + px).toString(36));
  		  };
  		  var _shared = createCommonjsModule(function (module) {
  		    var SHARED = '__core-js_shared__';
  		    var store = _global[SHARED] || (_global[SHARED] = {});
  		    (module.exports = function (key, value) {
  		      return store[key] || (store[key] = value !== undefined ? value : {});
  		    })('versions', []).push({
  		      version: _core.version,
  		      mode: 'global',
  		      copyright: '© 2019 Denis Pushkarev (zloirock.ru)'
  		    });
  		  });
  		  var _functionToString = _shared('native-function-to-string', Function.toString);
  		  var _redefine = createCommonjsModule(function (module) {
  		    var SRC = _uid('src');
  		    var TO_STRING = 'toString';
  		    var TPL = ('' + _functionToString).split(TO_STRING);
  		    _core.inspectSource = function (it) {
  		      return _functionToString.call(it);
  		    };
  		    (module.exports = function (O, key, val, safe) {
  		      var isFunction = typeof val == 'function';
  		      if (isFunction) {
  		        _has(val, 'name') || _hide(val, 'name', key);
  		      }
  		      if (O[key] === val) {
  		        return;
  		      }
  		      if (isFunction) {
  		        _has(val, SRC) || _hide(val, SRC, O[key] ? '' + O[key] : TPL.join(String(key)));
  		      }
  		      if (O === _global) {
  		        O[key] = val;
  		      } else if (!safe) {
  		        delete O[key];
  		        _hide(O, key, val);
  		      } else if (O[key]) {
  		        O[key] = val;
  		      } else {
  		        _hide(O, key, val);
  		      }
  		      // add fake Function#toString for correct work wrapped methods / constructors with methods like LoDash isNative
  		    })(Function.prototype, TO_STRING, function toString() {
  		      return typeof this == 'function' && this[SRC] || _functionToString.call(this);
  		    });
  		  });
  		  var _aFunction = function (it) {
  		    if (typeof it != 'function') {
  		      throw TypeError(it + ' is not a function!');
  		    }
  		    return it;
  		  };

  		  // optional / simple context binding

  		  var _ctx = function (fn, that, length) {
  		    _aFunction(fn);
  		    if (that === undefined) {
  		      return fn;
  		    }
  		    switch (length) {
  		      case 1:
  		        return function (a) {
  		          return fn.call(that, a);
  		        };
  		      case 2:
  		        return function (a, b) {
  		          return fn.call(that, a, b);
  		        };
  		      case 3:
  		        return function (a, b, c) {
  		          return fn.call(that, a, b, c);
  		        };
  		    }
  		    return function /* ...args */
  		    () {
  		      return fn.apply(that, arguments);
  		    };
  		  };
  		  var PROTOTYPE = 'prototype';
  		  var $export = function (type, name, source) {
  		    var IS_FORCED = type & $export.F;
  		    var IS_GLOBAL = type & $export.G;
  		    var IS_STATIC = type & $export.S;
  		    var IS_PROTO = type & $export.P;
  		    var IS_BIND = type & $export.B;
  		    var target = IS_GLOBAL ? _global : IS_STATIC ? _global[name] || (_global[name] = {}) : (_global[name] || {})[PROTOTYPE];
  		    var exports = IS_GLOBAL ? _core : _core[name] || (_core[name] = {});
  		    var expProto = exports[PROTOTYPE] || (exports[PROTOTYPE] = {});
  		    var key, own, out, exp;
  		    if (IS_GLOBAL) {
  		      source = name;
  		    }
  		    for (key in source) {
  		      // contains in native
  		      own = !IS_FORCED && target && target[key] !== undefined;
  		      // export native or passed
  		      out = (own ? target : source)[key];
  		      // bind timers to global for call from export context
  		      exp = IS_BIND && own ? _ctx(out, _global) : IS_PROTO && typeof out == 'function' ? _ctx(Function.call, out) : out;
  		      // extend global
  		      if (target) {
  		        _redefine(target, key, out, type & $export.U);
  		      }
  		      // export
  		      if (exports[key] != out) {
  		        _hide(exports, key, exp);
  		      }
  		      if (IS_PROTO && expProto[key] != out) {
  		        expProto[key] = out;
  		      }
  		    }
  		  };
  		  _global.core = _core;
  		  // type bitmap
  		  $export.F = 1; // forced
  		  $export.G = 2; // global
  		  $export.S = 4; // static
  		  $export.P = 8; // proto
  		  $export.B = 16; // bind
  		  $export.W = 32; // wrap
  		  $export.U = 64; // safe
  		  $export.R = 128; // real proto method for `library`
  		  var _export = $export;

  		  // 7.1.4 ToInteger
  		  var ceil = Math.ceil;
  		  var floor = Math.floor;
  		  var _toInteger = function (it) {
  		    return isNaN(it = +it) ? 0 : (it > 0 ? floor : ceil)(it);
  		  };

  		  // 7.2.1 RequireObjectCoercible(argument)
  		  var _defined = function (it) {
  		    if (it == undefined) {
  		      throw TypeError("Can't call method on  " + it);
  		    }
  		    return it;
  		  };

  		  // true  -> String#at
  		  // false -> String#codePointAt
  		  var _stringAt = function (TO_STRING) {
  		    return function (that, pos) {
  		      var s = String(_defined(that));
  		      var i = _toInteger(pos);
  		      var l = s.length;
  		      var a, b;
  		      if (i < 0 || i >= l) {
  		        return undefined;
  		      }
  		      a = s.charCodeAt(i);
  		      return a < 0xd800 || a > 0xdbff || i + 1 === l || (b = s.charCodeAt(i + 1)) < 0xdc00 || b > 0xdfff ? a : (a - 0xd800 << 10) + (b - 0xdc00) + 0x10000;
  		    };
  		  };
  		  var $at = _stringAt();
  		  _export(_export.P, 'String', {
  		    // 21.1.3.3 String.prototype.codePointAt(pos)
  		    codePointAt: function codePointAt(pos) {
  		      return $at(this, pos);
  		    }
  		  });
  		  _core.String.codePointAt;
  		  var max = Math.max;
  		  var min = Math.min;
  		  var _toAbsoluteIndex = function (index, length) {
  		    index = _toInteger(index);
  		    return index < 0 ? max(index + length, 0) : min(index, length);
  		  };
  		  var fromCharCode = String.fromCharCode;
  		  var $fromCodePoint = String.fromCodePoint;

  		  // length should be 1, old FF problem
  		  _export(_export.S + _export.F * (!!$fromCodePoint && $fromCodePoint.length != 1), 'String', {
  		    // 21.1.2.2 String.fromCodePoint(...codePoints)
  		    fromCodePoint: function fromCodePoint(x) {
  		      var arguments$1 = arguments;
  		      // eslint-disable-line no-unused-vars
  		      var res = [];
  		      var aLen = arguments.length;
  		      var i = 0;
  		      var code;
  		      while (aLen > i) {
  		        code = +arguments$1[i++];
  		        if (_toAbsoluteIndex(code, 0x10ffff) !== code) {
  		          throw RangeError(code + ' is not a valid code point');
  		        }
  		        res.push(code < 0x10000 ? fromCharCode(code) : fromCharCode(((code -= 0x10000) >> 10) + 0xd800, code % 0x400 + 0xdc00));
  		      }
  		      return res.join('');
  		    }
  		  });
  		  _core.String.fromCodePoint;

  		  // This is a generated file. Do not edit.
  		  var Space_Separator = /[\u1680\u2000-\u200A\u202F\u205F\u3000]/;
  		  var ID_Start = /[\xAA\xB5\xBA\xC0-\xD6\xD8-\xF6\xF8-\u02C1\u02C6-\u02D1\u02E0-\u02E4\u02EC\u02EE\u0370-\u0374\u0376\u0377\u037A-\u037D\u037F\u0386\u0388-\u038A\u038C\u038E-\u03A1\u03A3-\u03F5\u03F7-\u0481\u048A-\u052F\u0531-\u0556\u0559\u0561-\u0587\u05D0-\u05EA\u05F0-\u05F2\u0620-\u064A\u066E\u066F\u0671-\u06D3\u06D5\u06E5\u06E6\u06EE\u06EF\u06FA-\u06FC\u06FF\u0710\u0712-\u072F\u074D-\u07A5\u07B1\u07CA-\u07EA\u07F4\u07F5\u07FA\u0800-\u0815\u081A\u0824\u0828\u0840-\u0858\u0860-\u086A\u08A0-\u08B4\u08B6-\u08BD\u0904-\u0939\u093D\u0950\u0958-\u0961\u0971-\u0980\u0985-\u098C\u098F\u0990\u0993-\u09A8\u09AA-\u09B0\u09B2\u09B6-\u09B9\u09BD\u09CE\u09DC\u09DD\u09DF-\u09E1\u09F0\u09F1\u09FC\u0A05-\u0A0A\u0A0F\u0A10\u0A13-\u0A28\u0A2A-\u0A30\u0A32\u0A33\u0A35\u0A36\u0A38\u0A39\u0A59-\u0A5C\u0A5E\u0A72-\u0A74\u0A85-\u0A8D\u0A8F-\u0A91\u0A93-\u0AA8\u0AAA-\u0AB0\u0AB2\u0AB3\u0AB5-\u0AB9\u0ABD\u0AD0\u0AE0\u0AE1\u0AF9\u0B05-\u0B0C\u0B0F\u0B10\u0B13-\u0B28\u0B2A-\u0B30\u0B32\u0B33\u0B35-\u0B39\u0B3D\u0B5C\u0B5D\u0B5F-\u0B61\u0B71\u0B83\u0B85-\u0B8A\u0B8E-\u0B90\u0B92-\u0B95\u0B99\u0B9A\u0B9C\u0B9E\u0B9F\u0BA3\u0BA4\u0BA8-\u0BAA\u0BAE-\u0BB9\u0BD0\u0C05-\u0C0C\u0C0E-\u0C10\u0C12-\u0C28\u0C2A-\u0C39\u0C3D\u0C58-\u0C5A\u0C60\u0C61\u0C80\u0C85-\u0C8C\u0C8E-\u0C90\u0C92-\u0CA8\u0CAA-\u0CB3\u0CB5-\u0CB9\u0CBD\u0CDE\u0CE0\u0CE1\u0CF1\u0CF2\u0D05-\u0D0C\u0D0E-\u0D10\u0D12-\u0D3A\u0D3D\u0D4E\u0D54-\u0D56\u0D5F-\u0D61\u0D7A-\u0D7F\u0D85-\u0D96\u0D9A-\u0DB1\u0DB3-\u0DBB\u0DBD\u0DC0-\u0DC6\u0E01-\u0E30\u0E32\u0E33\u0E40-\u0E46\u0E81\u0E82\u0E84\u0E87\u0E88\u0E8A\u0E8D\u0E94-\u0E97\u0E99-\u0E9F\u0EA1-\u0EA3\u0EA5\u0EA7\u0EAA\u0EAB\u0EAD-\u0EB0\u0EB2\u0EB3\u0EBD\u0EC0-\u0EC4\u0EC6\u0EDC-\u0EDF\u0F00\u0F40-\u0F47\u0F49-\u0F6C\u0F88-\u0F8C\u1000-\u102A\u103F\u1050-\u1055\u105A-\u105D\u1061\u1065\u1066\u106E-\u1070\u1075-\u1081\u108E\u10A0-\u10C5\u10C7\u10CD\u10D0-\u10FA\u10FC-\u1248\u124A-\u124D\u1250-\u1256\u1258\u125A-\u125D\u1260-\u1288\u128A-\u128D\u1290-\u12B0\u12B2-\u12B5\u12B8-\u12BE\u12C0\u12C2-\u12C5\u12C8-\u12D6\u12D8-\u1310\u1312-\u1315\u1318-\u135A\u1380-\u138F\u13A0-\u13F5\u13F8-\u13FD\u1401-\u166C\u166F-\u167F\u1681-\u169A\u16A0-\u16EA\u16EE-\u16F8\u1700-\u170C\u170E-\u1711\u1720-\u1731\u1740-\u1751\u1760-\u176C\u176E-\u1770\u1780-\u17B3\u17D7\u17DC\u1820-\u1877\u1880-\u1884\u1887-\u18A8\u18AA\u18B0-\u18F5\u1900-\u191E\u1950-\u196D\u1970-\u1974\u1980-\u19AB\u19B0-\u19C9\u1A00-\u1A16\u1A20-\u1A54\u1AA7\u1B05-\u1B33\u1B45-\u1B4B\u1B83-\u1BA0\u1BAE\u1BAF\u1BBA-\u1BE5\u1C00-\u1C23\u1C4D-\u1C4F\u1C5A-\u1C7D\u1C80-\u1C88\u1CE9-\u1CEC\u1CEE-\u1CF1\u1CF5\u1CF6\u1D00-\u1DBF\u1E00-\u1F15\u1F18-\u1F1D\u1F20-\u1F45\u1F48-\u1F4D\u1F50-\u1F57\u1F59\u1F5B\u1F5D\u1F5F-\u1F7D\u1F80-\u1FB4\u1FB6-\u1FBC\u1FBE\u1FC2-\u1FC4\u1FC6-\u1FCC\u1FD0-\u1FD3\u1FD6-\u1FDB\u1FE0-\u1FEC\u1FF2-\u1FF4\u1FF6-\u1FFC\u2071\u207F\u2090-\u209C\u2102\u2107\u210A-\u2113\u2115\u2119-\u211D\u2124\u2126\u2128\u212A-\u212D\u212F-\u2139\u213C-\u213F\u2145-\u2149\u214E\u2160-\u2188\u2C00-\u2C2E\u2C30-\u2C5E\u2C60-\u2CE4\u2CEB-\u2CEE\u2CF2\u2CF3\u2D00-\u2D25\u2D27\u2D2D\u2D30-\u2D67\u2D6F\u2D80-\u2D96\u2DA0-\u2DA6\u2DA8-\u2DAE\u2DB0-\u2DB6\u2DB8-\u2DBE\u2DC0-\u2DC6\u2DC8-\u2DCE\u2DD0-\u2DD6\u2DD8-\u2DDE\u2E2F\u3005-\u3007\u3021-\u3029\u3031-\u3035\u3038-\u303C\u3041-\u3096\u309D-\u309F\u30A1-\u30FA\u30FC-\u30FF\u3105-\u312E\u3131-\u318E\u31A0-\u31BA\u31F0-\u31FF\u3400-\u4DB5\u4E00-\u9FEA\uA000-\uA48C\uA4D0-\uA4FD\uA500-\uA60C\uA610-\uA61F\uA62A\uA62B\uA640-\uA66E\uA67F-\uA69D\uA6A0-\uA6EF\uA717-\uA71F\uA722-\uA788\uA78B-\uA7AE\uA7B0-\uA7B7\uA7F7-\uA801\uA803-\uA805\uA807-\uA80A\uA80C-\uA822\uA840-\uA873\uA882-\uA8B3\uA8F2-\uA8F7\uA8FB\uA8FD\uA90A-\uA925\uA930-\uA946\uA960-\uA97C\uA984-\uA9B2\uA9CF\uA9E0-\uA9E4\uA9E6-\uA9EF\uA9FA-\uA9FE\uAA00-\uAA28\uAA40-\uAA42\uAA44-\uAA4B\uAA60-\uAA76\uAA7A\uAA7E-\uAAAF\uAAB1\uAAB5\uAAB6\uAAB9-\uAABD\uAAC0\uAAC2\uAADB-\uAADD\uAAE0-\uAAEA\uAAF2-\uAAF4\uAB01-\uAB06\uAB09-\uAB0E\uAB11-\uAB16\uAB20-\uAB26\uAB28-\uAB2E\uAB30-\uAB5A\uAB5C-\uAB65\uAB70-\uABE2\uAC00-\uD7A3\uD7B0-\uD7C6\uD7CB-\uD7FB\uF900-\uFA6D\uFA70-\uFAD9\uFB00-\uFB06\uFB13-\uFB17\uFB1D\uFB1F-\uFB28\uFB2A-\uFB36\uFB38-\uFB3C\uFB3E\uFB40\uFB41\uFB43\uFB44\uFB46-\uFBB1\uFBD3-\uFD3D\uFD50-\uFD8F\uFD92-\uFDC7\uFDF0-\uFDFB\uFE70-\uFE74\uFE76-\uFEFC\uFF21-\uFF3A\uFF41-\uFF5A\uFF66-\uFFBE\uFFC2-\uFFC7\uFFCA-\uFFCF\uFFD2-\uFFD7\uFFDA-\uFFDC]|\uD800[\uDC00-\uDC0B\uDC0D-\uDC26\uDC28-\uDC3A\uDC3C\uDC3D\uDC3F-\uDC4D\uDC50-\uDC5D\uDC80-\uDCFA\uDD40-\uDD74\uDE80-\uDE9C\uDEA0-\uDED0\uDF00-\uDF1F\uDF2D-\uDF4A\uDF50-\uDF75\uDF80-\uDF9D\uDFA0-\uDFC3\uDFC8-\uDFCF\uDFD1-\uDFD5]|\uD801[\uDC00-\uDC9D\uDCB0-\uDCD3\uDCD8-\uDCFB\uDD00-\uDD27\uDD30-\uDD63\uDE00-\uDF36\uDF40-\uDF55\uDF60-\uDF67]|\uD802[\uDC00-\uDC05\uDC08\uDC0A-\uDC35\uDC37\uDC38\uDC3C\uDC3F-\uDC55\uDC60-\uDC76\uDC80-\uDC9E\uDCE0-\uDCF2\uDCF4\uDCF5\uDD00-\uDD15\uDD20-\uDD39\uDD80-\uDDB7\uDDBE\uDDBF\uDE00\uDE10-\uDE13\uDE15-\uDE17\uDE19-\uDE33\uDE60-\uDE7C\uDE80-\uDE9C\uDEC0-\uDEC7\uDEC9-\uDEE4\uDF00-\uDF35\uDF40-\uDF55\uDF60-\uDF72\uDF80-\uDF91]|\uD803[\uDC00-\uDC48\uDC80-\uDCB2\uDCC0-\uDCF2]|\uD804[\uDC03-\uDC37\uDC83-\uDCAF\uDCD0-\uDCE8\uDD03-\uDD26\uDD50-\uDD72\uDD76\uDD83-\uDDB2\uDDC1-\uDDC4\uDDDA\uDDDC\uDE00-\uDE11\uDE13-\uDE2B\uDE80-\uDE86\uDE88\uDE8A-\uDE8D\uDE8F-\uDE9D\uDE9F-\uDEA8\uDEB0-\uDEDE\uDF05-\uDF0C\uDF0F\uDF10\uDF13-\uDF28\uDF2A-\uDF30\uDF32\uDF33\uDF35-\uDF39\uDF3D\uDF50\uDF5D-\uDF61]|\uD805[\uDC00-\uDC34\uDC47-\uDC4A\uDC80-\uDCAF\uDCC4\uDCC5\uDCC7\uDD80-\uDDAE\uDDD8-\uDDDB\uDE00-\uDE2F\uDE44\uDE80-\uDEAA\uDF00-\uDF19]|\uD806[\uDCA0-\uDCDF\uDCFF\uDE00\uDE0B-\uDE32\uDE3A\uDE50\uDE5C-\uDE83\uDE86-\uDE89\uDEC0-\uDEF8]|\uD807[\uDC00-\uDC08\uDC0A-\uDC2E\uDC40\uDC72-\uDC8F\uDD00-\uDD06\uDD08\uDD09\uDD0B-\uDD30\uDD46]|\uD808[\uDC00-\uDF99]|\uD809[\uDC00-\uDC6E\uDC80-\uDD43]|[\uD80C\uD81C-\uD820\uD840-\uD868\uD86A-\uD86C\uD86F-\uD872\uD874-\uD879][\uDC00-\uDFFF]|\uD80D[\uDC00-\uDC2E]|\uD811[\uDC00-\uDE46]|\uD81A[\uDC00-\uDE38\uDE40-\uDE5E\uDED0-\uDEED\uDF00-\uDF2F\uDF40-\uDF43\uDF63-\uDF77\uDF7D-\uDF8F]|\uD81B[\uDF00-\uDF44\uDF50\uDF93-\uDF9F\uDFE0\uDFE1]|\uD821[\uDC00-\uDFEC]|\uD822[\uDC00-\uDEF2]|\uD82C[\uDC00-\uDD1E\uDD70-\uDEFB]|\uD82F[\uDC00-\uDC6A\uDC70-\uDC7C\uDC80-\uDC88\uDC90-\uDC99]|\uD835[\uDC00-\uDC54\uDC56-\uDC9C\uDC9E\uDC9F\uDCA2\uDCA5\uDCA6\uDCA9-\uDCAC\uDCAE-\uDCB9\uDCBB\uDCBD-\uDCC3\uDCC5-\uDD05\uDD07-\uDD0A\uDD0D-\uDD14\uDD16-\uDD1C\uDD1E-\uDD39\uDD3B-\uDD3E\uDD40-\uDD44\uDD46\uDD4A-\uDD50\uDD52-\uDEA5\uDEA8-\uDEC0\uDEC2-\uDEDA\uDEDC-\uDEFA\uDEFC-\uDF14\uDF16-\uDF34\uDF36-\uDF4E\uDF50-\uDF6E\uDF70-\uDF88\uDF8A-\uDFA8\uDFAA-\uDFC2\uDFC4-\uDFCB]|\uD83A[\uDC00-\uDCC4\uDD00-\uDD43]|\uD83B[\uDE00-\uDE03\uDE05-\uDE1F\uDE21\uDE22\uDE24\uDE27\uDE29-\uDE32\uDE34-\uDE37\uDE39\uDE3B\uDE42\uDE47\uDE49\uDE4B\uDE4D-\uDE4F\uDE51\uDE52\uDE54\uDE57\uDE59\uDE5B\uDE5D\uDE5F\uDE61\uDE62\uDE64\uDE67-\uDE6A\uDE6C-\uDE72\uDE74-\uDE77\uDE79-\uDE7C\uDE7E\uDE80-\uDE89\uDE8B-\uDE9B\uDEA1-\uDEA3\uDEA5-\uDEA9\uDEAB-\uDEBB]|\uD869[\uDC00-\uDED6\uDF00-\uDFFF]|\uD86D[\uDC00-\uDF34\uDF40-\uDFFF]|\uD86E[\uDC00-\uDC1D\uDC20-\uDFFF]|\uD873[\uDC00-\uDEA1\uDEB0-\uDFFF]|\uD87A[\uDC00-\uDFE0]|\uD87E[\uDC00-\uDE1D]/;
  		  var ID_Continue = /[\xAA\xB5\xBA\xC0-\xD6\xD8-\xF6\xF8-\u02C1\u02C6-\u02D1\u02E0-\u02E4\u02EC\u02EE\u0300-\u0374\u0376\u0377\u037A-\u037D\u037F\u0386\u0388-\u038A\u038C\u038E-\u03A1\u03A3-\u03F5\u03F7-\u0481\u0483-\u0487\u048A-\u052F\u0531-\u0556\u0559\u0561-\u0587\u0591-\u05BD\u05BF\u05C1\u05C2\u05C4\u05C5\u05C7\u05D0-\u05EA\u05F0-\u05F2\u0610-\u061A\u0620-\u0669\u066E-\u06D3\u06D5-\u06DC\u06DF-\u06E8\u06EA-\u06FC\u06FF\u0710-\u074A\u074D-\u07B1\u07C0-\u07F5\u07FA\u0800-\u082D\u0840-\u085B\u0860-\u086A\u08A0-\u08B4\u08B6-\u08BD\u08D4-\u08E1\u08E3-\u0963\u0966-\u096F\u0971-\u0983\u0985-\u098C\u098F\u0990\u0993-\u09A8\u09AA-\u09B0\u09B2\u09B6-\u09B9\u09BC-\u09C4\u09C7\u09C8\u09CB-\u09CE\u09D7\u09DC\u09DD\u09DF-\u09E3\u09E6-\u09F1\u09FC\u0A01-\u0A03\u0A05-\u0A0A\u0A0F\u0A10\u0A13-\u0A28\u0A2A-\u0A30\u0A32\u0A33\u0A35\u0A36\u0A38\u0A39\u0A3C\u0A3E-\u0A42\u0A47\u0A48\u0A4B-\u0A4D\u0A51\u0A59-\u0A5C\u0A5E\u0A66-\u0A75\u0A81-\u0A83\u0A85-\u0A8D\u0A8F-\u0A91\u0A93-\u0AA8\u0AAA-\u0AB0\u0AB2\u0AB3\u0AB5-\u0AB9\u0ABC-\u0AC5\u0AC7-\u0AC9\u0ACB-\u0ACD\u0AD0\u0AE0-\u0AE3\u0AE6-\u0AEF\u0AF9-\u0AFF\u0B01-\u0B03\u0B05-\u0B0C\u0B0F\u0B10\u0B13-\u0B28\u0B2A-\u0B30\u0B32\u0B33\u0B35-\u0B39\u0B3C-\u0B44\u0B47\u0B48\u0B4B-\u0B4D\u0B56\u0B57\u0B5C\u0B5D\u0B5F-\u0B63\u0B66-\u0B6F\u0B71\u0B82\u0B83\u0B85-\u0B8A\u0B8E-\u0B90\u0B92-\u0B95\u0B99\u0B9A\u0B9C\u0B9E\u0B9F\u0BA3\u0BA4\u0BA8-\u0BAA\u0BAE-\u0BB9\u0BBE-\u0BC2\u0BC6-\u0BC8\u0BCA-\u0BCD\u0BD0\u0BD7\u0BE6-\u0BEF\u0C00-\u0C03\u0C05-\u0C0C\u0C0E-\u0C10\u0C12-\u0C28\u0C2A-\u0C39\u0C3D-\u0C44\u0C46-\u0C48\u0C4A-\u0C4D\u0C55\u0C56\u0C58-\u0C5A\u0C60-\u0C63\u0C66-\u0C6F\u0C80-\u0C83\u0C85-\u0C8C\u0C8E-\u0C90\u0C92-\u0CA8\u0CAA-\u0CB3\u0CB5-\u0CB9\u0CBC-\u0CC4\u0CC6-\u0CC8\u0CCA-\u0CCD\u0CD5\u0CD6\u0CDE\u0CE0-\u0CE3\u0CE6-\u0CEF\u0CF1\u0CF2\u0D00-\u0D03\u0D05-\u0D0C\u0D0E-\u0D10\u0D12-\u0D44\u0D46-\u0D48\u0D4A-\u0D4E\u0D54-\u0D57\u0D5F-\u0D63\u0D66-\u0D6F\u0D7A-\u0D7F\u0D82\u0D83\u0D85-\u0D96\u0D9A-\u0DB1\u0DB3-\u0DBB\u0DBD\u0DC0-\u0DC6\u0DCA\u0DCF-\u0DD4\u0DD6\u0DD8-\u0DDF\u0DE6-\u0DEF\u0DF2\u0DF3\u0E01-\u0E3A\u0E40-\u0E4E\u0E50-\u0E59\u0E81\u0E82\u0E84\u0E87\u0E88\u0E8A\u0E8D\u0E94-\u0E97\u0E99-\u0E9F\u0EA1-\u0EA3\u0EA5\u0EA7\u0EAA\u0EAB\u0EAD-\u0EB9\u0EBB-\u0EBD\u0EC0-\u0EC4\u0EC6\u0EC8-\u0ECD\u0ED0-\u0ED9\u0EDC-\u0EDF\u0F00\u0F18\u0F19\u0F20-\u0F29\u0F35\u0F37\u0F39\u0F3E-\u0F47\u0F49-\u0F6C\u0F71-\u0F84\u0F86-\u0F97\u0F99-\u0FBC\u0FC6\u1000-\u1049\u1050-\u109D\u10A0-\u10C5\u10C7\u10CD\u10D0-\u10FA\u10FC-\u1248\u124A-\u124D\u1250-\u1256\u1258\u125A-\u125D\u1260-\u1288\u128A-\u128D\u1290-\u12B0\u12B2-\u12B5\u12B8-\u12BE\u12C0\u12C2-\u12C5\u12C8-\u12D6\u12D8-\u1310\u1312-\u1315\u1318-\u135A\u135D-\u135F\u1380-\u138F\u13A0-\u13F5\u13F8-\u13FD\u1401-\u166C\u166F-\u167F\u1681-\u169A\u16A0-\u16EA\u16EE-\u16F8\u1700-\u170C\u170E-\u1714\u1720-\u1734\u1740-\u1753\u1760-\u176C\u176E-\u1770\u1772\u1773\u1780-\u17D3\u17D7\u17DC\u17DD\u17E0-\u17E9\u180B-\u180D\u1810-\u1819\u1820-\u1877\u1880-\u18AA\u18B0-\u18F5\u1900-\u191E\u1920-\u192B\u1930-\u193B\u1946-\u196D\u1970-\u1974\u1980-\u19AB\u19B0-\u19C9\u19D0-\u19D9\u1A00-\u1A1B\u1A20-\u1A5E\u1A60-\u1A7C\u1A7F-\u1A89\u1A90-\u1A99\u1AA7\u1AB0-\u1ABD\u1B00-\u1B4B\u1B50-\u1B59\u1B6B-\u1B73\u1B80-\u1BF3\u1C00-\u1C37\u1C40-\u1C49\u1C4D-\u1C7D\u1C80-\u1C88\u1CD0-\u1CD2\u1CD4-\u1CF9\u1D00-\u1DF9\u1DFB-\u1F15\u1F18-\u1F1D\u1F20-\u1F45\u1F48-\u1F4D\u1F50-\u1F57\u1F59\u1F5B\u1F5D\u1F5F-\u1F7D\u1F80-\u1FB4\u1FB6-\u1FBC\u1FBE\u1FC2-\u1FC4\u1FC6-\u1FCC\u1FD0-\u1FD3\u1FD6-\u1FDB\u1FE0-\u1FEC\u1FF2-\u1FF4\u1FF6-\u1FFC\u203F\u2040\u2054\u2071\u207F\u2090-\u209C\u20D0-\u20DC\u20E1\u20E5-\u20F0\u2102\u2107\u210A-\u2113\u2115\u2119-\u211D\u2124\u2126\u2128\u212A-\u212D\u212F-\u2139\u213C-\u213F\u2145-\u2149\u214E\u2160-\u2188\u2C00-\u2C2E\u2C30-\u2C5E\u2C60-\u2CE4\u2CEB-\u2CF3\u2D00-\u2D25\u2D27\u2D2D\u2D30-\u2D67\u2D6F\u2D7F-\u2D96\u2DA0-\u2DA6\u2DA8-\u2DAE\u2DB0-\u2DB6\u2DB8-\u2DBE\u2DC0-\u2DC6\u2DC8-\u2DCE\u2DD0-\u2DD6\u2DD8-\u2DDE\u2DE0-\u2DFF\u2E2F\u3005-\u3007\u3021-\u302F\u3031-\u3035\u3038-\u303C\u3041-\u3096\u3099\u309A\u309D-\u309F\u30A1-\u30FA\u30FC-\u30FF\u3105-\u312E\u3131-\u318E\u31A0-\u31BA\u31F0-\u31FF\u3400-\u4DB5\u4E00-\u9FEA\uA000-\uA48C\uA4D0-\uA4FD\uA500-\uA60C\uA610-\uA62B\uA640-\uA66F\uA674-\uA67D\uA67F-\uA6F1\uA717-\uA71F\uA722-\uA788\uA78B-\uA7AE\uA7B0-\uA7B7\uA7F7-\uA827\uA840-\uA873\uA880-\uA8C5\uA8D0-\uA8D9\uA8E0-\uA8F7\uA8FB\uA8FD\uA900-\uA92D\uA930-\uA953\uA960-\uA97C\uA980-\uA9C0\uA9CF-\uA9D9\uA9E0-\uA9FE\uAA00-\uAA36\uAA40-\uAA4D\uAA50-\uAA59\uAA60-\uAA76\uAA7A-\uAAC2\uAADB-\uAADD\uAAE0-\uAAEF\uAAF2-\uAAF6\uAB01-\uAB06\uAB09-\uAB0E\uAB11-\uAB16\uAB20-\uAB26\uAB28-\uAB2E\uAB30-\uAB5A\uAB5C-\uAB65\uAB70-\uABEA\uABEC\uABED\uABF0-\uABF9\uAC00-\uD7A3\uD7B0-\uD7C6\uD7CB-\uD7FB\uF900-\uFA6D\uFA70-\uFAD9\uFB00-\uFB06\uFB13-\uFB17\uFB1D-\uFB28\uFB2A-\uFB36\uFB38-\uFB3C\uFB3E\uFB40\uFB41\uFB43\uFB44\uFB46-\uFBB1\uFBD3-\uFD3D\uFD50-\uFD8F\uFD92-\uFDC7\uFDF0-\uFDFB\uFE00-\uFE0F\uFE20-\uFE2F\uFE33\uFE34\uFE4D-\uFE4F\uFE70-\uFE74\uFE76-\uFEFC\uFF10-\uFF19\uFF21-\uFF3A\uFF3F\uFF41-\uFF5A\uFF66-\uFFBE\uFFC2-\uFFC7\uFFCA-\uFFCF\uFFD2-\uFFD7\uFFDA-\uFFDC]|\uD800[\uDC00-\uDC0B\uDC0D-\uDC26\uDC28-\uDC3A\uDC3C\uDC3D\uDC3F-\uDC4D\uDC50-\uDC5D\uDC80-\uDCFA\uDD40-\uDD74\uDDFD\uDE80-\uDE9C\uDEA0-\uDED0\uDEE0\uDF00-\uDF1F\uDF2D-\uDF4A\uDF50-\uDF7A\uDF80-\uDF9D\uDFA0-\uDFC3\uDFC8-\uDFCF\uDFD1-\uDFD5]|\uD801[\uDC00-\uDC9D\uDCA0-\uDCA9\uDCB0-\uDCD3\uDCD8-\uDCFB\uDD00-\uDD27\uDD30-\uDD63\uDE00-\uDF36\uDF40-\uDF55\uDF60-\uDF67]|\uD802[\uDC00-\uDC05\uDC08\uDC0A-\uDC35\uDC37\uDC38\uDC3C\uDC3F-\uDC55\uDC60-\uDC76\uDC80-\uDC9E\uDCE0-\uDCF2\uDCF4\uDCF5\uDD00-\uDD15\uDD20-\uDD39\uDD80-\uDDB7\uDDBE\uDDBF\uDE00-\uDE03\uDE05\uDE06\uDE0C-\uDE13\uDE15-\uDE17\uDE19-\uDE33\uDE38-\uDE3A\uDE3F\uDE60-\uDE7C\uDE80-\uDE9C\uDEC0-\uDEC7\uDEC9-\uDEE6\uDF00-\uDF35\uDF40-\uDF55\uDF60-\uDF72\uDF80-\uDF91]|\uD803[\uDC00-\uDC48\uDC80-\uDCB2\uDCC0-\uDCF2]|\uD804[\uDC00-\uDC46\uDC66-\uDC6F\uDC7F-\uDCBA\uDCD0-\uDCE8\uDCF0-\uDCF9\uDD00-\uDD34\uDD36-\uDD3F\uDD50-\uDD73\uDD76\uDD80-\uDDC4\uDDCA-\uDDCC\uDDD0-\uDDDA\uDDDC\uDE00-\uDE11\uDE13-\uDE37\uDE3E\uDE80-\uDE86\uDE88\uDE8A-\uDE8D\uDE8F-\uDE9D\uDE9F-\uDEA8\uDEB0-\uDEEA\uDEF0-\uDEF9\uDF00-\uDF03\uDF05-\uDF0C\uDF0F\uDF10\uDF13-\uDF28\uDF2A-\uDF30\uDF32\uDF33\uDF35-\uDF39\uDF3C-\uDF44\uDF47\uDF48\uDF4B-\uDF4D\uDF50\uDF57\uDF5D-\uDF63\uDF66-\uDF6C\uDF70-\uDF74]|\uD805[\uDC00-\uDC4A\uDC50-\uDC59\uDC80-\uDCC5\uDCC7\uDCD0-\uDCD9\uDD80-\uDDB5\uDDB8-\uDDC0\uDDD8-\uDDDD\uDE00-\uDE40\uDE44\uDE50-\uDE59\uDE80-\uDEB7\uDEC0-\uDEC9\uDF00-\uDF19\uDF1D-\uDF2B\uDF30-\uDF39]|\uD806[\uDCA0-\uDCE9\uDCFF\uDE00-\uDE3E\uDE47\uDE50-\uDE83\uDE86-\uDE99\uDEC0-\uDEF8]|\uD807[\uDC00-\uDC08\uDC0A-\uDC36\uDC38-\uDC40\uDC50-\uDC59\uDC72-\uDC8F\uDC92-\uDCA7\uDCA9-\uDCB6\uDD00-\uDD06\uDD08\uDD09\uDD0B-\uDD36\uDD3A\uDD3C\uDD3D\uDD3F-\uDD47\uDD50-\uDD59]|\uD808[\uDC00-\uDF99]|\uD809[\uDC00-\uDC6E\uDC80-\uDD43]|[\uD80C\uD81C-\uD820\uD840-\uD868\uD86A-\uD86C\uD86F-\uD872\uD874-\uD879][\uDC00-\uDFFF]|\uD80D[\uDC00-\uDC2E]|\uD811[\uDC00-\uDE46]|\uD81A[\uDC00-\uDE38\uDE40-\uDE5E\uDE60-\uDE69\uDED0-\uDEED\uDEF0-\uDEF4\uDF00-\uDF36\uDF40-\uDF43\uDF50-\uDF59\uDF63-\uDF77\uDF7D-\uDF8F]|\uD81B[\uDF00-\uDF44\uDF50-\uDF7E\uDF8F-\uDF9F\uDFE0\uDFE1]|\uD821[\uDC00-\uDFEC]|\uD822[\uDC00-\uDEF2]|\uD82C[\uDC00-\uDD1E\uDD70-\uDEFB]|\uD82F[\uDC00-\uDC6A\uDC70-\uDC7C\uDC80-\uDC88\uDC90-\uDC99\uDC9D\uDC9E]|\uD834[\uDD65-\uDD69\uDD6D-\uDD72\uDD7B-\uDD82\uDD85-\uDD8B\uDDAA-\uDDAD\uDE42-\uDE44]|\uD835[\uDC00-\uDC54\uDC56-\uDC9C\uDC9E\uDC9F\uDCA2\uDCA5\uDCA6\uDCA9-\uDCAC\uDCAE-\uDCB9\uDCBB\uDCBD-\uDCC3\uDCC5-\uDD05\uDD07-\uDD0A\uDD0D-\uDD14\uDD16-\uDD1C\uDD1E-\uDD39\uDD3B-\uDD3E\uDD40-\uDD44\uDD46\uDD4A-\uDD50\uDD52-\uDEA5\uDEA8-\uDEC0\uDEC2-\uDEDA\uDEDC-\uDEFA\uDEFC-\uDF14\uDF16-\uDF34\uDF36-\uDF4E\uDF50-\uDF6E\uDF70-\uDF88\uDF8A-\uDFA8\uDFAA-\uDFC2\uDFC4-\uDFCB\uDFCE-\uDFFF]|\uD836[\uDE00-\uDE36\uDE3B-\uDE6C\uDE75\uDE84\uDE9B-\uDE9F\uDEA1-\uDEAF]|\uD838[\uDC00-\uDC06\uDC08-\uDC18\uDC1B-\uDC21\uDC23\uDC24\uDC26-\uDC2A]|\uD83A[\uDC00-\uDCC4\uDCD0-\uDCD6\uDD00-\uDD4A\uDD50-\uDD59]|\uD83B[\uDE00-\uDE03\uDE05-\uDE1F\uDE21\uDE22\uDE24\uDE27\uDE29-\uDE32\uDE34-\uDE37\uDE39\uDE3B\uDE42\uDE47\uDE49\uDE4B\uDE4D-\uDE4F\uDE51\uDE52\uDE54\uDE57\uDE59\uDE5B\uDE5D\uDE5F\uDE61\uDE62\uDE64\uDE67-\uDE6A\uDE6C-\uDE72\uDE74-\uDE77\uDE79-\uDE7C\uDE7E\uDE80-\uDE89\uDE8B-\uDE9B\uDEA1-\uDEA3\uDEA5-\uDEA9\uDEAB-\uDEBB]|\uD869[\uDC00-\uDED6\uDF00-\uDFFF]|\uD86D[\uDC00-\uDF34\uDF40-\uDFFF]|\uD86E[\uDC00-\uDC1D\uDC20-\uDFFF]|\uD873[\uDC00-\uDEA1\uDEB0-\uDFFF]|\uD87A[\uDC00-\uDFE0]|\uD87E[\uDC00-\uDE1D]|\uDB40[\uDD00-\uDDEF]/;
  		  var unicode = {
  		    Space_Separator: Space_Separator,
  		    ID_Start: ID_Start,
  		    ID_Continue: ID_Continue
  		  };
  		  var util = {
  		    isSpaceSeparator: function isSpaceSeparator(c) {
  		      return typeof c === 'string' && unicode.Space_Separator.test(c);
  		    },
  		    isIdStartChar: function isIdStartChar(c) {
  		      return typeof c === 'string' && (c >= 'a' && c <= 'z' || c >= 'A' && c <= 'Z' || c === '$' || c === '_' || unicode.ID_Start.test(c));
  		    },
  		    isIdContinueChar: function isIdContinueChar(c) {
  		      return typeof c === 'string' && (c >= 'a' && c <= 'z' || c >= 'A' && c <= 'Z' || c >= '0' && c <= '9' || c === '$' || c === '_' || c === '\u200C' || c === '\u200D' || unicode.ID_Continue.test(c));
  		    },
  		    isDigit: function isDigit(c) {
  		      return typeof c === 'string' && /[0-9]/.test(c);
  		    },
  		    isHexDigit: function isHexDigit(c) {
  		      return typeof c === 'string' && /[0-9A-Fa-f]/.test(c);
  		    }
  		  };
  		  var source;
  		  var parseState;
  		  var stack;
  		  var pos;
  		  var line;
  		  var column;
  		  var token;
  		  var key;
  		  var root;
  		  var parse = function parse(text, reviver) {
  		    source = String(text);
  		    parseState = 'start';
  		    stack = [];
  		    pos = 0;
  		    line = 1;
  		    column = 0;
  		    token = undefined;
  		    key = undefined;
  		    root = undefined;
  		    do {
  		      token = lex();

  		      // This code is unreachable.
  		      // if (!parseStates[parseState]) {
  		      //     throw invalidParseState()
  		      // }

  		      parseStates[parseState]();
  		    } while (token.type !== 'eof');
  		    if (typeof reviver === 'function') {
  		      return internalize({
  		        '': root
  		      }, '', reviver);
  		    }
  		    return root;
  		  };
  		  function internalize(holder, name, reviver) {
  		    var value = holder[name];
  		    if (value != null && typeof value === 'object') {
  		      if (Array.isArray(value)) {
  		        for (var i = 0; i < value.length; i++) {
  		          var key = String(i);
  		          var replacement = internalize(value, key, reviver);
  		          if (replacement === undefined) {
  		            delete value[key];
  		          } else {
  		            Object.defineProperty(value, key, {
  		              value: replacement,
  		              writable: true,
  		              enumerable: true,
  		              configurable: true
  		            });
  		          }
  		        }
  		      } else {
  		        for (var key$1 in value) {
  		          var replacement$1 = internalize(value, key$1, reviver);
  		          if (replacement$1 === undefined) {
  		            delete value[key$1];
  		          } else {
  		            Object.defineProperty(value, key$1, {
  		              value: replacement$1,
  		              writable: true,
  		              enumerable: true,
  		              configurable: true
  		            });
  		          }
  		        }
  		      }
  		    }
  		    return reviver.call(holder, name, value);
  		  }
  		  var lexState;
  		  var buffer;
  		  var doubleQuote;
  		  var sign;
  		  var c;
  		  function lex() {
  		    lexState = 'default';
  		    buffer = '';
  		    doubleQuote = false;
  		    sign = 1;
  		    for (;;) {
  		      c = peek();

  		      // This code is unreachable.
  		      // if (!lexStates[lexState]) {
  		      //     throw invalidLexState(lexState)
  		      // }

  		      var token = lexStates[lexState]();
  		      if (token) {
  		        return token;
  		      }
  		    }
  		  }
  		  function peek() {
  		    if (source[pos]) {
  		      return String.fromCodePoint(source.codePointAt(pos));
  		    }
  		  }
  		  function read() {
  		    var c = peek();
  		    if (c === '\n') {
  		      line++;
  		      column = 0;
  		    } else if (c) {
  		      column += c.length;
  		    } else {
  		      column++;
  		    }
  		    if (c) {
  		      pos += c.length;
  		    }
  		    return c;
  		  }
  		  var lexStates = {
  		    default: function default$1() {
  		      switch (c) {
  		        case '\t':
  		        case '\v':
  		        case '\f':
  		        case ' ':
  		        case '\u00A0':
  		        case '\uFEFF':
  		        case '\n':
  		        case '\r':
  		        case '\u2028':
  		        case '\u2029':
  		          read();
  		          return;
  		        case '/':
  		          read();
  		          lexState = 'comment';
  		          return;
  		        case undefined:
  		          read();
  		          return newToken('eof');
  		      }
  		      if (util.isSpaceSeparator(c)) {
  		        read();
  		        return;
  		      }

  		      // This code is unreachable.
  		      // if (!lexStates[parseState]) {
  		      //     throw invalidLexState(parseState)
  		      // }

  		      return lexStates[parseState]();
  		    },
  		    comment: function comment() {
  		      switch (c) {
  		        case '*':
  		          read();
  		          lexState = 'multiLineComment';
  		          return;
  		        case '/':
  		          read();
  		          lexState = 'singleLineComment';
  		          return;
  		      }
  		      throw invalidChar(read());
  		    },
  		    multiLineComment: function multiLineComment() {
  		      switch (c) {
  		        case '*':
  		          read();
  		          lexState = 'multiLineCommentAsterisk';
  		          return;
  		        case undefined:
  		          throw invalidChar(read());
  		      }
  		      read();
  		    },
  		    multiLineCommentAsterisk: function multiLineCommentAsterisk() {
  		      switch (c) {
  		        case '*':
  		          read();
  		          return;
  		        case '/':
  		          read();
  		          lexState = 'default';
  		          return;
  		        case undefined:
  		          throw invalidChar(read());
  		      }
  		      read();
  		      lexState = 'multiLineComment';
  		    },
  		    singleLineComment: function singleLineComment() {
  		      switch (c) {
  		        case '\n':
  		        case '\r':
  		        case '\u2028':
  		        case '\u2029':
  		          read();
  		          lexState = 'default';
  		          return;
  		        case undefined:
  		          read();
  		          return newToken('eof');
  		      }
  		      read();
  		    },
  		    value: function value() {
  		      switch (c) {
  		        case '{':
  		        case '[':
  		          return newToken('punctuator', read());
  		        case 'n':
  		          read();
  		          literal('ull');
  		          return newToken('null', null);
  		        case 't':
  		          read();
  		          literal('rue');
  		          return newToken('boolean', true);
  		        case 'f':
  		          read();
  		          literal('alse');
  		          return newToken('boolean', false);
  		        case '-':
  		        case '+':
  		          if (read() === '-') {
  		            sign = -1;
  		          }
  		          lexState = 'sign';
  		          return;
  		        case '.':
  		          buffer = read();
  		          lexState = 'decimalPointLeading';
  		          return;
  		        case '0':
  		          buffer = read();
  		          lexState = 'zero';
  		          return;
  		        case '1':
  		        case '2':
  		        case '3':
  		        case '4':
  		        case '5':
  		        case '6':
  		        case '7':
  		        case '8':
  		        case '9':
  		          buffer = read();
  		          lexState = 'decimalInteger';
  		          return;
  		        case 'I':
  		          read();
  		          literal('nfinity');
  		          return newToken('numeric', Infinity);
  		        case 'N':
  		          read();
  		          literal('aN');
  		          return newToken('numeric', NaN);
  		        case '"':
  		        case "'":
  		          doubleQuote = read() === '"';
  		          buffer = '';
  		          lexState = 'string';
  		          return;
  		      }
  		      throw invalidChar(read());
  		    },
  		    identifierNameStartEscape: function identifierNameStartEscape() {
  		      if (c !== 'u') {
  		        throw invalidChar(read());
  		      }
  		      read();
  		      var u = unicodeEscape();
  		      switch (u) {
  		        case '$':
  		        case '_':
  		          break;
  		        default:
  		          if (!util.isIdStartChar(u)) {
  		            throw invalidIdentifier();
  		          }
  		          break;
  		      }
  		      buffer += u;
  		      lexState = 'identifierName';
  		    },
  		    identifierName: function identifierName() {
  		      switch (c) {
  		        case '$':
  		        case '_':
  		        case '\u200C':
  		        case '\u200D':
  		          buffer += read();
  		          return;
  		        case '\\':
  		          read();
  		          lexState = 'identifierNameEscape';
  		          return;
  		      }
  		      if (util.isIdContinueChar(c)) {
  		        buffer += read();
  		        return;
  		      }
  		      return newToken('identifier', buffer);
  		    },
  		    identifierNameEscape: function identifierNameEscape() {
  		      if (c !== 'u') {
  		        throw invalidChar(read());
  		      }
  		      read();
  		      var u = unicodeEscape();
  		      switch (u) {
  		        case '$':
  		        case '_':
  		        case '\u200C':
  		        case '\u200D':
  		          break;
  		        default:
  		          if (!util.isIdContinueChar(u)) {
  		            throw invalidIdentifier();
  		          }
  		          break;
  		      }
  		      buffer += u;
  		      lexState = 'identifierName';
  		    },
  		    sign: function sign$1() {
  		      switch (c) {
  		        case '.':
  		          buffer = read();
  		          lexState = 'decimalPointLeading';
  		          return;
  		        case '0':
  		          buffer = read();
  		          lexState = 'zero';
  		          return;
  		        case '1':
  		        case '2':
  		        case '3':
  		        case '4':
  		        case '5':
  		        case '6':
  		        case '7':
  		        case '8':
  		        case '9':
  		          buffer = read();
  		          lexState = 'decimalInteger';
  		          return;
  		        case 'I':
  		          read();
  		          literal('nfinity');
  		          return newToken('numeric', sign * Infinity);
  		        case 'N':
  		          read();
  		          literal('aN');
  		          return newToken('numeric', NaN);
  		      }
  		      throw invalidChar(read());
  		    },
  		    zero: function zero() {
  		      switch (c) {
  		        case '.':
  		          buffer += read();
  		          lexState = 'decimalPoint';
  		          return;
  		        case 'e':
  		        case 'E':
  		          buffer += read();
  		          lexState = 'decimalExponent';
  		          return;
  		        case 'x':
  		        case 'X':
  		          buffer += read();
  		          lexState = 'hexadecimal';
  		          return;
  		      }
  		      return newToken('numeric', sign * 0);
  		    },
  		    decimalInteger: function decimalInteger() {
  		      switch (c) {
  		        case '.':
  		          buffer += read();
  		          lexState = 'decimalPoint';
  		          return;
  		        case 'e':
  		        case 'E':
  		          buffer += read();
  		          lexState = 'decimalExponent';
  		          return;
  		      }
  		      if (util.isDigit(c)) {
  		        buffer += read();
  		        return;
  		      }
  		      return newToken('numeric', sign * Number(buffer));
  		    },
  		    decimalPointLeading: function decimalPointLeading() {
  		      if (util.isDigit(c)) {
  		        buffer += read();
  		        lexState = 'decimalFraction';
  		        return;
  		      }
  		      throw invalidChar(read());
  		    },
  		    decimalPoint: function decimalPoint() {
  		      switch (c) {
  		        case 'e':
  		        case 'E':
  		          buffer += read();
  		          lexState = 'decimalExponent';
  		          return;
  		      }
  		      if (util.isDigit(c)) {
  		        buffer += read();
  		        lexState = 'decimalFraction';
  		        return;
  		      }
  		      return newToken('numeric', sign * Number(buffer));
  		    },
  		    decimalFraction: function decimalFraction() {
  		      switch (c) {
  		        case 'e':
  		        case 'E':
  		          buffer += read();
  		          lexState = 'decimalExponent';
  		          return;
  		      }
  		      if (util.isDigit(c)) {
  		        buffer += read();
  		        return;
  		      }
  		      return newToken('numeric', sign * Number(buffer));
  		    },
  		    decimalExponent: function decimalExponent() {
  		      switch (c) {
  		        case '+':
  		        case '-':
  		          buffer += read();
  		          lexState = 'decimalExponentSign';
  		          return;
  		      }
  		      if (util.isDigit(c)) {
  		        buffer += read();
  		        lexState = 'decimalExponentInteger';
  		        return;
  		      }
  		      throw invalidChar(read());
  		    },
  		    decimalExponentSign: function decimalExponentSign() {
  		      if (util.isDigit(c)) {
  		        buffer += read();
  		        lexState = 'decimalExponentInteger';
  		        return;
  		      }
  		      throw invalidChar(read());
  		    },
  		    decimalExponentInteger: function decimalExponentInteger() {
  		      if (util.isDigit(c)) {
  		        buffer += read();
  		        return;
  		      }
  		      return newToken('numeric', sign * Number(buffer));
  		    },
  		    hexadecimal: function hexadecimal() {
  		      if (util.isHexDigit(c)) {
  		        buffer += read();
  		        lexState = 'hexadecimalInteger';
  		        return;
  		      }
  		      throw invalidChar(read());
  		    },
  		    hexadecimalInteger: function hexadecimalInteger() {
  		      if (util.isHexDigit(c)) {
  		        buffer += read();
  		        return;
  		      }
  		      return newToken('numeric', sign * Number(buffer));
  		    },
  		    string: function string() {
  		      switch (c) {
  		        case '\\':
  		          read();
  		          buffer += escape();
  		          return;
  		        case '"':
  		          if (doubleQuote) {
  		            read();
  		            return newToken('string', buffer);
  		          }
  		          buffer += read();
  		          return;
  		        case "'":
  		          if (!doubleQuote) {
  		            read();
  		            return newToken('string', buffer);
  		          }
  		          buffer += read();
  		          return;
  		        case '\n':
  		        case '\r':
  		          throw invalidChar(read());
  		        case '\u2028':
  		        case '\u2029':
  		          separatorChar(c);
  		          break;
  		        case undefined:
  		          throw invalidChar(read());
  		      }
  		      buffer += read();
  		    },
  		    start: function start() {
  		      switch (c) {
  		        case '{':
  		        case '[':
  		          return newToken('punctuator', read());

  		        // This code is unreachable since the default lexState handles eof.
  		        // case undefined:
  		        //     return newToken('eof')
  		      }
  		      lexState = 'value';
  		    },
  		    beforePropertyName: function beforePropertyName() {
  		      switch (c) {
  		        case '$':
  		        case '_':
  		          buffer = read();
  		          lexState = 'identifierName';
  		          return;
  		        case '\\':
  		          read();
  		          lexState = 'identifierNameStartEscape';
  		          return;
  		        case '}':
  		          return newToken('punctuator', read());
  		        case '"':
  		        case "'":
  		          doubleQuote = read() === '"';
  		          lexState = 'string';
  		          return;
  		      }
  		      if (util.isIdStartChar(c)) {
  		        buffer += read();
  		        lexState = 'identifierName';
  		        return;
  		      }
  		      throw invalidChar(read());
  		    },
  		    afterPropertyName: function afterPropertyName() {
  		      if (c === ':') {
  		        return newToken('punctuator', read());
  		      }
  		      throw invalidChar(read());
  		    },
  		    beforePropertyValue: function beforePropertyValue() {
  		      lexState = 'value';
  		    },
  		    afterPropertyValue: function afterPropertyValue() {
  		      switch (c) {
  		        case ',':
  		        case '}':
  		          return newToken('punctuator', read());
  		      }
  		      throw invalidChar(read());
  		    },
  		    beforeArrayValue: function beforeArrayValue() {
  		      if (c === ']') {
  		        return newToken('punctuator', read());
  		      }
  		      lexState = 'value';
  		    },
  		    afterArrayValue: function afterArrayValue() {
  		      switch (c) {
  		        case ',':
  		        case ']':
  		          return newToken('punctuator', read());
  		      }
  		      throw invalidChar(read());
  		    },
  		    end: function end() {
  		      // This code is unreachable since it's handled by the default lexState.
  		      // if (c === undefined) {
  		      //     read()
  		      //     return newToken('eof')
  		      // }

  		      throw invalidChar(read());
  		    }
  		  };
  		  function newToken(type, value) {
  		    return {
  		      type: type,
  		      value: value,
  		      line: line,
  		      column: column
  		    };
  		  }
  		  function literal(s) {
  		    for (var i = 0, list = s; i < list.length; i += 1) {
  		      var c = list[i];
  		      var p = peek();
  		      if (p !== c) {
  		        throw invalidChar(read());
  		      }
  		      read();
  		    }
  		  }
  		  function escape() {
  		    var c = peek();
  		    switch (c) {
  		      case 'b':
  		        read();
  		        return '\b';
  		      case 'f':
  		        read();
  		        return '\f';
  		      case 'n':
  		        read();
  		        return '\n';
  		      case 'r':
  		        read();
  		        return '\r';
  		      case 't':
  		        read();
  		        return '\t';
  		      case 'v':
  		        read();
  		        return '\v';
  		      case '0':
  		        read();
  		        if (util.isDigit(peek())) {
  		          throw invalidChar(read());
  		        }
  		        return '\0';
  		      case 'x':
  		        read();
  		        return hexEscape();
  		      case 'u':
  		        read();
  		        return unicodeEscape();
  		      case '\n':
  		      case '\u2028':
  		      case '\u2029':
  		        read();
  		        return '';
  		      case '\r':
  		        read();
  		        if (peek() === '\n') {
  		          read();
  		        }
  		        return '';
  		      case '1':
  		      case '2':
  		      case '3':
  		      case '4':
  		      case '5':
  		      case '6':
  		      case '7':
  		      case '8':
  		      case '9':
  		        throw invalidChar(read());
  		      case undefined:
  		        throw invalidChar(read());
  		    }
  		    return read();
  		  }
  		  function hexEscape() {
  		    var buffer = '';
  		    var c = peek();
  		    if (!util.isHexDigit(c)) {
  		      throw invalidChar(read());
  		    }
  		    buffer += read();
  		    c = peek();
  		    if (!util.isHexDigit(c)) {
  		      throw invalidChar(read());
  		    }
  		    buffer += read();
  		    return String.fromCodePoint(parseInt(buffer, 16));
  		  }
  		  function unicodeEscape() {
  		    var buffer = '';
  		    var count = 4;
  		    while (count-- > 0) {
  		      var c = peek();
  		      if (!util.isHexDigit(c)) {
  		        throw invalidChar(read());
  		      }
  		      buffer += read();
  		    }
  		    return String.fromCodePoint(parseInt(buffer, 16));
  		  }
  		  var parseStates = {
  		    start: function start() {
  		      if (token.type === 'eof') {
  		        throw invalidEOF();
  		      }
  		      push();
  		    },
  		    beforePropertyName: function beforePropertyName() {
  		      switch (token.type) {
  		        case 'identifier':
  		        case 'string':
  		          key = token.value;
  		          parseState = 'afterPropertyName';
  		          return;
  		        case 'punctuator':
  		          // This code is unreachable since it's handled by the lexState.
  		          // if (token.value !== '}') {
  		          //     throw invalidToken()
  		          // }

  		          pop();
  		          return;
  		        case 'eof':
  		          throw invalidEOF();
  		      }

  		      // This code is unreachable since it's handled by the lexState.
  		      // throw invalidToken()
  		    },
  		    afterPropertyName: function afterPropertyName() {
  		      // This code is unreachable since it's handled by the lexState.
  		      // if (token.type !== 'punctuator' || token.value !== ':') {
  		      //     throw invalidToken()
  		      // }

  		      if (token.type === 'eof') {
  		        throw invalidEOF();
  		      }
  		      parseState = 'beforePropertyValue';
  		    },
  		    beforePropertyValue: function beforePropertyValue() {
  		      if (token.type === 'eof') {
  		        throw invalidEOF();
  		      }
  		      push();
  		    },
  		    beforeArrayValue: function beforeArrayValue() {
  		      if (token.type === 'eof') {
  		        throw invalidEOF();
  		      }
  		      if (token.type === 'punctuator' && token.value === ']') {
  		        pop();
  		        return;
  		      }
  		      push();
  		    },
  		    afterPropertyValue: function afterPropertyValue() {
  		      // This code is unreachable since it's handled by the lexState.
  		      // if (token.type !== 'punctuator') {
  		      //     throw invalidToken()
  		      // }

  		      if (token.type === 'eof') {
  		        throw invalidEOF();
  		      }
  		      switch (token.value) {
  		        case ',':
  		          parseState = 'beforePropertyName';
  		          return;
  		        case '}':
  		          pop();
  		      }

  		      // This code is unreachable since it's handled by the lexState.
  		      // throw invalidToken()
  		    },
  		    afterArrayValue: function afterArrayValue() {
  		      // This code is unreachable since it's handled by the lexState.
  		      // if (token.type !== 'punctuator') {
  		      //     throw invalidToken()
  		      // }

  		      if (token.type === 'eof') {
  		        throw invalidEOF();
  		      }
  		      switch (token.value) {
  		        case ',':
  		          parseState = 'beforeArrayValue';
  		          return;
  		        case ']':
  		          pop();
  		      }

  		      // This code is unreachable since it's handled by the lexState.
  		      // throw invalidToken()
  		    },
  		    end: function end() {
  		      // This code is unreachable since it's handled by the lexState.
  		      // if (token.type !== 'eof') {
  		      //     throw invalidToken()
  		      // }
  		    }
  		  };
  		  function push() {
  		    var value;
  		    switch (token.type) {
  		      case 'punctuator':
  		        switch (token.value) {
  		          case '{':
  		            value = {};
  		            break;
  		          case '[':
  		            value = [];
  		            break;
  		        }
  		        break;
  		      case 'null':
  		      case 'boolean':
  		      case 'numeric':
  		      case 'string':
  		        value = token.value;
  		        break;

  		      // This code is unreachable.
  		      // default:
  		      //     throw invalidToken()
  		    }
  		    if (root === undefined) {
  		      root = value;
  		    } else {
  		      var parent = stack[stack.length - 1];
  		      if (Array.isArray(parent)) {
  		        parent.push(value);
  		      } else {
  		        Object.defineProperty(parent, key, {
  		          value: value,
  		          writable: true,
  		          enumerable: true,
  		          configurable: true
  		        });
  		      }
  		    }
  		    if (value !== null && typeof value === 'object') {
  		      stack.push(value);
  		      if (Array.isArray(value)) {
  		        parseState = 'beforeArrayValue';
  		      } else {
  		        parseState = 'beforePropertyName';
  		      }
  		    } else {
  		      var current = stack[stack.length - 1];
  		      if (current == null) {
  		        parseState = 'end';
  		      } else if (Array.isArray(current)) {
  		        parseState = 'afterArrayValue';
  		      } else {
  		        parseState = 'afterPropertyValue';
  		      }
  		    }
  		  }
  		  function pop() {
  		    stack.pop();
  		    var current = stack[stack.length - 1];
  		    if (current == null) {
  		      parseState = 'end';
  		    } else if (Array.isArray(current)) {
  		      parseState = 'afterArrayValue';
  		    } else {
  		      parseState = 'afterPropertyValue';
  		    }
  		  }

  		  // This code is unreachable.
  		  // function invalidParseState () {
  		  //     return new Error(`JSON5: invalid parse state '${parseState}'`)
  		  // }

  		  // This code is unreachable.
  		  // function invalidLexState (state) {
  		  //     return new Error(`JSON5: invalid lex state '${state}'`)
  		  // }

  		  function invalidChar(c) {
  		    if (c === undefined) {
  		      return syntaxError("JSON5: invalid end of input at " + line + ":" + column);
  		    }
  		    return syntaxError("JSON5: invalid character '" + formatChar(c) + "' at " + line + ":" + column);
  		  }
  		  function invalidEOF() {
  		    return syntaxError("JSON5: invalid end of input at " + line + ":" + column);
  		  }

  		  // This code is unreachable.
  		  // function invalidToken () {
  		  //     if (token.type === 'eof') {
  		  //         return syntaxError(`JSON5: invalid end of input at ${line}:${column}`)
  		  //     }

  		  //     const c = String.fromCodePoint(token.value.codePointAt(0))
  		  //     return syntaxError(`JSON5: invalid character '${formatChar(c)}' at ${line}:${column}`)
  		  // }

  		  function invalidIdentifier() {
  		    column -= 5;
  		    return syntaxError("JSON5: invalid identifier character at " + line + ":" + column);
  		  }
  		  function separatorChar(c) {
  		    console.warn("JSON5: '" + formatChar(c) + "' in strings is not valid ECMAScript; consider escaping");
  		  }
  		  function formatChar(c) {
  		    var replacements = {
  		      "'": "\\'",
  		      '"': '\\"',
  		      '\\': '\\\\',
  		      '\b': '\\b',
  		      '\f': '\\f',
  		      '\n': '\\n',
  		      '\r': '\\r',
  		      '\t': '\\t',
  		      '\v': '\\v',
  		      '\0': '\\0',
  		      '\u2028': '\\u2028',
  		      '\u2029': '\\u2029'
  		    };
  		    if (replacements[c]) {
  		      return replacements[c];
  		    }
  		    if (c < ' ') {
  		      var hexString = c.charCodeAt(0).toString(16);
  		      return '\\x' + ('00' + hexString).substring(hexString.length);
  		    }
  		    return c;
  		  }
  		  function syntaxError(message) {
  		    var err = new SyntaxError(message);
  		    err.lineNumber = line;
  		    err.columnNumber = column;
  		    return err;
  		  }
  		  var stringify = function stringify(value, replacer, space) {
  		    var stack = [];
  		    var indent = '';
  		    var propertyList;
  		    var replacerFunc;
  		    var gap = '';
  		    var quote;
  		    if (replacer != null && typeof replacer === 'object' && !Array.isArray(replacer)) {
  		      space = replacer.space;
  		      quote = replacer.quote;
  		      replacer = replacer.replacer;
  		    }
  		    if (typeof replacer === 'function') {
  		      replacerFunc = replacer;
  		    } else if (Array.isArray(replacer)) {
  		      propertyList = [];
  		      for (var i = 0, list = replacer; i < list.length; i += 1) {
  		        var v = list[i];
  		        var item = void 0;
  		        if (typeof v === 'string') {
  		          item = v;
  		        } else if (typeof v === 'number' || v instanceof String || v instanceof Number) {
  		          item = String(v);
  		        }
  		        if (item !== undefined && propertyList.indexOf(item) < 0) {
  		          propertyList.push(item);
  		        }
  		      }
  		    }
  		    if (space instanceof Number) {
  		      space = Number(space);
  		    } else if (space instanceof String) {
  		      space = String(space);
  		    }
  		    if (typeof space === 'number') {
  		      if (space > 0) {
  		        space = Math.min(10, Math.floor(space));
  		        gap = '          '.substr(0, space);
  		      }
  		    } else if (typeof space === 'string') {
  		      gap = space.substr(0, 10);
  		    }
  		    return serializeProperty('', {
  		      '': value
  		    });
  		    function serializeProperty(key, holder) {
  		      var value = holder[key];
  		      if (value != null) {
  		        if (typeof value.toJSON5 === 'function') {
  		          value = value.toJSON5(key);
  		        } else if (typeof value.toJSON === 'function') {
  		          value = value.toJSON(key);
  		        }
  		      }
  		      if (replacerFunc) {
  		        value = replacerFunc.call(holder, key, value);
  		      }
  		      if (value instanceof Number) {
  		        value = Number(value);
  		      } else if (value instanceof String) {
  		        value = String(value);
  		      } else if (value instanceof Boolean) {
  		        value = value.valueOf();
  		      }
  		      switch (value) {
  		        case null:
  		          return 'null';
  		        case true:
  		          return 'true';
  		        case false:
  		          return 'false';
  		      }
  		      if (typeof value === 'string') {
  		        return quoteString(value);
  		      }
  		      if (typeof value === 'number') {
  		        return String(value);
  		      }
  		      if (typeof value === 'object') {
  		        return Array.isArray(value) ? serializeArray(value) : serializeObject(value);
  		      }
  		      return undefined;
  		    }
  		    function quoteString(value) {
  		      var quotes = {
  		        "'": 0.1,
  		        '"': 0.2
  		      };
  		      var replacements = {
  		        "'": "\\'",
  		        '"': '\\"',
  		        '\\': '\\\\',
  		        '\b': '\\b',
  		        '\f': '\\f',
  		        '\n': '\\n',
  		        '\r': '\\r',
  		        '\t': '\\t',
  		        '\v': '\\v',
  		        '\0': '\\0',
  		        '\u2028': '\\u2028',
  		        '\u2029': '\\u2029'
  		      };
  		      var product = '';
  		      for (var i = 0; i < value.length; i++) {
  		        var c = value[i];
  		        switch (c) {
  		          case "'":
  		          case '"':
  		            quotes[c]++;
  		            product += c;
  		            continue;
  		          case '\0':
  		            if (util.isDigit(value[i + 1])) {
  		              product += '\\x00';
  		              continue;
  		            }
  		        }
  		        if (replacements[c]) {
  		          product += replacements[c];
  		          continue;
  		        }
  		        if (c < ' ') {
  		          var hexString = c.charCodeAt(0).toString(16);
  		          product += '\\x' + ('00' + hexString).substring(hexString.length);
  		          continue;
  		        }
  		        product += c;
  		      }
  		      var quoteChar = quote || Object.keys(quotes).reduce(function (a, b) {
  		        return quotes[a] < quotes[b] ? a : b;
  		      });
  		      product = product.replace(new RegExp(quoteChar, 'g'), replacements[quoteChar]);
  		      return quoteChar + product + quoteChar;
  		    }
  		    function serializeObject(value) {
  		      if (stack.indexOf(value) >= 0) {
  		        throw TypeError('Converting circular structure to JSON5');
  		      }
  		      stack.push(value);
  		      var stepback = indent;
  		      indent = indent + gap;
  		      var keys = propertyList || Object.keys(value);
  		      var partial = [];
  		      for (var i = 0, list = keys; i < list.length; i += 1) {
  		        var key = list[i];
  		        var propertyString = serializeProperty(key, value);
  		        if (propertyString !== undefined) {
  		          var member = serializeKey(key) + ':';
  		          if (gap !== '') {
  		            member += ' ';
  		          }
  		          member += propertyString;
  		          partial.push(member);
  		        }
  		      }
  		      var final;
  		      if (partial.length === 0) {
  		        final = '{}';
  		      } else {
  		        var properties;
  		        if (gap === '') {
  		          properties = partial.join(',');
  		          final = '{' + properties + '}';
  		        } else {
  		          var separator = ',\n' + indent;
  		          properties = partial.join(separator);
  		          final = '{\n' + indent + properties + ',\n' + stepback + '}';
  		        }
  		      }
  		      stack.pop();
  		      indent = stepback;
  		      return final;
  		    }
  		    function serializeKey(key) {
  		      if (key.length === 0) {
  		        return quoteString(key);
  		      }
  		      var firstChar = String.fromCodePoint(key.codePointAt(0));
  		      if (!util.isIdStartChar(firstChar)) {
  		        return quoteString(key);
  		      }
  		      for (var i = firstChar.length; i < key.length; i++) {
  		        if (!util.isIdContinueChar(String.fromCodePoint(key.codePointAt(i)))) {
  		          return quoteString(key);
  		        }
  		      }
  		      return key;
  		    }
  		    function serializeArray(value) {
  		      if (stack.indexOf(value) >= 0) {
  		        throw TypeError('Converting circular structure to JSON5');
  		      }
  		      stack.push(value);
  		      var stepback = indent;
  		      indent = indent + gap;
  		      var partial = [];
  		      for (var i = 0; i < value.length; i++) {
  		        var propertyString = serializeProperty(String(i), value);
  		        partial.push(propertyString !== undefined ? propertyString : 'null');
  		      }
  		      var final;
  		      if (partial.length === 0) {
  		        final = '[]';
  		      } else {
  		        if (gap === '') {
  		          var properties = partial.join(',');
  		          final = '[' + properties + ']';
  		        } else {
  		          var separator = ',\n' + indent;
  		          var properties$1 = partial.join(separator);
  		          final = '[\n' + indent + properties$1 + ',\n' + stepback + ']';
  		        }
  		      }
  		      stack.pop();
  		      indent = stepback;
  		      return final;
  		    }
  		  };
  		  var JSON5 = {
  		    parse: parse,
  		    stringify: stringify
  		  };
  		  var lib = JSON5;
  		  var es5 = lib;
  		  return es5;
  		}); 
  	} (dist$1));
  	return dist$1.exports;
  }

  requireDist();

  var strftime = {exports: {}};

  var hasRequiredStrftime;

  function requireStrftime () {
  	if (hasRequiredStrftime) return strftime.exports;
  	hasRequiredStrftime = 1;
  	(function (module) {
  		(function () {
  		  var Locales = {
  		    de_DE: {
  		      days: ['Sonntag', 'Montag', 'Dienstag', 'Mittwoch', 'Donnerstag', 'Freitag', 'Samstag'],
  		      shortDays: ['So', 'Mo', 'Di', 'Mi', 'Do', 'Fr', 'Sa'],
  		      months: ['Januar', 'Februar', 'März', 'April', 'Mai', 'Juni', 'Juli', 'August', 'September', 'Oktober', 'November', 'Dezember'],
  		      shortMonths: ['Jan', 'Feb', 'Mär', 'Apr', 'Mai', 'Jun', 'Jul', 'Aug', 'Sep', 'Okt', 'Nov', 'Dez'],
  		      AM: 'AM',
  		      PM: 'PM',
  		      am: 'am',
  		      pm: 'pm',
  		      formats: {
  		        c: '%a %d %b %Y %X %Z',
  		        D: '%d.%m.%Y',
  		        F: '%Y-%m-%d',
  		        R: '%H:%M',
  		        r: '%I:%M:%S %p',
  		        T: '%H:%M:%S',
  		        v: '%e-%b-%Y',
  		        X: '%T',
  		        x: '%D'
  		      }
  		    },
  		    en_CA: {
  		      days: ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'],
  		      shortDays: ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat'],
  		      months: ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'],
  		      shortMonths: ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'],
  		      ordinalSuffixes: ['st', 'nd', 'rd', 'th', 'th', 'th', 'th', 'th', 'th', 'th', 'th', 'th', 'th', 'th', 'th', 'th', 'th', 'th', 'th', 'th', 'st', 'nd', 'rd', 'th', 'th', 'th', 'th', 'th', 'th', 'th', 'st'],
  		      AM: 'AM',
  		      PM: 'PM',
  		      am: 'am',
  		      pm: 'pm',
  		      formats: {
  		        c: '%a %d %b %Y %X %Z',
  		        D: '%d/%m/%y',
  		        F: '%Y-%m-%d',
  		        R: '%H:%M',
  		        r: '%I:%M:%S %p',
  		        T: '%H:%M:%S',
  		        v: '%e-%b-%Y',
  		        X: '%r',
  		        x: '%D'
  		      }
  		    },
  		    en_US: {
  		      days: ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'],
  		      shortDays: ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat'],
  		      months: ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'],
  		      shortMonths: ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'],
  		      ordinalSuffixes: ['st', 'nd', 'rd', 'th', 'th', 'th', 'th', 'th', 'th', 'th', 'th', 'th', 'th', 'th', 'th', 'th', 'th', 'th', 'th', 'th', 'st', 'nd', 'rd', 'th', 'th', 'th', 'th', 'th', 'th', 'th', 'st'],
  		      AM: 'AM',
  		      PM: 'PM',
  		      am: 'am',
  		      pm: 'pm',
  		      formats: {
  		        c: '%a %d %b %Y %X %Z',
  		        D: '%m/%d/%y',
  		        F: '%Y-%m-%d',
  		        R: '%H:%M',
  		        r: '%I:%M:%S %p',
  		        T: '%H:%M:%S',
  		        v: '%e-%b-%Y',
  		        X: '%r',
  		        x: '%D'
  		      }
  		    },
  		    es_MX: {
  		      days: ['domingo', 'lunes', 'martes', 'miércoles', 'jueves', 'viernes', 'sábado'],
  		      shortDays: ['dom', 'lun', 'mar', 'mié', 'jue', 'vie', 'sáb'],
  		      months: ['enero', 'febrero', 'marzo', 'abril', 'mayo', 'junio', 'julio', 'agosto', 'septiembre', 'octubre', 'noviembre', ' diciembre'],
  		      shortMonths: ['ene', 'feb', 'mar', 'abr', 'may', 'jun', 'jul', 'ago', 'sep', 'oct', 'nov', 'dic'],
  		      AM: 'AM',
  		      PM: 'PM',
  		      am: 'am',
  		      pm: 'pm',
  		      formats: {
  		        c: '%a %d %b %Y %X %Z',
  		        D: '%d/%m/%Y',
  		        F: '%Y-%m-%d',
  		        R: '%H:%M',
  		        r: '%I:%M:%S %p',
  		        T: '%H:%M:%S',
  		        v: '%e-%b-%Y',
  		        X: '%T',
  		        x: '%D'
  		      }
  		    },
  		    fr_FR: {
  		      days: ['dimanche', 'lundi', 'mardi', 'mercredi', 'jeudi', 'vendredi', 'samedi'],
  		      shortDays: ['dim.', 'lun.', 'mar.', 'mer.', 'jeu.', 'ven.', 'sam.'],
  		      months: ['janvier', 'février', 'mars', 'avril', 'mai', 'juin', 'juillet', 'août', 'septembre', 'octobre', 'novembre', 'décembre'],
  		      shortMonths: ['janv.', 'févr.', 'mars', 'avril', 'mai', 'juin', 'juil.', 'août', 'sept.', 'oct.', 'nov.', 'déc.'],
  		      AM: 'AM',
  		      PM: 'PM',
  		      am: 'am',
  		      pm: 'pm',
  		      formats: {
  		        c: '%a %d %b %Y %X %Z',
  		        D: '%d/%m/%Y',
  		        F: '%Y-%m-%d',
  		        R: '%H:%M',
  		        r: '%I:%M:%S %p',
  		        T: '%H:%M:%S',
  		        v: '%e-%b-%Y',
  		        X: '%T',
  		        x: '%D'
  		      }
  		    },
  		    it_IT: {
  		      days: ['domenica', 'lunedì', 'martedì', 'mercoledì', 'giovedì', 'venerdì', 'sabato'],
  		      shortDays: ['dom', 'lun', 'mar', 'mer', 'gio', 'ven', 'sab'],
  		      months: ['gennaio', 'febbraio', 'marzo', 'aprile', 'maggio', 'giugno', 'luglio', 'agosto', 'settembre', 'ottobre', 'novembre', 'dicembre'],
  		      shortMonths: ['pr', 'mag', 'giu', 'lug', 'ago', 'set', 'ott', 'nov', 'dic'],
  		      AM: 'AM',
  		      PM: 'PM',
  		      am: 'am',
  		      pm: 'pm',
  		      formats: {
  		        c: '%a %d %b %Y %X %Z',
  		        D: '%d/%m/%Y',
  		        F: '%Y-%m-%d',
  		        R: '%H:%M',
  		        r: '%I:%M:%S %p',
  		        T: '%H:%M:%S',
  		        v: '%e-%b-%Y',
  		        X: '%T',
  		        x: '%D'
  		      }
  		    },
  		    nl_NL: {
  		      days: ['zondag', 'maandag', 'dinsdag', 'woensdag', 'donderdag', 'vrijdag', 'zaterdag'],
  		      shortDays: ['zo', 'ma', 'di', 'wo', 'do', 'vr', 'za'],
  		      months: ['januari', 'februari', 'maart', 'april', 'mei', 'juni', 'juli', 'augustus', 'september', 'oktober', 'november', 'december'],
  		      shortMonths: ['jan', 'feb', 'mrt', 'apr', 'mei', 'jun', 'jul', 'aug', 'sep', 'okt', 'nov', 'dec'],
  		      AM: 'AM',
  		      PM: 'PM',
  		      am: 'am',
  		      pm: 'pm',
  		      formats: {
  		        c: '%a %d %b %Y %X %Z',
  		        D: '%d-%m-%y',
  		        F: '%Y-%m-%d',
  		        R: '%H:%M',
  		        r: '%I:%M:%S %p',
  		        T: '%H:%M:%S',
  		        v: '%e-%b-%Y',
  		        X: '%T',
  		        x: '%D'
  		      }
  		    },
  		    pt_BR: {
  		      days: ['domingo', 'segunda', 'terça', 'quarta', 'quinta', 'sexta', 'sábado'],
  		      shortDays: ['Dom', 'Seg', 'Ter', 'Qua', 'Qui', 'Sex', 'Sáb'],
  		      months: ['janeiro', 'fevereiro', 'março', 'abril', 'maio', 'junho', 'julho', 'agosto', 'setembro', 'outubro', 'novembro', 'dezembro'],
  		      shortMonths: ['Jan', 'Fev', 'Mar', 'Abr', 'Mai', 'Jun', 'Jul', 'Ago', 'Set', 'Out', 'Nov', 'Dez'],
  		      AM: 'AM',
  		      PM: 'PM',
  		      am: 'am',
  		      pm: 'pm',
  		      formats: {
  		        c: '%a %d %b %Y %X %Z',
  		        D: '%d-%m-%Y',
  		        F: '%Y-%m-%d',
  		        R: '%H:%M',
  		        r: '%I:%M:%S %p',
  		        T: '%H:%M:%S',
  		        v: '%e-%b-%Y',
  		        X: '%T',
  		        x: '%D'
  		      }
  		    },
  		    ru_RU: {
  		      days: ['Воскресенье', 'Понедельник', 'Вторник', 'Среда', 'Четверг', 'Пятница', 'Суббота'],
  		      shortDays: ['Вс', 'Пн', 'Вт', 'Ср', 'Чт', 'Пт', 'Сб'],
  		      months: ['Январь', 'Февраль', 'Март', 'Апрель', 'Май', 'Июнь', 'Июль', 'Август', 'Сентябрь', 'Октябрь', 'Ноябрь', 'Декабрь'],
  		      shortMonths: ['янв', 'фев', 'мар', 'апр', 'май', 'июн', 'июл', 'авг', 'сен', 'окт', 'ноя', 'дек'],
  		      AM: 'AM',
  		      PM: 'PM',
  		      am: 'am',
  		      pm: 'pm',
  		      formats: {
  		        c: '%a %d %b %Y %X',
  		        D: '%d.%m.%y',
  		        F: '%Y-%m-%d',
  		        R: '%H:%M',
  		        r: '%I:%M:%S %p',
  		        T: '%H:%M:%S',
  		        v: '%e-%b-%Y',
  		        X: '%T',
  		        x: '%D'
  		      }
  		    },
  		    tr_TR: {
  		      days: ['Pazar', 'Pazartesi', 'Salı', 'Çarşamba', 'Perşembe', 'Cuma', 'Cumartesi'],
  		      shortDays: ['Paz', 'Pzt', 'Sal', 'Çrş', 'Prş', 'Cum', 'Cts'],
  		      months: ['Ocak', 'Şubat', 'Mart', 'Nisan', 'Mayıs', 'Haziran', 'Temmuz', 'Ağustos', 'Eylül', 'Ekim', 'Kasım', 'Aralık'],
  		      shortMonths: ['Oca', 'Şub', 'Mar', 'Nis', 'May', 'Haz', 'Tem', 'Ağu', 'Eyl', 'Eki', 'Kas', 'Ara'],
  		      AM: 'ÖÖ',
  		      PM: 'ÖS',
  		      am: 'ÖÖ',
  		      pm: 'ÖS',
  		      formats: {
  		        c: '%a %d %b %Y %X %Z',
  		        D: '%d-%m-%Y',
  		        F: '%Y-%m-%d',
  		        R: '%H:%M',
  		        r: '%I:%M:%S %p',
  		        T: '%H:%M:%S',
  		        v: '%e-%b-%Y',
  		        X: '%T',
  		        x: '%D'
  		      }
  		    },
  		    // By michaeljayt<michaeljayt@gmail.com>
  		    // https://github.com/michaeljayt/strftime/commit/bcb4c12743811d51e568175aa7bff3fd2a77cef3
  		    zh_CN: {
  		      days: ['星期日', '星期一', '星期二', '星期三', '星期四', '星期五', '星期六'],
  		      shortDays: ['日', '一', '二', '三', '四', '五', '六'],
  		      months: ['一月份', '二月份', '三月份', '四月份', '五月份', '六月份', '七月份', '八月份', '九月份', '十月份', '十一月份', '十二月份'],
  		      shortMonths: ['一月', '二月', '三月', '四月', '五月', '六月', '七月', '八月', '九月', '十月', '十一月', '十二月'],
  		      AM: '上午',
  		      PM: '下午',
  		      am: '上午',
  		      pm: '下午',
  		      formats: {
  		        c: '%a %d %b %Y %X %Z',
  		        D: '%d/%m/%y',
  		        F: '%Y-%m-%d',
  		        R: '%H:%M',
  		        r: '%I:%M:%S %p',
  		        T: '%H:%M:%S',
  		        v: '%e-%b-%Y',
  		        X: '%r',
  		        x: '%D'
  		      }
  		    }
  		  };
  		  var DefaultLocale = Locales['en_US'],
  		    defaultStrftime = new Strftime(DefaultLocale, 0, false);

  		  // CommonJS / Node module
  		  {
  		    module.exports = defaultStrftime;
  		  }

  		  // Polyfill Date.now for old browsers.
  		  if (typeof Date.now !== 'function') {
  		    Date.now = function () {
  		      return +new Date();
  		    };
  		  }
  		  function Strftime(locale, customTimezoneOffset, useUtcTimezone) {
  		    var _locale = locale || DefaultLocale,
  		      _customTimezoneOffset = customTimezoneOffset || 0,
  		      _useUtcBasedDate = useUtcTimezone || false,
  		      // we store unix timestamp value here to not create new Date() each iteration (each millisecond)
  		      // Date.now() is 2 times faster than new Date()
  		      // while millisecond precise is enough here
  		      // this could be very helpful when strftime triggered a lot of times one by one
  		      _cachedDateTimestamp = 0,
  		      _cachedDate;
  		    function _strftime(format, date) {
  		      var timestamp;
  		      if (!date) {
  		        var currentTimestamp = Date.now();
  		        if (currentTimestamp > _cachedDateTimestamp) {
  		          _cachedDateTimestamp = currentTimestamp;
  		          _cachedDate = new Date(_cachedDateTimestamp);
  		          timestamp = _cachedDateTimestamp;
  		          if (_useUtcBasedDate) {
  		            // how to avoid duplication of date instantiation for utc here?
  		            // we tied to getTimezoneOffset of the current date
  		            _cachedDate = new Date(_cachedDateTimestamp + getTimestampToUtcOffsetFor(_cachedDate) + _customTimezoneOffset);
  		          }
  		        } else {
  		          timestamp = _cachedDateTimestamp;
  		        }
  		        date = _cachedDate;
  		      } else {
  		        timestamp = date.getTime();
  		        if (_useUtcBasedDate) {
  		          var utcOffset = getTimestampToUtcOffsetFor(date);
  		          date = new Date(timestamp + utcOffset + _customTimezoneOffset);
  		          // If we've crossed a DST boundary with this calculation we need to
  		          // adjust the new date accordingly or it will be off by an hour in UTC.
  		          if (getTimestampToUtcOffsetFor(date) !== utcOffset) {
  		            var newUTCOffset = getTimestampToUtcOffsetFor(date);
  		            date = new Date(timestamp + newUTCOffset + _customTimezoneOffset);
  		          }
  		        }
  		      }
  		      return _processFormat(format, date, _locale, timestamp);
  		    }
  		    function _processFormat(format, date, locale, timestamp) {
  		      var resultString = '',
  		        padding = null,
  		        isInScope = false,
  		        length = format.length,
  		        extendedTZ = false;
  		      for (var i = 0; i < length; i++) {
  		        var currentCharCode = format.charCodeAt(i);
  		        if (isInScope === true) {
  		          // '-'
  		          if (currentCharCode === 45) {
  		            padding = '';
  		            continue;
  		          }
  		          // '_'
  		          else if (currentCharCode === 95) {
  		            padding = ' ';
  		            continue;
  		          }
  		          // '0'
  		          else if (currentCharCode === 48) {
  		            padding = '0';
  		            continue;
  		          }
  		          // ':'
  		          else if (currentCharCode === 58) {
  		            if (extendedTZ) {
  		              warn("[WARNING] detected use of unsupported %:: or %::: modifiers to strftime");
  		            }
  		            extendedTZ = true;
  		            continue;
  		          }
  		          switch (currentCharCode) {
  		            // Examples for new Date(0) in GMT

  		            // '%'
  		            // case '%':
  		            case 37:
  		              resultString += '%';
  		              break;

  		            // 'Thursday'
  		            // case 'A':
  		            case 65:
  		              resultString += locale.days[date.getDay()];
  		              break;

  		            // 'January'
  		            // case 'B':
  		            case 66:
  		              resultString += locale.months[date.getMonth()];
  		              break;

  		            // '19'
  		            // case 'C':
  		            case 67:
  		              resultString += padTill2(Math.floor(date.getFullYear() / 100), padding);
  		              break;

  		            // '01/01/70'
  		            // case 'D':
  		            case 68:
  		              resultString += _processFormat(locale.formats.D, date, locale, timestamp);
  		              break;

  		            // '1970-01-01'
  		            // case 'F':
  		            case 70:
  		              resultString += _processFormat(locale.formats.F, date, locale, timestamp);
  		              break;

  		            // '00'
  		            // case 'H':
  		            case 72:
  		              resultString += padTill2(date.getHours(), padding);
  		              break;

  		            // '12'
  		            // case 'I':
  		            case 73:
  		              resultString += padTill2(hours12(date.getHours()), padding);
  		              break;

  		            // '000'
  		            // case 'L':
  		            case 76:
  		              resultString += padTill3(Math.floor(timestamp % 1000));
  		              break;

  		            // '00'
  		            // case 'M':
  		            case 77:
  		              resultString += padTill2(date.getMinutes(), padding);
  		              break;

  		            // 'am'
  		            // case 'P':
  		            case 80:
  		              resultString += date.getHours() < 12 ? locale.am : locale.pm;
  		              break;

  		            // '00:00'
  		            // case 'R':
  		            case 82:
  		              resultString += _processFormat(locale.formats.R, date, locale, timestamp);
  		              break;

  		            // '00'
  		            // case 'S':
  		            case 83:
  		              resultString += padTill2(date.getSeconds(), padding);
  		              break;

  		            // '00:00:00'
  		            // case 'T':
  		            case 84:
  		              resultString += _processFormat(locale.formats.T, date, locale, timestamp);
  		              break;

  		            // '00'
  		            // case 'U':
  		            case 85:
  		              resultString += padTill2(weekNumber(date, 'sunday'), padding);
  		              break;

  		            // '00'
  		            // case 'W':
  		            case 87:
  		              resultString += padTill2(weekNumber(date, 'monday'), padding);
  		              break;

  		            // '16:00:00'
  		            // case 'X':
  		            case 88:
  		              resultString += _processFormat(locale.formats.X, date, locale, timestamp);
  		              break;

  		            // '1970'
  		            // case 'Y':
  		            case 89:
  		              resultString += date.getFullYear();
  		              break;

  		            // 'GMT'
  		            // case 'Z':
  		            case 90:
  		              if (_useUtcBasedDate && _customTimezoneOffset === 0) {
  		                resultString += "GMT";
  		              } else {
  		                // fixme optimize
  		                var tzString = date.toString().match(/\(([\w\s]+)\)/);
  		                resultString += tzString && tzString[1] || '';
  		              }
  		              break;

  		            // 'Thu'
  		            // case 'a':
  		            case 97:
  		              resultString += locale.shortDays[date.getDay()];
  		              break;

  		            // 'Jan'
  		            // case 'b':
  		            case 98:
  		              resultString += locale.shortMonths[date.getMonth()];
  		              break;

  		            // ''
  		            // case 'c':
  		            case 99:
  		              resultString += _processFormat(locale.formats.c, date, locale, timestamp);
  		              break;

  		            // '01'
  		            // case 'd':
  		            case 100:
  		              resultString += padTill2(date.getDate(), padding);
  		              break;

  		            // ' 1'
  		            // case 'e':
  		            case 101:
  		              resultString += padTill2(date.getDate(), padding == null ? ' ' : padding);
  		              break;

  		            // 'Jan'
  		            // case 'h':
  		            case 104:
  		              resultString += locale.shortMonths[date.getMonth()];
  		              break;

  		            // '000'
  		            // case 'j':
  		            case 106:
  		              var y = new Date(date.getFullYear(), 0, 1);
  		              var day = Math.ceil((date.getTime() - y.getTime()) / (1000 * 60 * 60 * 24));
  		              resultString += padTill3(day);
  		              break;

  		            // ' 0'
  		            // case 'k':
  		            case 107:
  		              resultString += padTill2(date.getHours(), padding == null ? ' ' : padding);
  		              break;

  		            // '12'
  		            // case 'l':
  		            case 108:
  		              resultString += padTill2(hours12(date.getHours()), padding == null ? ' ' : padding);
  		              break;

  		            // '01'
  		            // case 'm':
  		            case 109:
  		              resultString += padTill2(date.getMonth() + 1, padding);
  		              break;

  		            // '\n'
  		            // case 'n':
  		            case 110:
  		              resultString += '\n';
  		              break;

  		            // '1st'
  		            // case 'o':
  		            case 111:
  		              // Try to use an ordinal suffix from the locale, but fall back to using the old
  		              // function for compatibility with old locales that lack them.
  		              var day = date.getDate();
  		              if (locale.ordinalSuffixes) {
  		                resultString += String(day) + (locale.ordinalSuffixes[day - 1] || ordinal(day));
  		              } else {
  		                resultString += String(day) + ordinal(day);
  		              }
  		              break;

  		            // 'AM'
  		            // case 'p':
  		            case 112:
  		              resultString += date.getHours() < 12 ? locale.AM : locale.PM;
  		              break;

  		            // '12:00:00 AM'
  		            // case 'r':
  		            case 114:
  		              resultString += _processFormat(locale.formats.r, date, locale, timestamp);
  		              break;

  		            // '0'
  		            // case 's':
  		            case 115:
  		              resultString += Math.floor(timestamp / 1000);
  		              break;

  		            // '\t'
  		            // case 't':
  		            case 116:
  		              resultString += '\t';
  		              break;

  		            // '4'
  		            // case 'u':
  		            case 117:
  		              var day = date.getDay();
  		              resultString += day === 0 ? 7 : day;
  		              break;
  		            // 1 - 7, Monday is first day of the week

  		            // ' 1-Jan-1970'
  		            // case 'v':
  		            case 118:
  		              resultString += _processFormat(locale.formats.v, date, locale, timestamp);
  		              break;

  		            // '4'
  		            // case 'w':
  		            case 119:
  		              resultString += date.getDay();
  		              break;
  		            // 0 - 6, Sunday is first day of the week

  		            // '12/31/69'
  		            // case 'x':
  		            case 120:
  		              resultString += _processFormat(locale.formats.x, date, locale, timestamp);
  		              break;

  		            // '70'
  		            // case 'y':
  		            case 121:
  		              resultString += ('' + date.getFullYear()).slice(2);
  		              break;

  		            // '+0000'
  		            // case 'z':
  		            case 122:
  		              if (_useUtcBasedDate && _customTimezoneOffset === 0) {
  		                resultString += extendedTZ ? "+00:00" : "+0000";
  		              } else {
  		                var off;
  		                if (_customTimezoneOffset !== 0) {
  		                  off = _customTimezoneOffset / (60 * 1000);
  		                } else {
  		                  off = -date.getTimezoneOffset();
  		                }
  		                var sign = off < 0 ? '-' : '+';
  		                var sep = extendedTZ ? ':' : '';
  		                var hours = Math.floor(Math.abs(off / 60));
  		                var mins = Math.abs(off % 60);
  		                resultString += sign + padTill2(hours) + sep + padTill2(mins);
  		              }
  		              break;
  		            default:
  		              if (isInScope) {
  		                resultString += '%';
  		              }
  		              resultString += format[i];
  		              break;
  		          }
  		          padding = null;
  		          isInScope = false;
  		          continue;
  		        }

  		        // '%'
  		        if (currentCharCode === 37) {
  		          isInScope = true;
  		          continue;
  		        }
  		        resultString += format[i];
  		      }
  		      return resultString;
  		    }
  		    var strftime = _strftime;
  		    strftime.localize = function (locale) {
  		      return new Strftime(locale || _locale, _customTimezoneOffset, _useUtcBasedDate);
  		    };
  		    strftime.localizeByIdentifier = function (localeIdentifier) {
  		      var locale = Locales[localeIdentifier];
  		      if (!locale) {
  		        warn('[WARNING] No locale found with identifier "' + localeIdentifier + '".');
  		        return strftime;
  		      }
  		      return strftime.localize(locale);
  		    };
  		    strftime.timezone = function (timezone) {
  		      var customTimezoneOffset = _customTimezoneOffset;
  		      var useUtcBasedDate = _useUtcBasedDate;
  		      var timezoneType = typeof timezone;
  		      if (timezoneType === 'number' || timezoneType === 'string') {
  		        useUtcBasedDate = true;

  		        // ISO 8601 format timezone string, [-+]HHMM
  		        if (timezoneType === 'string') {
  		          var sign = timezone[0] === '-' ? -1 : 1,
  		            hours = parseInt(timezone.slice(1, 3), 10),
  		            minutes = parseInt(timezone.slice(3, 5), 10);
  		          customTimezoneOffset = sign * (60 * hours + minutes) * 60 * 1000;
  		          // in minutes: 420
  		        } else if (timezoneType === 'number') {
  		          customTimezoneOffset = timezone * 60 * 1000;
  		        }
  		      }
  		      return new Strftime(_locale, customTimezoneOffset, useUtcBasedDate);
  		    };
  		    strftime.utc = function () {
  		      return new Strftime(_locale, _customTimezoneOffset, true);
  		    };
  		    return strftime;
  		  }
  		  function padTill2(numberToPad, paddingChar) {
  		    if (paddingChar === '' || numberToPad > 9) {
  		      return numberToPad;
  		    }
  		    if (paddingChar == null) {
  		      paddingChar = '0';
  		    }
  		    return paddingChar + numberToPad;
  		  }
  		  function padTill3(numberToPad) {
  		    if (numberToPad > 99) {
  		      return numberToPad;
  		    }
  		    if (numberToPad > 9) {
  		      return '0' + numberToPad;
  		    }
  		    return '00' + numberToPad;
  		  }
  		  function hours12(hour) {
  		    if (hour === 0) {
  		      return 12;
  		    } else if (hour > 12) {
  		      return hour - 12;
  		    }
  		    return hour;
  		  }

  		  // firstWeekday: 'sunday' or 'monday', default is 'sunday'
  		  //
  		  // Pilfered & ported from Ruby's strftime implementation.
  		  function weekNumber(date, firstWeekday) {
  		    firstWeekday = firstWeekday || 'sunday';

  		    // This works by shifting the weekday back by one day if we
  		    // are treating Monday as the first day of the week.
  		    var weekday = date.getDay();
  		    if (firstWeekday === 'monday') {
  		      if (weekday === 0)
  		        // Sunday
  		        weekday = 6;else weekday--;
  		    }
  		    var firstDayOfYearUtc = Date.UTC(date.getFullYear(), 0, 1),
  		      dateUtc = Date.UTC(date.getFullYear(), date.getMonth(), date.getDate()),
  		      yday = Math.floor((dateUtc - firstDayOfYearUtc) / 86400000),
  		      weekNum = (yday + 7 - weekday) / 7;
  		    return Math.floor(weekNum);
  		  }

  		  // Get the ordinal suffix for a number: st, nd, rd, or th
  		  function ordinal(number) {
  		    var i = number % 10;
  		    var ii = number % 100;
  		    if (ii >= 11 && ii <= 13 || i === 0 || i >= 4) {
  		      return 'th';
  		    }
  		    switch (i) {
  		      case 1:
  		        return 'st';
  		      case 2:
  		        return 'nd';
  		      case 3:
  		        return 'rd';
  		    }
  		  }
  		  function getTimestampToUtcOffsetFor(date) {
  		    return (date.getTimezoneOffset() || 0) * 60000;
  		  }
  		  function warn(message) {
  		    if (typeof console !== 'undefined' && typeof console.warn == 'function') {
  		      console.warn(message);
  		    }
  		  }
  		})(); 
  	} (strftime));
  	return strftime.exports;
  }

  requireStrftime();

  var momentWithLocales$1 = {exports: {}};

  var momentWithLocales = momentWithLocales$1.exports;

  var hasRequiredMomentWithLocales;

  function requireMomentWithLocales () {
  	if (hasRequiredMomentWithLocales) return momentWithLocales$1.exports;
  	hasRequiredMomentWithLocales = 1;
  	(function (module, exports) {
  		(function (global, factory) {
  		  module.exports = factory() ;
  		})(momentWithLocales, function () {

  		  var hookCallback;
  		  function hooks() {
  		    return hookCallback.apply(null, arguments);
  		  }

  		  // This is done to register the method called with moment()
  		  // without creating circular dependencies.
  		  function setHookCallback(callback) {
  		    hookCallback = callback;
  		  }
  		  function isArray(input) {
  		    return input instanceof Array || Object.prototype.toString.call(input) === '[object Array]';
  		  }
  		  function isObject(input) {
  		    // IE8 will treat undefined and null as object if it wasn't for
  		    // input != null
  		    return input != null && Object.prototype.toString.call(input) === '[object Object]';
  		  }
  		  function hasOwnProp(a, b) {
  		    return Object.prototype.hasOwnProperty.call(a, b);
  		  }
  		  function isObjectEmpty(obj) {
  		    if (Object.getOwnPropertyNames) {
  		      return Object.getOwnPropertyNames(obj).length === 0;
  		    } else {
  		      var k;
  		      for (k in obj) {
  		        if (hasOwnProp(obj, k)) {
  		          return false;
  		        }
  		      }
  		      return true;
  		    }
  		  }
  		  function isUndefined(input) {
  		    return input === void 0;
  		  }
  		  function isNumber(input) {
  		    return typeof input === 'number' || Object.prototype.toString.call(input) === '[object Number]';
  		  }
  		  function isDate(input) {
  		    return input instanceof Date || Object.prototype.toString.call(input) === '[object Date]';
  		  }
  		  function map(arr, fn) {
  		    var res = [],
  		      i,
  		      arrLen = arr.length;
  		    for (i = 0; i < arrLen; ++i) {
  		      res.push(fn(arr[i], i));
  		    }
  		    return res;
  		  }
  		  function extend(a, b) {
  		    for (var i in b) {
  		      if (hasOwnProp(b, i)) {
  		        a[i] = b[i];
  		      }
  		    }
  		    if (hasOwnProp(b, 'toString')) {
  		      a.toString = b.toString;
  		    }
  		    if (hasOwnProp(b, 'valueOf')) {
  		      a.valueOf = b.valueOf;
  		    }
  		    return a;
  		  }
  		  function createUTC(input, format, locale, strict) {
  		    return createLocalOrUTC(input, format, locale, strict, true).utc();
  		  }
  		  function defaultParsingFlags() {
  		    // We need to deep clone this object.
  		    return {
  		      empty: false,
  		      unusedTokens: [],
  		      unusedInput: [],
  		      overflow: -2,
  		      charsLeftOver: 0,
  		      nullInput: false,
  		      invalidEra: null,
  		      invalidMonth: null,
  		      invalidFormat: false,
  		      userInvalidated: false,
  		      iso: false,
  		      parsedDateParts: [],
  		      era: null,
  		      meridiem: null,
  		      rfc2822: false,
  		      weekdayMismatch: false
  		    };
  		  }
  		  function getParsingFlags(m) {
  		    if (m._pf == null) {
  		      m._pf = defaultParsingFlags();
  		    }
  		    return m._pf;
  		  }
  		  var some;
  		  if (Array.prototype.some) {
  		    some = Array.prototype.some;
  		  } else {
  		    some = function (fun) {
  		      var t = Object(this),
  		        len = t.length >>> 0,
  		        i;
  		      for (i = 0; i < len; i++) {
  		        if (i in t && fun.call(this, t[i], i, t)) {
  		          return true;
  		        }
  		      }
  		      return false;
  		    };
  		  }
  		  function isValid(m) {
  		    var flags = null,
  		      parsedParts = false,
  		      isNowValid = m._d && !isNaN(m._d.getTime());
  		    if (isNowValid) {
  		      flags = getParsingFlags(m);
  		      parsedParts = some.call(flags.parsedDateParts, function (i) {
  		        return i != null;
  		      });
  		      isNowValid = flags.overflow < 0 && !flags.empty && !flags.invalidEra && !flags.invalidMonth && !flags.invalidWeekday && !flags.weekdayMismatch && !flags.nullInput && !flags.invalidFormat && !flags.userInvalidated && (!flags.meridiem || flags.meridiem && parsedParts);
  		      if (m._strict) {
  		        isNowValid = isNowValid && flags.charsLeftOver === 0 && flags.unusedTokens.length === 0 && flags.bigHour === undefined;
  		      }
  		    }
  		    if (Object.isFrozen == null || !Object.isFrozen(m)) {
  		      m._isValid = isNowValid;
  		    } else {
  		      return isNowValid;
  		    }
  		    return m._isValid;
  		  }
  		  function createInvalid(flags) {
  		    var m = createUTC(NaN);
  		    if (flags != null) {
  		      extend(getParsingFlags(m), flags);
  		    } else {
  		      getParsingFlags(m).userInvalidated = true;
  		    }
  		    return m;
  		  }

  		  // Plugins that add properties should also add the key here (null value),
  		  // so we can properly clone ourselves.
  		  var momentProperties = hooks.momentProperties = [],
  		    updateInProgress = false;
  		  function copyConfig(to, from) {
  		    var i,
  		      prop,
  		      val,
  		      momentPropertiesLen = momentProperties.length;
  		    if (!isUndefined(from._isAMomentObject)) {
  		      to._isAMomentObject = from._isAMomentObject;
  		    }
  		    if (!isUndefined(from._i)) {
  		      to._i = from._i;
  		    }
  		    if (!isUndefined(from._f)) {
  		      to._f = from._f;
  		    }
  		    if (!isUndefined(from._l)) {
  		      to._l = from._l;
  		    }
  		    if (!isUndefined(from._strict)) {
  		      to._strict = from._strict;
  		    }
  		    if (!isUndefined(from._tzm)) {
  		      to._tzm = from._tzm;
  		    }
  		    if (!isUndefined(from._isUTC)) {
  		      to._isUTC = from._isUTC;
  		    }
  		    if (!isUndefined(from._offset)) {
  		      to._offset = from._offset;
  		    }
  		    if (!isUndefined(from._pf)) {
  		      to._pf = getParsingFlags(from);
  		    }
  		    if (!isUndefined(from._locale)) {
  		      to._locale = from._locale;
  		    }
  		    if (momentPropertiesLen > 0) {
  		      for (i = 0; i < momentPropertiesLen; i++) {
  		        prop = momentProperties[i];
  		        val = from[prop];
  		        if (!isUndefined(val)) {
  		          to[prop] = val;
  		        }
  		      }
  		    }
  		    return to;
  		  }

  		  // Moment prototype object
  		  function Moment(config) {
  		    copyConfig(this, config);
  		    this._d = new Date(config._d != null ? config._d.getTime() : NaN);
  		    if (!this.isValid()) {
  		      this._d = new Date(NaN);
  		    }
  		    // Prevent infinite loop in case updateOffset creates new moment
  		    // objects.
  		    if (updateInProgress === false) {
  		      updateInProgress = true;
  		      hooks.updateOffset(this);
  		      updateInProgress = false;
  		    }
  		  }
  		  function isMoment(obj) {
  		    return obj instanceof Moment || obj != null && obj._isAMomentObject != null;
  		  }
  		  function warn(msg) {
  		    if (hooks.suppressDeprecationWarnings === false && typeof console !== 'undefined' && console.warn) {
  		      console.warn('Deprecation warning: ' + msg);
  		    }
  		  }
  		  function deprecate(msg, fn) {
  		    var firstTime = true;
  		    return extend(function () {
  		      if (hooks.deprecationHandler != null) {
  		        hooks.deprecationHandler(null, msg);
  		      }
  		      if (firstTime) {
  		        var args = [],
  		          arg,
  		          i,
  		          key,
  		          argLen = arguments.length;
  		        for (i = 0; i < argLen; i++) {
  		          arg = '';
  		          if (typeof arguments[i] === 'object') {
  		            arg += '\n[' + i + '] ';
  		            for (key in arguments[0]) {
  		              if (hasOwnProp(arguments[0], key)) {
  		                arg += key + ': ' + arguments[0][key] + ', ';
  		              }
  		            }
  		            arg = arg.slice(0, -2); // Remove trailing comma and space
  		          } else {
  		            arg = arguments[i];
  		          }
  		          args.push(arg);
  		        }
  		        warn(msg + '\nArguments: ' + Array.prototype.slice.call(args).join('') + '\n' + new Error().stack);
  		        firstTime = false;
  		      }
  		      return fn.apply(this, arguments);
  		    }, fn);
  		  }
  		  var deprecations = {};
  		  function deprecateSimple(name, msg) {
  		    if (hooks.deprecationHandler != null) {
  		      hooks.deprecationHandler(name, msg);
  		    }
  		    if (!deprecations[name]) {
  		      warn(msg);
  		      deprecations[name] = true;
  		    }
  		  }
  		  hooks.suppressDeprecationWarnings = false;
  		  hooks.deprecationHandler = null;
  		  function isFunction(input) {
  		    return typeof Function !== 'undefined' && input instanceof Function || Object.prototype.toString.call(input) === '[object Function]';
  		  }
  		  function set(config) {
  		    var prop, i;
  		    for (i in config) {
  		      if (hasOwnProp(config, i)) {
  		        prop = config[i];
  		        if (isFunction(prop)) {
  		          this[i] = prop;
  		        } else {
  		          this['_' + i] = prop;
  		        }
  		      }
  		    }
  		    this._config = config;
  		    // Lenient ordinal parsing accepts just a number in addition to
  		    // number + (possibly) stuff coming from _dayOfMonthOrdinalParse.
  		    // TODO: Remove "ordinalParse" fallback in next major release.
  		    this._dayOfMonthOrdinalParseLenient = new RegExp((this._dayOfMonthOrdinalParse.source || this._ordinalParse.source) + '|' + /\d{1,2}/.source);
  		  }
  		  function mergeConfigs(parentConfig, childConfig) {
  		    var res = extend({}, parentConfig),
  		      prop;
  		    for (prop in childConfig) {
  		      if (hasOwnProp(childConfig, prop)) {
  		        if (isObject(parentConfig[prop]) && isObject(childConfig[prop])) {
  		          res[prop] = {};
  		          extend(res[prop], parentConfig[prop]);
  		          extend(res[prop], childConfig[prop]);
  		        } else if (childConfig[prop] != null) {
  		          res[prop] = childConfig[prop];
  		        } else {
  		          delete res[prop];
  		        }
  		      }
  		    }
  		    for (prop in parentConfig) {
  		      if (hasOwnProp(parentConfig, prop) && !hasOwnProp(childConfig, prop) && isObject(parentConfig[prop])) {
  		        // make sure changes to properties don't modify parent config
  		        res[prop] = extend({}, res[prop]);
  		      }
  		    }
  		    return res;
  		  }
  		  function Locale(config) {
  		    if (config != null) {
  		      this.set(config);
  		    }
  		  }
  		  var keys;
  		  if (Object.keys) {
  		    keys = Object.keys;
  		  } else {
  		    keys = function (obj) {
  		      var i,
  		        res = [];
  		      for (i in obj) {
  		        if (hasOwnProp(obj, i)) {
  		          res.push(i);
  		        }
  		      }
  		      return res;
  		    };
  		  }
  		  var defaultCalendar = {
  		    sameDay: '[Today at] LT',
  		    nextDay: '[Tomorrow at] LT',
  		    nextWeek: 'dddd [at] LT',
  		    lastDay: '[Yesterday at] LT',
  		    lastWeek: '[Last] dddd [at] LT',
  		    sameElse: 'L'
  		  };
  		  function calendar(key, mom, now) {
  		    var output = this._calendar[key] || this._calendar['sameElse'];
  		    return isFunction(output) ? output.call(mom, now) : output;
  		  }
  		  function zeroFill(number, targetLength, forceSign) {
  		    var absNumber = '' + Math.abs(number),
  		      zerosToFill = targetLength - absNumber.length,
  		      sign = number >= 0;
  		    return (sign ? forceSign ? '+' : '' : '-') + Math.pow(10, Math.max(0, zerosToFill)).toString().substr(1) + absNumber;
  		  }
  		  var formattingTokens = /(\[[^\[]*\])|(\\)?([Hh]mm(ss)?|Mo|MM?M?M?|Do|DDDo|DD?D?D?|ddd?d?|do?|w[o|w]?|W[o|W]?|Qo?|N{1,5}|YYYYYY|YYYYY|YYYY|YY|y{2,4}|yo?|gg(ggg?)?|GG(GGG?)?|e|E|a|A|hh?|HH?|kk?|mm?|ss?|S{1,9}|x|X|zz?|ZZ?|.)/g,
  		    localFormattingTokens = /(\[[^\[]*\])|(\\)?(LTS|LT|LL?L?L?|l{1,4})/g,
  		    formatFunctions = {},
  		    formatTokenFunctions = {};

  		  // token:    'M'
  		  // padded:   ['MM', 2]
  		  // ordinal:  'Mo'
  		  // callback: function () { this.month() + 1 }
  		  function addFormatToken(token, padded, ordinal, callback) {
  		    var func = callback;
  		    if (typeof callback === 'string') {
  		      func = function () {
  		        return this[callback]();
  		      };
  		    }
  		    if (token) {
  		      formatTokenFunctions[token] = func;
  		    }
  		    if (padded) {
  		      formatTokenFunctions[padded[0]] = function () {
  		        return zeroFill(func.apply(this, arguments), padded[1], padded[2]);
  		      };
  		    }
  		    if (ordinal) {
  		      formatTokenFunctions[ordinal] = function () {
  		        return this.localeData().ordinal(func.apply(this, arguments), token);
  		      };
  		    }
  		  }
  		  function removeFormattingTokens(input) {
  		    if (input.match(/\[[\s\S]/)) {
  		      return input.replace(/^\[|\]$/g, '');
  		    }
  		    return input.replace(/\\/g, '');
  		  }
  		  function makeFormatFunction(format) {
  		    var array = format.match(formattingTokens),
  		      i,
  		      length;
  		    for (i = 0, length = array.length; i < length; i++) {
  		      if (formatTokenFunctions[array[i]]) {
  		        array[i] = formatTokenFunctions[array[i]];
  		      } else {
  		        array[i] = removeFormattingTokens(array[i]);
  		      }
  		    }
  		    return function (mom) {
  		      var output = '',
  		        i;
  		      for (i = 0; i < length; i++) {
  		        output += isFunction(array[i]) ? array[i].call(mom, format) : array[i];
  		      }
  		      return output;
  		    };
  		  }

  		  // format date using native date object
  		  function formatMoment(m, format) {
  		    if (!m.isValid()) {
  		      return m.localeData().invalidDate();
  		    }
  		    format = expandFormat(format, m.localeData());
  		    formatFunctions[format] = formatFunctions[format] || makeFormatFunction(format);
  		    return formatFunctions[format](m);
  		  }
  		  function expandFormat(format, locale) {
  		    var i = 5;
  		    function replaceLongDateFormatTokens(input) {
  		      return locale.longDateFormat(input) || input;
  		    }
  		    localFormattingTokens.lastIndex = 0;
  		    while (i >= 0 && localFormattingTokens.test(format)) {
  		      format = format.replace(localFormattingTokens, replaceLongDateFormatTokens);
  		      localFormattingTokens.lastIndex = 0;
  		      i -= 1;
  		    }
  		    return format;
  		  }
  		  var defaultLongDateFormat = {
  		    LTS: 'h:mm:ss A',
  		    LT: 'h:mm A',
  		    L: 'MM/DD/YYYY',
  		    LL: 'MMMM D, YYYY',
  		    LLL: 'MMMM D, YYYY h:mm A',
  		    LLLL: 'dddd, MMMM D, YYYY h:mm A'
  		  };
  		  function longDateFormat(key) {
  		    var format = this._longDateFormat[key],
  		      formatUpper = this._longDateFormat[key.toUpperCase()];
  		    if (format || !formatUpper) {
  		      return format;
  		    }
  		    this._longDateFormat[key] = formatUpper.match(formattingTokens).map(function (tok) {
  		      if (tok === 'MMMM' || tok === 'MM' || tok === 'DD' || tok === 'dddd') {
  		        return tok.slice(1);
  		      }
  		      return tok;
  		    }).join('');
  		    return this._longDateFormat[key];
  		  }
  		  var defaultInvalidDate = 'Invalid date';
  		  function invalidDate() {
  		    return this._invalidDate;
  		  }
  		  var defaultOrdinal = '%d',
  		    defaultDayOfMonthOrdinalParse = /\d{1,2}/;
  		  function ordinal(number) {
  		    return this._ordinal.replace('%d', number);
  		  }
  		  var defaultRelativeTime = {
  		    future: 'in %s',
  		    past: '%s ago',
  		    s: 'a few seconds',
  		    ss: '%d seconds',
  		    m: 'a minute',
  		    mm: '%d minutes',
  		    h: 'an hour',
  		    hh: '%d hours',
  		    d: 'a day',
  		    dd: '%d days',
  		    w: 'a week',
  		    ww: '%d weeks',
  		    M: 'a month',
  		    MM: '%d months',
  		    y: 'a year',
  		    yy: '%d years'
  		  };
  		  function relativeTime(number, withoutSuffix, string, isFuture) {
  		    var output = this._relativeTime[string];
  		    return isFunction(output) ? output(number, withoutSuffix, string, isFuture) : output.replace(/%d/i, number);
  		  }
  		  function pastFuture(diff, output) {
  		    var format = this._relativeTime[diff > 0 ? 'future' : 'past'];
  		    return isFunction(format) ? format(output) : format.replace(/%s/i, output);
  		  }
  		  var aliases = {
  		    D: 'date',
  		    dates: 'date',
  		    date: 'date',
  		    d: 'day',
  		    days: 'day',
  		    day: 'day',
  		    e: 'weekday',
  		    weekdays: 'weekday',
  		    weekday: 'weekday',
  		    E: 'isoWeekday',
  		    isoweekdays: 'isoWeekday',
  		    isoweekday: 'isoWeekday',
  		    DDD: 'dayOfYear',
  		    dayofyears: 'dayOfYear',
  		    dayofyear: 'dayOfYear',
  		    h: 'hour',
  		    hours: 'hour',
  		    hour: 'hour',
  		    ms: 'millisecond',
  		    milliseconds: 'millisecond',
  		    millisecond: 'millisecond',
  		    m: 'minute',
  		    minutes: 'minute',
  		    minute: 'minute',
  		    M: 'month',
  		    months: 'month',
  		    month: 'month',
  		    Q: 'quarter',
  		    quarters: 'quarter',
  		    quarter: 'quarter',
  		    s: 'second',
  		    seconds: 'second',
  		    second: 'second',
  		    gg: 'weekYear',
  		    weekyears: 'weekYear',
  		    weekyear: 'weekYear',
  		    GG: 'isoWeekYear',
  		    isoweekyears: 'isoWeekYear',
  		    isoweekyear: 'isoWeekYear',
  		    w: 'week',
  		    weeks: 'week',
  		    week: 'week',
  		    W: 'isoWeek',
  		    isoweeks: 'isoWeek',
  		    isoweek: 'isoWeek',
  		    y: 'year',
  		    years: 'year',
  		    year: 'year'
  		  };
  		  function normalizeUnits(units) {
  		    return typeof units === 'string' ? aliases[units] || aliases[units.toLowerCase()] : undefined;
  		  }
  		  function normalizeObjectUnits(inputObject) {
  		    var normalizedInput = {},
  		      normalizedProp,
  		      prop;
  		    for (prop in inputObject) {
  		      if (hasOwnProp(inputObject, prop)) {
  		        normalizedProp = normalizeUnits(prop);
  		        if (normalizedProp) {
  		          normalizedInput[normalizedProp] = inputObject[prop];
  		        }
  		      }
  		    }
  		    return normalizedInput;
  		  }
  		  var priorities = {
  		    date: 9,
  		    day: 11,
  		    weekday: 11,
  		    isoWeekday: 11,
  		    dayOfYear: 4,
  		    hour: 13,
  		    millisecond: 16,
  		    minute: 14,
  		    month: 8,
  		    quarter: 7,
  		    second: 15,
  		    weekYear: 1,
  		    isoWeekYear: 1,
  		    week: 5,
  		    isoWeek: 5,
  		    year: 1
  		  };
  		  function getPrioritizedUnits(unitsObj) {
  		    var units = [],
  		      u;
  		    for (u in unitsObj) {
  		      if (hasOwnProp(unitsObj, u)) {
  		        units.push({
  		          unit: u,
  		          priority: priorities[u]
  		        });
  		      }
  		    }
  		    units.sort(function (a, b) {
  		      return a.priority - b.priority;
  		    });
  		    return units;
  		  }
  		  var match1 = /\d/,
  		    //       0 - 9
  		    match2 = /\d\d/,
  		    //      00 - 99
  		    match3 = /\d{3}/,
  		    //     000 - 999
  		    match4 = /\d{4}/,
  		    //    0000 - 9999
  		    match6 = /[+-]?\d{6}/,
  		    // -999999 - 999999
  		    match1to2 = /\d\d?/,
  		    //       0 - 99
  		    match3to4 = /\d\d\d\d?/,
  		    //     999 - 9999
  		    match5to6 = /\d\d\d\d\d\d?/,
  		    //   99999 - 999999
  		    match1to3 = /\d{1,3}/,
  		    //       0 - 999
  		    match1to4 = /\d{1,4}/,
  		    //       0 - 9999
  		    match1to6 = /[+-]?\d{1,6}/,
  		    // -999999 - 999999
  		    matchUnsigned = /\d+/,
  		    //       0 - inf
  		    matchSigned = /[+-]?\d+/,
  		    //    -inf - inf
  		    matchOffset = /Z|[+-]\d\d:?\d\d/gi,
  		    // +00:00 -00:00 +0000 -0000 or Z
  		    matchShortOffset = /Z|[+-]\d\d(?::?\d\d)?/gi,
  		    // +00 -00 +00:00 -00:00 +0000 -0000 or Z
  		    matchTimestamp = /[+-]?\d+(\.\d{1,3})?/,
  		    // 123456789 123456789.123
  		    // any word (or two) characters or numbers including two/three word month in arabic.
  		    // includes scottish gaelic two word and hyphenated months
  		    matchWord = /[0-9]{0,256}['a-z\u00A0-\u05FF\u0700-\uD7FF\uF900-\uFDCF\uFDF0-\uFF07\uFF10-\uFFEF]{1,256}|[\u0600-\u06FF\/]{1,256}(\s*?[\u0600-\u06FF]{1,256}){1,2}/i,
  		    match1to2NoLeadingZero = /^[1-9]\d?/,
  		    //         1-99
  		    match1to2HasZero = /^([1-9]\d|\d)/,
  		    //           0-99
  		    regexes;
  		  regexes = {};
  		  function addRegexToken(token, regex, strictRegex) {
  		    regexes[token] = isFunction(regex) ? regex : function (isStrict, localeData) {
  		      return isStrict && strictRegex ? strictRegex : regex;
  		    };
  		  }
  		  function getParseRegexForToken(token, config) {
  		    if (!hasOwnProp(regexes, token)) {
  		      return new RegExp(unescapeFormat(token));
  		    }
  		    return regexes[token](config._strict, config._locale);
  		  }

  		  // Code from http://stackoverflow.com/questions/3561493/is-there-a-regexp-escape-function-in-javascript
  		  function unescapeFormat(s) {
  		    return regexEscape(s.replace('\\', '').replace(/\\(\[)|\\(\])|\[([^\]\[]*)\]|\\(.)/g, function (matched, p1, p2, p3, p4) {
  		      return p1 || p2 || p3 || p4;
  		    }));
  		  }
  		  function regexEscape(s) {
  		    return s.replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&');
  		  }
  		  function absFloor(number) {
  		    if (number < 0) {
  		      // -0 -> 0
  		      return Math.ceil(number) || 0;
  		    } else {
  		      return Math.floor(number);
  		    }
  		  }
  		  function toInt(argumentForCoercion) {
  		    var coercedNumber = +argumentForCoercion,
  		      value = 0;
  		    if (coercedNumber !== 0 && isFinite(coercedNumber)) {
  		      value = absFloor(coercedNumber);
  		    }
  		    return value;
  		  }
  		  var tokens = {};
  		  function addParseToken(token, callback) {
  		    var i,
  		      func = callback,
  		      tokenLen;
  		    if (typeof token === 'string') {
  		      token = [token];
  		    }
  		    if (isNumber(callback)) {
  		      func = function (input, array) {
  		        array[callback] = toInt(input);
  		      };
  		    }
  		    tokenLen = token.length;
  		    for (i = 0; i < tokenLen; i++) {
  		      tokens[token[i]] = func;
  		    }
  		  }
  		  function addWeekParseToken(token, callback) {
  		    addParseToken(token, function (input, array, config, token) {
  		      config._w = config._w || {};
  		      callback(input, config._w, config, token);
  		    });
  		  }
  		  function addTimeToArrayFromToken(token, input, config) {
  		    if (input != null && hasOwnProp(tokens, token)) {
  		      tokens[token](input, config._a, config, token);
  		    }
  		  }
  		  function isLeapYear(year) {
  		    return year % 4 === 0 && year % 100 !== 0 || year % 400 === 0;
  		  }
  		  var YEAR = 0,
  		    MONTH = 1,
  		    DATE = 2,
  		    HOUR = 3,
  		    MINUTE = 4,
  		    SECOND = 5,
  		    MILLISECOND = 6,
  		    WEEK = 7,
  		    WEEKDAY = 8;

  		  // FORMATTING

  		  addFormatToken('Y', 0, 0, function () {
  		    var y = this.year();
  		    return y <= 9999 ? zeroFill(y, 4) : '+' + y;
  		  });
  		  addFormatToken(0, ['YY', 2], 0, function () {
  		    return this.year() % 100;
  		  });
  		  addFormatToken(0, ['YYYY', 4], 0, 'year');
  		  addFormatToken(0, ['YYYYY', 5], 0, 'year');
  		  addFormatToken(0, ['YYYYYY', 6, true], 0, 'year');

  		  // PARSING

  		  addRegexToken('Y', matchSigned);
  		  addRegexToken('YY', match1to2, match2);
  		  addRegexToken('YYYY', match1to4, match4);
  		  addRegexToken('YYYYY', match1to6, match6);
  		  addRegexToken('YYYYYY', match1to6, match6);
  		  addParseToken(['YYYYY', 'YYYYYY'], YEAR);
  		  addParseToken('YYYY', function (input, array) {
  		    array[YEAR] = input.length === 2 ? hooks.parseTwoDigitYear(input) : toInt(input);
  		  });
  		  addParseToken('YY', function (input, array) {
  		    array[YEAR] = hooks.parseTwoDigitYear(input);
  		  });
  		  addParseToken('Y', function (input, array) {
  		    array[YEAR] = parseInt(input, 10);
  		  });

  		  // HELPERS

  		  function daysInYear(year) {
  		    return isLeapYear(year) ? 366 : 365;
  		  }

  		  // HOOKS

  		  hooks.parseTwoDigitYear = function (input) {
  		    return toInt(input) + (toInt(input) > 68 ? 1900 : 2000);
  		  };

  		  // MOMENTS

  		  var getSetYear = makeGetSet('FullYear', true);
  		  function getIsLeapYear() {
  		    return isLeapYear(this.year());
  		  }
  		  function makeGetSet(unit, keepTime) {
  		    return function (value) {
  		      if (value != null) {
  		        set$1(this, unit, value);
  		        hooks.updateOffset(this, keepTime);
  		        return this;
  		      } else {
  		        return get(this, unit);
  		      }
  		    };
  		  }
  		  function get(mom, unit) {
  		    if (!mom.isValid()) {
  		      return NaN;
  		    }
  		    var d = mom._d,
  		      isUTC = mom._isUTC;
  		    switch (unit) {
  		      case 'Milliseconds':
  		        return isUTC ? d.getUTCMilliseconds() : d.getMilliseconds();
  		      case 'Seconds':
  		        return isUTC ? d.getUTCSeconds() : d.getSeconds();
  		      case 'Minutes':
  		        return isUTC ? d.getUTCMinutes() : d.getMinutes();
  		      case 'Hours':
  		        return isUTC ? d.getUTCHours() : d.getHours();
  		      case 'Date':
  		        return isUTC ? d.getUTCDate() : d.getDate();
  		      case 'Day':
  		        return isUTC ? d.getUTCDay() : d.getDay();
  		      case 'Month':
  		        return isUTC ? d.getUTCMonth() : d.getMonth();
  		      case 'FullYear':
  		        return isUTC ? d.getUTCFullYear() : d.getFullYear();
  		      default:
  		        return NaN;
  		      // Just in case
  		    }
  		  }
  		  function set$1(mom, unit, value) {
  		    var d, isUTC, year, month, date;
  		    if (!mom.isValid() || isNaN(value)) {
  		      return;
  		    }
  		    d = mom._d;
  		    isUTC = mom._isUTC;
  		    switch (unit) {
  		      case 'Milliseconds':
  		        return void (isUTC ? d.setUTCMilliseconds(value) : d.setMilliseconds(value));
  		      case 'Seconds':
  		        return void (isUTC ? d.setUTCSeconds(value) : d.setSeconds(value));
  		      case 'Minutes':
  		        return void (isUTC ? d.setUTCMinutes(value) : d.setMinutes(value));
  		      case 'Hours':
  		        return void (isUTC ? d.setUTCHours(value) : d.setHours(value));
  		      case 'Date':
  		        return void (isUTC ? d.setUTCDate(value) : d.setDate(value));
  		      // case 'Day': // Not real
  		      //    return void (isUTC ? d.setUTCDay(value) : d.setDay(value));
  		      // case 'Month': // Not used because we need to pass two variables
  		      //     return void (isUTC ? d.setUTCMonth(value) : d.setMonth(value));
  		      case 'FullYear':
  		        break;
  		      // See below ...
  		      default:
  		        return;
  		      // Just in case
  		    }
  		    year = value;
  		    month = mom.month();
  		    date = mom.date();
  		    date = date === 29 && month === 1 && !isLeapYear(year) ? 28 : date;
  		    void (isUTC ? d.setUTCFullYear(year, month, date) : d.setFullYear(year, month, date));
  		  }

  		  // MOMENTS

  		  function stringGet(units) {
  		    units = normalizeUnits(units);
  		    if (isFunction(this[units])) {
  		      return this[units]();
  		    }
  		    return this;
  		  }
  		  function stringSet(units, value) {
  		    if (typeof units === 'object') {
  		      units = normalizeObjectUnits(units);
  		      var prioritized = getPrioritizedUnits(units),
  		        i,
  		        prioritizedLen = prioritized.length;
  		      for (i = 0; i < prioritizedLen; i++) {
  		        this[prioritized[i].unit](units[prioritized[i].unit]);
  		      }
  		    } else {
  		      units = normalizeUnits(units);
  		      if (isFunction(this[units])) {
  		        return this[units](value);
  		      }
  		    }
  		    return this;
  		  }
  		  function mod(n, x) {
  		    return (n % x + x) % x;
  		  }
  		  var indexOf;
  		  if (Array.prototype.indexOf) {
  		    indexOf = Array.prototype.indexOf;
  		  } else {
  		    indexOf = function (o) {
  		      // I know
  		      var i;
  		      for (i = 0; i < this.length; ++i) {
  		        if (this[i] === o) {
  		          return i;
  		        }
  		      }
  		      return -1;
  		    };
  		  }
  		  function daysInMonth(year, month) {
  		    if (isNaN(year) || isNaN(month)) {
  		      return NaN;
  		    }
  		    var modMonth = mod(month, 12);
  		    year += (month - modMonth) / 12;
  		    return modMonth === 1 ? isLeapYear(year) ? 29 : 28 : 31 - modMonth % 7 % 2;
  		  }

  		  // FORMATTING

  		  addFormatToken('M', ['MM', 2], 'Mo', function () {
  		    return this.month() + 1;
  		  });
  		  addFormatToken('MMM', 0, 0, function (format) {
  		    return this.localeData().monthsShort(this, format);
  		  });
  		  addFormatToken('MMMM', 0, 0, function (format) {
  		    return this.localeData().months(this, format);
  		  });

  		  // PARSING

  		  addRegexToken('M', match1to2, match1to2NoLeadingZero);
  		  addRegexToken('MM', match1to2, match2);
  		  addRegexToken('MMM', function (isStrict, locale) {
  		    return locale.monthsShortRegex(isStrict);
  		  });
  		  addRegexToken('MMMM', function (isStrict, locale) {
  		    return locale.monthsRegex(isStrict);
  		  });
  		  addParseToken(['M', 'MM'], function (input, array) {
  		    array[MONTH] = toInt(input) - 1;
  		  });
  		  addParseToken(['MMM', 'MMMM'], function (input, array, config, token) {
  		    var month = config._locale.monthsParse(input, token, config._strict);
  		    // if we didn't find a month name, mark the date as invalid.
  		    if (month != null) {
  		      array[MONTH] = month;
  		    } else {
  		      getParsingFlags(config).invalidMonth = input;
  		    }
  		  });

  		  // LOCALES

  		  var defaultLocaleMonths = 'January_February_March_April_May_June_July_August_September_October_November_December'.split('_'),
  		    defaultLocaleMonthsShort = 'Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec'.split('_'),
  		    MONTHS_IN_FORMAT = /D[oD]?(\[[^\[\]]*\]|\s)+MMMM?/,
  		    defaultMonthsShortRegex = matchWord,
  		    defaultMonthsRegex = matchWord;
  		  function localeMonths(m, format) {
  		    if (!m) {
  		      return isArray(this._months) ? this._months : this._months['standalone'];
  		    }
  		    return isArray(this._months) ? this._months[m.month()] : this._months[(this._months.isFormat || MONTHS_IN_FORMAT).test(format) ? 'format' : 'standalone'][m.month()];
  		  }
  		  function localeMonthsShort(m, format) {
  		    if (!m) {
  		      return isArray(this._monthsShort) ? this._monthsShort : this._monthsShort['standalone'];
  		    }
  		    return isArray(this._monthsShort) ? this._monthsShort[m.month()] : this._monthsShort[MONTHS_IN_FORMAT.test(format) ? 'format' : 'standalone'][m.month()];
  		  }
  		  function handleStrictParse(monthName, format, strict) {
  		    var i,
  		      ii,
  		      mom,
  		      llc = monthName.toLocaleLowerCase();
  		    if (!this._monthsParse) {
  		      // this is not used
  		      this._monthsParse = [];
  		      this._longMonthsParse = [];
  		      this._shortMonthsParse = [];
  		      for (i = 0; i < 12; ++i) {
  		        mom = createUTC([2000, i]);
  		        this._shortMonthsParse[i] = this.monthsShort(mom, '').toLocaleLowerCase();
  		        this._longMonthsParse[i] = this.months(mom, '').toLocaleLowerCase();
  		      }
  		    }
  		    if (strict) {
  		      if (format === 'MMM') {
  		        ii = indexOf.call(this._shortMonthsParse, llc);
  		        return ii !== -1 ? ii : null;
  		      } else {
  		        ii = indexOf.call(this._longMonthsParse, llc);
  		        return ii !== -1 ? ii : null;
  		      }
  		    } else {
  		      if (format === 'MMM') {
  		        ii = indexOf.call(this._shortMonthsParse, llc);
  		        if (ii !== -1) {
  		          return ii;
  		        }
  		        ii = indexOf.call(this._longMonthsParse, llc);
  		        return ii !== -1 ? ii : null;
  		      } else {
  		        ii = indexOf.call(this._longMonthsParse, llc);
  		        if (ii !== -1) {
  		          return ii;
  		        }
  		        ii = indexOf.call(this._shortMonthsParse, llc);
  		        return ii !== -1 ? ii : null;
  		      }
  		    }
  		  }
  		  function localeMonthsParse(monthName, format, strict) {
  		    var i, mom, regex;
  		    if (this._monthsParseExact) {
  		      return handleStrictParse.call(this, monthName, format, strict);
  		    }
  		    if (!this._monthsParse) {
  		      this._monthsParse = [];
  		      this._longMonthsParse = [];
  		      this._shortMonthsParse = [];
  		    }

  		    // TODO: add sorting
  		    // Sorting makes sure if one month (or abbr) is a prefix of another
  		    // see sorting in computeMonthsParse
  		    for (i = 0; i < 12; i++) {
  		      // make the regex if we don't have it already
  		      mom = createUTC([2000, i]);
  		      if (strict && !this._longMonthsParse[i]) {
  		        this._longMonthsParse[i] = new RegExp('^' + this.months(mom, '').replace('.', '') + '$', 'i');
  		        this._shortMonthsParse[i] = new RegExp('^' + this.monthsShort(mom, '').replace('.', '') + '$', 'i');
  		      }
  		      if (!strict && !this._monthsParse[i]) {
  		        regex = '^' + this.months(mom, '') + '|^' + this.monthsShort(mom, '');
  		        this._monthsParse[i] = new RegExp(regex.replace('.', ''), 'i');
  		      }
  		      // test the regex
  		      if (strict && format === 'MMMM' && this._longMonthsParse[i].test(monthName)) {
  		        return i;
  		      } else if (strict && format === 'MMM' && this._shortMonthsParse[i].test(monthName)) {
  		        return i;
  		      } else if (!strict && this._monthsParse[i].test(monthName)) {
  		        return i;
  		      }
  		    }
  		  }

  		  // MOMENTS

  		  function setMonth(mom, value) {
  		    if (!mom.isValid()) {
  		      // No op
  		      return mom;
  		    }
  		    if (typeof value === 'string') {
  		      if (/^\d+$/.test(value)) {
  		        value = toInt(value);
  		      } else {
  		        value = mom.localeData().monthsParse(value);
  		        // TODO: Another silent failure?
  		        if (!isNumber(value)) {
  		          return mom;
  		        }
  		      }
  		    }
  		    var month = value,
  		      date = mom.date();
  		    date = date < 29 ? date : Math.min(date, daysInMonth(mom.year(), month));
  		    void (mom._isUTC ? mom._d.setUTCMonth(month, date) : mom._d.setMonth(month, date));
  		    return mom;
  		  }
  		  function getSetMonth(value) {
  		    if (value != null) {
  		      setMonth(this, value);
  		      hooks.updateOffset(this, true);
  		      return this;
  		    } else {
  		      return get(this, 'Month');
  		    }
  		  }
  		  function getDaysInMonth() {
  		    return daysInMonth(this.year(), this.month());
  		  }
  		  function monthsShortRegex(isStrict) {
  		    if (this._monthsParseExact) {
  		      if (!hasOwnProp(this, '_monthsRegex')) {
  		        computeMonthsParse.call(this);
  		      }
  		      if (isStrict) {
  		        return this._monthsShortStrictRegex;
  		      } else {
  		        return this._monthsShortRegex;
  		      }
  		    } else {
  		      if (!hasOwnProp(this, '_monthsShortRegex')) {
  		        this._monthsShortRegex = defaultMonthsShortRegex;
  		      }
  		      return this._monthsShortStrictRegex && isStrict ? this._monthsShortStrictRegex : this._monthsShortRegex;
  		    }
  		  }
  		  function monthsRegex(isStrict) {
  		    if (this._monthsParseExact) {
  		      if (!hasOwnProp(this, '_monthsRegex')) {
  		        computeMonthsParse.call(this);
  		      }
  		      if (isStrict) {
  		        return this._monthsStrictRegex;
  		      } else {
  		        return this._monthsRegex;
  		      }
  		    } else {
  		      if (!hasOwnProp(this, '_monthsRegex')) {
  		        this._monthsRegex = defaultMonthsRegex;
  		      }
  		      return this._monthsStrictRegex && isStrict ? this._monthsStrictRegex : this._monthsRegex;
  		    }
  		  }
  		  function computeMonthsParse() {
  		    function cmpLenRev(a, b) {
  		      return b.length - a.length;
  		    }
  		    var shortPieces = [],
  		      longPieces = [],
  		      mixedPieces = [],
  		      i,
  		      mom,
  		      shortP,
  		      longP;
  		    for (i = 0; i < 12; i++) {
  		      // make the regex if we don't have it already
  		      mom = createUTC([2000, i]);
  		      shortP = regexEscape(this.monthsShort(mom, ''));
  		      longP = regexEscape(this.months(mom, ''));
  		      shortPieces.push(shortP);
  		      longPieces.push(longP);
  		      mixedPieces.push(longP);
  		      mixedPieces.push(shortP);
  		    }
  		    // Sorting makes sure if one month (or abbr) is a prefix of another it
  		    // will match the longer piece.
  		    shortPieces.sort(cmpLenRev);
  		    longPieces.sort(cmpLenRev);
  		    mixedPieces.sort(cmpLenRev);
  		    this._monthsRegex = new RegExp('^(' + mixedPieces.join('|') + ')', 'i');
  		    this._monthsShortRegex = this._monthsRegex;
  		    this._monthsStrictRegex = new RegExp('^(' + longPieces.join('|') + ')', 'i');
  		    this._monthsShortStrictRegex = new RegExp('^(' + shortPieces.join('|') + ')', 'i');
  		  }
  		  function createDate(y, m, d, h, M, s, ms) {
  		    // can't just apply() to create a date:
  		    // https://stackoverflow.com/q/181348
  		    var date;
  		    // the date constructor remaps years 0-99 to 1900-1999
  		    if (y < 100 && y >= 0) {
  		      // preserve leap years using a full 400 year cycle, then reset
  		      date = new Date(y + 400, m, d, h, M, s, ms);
  		      if (isFinite(date.getFullYear())) {
  		        date.setFullYear(y);
  		      }
  		    } else {
  		      date = new Date(y, m, d, h, M, s, ms);
  		    }
  		    return date;
  		  }
  		  function createUTCDate(y) {
  		    var date, args;
  		    // the Date.UTC function remaps years 0-99 to 1900-1999
  		    if (y < 100 && y >= 0) {
  		      args = Array.prototype.slice.call(arguments);
  		      // preserve leap years using a full 400 year cycle, then reset
  		      args[0] = y + 400;
  		      date = new Date(Date.UTC.apply(null, args));
  		      if (isFinite(date.getUTCFullYear())) {
  		        date.setUTCFullYear(y);
  		      }
  		    } else {
  		      date = new Date(Date.UTC.apply(null, arguments));
  		    }
  		    return date;
  		  }

  		  // start-of-first-week - start-of-year
  		  function firstWeekOffset(year, dow, doy) {
  		    var
  		      // first-week day -- which january is always in the first week (4 for iso, 1 for other)
  		      fwd = 7 + dow - doy,
  		      // first-week day local weekday -- which local weekday is fwd
  		      fwdlw = (7 + createUTCDate(year, 0, fwd).getUTCDay() - dow) % 7;
  		    return -fwdlw + fwd - 1;
  		  }

  		  // https://en.wikipedia.org/wiki/ISO_week_date#Calculating_a_date_given_the_year.2C_week_number_and_weekday
  		  function dayOfYearFromWeeks(year, week, weekday, dow, doy) {
  		    var localWeekday = (7 + weekday - dow) % 7,
  		      weekOffset = firstWeekOffset(year, dow, doy),
  		      dayOfYear = 1 + 7 * (week - 1) + localWeekday + weekOffset,
  		      resYear,
  		      resDayOfYear;
  		    if (dayOfYear <= 0) {
  		      resYear = year - 1;
  		      resDayOfYear = daysInYear(resYear) + dayOfYear;
  		    } else if (dayOfYear > daysInYear(year)) {
  		      resYear = year + 1;
  		      resDayOfYear = dayOfYear - daysInYear(year);
  		    } else {
  		      resYear = year;
  		      resDayOfYear = dayOfYear;
  		    }
  		    return {
  		      year: resYear,
  		      dayOfYear: resDayOfYear
  		    };
  		  }
  		  function weekOfYear(mom, dow, doy) {
  		    var weekOffset = firstWeekOffset(mom.year(), dow, doy),
  		      week = Math.floor((mom.dayOfYear() - weekOffset - 1) / 7) + 1,
  		      resWeek,
  		      resYear;
  		    if (week < 1) {
  		      resYear = mom.year() - 1;
  		      resWeek = week + weeksInYear(resYear, dow, doy);
  		    } else if (week > weeksInYear(mom.year(), dow, doy)) {
  		      resWeek = week - weeksInYear(mom.year(), dow, doy);
  		      resYear = mom.year() + 1;
  		    } else {
  		      resYear = mom.year();
  		      resWeek = week;
  		    }
  		    return {
  		      week: resWeek,
  		      year: resYear
  		    };
  		  }
  		  function weeksInYear(year, dow, doy) {
  		    var weekOffset = firstWeekOffset(year, dow, doy),
  		      weekOffsetNext = firstWeekOffset(year + 1, dow, doy);
  		    return (daysInYear(year) - weekOffset + weekOffsetNext) / 7;
  		  }

  		  // FORMATTING

  		  addFormatToken('w', ['ww', 2], 'wo', 'week');
  		  addFormatToken('W', ['WW', 2], 'Wo', 'isoWeek');

  		  // PARSING

  		  addRegexToken('w', match1to2, match1to2NoLeadingZero);
  		  addRegexToken('ww', match1to2, match2);
  		  addRegexToken('W', match1to2, match1to2NoLeadingZero);
  		  addRegexToken('WW', match1to2, match2);
  		  addWeekParseToken(['w', 'ww', 'W', 'WW'], function (input, week, config, token) {
  		    week[token.substr(0, 1)] = toInt(input);
  		  });

  		  // HELPERS

  		  // LOCALES

  		  function localeWeek(mom) {
  		    return weekOfYear(mom, this._week.dow, this._week.doy).week;
  		  }
  		  var defaultLocaleWeek = {
  		    dow: 0,
  		    // Sunday is the first day of the week.
  		    doy: 6 // The week that contains Jan 6th is the first week of the year.
  		  };
  		  function localeFirstDayOfWeek() {
  		    return this._week.dow;
  		  }
  		  function localeFirstDayOfYear() {
  		    return this._week.doy;
  		  }

  		  // MOMENTS

  		  function getSetWeek(input) {
  		    var week = this.localeData().week(this);
  		    return input == null ? week : this.add((input - week) * 7, 'd');
  		  }
  		  function getSetISOWeek(input) {
  		    var week = weekOfYear(this, 1, 4).week;
  		    return input == null ? week : this.add((input - week) * 7, 'd');
  		  }

  		  // FORMATTING

  		  addFormatToken('d', 0, 'do', 'day');
  		  addFormatToken('dd', 0, 0, function (format) {
  		    return this.localeData().weekdaysMin(this, format);
  		  });
  		  addFormatToken('ddd', 0, 0, function (format) {
  		    return this.localeData().weekdaysShort(this, format);
  		  });
  		  addFormatToken('dddd', 0, 0, function (format) {
  		    return this.localeData().weekdays(this, format);
  		  });
  		  addFormatToken('e', 0, 0, 'weekday');
  		  addFormatToken('E', 0, 0, 'isoWeekday');

  		  // PARSING

  		  addRegexToken('d', match1to2);
  		  addRegexToken('e', match1to2);
  		  addRegexToken('E', match1to2);
  		  addRegexToken('dd', function (isStrict, locale) {
  		    return locale.weekdaysMinRegex(isStrict);
  		  });
  		  addRegexToken('ddd', function (isStrict, locale) {
  		    return locale.weekdaysShortRegex(isStrict);
  		  });
  		  addRegexToken('dddd', function (isStrict, locale) {
  		    return locale.weekdaysRegex(isStrict);
  		  });
  		  addWeekParseToken(['dd', 'ddd', 'dddd'], function (input, week, config, token) {
  		    var weekday = config._locale.weekdaysParse(input, token, config._strict);
  		    // if we didn't get a weekday name, mark the date as invalid
  		    if (weekday != null) {
  		      week.d = weekday;
  		    } else {
  		      getParsingFlags(config).invalidWeekday = input;
  		    }
  		  });
  		  addWeekParseToken(['d', 'e', 'E'], function (input, week, config, token) {
  		    week[token] = toInt(input);
  		  });

  		  // HELPERS

  		  function parseWeekday(input, locale) {
  		    if (typeof input !== 'string') {
  		      return input;
  		    }
  		    if (!isNaN(input)) {
  		      return parseInt(input, 10);
  		    }
  		    input = locale.weekdaysParse(input);
  		    if (typeof input === 'number') {
  		      return input;
  		    }
  		    return null;
  		  }
  		  function parseIsoWeekday(input, locale) {
  		    if (typeof input === 'string') {
  		      return locale.weekdaysParse(input) % 7 || 7;
  		    }
  		    return isNaN(input) ? null : input;
  		  }

  		  // LOCALES
  		  function shiftWeekdays(ws, n) {
  		    return ws.slice(n, 7).concat(ws.slice(0, n));
  		  }
  		  var defaultLocaleWeekdays = 'Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday'.split('_'),
  		    defaultLocaleWeekdaysShort = 'Sun_Mon_Tue_Wed_Thu_Fri_Sat'.split('_'),
  		    defaultLocaleWeekdaysMin = 'Su_Mo_Tu_We_Th_Fr_Sa'.split('_'),
  		    defaultWeekdaysRegex = matchWord,
  		    defaultWeekdaysShortRegex = matchWord,
  		    defaultWeekdaysMinRegex = matchWord;
  		  function localeWeekdays(m, format) {
  		    var weekdays = isArray(this._weekdays) ? this._weekdays : this._weekdays[m && m !== true && this._weekdays.isFormat.test(format) ? 'format' : 'standalone'];
  		    return m === true ? shiftWeekdays(weekdays, this._week.dow) : m ? weekdays[m.day()] : weekdays;
  		  }
  		  function localeWeekdaysShort(m) {
  		    return m === true ? shiftWeekdays(this._weekdaysShort, this._week.dow) : m ? this._weekdaysShort[m.day()] : this._weekdaysShort;
  		  }
  		  function localeWeekdaysMin(m) {
  		    return m === true ? shiftWeekdays(this._weekdaysMin, this._week.dow) : m ? this._weekdaysMin[m.day()] : this._weekdaysMin;
  		  }
  		  function handleStrictParse$1(weekdayName, format, strict) {
  		    var i,
  		      ii,
  		      mom,
  		      llc = weekdayName.toLocaleLowerCase();
  		    if (!this._weekdaysParse) {
  		      this._weekdaysParse = [];
  		      this._shortWeekdaysParse = [];
  		      this._minWeekdaysParse = [];
  		      for (i = 0; i < 7; ++i) {
  		        mom = createUTC([2000, 1]).day(i);
  		        this._minWeekdaysParse[i] = this.weekdaysMin(mom, '').toLocaleLowerCase();
  		        this._shortWeekdaysParse[i] = this.weekdaysShort(mom, '').toLocaleLowerCase();
  		        this._weekdaysParse[i] = this.weekdays(mom, '').toLocaleLowerCase();
  		      }
  		    }
  		    if (strict) {
  		      if (format === 'dddd') {
  		        ii = indexOf.call(this._weekdaysParse, llc);
  		        return ii !== -1 ? ii : null;
  		      } else if (format === 'ddd') {
  		        ii = indexOf.call(this._shortWeekdaysParse, llc);
  		        return ii !== -1 ? ii : null;
  		      } else {
  		        ii = indexOf.call(this._minWeekdaysParse, llc);
  		        return ii !== -1 ? ii : null;
  		      }
  		    } else {
  		      if (format === 'dddd') {
  		        ii = indexOf.call(this._weekdaysParse, llc);
  		        if (ii !== -1) {
  		          return ii;
  		        }
  		        ii = indexOf.call(this._shortWeekdaysParse, llc);
  		        if (ii !== -1) {
  		          return ii;
  		        }
  		        ii = indexOf.call(this._minWeekdaysParse, llc);
  		        return ii !== -1 ? ii : null;
  		      } else if (format === 'ddd') {
  		        ii = indexOf.call(this._shortWeekdaysParse, llc);
  		        if (ii !== -1) {
  		          return ii;
  		        }
  		        ii = indexOf.call(this._weekdaysParse, llc);
  		        if (ii !== -1) {
  		          return ii;
  		        }
  		        ii = indexOf.call(this._minWeekdaysParse, llc);
  		        return ii !== -1 ? ii : null;
  		      } else {
  		        ii = indexOf.call(this._minWeekdaysParse, llc);
  		        if (ii !== -1) {
  		          return ii;
  		        }
  		        ii = indexOf.call(this._weekdaysParse, llc);
  		        if (ii !== -1) {
  		          return ii;
  		        }
  		        ii = indexOf.call(this._shortWeekdaysParse, llc);
  		        return ii !== -1 ? ii : null;
  		      }
  		    }
  		  }
  		  function localeWeekdaysParse(weekdayName, format, strict) {
  		    var i, mom, regex;
  		    if (this._weekdaysParseExact) {
  		      return handleStrictParse$1.call(this, weekdayName, format, strict);
  		    }
  		    if (!this._weekdaysParse) {
  		      this._weekdaysParse = [];
  		      this._minWeekdaysParse = [];
  		      this._shortWeekdaysParse = [];
  		      this._fullWeekdaysParse = [];
  		    }
  		    for (i = 0; i < 7; i++) {
  		      // make the regex if we don't have it already

  		      mom = createUTC([2000, 1]).day(i);
  		      if (strict && !this._fullWeekdaysParse[i]) {
  		        this._fullWeekdaysParse[i] = new RegExp('^' + this.weekdays(mom, '').replace('.', '\\.?') + '$', 'i');
  		        this._shortWeekdaysParse[i] = new RegExp('^' + this.weekdaysShort(mom, '').replace('.', '\\.?') + '$', 'i');
  		        this._minWeekdaysParse[i] = new RegExp('^' + this.weekdaysMin(mom, '').replace('.', '\\.?') + '$', 'i');
  		      }
  		      if (!this._weekdaysParse[i]) {
  		        regex = '^' + this.weekdays(mom, '') + '|^' + this.weekdaysShort(mom, '') + '|^' + this.weekdaysMin(mom, '');
  		        this._weekdaysParse[i] = new RegExp(regex.replace('.', ''), 'i');
  		      }
  		      // test the regex
  		      if (strict && format === 'dddd' && this._fullWeekdaysParse[i].test(weekdayName)) {
  		        return i;
  		      } else if (strict && format === 'ddd' && this._shortWeekdaysParse[i].test(weekdayName)) {
  		        return i;
  		      } else if (strict && format === 'dd' && this._minWeekdaysParse[i].test(weekdayName)) {
  		        return i;
  		      } else if (!strict && this._weekdaysParse[i].test(weekdayName)) {
  		        return i;
  		      }
  		    }
  		  }

  		  // MOMENTS

  		  function getSetDayOfWeek(input) {
  		    if (!this.isValid()) {
  		      return input != null ? this : NaN;
  		    }
  		    var day = get(this, 'Day');
  		    if (input != null) {
  		      input = parseWeekday(input, this.localeData());
  		      return this.add(input - day, 'd');
  		    } else {
  		      return day;
  		    }
  		  }
  		  function getSetLocaleDayOfWeek(input) {
  		    if (!this.isValid()) {
  		      return input != null ? this : NaN;
  		    }
  		    var weekday = (this.day() + 7 - this.localeData()._week.dow) % 7;
  		    return input == null ? weekday : this.add(input - weekday, 'd');
  		  }
  		  function getSetISODayOfWeek(input) {
  		    if (!this.isValid()) {
  		      return input != null ? this : NaN;
  		    }

  		    // behaves the same as moment#day except
  		    // as a getter, returns 7 instead of 0 (1-7 range instead of 0-6)
  		    // as a setter, sunday should belong to the previous week.

  		    if (input != null) {
  		      var weekday = parseIsoWeekday(input, this.localeData());
  		      return this.day(this.day() % 7 ? weekday : weekday - 7);
  		    } else {
  		      return this.day() || 7;
  		    }
  		  }
  		  function weekdaysRegex(isStrict) {
  		    if (this._weekdaysParseExact) {
  		      if (!hasOwnProp(this, '_weekdaysRegex')) {
  		        computeWeekdaysParse.call(this);
  		      }
  		      if (isStrict) {
  		        return this._weekdaysStrictRegex;
  		      } else {
  		        return this._weekdaysRegex;
  		      }
  		    } else {
  		      if (!hasOwnProp(this, '_weekdaysRegex')) {
  		        this._weekdaysRegex = defaultWeekdaysRegex;
  		      }
  		      return this._weekdaysStrictRegex && isStrict ? this._weekdaysStrictRegex : this._weekdaysRegex;
  		    }
  		  }
  		  function weekdaysShortRegex(isStrict) {
  		    if (this._weekdaysParseExact) {
  		      if (!hasOwnProp(this, '_weekdaysRegex')) {
  		        computeWeekdaysParse.call(this);
  		      }
  		      if (isStrict) {
  		        return this._weekdaysShortStrictRegex;
  		      } else {
  		        return this._weekdaysShortRegex;
  		      }
  		    } else {
  		      if (!hasOwnProp(this, '_weekdaysShortRegex')) {
  		        this._weekdaysShortRegex = defaultWeekdaysShortRegex;
  		      }
  		      return this._weekdaysShortStrictRegex && isStrict ? this._weekdaysShortStrictRegex : this._weekdaysShortRegex;
  		    }
  		  }
  		  function weekdaysMinRegex(isStrict) {
  		    if (this._weekdaysParseExact) {
  		      if (!hasOwnProp(this, '_weekdaysRegex')) {
  		        computeWeekdaysParse.call(this);
  		      }
  		      if (isStrict) {
  		        return this._weekdaysMinStrictRegex;
  		      } else {
  		        return this._weekdaysMinRegex;
  		      }
  		    } else {
  		      if (!hasOwnProp(this, '_weekdaysMinRegex')) {
  		        this._weekdaysMinRegex = defaultWeekdaysMinRegex;
  		      }
  		      return this._weekdaysMinStrictRegex && isStrict ? this._weekdaysMinStrictRegex : this._weekdaysMinRegex;
  		    }
  		  }
  		  function computeWeekdaysParse() {
  		    function cmpLenRev(a, b) {
  		      return b.length - a.length;
  		    }
  		    var minPieces = [],
  		      shortPieces = [],
  		      longPieces = [],
  		      mixedPieces = [],
  		      i,
  		      mom,
  		      minp,
  		      shortp,
  		      longp;
  		    for (i = 0; i < 7; i++) {
  		      // make the regex if we don't have it already
  		      mom = createUTC([2000, 1]).day(i);
  		      minp = regexEscape(this.weekdaysMin(mom, ''));
  		      shortp = regexEscape(this.weekdaysShort(mom, ''));
  		      longp = regexEscape(this.weekdays(mom, ''));
  		      minPieces.push(minp);
  		      shortPieces.push(shortp);
  		      longPieces.push(longp);
  		      mixedPieces.push(minp);
  		      mixedPieces.push(shortp);
  		      mixedPieces.push(longp);
  		    }
  		    // Sorting makes sure if one weekday (or abbr) is a prefix of another it
  		    // will match the longer piece.
  		    minPieces.sort(cmpLenRev);
  		    shortPieces.sort(cmpLenRev);
  		    longPieces.sort(cmpLenRev);
  		    mixedPieces.sort(cmpLenRev);
  		    this._weekdaysRegex = new RegExp('^(' + mixedPieces.join('|') + ')', 'i');
  		    this._weekdaysShortRegex = this._weekdaysRegex;
  		    this._weekdaysMinRegex = this._weekdaysRegex;
  		    this._weekdaysStrictRegex = new RegExp('^(' + longPieces.join('|') + ')', 'i');
  		    this._weekdaysShortStrictRegex = new RegExp('^(' + shortPieces.join('|') + ')', 'i');
  		    this._weekdaysMinStrictRegex = new RegExp('^(' + minPieces.join('|') + ')', 'i');
  		  }

  		  // FORMATTING

  		  function hFormat() {
  		    return this.hours() % 12 || 12;
  		  }
  		  function kFormat() {
  		    return this.hours() || 24;
  		  }
  		  addFormatToken('H', ['HH', 2], 0, 'hour');
  		  addFormatToken('h', ['hh', 2], 0, hFormat);
  		  addFormatToken('k', ['kk', 2], 0, kFormat);
  		  addFormatToken('hmm', 0, 0, function () {
  		    return '' + hFormat.apply(this) + zeroFill(this.minutes(), 2);
  		  });
  		  addFormatToken('hmmss', 0, 0, function () {
  		    return '' + hFormat.apply(this) + zeroFill(this.minutes(), 2) + zeroFill(this.seconds(), 2);
  		  });
  		  addFormatToken('Hmm', 0, 0, function () {
  		    return '' + this.hours() + zeroFill(this.minutes(), 2);
  		  });
  		  addFormatToken('Hmmss', 0, 0, function () {
  		    return '' + this.hours() + zeroFill(this.minutes(), 2) + zeroFill(this.seconds(), 2);
  		  });
  		  function meridiem(token, lowercase) {
  		    addFormatToken(token, 0, 0, function () {
  		      return this.localeData().meridiem(this.hours(), this.minutes(), lowercase);
  		    });
  		  }
  		  meridiem('a', true);
  		  meridiem('A', false);

  		  // PARSING

  		  function matchMeridiem(isStrict, locale) {
  		    return locale._meridiemParse;
  		  }
  		  addRegexToken('a', matchMeridiem);
  		  addRegexToken('A', matchMeridiem);
  		  addRegexToken('H', match1to2, match1to2HasZero);
  		  addRegexToken('h', match1to2, match1to2NoLeadingZero);
  		  addRegexToken('k', match1to2, match1to2NoLeadingZero);
  		  addRegexToken('HH', match1to2, match2);
  		  addRegexToken('hh', match1to2, match2);
  		  addRegexToken('kk', match1to2, match2);
  		  addRegexToken('hmm', match3to4);
  		  addRegexToken('hmmss', match5to6);
  		  addRegexToken('Hmm', match3to4);
  		  addRegexToken('Hmmss', match5to6);
  		  addParseToken(['H', 'HH'], HOUR);
  		  addParseToken(['k', 'kk'], function (input, array, config) {
  		    var kInput = toInt(input);
  		    array[HOUR] = kInput === 24 ? 0 : kInput;
  		  });
  		  addParseToken(['a', 'A'], function (input, array, config) {
  		    config._isPm = config._locale.isPM(input);
  		    config._meridiem = input;
  		  });
  		  addParseToken(['h', 'hh'], function (input, array, config) {
  		    array[HOUR] = toInt(input);
  		    getParsingFlags(config).bigHour = true;
  		  });
  		  addParseToken('hmm', function (input, array, config) {
  		    var pos = input.length - 2;
  		    array[HOUR] = toInt(input.substr(0, pos));
  		    array[MINUTE] = toInt(input.substr(pos));
  		    getParsingFlags(config).bigHour = true;
  		  });
  		  addParseToken('hmmss', function (input, array, config) {
  		    var pos1 = input.length - 4,
  		      pos2 = input.length - 2;
  		    array[HOUR] = toInt(input.substr(0, pos1));
  		    array[MINUTE] = toInt(input.substr(pos1, 2));
  		    array[SECOND] = toInt(input.substr(pos2));
  		    getParsingFlags(config).bigHour = true;
  		  });
  		  addParseToken('Hmm', function (input, array, config) {
  		    var pos = input.length - 2;
  		    array[HOUR] = toInt(input.substr(0, pos));
  		    array[MINUTE] = toInt(input.substr(pos));
  		  });
  		  addParseToken('Hmmss', function (input, array, config) {
  		    var pos1 = input.length - 4,
  		      pos2 = input.length - 2;
  		    array[HOUR] = toInt(input.substr(0, pos1));
  		    array[MINUTE] = toInt(input.substr(pos1, 2));
  		    array[SECOND] = toInt(input.substr(pos2));
  		  });

  		  // LOCALES

  		  function localeIsPM(input) {
  		    // IE8 Quirks Mode & IE7 Standards Mode do not allow accessing strings like arrays
  		    // Using charAt should be more compatible.
  		    return (input + '').toLowerCase().charAt(0) === 'p';
  		  }
  		  var defaultLocaleMeridiemParse = /[ap]\.?m?\.?/i,
  		    // Setting the hour should keep the time, because the user explicitly
  		    // specified which hour they want. So trying to maintain the same hour (in
  		    // a new timezone) makes sense. Adding/subtracting hours does not follow
  		    // this rule.
  		    getSetHour = makeGetSet('Hours', true);
  		  function localeMeridiem(hours, minutes, isLower) {
  		    if (hours > 11) {
  		      return isLower ? 'pm' : 'PM';
  		    } else {
  		      return isLower ? 'am' : 'AM';
  		    }
  		  }
  		  var baseConfig = {
  		    calendar: defaultCalendar,
  		    longDateFormat: defaultLongDateFormat,
  		    invalidDate: defaultInvalidDate,
  		    ordinal: defaultOrdinal,
  		    dayOfMonthOrdinalParse: defaultDayOfMonthOrdinalParse,
  		    relativeTime: defaultRelativeTime,
  		    months: defaultLocaleMonths,
  		    monthsShort: defaultLocaleMonthsShort,
  		    week: defaultLocaleWeek,
  		    weekdays: defaultLocaleWeekdays,
  		    weekdaysMin: defaultLocaleWeekdaysMin,
  		    weekdaysShort: defaultLocaleWeekdaysShort,
  		    meridiemParse: defaultLocaleMeridiemParse
  		  };

  		  // internal storage for locale config files
  		  var locales = {},
  		    localeFamilies = {},
  		    globalLocale;
  		  function commonPrefix(arr1, arr2) {
  		    var i,
  		      minl = Math.min(arr1.length, arr2.length);
  		    for (i = 0; i < minl; i += 1) {
  		      if (arr1[i] !== arr2[i]) {
  		        return i;
  		      }
  		    }
  		    return minl;
  		  }
  		  function normalizeLocale(key) {
  		    return key ? key.toLowerCase().replace('_', '-') : key;
  		  }

  		  // pick the locale from the array
  		  // try ['en-au', 'en-gb'] as 'en-au', 'en-gb', 'en', as in move through the list trying each
  		  // substring from most specific to least, but move to the next array item if it's a more specific variant than the current root
  		  function chooseLocale(names) {
  		    var i = 0,
  		      j,
  		      next,
  		      locale,
  		      split;
  		    while (i < names.length) {
  		      split = normalizeLocale(names[i]).split('-');
  		      j = split.length;
  		      next = normalizeLocale(names[i + 1]);
  		      next = next ? next.split('-') : null;
  		      while (j > 0) {
  		        locale = loadLocale(split.slice(0, j).join('-'));
  		        if (locale) {
  		          return locale;
  		        }
  		        if (next && next.length >= j && commonPrefix(split, next) >= j - 1) {
  		          //the next array item is better than a shallower substring of this one
  		          break;
  		        }
  		        j--;
  		      }
  		      i++;
  		    }
  		    return globalLocale;
  		  }
  		  function isLocaleNameSane(name) {
  		    // Prevent names that look like filesystem paths, i.e contain '/' or '\'
  		    // Ensure name is available and function returns boolean
  		    return !!(name && name.match('^[^/\\\\]*$'));
  		  }
  		  function loadLocale(name) {
  		    var oldLocale = null,
  		      aliasedRequire;
  		    // TODO: Find a better way to register and load all the locales in Node
  		    if (locales[name] === undefined && 'object' !== 'undefined' && module && module.exports && isLocaleNameSane(name)) {
  		      try {
  		        oldLocale = globalLocale._abbr;
  		        aliasedRequire = commonjsRequire;
  		        aliasedRequire('./locale/' + name);
  		        getSetGlobalLocale(oldLocale);
  		      } catch (e) {
  		        // mark as not found to avoid repeating expensive file require call causing high CPU
  		        // when trying to find en-US, en_US, en-us for every format call
  		        locales[name] = null; // null means not found
  		      }
  		    }
  		    return locales[name];
  		  }

  		  // This function will load locale and then set the global locale.  If
  		  // no arguments are passed in, it will simply return the current global
  		  // locale key.
  		  function getSetGlobalLocale(key, values) {
  		    var data;
  		    if (key) {
  		      if (isUndefined(values)) {
  		        data = getLocale(key);
  		      } else {
  		        data = defineLocale(key, values);
  		      }
  		      if (data) {
  		        // moment.duration._locale = moment._locale = data;
  		        globalLocale = data;
  		      } else {
  		        if (typeof console !== 'undefined' && console.warn) {
  		          //warn user if arguments are passed but the locale could not be set
  		          console.warn('Locale ' + key + ' not found. Did you forget to load it?');
  		        }
  		      }
  		    }
  		    return globalLocale._abbr;
  		  }
  		  function defineLocale(name, config) {
  		    if (config !== null) {
  		      var locale,
  		        parentConfig = baseConfig;
  		      config.abbr = name;
  		      if (locales[name] != null) {
  		        deprecateSimple('defineLocaleOverride', 'use moment.updateLocale(localeName, config) to change ' + 'an existing locale. moment.defineLocale(localeName, ' + 'config) should only be used for creating a new locale ' + 'See http://momentjs.com/guides/#/warnings/define-locale/ for more info.');
  		        parentConfig = locales[name]._config;
  		      } else if (config.parentLocale != null) {
  		        if (locales[config.parentLocale] != null) {
  		          parentConfig = locales[config.parentLocale]._config;
  		        } else {
  		          locale = loadLocale(config.parentLocale);
  		          if (locale != null) {
  		            parentConfig = locale._config;
  		          } else {
  		            if (!localeFamilies[config.parentLocale]) {
  		              localeFamilies[config.parentLocale] = [];
  		            }
  		            localeFamilies[config.parentLocale].push({
  		              name: name,
  		              config: config
  		            });
  		            return null;
  		          }
  		        }
  		      }
  		      locales[name] = new Locale(mergeConfigs(parentConfig, config));
  		      if (localeFamilies[name]) {
  		        localeFamilies[name].forEach(function (x) {
  		          defineLocale(x.name, x.config);
  		        });
  		      }

  		      // backwards compat for now: also set the locale
  		      // make sure we set the locale AFTER all child locales have been
  		      // created, so we won't end up with the child locale set.
  		      getSetGlobalLocale(name);
  		      return locales[name];
  		    } else {
  		      // useful for testing
  		      delete locales[name];
  		      return null;
  		    }
  		  }
  		  function updateLocale(name, config) {
  		    if (config != null) {
  		      var locale,
  		        tmpLocale,
  		        parentConfig = baseConfig;
  		      if (locales[name] != null && locales[name].parentLocale != null) {
  		        // Update existing child locale in-place to avoid memory-leaks
  		        locales[name].set(mergeConfigs(locales[name]._config, config));
  		      } else {
  		        // MERGE
  		        tmpLocale = loadLocale(name);
  		        if (tmpLocale != null) {
  		          parentConfig = tmpLocale._config;
  		        }
  		        config = mergeConfigs(parentConfig, config);
  		        if (tmpLocale == null) {
  		          // updateLocale is called for creating a new locale
  		          // Set abbr so it will have a name (getters return
  		          // undefined otherwise).
  		          config.abbr = name;
  		        }
  		        locale = new Locale(config);
  		        locale.parentLocale = locales[name];
  		        locales[name] = locale;
  		      }

  		      // backwards compat for now: also set the locale
  		      getSetGlobalLocale(name);
  		    } else {
  		      // pass null for config to unupdate, useful for tests
  		      if (locales[name] != null) {
  		        if (locales[name].parentLocale != null) {
  		          locales[name] = locales[name].parentLocale;
  		          if (name === getSetGlobalLocale()) {
  		            getSetGlobalLocale(name);
  		          }
  		        } else if (locales[name] != null) {
  		          delete locales[name];
  		        }
  		      }
  		    }
  		    return locales[name];
  		  }

  		  // returns locale data
  		  function getLocale(key) {
  		    var locale;
  		    if (key && key._locale && key._locale._abbr) {
  		      key = key._locale._abbr;
  		    }
  		    if (!key) {
  		      return globalLocale;
  		    }
  		    if (!isArray(key)) {
  		      //short-circuit everything else
  		      locale = loadLocale(key);
  		      if (locale) {
  		        return locale;
  		      }
  		      key = [key];
  		    }
  		    return chooseLocale(key);
  		  }
  		  function listLocales() {
  		    return keys(locales);
  		  }
  		  function checkOverflow(m) {
  		    var overflow,
  		      a = m._a;
  		    if (a && getParsingFlags(m).overflow === -2) {
  		      overflow = a[MONTH] < 0 || a[MONTH] > 11 ? MONTH : a[DATE] < 1 || a[DATE] > daysInMonth(a[YEAR], a[MONTH]) ? DATE : a[HOUR] < 0 || a[HOUR] > 24 || a[HOUR] === 24 && (a[MINUTE] !== 0 || a[SECOND] !== 0 || a[MILLISECOND] !== 0) ? HOUR : a[MINUTE] < 0 || a[MINUTE] > 59 ? MINUTE : a[SECOND] < 0 || a[SECOND] > 59 ? SECOND : a[MILLISECOND] < 0 || a[MILLISECOND] > 999 ? MILLISECOND : -1;
  		      if (getParsingFlags(m)._overflowDayOfYear && (overflow < YEAR || overflow > DATE)) {
  		        overflow = DATE;
  		      }
  		      if (getParsingFlags(m)._overflowWeeks && overflow === -1) {
  		        overflow = WEEK;
  		      }
  		      if (getParsingFlags(m)._overflowWeekday && overflow === -1) {
  		        overflow = WEEKDAY;
  		      }
  		      getParsingFlags(m).overflow = overflow;
  		    }
  		    return m;
  		  }

  		  // iso 8601 regex
  		  // 0000-00-00 0000-W00 or 0000-W00-0 + T + 00 or 00:00 or 00:00:00 or 00:00:00.000 + +00:00 or +0000 or +00)
  		  var extendedIsoRegex = /^\s*((?:[+-]\d{6}|\d{4})-(?:\d\d-\d\d|W\d\d-\d|W\d\d|\d\d\d|\d\d))(?:(T| )(\d\d(?::\d\d(?::\d\d(?:[.,]\d+)?)?)?)([+-]\d\d(?::?\d\d)?|\s*Z)?)?$/,
  		    basicIsoRegex = /^\s*((?:[+-]\d{6}|\d{4})(?:\d\d\d\d|W\d\d\d|W\d\d|\d\d\d|\d\d|))(?:(T| )(\d\d(?:\d\d(?:\d\d(?:[.,]\d+)?)?)?)([+-]\d\d(?::?\d\d)?|\s*Z)?)?$/,
  		    tzRegex = /Z|[+-]\d\d(?::?\d\d)?/,
  		    isoDates = [['YYYYYY-MM-DD', /[+-]\d{6}-\d\d-\d\d/], ['YYYY-MM-DD', /\d{4}-\d\d-\d\d/], ['GGGG-[W]WW-E', /\d{4}-W\d\d-\d/], ['GGGG-[W]WW', /\d{4}-W\d\d/, false], ['YYYY-DDD', /\d{4}-\d{3}/], ['YYYY-MM', /\d{4}-\d\d/, false], ['YYYYYYMMDD', /[+-]\d{10}/], ['YYYYMMDD', /\d{8}/], ['GGGG[W]WWE', /\d{4}W\d{3}/], ['GGGG[W]WW', /\d{4}W\d{2}/, false], ['YYYYDDD', /\d{7}/], ['YYYYMM', /\d{6}/, false], ['YYYY', /\d{4}/, false]],
  		    // iso time formats and regexes
  		    isoTimes = [['HH:mm:ss.SSSS', /\d\d:\d\d:\d\d\.\d+/], ['HH:mm:ss,SSSS', /\d\d:\d\d:\d\d,\d+/], ['HH:mm:ss', /\d\d:\d\d:\d\d/], ['HH:mm', /\d\d:\d\d/], ['HHmmss.SSSS', /\d\d\d\d\d\d\.\d+/], ['HHmmss,SSSS', /\d\d\d\d\d\d,\d+/], ['HHmmss', /\d\d\d\d\d\d/], ['HHmm', /\d\d\d\d/], ['HH', /\d\d/]],
  		    aspNetJsonRegex = /^\/?Date\((-?\d+)/i,
  		    // RFC 2822 regex: For details see https://tools.ietf.org/html/rfc2822#section-3.3
  		    rfc2822 = /^(?:(Mon|Tue|Wed|Thu|Fri|Sat|Sun),?\s)?(\d{1,2})\s(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)\s(\d{2,4})\s(\d\d):(\d\d)(?::(\d\d))?\s(?:(UT|GMT|[ECMP][SD]T)|([Zz])|([+-]\d{4}))$/,
  		    obsOffsets = {
  		      UT: 0,
  		      GMT: 0,
  		      EDT: -4 * 60,
  		      EST: -5 * 60,
  		      CDT: -5 * 60,
  		      CST: -6 * 60,
  		      MDT: -6 * 60,
  		      MST: -7 * 60,
  		      PDT: -7 * 60,
  		      PST: -8 * 60
  		    };

  		  // date from iso format
  		  function configFromISO(config) {
  		    var i,
  		      l,
  		      string = config._i,
  		      match = extendedIsoRegex.exec(string) || basicIsoRegex.exec(string),
  		      allowTime,
  		      dateFormat,
  		      timeFormat,
  		      tzFormat,
  		      isoDatesLen = isoDates.length,
  		      isoTimesLen = isoTimes.length;
  		    if (match) {
  		      getParsingFlags(config).iso = true;
  		      for (i = 0, l = isoDatesLen; i < l; i++) {
  		        if (isoDates[i][1].exec(match[1])) {
  		          dateFormat = isoDates[i][0];
  		          allowTime = isoDates[i][2] !== false;
  		          break;
  		        }
  		      }
  		      if (dateFormat == null) {
  		        config._isValid = false;
  		        return;
  		      }
  		      if (match[3]) {
  		        for (i = 0, l = isoTimesLen; i < l; i++) {
  		          if (isoTimes[i][1].exec(match[3])) {
  		            // match[2] should be 'T' or space
  		            timeFormat = (match[2] || ' ') + isoTimes[i][0];
  		            break;
  		          }
  		        }
  		        if (timeFormat == null) {
  		          config._isValid = false;
  		          return;
  		        }
  		      }
  		      if (!allowTime && timeFormat != null) {
  		        config._isValid = false;
  		        return;
  		      }
  		      if (match[4]) {
  		        if (tzRegex.exec(match[4])) {
  		          tzFormat = 'Z';
  		        } else {
  		          config._isValid = false;
  		          return;
  		        }
  		      }
  		      config._f = dateFormat + (timeFormat || '') + (tzFormat || '');
  		      configFromStringAndFormat(config);
  		    } else {
  		      config._isValid = false;
  		    }
  		  }
  		  function extractFromRFC2822Strings(yearStr, monthStr, dayStr, hourStr, minuteStr, secondStr) {
  		    var result = [untruncateYear(yearStr), defaultLocaleMonthsShort.indexOf(monthStr), parseInt(dayStr, 10), parseInt(hourStr, 10), parseInt(minuteStr, 10)];
  		    if (secondStr) {
  		      result.push(parseInt(secondStr, 10));
  		    }
  		    return result;
  		  }
  		  function untruncateYear(yearStr) {
  		    var year = parseInt(yearStr, 10);
  		    if (year <= 49) {
  		      return 2000 + year;
  		    } else if (year <= 999) {
  		      return 1900 + year;
  		    }
  		    return year;
  		  }
  		  function preprocessRFC2822(s) {
  		    // Remove comments and folding whitespace and replace multiple-spaces with a single space
  		    return s.replace(/\([^()]*\)|[\n\t]/g, ' ').replace(/(\s\s+)/g, ' ').replace(/^\s\s*/, '').replace(/\s\s*$/, '');
  		  }
  		  function checkWeekday(weekdayStr, parsedInput, config) {
  		    if (weekdayStr) {
  		      // TODO: Replace the vanilla JS Date object with an independent day-of-week check.
  		      var weekdayProvided = defaultLocaleWeekdaysShort.indexOf(weekdayStr),
  		        weekdayActual = new Date(parsedInput[0], parsedInput[1], parsedInput[2]).getDay();
  		      if (weekdayProvided !== weekdayActual) {
  		        getParsingFlags(config).weekdayMismatch = true;
  		        config._isValid = false;
  		        return false;
  		      }
  		    }
  		    return true;
  		  }
  		  function calculateOffset(obsOffset, militaryOffset, numOffset) {
  		    if (obsOffset) {
  		      return obsOffsets[obsOffset];
  		    } else if (militaryOffset) {
  		      // the only allowed military tz is Z
  		      return 0;
  		    } else {
  		      var hm = parseInt(numOffset, 10),
  		        m = hm % 100,
  		        h = (hm - m) / 100;
  		      return h * 60 + m;
  		    }
  		  }

  		  // date and time from ref 2822 format
  		  function configFromRFC2822(config) {
  		    var match = rfc2822.exec(preprocessRFC2822(config._i)),
  		      parsedArray;
  		    if (match) {
  		      parsedArray = extractFromRFC2822Strings(match[4], match[3], match[2], match[5], match[6], match[7]);
  		      if (!checkWeekday(match[1], parsedArray, config)) {
  		        return;
  		      }
  		      config._a = parsedArray;
  		      config._tzm = calculateOffset(match[8], match[9], match[10]);
  		      config._d = createUTCDate.apply(null, config._a);
  		      config._d.setUTCMinutes(config._d.getUTCMinutes() - config._tzm);
  		      getParsingFlags(config).rfc2822 = true;
  		    } else {
  		      config._isValid = false;
  		    }
  		  }

  		  // date from 1) ASP.NET, 2) ISO, 3) RFC 2822 formats, or 4) optional fallback if parsing isn't strict
  		  function configFromString(config) {
  		    var matched = aspNetJsonRegex.exec(config._i);
  		    if (matched !== null) {
  		      config._d = new Date(+matched[1]);
  		      return;
  		    }
  		    configFromISO(config);
  		    if (config._isValid === false) {
  		      delete config._isValid;
  		    } else {
  		      return;
  		    }
  		    configFromRFC2822(config);
  		    if (config._isValid === false) {
  		      delete config._isValid;
  		    } else {
  		      return;
  		    }
  		    if (config._strict) {
  		      config._isValid = false;
  		    } else {
  		      // Final attempt, use Input Fallback
  		      hooks.createFromInputFallback(config);
  		    }
  		  }
  		  hooks.createFromInputFallback = deprecate('value provided is not in a recognized RFC2822 or ISO format. moment construction falls back to js Date(), ' + 'which is not reliable across all browsers and versions. Non RFC2822/ISO date formats are ' + 'discouraged. Please refer to http://momentjs.com/guides/#/warnings/js-date/ for more info.', function (config) {
  		    config._d = new Date(config._i + (config._useUTC ? ' UTC' : ''));
  		  });

  		  // Pick the first defined of two or three arguments.
  		  function defaults(a, b, c) {
  		    if (a != null) {
  		      return a;
  		    }
  		    if (b != null) {
  		      return b;
  		    }
  		    return c;
  		  }
  		  function currentDateArray(config) {
  		    // hooks is actually the exported moment object
  		    var nowValue = new Date(hooks.now());
  		    if (config._useUTC) {
  		      return [nowValue.getUTCFullYear(), nowValue.getUTCMonth(), nowValue.getUTCDate()];
  		    }
  		    return [nowValue.getFullYear(), nowValue.getMonth(), nowValue.getDate()];
  		  }

  		  // convert an array to a date.
  		  // the array should mirror the parameters below
  		  // note: all values past the year are optional and will default to the lowest possible value.
  		  // [year, month, day , hour, minute, second, millisecond]
  		  function configFromArray(config) {
  		    var i,
  		      date,
  		      input = [],
  		      currentDate,
  		      expectedWeekday,
  		      yearToUse;
  		    if (config._d) {
  		      return;
  		    }
  		    currentDate = currentDateArray(config);

  		    //compute day of the year from weeks and weekdays
  		    if (config._w && config._a[DATE] == null && config._a[MONTH] == null) {
  		      dayOfYearFromWeekInfo(config);
  		    }

  		    //if the day of the year is set, figure out what it is
  		    if (config._dayOfYear != null) {
  		      yearToUse = defaults(config._a[YEAR], currentDate[YEAR]);
  		      if (config._dayOfYear > daysInYear(yearToUse) || config._dayOfYear === 0) {
  		        getParsingFlags(config)._overflowDayOfYear = true;
  		      }
  		      date = createUTCDate(yearToUse, 0, config._dayOfYear);
  		      config._a[MONTH] = date.getUTCMonth();
  		      config._a[DATE] = date.getUTCDate();
  		    }

  		    // Default to current date.
  		    // * if no year, month, day of month are given, default to today
  		    // * if day of month is given, default month and year
  		    // * if month is given, default only year
  		    // * if year is given, don't default anything
  		    for (i = 0; i < 3 && config._a[i] == null; ++i) {
  		      config._a[i] = input[i] = currentDate[i];
  		    }

  		    // Zero out whatever was not defaulted, including time
  		    for (; i < 7; i++) {
  		      config._a[i] = input[i] = config._a[i] == null ? i === 2 ? 1 : 0 : config._a[i];
  		    }

  		    // Check for 24:00:00.000
  		    if (config._a[HOUR] === 24 && config._a[MINUTE] === 0 && config._a[SECOND] === 0 && config._a[MILLISECOND] === 0) {
  		      config._nextDay = true;
  		      config._a[HOUR] = 0;
  		    }
  		    config._d = (config._useUTC ? createUTCDate : createDate).apply(null, input);
  		    expectedWeekday = config._useUTC ? config._d.getUTCDay() : config._d.getDay();

  		    // Apply timezone offset from input. The actual utcOffset can be changed
  		    // with parseZone.
  		    if (config._tzm != null) {
  		      config._d.setUTCMinutes(config._d.getUTCMinutes() - config._tzm);
  		    }
  		    if (config._nextDay) {
  		      config._a[HOUR] = 24;
  		    }

  		    // check for mismatching day of week
  		    if (config._w && typeof config._w.d !== 'undefined' && config._w.d !== expectedWeekday) {
  		      getParsingFlags(config).weekdayMismatch = true;
  		    }
  		  }
  		  function dayOfYearFromWeekInfo(config) {
  		    var w, weekYear, week, weekday, dow, doy, temp, weekdayOverflow, curWeek;
  		    w = config._w;
  		    if (w.GG != null || w.W != null || w.E != null) {
  		      dow = 1;
  		      doy = 4;

  		      // TODO: We need to take the current isoWeekYear, but that depends on
  		      // how we interpret now (local, utc, fixed offset). So create
  		      // a now version of current config (take local/utc/offset flags, and
  		      // create now).
  		      weekYear = defaults(w.GG, config._a[YEAR], weekOfYear(createLocal(), 1, 4).year);
  		      week = defaults(w.W, 1);
  		      weekday = defaults(w.E, 1);
  		      if (weekday < 1 || weekday > 7) {
  		        weekdayOverflow = true;
  		      }
  		    } else {
  		      dow = config._locale._week.dow;
  		      doy = config._locale._week.doy;
  		      curWeek = weekOfYear(createLocal(), dow, doy);
  		      weekYear = defaults(w.gg, config._a[YEAR], curWeek.year);

  		      // Default to current week.
  		      week = defaults(w.w, curWeek.week);
  		      if (w.d != null) {
  		        // weekday -- low day numbers are considered next week
  		        weekday = w.d;
  		        if (weekday < 0 || weekday > 6) {
  		          weekdayOverflow = true;
  		        }
  		      } else if (w.e != null) {
  		        // local weekday -- counting starts from beginning of week
  		        weekday = w.e + dow;
  		        if (w.e < 0 || w.e > 6) {
  		          weekdayOverflow = true;
  		        }
  		      } else {
  		        // default to beginning of week
  		        weekday = dow;
  		      }
  		    }
  		    if (week < 1 || week > weeksInYear(weekYear, dow, doy)) {
  		      getParsingFlags(config)._overflowWeeks = true;
  		    } else if (weekdayOverflow != null) {
  		      getParsingFlags(config)._overflowWeekday = true;
  		    } else {
  		      temp = dayOfYearFromWeeks(weekYear, week, weekday, dow, doy);
  		      config._a[YEAR] = temp.year;
  		      config._dayOfYear = temp.dayOfYear;
  		    }
  		  }

  		  // constant that refers to the ISO standard
  		  hooks.ISO_8601 = function () {};

  		  // constant that refers to the RFC 2822 form
  		  hooks.RFC_2822 = function () {};

  		  // date from string and format string
  		  function configFromStringAndFormat(config) {
  		    // TODO: Move this to another part of the creation flow to prevent circular deps
  		    if (config._f === hooks.ISO_8601) {
  		      configFromISO(config);
  		      return;
  		    }
  		    if (config._f === hooks.RFC_2822) {
  		      configFromRFC2822(config);
  		      return;
  		    }
  		    config._a = [];
  		    getParsingFlags(config).empty = true;

  		    // This array is used to make a Date, either with `new Date` or `Date.UTC`
  		    var string = '' + config._i,
  		      i,
  		      parsedInput,
  		      tokens,
  		      token,
  		      skipped,
  		      stringLength = string.length,
  		      totalParsedInputLength = 0,
  		      era,
  		      tokenLen;
  		    tokens = expandFormat(config._f, config._locale).match(formattingTokens) || [];
  		    tokenLen = tokens.length;
  		    for (i = 0; i < tokenLen; i++) {
  		      token = tokens[i];
  		      parsedInput = (string.match(getParseRegexForToken(token, config)) || [])[0];
  		      if (parsedInput) {
  		        skipped = string.substr(0, string.indexOf(parsedInput));
  		        if (skipped.length > 0) {
  		          getParsingFlags(config).unusedInput.push(skipped);
  		        }
  		        string = string.slice(string.indexOf(parsedInput) + parsedInput.length);
  		        totalParsedInputLength += parsedInput.length;
  		      }
  		      // don't parse if it's not a known token
  		      if (formatTokenFunctions[token]) {
  		        if (parsedInput) {
  		          getParsingFlags(config).empty = false;
  		        } else {
  		          getParsingFlags(config).unusedTokens.push(token);
  		        }
  		        addTimeToArrayFromToken(token, parsedInput, config);
  		      } else if (config._strict && !parsedInput) {
  		        getParsingFlags(config).unusedTokens.push(token);
  		      }
  		    }

  		    // add remaining unparsed input length to the string
  		    getParsingFlags(config).charsLeftOver = stringLength - totalParsedInputLength;
  		    if (string.length > 0) {
  		      getParsingFlags(config).unusedInput.push(string);
  		    }

  		    // clear _12h flag if hour is <= 12
  		    if (config._a[HOUR] <= 12 && getParsingFlags(config).bigHour === true && config._a[HOUR] > 0) {
  		      getParsingFlags(config).bigHour = undefined;
  		    }
  		    getParsingFlags(config).parsedDateParts = config._a.slice(0);
  		    getParsingFlags(config).meridiem = config._meridiem;
  		    // handle meridiem
  		    config._a[HOUR] = meridiemFixWrap(config._locale, config._a[HOUR], config._meridiem);

  		    // handle era
  		    era = getParsingFlags(config).era;
  		    if (era !== null) {
  		      config._a[YEAR] = config._locale.erasConvertYear(era, config._a[YEAR]);
  		    }
  		    configFromArray(config);
  		    checkOverflow(config);
  		  }
  		  function meridiemFixWrap(locale, hour, meridiem) {
  		    var isPm;
  		    if (meridiem == null) {
  		      // nothing to do
  		      return hour;
  		    }
  		    if (locale.meridiemHour != null) {
  		      return locale.meridiemHour(hour, meridiem);
  		    } else if (locale.isPM != null) {
  		      // Fallback
  		      isPm = locale.isPM(meridiem);
  		      if (isPm && hour < 12) {
  		        hour += 12;
  		      }
  		      if (!isPm && hour === 12) {
  		        hour = 0;
  		      }
  		      return hour;
  		    } else {
  		      // this is not supposed to happen
  		      return hour;
  		    }
  		  }

  		  // date from string and array of format strings
  		  function configFromStringAndArray(config) {
  		    var tempConfig,
  		      bestMoment,
  		      scoreToBeat,
  		      i,
  		      currentScore,
  		      validFormatFound,
  		      bestFormatIsValid = false,
  		      configfLen = config._f.length;
  		    if (configfLen === 0) {
  		      getParsingFlags(config).invalidFormat = true;
  		      config._d = new Date(NaN);
  		      return;
  		    }
  		    for (i = 0; i < configfLen; i++) {
  		      currentScore = 0;
  		      validFormatFound = false;
  		      tempConfig = copyConfig({}, config);
  		      if (config._useUTC != null) {
  		        tempConfig._useUTC = config._useUTC;
  		      }
  		      tempConfig._f = config._f[i];
  		      configFromStringAndFormat(tempConfig);
  		      if (isValid(tempConfig)) {
  		        validFormatFound = true;
  		      }

  		      // if there is any input that was not parsed add a penalty for that format
  		      currentScore += getParsingFlags(tempConfig).charsLeftOver;

  		      //or tokens
  		      currentScore += getParsingFlags(tempConfig).unusedTokens.length * 10;
  		      getParsingFlags(tempConfig).score = currentScore;
  		      if (!bestFormatIsValid) {
  		        if (scoreToBeat == null || currentScore < scoreToBeat || validFormatFound) {
  		          scoreToBeat = currentScore;
  		          bestMoment = tempConfig;
  		          if (validFormatFound) {
  		            bestFormatIsValid = true;
  		          }
  		        }
  		      } else {
  		        if (currentScore < scoreToBeat) {
  		          scoreToBeat = currentScore;
  		          bestMoment = tempConfig;
  		        }
  		      }
  		    }
  		    extend(config, bestMoment || tempConfig);
  		  }
  		  function configFromObject(config) {
  		    if (config._d) {
  		      return;
  		    }
  		    var i = normalizeObjectUnits(config._i),
  		      dayOrDate = i.day === undefined ? i.date : i.day;
  		    config._a = map([i.year, i.month, dayOrDate, i.hour, i.minute, i.second, i.millisecond], function (obj) {
  		      return obj && parseInt(obj, 10);
  		    });
  		    configFromArray(config);
  		  }
  		  function createFromConfig(config) {
  		    var res = new Moment(checkOverflow(prepareConfig(config)));
  		    if (res._nextDay) {
  		      // Adding is smart enough around DST
  		      res.add(1, 'd');
  		      res._nextDay = undefined;
  		    }
  		    return res;
  		  }
  		  function prepareConfig(config) {
  		    var input = config._i,
  		      format = config._f;
  		    config._locale = config._locale || getLocale(config._l);
  		    if (input === null || format === undefined && input === '') {
  		      return createInvalid({
  		        nullInput: true
  		      });
  		    }
  		    if (typeof input === 'string') {
  		      config._i = input = config._locale.preparse(input);
  		    }
  		    if (isMoment(input)) {
  		      return new Moment(checkOverflow(input));
  		    } else if (isDate(input)) {
  		      config._d = input;
  		    } else if (isArray(format)) {
  		      configFromStringAndArray(config);
  		    } else if (format) {
  		      configFromStringAndFormat(config);
  		    } else {
  		      configFromInput(config);
  		    }
  		    if (!isValid(config)) {
  		      config._d = null;
  		    }
  		    return config;
  		  }
  		  function configFromInput(config) {
  		    var input = config._i;
  		    if (isUndefined(input)) {
  		      config._d = new Date(hooks.now());
  		    } else if (isDate(input)) {
  		      config._d = new Date(input.valueOf());
  		    } else if (typeof input === 'string') {
  		      configFromString(config);
  		    } else if (isArray(input)) {
  		      config._a = map(input.slice(0), function (obj) {
  		        return parseInt(obj, 10);
  		      });
  		      configFromArray(config);
  		    } else if (isObject(input)) {
  		      configFromObject(config);
  		    } else if (isNumber(input)) {
  		      // from milliseconds
  		      config._d = new Date(input);
  		    } else {
  		      hooks.createFromInputFallback(config);
  		    }
  		  }
  		  function createLocalOrUTC(input, format, locale, strict, isUTC) {
  		    var c = {};
  		    if (format === true || format === false) {
  		      strict = format;
  		      format = undefined;
  		    }
  		    if (locale === true || locale === false) {
  		      strict = locale;
  		      locale = undefined;
  		    }
  		    if (isObject(input) && isObjectEmpty(input) || isArray(input) && input.length === 0) {
  		      input = undefined;
  		    }
  		    // object construction must be done this way.
  		    // https://github.com/moment/moment/issues/1423
  		    c._isAMomentObject = true;
  		    c._useUTC = c._isUTC = isUTC;
  		    c._l = locale;
  		    c._i = input;
  		    c._f = format;
  		    c._strict = strict;
  		    return createFromConfig(c);
  		  }
  		  function createLocal(input, format, locale, strict) {
  		    return createLocalOrUTC(input, format, locale, strict, false);
  		  }
  		  var prototypeMin = deprecate('moment().min is deprecated, use moment.max instead. http://momentjs.com/guides/#/warnings/min-max/', function () {
  		      var other = createLocal.apply(null, arguments);
  		      if (this.isValid() && other.isValid()) {
  		        return other < this ? this : other;
  		      } else {
  		        return createInvalid();
  		      }
  		    }),
  		    prototypeMax = deprecate('moment().max is deprecated, use moment.min instead. http://momentjs.com/guides/#/warnings/min-max/', function () {
  		      var other = createLocal.apply(null, arguments);
  		      if (this.isValid() && other.isValid()) {
  		        return other > this ? this : other;
  		      } else {
  		        return createInvalid();
  		      }
  		    });

  		  // Pick a moment m from moments so that m[fn](other) is true for all
  		  // other. This relies on the function fn to be transitive.
  		  //
  		  // moments should either be an array of moment objects or an array, whose
  		  // first element is an array of moment objects.
  		  function pickBy(fn, moments) {
  		    var res, i;
  		    if (moments.length === 1 && isArray(moments[0])) {
  		      moments = moments[0];
  		    }
  		    if (!moments.length) {
  		      return createLocal();
  		    }
  		    res = moments[0];
  		    for (i = 1; i < moments.length; ++i) {
  		      if (!moments[i].isValid() || moments[i][fn](res)) {
  		        res = moments[i];
  		      }
  		    }
  		    return res;
  		  }

  		  // TODO: Use [].sort instead?
  		  function min() {
  		    var args = [].slice.call(arguments, 0);
  		    return pickBy('isBefore', args);
  		  }
  		  function max() {
  		    var args = [].slice.call(arguments, 0);
  		    return pickBy('isAfter', args);
  		  }
  		  var now = function () {
  		    return Date.now ? Date.now() : +new Date();
  		  };
  		  var ordering = ['year', 'quarter', 'month', 'week', 'day', 'hour', 'minute', 'second', 'millisecond'];
  		  function isDurationValid(m) {
  		    var key,
  		      unitHasDecimal = false,
  		      i,
  		      orderLen = ordering.length;
  		    for (key in m) {
  		      if (hasOwnProp(m, key) && !(indexOf.call(ordering, key) !== -1 && (m[key] == null || !isNaN(m[key])))) {
  		        return false;
  		      }
  		    }
  		    for (i = 0; i < orderLen; ++i) {
  		      if (m[ordering[i]]) {
  		        if (unitHasDecimal) {
  		          return false; // only allow non-integers for smallest unit
  		        }
  		        if (parseFloat(m[ordering[i]]) !== toInt(m[ordering[i]])) {
  		          unitHasDecimal = true;
  		        }
  		      }
  		    }
  		    return true;
  		  }
  		  function isValid$1() {
  		    return this._isValid;
  		  }
  		  function createInvalid$1() {
  		    return createDuration(NaN);
  		  }
  		  function Duration(duration) {
  		    var normalizedInput = normalizeObjectUnits(duration),
  		      years = normalizedInput.year || 0,
  		      quarters = normalizedInput.quarter || 0,
  		      months = normalizedInput.month || 0,
  		      weeks = normalizedInput.week || normalizedInput.isoWeek || 0,
  		      days = normalizedInput.day || 0,
  		      hours = normalizedInput.hour || 0,
  		      minutes = normalizedInput.minute || 0,
  		      seconds = normalizedInput.second || 0,
  		      milliseconds = normalizedInput.millisecond || 0;
  		    this._isValid = isDurationValid(normalizedInput);

  		    // representation for dateAddRemove
  		    this._milliseconds = +milliseconds + seconds * 1e3 +
  		    // 1000
  		    minutes * 6e4 +
  		    // 1000 * 60
  		    hours * 1000 * 60 * 60; //using 1000 * 60 * 60 instead of 36e5 to avoid floating point rounding errors https://github.com/moment/moment/issues/2978
  		    // Because of dateAddRemove treats 24 hours as different from a
  		    // day when working around DST, we need to store them separately
  		    this._days = +days + weeks * 7;
  		    // It is impossible to translate months into days without knowing
  		    // which months you are are talking about, so we have to store
  		    // it separately.
  		    this._months = +months + quarters * 3 + years * 12;
  		    this._data = {};
  		    this._locale = getLocale();
  		    this._bubble();
  		  }
  		  function isDuration(obj) {
  		    return obj instanceof Duration;
  		  }
  		  function absRound(number) {
  		    if (number < 0) {
  		      return Math.round(-1 * number) * -1;
  		    } else {
  		      return Math.round(number);
  		    }
  		  }

  		  // compare two arrays, return the number of differences
  		  function compareArrays(array1, array2, dontConvert) {
  		    var len = Math.min(array1.length, array2.length),
  		      lengthDiff = Math.abs(array1.length - array2.length),
  		      diffs = 0,
  		      i;
  		    for (i = 0; i < len; i++) {
  		      if (toInt(array1[i]) !== toInt(array2[i])) {
  		        diffs++;
  		      }
  		    }
  		    return diffs + lengthDiff;
  		  }

  		  // FORMATTING

  		  function offset(token, separator) {
  		    addFormatToken(token, 0, 0, function () {
  		      var offset = this.utcOffset(),
  		        sign = '+';
  		      if (offset < 0) {
  		        offset = -offset;
  		        sign = '-';
  		      }
  		      return sign + zeroFill(~~(offset / 60), 2) + separator + zeroFill(~~offset % 60, 2);
  		    });
  		  }
  		  offset('Z', ':');
  		  offset('ZZ', '');

  		  // PARSING

  		  addRegexToken('Z', matchShortOffset);
  		  addRegexToken('ZZ', matchShortOffset);
  		  addParseToken(['Z', 'ZZ'], function (input, array, config) {
  		    config._useUTC = true;
  		    config._tzm = offsetFromString(matchShortOffset, input);
  		  });

  		  // HELPERS

  		  // timezone chunker
  		  // '+10:00' > ['10',  '00']
  		  // '-1530'  > ['-15', '30']
  		  var chunkOffset = /([\+\-]|\d\d)/gi;
  		  function offsetFromString(matcher, string) {
  		    var matches = (string || '').match(matcher),
  		      chunk,
  		      parts,
  		      minutes;
  		    if (matches === null) {
  		      return null;
  		    }
  		    chunk = matches[matches.length - 1] || [];
  		    parts = (chunk + '').match(chunkOffset) || ['-', 0, 0];
  		    minutes = +(parts[1] * 60) + toInt(parts[2]);
  		    return minutes === 0 ? 0 : parts[0] === '+' ? minutes : -minutes;
  		  }

  		  // Return a moment from input, that is local/utc/zone equivalent to model.
  		  function cloneWithOffset(input, model) {
  		    var res, diff;
  		    if (model._isUTC) {
  		      res = model.clone();
  		      diff = (isMoment(input) || isDate(input) ? input.valueOf() : createLocal(input).valueOf()) - res.valueOf();
  		      // Use low-level api, because this fn is low-level api.
  		      res._d.setTime(res._d.valueOf() + diff);
  		      hooks.updateOffset(res, false);
  		      return res;
  		    } else {
  		      return createLocal(input).local();
  		    }
  		  }
  		  function getDateOffset(m) {
  		    // On Firefox.24 Date#getTimezoneOffset returns a floating point.
  		    // https://github.com/moment/moment/pull/1871
  		    return -Math.round(m._d.getTimezoneOffset());
  		  }

  		  // HOOKS

  		  // This function will be called whenever a moment is mutated.
  		  // It is intended to keep the offset in sync with the timezone.
  		  hooks.updateOffset = function () {};

  		  // MOMENTS

  		  // keepLocalTime = true means only change the timezone, without
  		  // affecting the local hour. So 5:31:26 +0300 --[utcOffset(2, true)]-->
  		  // 5:31:26 +0200 It is possible that 5:31:26 doesn't exist with offset
  		  // +0200, so we adjust the time as needed, to be valid.
  		  //
  		  // Keeping the time actually adds/subtracts (one hour)
  		  // from the actual represented time. That is why we call updateOffset
  		  // a second time. In case it wants us to change the offset again
  		  // _changeInProgress == true case, then we have to adjust, because
  		  // there is no such time in the given timezone.
  		  function getSetOffset(input, keepLocalTime, keepMinutes) {
  		    var offset = this._offset || 0,
  		      localAdjust;
  		    if (!this.isValid()) {
  		      return input != null ? this : NaN;
  		    }
  		    if (input != null) {
  		      if (typeof input === 'string') {
  		        input = offsetFromString(matchShortOffset, input);
  		        if (input === null) {
  		          return this;
  		        }
  		      } else if (Math.abs(input) < 16 && !keepMinutes) {
  		        input = input * 60;
  		      }
  		      if (!this._isUTC && keepLocalTime) {
  		        localAdjust = getDateOffset(this);
  		      }
  		      this._offset = input;
  		      this._isUTC = true;
  		      if (localAdjust != null) {
  		        this.add(localAdjust, 'm');
  		      }
  		      if (offset !== input) {
  		        if (!keepLocalTime || this._changeInProgress) {
  		          addSubtract(this, createDuration(input - offset, 'm'), 1, false);
  		        } else if (!this._changeInProgress) {
  		          this._changeInProgress = true;
  		          hooks.updateOffset(this, true);
  		          this._changeInProgress = null;
  		        }
  		      }
  		      return this;
  		    } else {
  		      return this._isUTC ? offset : getDateOffset(this);
  		    }
  		  }
  		  function getSetZone(input, keepLocalTime) {
  		    if (input != null) {
  		      if (typeof input !== 'string') {
  		        input = -input;
  		      }
  		      this.utcOffset(input, keepLocalTime);
  		      return this;
  		    } else {
  		      return -this.utcOffset();
  		    }
  		  }
  		  function setOffsetToUTC(keepLocalTime) {
  		    return this.utcOffset(0, keepLocalTime);
  		  }
  		  function setOffsetToLocal(keepLocalTime) {
  		    if (this._isUTC) {
  		      this.utcOffset(0, keepLocalTime);
  		      this._isUTC = false;
  		      if (keepLocalTime) {
  		        this.subtract(getDateOffset(this), 'm');
  		      }
  		    }
  		    return this;
  		  }
  		  function setOffsetToParsedOffset() {
  		    if (this._tzm != null) {
  		      this.utcOffset(this._tzm, false, true);
  		    } else if (typeof this._i === 'string') {
  		      var tZone = offsetFromString(matchOffset, this._i);
  		      if (tZone != null) {
  		        this.utcOffset(tZone);
  		      } else {
  		        this.utcOffset(0, true);
  		      }
  		    }
  		    return this;
  		  }
  		  function hasAlignedHourOffset(input) {
  		    if (!this.isValid()) {
  		      return false;
  		    }
  		    input = input ? createLocal(input).utcOffset() : 0;
  		    return (this.utcOffset() - input) % 60 === 0;
  		  }
  		  function isDaylightSavingTime() {
  		    return this.utcOffset() > this.clone().month(0).utcOffset() || this.utcOffset() > this.clone().month(5).utcOffset();
  		  }
  		  function isDaylightSavingTimeShifted() {
  		    if (!isUndefined(this._isDSTShifted)) {
  		      return this._isDSTShifted;
  		    }
  		    var c = {},
  		      other;
  		    copyConfig(c, this);
  		    c = prepareConfig(c);
  		    if (c._a) {
  		      other = c._isUTC ? createUTC(c._a) : createLocal(c._a);
  		      this._isDSTShifted = this.isValid() && compareArrays(c._a, other.toArray()) > 0;
  		    } else {
  		      this._isDSTShifted = false;
  		    }
  		    return this._isDSTShifted;
  		  }
  		  function isLocal() {
  		    return this.isValid() ? !this._isUTC : false;
  		  }
  		  function isUtcOffset() {
  		    return this.isValid() ? this._isUTC : false;
  		  }
  		  function isUtc() {
  		    return this.isValid() ? this._isUTC && this._offset === 0 : false;
  		  }

  		  // ASP.NET json date format regex
  		  var aspNetRegex = /^(-|\+)?(?:(\d*)[. ])?(\d+):(\d+)(?::(\d+)(\.\d*)?)?$/,
  		    // from http://docs.closure-library.googlecode.com/git/closure_goog_date_date.js.source.html
  		    // somewhat more in line with 4.4.3.2 2004 spec, but allows decimal anywhere
  		    // and further modified to allow for strings containing both week and day
  		    isoRegex = /^(-|\+)?P(?:([-+]?[0-9,.]*)Y)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)W)?(?:([-+]?[0-9,.]*)D)?(?:T(?:([-+]?[0-9,.]*)H)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)S)?)?$/;
  		  function createDuration(input, key) {
  		    var duration = input,
  		      // matching against regexp is expensive, do it on demand
  		      match = null,
  		      sign,
  		      ret,
  		      diffRes;
  		    if (isDuration(input)) {
  		      duration = {
  		        ms: input._milliseconds,
  		        d: input._days,
  		        M: input._months
  		      };
  		    } else if (isNumber(input) || !isNaN(+input)) {
  		      duration = {};
  		      if (key) {
  		        duration[key] = +input;
  		      } else {
  		        duration.milliseconds = +input;
  		      }
  		    } else if (match = aspNetRegex.exec(input)) {
  		      sign = match[1] === '-' ? -1 : 1;
  		      duration = {
  		        y: 0,
  		        d: toInt(match[DATE]) * sign,
  		        h: toInt(match[HOUR]) * sign,
  		        m: toInt(match[MINUTE]) * sign,
  		        s: toInt(match[SECOND]) * sign,
  		        ms: toInt(absRound(match[MILLISECOND] * 1000)) * sign // the millisecond decimal point is included in the match
  		      };
  		    } else if (match = isoRegex.exec(input)) {
  		      sign = match[1] === '-' ? -1 : 1;
  		      duration = {
  		        y: parseIso(match[2], sign),
  		        M: parseIso(match[3], sign),
  		        w: parseIso(match[4], sign),
  		        d: parseIso(match[5], sign),
  		        h: parseIso(match[6], sign),
  		        m: parseIso(match[7], sign),
  		        s: parseIso(match[8], sign)
  		      };
  		    } else if (duration == null) {
  		      // checks for null or undefined
  		      duration = {};
  		    } else if (typeof duration === 'object' && ('from' in duration || 'to' in duration)) {
  		      diffRes = momentsDifference(createLocal(duration.from), createLocal(duration.to));
  		      duration = {};
  		      duration.ms = diffRes.milliseconds;
  		      duration.M = diffRes.months;
  		    }
  		    ret = new Duration(duration);
  		    if (isDuration(input) && hasOwnProp(input, '_locale')) {
  		      ret._locale = input._locale;
  		    }
  		    if (isDuration(input) && hasOwnProp(input, '_isValid')) {
  		      ret._isValid = input._isValid;
  		    }
  		    return ret;
  		  }
  		  createDuration.fn = Duration.prototype;
  		  createDuration.invalid = createInvalid$1;
  		  function parseIso(inp, sign) {
  		    // We'd normally use ~~inp for this, but unfortunately it also
  		    // converts floats to ints.
  		    // inp may be undefined, so careful calling replace on it.
  		    var res = inp && parseFloat(inp.replace(',', '.'));
  		    // apply sign while we're at it
  		    return (isNaN(res) ? 0 : res) * sign;
  		  }
  		  function positiveMomentsDifference(base, other) {
  		    var res = {};
  		    res.months = other.month() - base.month() + (other.year() - base.year()) * 12;
  		    if (base.clone().add(res.months, 'M').isAfter(other)) {
  		      --res.months;
  		    }
  		    res.milliseconds = +other - +base.clone().add(res.months, 'M');
  		    return res;
  		  }
  		  function momentsDifference(base, other) {
  		    var res;
  		    if (!(base.isValid() && other.isValid())) {
  		      return {
  		        milliseconds: 0,
  		        months: 0
  		      };
  		    }
  		    other = cloneWithOffset(other, base);
  		    if (base.isBefore(other)) {
  		      res = positiveMomentsDifference(base, other);
  		    } else {
  		      res = positiveMomentsDifference(other, base);
  		      res.milliseconds = -res.milliseconds;
  		      res.months = -res.months;
  		    }
  		    return res;
  		  }

  		  // TODO: remove 'name' arg after deprecation is removed
  		  function createAdder(direction, name) {
  		    return function (val, period) {
  		      var dur, tmp;
  		      //invert the arguments, but complain about it
  		      if (period !== null && !isNaN(+period)) {
  		        deprecateSimple(name, 'moment().' + name + '(period, number) is deprecated. Please use moment().' + name + '(number, period). ' + 'See http://momentjs.com/guides/#/warnings/add-inverted-param/ for more info.');
  		        tmp = val;
  		        val = period;
  		        period = tmp;
  		      }
  		      dur = createDuration(val, period);
  		      addSubtract(this, dur, direction);
  		      return this;
  		    };
  		  }
  		  function addSubtract(mom, duration, isAdding, updateOffset) {
  		    var milliseconds = duration._milliseconds,
  		      days = absRound(duration._days),
  		      months = absRound(duration._months);
  		    if (!mom.isValid()) {
  		      // No op
  		      return;
  		    }
  		    updateOffset = updateOffset == null ? true : updateOffset;
  		    if (months) {
  		      setMonth(mom, get(mom, 'Month') + months * isAdding);
  		    }
  		    if (days) {
  		      set$1(mom, 'Date', get(mom, 'Date') + days * isAdding);
  		    }
  		    if (milliseconds) {
  		      mom._d.setTime(mom._d.valueOf() + milliseconds * isAdding);
  		    }
  		    if (updateOffset) {
  		      hooks.updateOffset(mom, days || months);
  		    }
  		  }
  		  var add = createAdder(1, 'add'),
  		    subtract = createAdder(-1, 'subtract');
  		  function isString(input) {
  		    return typeof input === 'string' || input instanceof String;
  		  }

  		  // type MomentInput = Moment | Date | string | number | (number | string)[] | MomentInputObject | void; // null | undefined
  		  function isMomentInput(input) {
  		    return isMoment(input) || isDate(input) || isString(input) || isNumber(input) || isNumberOrStringArray(input) || isMomentInputObject(input) || input === null || input === undefined;
  		  }
  		  function isMomentInputObject(input) {
  		    var objectTest = isObject(input) && !isObjectEmpty(input),
  		      propertyTest = false,
  		      properties = ['years', 'year', 'y', 'months', 'month', 'M', 'days', 'day', 'd', 'dates', 'date', 'D', 'hours', 'hour', 'h', 'minutes', 'minute', 'm', 'seconds', 'second', 's', 'milliseconds', 'millisecond', 'ms'],
  		      i,
  		      property,
  		      propertyLen = properties.length;
  		    for (i = 0; i < propertyLen; i += 1) {
  		      property = properties[i];
  		      propertyTest = propertyTest || hasOwnProp(input, property);
  		    }
  		    return objectTest && propertyTest;
  		  }
  		  function isNumberOrStringArray(input) {
  		    var arrayTest = isArray(input),
  		      dataTypeTest = false;
  		    if (arrayTest) {
  		      dataTypeTest = input.filter(function (item) {
  		        return !isNumber(item) && isString(input);
  		      }).length === 0;
  		    }
  		    return arrayTest && dataTypeTest;
  		  }
  		  function isCalendarSpec(input) {
  		    var objectTest = isObject(input) && !isObjectEmpty(input),
  		      propertyTest = false,
  		      properties = ['sameDay', 'nextDay', 'lastDay', 'nextWeek', 'lastWeek', 'sameElse'],
  		      i,
  		      property;
  		    for (i = 0; i < properties.length; i += 1) {
  		      property = properties[i];
  		      propertyTest = propertyTest || hasOwnProp(input, property);
  		    }
  		    return objectTest && propertyTest;
  		  }
  		  function getCalendarFormat(myMoment, now) {
  		    var diff = myMoment.diff(now, 'days', true);
  		    return diff < -6 ? 'sameElse' : diff < -1 ? 'lastWeek' : diff < 0 ? 'lastDay' : diff < 1 ? 'sameDay' : diff < 2 ? 'nextDay' : diff < 7 ? 'nextWeek' : 'sameElse';
  		  }
  		  function calendar$1(time, formats) {
  		    // Support for single parameter, formats only overload to the calendar function
  		    if (arguments.length === 1) {
  		      if (!arguments[0]) {
  		        time = undefined;
  		        formats = undefined;
  		      } else if (isMomentInput(arguments[0])) {
  		        time = arguments[0];
  		        formats = undefined;
  		      } else if (isCalendarSpec(arguments[0])) {
  		        formats = arguments[0];
  		        time = undefined;
  		      }
  		    }
  		    // We want to compare the start of today, vs this.
  		    // Getting start-of-today depends on whether we're local/utc/offset or not.
  		    var now = time || createLocal(),
  		      sod = cloneWithOffset(now, this).startOf('day'),
  		      format = hooks.calendarFormat(this, sod) || 'sameElse',
  		      output = formats && (isFunction(formats[format]) ? formats[format].call(this, now) : formats[format]);
  		    return this.format(output || this.localeData().calendar(format, this, createLocal(now)));
  		  }
  		  function clone() {
  		    return new Moment(this);
  		  }
  		  function isAfter(input, units) {
  		    var localInput = isMoment(input) ? input : createLocal(input);
  		    if (!(this.isValid() && localInput.isValid())) {
  		      return false;
  		    }
  		    units = normalizeUnits(units) || 'millisecond';
  		    if (units === 'millisecond') {
  		      return this.valueOf() > localInput.valueOf();
  		    } else {
  		      return localInput.valueOf() < this.clone().startOf(units).valueOf();
  		    }
  		  }
  		  function isBefore(input, units) {
  		    var localInput = isMoment(input) ? input : createLocal(input);
  		    if (!(this.isValid() && localInput.isValid())) {
  		      return false;
  		    }
  		    units = normalizeUnits(units) || 'millisecond';
  		    if (units === 'millisecond') {
  		      return this.valueOf() < localInput.valueOf();
  		    } else {
  		      return this.clone().endOf(units).valueOf() < localInput.valueOf();
  		    }
  		  }
  		  function isBetween(from, to, units, inclusivity) {
  		    var localFrom = isMoment(from) ? from : createLocal(from),
  		      localTo = isMoment(to) ? to : createLocal(to);
  		    if (!(this.isValid() && localFrom.isValid() && localTo.isValid())) {
  		      return false;
  		    }
  		    inclusivity = inclusivity || '()';
  		    return (inclusivity[0] === '(' ? this.isAfter(localFrom, units) : !this.isBefore(localFrom, units)) && (inclusivity[1] === ')' ? this.isBefore(localTo, units) : !this.isAfter(localTo, units));
  		  }
  		  function isSame(input, units) {
  		    var localInput = isMoment(input) ? input : createLocal(input),
  		      inputMs;
  		    if (!(this.isValid() && localInput.isValid())) {
  		      return false;
  		    }
  		    units = normalizeUnits(units) || 'millisecond';
  		    if (units === 'millisecond') {
  		      return this.valueOf() === localInput.valueOf();
  		    } else {
  		      inputMs = localInput.valueOf();
  		      return this.clone().startOf(units).valueOf() <= inputMs && inputMs <= this.clone().endOf(units).valueOf();
  		    }
  		  }
  		  function isSameOrAfter(input, units) {
  		    return this.isSame(input, units) || this.isAfter(input, units);
  		  }
  		  function isSameOrBefore(input, units) {
  		    return this.isSame(input, units) || this.isBefore(input, units);
  		  }
  		  function diff(input, units, asFloat) {
  		    var that, zoneDelta, output;
  		    if (!this.isValid()) {
  		      return NaN;
  		    }
  		    that = cloneWithOffset(input, this);
  		    if (!that.isValid()) {
  		      return NaN;
  		    }
  		    zoneDelta = (that.utcOffset() - this.utcOffset()) * 6e4;
  		    units = normalizeUnits(units);
  		    switch (units) {
  		      case 'year':
  		        output = monthDiff(this, that) / 12;
  		        break;
  		      case 'month':
  		        output = monthDiff(this, that);
  		        break;
  		      case 'quarter':
  		        output = monthDiff(this, that) / 3;
  		        break;
  		      case 'second':
  		        output = (this - that) / 1e3;
  		        break;
  		      // 1000
  		      case 'minute':
  		        output = (this - that) / 6e4;
  		        break;
  		      // 1000 * 60
  		      case 'hour':
  		        output = (this - that) / 36e5;
  		        break;
  		      // 1000 * 60 * 60
  		      case 'day':
  		        output = (this - that - zoneDelta) / 864e5;
  		        break;
  		      // 1000 * 60 * 60 * 24, negate dst
  		      case 'week':
  		        output = (this - that - zoneDelta) / 6048e5;
  		        break;
  		      // 1000 * 60 * 60 * 24 * 7, negate dst
  		      default:
  		        output = this - that;
  		    }
  		    return asFloat ? output : absFloor(output);
  		  }
  		  function monthDiff(a, b) {
  		    if (a.date() < b.date()) {
  		      // end-of-month calculations work correct when the start month has more
  		      // days than the end month.
  		      return -monthDiff(b, a);
  		    }
  		    // difference in months
  		    var wholeMonthDiff = (b.year() - a.year()) * 12 + (b.month() - a.month()),
  		      // b is in (anchor - 1 month, anchor + 1 month)
  		      anchor = a.clone().add(wholeMonthDiff, 'months'),
  		      anchor2,
  		      adjust;
  		    if (b - anchor < 0) {
  		      anchor2 = a.clone().add(wholeMonthDiff - 1, 'months');
  		      // linear across the month
  		      adjust = (b - anchor) / (anchor - anchor2);
  		    } else {
  		      anchor2 = a.clone().add(wholeMonthDiff + 1, 'months');
  		      // linear across the month
  		      adjust = (b - anchor) / (anchor2 - anchor);
  		    }

  		    //check for negative zero, return zero if negative zero
  		    return -(wholeMonthDiff + adjust) || 0;
  		  }
  		  hooks.defaultFormat = 'YYYY-MM-DDTHH:mm:ssZ';
  		  hooks.defaultFormatUtc = 'YYYY-MM-DDTHH:mm:ss[Z]';
  		  function toString() {
  		    return this.clone().locale('en').format('ddd MMM DD YYYY HH:mm:ss [GMT]ZZ');
  		  }
  		  function toISOString(keepOffset) {
  		    if (!this.isValid()) {
  		      return null;
  		    }
  		    var utc = keepOffset !== true,
  		      m = utc ? this.clone().utc() : this;
  		    if (m.year() < 0 || m.year() > 9999) {
  		      return formatMoment(m, utc ? 'YYYYYY-MM-DD[T]HH:mm:ss.SSS[Z]' : 'YYYYYY-MM-DD[T]HH:mm:ss.SSSZ');
  		    }
  		    if (isFunction(Date.prototype.toISOString)) {
  		      // native implementation is ~50x faster, use it when we can
  		      if (utc) {
  		        return this.toDate().toISOString();
  		      } else {
  		        return new Date(this.valueOf() + this.utcOffset() * 60 * 1000).toISOString().replace('Z', formatMoment(m, 'Z'));
  		      }
  		    }
  		    return formatMoment(m, utc ? 'YYYY-MM-DD[T]HH:mm:ss.SSS[Z]' : 'YYYY-MM-DD[T]HH:mm:ss.SSSZ');
  		  }

  		  /**
  		   * Return a human readable representation of a moment that can
  		   * also be evaluated to get a new moment which is the same
  		   *
  		   * @link https://nodejs.org/dist/latest/docs/api/util.html#util_custom_inspect_function_on_objects
  		   */
  		  function inspect() {
  		    if (!this.isValid()) {
  		      return 'moment.invalid(/* ' + this._i + ' */)';
  		    }
  		    var func = 'moment',
  		      zone = '',
  		      prefix,
  		      year,
  		      datetime,
  		      suffix;
  		    if (!this.isLocal()) {
  		      func = this.utcOffset() === 0 ? 'moment.utc' : 'moment.parseZone';
  		      zone = 'Z';
  		    }
  		    prefix = '[' + func + '("]';
  		    year = 0 <= this.year() && this.year() <= 9999 ? 'YYYY' : 'YYYYYY';
  		    datetime = '-MM-DD[T]HH:mm:ss.SSS';
  		    suffix = zone + '[")]';
  		    return this.format(prefix + year + datetime + suffix);
  		  }
  		  function format(inputString) {
  		    if (!inputString) {
  		      inputString = this.isUtc() ? hooks.defaultFormatUtc : hooks.defaultFormat;
  		    }
  		    var output = formatMoment(this, inputString);
  		    return this.localeData().postformat(output);
  		  }
  		  function from(time, withoutSuffix) {
  		    if (this.isValid() && (isMoment(time) && time.isValid() || createLocal(time).isValid())) {
  		      return createDuration({
  		        to: this,
  		        from: time
  		      }).locale(this.locale()).humanize(!withoutSuffix);
  		    } else {
  		      return this.localeData().invalidDate();
  		    }
  		  }
  		  function fromNow(withoutSuffix) {
  		    return this.from(createLocal(), withoutSuffix);
  		  }
  		  function to(time, withoutSuffix) {
  		    if (this.isValid() && (isMoment(time) && time.isValid() || createLocal(time).isValid())) {
  		      return createDuration({
  		        from: this,
  		        to: time
  		      }).locale(this.locale()).humanize(!withoutSuffix);
  		    } else {
  		      return this.localeData().invalidDate();
  		    }
  		  }
  		  function toNow(withoutSuffix) {
  		    return this.to(createLocal(), withoutSuffix);
  		  }

  		  // If passed a locale key, it will set the locale for this
  		  // instance.  Otherwise, it will return the locale configuration
  		  // variables for this instance.
  		  function locale(key) {
  		    var newLocaleData;
  		    if (key === undefined) {
  		      return this._locale._abbr;
  		    } else {
  		      newLocaleData = getLocale(key);
  		      if (newLocaleData != null) {
  		        this._locale = newLocaleData;
  		      }
  		      return this;
  		    }
  		  }
  		  var lang = deprecate('moment().lang() is deprecated. Instead, use moment().localeData() to get the language configuration. Use moment().locale() to change languages.', function (key) {
  		    if (key === undefined) {
  		      return this.localeData();
  		    } else {
  		      return this.locale(key);
  		    }
  		  });
  		  function localeData() {
  		    return this._locale;
  		  }
  		  var MS_PER_SECOND = 1000,
  		    MS_PER_MINUTE = 60 * MS_PER_SECOND,
  		    MS_PER_HOUR = 60 * MS_PER_MINUTE,
  		    MS_PER_400_YEARS = (365 * 400 + 97) * 24 * MS_PER_HOUR;

  		  // actual modulo - handles negative numbers (for dates before 1970):
  		  function mod$1(dividend, divisor) {
  		    return (dividend % divisor + divisor) % divisor;
  		  }
  		  function localStartOfDate(y, m, d) {
  		    // the date constructor remaps years 0-99 to 1900-1999
  		    if (y < 100 && y >= 0) {
  		      // preserve leap years using a full 400 year cycle, then reset
  		      return new Date(y + 400, m, d) - MS_PER_400_YEARS;
  		    } else {
  		      return new Date(y, m, d).valueOf();
  		    }
  		  }
  		  function utcStartOfDate(y, m, d) {
  		    // Date.UTC remaps years 0-99 to 1900-1999
  		    if (y < 100 && y >= 0) {
  		      // preserve leap years using a full 400 year cycle, then reset
  		      return Date.UTC(y + 400, m, d) - MS_PER_400_YEARS;
  		    } else {
  		      return Date.UTC(y, m, d);
  		    }
  		  }
  		  function startOf(units) {
  		    var time, startOfDate;
  		    units = normalizeUnits(units);
  		    if (units === undefined || units === 'millisecond' || !this.isValid()) {
  		      return this;
  		    }
  		    startOfDate = this._isUTC ? utcStartOfDate : localStartOfDate;
  		    switch (units) {
  		      case 'year':
  		        time = startOfDate(this.year(), 0, 1);
  		        break;
  		      case 'quarter':
  		        time = startOfDate(this.year(), this.month() - this.month() % 3, 1);
  		        break;
  		      case 'month':
  		        time = startOfDate(this.year(), this.month(), 1);
  		        break;
  		      case 'week':
  		        time = startOfDate(this.year(), this.month(), this.date() - this.weekday());
  		        break;
  		      case 'isoWeek':
  		        time = startOfDate(this.year(), this.month(), this.date() - (this.isoWeekday() - 1));
  		        break;
  		      case 'day':
  		      case 'date':
  		        time = startOfDate(this.year(), this.month(), this.date());
  		        break;
  		      case 'hour':
  		        time = this._d.valueOf();
  		        time -= mod$1(time + (this._isUTC ? 0 : this.utcOffset() * MS_PER_MINUTE), MS_PER_HOUR);
  		        break;
  		      case 'minute':
  		        time = this._d.valueOf();
  		        time -= mod$1(time, MS_PER_MINUTE);
  		        break;
  		      case 'second':
  		        time = this._d.valueOf();
  		        time -= mod$1(time, MS_PER_SECOND);
  		        break;
  		    }
  		    this._d.setTime(time);
  		    hooks.updateOffset(this, true);
  		    return this;
  		  }
  		  function endOf(units) {
  		    var time, startOfDate;
  		    units = normalizeUnits(units);
  		    if (units === undefined || units === 'millisecond' || !this.isValid()) {
  		      return this;
  		    }
  		    startOfDate = this._isUTC ? utcStartOfDate : localStartOfDate;
  		    switch (units) {
  		      case 'year':
  		        time = startOfDate(this.year() + 1, 0, 1) - 1;
  		        break;
  		      case 'quarter':
  		        time = startOfDate(this.year(), this.month() - this.month() % 3 + 3, 1) - 1;
  		        break;
  		      case 'month':
  		        time = startOfDate(this.year(), this.month() + 1, 1) - 1;
  		        break;
  		      case 'week':
  		        time = startOfDate(this.year(), this.month(), this.date() - this.weekday() + 7) - 1;
  		        break;
  		      case 'isoWeek':
  		        time = startOfDate(this.year(), this.month(), this.date() - (this.isoWeekday() - 1) + 7) - 1;
  		        break;
  		      case 'day':
  		      case 'date':
  		        time = startOfDate(this.year(), this.month(), this.date() + 1) - 1;
  		        break;
  		      case 'hour':
  		        time = this._d.valueOf();
  		        time += MS_PER_HOUR - mod$1(time + (this._isUTC ? 0 : this.utcOffset() * MS_PER_MINUTE), MS_PER_HOUR) - 1;
  		        break;
  		      case 'minute':
  		        time = this._d.valueOf();
  		        time += MS_PER_MINUTE - mod$1(time, MS_PER_MINUTE) - 1;
  		        break;
  		      case 'second':
  		        time = this._d.valueOf();
  		        time += MS_PER_SECOND - mod$1(time, MS_PER_SECOND) - 1;
  		        break;
  		    }
  		    this._d.setTime(time);
  		    hooks.updateOffset(this, true);
  		    return this;
  		  }
  		  function valueOf() {
  		    return this._d.valueOf() - (this._offset || 0) * 60000;
  		  }
  		  function unix() {
  		    return Math.floor(this.valueOf() / 1000);
  		  }
  		  function toDate() {
  		    return new Date(this.valueOf());
  		  }
  		  function toArray() {
  		    var m = this;
  		    return [m.year(), m.month(), m.date(), m.hour(), m.minute(), m.second(), m.millisecond()];
  		  }
  		  function toObject() {
  		    var m = this;
  		    return {
  		      years: m.year(),
  		      months: m.month(),
  		      date: m.date(),
  		      hours: m.hours(),
  		      minutes: m.minutes(),
  		      seconds: m.seconds(),
  		      milliseconds: m.milliseconds()
  		    };
  		  }
  		  function toJSON() {
  		    // new Date(NaN).toJSON() === null
  		    return this.isValid() ? this.toISOString() : null;
  		  }
  		  function isValid$2() {
  		    return isValid(this);
  		  }
  		  function parsingFlags() {
  		    return extend({}, getParsingFlags(this));
  		  }
  		  function invalidAt() {
  		    return getParsingFlags(this).overflow;
  		  }
  		  function creationData() {
  		    return {
  		      input: this._i,
  		      format: this._f,
  		      locale: this._locale,
  		      isUTC: this._isUTC,
  		      strict: this._strict
  		    };
  		  }
  		  addFormatToken('N', 0, 0, 'eraAbbr');
  		  addFormatToken('NN', 0, 0, 'eraAbbr');
  		  addFormatToken('NNN', 0, 0, 'eraAbbr');
  		  addFormatToken('NNNN', 0, 0, 'eraName');
  		  addFormatToken('NNNNN', 0, 0, 'eraNarrow');
  		  addFormatToken('y', ['y', 1], 'yo', 'eraYear');
  		  addFormatToken('y', ['yy', 2], 0, 'eraYear');
  		  addFormatToken('y', ['yyy', 3], 0, 'eraYear');
  		  addFormatToken('y', ['yyyy', 4], 0, 'eraYear');
  		  addRegexToken('N', matchEraAbbr);
  		  addRegexToken('NN', matchEraAbbr);
  		  addRegexToken('NNN', matchEraAbbr);
  		  addRegexToken('NNNN', matchEraName);
  		  addRegexToken('NNNNN', matchEraNarrow);
  		  addParseToken(['N', 'NN', 'NNN', 'NNNN', 'NNNNN'], function (input, array, config, token) {
  		    var era = config._locale.erasParse(input, token, config._strict);
  		    if (era) {
  		      getParsingFlags(config).era = era;
  		    } else {
  		      getParsingFlags(config).invalidEra = input;
  		    }
  		  });
  		  addRegexToken('y', matchUnsigned);
  		  addRegexToken('yy', matchUnsigned);
  		  addRegexToken('yyy', matchUnsigned);
  		  addRegexToken('yyyy', matchUnsigned);
  		  addRegexToken('yo', matchEraYearOrdinal);
  		  addParseToken(['y', 'yy', 'yyy', 'yyyy'], YEAR);
  		  addParseToken(['yo'], function (input, array, config, token) {
  		    var match;
  		    if (config._locale._eraYearOrdinalRegex) {
  		      match = input.match(config._locale._eraYearOrdinalRegex);
  		    }
  		    if (config._locale.eraYearOrdinalParse) {
  		      array[YEAR] = config._locale.eraYearOrdinalParse(input, match);
  		    } else {
  		      array[YEAR] = parseInt(input, 10);
  		    }
  		  });
  		  function localeEras(m, format) {
  		    var i,
  		      l,
  		      date,
  		      eras = this._eras || getLocale('en')._eras;
  		    for (i = 0, l = eras.length; i < l; ++i) {
  		      switch (typeof eras[i].since) {
  		        case 'string':
  		          // truncate time
  		          date = hooks(eras[i].since).startOf('day');
  		          eras[i].since = date.valueOf();
  		          break;
  		      }
  		      switch (typeof eras[i].until) {
  		        case 'undefined':
  		          eras[i].until = +Infinity;
  		          break;
  		        case 'string':
  		          // truncate time
  		          date = hooks(eras[i].until).startOf('day').valueOf();
  		          eras[i].until = date.valueOf();
  		          break;
  		      }
  		    }
  		    return eras;
  		  }
  		  function localeErasParse(eraName, format, strict) {
  		    var i,
  		      l,
  		      eras = this.eras(),
  		      name,
  		      abbr,
  		      narrow;
  		    eraName = eraName.toUpperCase();
  		    for (i = 0, l = eras.length; i < l; ++i) {
  		      name = eras[i].name.toUpperCase();
  		      abbr = eras[i].abbr.toUpperCase();
  		      narrow = eras[i].narrow.toUpperCase();
  		      if (strict) {
  		        switch (format) {
  		          case 'N':
  		          case 'NN':
  		          case 'NNN':
  		            if (abbr === eraName) {
  		              return eras[i];
  		            }
  		            break;
  		          case 'NNNN':
  		            if (name === eraName) {
  		              return eras[i];
  		            }
  		            break;
  		          case 'NNNNN':
  		            if (narrow === eraName) {
  		              return eras[i];
  		            }
  		            break;
  		        }
  		      } else if ([name, abbr, narrow].indexOf(eraName) >= 0) {
  		        return eras[i];
  		      }
  		    }
  		  }
  		  function localeErasConvertYear(era, year) {
  		    var dir = era.since <= era.until ? +1 : -1;
  		    if (year === undefined) {
  		      return hooks(era.since).year();
  		    } else {
  		      return hooks(era.since).year() + (year - era.offset) * dir;
  		    }
  		  }
  		  function getEraName() {
  		    var i,
  		      l,
  		      val,
  		      eras = this.localeData().eras();
  		    for (i = 0, l = eras.length; i < l; ++i) {
  		      // truncate time
  		      val = this.clone().startOf('day').valueOf();
  		      if (eras[i].since <= val && val <= eras[i].until) {
  		        return eras[i].name;
  		      }
  		      if (eras[i].until <= val && val <= eras[i].since) {
  		        return eras[i].name;
  		      }
  		    }
  		    return '';
  		  }
  		  function getEraNarrow() {
  		    var i,
  		      l,
  		      val,
  		      eras = this.localeData().eras();
  		    for (i = 0, l = eras.length; i < l; ++i) {
  		      // truncate time
  		      val = this.clone().startOf('day').valueOf();
  		      if (eras[i].since <= val && val <= eras[i].until) {
  		        return eras[i].narrow;
  		      }
  		      if (eras[i].until <= val && val <= eras[i].since) {
  		        return eras[i].narrow;
  		      }
  		    }
  		    return '';
  		  }
  		  function getEraAbbr() {
  		    var i,
  		      l,
  		      val,
  		      eras = this.localeData().eras();
  		    for (i = 0, l = eras.length; i < l; ++i) {
  		      // truncate time
  		      val = this.clone().startOf('day').valueOf();
  		      if (eras[i].since <= val && val <= eras[i].until) {
  		        return eras[i].abbr;
  		      }
  		      if (eras[i].until <= val && val <= eras[i].since) {
  		        return eras[i].abbr;
  		      }
  		    }
  		    return '';
  		  }
  		  function getEraYear() {
  		    var i,
  		      l,
  		      dir,
  		      val,
  		      eras = this.localeData().eras();
  		    for (i = 0, l = eras.length; i < l; ++i) {
  		      dir = eras[i].since <= eras[i].until ? +1 : -1;

  		      // truncate time
  		      val = this.clone().startOf('day').valueOf();
  		      if (eras[i].since <= val && val <= eras[i].until || eras[i].until <= val && val <= eras[i].since) {
  		        return (this.year() - hooks(eras[i].since).year()) * dir + eras[i].offset;
  		      }
  		    }
  		    return this.year();
  		  }
  		  function erasNameRegex(isStrict) {
  		    if (!hasOwnProp(this, '_erasNameRegex')) {
  		      computeErasParse.call(this);
  		    }
  		    return isStrict ? this._erasNameRegex : this._erasRegex;
  		  }
  		  function erasAbbrRegex(isStrict) {
  		    if (!hasOwnProp(this, '_erasAbbrRegex')) {
  		      computeErasParse.call(this);
  		    }
  		    return isStrict ? this._erasAbbrRegex : this._erasRegex;
  		  }
  		  function erasNarrowRegex(isStrict) {
  		    if (!hasOwnProp(this, '_erasNarrowRegex')) {
  		      computeErasParse.call(this);
  		    }
  		    return isStrict ? this._erasNarrowRegex : this._erasRegex;
  		  }
  		  function matchEraAbbr(isStrict, locale) {
  		    return locale.erasAbbrRegex(isStrict);
  		  }
  		  function matchEraName(isStrict, locale) {
  		    return locale.erasNameRegex(isStrict);
  		  }
  		  function matchEraNarrow(isStrict, locale) {
  		    return locale.erasNarrowRegex(isStrict);
  		  }
  		  function matchEraYearOrdinal(isStrict, locale) {
  		    return locale._eraYearOrdinalRegex || matchUnsigned;
  		  }
  		  function computeErasParse() {
  		    var abbrPieces = [],
  		      namePieces = [],
  		      narrowPieces = [],
  		      mixedPieces = [],
  		      i,
  		      l,
  		      erasName,
  		      erasAbbr,
  		      erasNarrow,
  		      eras = this.eras();
  		    for (i = 0, l = eras.length; i < l; ++i) {
  		      erasName = regexEscape(eras[i].name);
  		      erasAbbr = regexEscape(eras[i].abbr);
  		      erasNarrow = regexEscape(eras[i].narrow);
  		      namePieces.push(erasName);
  		      abbrPieces.push(erasAbbr);
  		      narrowPieces.push(erasNarrow);
  		      mixedPieces.push(erasName);
  		      mixedPieces.push(erasAbbr);
  		      mixedPieces.push(erasNarrow);
  		    }
  		    this._erasRegex = new RegExp('^(' + mixedPieces.join('|') + ')', 'i');
  		    this._erasNameRegex = new RegExp('^(' + namePieces.join('|') + ')', 'i');
  		    this._erasAbbrRegex = new RegExp('^(' + abbrPieces.join('|') + ')', 'i');
  		    this._erasNarrowRegex = new RegExp('^(' + narrowPieces.join('|') + ')', 'i');
  		  }

  		  // FORMATTING

  		  addFormatToken(0, ['gg', 2], 0, function () {
  		    return this.weekYear() % 100;
  		  });
  		  addFormatToken(0, ['GG', 2], 0, function () {
  		    return this.isoWeekYear() % 100;
  		  });
  		  function addWeekYearFormatToken(token, getter) {
  		    addFormatToken(0, [token, token.length], 0, getter);
  		  }
  		  addWeekYearFormatToken('gggg', 'weekYear');
  		  addWeekYearFormatToken('ggggg', 'weekYear');
  		  addWeekYearFormatToken('GGGG', 'isoWeekYear');
  		  addWeekYearFormatToken('GGGGG', 'isoWeekYear');

  		  // ALIASES

  		  // PARSING

  		  addRegexToken('G', matchSigned);
  		  addRegexToken('g', matchSigned);
  		  addRegexToken('GG', match1to2, match2);
  		  addRegexToken('gg', match1to2, match2);
  		  addRegexToken('GGGG', match1to4, match4);
  		  addRegexToken('gggg', match1to4, match4);
  		  addRegexToken('GGGGG', match1to6, match6);
  		  addRegexToken('ggggg', match1to6, match6);
  		  addWeekParseToken(['gggg', 'ggggg', 'GGGG', 'GGGGG'], function (input, week, config, token) {
  		    week[token.substr(0, 2)] = toInt(input);
  		  });
  		  addWeekParseToken(['gg', 'GG'], function (input, week, config, token) {
  		    week[token] = hooks.parseTwoDigitYear(input);
  		  });

  		  // MOMENTS

  		  function getSetWeekYear(input) {
  		    return getSetWeekYearHelper.call(this, input, this.week(), this.weekday() + this.localeData()._week.dow, this.localeData()._week.dow, this.localeData()._week.doy);
  		  }
  		  function getSetISOWeekYear(input) {
  		    return getSetWeekYearHelper.call(this, input, this.isoWeek(), this.isoWeekday(), 1, 4);
  		  }
  		  function getISOWeeksInYear() {
  		    return weeksInYear(this.year(), 1, 4);
  		  }
  		  function getISOWeeksInISOWeekYear() {
  		    return weeksInYear(this.isoWeekYear(), 1, 4);
  		  }
  		  function getWeeksInYear() {
  		    var weekInfo = this.localeData()._week;
  		    return weeksInYear(this.year(), weekInfo.dow, weekInfo.doy);
  		  }
  		  function getWeeksInWeekYear() {
  		    var weekInfo = this.localeData()._week;
  		    return weeksInYear(this.weekYear(), weekInfo.dow, weekInfo.doy);
  		  }
  		  function getSetWeekYearHelper(input, week, weekday, dow, doy) {
  		    var weeksTarget;
  		    if (input == null) {
  		      return weekOfYear(this, dow, doy).year;
  		    } else {
  		      weeksTarget = weeksInYear(input, dow, doy);
  		      if (week > weeksTarget) {
  		        week = weeksTarget;
  		      }
  		      return setWeekAll.call(this, input, week, weekday, dow, doy);
  		    }
  		  }
  		  function setWeekAll(weekYear, week, weekday, dow, doy) {
  		    var dayOfYearData = dayOfYearFromWeeks(weekYear, week, weekday, dow, doy),
  		      date = createUTCDate(dayOfYearData.year, 0, dayOfYearData.dayOfYear);
  		    this.year(date.getUTCFullYear());
  		    this.month(date.getUTCMonth());
  		    this.date(date.getUTCDate());
  		    return this;
  		  }

  		  // FORMATTING

  		  addFormatToken('Q', 0, 'Qo', 'quarter');

  		  // PARSING

  		  addRegexToken('Q', match1);
  		  addParseToken('Q', function (input, array) {
  		    array[MONTH] = (toInt(input) - 1) * 3;
  		  });

  		  // MOMENTS

  		  function getSetQuarter(input) {
  		    return input == null ? Math.ceil((this.month() + 1) / 3) : this.month((input - 1) * 3 + this.month() % 3);
  		  }

  		  // FORMATTING

  		  addFormatToken('D', ['DD', 2], 'Do', 'date');

  		  // PARSING

  		  addRegexToken('D', match1to2, match1to2NoLeadingZero);
  		  addRegexToken('DD', match1to2, match2);
  		  addRegexToken('Do', function (isStrict, locale) {
  		    // TODO: Remove "ordinalParse" fallback in next major release.
  		    return isStrict ? locale._dayOfMonthOrdinalParse || locale._ordinalParse : locale._dayOfMonthOrdinalParseLenient;
  		  });
  		  addParseToken(['D', 'DD'], DATE);
  		  addParseToken('Do', function (input, array) {
  		    array[DATE] = toInt(input.match(match1to2)[0]);
  		  });

  		  // MOMENTS

  		  var getSetDayOfMonth = makeGetSet('Date', true);

  		  // FORMATTING

  		  addFormatToken('DDD', ['DDDD', 3], 'DDDo', 'dayOfYear');

  		  // PARSING

  		  addRegexToken('DDD', match1to3);
  		  addRegexToken('DDDD', match3);
  		  addParseToken(['DDD', 'DDDD'], function (input, array, config) {
  		    config._dayOfYear = toInt(input);
  		  });

  		  // HELPERS

  		  // MOMENTS

  		  function getSetDayOfYear(input) {
  		    var dayOfYear = Math.round((this.clone().startOf('day') - this.clone().startOf('year')) / 864e5) + 1;
  		    return input == null ? dayOfYear : this.add(input - dayOfYear, 'd');
  		  }

  		  // FORMATTING

  		  addFormatToken('m', ['mm', 2], 0, 'minute');

  		  // PARSING

  		  addRegexToken('m', match1to2, match1to2HasZero);
  		  addRegexToken('mm', match1to2, match2);
  		  addParseToken(['m', 'mm'], MINUTE);

  		  // MOMENTS

  		  var getSetMinute = makeGetSet('Minutes', false);

  		  // FORMATTING

  		  addFormatToken('s', ['ss', 2], 0, 'second');

  		  // PARSING

  		  addRegexToken('s', match1to2, match1to2HasZero);
  		  addRegexToken('ss', match1to2, match2);
  		  addParseToken(['s', 'ss'], SECOND);

  		  // MOMENTS

  		  var getSetSecond = makeGetSet('Seconds', false);

  		  // FORMATTING

  		  addFormatToken('S', 0, 0, function () {
  		    return ~~(this.millisecond() / 100);
  		  });
  		  addFormatToken(0, ['SS', 2], 0, function () {
  		    return ~~(this.millisecond() / 10);
  		  });
  		  addFormatToken(0, ['SSS', 3], 0, 'millisecond');
  		  addFormatToken(0, ['SSSS', 4], 0, function () {
  		    return this.millisecond() * 10;
  		  });
  		  addFormatToken(0, ['SSSSS', 5], 0, function () {
  		    return this.millisecond() * 100;
  		  });
  		  addFormatToken(0, ['SSSSSS', 6], 0, function () {
  		    return this.millisecond() * 1000;
  		  });
  		  addFormatToken(0, ['SSSSSSS', 7], 0, function () {
  		    return this.millisecond() * 10000;
  		  });
  		  addFormatToken(0, ['SSSSSSSS', 8], 0, function () {
  		    return this.millisecond() * 100000;
  		  });
  		  addFormatToken(0, ['SSSSSSSSS', 9], 0, function () {
  		    return this.millisecond() * 1000000;
  		  });

  		  // PARSING

  		  addRegexToken('S', match1to3, match1);
  		  addRegexToken('SS', match1to3, match2);
  		  addRegexToken('SSS', match1to3, match3);
  		  var token, getSetMillisecond;
  		  for (token = 'SSSS'; token.length <= 9; token += 'S') {
  		    addRegexToken(token, matchUnsigned);
  		  }
  		  function parseMs(input, array) {
  		    array[MILLISECOND] = toInt(('0.' + input) * 1000);
  		  }
  		  for (token = 'S'; token.length <= 9; token += 'S') {
  		    addParseToken(token, parseMs);
  		  }
  		  getSetMillisecond = makeGetSet('Milliseconds', false);

  		  // FORMATTING

  		  addFormatToken('z', 0, 0, 'zoneAbbr');
  		  addFormatToken('zz', 0, 0, 'zoneName');

  		  // MOMENTS

  		  function getZoneAbbr() {
  		    return this._isUTC ? 'UTC' : '';
  		  }
  		  function getZoneName() {
  		    return this._isUTC ? 'Coordinated Universal Time' : '';
  		  }
  		  var proto = Moment.prototype;
  		  proto.add = add;
  		  proto.calendar = calendar$1;
  		  proto.clone = clone;
  		  proto.diff = diff;
  		  proto.endOf = endOf;
  		  proto.format = format;
  		  proto.from = from;
  		  proto.fromNow = fromNow;
  		  proto.to = to;
  		  proto.toNow = toNow;
  		  proto.get = stringGet;
  		  proto.invalidAt = invalidAt;
  		  proto.isAfter = isAfter;
  		  proto.isBefore = isBefore;
  		  proto.isBetween = isBetween;
  		  proto.isSame = isSame;
  		  proto.isSameOrAfter = isSameOrAfter;
  		  proto.isSameOrBefore = isSameOrBefore;
  		  proto.isValid = isValid$2;
  		  proto.lang = lang;
  		  proto.locale = locale;
  		  proto.localeData = localeData;
  		  proto.max = prototypeMax;
  		  proto.min = prototypeMin;
  		  proto.parsingFlags = parsingFlags;
  		  proto.set = stringSet;
  		  proto.startOf = startOf;
  		  proto.subtract = subtract;
  		  proto.toArray = toArray;
  		  proto.toObject = toObject;
  		  proto.toDate = toDate;
  		  proto.toISOString = toISOString;
  		  proto.inspect = inspect;
  		  if (typeof Symbol !== 'undefined' && Symbol.for != null) {
  		    proto[Symbol.for('nodejs.util.inspect.custom')] = function () {
  		      return 'Moment<' + this.format() + '>';
  		    };
  		  }
  		  proto.toJSON = toJSON;
  		  proto.toString = toString;
  		  proto.unix = unix;
  		  proto.valueOf = valueOf;
  		  proto.creationData = creationData;
  		  proto.eraName = getEraName;
  		  proto.eraNarrow = getEraNarrow;
  		  proto.eraAbbr = getEraAbbr;
  		  proto.eraYear = getEraYear;
  		  proto.year = getSetYear;
  		  proto.isLeapYear = getIsLeapYear;
  		  proto.weekYear = getSetWeekYear;
  		  proto.isoWeekYear = getSetISOWeekYear;
  		  proto.quarter = proto.quarters = getSetQuarter;
  		  proto.month = getSetMonth;
  		  proto.daysInMonth = getDaysInMonth;
  		  proto.week = proto.weeks = getSetWeek;
  		  proto.isoWeek = proto.isoWeeks = getSetISOWeek;
  		  proto.weeksInYear = getWeeksInYear;
  		  proto.weeksInWeekYear = getWeeksInWeekYear;
  		  proto.isoWeeksInYear = getISOWeeksInYear;
  		  proto.isoWeeksInISOWeekYear = getISOWeeksInISOWeekYear;
  		  proto.date = getSetDayOfMonth;
  		  proto.day = proto.days = getSetDayOfWeek;
  		  proto.weekday = getSetLocaleDayOfWeek;
  		  proto.isoWeekday = getSetISODayOfWeek;
  		  proto.dayOfYear = getSetDayOfYear;
  		  proto.hour = proto.hours = getSetHour;
  		  proto.minute = proto.minutes = getSetMinute;
  		  proto.second = proto.seconds = getSetSecond;
  		  proto.millisecond = proto.milliseconds = getSetMillisecond;
  		  proto.utcOffset = getSetOffset;
  		  proto.utc = setOffsetToUTC;
  		  proto.local = setOffsetToLocal;
  		  proto.parseZone = setOffsetToParsedOffset;
  		  proto.hasAlignedHourOffset = hasAlignedHourOffset;
  		  proto.isDST = isDaylightSavingTime;
  		  proto.isLocal = isLocal;
  		  proto.isUtcOffset = isUtcOffset;
  		  proto.isUtc = isUtc;
  		  proto.isUTC = isUtc;
  		  proto.zoneAbbr = getZoneAbbr;
  		  proto.zoneName = getZoneName;
  		  proto.dates = deprecate('dates accessor is deprecated. Use date instead.', getSetDayOfMonth);
  		  proto.months = deprecate('months accessor is deprecated. Use month instead', getSetMonth);
  		  proto.years = deprecate('years accessor is deprecated. Use year instead', getSetYear);
  		  proto.zone = deprecate('moment().zone is deprecated, use moment().utcOffset instead. http://momentjs.com/guides/#/warnings/zone/', getSetZone);
  		  proto.isDSTShifted = deprecate('isDSTShifted is deprecated. See http://momentjs.com/guides/#/warnings/dst-shifted/ for more information', isDaylightSavingTimeShifted);
  		  function createUnix(input) {
  		    return createLocal(input * 1000);
  		  }
  		  function createInZone() {
  		    return createLocal.apply(null, arguments).parseZone();
  		  }
  		  function preParsePostFormat(string) {
  		    return string;
  		  }
  		  var proto$1 = Locale.prototype;
  		  proto$1.calendar = calendar;
  		  proto$1.longDateFormat = longDateFormat;
  		  proto$1.invalidDate = invalidDate;
  		  proto$1.ordinal = ordinal;
  		  proto$1.preparse = preParsePostFormat;
  		  proto$1.postformat = preParsePostFormat;
  		  proto$1.relativeTime = relativeTime;
  		  proto$1.pastFuture = pastFuture;
  		  proto$1.set = set;
  		  proto$1.eras = localeEras;
  		  proto$1.erasParse = localeErasParse;
  		  proto$1.erasConvertYear = localeErasConvertYear;
  		  proto$1.erasAbbrRegex = erasAbbrRegex;
  		  proto$1.erasNameRegex = erasNameRegex;
  		  proto$1.erasNarrowRegex = erasNarrowRegex;
  		  proto$1.months = localeMonths;
  		  proto$1.monthsShort = localeMonthsShort;
  		  proto$1.monthsParse = localeMonthsParse;
  		  proto$1.monthsRegex = monthsRegex;
  		  proto$1.monthsShortRegex = monthsShortRegex;
  		  proto$1.week = localeWeek;
  		  proto$1.firstDayOfYear = localeFirstDayOfYear;
  		  proto$1.firstDayOfWeek = localeFirstDayOfWeek;
  		  proto$1.weekdays = localeWeekdays;
  		  proto$1.weekdaysMin = localeWeekdaysMin;
  		  proto$1.weekdaysShort = localeWeekdaysShort;
  		  proto$1.weekdaysParse = localeWeekdaysParse;
  		  proto$1.weekdaysRegex = weekdaysRegex;
  		  proto$1.weekdaysShortRegex = weekdaysShortRegex;
  		  proto$1.weekdaysMinRegex = weekdaysMinRegex;
  		  proto$1.isPM = localeIsPM;
  		  proto$1.meridiem = localeMeridiem;
  		  function get$1(format, index, field, setter) {
  		    var locale = getLocale(),
  		      utc = createUTC().set(setter, index);
  		    return locale[field](utc, format);
  		  }
  		  function listMonthsImpl(format, index, field) {
  		    if (isNumber(format)) {
  		      index = format;
  		      format = undefined;
  		    }
  		    format = format || '';
  		    if (index != null) {
  		      return get$1(format, index, field, 'month');
  		    }
  		    var i,
  		      out = [];
  		    for (i = 0; i < 12; i++) {
  		      out[i] = get$1(format, i, field, 'month');
  		    }
  		    return out;
  		  }

  		  // ()
  		  // (5)
  		  // (fmt, 5)
  		  // (fmt)
  		  // (true)
  		  // (true, 5)
  		  // (true, fmt, 5)
  		  // (true, fmt)
  		  function listWeekdaysImpl(localeSorted, format, index, field) {
  		    if (typeof localeSorted === 'boolean') {
  		      if (isNumber(format)) {
  		        index = format;
  		        format = undefined;
  		      }
  		      format = format || '';
  		    } else {
  		      format = localeSorted;
  		      index = format;
  		      localeSorted = false;
  		      if (isNumber(format)) {
  		        index = format;
  		        format = undefined;
  		      }
  		      format = format || '';
  		    }
  		    var locale = getLocale(),
  		      shift = localeSorted ? locale._week.dow : 0,
  		      i,
  		      out = [];
  		    if (index != null) {
  		      return get$1(format, (index + shift) % 7, field, 'day');
  		    }
  		    for (i = 0; i < 7; i++) {
  		      out[i] = get$1(format, (i + shift) % 7, field, 'day');
  		    }
  		    return out;
  		  }
  		  function listMonths(format, index) {
  		    return listMonthsImpl(format, index, 'months');
  		  }
  		  function listMonthsShort(format, index) {
  		    return listMonthsImpl(format, index, 'monthsShort');
  		  }
  		  function listWeekdays(localeSorted, format, index) {
  		    return listWeekdaysImpl(localeSorted, format, index, 'weekdays');
  		  }
  		  function listWeekdaysShort(localeSorted, format, index) {
  		    return listWeekdaysImpl(localeSorted, format, index, 'weekdaysShort');
  		  }
  		  function listWeekdaysMin(localeSorted, format, index) {
  		    return listWeekdaysImpl(localeSorted, format, index, 'weekdaysMin');
  		  }
  		  getSetGlobalLocale('en', {
  		    eras: [{
  		      since: '0001-01-01',
  		      until: +Infinity,
  		      offset: 1,
  		      name: 'Anno Domini',
  		      narrow: 'AD',
  		      abbr: 'AD'
  		    }, {
  		      since: '0000-12-31',
  		      until: -Infinity,
  		      offset: 1,
  		      name: 'Before Christ',
  		      narrow: 'BC',
  		      abbr: 'BC'
  		    }],
  		    dayOfMonthOrdinalParse: /\d{1,2}(th|st|nd|rd)/,
  		    ordinal: function (number) {
  		      var b = number % 10,
  		        output = toInt(number % 100 / 10) === 1 ? 'th' : b === 1 ? 'st' : b === 2 ? 'nd' : b === 3 ? 'rd' : 'th';
  		      return number + output;
  		    }
  		  });

  		  // Side effect imports

  		  hooks.lang = deprecate('moment.lang is deprecated. Use moment.locale instead.', getSetGlobalLocale);
  		  hooks.langData = deprecate('moment.langData is deprecated. Use moment.localeData instead.', getLocale);
  		  var mathAbs = Math.abs;
  		  function abs() {
  		    var data = this._data;
  		    this._milliseconds = mathAbs(this._milliseconds);
  		    this._days = mathAbs(this._days);
  		    this._months = mathAbs(this._months);
  		    data.milliseconds = mathAbs(data.milliseconds);
  		    data.seconds = mathAbs(data.seconds);
  		    data.minutes = mathAbs(data.minutes);
  		    data.hours = mathAbs(data.hours);
  		    data.months = mathAbs(data.months);
  		    data.years = mathAbs(data.years);
  		    return this;
  		  }
  		  function addSubtract$1(duration, input, value, direction) {
  		    var other = createDuration(input, value);
  		    duration._milliseconds += direction * other._milliseconds;
  		    duration._days += direction * other._days;
  		    duration._months += direction * other._months;
  		    return duration._bubble();
  		  }

  		  // supports only 2.0-style add(1, 's') or add(duration)
  		  function add$1(input, value) {
  		    return addSubtract$1(this, input, value, 1);
  		  }

  		  // supports only 2.0-style subtract(1, 's') or subtract(duration)
  		  function subtract$1(input, value) {
  		    return addSubtract$1(this, input, value, -1);
  		  }
  		  function absCeil(number) {
  		    if (number < 0) {
  		      return Math.floor(number);
  		    } else {
  		      return Math.ceil(number);
  		    }
  		  }
  		  function bubble() {
  		    var milliseconds = this._milliseconds,
  		      days = this._days,
  		      months = this._months,
  		      data = this._data,
  		      seconds,
  		      minutes,
  		      hours,
  		      years,
  		      monthsFromDays;

  		    // if we have a mix of positive and negative values, bubble down first
  		    // check: https://github.com/moment/moment/issues/2166
  		    if (!(milliseconds >= 0 && days >= 0 && months >= 0 || milliseconds <= 0 && days <= 0 && months <= 0)) {
  		      milliseconds += absCeil(monthsToDays(months) + days) * 864e5;
  		      days = 0;
  		      months = 0;
  		    }

  		    // The following code bubbles up values, see the tests for
  		    // examples of what that means.
  		    data.milliseconds = milliseconds % 1000;
  		    seconds = absFloor(milliseconds / 1000);
  		    data.seconds = seconds % 60;
  		    minutes = absFloor(seconds / 60);
  		    data.minutes = minutes % 60;
  		    hours = absFloor(minutes / 60);
  		    data.hours = hours % 24;
  		    days += absFloor(hours / 24);

  		    // convert days to months
  		    monthsFromDays = absFloor(daysToMonths(days));
  		    months += monthsFromDays;
  		    days -= absCeil(monthsToDays(monthsFromDays));

  		    // 12 months -> 1 year
  		    years = absFloor(months / 12);
  		    months %= 12;
  		    data.days = days;
  		    data.months = months;
  		    data.years = years;
  		    return this;
  		  }
  		  function daysToMonths(days) {
  		    // 400 years have 146097 days (taking into account leap year rules)
  		    // 400 years have 12 months === 4800
  		    return days * 4800 / 146097;
  		  }
  		  function monthsToDays(months) {
  		    // the reverse of daysToMonths
  		    return months * 146097 / 4800;
  		  }
  		  function as(units) {
  		    if (!this.isValid()) {
  		      return NaN;
  		    }
  		    var days,
  		      months,
  		      milliseconds = this._milliseconds;
  		    units = normalizeUnits(units);
  		    if (units === 'month' || units === 'quarter' || units === 'year') {
  		      days = this._days + milliseconds / 864e5;
  		      months = this._months + daysToMonths(days);
  		      switch (units) {
  		        case 'month':
  		          return months;
  		        case 'quarter':
  		          return months / 3;
  		        case 'year':
  		          return months / 12;
  		      }
  		    } else {
  		      // handle milliseconds separately because of floating point math errors (issue #1867)
  		      days = this._days + Math.round(monthsToDays(this._months));
  		      switch (units) {
  		        case 'week':
  		          return days / 7 + milliseconds / 6048e5;
  		        case 'day':
  		          return days + milliseconds / 864e5;
  		        case 'hour':
  		          return days * 24 + milliseconds / 36e5;
  		        case 'minute':
  		          return days * 1440 + milliseconds / 6e4;
  		        case 'second':
  		          return days * 86400 + milliseconds / 1000;
  		        // Math.floor prevents floating point math errors here
  		        case 'millisecond':
  		          return Math.floor(days * 864e5) + milliseconds;
  		        default:
  		          throw new Error('Unknown unit ' + units);
  		      }
  		    }
  		  }
  		  function makeAs(alias) {
  		    return function () {
  		      return this.as(alias);
  		    };
  		  }
  		  var asMilliseconds = makeAs('ms'),
  		    asSeconds = makeAs('s'),
  		    asMinutes = makeAs('m'),
  		    asHours = makeAs('h'),
  		    asDays = makeAs('d'),
  		    asWeeks = makeAs('w'),
  		    asMonths = makeAs('M'),
  		    asQuarters = makeAs('Q'),
  		    asYears = makeAs('y'),
  		    valueOf$1 = asMilliseconds;
  		  function clone$1() {
  		    return createDuration(this);
  		  }
  		  function get$2(units) {
  		    units = normalizeUnits(units);
  		    return this.isValid() ? this[units + 's']() : NaN;
  		  }
  		  function makeGetter(name) {
  		    return function () {
  		      return this.isValid() ? this._data[name] : NaN;
  		    };
  		  }
  		  var milliseconds = makeGetter('milliseconds'),
  		    seconds = makeGetter('seconds'),
  		    minutes = makeGetter('minutes'),
  		    hours = makeGetter('hours'),
  		    days = makeGetter('days'),
  		    months = makeGetter('months'),
  		    years = makeGetter('years');
  		  function weeks() {
  		    return absFloor(this.days() / 7);
  		  }
  		  var round = Math.round,
  		    thresholds = {
  		      ss: 44,
  		      // a few seconds to seconds
  		      s: 45,
  		      // seconds to minute
  		      m: 45,
  		      // minutes to hour
  		      h: 22,
  		      // hours to day
  		      d: 26,
  		      // days to month/week
  		      w: null,
  		      // weeks to month
  		      M: 11 // months to year
  		    };

  		  // helper function for moment.fn.from, moment.fn.fromNow, and moment.duration.fn.humanize
  		  function substituteTimeAgo(string, number, withoutSuffix, isFuture, locale) {
  		    return locale.relativeTime(number || 1, !!withoutSuffix, string, isFuture);
  		  }
  		  function relativeTime$1(posNegDuration, withoutSuffix, thresholds, locale) {
  		    var duration = createDuration(posNegDuration).abs(),
  		      seconds = round(duration.as('s')),
  		      minutes = round(duration.as('m')),
  		      hours = round(duration.as('h')),
  		      days = round(duration.as('d')),
  		      months = round(duration.as('M')),
  		      weeks = round(duration.as('w')),
  		      years = round(duration.as('y')),
  		      a = seconds <= thresholds.ss && ['s', seconds] || seconds < thresholds.s && ['ss', seconds] || minutes <= 1 && ['m'] || minutes < thresholds.m && ['mm', minutes] || hours <= 1 && ['h'] || hours < thresholds.h && ['hh', hours] || days <= 1 && ['d'] || days < thresholds.d && ['dd', days];
  		    if (thresholds.w != null) {
  		      a = a || weeks <= 1 && ['w'] || weeks < thresholds.w && ['ww', weeks];
  		    }
  		    a = a || months <= 1 && ['M'] || months < thresholds.M && ['MM', months] || years <= 1 && ['y'] || ['yy', years];
  		    a[2] = withoutSuffix;
  		    a[3] = +posNegDuration > 0;
  		    a[4] = locale;
  		    return substituteTimeAgo.apply(null, a);
  		  }

  		  // This function allows you to set the rounding function for relative time strings
  		  function getSetRelativeTimeRounding(roundingFunction) {
  		    if (roundingFunction === undefined) {
  		      return round;
  		    }
  		    if (typeof roundingFunction === 'function') {
  		      round = roundingFunction;
  		      return true;
  		    }
  		    return false;
  		  }

  		  // This function allows you to set a threshold for relative time strings
  		  function getSetRelativeTimeThreshold(threshold, limit) {
  		    if (thresholds[threshold] === undefined) {
  		      return false;
  		    }
  		    if (limit === undefined) {
  		      return thresholds[threshold];
  		    }
  		    thresholds[threshold] = limit;
  		    if (threshold === 's') {
  		      thresholds.ss = limit - 1;
  		    }
  		    return true;
  		  }
  		  function humanize(argWithSuffix, argThresholds) {
  		    if (!this.isValid()) {
  		      return this.localeData().invalidDate();
  		    }
  		    var withSuffix = false,
  		      th = thresholds,
  		      locale,
  		      output;
  		    if (typeof argWithSuffix === 'object') {
  		      argThresholds = argWithSuffix;
  		      argWithSuffix = false;
  		    }
  		    if (typeof argWithSuffix === 'boolean') {
  		      withSuffix = argWithSuffix;
  		    }
  		    if (typeof argThresholds === 'object') {
  		      th = Object.assign({}, thresholds, argThresholds);
  		      if (argThresholds.s != null && argThresholds.ss == null) {
  		        th.ss = argThresholds.s - 1;
  		      }
  		    }
  		    locale = this.localeData();
  		    output = relativeTime$1(this, !withSuffix, th, locale);
  		    if (withSuffix) {
  		      output = locale.pastFuture(+this, output);
  		    }
  		    return locale.postformat(output);
  		  }
  		  var abs$1 = Math.abs;
  		  function sign(x) {
  		    return (x > 0) - (x < 0) || +x;
  		  }
  		  function toISOString$1() {
  		    // for ISO strings we do not use the normal bubbling rules:
  		    //  * milliseconds bubble up until they become hours
  		    //  * days do not bubble at all
  		    //  * months bubble up until they become years
  		    // This is because there is no context-free conversion between hours and days
  		    // (think of clock changes)
  		    // and also not between days and months (28-31 days per month)
  		    if (!this.isValid()) {
  		      return this.localeData().invalidDate();
  		    }
  		    var seconds = abs$1(this._milliseconds) / 1000,
  		      days = abs$1(this._days),
  		      months = abs$1(this._months),
  		      minutes,
  		      hours,
  		      years,
  		      s,
  		      total = this.asSeconds(),
  		      totalSign,
  		      ymSign,
  		      daysSign,
  		      hmsSign;
  		    if (!total) {
  		      // this is the same as C#'s (Noda) and python (isodate)...
  		      // but not other JS (goog.date)
  		      return 'P0D';
  		    }

  		    // 3600 seconds -> 60 minutes -> 1 hour
  		    minutes = absFloor(seconds / 60);
  		    hours = absFloor(minutes / 60);
  		    seconds %= 60;
  		    minutes %= 60;

  		    // 12 months -> 1 year
  		    years = absFloor(months / 12);
  		    months %= 12;

  		    // inspired by https://github.com/dordille/moment-isoduration/blob/master/moment.isoduration.js
  		    s = seconds ? seconds.toFixed(3).replace(/\.?0+$/, '') : '';
  		    totalSign = total < 0 ? '-' : '';
  		    ymSign = sign(this._months) !== sign(total) ? '-' : '';
  		    daysSign = sign(this._days) !== sign(total) ? '-' : '';
  		    hmsSign = sign(this._milliseconds) !== sign(total) ? '-' : '';
  		    return totalSign + 'P' + (years ? ymSign + years + 'Y' : '') + (months ? ymSign + months + 'M' : '') + (days ? daysSign + days + 'D' : '') + (hours || minutes || seconds ? 'T' : '') + (hours ? hmsSign + hours + 'H' : '') + (minutes ? hmsSign + minutes + 'M' : '') + (seconds ? hmsSign + s + 'S' : '');
  		  }
  		  var proto$2 = Duration.prototype;
  		  proto$2.isValid = isValid$1;
  		  proto$2.abs = abs;
  		  proto$2.add = add$1;
  		  proto$2.subtract = subtract$1;
  		  proto$2.as = as;
  		  proto$2.asMilliseconds = asMilliseconds;
  		  proto$2.asSeconds = asSeconds;
  		  proto$2.asMinutes = asMinutes;
  		  proto$2.asHours = asHours;
  		  proto$2.asDays = asDays;
  		  proto$2.asWeeks = asWeeks;
  		  proto$2.asMonths = asMonths;
  		  proto$2.asQuarters = asQuarters;
  		  proto$2.asYears = asYears;
  		  proto$2.valueOf = valueOf$1;
  		  proto$2._bubble = bubble;
  		  proto$2.clone = clone$1;
  		  proto$2.get = get$2;
  		  proto$2.milliseconds = milliseconds;
  		  proto$2.seconds = seconds;
  		  proto$2.minutes = minutes;
  		  proto$2.hours = hours;
  		  proto$2.days = days;
  		  proto$2.weeks = weeks;
  		  proto$2.months = months;
  		  proto$2.years = years;
  		  proto$2.humanize = humanize;
  		  proto$2.toISOString = toISOString$1;
  		  proto$2.toString = toISOString$1;
  		  proto$2.toJSON = toISOString$1;
  		  proto$2.locale = locale;
  		  proto$2.localeData = localeData;
  		  proto$2.toIsoString = deprecate('toIsoString() is deprecated. Please use toISOString() instead (notice the capitals)', toISOString$1);
  		  proto$2.lang = lang;

  		  // FORMATTING

  		  addFormatToken('X', 0, 0, 'unix');
  		  addFormatToken('x', 0, 0, 'valueOf');

  		  // PARSING

  		  addRegexToken('x', matchSigned);
  		  addRegexToken('X', matchTimestamp);
  		  addParseToken('X', function (input, array, config) {
  		    config._d = new Date(parseFloat(input) * 1000);
  		  });
  		  addParseToken('x', function (input, array, config) {
  		    config._d = new Date(toInt(input));
  		  });

  		  //! moment.js

  		  hooks.version = '2.30.1';
  		  setHookCallback(createLocal);
  		  hooks.fn = proto;
  		  hooks.min = min;
  		  hooks.max = max;
  		  hooks.now = now;
  		  hooks.utc = createUTC;
  		  hooks.unix = createUnix;
  		  hooks.months = listMonths;
  		  hooks.isDate = isDate;
  		  hooks.locale = getSetGlobalLocale;
  		  hooks.invalid = createInvalid;
  		  hooks.duration = createDuration;
  		  hooks.isMoment = isMoment;
  		  hooks.weekdays = listWeekdays;
  		  hooks.parseZone = createInZone;
  		  hooks.localeData = getLocale;
  		  hooks.isDuration = isDuration;
  		  hooks.monthsShort = listMonthsShort;
  		  hooks.weekdaysMin = listWeekdaysMin;
  		  hooks.defineLocale = defineLocale;
  		  hooks.updateLocale = updateLocale;
  		  hooks.locales = listLocales;
  		  hooks.weekdaysShort = listWeekdaysShort;
  		  hooks.normalizeUnits = normalizeUnits;
  		  hooks.relativeTimeRounding = getSetRelativeTimeRounding;
  		  hooks.relativeTimeThreshold = getSetRelativeTimeThreshold;
  		  hooks.calendarFormat = getCalendarFormat;
  		  hooks.prototype = proto;

  		  // currently HTML5 input type only supports 24-hour formats
  		  hooks.HTML5_FMT = {
  		    DATETIME_LOCAL: 'YYYY-MM-DDTHH:mm',
  		    // <input type="datetime-local" />
  		    DATETIME_LOCAL_SECONDS: 'YYYY-MM-DDTHH:mm:ss',
  		    // <input type="datetime-local" step="1" />
  		    DATETIME_LOCAL_MS: 'YYYY-MM-DDTHH:mm:ss.SSS',
  		    // <input type="datetime-local" step="0.001" />
  		    DATE: 'YYYY-MM-DD',
  		    // <input type="date" />
  		    TIME: 'HH:mm',
  		    // <input type="time" />
  		    TIME_SECONDS: 'HH:mm:ss',
  		    // <input type="time" step="1" />
  		    TIME_MS: 'HH:mm:ss.SSS',
  		    // <input type="time" step="0.001" />
  		    WEEK: 'GGGG-[W]WW',
  		    // <input type="week" />
  		    MONTH: 'YYYY-MM' // <input type="month" />
  		  };

  		  //! moment.js locale configuration

  		  hooks.defineLocale('af', {
  		    months: 'Januarie_Februarie_Maart_April_Mei_Junie_Julie_Augustus_September_Oktober_November_Desember'.split('_'),
  		    monthsShort: 'Jan_Feb_Mrt_Apr_Mei_Jun_Jul_Aug_Sep_Okt_Nov_Des'.split('_'),
  		    weekdays: 'Sondag_Maandag_Dinsdag_Woensdag_Donderdag_Vrydag_Saterdag'.split('_'),
  		    weekdaysShort: 'Son_Maa_Din_Woe_Don_Vry_Sat'.split('_'),
  		    weekdaysMin: 'So_Ma_Di_Wo_Do_Vr_Sa'.split('_'),
  		    meridiemParse: /vm|nm/i,
  		    isPM: function (input) {
  		      return /^nm$/i.test(input);
  		    },
  		    meridiem: function (hours, minutes, isLower) {
  		      if (hours < 12) {
  		        return isLower ? 'vm' : 'VM';
  		      } else {
  		        return isLower ? 'nm' : 'NM';
  		      }
  		    },
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd, D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Vandag om] LT',
  		      nextDay: '[Môre om] LT',
  		      nextWeek: 'dddd [om] LT',
  		      lastDay: '[Gister om] LT',
  		      lastWeek: '[Laas] dddd [om] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'oor %s',
  		      past: '%s gelede',
  		      s: "'n paar sekondes",
  		      ss: '%d sekondes',
  		      m: "'n minuut",
  		      mm: '%d minute',
  		      h: "'n uur",
  		      hh: '%d ure',
  		      d: "'n dag",
  		      dd: '%d dae',
  		      M: "'n maand",
  		      MM: '%d maande',
  		      y: "'n jaar",
  		      yy: '%d jaar'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(ste|de)/,
  		    ordinal: function (number) {
  		      return number + (number === 1 || number === 8 || number >= 20 ? 'ste' : 'de'); // Thanks to Joris Röling : https://github.com/jjupiter
  		    },
  		    week: {
  		      dow: 1,
  		      // Maandag is die eerste dag van die week.
  		      doy: 4 // Die week wat die 4de Januarie bevat is die eerste week van die jaar.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var pluralForm = function (n) {
  		      return n === 0 ? 0 : n === 1 ? 1 : n === 2 ? 2 : n % 100 >= 3 && n % 100 <= 10 ? 3 : n % 100 >= 11 ? 4 : 5;
  		    },
  		    plurals = {
  		      s: ['أقل من ثانية', 'ثانية واحدة', ['ثانيتان', 'ثانيتين'], '%d ثوان', '%d ثانية', '%d ثانية'],
  		      m: ['أقل من دقيقة', 'دقيقة واحدة', ['دقيقتان', 'دقيقتين'], '%d دقائق', '%d دقيقة', '%d دقيقة'],
  		      h: ['أقل من ساعة', 'ساعة واحدة', ['ساعتان', 'ساعتين'], '%d ساعات', '%d ساعة', '%d ساعة'],
  		      d: ['أقل من يوم', 'يوم واحد', ['يومان', 'يومين'], '%d أيام', '%d يومًا', '%d يوم'],
  		      M: ['أقل من شهر', 'شهر واحد', ['شهران', 'شهرين'], '%d أشهر', '%d شهرا', '%d شهر'],
  		      y: ['أقل من عام', 'عام واحد', ['عامان', 'عامين'], '%d أعوام', '%d عامًا', '%d عام']
  		    },
  		    pluralize = function (u) {
  		      return function (number, withoutSuffix, string, isFuture) {
  		        var f = pluralForm(number),
  		          str = plurals[u][pluralForm(number)];
  		        if (f === 2) {
  		          str = str[withoutSuffix ? 0 : 1];
  		        }
  		        return str.replace(/%d/i, number);
  		      };
  		    },
  		    months$1 = ['جانفي', 'فيفري', 'مارس', 'أفريل', 'ماي', 'جوان', 'جويلية', 'أوت', 'سبتمبر', 'أكتوبر', 'نوفمبر', 'ديسمبر'];
  		  hooks.defineLocale('ar-dz', {
  		    months: months$1,
  		    monthsShort: months$1,
  		    weekdays: 'الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت'.split('_'),
  		    weekdaysShort: 'أحد_إثنين_ثلاثاء_أربعاء_خميس_جمعة_سبت'.split('_'),
  		    weekdaysMin: 'ح_ن_ث_ر_خ_ج_س'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'D/\u200FM/\u200FYYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd D MMMM YYYY HH:mm'
  		    },
  		    meridiemParse: /ص|م/,
  		    isPM: function (input) {
  		      return 'م' === input;
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 12) {
  		        return 'ص';
  		      } else {
  		        return 'م';
  		      }
  		    },
  		    calendar: {
  		      sameDay: '[اليوم عند الساعة] LT',
  		      nextDay: '[غدًا عند الساعة] LT',
  		      nextWeek: 'dddd [عند الساعة] LT',
  		      lastDay: '[أمس عند الساعة] LT',
  		      lastWeek: 'dddd [عند الساعة] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'بعد %s',
  		      past: 'منذ %s',
  		      s: pluralize('s'),
  		      ss: pluralize('s'),
  		      m: pluralize('m'),
  		      mm: pluralize('m'),
  		      h: pluralize('h'),
  		      hh: pluralize('h'),
  		      d: pluralize('d'),
  		      dd: pluralize('d'),
  		      M: pluralize('M'),
  		      MM: pluralize('M'),
  		      y: pluralize('y'),
  		      yy: pluralize('y')
  		    },
  		    postformat: function (string) {
  		      return string.replace(/,/g, '،');
  		    },
  		    week: {
  		      dow: 0,
  		      // Sunday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('ar-kw', {
  		    months: 'يناير_فبراير_مارس_أبريل_ماي_يونيو_يوليوز_غشت_شتنبر_أكتوبر_نونبر_دجنبر'.split('_'),
  		    monthsShort: 'يناير_فبراير_مارس_أبريل_ماي_يونيو_يوليوز_غشت_شتنبر_أكتوبر_نونبر_دجنبر'.split('_'),
  		    weekdays: 'الأحد_الإتنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت'.split('_'),
  		    weekdaysShort: 'احد_اتنين_ثلاثاء_اربعاء_خميس_جمعة_سبت'.split('_'),
  		    weekdaysMin: 'ح_ن_ث_ر_خ_ج_س'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[اليوم على الساعة] LT',
  		      nextDay: '[غدا على الساعة] LT',
  		      nextWeek: 'dddd [على الساعة] LT',
  		      lastDay: '[أمس على الساعة] LT',
  		      lastWeek: 'dddd [على الساعة] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'في %s',
  		      past: 'منذ %s',
  		      s: 'ثوان',
  		      ss: '%d ثانية',
  		      m: 'دقيقة',
  		      mm: '%d دقائق',
  		      h: 'ساعة',
  		      hh: '%d ساعات',
  		      d: 'يوم',
  		      dd: '%d أيام',
  		      M: 'شهر',
  		      MM: '%d أشهر',
  		      y: 'سنة',
  		      yy: '%d سنوات'
  		    },
  		    week: {
  		      dow: 0,
  		      // Sunday is the first day of the week.
  		      doy: 12 // The week that contains Jan 12th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var symbolMap = {
  		      1: '1',
  		      2: '2',
  		      3: '3',
  		      4: '4',
  		      5: '5',
  		      6: '6',
  		      7: '7',
  		      8: '8',
  		      9: '9',
  		      0: '0'
  		    },
  		    pluralForm$1 = function (n) {
  		      return n === 0 ? 0 : n === 1 ? 1 : n === 2 ? 2 : n % 100 >= 3 && n % 100 <= 10 ? 3 : n % 100 >= 11 ? 4 : 5;
  		    },
  		    plurals$1 = {
  		      s: ['أقل من ثانية', 'ثانية واحدة', ['ثانيتان', 'ثانيتين'], '%d ثوان', '%d ثانية', '%d ثانية'],
  		      m: ['أقل من دقيقة', 'دقيقة واحدة', ['دقيقتان', 'دقيقتين'], '%d دقائق', '%d دقيقة', '%d دقيقة'],
  		      h: ['أقل من ساعة', 'ساعة واحدة', ['ساعتان', 'ساعتين'], '%d ساعات', '%d ساعة', '%d ساعة'],
  		      d: ['أقل من يوم', 'يوم واحد', ['يومان', 'يومين'], '%d أيام', '%d يومًا', '%d يوم'],
  		      M: ['أقل من شهر', 'شهر واحد', ['شهران', 'شهرين'], '%d أشهر', '%d شهرا', '%d شهر'],
  		      y: ['أقل من عام', 'عام واحد', ['عامان', 'عامين'], '%d أعوام', '%d عامًا', '%d عام']
  		    },
  		    pluralize$1 = function (u) {
  		      return function (number, withoutSuffix, string, isFuture) {
  		        var f = pluralForm$1(number),
  		          str = plurals$1[u][pluralForm$1(number)];
  		        if (f === 2) {
  		          str = str[withoutSuffix ? 0 : 1];
  		        }
  		        return str.replace(/%d/i, number);
  		      };
  		    },
  		    months$2 = ['يناير', 'فبراير', 'مارس', 'أبريل', 'مايو', 'يونيو', 'يوليو', 'أغسطس', 'سبتمبر', 'أكتوبر', 'نوفمبر', 'ديسمبر'];
  		  hooks.defineLocale('ar-ly', {
  		    months: months$2,
  		    monthsShort: months$2,
  		    weekdays: 'الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت'.split('_'),
  		    weekdaysShort: 'أحد_إثنين_ثلاثاء_أربعاء_خميس_جمعة_سبت'.split('_'),
  		    weekdaysMin: 'ح_ن_ث_ر_خ_ج_س'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'D/\u200FM/\u200FYYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd D MMMM YYYY HH:mm'
  		    },
  		    meridiemParse: /ص|م/,
  		    isPM: function (input) {
  		      return 'م' === input;
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 12) {
  		        return 'ص';
  		      } else {
  		        return 'م';
  		      }
  		    },
  		    calendar: {
  		      sameDay: '[اليوم عند الساعة] LT',
  		      nextDay: '[غدًا عند الساعة] LT',
  		      nextWeek: 'dddd [عند الساعة] LT',
  		      lastDay: '[أمس عند الساعة] LT',
  		      lastWeek: 'dddd [عند الساعة] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'بعد %s',
  		      past: 'منذ %s',
  		      s: pluralize$1('s'),
  		      ss: pluralize$1('s'),
  		      m: pluralize$1('m'),
  		      mm: pluralize$1('m'),
  		      h: pluralize$1('h'),
  		      hh: pluralize$1('h'),
  		      d: pluralize$1('d'),
  		      dd: pluralize$1('d'),
  		      M: pluralize$1('M'),
  		      MM: pluralize$1('M'),
  		      y: pluralize$1('y'),
  		      yy: pluralize$1('y')
  		    },
  		    preparse: function (string) {
  		      return string.replace(/،/g, ',');
  		    },
  		    postformat: function (string) {
  		      return string.replace(/\d/g, function (match) {
  		        return symbolMap[match];
  		      }).replace(/,/g, '،');
  		    },
  		    week: {
  		      dow: 6,
  		      // Saturday is the first day of the week.
  		      doy: 12 // The week that contains Jan 12th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('ar-ma', {
  		    months: 'يناير_فبراير_مارس_أبريل_ماي_يونيو_يوليوز_غشت_شتنبر_أكتوبر_نونبر_دجنبر'.split('_'),
  		    monthsShort: 'يناير_فبراير_مارس_أبريل_ماي_يونيو_يوليوز_غشت_شتنبر_أكتوبر_نونبر_دجنبر'.split('_'),
  		    weekdays: 'الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت'.split('_'),
  		    weekdaysShort: 'احد_اثنين_ثلاثاء_اربعاء_خميس_جمعة_سبت'.split('_'),
  		    weekdaysMin: 'ح_ن_ث_ر_خ_ج_س'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[اليوم على الساعة] LT',
  		      nextDay: '[غدا على الساعة] LT',
  		      nextWeek: 'dddd [على الساعة] LT',
  		      lastDay: '[أمس على الساعة] LT',
  		      lastWeek: 'dddd [على الساعة] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'في %s',
  		      past: 'منذ %s',
  		      s: 'ثوان',
  		      ss: '%d ثانية',
  		      m: 'دقيقة',
  		      mm: '%d دقائق',
  		      h: 'ساعة',
  		      hh: '%d ساعات',
  		      d: 'يوم',
  		      dd: '%d أيام',
  		      M: 'شهر',
  		      MM: '%d أشهر',
  		      y: 'سنة',
  		      yy: '%d سنوات'
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var symbolMap$1 = {
  		      1: '١',
  		      2: '٢',
  		      3: '٣',
  		      4: '٤',
  		      5: '٥',
  		      6: '٦',
  		      7: '٧',
  		      8: '٨',
  		      9: '٩',
  		      0: '٠'
  		    },
  		    numberMap = {
  		      '١': '1',
  		      '٢': '2',
  		      '٣': '3',
  		      '٤': '4',
  		      '٥': '5',
  		      '٦': '6',
  		      '٧': '7',
  		      '٨': '8',
  		      '٩': '9',
  		      '٠': '0'
  		    };
  		  hooks.defineLocale('ar-ps', {
  		    months: 'كانون الثاني_شباط_آذار_نيسان_أيّار_حزيران_تمّوز_آب_أيلول_تشري الأوّل_تشرين الثاني_كانون الأوّل'.split('_'),
  		    monthsShort: 'ك٢_شباط_آذار_نيسان_أيّار_حزيران_تمّوز_آب_أيلول_ت١_ت٢_ك١'.split('_'),
  		    weekdays: 'الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت'.split('_'),
  		    weekdaysShort: 'أحد_إثنين_ثلاثاء_أربعاء_خميس_جمعة_سبت'.split('_'),
  		    weekdaysMin: 'ح_ن_ث_ر_خ_ج_س'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd D MMMM YYYY HH:mm'
  		    },
  		    meridiemParse: /ص|م/,
  		    isPM: function (input) {
  		      return 'م' === input;
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 12) {
  		        return 'ص';
  		      } else {
  		        return 'م';
  		      }
  		    },
  		    calendar: {
  		      sameDay: '[اليوم على الساعة] LT',
  		      nextDay: '[غدا على الساعة] LT',
  		      nextWeek: 'dddd [على الساعة] LT',
  		      lastDay: '[أمس على الساعة] LT',
  		      lastWeek: 'dddd [على الساعة] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'في %s',
  		      past: 'منذ %s',
  		      s: 'ثوان',
  		      ss: '%d ثانية',
  		      m: 'دقيقة',
  		      mm: '%d دقائق',
  		      h: 'ساعة',
  		      hh: '%d ساعات',
  		      d: 'يوم',
  		      dd: '%d أيام',
  		      M: 'شهر',
  		      MM: '%d أشهر',
  		      y: 'سنة',
  		      yy: '%d سنوات'
  		    },
  		    preparse: function (string) {
  		      return string.replace(/[٣٤٥٦٧٨٩٠]/g, function (match) {
  		        return numberMap[match];
  		      }).split('') // reversed since negative lookbehind not supported everywhere
  		      .reverse().join('').replace(/[١٢](?![\u062a\u0643])/g, function (match) {
  		        return numberMap[match];
  		      }).split('').reverse().join('').replace(/،/g, ',');
  		    },
  		    postformat: function (string) {
  		      return string.replace(/\d/g, function (match) {
  		        return symbolMap$1[match];
  		      }).replace(/,/g, '،');
  		    },
  		    week: {
  		      dow: 0,
  		      // Sunday is the first day of the week.
  		      doy: 6 // The week that contains Jan 6th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var symbolMap$2 = {
  		      1: '١',
  		      2: '٢',
  		      3: '٣',
  		      4: '٤',
  		      5: '٥',
  		      6: '٦',
  		      7: '٧',
  		      8: '٨',
  		      9: '٩',
  		      0: '٠'
  		    },
  		    numberMap$1 = {
  		      '١': '1',
  		      '٢': '2',
  		      '٣': '3',
  		      '٤': '4',
  		      '٥': '5',
  		      '٦': '6',
  		      '٧': '7',
  		      '٨': '8',
  		      '٩': '9',
  		      '٠': '0'
  		    };
  		  hooks.defineLocale('ar-sa', {
  		    months: 'يناير_فبراير_مارس_أبريل_مايو_يونيو_يوليو_أغسطس_سبتمبر_أكتوبر_نوفمبر_ديسمبر'.split('_'),
  		    monthsShort: 'يناير_فبراير_مارس_أبريل_مايو_يونيو_يوليو_أغسطس_سبتمبر_أكتوبر_نوفمبر_ديسمبر'.split('_'),
  		    weekdays: 'الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت'.split('_'),
  		    weekdaysShort: 'أحد_إثنين_ثلاثاء_أربعاء_خميس_جمعة_سبت'.split('_'),
  		    weekdaysMin: 'ح_ن_ث_ر_خ_ج_س'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd D MMMM YYYY HH:mm'
  		    },
  		    meridiemParse: /ص|م/,
  		    isPM: function (input) {
  		      return 'م' === input;
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 12) {
  		        return 'ص';
  		      } else {
  		        return 'م';
  		      }
  		    },
  		    calendar: {
  		      sameDay: '[اليوم على الساعة] LT',
  		      nextDay: '[غدا على الساعة] LT',
  		      nextWeek: 'dddd [على الساعة] LT',
  		      lastDay: '[أمس على الساعة] LT',
  		      lastWeek: 'dddd [على الساعة] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'في %s',
  		      past: 'منذ %s',
  		      s: 'ثوان',
  		      ss: '%d ثانية',
  		      m: 'دقيقة',
  		      mm: '%d دقائق',
  		      h: 'ساعة',
  		      hh: '%d ساعات',
  		      d: 'يوم',
  		      dd: '%d أيام',
  		      M: 'شهر',
  		      MM: '%d أشهر',
  		      y: 'سنة',
  		      yy: '%d سنوات'
  		    },
  		    preparse: function (string) {
  		      return string.replace(/[١٢٣٤٥٦٧٨٩٠]/g, function (match) {
  		        return numberMap$1[match];
  		      }).replace(/،/g, ',');
  		    },
  		    postformat: function (string) {
  		      return string.replace(/\d/g, function (match) {
  		        return symbolMap$2[match];
  		      }).replace(/,/g, '،');
  		    },
  		    week: {
  		      dow: 0,
  		      // Sunday is the first day of the week.
  		      doy: 6 // The week that contains Jan 6th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('ar-tn', {
  		    months: 'جانفي_فيفري_مارس_أفريل_ماي_جوان_جويلية_أوت_سبتمبر_أكتوبر_نوفمبر_ديسمبر'.split('_'),
  		    monthsShort: 'جانفي_فيفري_مارس_أفريل_ماي_جوان_جويلية_أوت_سبتمبر_أكتوبر_نوفمبر_ديسمبر'.split('_'),
  		    weekdays: 'الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت'.split('_'),
  		    weekdaysShort: 'أحد_إثنين_ثلاثاء_أربعاء_خميس_جمعة_سبت'.split('_'),
  		    weekdaysMin: 'ح_ن_ث_ر_خ_ج_س'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[اليوم على الساعة] LT',
  		      nextDay: '[غدا على الساعة] LT',
  		      nextWeek: 'dddd [على الساعة] LT',
  		      lastDay: '[أمس على الساعة] LT',
  		      lastWeek: 'dddd [على الساعة] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'في %s',
  		      past: 'منذ %s',
  		      s: 'ثوان',
  		      ss: '%d ثانية',
  		      m: 'دقيقة',
  		      mm: '%d دقائق',
  		      h: 'ساعة',
  		      hh: '%d ساعات',
  		      d: 'يوم',
  		      dd: '%d أيام',
  		      M: 'شهر',
  		      MM: '%d أشهر',
  		      y: 'سنة',
  		      yy: '%d سنوات'
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var symbolMap$3 = {
  		      1: '١',
  		      2: '٢',
  		      3: '٣',
  		      4: '٤',
  		      5: '٥',
  		      6: '٦',
  		      7: '٧',
  		      8: '٨',
  		      9: '٩',
  		      0: '٠'
  		    },
  		    numberMap$2 = {
  		      '١': '1',
  		      '٢': '2',
  		      '٣': '3',
  		      '٤': '4',
  		      '٥': '5',
  		      '٦': '6',
  		      '٧': '7',
  		      '٨': '8',
  		      '٩': '9',
  		      '٠': '0'
  		    },
  		    pluralForm$2 = function (n) {
  		      return n === 0 ? 0 : n === 1 ? 1 : n === 2 ? 2 : n % 100 >= 3 && n % 100 <= 10 ? 3 : n % 100 >= 11 ? 4 : 5;
  		    },
  		    plurals$2 = {
  		      s: ['أقل من ثانية', 'ثانية واحدة', ['ثانيتان', 'ثانيتين'], '%d ثوان', '%d ثانية', '%d ثانية'],
  		      m: ['أقل من دقيقة', 'دقيقة واحدة', ['دقيقتان', 'دقيقتين'], '%d دقائق', '%d دقيقة', '%d دقيقة'],
  		      h: ['أقل من ساعة', 'ساعة واحدة', ['ساعتان', 'ساعتين'], '%d ساعات', '%d ساعة', '%d ساعة'],
  		      d: ['أقل من يوم', 'يوم واحد', ['يومان', 'يومين'], '%d أيام', '%d يومًا', '%d يوم'],
  		      M: ['أقل من شهر', 'شهر واحد', ['شهران', 'شهرين'], '%d أشهر', '%d شهرا', '%d شهر'],
  		      y: ['أقل من عام', 'عام واحد', ['عامان', 'عامين'], '%d أعوام', '%d عامًا', '%d عام']
  		    },
  		    pluralize$2 = function (u) {
  		      return function (number, withoutSuffix, string, isFuture) {
  		        var f = pluralForm$2(number),
  		          str = plurals$2[u][pluralForm$2(number)];
  		        if (f === 2) {
  		          str = str[withoutSuffix ? 0 : 1];
  		        }
  		        return str.replace(/%d/i, number);
  		      };
  		    },
  		    months$3 = ['يناير', 'فبراير', 'مارس', 'أبريل', 'مايو', 'يونيو', 'يوليو', 'أغسطس', 'سبتمبر', 'أكتوبر', 'نوفمبر', 'ديسمبر'];
  		  hooks.defineLocale('ar', {
  		    months: months$3,
  		    monthsShort: months$3,
  		    weekdays: 'الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت'.split('_'),
  		    weekdaysShort: 'أحد_إثنين_ثلاثاء_أربعاء_خميس_جمعة_سبت'.split('_'),
  		    weekdaysMin: 'ح_ن_ث_ر_خ_ج_س'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'D/\u200FM/\u200FYYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd D MMMM YYYY HH:mm'
  		    },
  		    meridiemParse: /ص|م/,
  		    isPM: function (input) {
  		      return 'م' === input;
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 12) {
  		        return 'ص';
  		      } else {
  		        return 'م';
  		      }
  		    },
  		    calendar: {
  		      sameDay: '[اليوم عند الساعة] LT',
  		      nextDay: '[غدًا عند الساعة] LT',
  		      nextWeek: 'dddd [عند الساعة] LT',
  		      lastDay: '[أمس عند الساعة] LT',
  		      lastWeek: 'dddd [عند الساعة] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'بعد %s',
  		      past: 'منذ %s',
  		      s: pluralize$2('s'),
  		      ss: pluralize$2('s'),
  		      m: pluralize$2('m'),
  		      mm: pluralize$2('m'),
  		      h: pluralize$2('h'),
  		      hh: pluralize$2('h'),
  		      d: pluralize$2('d'),
  		      dd: pluralize$2('d'),
  		      M: pluralize$2('M'),
  		      MM: pluralize$2('M'),
  		      y: pluralize$2('y'),
  		      yy: pluralize$2('y')
  		    },
  		    preparse: function (string) {
  		      return string.replace(/[١٢٣٤٥٦٧٨٩٠]/g, function (match) {
  		        return numberMap$2[match];
  		      }).replace(/،/g, ',');
  		    },
  		    postformat: function (string) {
  		      return string.replace(/\d/g, function (match) {
  		        return symbolMap$3[match];
  		      }).replace(/,/g, '،');
  		    },
  		    week: {
  		      dow: 6,
  		      // Saturday is the first day of the week.
  		      doy: 12 // The week that contains Jan 12th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var suffixes = {
  		    1: '-inci',
  		    5: '-inci',
  		    8: '-inci',
  		    70: '-inci',
  		    80: '-inci',
  		    2: '-nci',
  		    7: '-nci',
  		    20: '-nci',
  		    50: '-nci',
  		    3: '-üncü',
  		    4: '-üncü',
  		    100: '-üncü',
  		    6: '-ncı',
  		    9: '-uncu',
  		    10: '-uncu',
  		    30: '-uncu',
  		    60: '-ıncı',
  		    90: '-ıncı'
  		  };
  		  hooks.defineLocale('az', {
  		    months: 'yanvar_fevral_mart_aprel_may_iyun_iyul_avqust_sentyabr_oktyabr_noyabr_dekabr'.split('_'),
  		    monthsShort: 'yan_fev_mar_apr_may_iyn_iyl_avq_sen_okt_noy_dek'.split('_'),
  		    weekdays: 'Bazar_Bazar ertəsi_Çərşənbə axşamı_Çərşənbə_Cümə axşamı_Cümə_Şənbə'.split('_'),
  		    weekdaysShort: 'Baz_BzE_ÇAx_Çər_CAx_Cüm_Şən'.split('_'),
  		    weekdaysMin: 'Bz_BE_ÇA_Çə_CA_Cü_Şə'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd, D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[bugün saat] LT',
  		      nextDay: '[sabah saat] LT',
  		      nextWeek: '[gələn həftə] dddd [saat] LT',
  		      lastDay: '[dünən] LT',
  		      lastWeek: '[keçən həftə] dddd [saat] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%s sonra',
  		      past: '%s əvvəl',
  		      s: 'bir neçə saniyə',
  		      ss: '%d saniyə',
  		      m: 'bir dəqiqə',
  		      mm: '%d dəqiqə',
  		      h: 'bir saat',
  		      hh: '%d saat',
  		      d: 'bir gün',
  		      dd: '%d gün',
  		      M: 'bir ay',
  		      MM: '%d ay',
  		      y: 'bir il',
  		      yy: '%d il'
  		    },
  		    meridiemParse: /gecə|səhər|gündüz|axşam/,
  		    isPM: function (input) {
  		      return /^(gündüz|axşam)$/.test(input);
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 4) {
  		        return 'gecə';
  		      } else if (hour < 12) {
  		        return 'səhər';
  		      } else if (hour < 17) {
  		        return 'gündüz';
  		      } else {
  		        return 'axşam';
  		      }
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}-(ıncı|inci|nci|üncü|ncı|uncu)/,
  		    ordinal: function (number) {
  		      if (number === 0) {
  		        // special case for zero
  		        return number + '-ıncı';
  		      }
  		      var a = number % 10,
  		        b = number % 100 - a,
  		        c = number >= 100 ? 100 : null;
  		      return number + (suffixes[a] || suffixes[b] || suffixes[c]);
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 7 // The week that contains Jan 7th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  function plural(word, num) {
  		    var forms = word.split('_');
  		    return num % 10 === 1 && num % 100 !== 11 ? forms[0] : num % 10 >= 2 && num % 10 <= 4 && (num % 100 < 10 || num % 100 >= 20) ? forms[1] : forms[2];
  		  }
  		  function relativeTimeWithPlural(number, withoutSuffix, key) {
  		    var format = {
  		      ss: withoutSuffix ? 'секунда_секунды_секунд' : 'секунду_секунды_секунд',
  		      mm: withoutSuffix ? 'хвіліна_хвіліны_хвілін' : 'хвіліну_хвіліны_хвілін',
  		      hh: withoutSuffix ? 'гадзіна_гадзіны_гадзін' : 'гадзіну_гадзіны_гадзін',
  		      dd: 'дзень_дні_дзён',
  		      MM: 'месяц_месяцы_месяцаў',
  		      yy: 'год_гады_гадоў'
  		    };
  		    if (key === 'm') {
  		      return withoutSuffix ? 'хвіліна' : 'хвіліну';
  		    } else if (key === 'h') {
  		      return withoutSuffix ? 'гадзіна' : 'гадзіну';
  		    } else {
  		      return number + ' ' + plural(format[key], +number);
  		    }
  		  }
  		  hooks.defineLocale('be', {
  		    months: {
  		      format: 'студзеня_лютага_сакавіка_красавіка_траўня_чэрвеня_ліпеня_жніўня_верасня_кастрычніка_лістапада_снежня'.split('_'),
  		      standalone: 'студзень_люты_сакавік_красавік_травень_чэрвень_ліпень_жнівень_верасень_кастрычнік_лістапад_снежань'.split('_')
  		    },
  		    monthsShort: 'студ_лют_сак_крас_трав_чэрв_ліп_жнів_вер_каст_ліст_снеж'.split('_'),
  		    weekdays: {
  		      format: 'нядзелю_панядзелак_аўторак_сераду_чацвер_пятніцу_суботу'.split('_'),
  		      standalone: 'нядзеля_панядзелак_аўторак_серада_чацвер_пятніца_субота'.split('_'),
  		      isFormat: /\[ ?[Ууў] ?(?:мінулую|наступную)? ?\] ?dddd/
  		    },
  		    weekdaysShort: 'нд_пн_ат_ср_чц_пт_сб'.split('_'),
  		    weekdaysMin: 'нд_пн_ат_ср_чц_пт_сб'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'D MMMM YYYY г.',
  		      LLL: 'D MMMM YYYY г., HH:mm',
  		      LLLL: 'dddd, D MMMM YYYY г., HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Сёння ў] LT',
  		      nextDay: '[Заўтра ў] LT',
  		      lastDay: '[Учора ў] LT',
  		      nextWeek: function () {
  		        return '[У] dddd [ў] LT';
  		      },
  		      lastWeek: function () {
  		        switch (this.day()) {
  		          case 0:
  		          case 3:
  		          case 5:
  		          case 6:
  		            return '[У мінулую] dddd [ў] LT';
  		          case 1:
  		          case 2:
  		          case 4:
  		            return '[У мінулы] dddd [ў] LT';
  		        }
  		      },
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'праз %s',
  		      past: '%s таму',
  		      s: 'некалькі секунд',
  		      m: relativeTimeWithPlural,
  		      mm: relativeTimeWithPlural,
  		      h: relativeTimeWithPlural,
  		      hh: relativeTimeWithPlural,
  		      d: 'дзень',
  		      dd: relativeTimeWithPlural,
  		      M: 'месяц',
  		      MM: relativeTimeWithPlural,
  		      y: 'год',
  		      yy: relativeTimeWithPlural
  		    },
  		    meridiemParse: /ночы|раніцы|дня|вечара/,
  		    isPM: function (input) {
  		      return /^(дня|вечара)$/.test(input);
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 4) {
  		        return 'ночы';
  		      } else if (hour < 12) {
  		        return 'раніцы';
  		      } else if (hour < 17) {
  		        return 'дня';
  		      } else {
  		        return 'вечара';
  		      }
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}-(і|ы|га)/,
  		    ordinal: function (number, period) {
  		      switch (period) {
  		        case 'M':
  		        case 'd':
  		        case 'DDD':
  		        case 'w':
  		        case 'W':
  		          return (number % 10 === 2 || number % 10 === 3) && number % 100 !== 12 && number % 100 !== 13 ? number + '-і' : number + '-ы';
  		        case 'D':
  		          return number + '-га';
  		        default:
  		          return number;
  		      }
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 7 // The week that contains Jan 7th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('bg', {
  		    months: 'януари_февруари_март_април_май_юни_юли_август_септември_октомври_ноември_декември'.split('_'),
  		    monthsShort: 'яну_фев_мар_апр_май_юни_юли_авг_сеп_окт_ное_дек'.split('_'),
  		    weekdays: 'неделя_понеделник_вторник_сряда_четвъртък_петък_събота'.split('_'),
  		    weekdaysShort: 'нед_пон_вто_сря_чет_пет_съб'.split('_'),
  		    weekdaysMin: 'нд_пн_вт_ср_чт_пт_сб'.split('_'),
  		    longDateFormat: {
  		      LT: 'H:mm',
  		      LTS: 'H:mm:ss',
  		      L: 'D.MM.YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY H:mm',
  		      LLLL: 'dddd, D MMMM YYYY H:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Днес в] LT',
  		      nextDay: '[Утре в] LT',
  		      nextWeek: 'dddd [в] LT',
  		      lastDay: '[Вчера в] LT',
  		      lastWeek: function () {
  		        switch (this.day()) {
  		          case 0:
  		          case 3:
  		          case 6:
  		            return '[Миналата] dddd [в] LT';
  		          case 1:
  		          case 2:
  		          case 4:
  		          case 5:
  		            return '[Миналия] dddd [в] LT';
  		        }
  		      },
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'след %s',
  		      past: 'преди %s',
  		      s: 'няколко секунди',
  		      ss: '%d секунди',
  		      m: 'минута',
  		      mm: '%d минути',
  		      h: 'час',
  		      hh: '%d часа',
  		      d: 'ден',
  		      dd: '%d дена',
  		      w: 'седмица',
  		      ww: '%d седмици',
  		      M: 'месец',
  		      MM: '%d месеца',
  		      y: 'година',
  		      yy: '%d години'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}-(ев|ен|ти|ви|ри|ми)/,
  		    ordinal: function (number) {
  		      var lastDigit = number % 10,
  		        last2Digits = number % 100;
  		      if (number === 0) {
  		        return number + '-ев';
  		      } else if (last2Digits === 0) {
  		        return number + '-ен';
  		      } else if (last2Digits > 10 && last2Digits < 20) {
  		        return number + '-ти';
  		      } else if (lastDigit === 1) {
  		        return number + '-ви';
  		      } else if (lastDigit === 2) {
  		        return number + '-ри';
  		      } else if (lastDigit === 7 || lastDigit === 8) {
  		        return number + '-ми';
  		      } else {
  		        return number + '-ти';
  		      }
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 7 // The week that contains Jan 7th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('bm', {
  		    months: 'Zanwuyekalo_Fewuruyekalo_Marisikalo_Awirilikalo_Mɛkalo_Zuwɛnkalo_Zuluyekalo_Utikalo_Sɛtanburukalo_ɔkutɔburukalo_Nowanburukalo_Desanburukalo'.split('_'),
  		    monthsShort: 'Zan_Few_Mar_Awi_Mɛ_Zuw_Zul_Uti_Sɛt_ɔku_Now_Des'.split('_'),
  		    weekdays: 'Kari_Ntɛnɛn_Tarata_Araba_Alamisa_Juma_Sibiri'.split('_'),
  		    weekdaysShort: 'Kar_Ntɛ_Tar_Ara_Ala_Jum_Sib'.split('_'),
  		    weekdaysMin: 'Ka_Nt_Ta_Ar_Al_Ju_Si'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'MMMM [tile] D [san] YYYY',
  		      LLL: 'MMMM [tile] D [san] YYYY [lɛrɛ] HH:mm',
  		      LLLL: 'dddd MMMM [tile] D [san] YYYY [lɛrɛ] HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Bi lɛrɛ] LT',
  		      nextDay: '[Sini lɛrɛ] LT',
  		      nextWeek: 'dddd [don lɛrɛ] LT',
  		      lastDay: '[Kunu lɛrɛ] LT',
  		      lastWeek: 'dddd [tɛmɛnen lɛrɛ] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%s kɔnɔ',
  		      past: 'a bɛ %s bɔ',
  		      s: 'sanga dama dama',
  		      ss: 'sekondi %d',
  		      m: 'miniti kelen',
  		      mm: 'miniti %d',
  		      h: 'lɛrɛ kelen',
  		      hh: 'lɛrɛ %d',
  		      d: 'tile kelen',
  		      dd: 'tile %d',
  		      M: 'kalo kelen',
  		      MM: 'kalo %d',
  		      y: 'san kelen',
  		      yy: 'san %d'
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var symbolMap$4 = {
  		      1: '১',
  		      2: '২',
  		      3: '৩',
  		      4: '৪',
  		      5: '৫',
  		      6: '৬',
  		      7: '৭',
  		      8: '৮',
  		      9: '৯',
  		      0: '০'
  		    },
  		    numberMap$3 = {
  		      '১': '1',
  		      '২': '2',
  		      '৩': '3',
  		      '৪': '4',
  		      '৫': '5',
  		      '৬': '6',
  		      '৭': '7',
  		      '৮': '8',
  		      '৯': '9',
  		      '০': '0'
  		    };
  		  hooks.defineLocale('bn-bd', {
  		    months: 'জানুয়ারি_ফেব্রুয়ারি_মার্চ_এপ্রিল_মে_জুন_জুলাই_আগস্ট_সেপ্টেম্বর_অক্টোবর_নভেম্বর_ডিসেম্বর'.split('_'),
  		    monthsShort: 'জানু_ফেব্রু_মার্চ_এপ্রিল_মে_জুন_জুলাই_আগস্ট_সেপ্ট_অক্টো_নভে_ডিসে'.split('_'),
  		    weekdays: 'রবিবার_সোমবার_মঙ্গলবার_বুধবার_বৃহস্পতিবার_শুক্রবার_শনিবার'.split('_'),
  		    weekdaysShort: 'রবি_সোম_মঙ্গল_বুধ_বৃহস্পতি_শুক্র_শনি'.split('_'),
  		    weekdaysMin: 'রবি_সোম_মঙ্গল_বুধ_বৃহ_শুক্র_শনি'.split('_'),
  		    longDateFormat: {
  		      LT: 'A h:mm সময়',
  		      LTS: 'A h:mm:ss সময়',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY, A h:mm সময়',
  		      LLLL: 'dddd, D MMMM YYYY, A h:mm সময়'
  		    },
  		    calendar: {
  		      sameDay: '[আজ] LT',
  		      nextDay: '[আগামীকাল] LT',
  		      nextWeek: 'dddd, LT',
  		      lastDay: '[গতকাল] LT',
  		      lastWeek: '[গত] dddd, LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%s পরে',
  		      past: '%s আগে',
  		      s: 'কয়েক সেকেন্ড',
  		      ss: '%d সেকেন্ড',
  		      m: 'এক মিনিট',
  		      mm: '%d মিনিট',
  		      h: 'এক ঘন্টা',
  		      hh: '%d ঘন্টা',
  		      d: 'এক দিন',
  		      dd: '%d দিন',
  		      M: 'এক মাস',
  		      MM: '%d মাস',
  		      y: 'এক বছর',
  		      yy: '%d বছর'
  		    },
  		    preparse: function (string) {
  		      return string.replace(/[১২৩৪৫৬৭৮৯০]/g, function (match) {
  		        return numberMap$3[match];
  		      });
  		    },
  		    postformat: function (string) {
  		      return string.replace(/\d/g, function (match) {
  		        return symbolMap$4[match];
  		      });
  		    },
  		    meridiemParse: /রাত|ভোর|সকাল|দুপুর|বিকাল|সন্ধ্যা|রাত/,
  		    meridiemHour: function (hour, meridiem) {
  		      if (hour === 12) {
  		        hour = 0;
  		      }
  		      if (meridiem === 'রাত') {
  		        return hour < 4 ? hour : hour + 12;
  		      } else if (meridiem === 'ভোর') {
  		        return hour;
  		      } else if (meridiem === 'সকাল') {
  		        return hour;
  		      } else if (meridiem === 'দুপুর') {
  		        return hour >= 3 ? hour : hour + 12;
  		      } else if (meridiem === 'বিকাল') {
  		        return hour + 12;
  		      } else if (meridiem === 'সন্ধ্যা') {
  		        return hour + 12;
  		      }
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 4) {
  		        return 'রাত';
  		      } else if (hour < 6) {
  		        return 'ভোর';
  		      } else if (hour < 12) {
  		        return 'সকাল';
  		      } else if (hour < 15) {
  		        return 'দুপুর';
  		      } else if (hour < 18) {
  		        return 'বিকাল';
  		      } else if (hour < 20) {
  		        return 'সন্ধ্যা';
  		      } else {
  		        return 'রাত';
  		      }
  		    },
  		    week: {
  		      dow: 0,
  		      // Sunday is the first day of the week.
  		      doy: 6 // The week that contains Jan 6th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var symbolMap$5 = {
  		      1: '১',
  		      2: '২',
  		      3: '৩',
  		      4: '৪',
  		      5: '৫',
  		      6: '৬',
  		      7: '৭',
  		      8: '৮',
  		      9: '৯',
  		      0: '০'
  		    },
  		    numberMap$4 = {
  		      '১': '1',
  		      '২': '2',
  		      '৩': '3',
  		      '৪': '4',
  		      '৫': '5',
  		      '৬': '6',
  		      '৭': '7',
  		      '৮': '8',
  		      '৯': '9',
  		      '০': '0'
  		    };
  		  hooks.defineLocale('bn', {
  		    months: 'জানুয়ারি_ফেব্রুয়ারি_মার্চ_এপ্রিল_মে_জুন_জুলাই_আগস্ট_সেপ্টেম্বর_অক্টোবর_নভেম্বর_ডিসেম্বর'.split('_'),
  		    monthsShort: 'জানু_ফেব্রু_মার্চ_এপ্রিল_মে_জুন_জুলাই_আগস্ট_সেপ্ট_অক্টো_নভে_ডিসে'.split('_'),
  		    weekdays: 'রবিবার_সোমবার_মঙ্গলবার_বুধবার_বৃহস্পতিবার_শুক্রবার_শনিবার'.split('_'),
  		    weekdaysShort: 'রবি_সোম_মঙ্গল_বুধ_বৃহস্পতি_শুক্র_শনি'.split('_'),
  		    weekdaysMin: 'রবি_সোম_মঙ্গল_বুধ_বৃহ_শুক্র_শনি'.split('_'),
  		    longDateFormat: {
  		      LT: 'A h:mm সময়',
  		      LTS: 'A h:mm:ss সময়',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY, A h:mm সময়',
  		      LLLL: 'dddd, D MMMM YYYY, A h:mm সময়'
  		    },
  		    calendar: {
  		      sameDay: '[আজ] LT',
  		      nextDay: '[আগামীকাল] LT',
  		      nextWeek: 'dddd, LT',
  		      lastDay: '[গতকাল] LT',
  		      lastWeek: '[গত] dddd, LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%s পরে',
  		      past: '%s আগে',
  		      s: 'কয়েক সেকেন্ড',
  		      ss: '%d সেকেন্ড',
  		      m: 'এক মিনিট',
  		      mm: '%d মিনিট',
  		      h: 'এক ঘন্টা',
  		      hh: '%d ঘন্টা',
  		      d: 'এক দিন',
  		      dd: '%d দিন',
  		      M: 'এক মাস',
  		      MM: '%d মাস',
  		      y: 'এক বছর',
  		      yy: '%d বছর'
  		    },
  		    preparse: function (string) {
  		      return string.replace(/[১২৩৪৫৬৭৮৯০]/g, function (match) {
  		        return numberMap$4[match];
  		      });
  		    },
  		    postformat: function (string) {
  		      return string.replace(/\d/g, function (match) {
  		        return symbolMap$5[match];
  		      });
  		    },
  		    meridiemParse: /রাত|সকাল|দুপুর|বিকাল|রাত/,
  		    meridiemHour: function (hour, meridiem) {
  		      if (hour === 12) {
  		        hour = 0;
  		      }
  		      if (meridiem === 'রাত' && hour >= 4 || meridiem === 'দুপুর' && hour < 5 || meridiem === 'বিকাল') {
  		        return hour + 12;
  		      } else {
  		        return hour;
  		      }
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 4) {
  		        return 'রাত';
  		      } else if (hour < 10) {
  		        return 'সকাল';
  		      } else if (hour < 17) {
  		        return 'দুপুর';
  		      } else if (hour < 20) {
  		        return 'বিকাল';
  		      } else {
  		        return 'রাত';
  		      }
  		    },
  		    week: {
  		      dow: 0,
  		      // Sunday is the first day of the week.
  		      doy: 6 // The week that contains Jan 6th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var symbolMap$6 = {
  		      1: '༡',
  		      2: '༢',
  		      3: '༣',
  		      4: '༤',
  		      5: '༥',
  		      6: '༦',
  		      7: '༧',
  		      8: '༨',
  		      9: '༩',
  		      0: '༠'
  		    },
  		    numberMap$5 = {
  		      '༡': '1',
  		      '༢': '2',
  		      '༣': '3',
  		      '༤': '4',
  		      '༥': '5',
  		      '༦': '6',
  		      '༧': '7',
  		      '༨': '8',
  		      '༩': '9',
  		      '༠': '0'
  		    };
  		  hooks.defineLocale('bo', {
  		    months: 'ཟླ་བ་དང་པོ_ཟླ་བ་གཉིས་པ_ཟླ་བ་གསུམ་པ_ཟླ་བ་བཞི་པ_ཟླ་བ་ལྔ་པ_ཟླ་བ་དྲུག་པ_ཟླ་བ་བདུན་པ_ཟླ་བ་བརྒྱད་པ_ཟླ་བ་དགུ་པ_ཟླ་བ་བཅུ་པ_ཟླ་བ་བཅུ་གཅིག་པ_ཟླ་བ་བཅུ་གཉིས་པ'.split('_'),
  		    monthsShort: 'ཟླ་1_ཟླ་2_ཟླ་3_ཟླ་4_ཟླ་5_ཟླ་6_ཟླ་7_ཟླ་8_ཟླ་9_ཟླ་10_ཟླ་11_ཟླ་12'.split('_'),
  		    monthsShortRegex: /^(ཟླ་\d{1,2})/,
  		    monthsParseExact: true,
  		    weekdays: 'གཟའ་ཉི་མ་_གཟའ་ཟླ་བ་_གཟའ་མིག་དམར་_གཟའ་ལྷག་པ་_གཟའ་ཕུར་བུ_གཟའ་པ་སངས་_གཟའ་སྤེན་པ་'.split('_'),
  		    weekdaysShort: 'ཉི་མ་_ཟླ་བ་_མིག་དམར་_ལྷག་པ་_ཕུར་བུ_པ་སངས་_སྤེན་པ་'.split('_'),
  		    weekdaysMin: 'ཉི_ཟླ_མིག_ལྷག_ཕུར_སངས_སྤེན'.split('_'),
  		    longDateFormat: {
  		      LT: 'A h:mm',
  		      LTS: 'A h:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY, A h:mm',
  		      LLLL: 'dddd, D MMMM YYYY, A h:mm'
  		    },
  		    calendar: {
  		      sameDay: '[དི་རིང] LT',
  		      nextDay: '[སང་ཉིན] LT',
  		      nextWeek: '[བདུན་ཕྲག་རྗེས་མ], LT',
  		      lastDay: '[ཁ་སང] LT',
  		      lastWeek: '[བདུན་ཕྲག་མཐའ་མ] dddd, LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%s ལ་',
  		      past: '%s སྔན་ལ',
  		      s: 'ལམ་སང',
  		      ss: '%d སྐར་ཆ།',
  		      m: 'སྐར་མ་གཅིག',
  		      mm: '%d སྐར་མ',
  		      h: 'ཆུ་ཚོད་གཅིག',
  		      hh: '%d ཆུ་ཚོད',
  		      d: 'ཉིན་གཅིག',
  		      dd: '%d ཉིན་',
  		      M: 'ཟླ་བ་གཅིག',
  		      MM: '%d ཟླ་བ',
  		      y: 'ལོ་གཅིག',
  		      yy: '%d ལོ'
  		    },
  		    preparse: function (string) {
  		      return string.replace(/[༡༢༣༤༥༦༧༨༩༠]/g, function (match) {
  		        return numberMap$5[match];
  		      });
  		    },
  		    postformat: function (string) {
  		      return string.replace(/\d/g, function (match) {
  		        return symbolMap$6[match];
  		      });
  		    },
  		    meridiemParse: /མཚན་མོ|ཞོགས་ཀས|ཉིན་གུང|དགོང་དག|མཚན་མོ/,
  		    meridiemHour: function (hour, meridiem) {
  		      if (hour === 12) {
  		        hour = 0;
  		      }
  		      if (meridiem === 'མཚན་མོ' && hour >= 4 || meridiem === 'ཉིན་གུང' && hour < 5 || meridiem === 'དགོང་དག') {
  		        return hour + 12;
  		      } else {
  		        return hour;
  		      }
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 4) {
  		        return 'མཚན་མོ';
  		      } else if (hour < 10) {
  		        return 'ཞོགས་ཀས';
  		      } else if (hour < 17) {
  		        return 'ཉིན་གུང';
  		      } else if (hour < 20) {
  		        return 'དགོང་དག';
  		      } else {
  		        return 'མཚན་མོ';
  		      }
  		    },
  		    week: {
  		      dow: 0,
  		      // Sunday is the first day of the week.
  		      doy: 6 // The week that contains Jan 6th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  function relativeTimeWithMutation(number, withoutSuffix, key) {
  		    var format = {
  		      mm: 'munutenn',
  		      MM: 'miz',
  		      dd: 'devezh'
  		    };
  		    return number + ' ' + mutation(format[key], number);
  		  }
  		  function specialMutationForYears(number) {
  		    switch (lastNumber(number)) {
  		      case 1:
  		      case 3:
  		      case 4:
  		      case 5:
  		      case 9:
  		        return number + ' bloaz';
  		      default:
  		        return number + ' vloaz';
  		    }
  		  }
  		  function lastNumber(number) {
  		    if (number > 9) {
  		      return lastNumber(number % 10);
  		    }
  		    return number;
  		  }
  		  function mutation(text, number) {
  		    if (number === 2) {
  		      return softMutation(text);
  		    }
  		    return text;
  		  }
  		  function softMutation(text) {
  		    var mutationTable = {
  		      m: 'v',
  		      b: 'v',
  		      d: 'z'
  		    };
  		    if (mutationTable[text.charAt(0)] === undefined) {
  		      return text;
  		    }
  		    return mutationTable[text.charAt(0)] + text.substring(1);
  		  }
  		  var monthsParse = [/^gen/i, /^c[ʼ\']hwe/i, /^meu/i, /^ebr/i, /^mae/i, /^(mez|eve)/i, /^gou/i, /^eos/i, /^gwe/i, /^her/i, /^du/i, /^ker/i],
  		    monthsRegex$1 = /^(genver|c[ʼ\']hwevrer|meurzh|ebrel|mae|mezheven|gouere|eost|gwengolo|here|du|kerzu|gen|c[ʼ\']hwe|meu|ebr|mae|eve|gou|eos|gwe|her|du|ker)/i,
  		    monthsStrictRegex = /^(genver|c[ʼ\']hwevrer|meurzh|ebrel|mae|mezheven|gouere|eost|gwengolo|here|du|kerzu)/i,
  		    monthsShortStrictRegex = /^(gen|c[ʼ\']hwe|meu|ebr|mae|eve|gou|eos|gwe|her|du|ker)/i,
  		    fullWeekdaysParse = [/^sul/i, /^lun/i, /^meurzh/i, /^merc[ʼ\']her/i, /^yaou/i, /^gwener/i, /^sadorn/i],
  		    shortWeekdaysParse = [/^Sul/i, /^Lun/i, /^Meu/i, /^Mer/i, /^Yao/i, /^Gwe/i, /^Sad/i],
  		    minWeekdaysParse = [/^Su/i, /^Lu/i, /^Me([^r]|$)/i, /^Mer/i, /^Ya/i, /^Gw/i, /^Sa/i];
  		  hooks.defineLocale('br', {
  		    months: 'Genver_Cʼhwevrer_Meurzh_Ebrel_Mae_Mezheven_Gouere_Eost_Gwengolo_Here_Du_Kerzu'.split('_'),
  		    monthsShort: 'Gen_Cʼhwe_Meu_Ebr_Mae_Eve_Gou_Eos_Gwe_Her_Du_Ker'.split('_'),
  		    weekdays: 'Sul_Lun_Meurzh_Mercʼher_Yaou_Gwener_Sadorn'.split('_'),
  		    weekdaysShort: 'Sul_Lun_Meu_Mer_Yao_Gwe_Sad'.split('_'),
  		    weekdaysMin: 'Su_Lu_Me_Mer_Ya_Gw_Sa'.split('_'),
  		    weekdaysParse: minWeekdaysParse,
  		    fullWeekdaysParse: fullWeekdaysParse,
  		    shortWeekdaysParse: shortWeekdaysParse,
  		    minWeekdaysParse: minWeekdaysParse,
  		    monthsRegex: monthsRegex$1,
  		    monthsShortRegex: monthsRegex$1,
  		    monthsStrictRegex: monthsStrictRegex,
  		    monthsShortStrictRegex: monthsShortStrictRegex,
  		    monthsParse: monthsParse,
  		    longMonthsParse: monthsParse,
  		    shortMonthsParse: monthsParse,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D [a viz] MMMM YYYY',
  		      LLL: 'D [a viz] MMMM YYYY HH:mm',
  		      LLLL: 'dddd, D [a viz] MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Hiziv da] LT',
  		      nextDay: '[Warcʼhoazh da] LT',
  		      nextWeek: 'dddd [da] LT',
  		      lastDay: '[Decʼh da] LT',
  		      lastWeek: 'dddd [paset da] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'a-benn %s',
  		      past: '%s ʼzo',
  		      s: 'un nebeud segondennoù',
  		      ss: '%d eilenn',
  		      m: 'ur vunutenn',
  		      mm: relativeTimeWithMutation,
  		      h: 'un eur',
  		      hh: '%d eur',
  		      d: 'un devezh',
  		      dd: relativeTimeWithMutation,
  		      M: 'ur miz',
  		      MM: relativeTimeWithMutation,
  		      y: 'ur bloaz',
  		      yy: specialMutationForYears
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(añ|vet)/,
  		    ordinal: function (number) {
  		      var output = number === 1 ? 'añ' : 'vet';
  		      return number + output;
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    },
  		    meridiemParse: /a.m.|g.m./,
  		    // goude merenn | a-raok merenn
  		    isPM: function (token) {
  		      return token === 'g.m.';
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      return hour < 12 ? 'a.m.' : 'g.m.';
  		    }
  		  });

  		  //! moment.js locale configuration

  		  function processRelativeTime(number, withoutSuffix, key, isFuture) {
  		    switch (key) {
  		      case 'm':
  		        return withoutSuffix ? 'jedna minuta' : isFuture ? 'jednu minutu' : 'jedne minute';
  		    }
  		  }
  		  function translate(number, withoutSuffix, key) {
  		    var result = number + ' ';
  		    switch (key) {
  		      case 'ss':
  		        if (number === 1) {
  		          result += 'sekunda';
  		        } else if (number === 2 || number === 3 || number === 4) {
  		          result += 'sekunde';
  		        } else {
  		          result += 'sekundi';
  		        }
  		        return result;
  		      case 'mm':
  		        if (number === 1) {
  		          result += 'minuta';
  		        } else if (number === 2 || number === 3 || number === 4) {
  		          result += 'minute';
  		        } else {
  		          result += 'minuta';
  		        }
  		        return result;
  		      case 'h':
  		        return withoutSuffix ? 'jedan sat' : 'jedan sat';
  		      case 'hh':
  		        if (number === 1) {
  		          result += 'sat';
  		        } else if (number === 2 || number === 3 || number === 4) {
  		          result += 'sata';
  		        } else {
  		          result += 'sati';
  		        }
  		        return result;
  		      case 'dd':
  		        if (number === 1) {
  		          result += 'dan';
  		        } else {
  		          result += 'dana';
  		        }
  		        return result;
  		      case 'MM':
  		        if (number === 1) {
  		          result += 'mjesec';
  		        } else if (number === 2 || number === 3 || number === 4) {
  		          result += 'mjeseca';
  		        } else {
  		          result += 'mjeseci';
  		        }
  		        return result;
  		      case 'yy':
  		        if (number === 1) {
  		          result += 'godina';
  		        } else if (number === 2 || number === 3 || number === 4) {
  		          result += 'godine';
  		        } else {
  		          result += 'godina';
  		        }
  		        return result;
  		    }
  		  }
  		  hooks.defineLocale('bs', {
  		    months: 'januar_februar_mart_april_maj_juni_juli_august_septembar_oktobar_novembar_decembar'.split('_'),
  		    monthsShort: 'jan._feb._mar._apr._maj._jun._jul._aug._sep._okt._nov._dec.'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'nedjelja_ponedjeljak_utorak_srijeda_četvrtak_petak_subota'.split('_'),
  		    weekdaysShort: 'ned._pon._uto._sri._čet._pet._sub.'.split('_'),
  		    weekdaysMin: 'ne_po_ut_sr_če_pe_su'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'H:mm',
  		      LTS: 'H:mm:ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'D. MMMM YYYY',
  		      LLL: 'D. MMMM YYYY H:mm',
  		      LLLL: 'dddd, D. MMMM YYYY H:mm'
  		    },
  		    calendar: {
  		      sameDay: '[danas u] LT',
  		      nextDay: '[sutra u] LT',
  		      nextWeek: function () {
  		        switch (this.day()) {
  		          case 0:
  		            return '[u] [nedjelju] [u] LT';
  		          case 3:
  		            return '[u] [srijedu] [u] LT';
  		          case 6:
  		            return '[u] [subotu] [u] LT';
  		          case 1:
  		          case 2:
  		          case 4:
  		          case 5:
  		            return '[u] dddd [u] LT';
  		        }
  		      },
  		      lastDay: '[jučer u] LT',
  		      lastWeek: function () {
  		        switch (this.day()) {
  		          case 0:
  		          case 3:
  		            return '[prošlu] dddd [u] LT';
  		          case 6:
  		            return '[prošle] [subote] [u] LT';
  		          case 1:
  		          case 2:
  		          case 4:
  		          case 5:
  		            return '[prošli] dddd [u] LT';
  		        }
  		      },
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'za %s',
  		      past: 'prije %s',
  		      s: 'par sekundi',
  		      ss: translate,
  		      m: processRelativeTime,
  		      mm: translate,
  		      h: translate,
  		      hh: translate,
  		      d: 'dan',
  		      dd: translate,
  		      M: 'mjesec',
  		      MM: translate,
  		      y: 'godinu',
  		      yy: translate
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}\./,
  		    ordinal: '%d.',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 7 // The week that contains Jan 7th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('ca', {
  		    months: {
  		      standalone: 'gener_febrer_març_abril_maig_juny_juliol_agost_setembre_octubre_novembre_desembre'.split('_'),
  		      format: "de gener_de febrer_de març_d'abril_de maig_de juny_de juliol_d'agost_de setembre_d'octubre_de novembre_de desembre".split('_'),
  		      isFormat: /D[oD]?(\s)+MMMM/
  		    },
  		    monthsShort: 'gen._febr._març_abr._maig_juny_jul._ag._set._oct._nov._des.'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'diumenge_dilluns_dimarts_dimecres_dijous_divendres_dissabte'.split('_'),
  		    weekdaysShort: 'dg._dl._dt._dc._dj._dv._ds.'.split('_'),
  		    weekdaysMin: 'dg_dl_dt_dc_dj_dv_ds'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'H:mm',
  		      LTS: 'H:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM [de] YYYY',
  		      ll: 'D MMM YYYY',
  		      LLL: 'D MMMM [de] YYYY [a les] H:mm',
  		      lll: 'D MMM YYYY, H:mm',
  		      LLLL: 'dddd D MMMM [de] YYYY [a les] H:mm',
  		      llll: 'ddd D MMM YYYY, H:mm'
  		    },
  		    calendar: {
  		      sameDay: function () {
  		        return '[avui a ' + (this.hours() !== 1 ? 'les' : 'la') + '] LT';
  		      },
  		      nextDay: function () {
  		        return '[demà a ' + (this.hours() !== 1 ? 'les' : 'la') + '] LT';
  		      },
  		      nextWeek: function () {
  		        return 'dddd [a ' + (this.hours() !== 1 ? 'les' : 'la') + '] LT';
  		      },
  		      lastDay: function () {
  		        return '[ahir a ' + (this.hours() !== 1 ? 'les' : 'la') + '] LT';
  		      },
  		      lastWeek: function () {
  		        return '[el] dddd [passat a ' + (this.hours() !== 1 ? 'les' : 'la') + '] LT';
  		      },
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: "d'aquí %s",
  		      past: 'fa %s',
  		      s: 'uns segons',
  		      ss: '%d segons',
  		      m: 'un minut',
  		      mm: '%d minuts',
  		      h: 'una hora',
  		      hh: '%d hores',
  		      d: 'un dia',
  		      dd: '%d dies',
  		      M: 'un mes',
  		      MM: '%d mesos',
  		      y: 'un any',
  		      yy: '%d anys'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(r|n|t|è|a)/,
  		    ordinal: function (number, period) {
  		      var output = number === 1 ? 'r' : number === 2 ? 'n' : number === 3 ? 'r' : number === 4 ? 't' : 'è';
  		      if (period === 'w' || period === 'W') {
  		        output = 'a';
  		      }
  		      return number + output;
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var months$4 = {
  		      standalone: 'leden_únor_březen_duben_květen_červen_červenec_srpen_září_říjen_listopad_prosinec'.split('_'),
  		      format: 'ledna_února_března_dubna_května_června_července_srpna_září_října_listopadu_prosince'.split('_'),
  		      isFormat: /DD?[o.]?(\[[^\[\]]*\]|\s)+MMMM/
  		    },
  		    monthsShort = 'led_úno_bře_dub_kvě_čvn_čvc_srp_zář_říj_lis_pro'.split('_'),
  		    monthsParse$1 = [/^led/i, /^úno/i, /^bře/i, /^dub/i, /^kvě/i, /^(čvn|červen$|června)/i, /^(čvc|červenec|července)/i, /^srp/i, /^zář/i, /^říj/i, /^lis/i, /^pro/i],
  		    // NOTE: 'červen' is substring of 'červenec'; therefore 'červenec' must precede 'červen' in the regex to be fully matched.
  		    // Otherwise parser matches '1. červenec' as '1. červen' + 'ec'.
  		    monthsRegex$2 = /^(leden|únor|březen|duben|květen|červenec|července|červen|června|srpen|září|říjen|listopad|prosinec|led|úno|bře|dub|kvě|čvn|čvc|srp|zář|říj|lis|pro)/i;
  		  function plural$1(n) {
  		    return n > 1 && n < 5 && ~~(n / 10) !== 1;
  		  }
  		  function translate$1(number, withoutSuffix, key, isFuture) {
  		    var result = number + ' ';
  		    switch (key) {
  		      case 's':
  		        // a few seconds / in a few seconds / a few seconds ago
  		        return withoutSuffix || isFuture ? 'pár sekund' : 'pár sekundami';
  		      case 'ss':
  		        // 9 seconds / in 9 seconds / 9 seconds ago
  		        if (withoutSuffix || isFuture) {
  		          return result + (plural$1(number) ? 'sekundy' : 'sekund');
  		        } else {
  		          return result + 'sekundami';
  		        }
  		      case 'm':
  		        // a minute / in a minute / a minute ago
  		        return withoutSuffix ? 'minuta' : isFuture ? 'minutu' : 'minutou';
  		      case 'mm':
  		        // 9 minutes / in 9 minutes / 9 minutes ago
  		        if (withoutSuffix || isFuture) {
  		          return result + (plural$1(number) ? 'minuty' : 'minut');
  		        } else {
  		          return result + 'minutami';
  		        }
  		      case 'h':
  		        // an hour / in an hour / an hour ago
  		        return withoutSuffix ? 'hodina' : isFuture ? 'hodinu' : 'hodinou';
  		      case 'hh':
  		        // 9 hours / in 9 hours / 9 hours ago
  		        if (withoutSuffix || isFuture) {
  		          return result + (plural$1(number) ? 'hodiny' : 'hodin');
  		        } else {
  		          return result + 'hodinami';
  		        }
  		      case 'd':
  		        // a day / in a day / a day ago
  		        return withoutSuffix || isFuture ? 'den' : 'dnem';
  		      case 'dd':
  		        // 9 days / in 9 days / 9 days ago
  		        if (withoutSuffix || isFuture) {
  		          return result + (plural$1(number) ? 'dny' : 'dní');
  		        } else {
  		          return result + 'dny';
  		        }
  		      case 'M':
  		        // a month / in a month / a month ago
  		        return withoutSuffix || isFuture ? 'měsíc' : 'měsícem';
  		      case 'MM':
  		        // 9 months / in 9 months / 9 months ago
  		        if (withoutSuffix || isFuture) {
  		          return result + (plural$1(number) ? 'měsíce' : 'měsíců');
  		        } else {
  		          return result + 'měsíci';
  		        }
  		      case 'y':
  		        // a year / in a year / a year ago
  		        return withoutSuffix || isFuture ? 'rok' : 'rokem';
  		      case 'yy':
  		        // 9 years / in 9 years / 9 years ago
  		        if (withoutSuffix || isFuture) {
  		          return result + (plural$1(number) ? 'roky' : 'let');
  		        } else {
  		          return result + 'lety';
  		        }
  		    }
  		  }
  		  hooks.defineLocale('cs', {
  		    months: months$4,
  		    monthsShort: monthsShort,
  		    monthsRegex: monthsRegex$2,
  		    monthsShortRegex: monthsRegex$2,
  		    // NOTE: 'červen' is substring of 'červenec'; therefore 'červenec' must precede 'červen' in the regex to be fully matched.
  		    // Otherwise parser matches '1. červenec' as '1. červen' + 'ec'.
  		    monthsStrictRegex: /^(leden|ledna|února|únor|březen|března|duben|dubna|květen|května|červenec|července|červen|června|srpen|srpna|září|říjen|října|listopadu|listopad|prosinec|prosince)/i,
  		    monthsShortStrictRegex: /^(led|úno|bře|dub|kvě|čvn|čvc|srp|zář|říj|lis|pro)/i,
  		    monthsParse: monthsParse$1,
  		    longMonthsParse: monthsParse$1,
  		    shortMonthsParse: monthsParse$1,
  		    weekdays: 'neděle_pondělí_úterý_středa_čtvrtek_pátek_sobota'.split('_'),
  		    weekdaysShort: 'ne_po_út_st_čt_pá_so'.split('_'),
  		    weekdaysMin: 'ne_po_út_st_čt_pá_so'.split('_'),
  		    longDateFormat: {
  		      LT: 'H:mm',
  		      LTS: 'H:mm:ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'D. MMMM YYYY',
  		      LLL: 'D. MMMM YYYY H:mm',
  		      LLLL: 'dddd D. MMMM YYYY H:mm',
  		      l: 'D. M. YYYY'
  		    },
  		    calendar: {
  		      sameDay: '[dnes v] LT',
  		      nextDay: '[zítra v] LT',
  		      nextWeek: function () {
  		        switch (this.day()) {
  		          case 0:
  		            return '[v neděli v] LT';
  		          case 1:
  		          case 2:
  		            return '[v] dddd [v] LT';
  		          case 3:
  		            return '[ve středu v] LT';
  		          case 4:
  		            return '[ve čtvrtek v] LT';
  		          case 5:
  		            return '[v pátek v] LT';
  		          case 6:
  		            return '[v sobotu v] LT';
  		        }
  		      },
  		      lastDay: '[včera v] LT',
  		      lastWeek: function () {
  		        switch (this.day()) {
  		          case 0:
  		            return '[minulou neděli v] LT';
  		          case 1:
  		          case 2:
  		            return '[minulé] dddd [v] LT';
  		          case 3:
  		            return '[minulou středu v] LT';
  		          case 4:
  		          case 5:
  		            return '[minulý] dddd [v] LT';
  		          case 6:
  		            return '[minulou sobotu v] LT';
  		        }
  		      },
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'za %s',
  		      past: 'před %s',
  		      s: translate$1,
  		      ss: translate$1,
  		      m: translate$1,
  		      mm: translate$1,
  		      h: translate$1,
  		      hh: translate$1,
  		      d: translate$1,
  		      dd: translate$1,
  		      M: translate$1,
  		      MM: translate$1,
  		      y: translate$1,
  		      yy: translate$1
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}\./,
  		    ordinal: '%d.',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('cv', {
  		    months: 'кӑрлач_нарӑс_пуш_ака_май_ҫӗртме_утӑ_ҫурла_авӑн_юпа_чӳк_раштав'.split('_'),
  		    monthsShort: 'кӑр_нар_пуш_ака_май_ҫӗр_утӑ_ҫур_авн_юпа_чӳк_раш'.split('_'),
  		    weekdays: 'вырсарникун_тунтикун_ытларикун_юнкун_кӗҫнерникун_эрнекун_шӑматкун'.split('_'),
  		    weekdaysShort: 'выр_тун_ытл_юн_кӗҫ_эрн_шӑм'.split('_'),
  		    weekdaysMin: 'вр_тн_ыт_юн_кҫ_эр_шм'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD-MM-YYYY',
  		      LL: 'YYYY [ҫулхи] MMMM [уйӑхӗн] D[-мӗшӗ]',
  		      LLL: 'YYYY [ҫулхи] MMMM [уйӑхӗн] D[-мӗшӗ], HH:mm',
  		      LLLL: 'dddd, YYYY [ҫулхи] MMMM [уйӑхӗн] D[-мӗшӗ], HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Паян] LT [сехетре]',
  		      nextDay: '[Ыран] LT [сехетре]',
  		      lastDay: '[Ӗнер] LT [сехетре]',
  		      nextWeek: '[Ҫитес] dddd LT [сехетре]',
  		      lastWeek: '[Иртнӗ] dddd LT [сехетре]',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: function (output) {
  		        var affix = /сехет$/i.exec(output) ? 'рен' : /ҫул$/i.exec(output) ? 'тан' : 'ран';
  		        return output + affix;
  		      },
  		      past: '%s каялла',
  		      s: 'пӗр-ик ҫеккунт',
  		      ss: '%d ҫеккунт',
  		      m: 'пӗр минут',
  		      mm: '%d минут',
  		      h: 'пӗр сехет',
  		      hh: '%d сехет',
  		      d: 'пӗр кун',
  		      dd: '%d кун',
  		      M: 'пӗр уйӑх',
  		      MM: '%d уйӑх',
  		      y: 'пӗр ҫул',
  		      yy: '%d ҫул'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}-мӗш/,
  		    ordinal: '%d-мӗш',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 7 // The week that contains Jan 7th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('cy', {
  		    months: 'Ionawr_Chwefror_Mawrth_Ebrill_Mai_Mehefin_Gorffennaf_Awst_Medi_Hydref_Tachwedd_Rhagfyr'.split('_'),
  		    monthsShort: 'Ion_Chwe_Maw_Ebr_Mai_Meh_Gor_Aws_Med_Hyd_Tach_Rhag'.split('_'),
  		    weekdays: 'Dydd Sul_Dydd Llun_Dydd Mawrth_Dydd Mercher_Dydd Iau_Dydd Gwener_Dydd Sadwrn'.split('_'),
  		    weekdaysShort: 'Sul_Llun_Maw_Mer_Iau_Gwe_Sad'.split('_'),
  		    weekdaysMin: 'Su_Ll_Ma_Me_Ia_Gw_Sa'.split('_'),
  		    weekdaysParseExact: true,
  		    // time formats are the same as en-gb
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd, D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Heddiw am] LT',
  		      nextDay: '[Yfory am] LT',
  		      nextWeek: 'dddd [am] LT',
  		      lastDay: '[Ddoe am] LT',
  		      lastWeek: 'dddd [diwethaf am] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'mewn %s',
  		      past: '%s yn ôl',
  		      s: 'ychydig eiliadau',
  		      ss: '%d eiliad',
  		      m: 'munud',
  		      mm: '%d munud',
  		      h: 'awr',
  		      hh: '%d awr',
  		      d: 'diwrnod',
  		      dd: '%d diwrnod',
  		      M: 'mis',
  		      MM: '%d mis',
  		      y: 'blwyddyn',
  		      yy: '%d flynedd'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(fed|ain|af|il|ydd|ed|eg)/,
  		    // traditional ordinal numbers above 31 are not commonly used in colloquial Welsh
  		    ordinal: function (number) {
  		      var b = number,
  		        output = '',
  		        lookup = ['', 'af', 'il', 'ydd', 'ydd', 'ed', 'ed', 'ed', 'fed', 'fed', 'fed',
  		        // 1af to 10fed
  		        'eg', 'fed', 'eg', 'eg', 'fed', 'eg', 'eg', 'fed', 'eg', 'fed' // 11eg to 20fed
  		        ];
  		      if (b > 20) {
  		        if (b === 40 || b === 50 || b === 60 || b === 80 || b === 100) {
  		          output = 'fed'; // not 30ain, 70ain or 90ain
  		        } else {
  		          output = 'ain';
  		        }
  		      } else if (b > 0) {
  		        output = lookup[b];
  		      }
  		      return number + output;
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('da', {
  		    months: 'januar_februar_marts_april_maj_juni_juli_august_september_oktober_november_december'.split('_'),
  		    monthsShort: 'jan_feb_mar_apr_maj_jun_jul_aug_sep_okt_nov_dec'.split('_'),
  		    weekdays: 'søndag_mandag_tirsdag_onsdag_torsdag_fredag_lørdag'.split('_'),
  		    weekdaysShort: 'søn_man_tir_ons_tor_fre_lør'.split('_'),
  		    weekdaysMin: 'sø_ma_ti_on_to_fr_lø'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'D. MMMM YYYY',
  		      LLL: 'D. MMMM YYYY HH:mm',
  		      LLLL: 'dddd [d.] D. MMMM YYYY [kl.] HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[i dag kl.] LT',
  		      nextDay: '[i morgen kl.] LT',
  		      nextWeek: 'på dddd [kl.] LT',
  		      lastDay: '[i går kl.] LT',
  		      lastWeek: '[i] dddd[s kl.] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'om %s',
  		      past: '%s siden',
  		      s: 'få sekunder',
  		      ss: '%d sekunder',
  		      m: 'et minut',
  		      mm: '%d minutter',
  		      h: 'en time',
  		      hh: '%d timer',
  		      d: 'en dag',
  		      dd: '%d dage',
  		      M: 'en måned',
  		      MM: '%d måneder',
  		      y: 'et år',
  		      yy: '%d år'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}\./,
  		    ordinal: '%d.',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  function processRelativeTime$1(number, withoutSuffix, key, isFuture) {
  		    var format = {
  		      m: ['eine Minute', 'einer Minute'],
  		      h: ['eine Stunde', 'einer Stunde'],
  		      d: ['ein Tag', 'einem Tag'],
  		      dd: [number + ' Tage', number + ' Tagen'],
  		      w: ['eine Woche', 'einer Woche'],
  		      M: ['ein Monat', 'einem Monat'],
  		      MM: [number + ' Monate', number + ' Monaten'],
  		      y: ['ein Jahr', 'einem Jahr'],
  		      yy: [number + ' Jahre', number + ' Jahren']
  		    };
  		    return withoutSuffix ? format[key][0] : format[key][1];
  		  }
  		  hooks.defineLocale('de-at', {
  		    months: 'Jänner_Februar_März_April_Mai_Juni_Juli_August_September_Oktober_November_Dezember'.split('_'),
  		    monthsShort: 'Jän._Feb._März_Apr._Mai_Juni_Juli_Aug._Sep._Okt._Nov._Dez.'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'Sonntag_Montag_Dienstag_Mittwoch_Donnerstag_Freitag_Samstag'.split('_'),
  		    weekdaysShort: 'So._Mo._Di._Mi._Do._Fr._Sa.'.split('_'),
  		    weekdaysMin: 'So_Mo_Di_Mi_Do_Fr_Sa'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'D. MMMM YYYY',
  		      LLL: 'D. MMMM YYYY HH:mm',
  		      LLLL: 'dddd, D. MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[heute um] LT [Uhr]',
  		      sameElse: 'L',
  		      nextDay: '[morgen um] LT [Uhr]',
  		      nextWeek: 'dddd [um] LT [Uhr]',
  		      lastDay: '[gestern um] LT [Uhr]',
  		      lastWeek: '[letzten] dddd [um] LT [Uhr]'
  		    },
  		    relativeTime: {
  		      future: 'in %s',
  		      past: 'vor %s',
  		      s: 'ein paar Sekunden',
  		      ss: '%d Sekunden',
  		      m: processRelativeTime$1,
  		      mm: '%d Minuten',
  		      h: processRelativeTime$1,
  		      hh: '%d Stunden',
  		      d: processRelativeTime$1,
  		      dd: processRelativeTime$1,
  		      w: processRelativeTime$1,
  		      ww: '%d Wochen',
  		      M: processRelativeTime$1,
  		      MM: processRelativeTime$1,
  		      y: processRelativeTime$1,
  		      yy: processRelativeTime$1
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}\./,
  		    ordinal: '%d.',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  function processRelativeTime$2(number, withoutSuffix, key, isFuture) {
  		    var format = {
  		      m: ['eine Minute', 'einer Minute'],
  		      h: ['eine Stunde', 'einer Stunde'],
  		      d: ['ein Tag', 'einem Tag'],
  		      dd: [number + ' Tage', number + ' Tagen'],
  		      w: ['eine Woche', 'einer Woche'],
  		      M: ['ein Monat', 'einem Monat'],
  		      MM: [number + ' Monate', number + ' Monaten'],
  		      y: ['ein Jahr', 'einem Jahr'],
  		      yy: [number + ' Jahre', number + ' Jahren']
  		    };
  		    return withoutSuffix ? format[key][0] : format[key][1];
  		  }
  		  hooks.defineLocale('de-ch', {
  		    months: 'Januar_Februar_März_April_Mai_Juni_Juli_August_September_Oktober_November_Dezember'.split('_'),
  		    monthsShort: 'Jan._Feb._März_Apr._Mai_Juni_Juli_Aug._Sep._Okt._Nov._Dez.'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'Sonntag_Montag_Dienstag_Mittwoch_Donnerstag_Freitag_Samstag'.split('_'),
  		    weekdaysShort: 'So_Mo_Di_Mi_Do_Fr_Sa'.split('_'),
  		    weekdaysMin: 'So_Mo_Di_Mi_Do_Fr_Sa'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'D. MMMM YYYY',
  		      LLL: 'D. MMMM YYYY HH:mm',
  		      LLLL: 'dddd, D. MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[heute um] LT [Uhr]',
  		      sameElse: 'L',
  		      nextDay: '[morgen um] LT [Uhr]',
  		      nextWeek: 'dddd [um] LT [Uhr]',
  		      lastDay: '[gestern um] LT [Uhr]',
  		      lastWeek: '[letzten] dddd [um] LT [Uhr]'
  		    },
  		    relativeTime: {
  		      future: 'in %s',
  		      past: 'vor %s',
  		      s: 'ein paar Sekunden',
  		      ss: '%d Sekunden',
  		      m: processRelativeTime$2,
  		      mm: '%d Minuten',
  		      h: processRelativeTime$2,
  		      hh: '%d Stunden',
  		      d: processRelativeTime$2,
  		      dd: processRelativeTime$2,
  		      w: processRelativeTime$2,
  		      ww: '%d Wochen',
  		      M: processRelativeTime$2,
  		      MM: processRelativeTime$2,
  		      y: processRelativeTime$2,
  		      yy: processRelativeTime$2
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}\./,
  		    ordinal: '%d.',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  function processRelativeTime$3(number, withoutSuffix, key, isFuture) {
  		    var format = {
  		      m: ['eine Minute', 'einer Minute'],
  		      h: ['eine Stunde', 'einer Stunde'],
  		      d: ['ein Tag', 'einem Tag'],
  		      dd: [number + ' Tage', number + ' Tagen'],
  		      w: ['eine Woche', 'einer Woche'],
  		      M: ['ein Monat', 'einem Monat'],
  		      MM: [number + ' Monate', number + ' Monaten'],
  		      y: ['ein Jahr', 'einem Jahr'],
  		      yy: [number + ' Jahre', number + ' Jahren']
  		    };
  		    return withoutSuffix ? format[key][0] : format[key][1];
  		  }
  		  hooks.defineLocale('de', {
  		    months: 'Januar_Februar_März_April_Mai_Juni_Juli_August_September_Oktober_November_Dezember'.split('_'),
  		    monthsShort: 'Jan._Feb._März_Apr._Mai_Juni_Juli_Aug._Sep._Okt._Nov._Dez.'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'Sonntag_Montag_Dienstag_Mittwoch_Donnerstag_Freitag_Samstag'.split('_'),
  		    weekdaysShort: 'So._Mo._Di._Mi._Do._Fr._Sa.'.split('_'),
  		    weekdaysMin: 'So_Mo_Di_Mi_Do_Fr_Sa'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'D. MMMM YYYY',
  		      LLL: 'D. MMMM YYYY HH:mm',
  		      LLLL: 'dddd, D. MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[heute um] LT [Uhr]',
  		      sameElse: 'L',
  		      nextDay: '[morgen um] LT [Uhr]',
  		      nextWeek: 'dddd [um] LT [Uhr]',
  		      lastDay: '[gestern um] LT [Uhr]',
  		      lastWeek: '[letzten] dddd [um] LT [Uhr]'
  		    },
  		    relativeTime: {
  		      future: 'in %s',
  		      past: 'vor %s',
  		      s: 'ein paar Sekunden',
  		      ss: '%d Sekunden',
  		      m: processRelativeTime$3,
  		      mm: '%d Minuten',
  		      h: processRelativeTime$3,
  		      hh: '%d Stunden',
  		      d: processRelativeTime$3,
  		      dd: processRelativeTime$3,
  		      w: processRelativeTime$3,
  		      ww: '%d Wochen',
  		      M: processRelativeTime$3,
  		      MM: processRelativeTime$3,
  		      y: processRelativeTime$3,
  		      yy: processRelativeTime$3
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}\./,
  		    ordinal: '%d.',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var months$5 = ['ޖެނުއަރީ', 'ފެބްރުއަރީ', 'މާރިޗު', 'އޭޕްރީލު', 'މޭ', 'ޖޫން', 'ޖުލައި', 'އޯގަސްޓު', 'ސެޕްޓެމްބަރު', 'އޮކްޓޯބަރު', 'ނޮވެމްބަރު', 'ޑިސެމްބަރު'],
  		    weekdays = ['އާދިއްތަ', 'ހޯމަ', 'އަންގާރަ', 'ބުދަ', 'ބުރާސްފަތި', 'ހުކުރު', 'ހޮނިހިރު'];
  		  hooks.defineLocale('dv', {
  		    months: months$5,
  		    monthsShort: months$5,
  		    weekdays: weekdays,
  		    weekdaysShort: weekdays,
  		    weekdaysMin: 'އާދި_ހޯމަ_އަން_ބުދަ_ބުރާ_ހުކު_ހޮނި'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'D/M/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd D MMMM YYYY HH:mm'
  		    },
  		    meridiemParse: /މކ|މފ/,
  		    isPM: function (input) {
  		      return 'މފ' === input;
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 12) {
  		        return 'މކ';
  		      } else {
  		        return 'މފ';
  		      }
  		    },
  		    calendar: {
  		      sameDay: '[މިއަދު] LT',
  		      nextDay: '[މާދަމާ] LT',
  		      nextWeek: 'dddd LT',
  		      lastDay: '[އިއްޔެ] LT',
  		      lastWeek: '[ފާއިތުވި] dddd LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'ތެރޭގައި %s',
  		      past: 'ކުރިން %s',
  		      s: 'ސިކުންތުކޮޅެއް',
  		      ss: 'd% ސިކުންތު',
  		      m: 'މިނިޓެއް',
  		      mm: 'މިނިޓު %d',
  		      h: 'ގަޑިއިރެއް',
  		      hh: 'ގަޑިއިރު %d',
  		      d: 'ދުވަހެއް',
  		      dd: 'ދުވަސް %d',
  		      M: 'މަހެއް',
  		      MM: 'މަސް %d',
  		      y: 'އަހަރެއް',
  		      yy: 'އަހަރު %d'
  		    },
  		    preparse: function (string) {
  		      return string.replace(/،/g, ',');
  		    },
  		    postformat: function (string) {
  		      return string.replace(/,/g, '،');
  		    },
  		    week: {
  		      dow: 7,
  		      // Sunday is the first day of the week.
  		      doy: 12 // The week that contains Jan 12th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  function isFunction$1(input) {
  		    return typeof Function !== 'undefined' && input instanceof Function || Object.prototype.toString.call(input) === '[object Function]';
  		  }
  		  hooks.defineLocale('el', {
  		    monthsNominativeEl: 'Ιανουάριος_Φεβρουάριος_Μάρτιος_Απρίλιος_Μάιος_Ιούνιος_Ιούλιος_Αύγουστος_Σεπτέμβριος_Οκτώβριος_Νοέμβριος_Δεκέμβριος'.split('_'),
  		    monthsGenitiveEl: 'Ιανουαρίου_Φεβρουαρίου_Μαρτίου_Απριλίου_Μαΐου_Ιουνίου_Ιουλίου_Αυγούστου_Σεπτεμβρίου_Οκτωβρίου_Νοεμβρίου_Δεκεμβρίου'.split('_'),
  		    months: function (momentToFormat, format) {
  		      if (!momentToFormat) {
  		        return this._monthsNominativeEl;
  		      } else if (typeof format === 'string' && /D/.test(format.substring(0, format.indexOf('MMMM')))) {
  		        // if there is a day number before 'MMMM'
  		        return this._monthsGenitiveEl[momentToFormat.month()];
  		      } else {
  		        return this._monthsNominativeEl[momentToFormat.month()];
  		      }
  		    },
  		    monthsShort: 'Ιαν_Φεβ_Μαρ_Απρ_Μαϊ_Ιουν_Ιουλ_Αυγ_Σεπ_Οκτ_Νοε_Δεκ'.split('_'),
  		    weekdays: 'Κυριακή_Δευτέρα_Τρίτη_Τετάρτη_Πέμπτη_Παρασκευή_Σάββατο'.split('_'),
  		    weekdaysShort: 'Κυρ_Δευ_Τρι_Τετ_Πεμ_Παρ_Σαβ'.split('_'),
  		    weekdaysMin: 'Κυ_Δε_Τρ_Τε_Πε_Πα_Σα'.split('_'),
  		    meridiem: function (hours, minutes, isLower) {
  		      if (hours > 11) {
  		        return isLower ? 'μμ' : 'ΜΜ';
  		      } else {
  		        return isLower ? 'πμ' : 'ΠΜ';
  		      }
  		    },
  		    isPM: function (input) {
  		      return (input + '').toLowerCase()[0] === 'μ';
  		    },
  		    meridiemParse: /[ΠΜ]\.?Μ?\.?/i,
  		    longDateFormat: {
  		      LT: 'h:mm A',
  		      LTS: 'h:mm:ss A',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY h:mm A',
  		      LLLL: 'dddd, D MMMM YYYY h:mm A'
  		    },
  		    calendarEl: {
  		      sameDay: '[Σήμερα {}] LT',
  		      nextDay: '[Αύριο {}] LT',
  		      nextWeek: 'dddd [{}] LT',
  		      lastDay: '[Χθες {}] LT',
  		      lastWeek: function () {
  		        switch (this.day()) {
  		          case 6:
  		            return '[το προηγούμενο] dddd [{}] LT';
  		          default:
  		            return '[την προηγούμενη] dddd [{}] LT';
  		        }
  		      },
  		      sameElse: 'L'
  		    },
  		    calendar: function (key, mom) {
  		      var output = this._calendarEl[key],
  		        hours = mom && mom.hours();
  		      if (isFunction$1(output)) {
  		        output = output.apply(mom);
  		      }
  		      return output.replace('{}', hours % 12 === 1 ? 'στη' : 'στις');
  		    },
  		    relativeTime: {
  		      future: 'σε %s',
  		      past: '%s πριν',
  		      s: 'λίγα δευτερόλεπτα',
  		      ss: '%d δευτερόλεπτα',
  		      m: 'ένα λεπτό',
  		      mm: '%d λεπτά',
  		      h: 'μία ώρα',
  		      hh: '%d ώρες',
  		      d: 'μία μέρα',
  		      dd: '%d μέρες',
  		      M: 'ένας μήνας',
  		      MM: '%d μήνες',
  		      y: 'ένας χρόνος',
  		      yy: '%d χρόνια'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}η/,
  		    ordinal: '%dη',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4st is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('en-au', {
  		    months: 'January_February_March_April_May_June_July_August_September_October_November_December'.split('_'),
  		    monthsShort: 'Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec'.split('_'),
  		    weekdays: 'Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday'.split('_'),
  		    weekdaysShort: 'Sun_Mon_Tue_Wed_Thu_Fri_Sat'.split('_'),
  		    weekdaysMin: 'Su_Mo_Tu_We_Th_Fr_Sa'.split('_'),
  		    longDateFormat: {
  		      LT: 'h:mm A',
  		      LTS: 'h:mm:ss A',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY h:mm A',
  		      LLLL: 'dddd, D MMMM YYYY h:mm A'
  		    },
  		    calendar: {
  		      sameDay: '[Today at] LT',
  		      nextDay: '[Tomorrow at] LT',
  		      nextWeek: 'dddd [at] LT',
  		      lastDay: '[Yesterday at] LT',
  		      lastWeek: '[Last] dddd [at] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'in %s',
  		      past: '%s ago',
  		      s: 'a few seconds',
  		      ss: '%d seconds',
  		      m: 'a minute',
  		      mm: '%d minutes',
  		      h: 'an hour',
  		      hh: '%d hours',
  		      d: 'a day',
  		      dd: '%d days',
  		      M: 'a month',
  		      MM: '%d months',
  		      y: 'a year',
  		      yy: '%d years'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(st|nd|rd|th)/,
  		    ordinal: function (number) {
  		      var b = number % 10,
  		        output = ~~(number % 100 / 10) === 1 ? 'th' : b === 1 ? 'st' : b === 2 ? 'nd' : b === 3 ? 'rd' : 'th';
  		      return number + output;
  		    },
  		    week: {
  		      dow: 0,
  		      // Sunday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('en-ca', {
  		    months: 'January_February_March_April_May_June_July_August_September_October_November_December'.split('_'),
  		    monthsShort: 'Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec'.split('_'),
  		    weekdays: 'Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday'.split('_'),
  		    weekdaysShort: 'Sun_Mon_Tue_Wed_Thu_Fri_Sat'.split('_'),
  		    weekdaysMin: 'Su_Mo_Tu_We_Th_Fr_Sa'.split('_'),
  		    longDateFormat: {
  		      LT: 'h:mm A',
  		      LTS: 'h:mm:ss A',
  		      L: 'YYYY-MM-DD',
  		      LL: 'MMMM D, YYYY',
  		      LLL: 'MMMM D, YYYY h:mm A',
  		      LLLL: 'dddd, MMMM D, YYYY h:mm A'
  		    },
  		    calendar: {
  		      sameDay: '[Today at] LT',
  		      nextDay: '[Tomorrow at] LT',
  		      nextWeek: 'dddd [at] LT',
  		      lastDay: '[Yesterday at] LT',
  		      lastWeek: '[Last] dddd [at] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'in %s',
  		      past: '%s ago',
  		      s: 'a few seconds',
  		      ss: '%d seconds',
  		      m: 'a minute',
  		      mm: '%d minutes',
  		      h: 'an hour',
  		      hh: '%d hours',
  		      d: 'a day',
  		      dd: '%d days',
  		      M: 'a month',
  		      MM: '%d months',
  		      y: 'a year',
  		      yy: '%d years'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(st|nd|rd|th)/,
  		    ordinal: function (number) {
  		      var b = number % 10,
  		        output = ~~(number % 100 / 10) === 1 ? 'th' : b === 1 ? 'st' : b === 2 ? 'nd' : b === 3 ? 'rd' : 'th';
  		      return number + output;
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('en-gb', {
  		    months: 'January_February_March_April_May_June_July_August_September_October_November_December'.split('_'),
  		    monthsShort: 'Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec'.split('_'),
  		    weekdays: 'Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday'.split('_'),
  		    weekdaysShort: 'Sun_Mon_Tue_Wed_Thu_Fri_Sat'.split('_'),
  		    weekdaysMin: 'Su_Mo_Tu_We_Th_Fr_Sa'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd, D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Today at] LT',
  		      nextDay: '[Tomorrow at] LT',
  		      nextWeek: 'dddd [at] LT',
  		      lastDay: '[Yesterday at] LT',
  		      lastWeek: '[Last] dddd [at] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'in %s',
  		      past: '%s ago',
  		      s: 'a few seconds',
  		      ss: '%d seconds',
  		      m: 'a minute',
  		      mm: '%d minutes',
  		      h: 'an hour',
  		      hh: '%d hours',
  		      d: 'a day',
  		      dd: '%d days',
  		      M: 'a month',
  		      MM: '%d months',
  		      y: 'a year',
  		      yy: '%d years'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(st|nd|rd|th)/,
  		    ordinal: function (number) {
  		      var b = number % 10,
  		        output = ~~(number % 100 / 10) === 1 ? 'th' : b === 1 ? 'st' : b === 2 ? 'nd' : b === 3 ? 'rd' : 'th';
  		      return number + output;
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('en-ie', {
  		    months: 'January_February_March_April_May_June_July_August_September_October_November_December'.split('_'),
  		    monthsShort: 'Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec'.split('_'),
  		    weekdays: 'Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday'.split('_'),
  		    weekdaysShort: 'Sun_Mon_Tue_Wed_Thu_Fri_Sat'.split('_'),
  		    weekdaysMin: 'Su_Mo_Tu_We_Th_Fr_Sa'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Today at] LT',
  		      nextDay: '[Tomorrow at] LT',
  		      nextWeek: 'dddd [at] LT',
  		      lastDay: '[Yesterday at] LT',
  		      lastWeek: '[Last] dddd [at] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'in %s',
  		      past: '%s ago',
  		      s: 'a few seconds',
  		      ss: '%d seconds',
  		      m: 'a minute',
  		      mm: '%d minutes',
  		      h: 'an hour',
  		      hh: '%d hours',
  		      d: 'a day',
  		      dd: '%d days',
  		      M: 'a month',
  		      MM: '%d months',
  		      y: 'a year',
  		      yy: '%d years'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(st|nd|rd|th)/,
  		    ordinal: function (number) {
  		      var b = number % 10,
  		        output = ~~(number % 100 / 10) === 1 ? 'th' : b === 1 ? 'st' : b === 2 ? 'nd' : b === 3 ? 'rd' : 'th';
  		      return number + output;
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('en-il', {
  		    months: 'January_February_March_April_May_June_July_August_September_October_November_December'.split('_'),
  		    monthsShort: 'Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec'.split('_'),
  		    weekdays: 'Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday'.split('_'),
  		    weekdaysShort: 'Sun_Mon_Tue_Wed_Thu_Fri_Sat'.split('_'),
  		    weekdaysMin: 'Su_Mo_Tu_We_Th_Fr_Sa'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd, D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Today at] LT',
  		      nextDay: '[Tomorrow at] LT',
  		      nextWeek: 'dddd [at] LT',
  		      lastDay: '[Yesterday at] LT',
  		      lastWeek: '[Last] dddd [at] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'in %s',
  		      past: '%s ago',
  		      s: 'a few seconds',
  		      ss: '%d seconds',
  		      m: 'a minute',
  		      mm: '%d minutes',
  		      h: 'an hour',
  		      hh: '%d hours',
  		      d: 'a day',
  		      dd: '%d days',
  		      M: 'a month',
  		      MM: '%d months',
  		      y: 'a year',
  		      yy: '%d years'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(st|nd|rd|th)/,
  		    ordinal: function (number) {
  		      var b = number % 10,
  		        output = ~~(number % 100 / 10) === 1 ? 'th' : b === 1 ? 'st' : b === 2 ? 'nd' : b === 3 ? 'rd' : 'th';
  		      return number + output;
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('en-in', {
  		    months: 'January_February_March_April_May_June_July_August_September_October_November_December'.split('_'),
  		    monthsShort: 'Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec'.split('_'),
  		    weekdays: 'Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday'.split('_'),
  		    weekdaysShort: 'Sun_Mon_Tue_Wed_Thu_Fri_Sat'.split('_'),
  		    weekdaysMin: 'Su_Mo_Tu_We_Th_Fr_Sa'.split('_'),
  		    longDateFormat: {
  		      LT: 'h:mm A',
  		      LTS: 'h:mm:ss A',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY h:mm A',
  		      LLLL: 'dddd, D MMMM YYYY h:mm A'
  		    },
  		    calendar: {
  		      sameDay: '[Today at] LT',
  		      nextDay: '[Tomorrow at] LT',
  		      nextWeek: 'dddd [at] LT',
  		      lastDay: '[Yesterday at] LT',
  		      lastWeek: '[Last] dddd [at] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'in %s',
  		      past: '%s ago',
  		      s: 'a few seconds',
  		      ss: '%d seconds',
  		      m: 'a minute',
  		      mm: '%d minutes',
  		      h: 'an hour',
  		      hh: '%d hours',
  		      d: 'a day',
  		      dd: '%d days',
  		      M: 'a month',
  		      MM: '%d months',
  		      y: 'a year',
  		      yy: '%d years'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(st|nd|rd|th)/,
  		    ordinal: function (number) {
  		      var b = number % 10,
  		        output = ~~(number % 100 / 10) === 1 ? 'th' : b === 1 ? 'st' : b === 2 ? 'nd' : b === 3 ? 'rd' : 'th';
  		      return number + output;
  		    },
  		    week: {
  		      dow: 0,
  		      // Sunday is the first day of the week.
  		      doy: 6 // The week that contains Jan 1st is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('en-nz', {
  		    months: 'January_February_March_April_May_June_July_August_September_October_November_December'.split('_'),
  		    monthsShort: 'Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec'.split('_'),
  		    weekdays: 'Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday'.split('_'),
  		    weekdaysShort: 'Sun_Mon_Tue_Wed_Thu_Fri_Sat'.split('_'),
  		    weekdaysMin: 'Su_Mo_Tu_We_Th_Fr_Sa'.split('_'),
  		    longDateFormat: {
  		      LT: 'h:mm A',
  		      LTS: 'h:mm:ss A',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY h:mm A',
  		      LLLL: 'dddd, D MMMM YYYY h:mm A'
  		    },
  		    calendar: {
  		      sameDay: '[Today at] LT',
  		      nextDay: '[Tomorrow at] LT',
  		      nextWeek: 'dddd [at] LT',
  		      lastDay: '[Yesterday at] LT',
  		      lastWeek: '[Last] dddd [at] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'in %s',
  		      past: '%s ago',
  		      s: 'a few seconds',
  		      ss: '%d seconds',
  		      m: 'a minute',
  		      mm: '%d minutes',
  		      h: 'an hour',
  		      hh: '%d hours',
  		      d: 'a day',
  		      dd: '%d days',
  		      M: 'a month',
  		      MM: '%d months',
  		      y: 'a year',
  		      yy: '%d years'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(st|nd|rd|th)/,
  		    ordinal: function (number) {
  		      var b = number % 10,
  		        output = ~~(number % 100 / 10) === 1 ? 'th' : b === 1 ? 'st' : b === 2 ? 'nd' : b === 3 ? 'rd' : 'th';
  		      return number + output;
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('en-sg', {
  		    months: 'January_February_March_April_May_June_July_August_September_October_November_December'.split('_'),
  		    monthsShort: 'Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec'.split('_'),
  		    weekdays: 'Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday'.split('_'),
  		    weekdaysShort: 'Sun_Mon_Tue_Wed_Thu_Fri_Sat'.split('_'),
  		    weekdaysMin: 'Su_Mo_Tu_We_Th_Fr_Sa'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd, D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Today at] LT',
  		      nextDay: '[Tomorrow at] LT',
  		      nextWeek: 'dddd [at] LT',
  		      lastDay: '[Yesterday at] LT',
  		      lastWeek: '[Last] dddd [at] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'in %s',
  		      past: '%s ago',
  		      s: 'a few seconds',
  		      ss: '%d seconds',
  		      m: 'a minute',
  		      mm: '%d minutes',
  		      h: 'an hour',
  		      hh: '%d hours',
  		      d: 'a day',
  		      dd: '%d days',
  		      M: 'a month',
  		      MM: '%d months',
  		      y: 'a year',
  		      yy: '%d years'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(st|nd|rd|th)/,
  		    ordinal: function (number) {
  		      var b = number % 10,
  		        output = ~~(number % 100 / 10) === 1 ? 'th' : b === 1 ? 'st' : b === 2 ? 'nd' : b === 3 ? 'rd' : 'th';
  		      return number + output;
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('eo', {
  		    months: 'januaro_februaro_marto_aprilo_majo_junio_julio_aŭgusto_septembro_oktobro_novembro_decembro'.split('_'),
  		    monthsShort: 'jan_feb_mart_apr_maj_jun_jul_aŭg_sept_okt_nov_dec'.split('_'),
  		    weekdays: 'dimanĉo_lundo_mardo_merkredo_ĵaŭdo_vendredo_sabato'.split('_'),
  		    weekdaysShort: 'dim_lun_mard_merk_ĵaŭ_ven_sab'.split('_'),
  		    weekdaysMin: 'di_lu_ma_me_ĵa_ve_sa'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'YYYY-MM-DD',
  		      LL: '[la] D[-an de] MMMM, YYYY',
  		      LLL: '[la] D[-an de] MMMM, YYYY HH:mm',
  		      LLLL: 'dddd[n], [la] D[-an de] MMMM, YYYY HH:mm',
  		      llll: 'ddd, [la] D[-an de] MMM, YYYY HH:mm'
  		    },
  		    meridiemParse: /[ap]\.t\.m/i,
  		    isPM: function (input) {
  		      return input.charAt(0).toLowerCase() === 'p';
  		    },
  		    meridiem: function (hours, minutes, isLower) {
  		      if (hours > 11) {
  		        return isLower ? 'p.t.m.' : 'P.T.M.';
  		      } else {
  		        return isLower ? 'a.t.m.' : 'A.T.M.';
  		      }
  		    },
  		    calendar: {
  		      sameDay: '[Hodiaŭ je] LT',
  		      nextDay: '[Morgaŭ je] LT',
  		      nextWeek: 'dddd[n je] LT',
  		      lastDay: '[Hieraŭ je] LT',
  		      lastWeek: '[pasintan] dddd[n je] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'post %s',
  		      past: 'antaŭ %s',
  		      s: 'kelkaj sekundoj',
  		      ss: '%d sekundoj',
  		      m: 'unu minuto',
  		      mm: '%d minutoj',
  		      h: 'unu horo',
  		      hh: '%d horoj',
  		      d: 'unu tago',
  		      //ne 'diurno', ĉar estas uzita por proksimumo
  		      dd: '%d tagoj',
  		      M: 'unu monato',
  		      MM: '%d monatoj',
  		      y: 'unu jaro',
  		      yy: '%d jaroj'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}a/,
  		    ordinal: '%da',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 7 // The week that contains Jan 7th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var monthsShortDot = 'ene._feb._mar._abr._may._jun._jul._ago._sep._oct._nov._dic.'.split('_'),
  		    monthsShort$1 = 'ene_feb_mar_abr_may_jun_jul_ago_sep_oct_nov_dic'.split('_'),
  		    monthsParse$2 = [/^ene/i, /^feb/i, /^mar/i, /^abr/i, /^may/i, /^jun/i, /^jul/i, /^ago/i, /^sep/i, /^oct/i, /^nov/i, /^dic/i],
  		    monthsRegex$3 = /^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre|ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i;
  		  hooks.defineLocale('es-do', {
  		    months: 'enero_febrero_marzo_abril_mayo_junio_julio_agosto_septiembre_octubre_noviembre_diciembre'.split('_'),
  		    monthsShort: function (m, format) {
  		      if (!m) {
  		        return monthsShortDot;
  		      } else if (/-MMM-/.test(format)) {
  		        return monthsShort$1[m.month()];
  		      } else {
  		        return monthsShortDot[m.month()];
  		      }
  		    },
  		    monthsRegex: monthsRegex$3,
  		    monthsShortRegex: monthsRegex$3,
  		    monthsStrictRegex: /^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre)/i,
  		    monthsShortStrictRegex: /^(ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,
  		    monthsParse: monthsParse$2,
  		    longMonthsParse: monthsParse$2,
  		    shortMonthsParse: monthsParse$2,
  		    weekdays: 'domingo_lunes_martes_miércoles_jueves_viernes_sábado'.split('_'),
  		    weekdaysShort: 'dom._lun._mar._mié._jue._vie._sáb.'.split('_'),
  		    weekdaysMin: 'do_lu_ma_mi_ju_vi_sá'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'h:mm A',
  		      LTS: 'h:mm:ss A',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D [de] MMMM [de] YYYY',
  		      LLL: 'D [de] MMMM [de] YYYY h:mm A',
  		      LLLL: 'dddd, D [de] MMMM [de] YYYY h:mm A'
  		    },
  		    calendar: {
  		      sameDay: function () {
  		        return '[hoy a la' + (this.hours() !== 1 ? 's' : '') + '] LT';
  		      },
  		      nextDay: function () {
  		        return '[mañana a la' + (this.hours() !== 1 ? 's' : '') + '] LT';
  		      },
  		      nextWeek: function () {
  		        return 'dddd [a la' + (this.hours() !== 1 ? 's' : '') + '] LT';
  		      },
  		      lastDay: function () {
  		        return '[ayer a la' + (this.hours() !== 1 ? 's' : '') + '] LT';
  		      },
  		      lastWeek: function () {
  		        return '[el] dddd [pasado a la' + (this.hours() !== 1 ? 's' : '') + '] LT';
  		      },
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'en %s',
  		      past: 'hace %s',
  		      s: 'unos segundos',
  		      ss: '%d segundos',
  		      m: 'un minuto',
  		      mm: '%d minutos',
  		      h: 'una hora',
  		      hh: '%d horas',
  		      d: 'un día',
  		      dd: '%d días',
  		      w: 'una semana',
  		      ww: '%d semanas',
  		      M: 'un mes',
  		      MM: '%d meses',
  		      y: 'un año',
  		      yy: '%d años'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}º/,
  		    ordinal: '%dº',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var monthsShortDot$1 = 'ene._feb._mar._abr._may._jun._jul._ago._sep._oct._nov._dic.'.split('_'),
  		    monthsShort$2 = 'ene_feb_mar_abr_may_jun_jul_ago_sep_oct_nov_dic'.split('_'),
  		    monthsParse$3 = [/^ene/i, /^feb/i, /^mar/i, /^abr/i, /^may/i, /^jun/i, /^jul/i, /^ago/i, /^sep/i, /^oct/i, /^nov/i, /^dic/i],
  		    monthsRegex$4 = /^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre|ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i;
  		  hooks.defineLocale('es-mx', {
  		    months: 'enero_febrero_marzo_abril_mayo_junio_julio_agosto_septiembre_octubre_noviembre_diciembre'.split('_'),
  		    monthsShort: function (m, format) {
  		      if (!m) {
  		        return monthsShortDot$1;
  		      } else if (/-MMM-/.test(format)) {
  		        return monthsShort$2[m.month()];
  		      } else {
  		        return monthsShortDot$1[m.month()];
  		      }
  		    },
  		    monthsRegex: monthsRegex$4,
  		    monthsShortRegex: monthsRegex$4,
  		    monthsStrictRegex: /^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre)/i,
  		    monthsShortStrictRegex: /^(ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,
  		    monthsParse: monthsParse$3,
  		    longMonthsParse: monthsParse$3,
  		    shortMonthsParse: monthsParse$3,
  		    weekdays: 'domingo_lunes_martes_miércoles_jueves_viernes_sábado'.split('_'),
  		    weekdaysShort: 'dom._lun._mar._mié._jue._vie._sáb.'.split('_'),
  		    weekdaysMin: 'do_lu_ma_mi_ju_vi_sá'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'H:mm',
  		      LTS: 'H:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D [de] MMMM [de] YYYY',
  		      LLL: 'D [de] MMMM [de] YYYY H:mm',
  		      LLLL: 'dddd, D [de] MMMM [de] YYYY H:mm'
  		    },
  		    calendar: {
  		      sameDay: function () {
  		        return '[hoy a la' + (this.hours() !== 1 ? 's' : '') + '] LT';
  		      },
  		      nextDay: function () {
  		        return '[mañana a la' + (this.hours() !== 1 ? 's' : '') + '] LT';
  		      },
  		      nextWeek: function () {
  		        return 'dddd [a la' + (this.hours() !== 1 ? 's' : '') + '] LT';
  		      },
  		      lastDay: function () {
  		        return '[ayer a la' + (this.hours() !== 1 ? 's' : '') + '] LT';
  		      },
  		      lastWeek: function () {
  		        return '[el] dddd [pasado a la' + (this.hours() !== 1 ? 's' : '') + '] LT';
  		      },
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'en %s',
  		      past: 'hace %s',
  		      s: 'unos segundos',
  		      ss: '%d segundos',
  		      m: 'un minuto',
  		      mm: '%d minutos',
  		      h: 'una hora',
  		      hh: '%d horas',
  		      d: 'un día',
  		      dd: '%d días',
  		      w: 'una semana',
  		      ww: '%d semanas',
  		      M: 'un mes',
  		      MM: '%d meses',
  		      y: 'un año',
  		      yy: '%d años'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}º/,
  		    ordinal: '%dº',
  		    week: {
  		      dow: 0,
  		      // Sunday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    },
  		    invalidDate: 'Fecha inválida'
  		  });

  		  //! moment.js locale configuration

  		  var monthsShortDot$2 = 'ene._feb._mar._abr._may._jun._jul._ago._sep._oct._nov._dic.'.split('_'),
  		    monthsShort$3 = 'ene_feb_mar_abr_may_jun_jul_ago_sep_oct_nov_dic'.split('_'),
  		    monthsParse$4 = [/^ene/i, /^feb/i, /^mar/i, /^abr/i, /^may/i, /^jun/i, /^jul/i, /^ago/i, /^sep/i, /^oct/i, /^nov/i, /^dic/i],
  		    monthsRegex$5 = /^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre|ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i;
  		  hooks.defineLocale('es-us', {
  		    months: 'enero_febrero_marzo_abril_mayo_junio_julio_agosto_septiembre_octubre_noviembre_diciembre'.split('_'),
  		    monthsShort: function (m, format) {
  		      if (!m) {
  		        return monthsShortDot$2;
  		      } else if (/-MMM-/.test(format)) {
  		        return monthsShort$3[m.month()];
  		      } else {
  		        return monthsShortDot$2[m.month()];
  		      }
  		    },
  		    monthsRegex: monthsRegex$5,
  		    monthsShortRegex: monthsRegex$5,
  		    monthsStrictRegex: /^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre)/i,
  		    monthsShortStrictRegex: /^(ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,
  		    monthsParse: monthsParse$4,
  		    longMonthsParse: monthsParse$4,
  		    shortMonthsParse: monthsParse$4,
  		    weekdays: 'domingo_lunes_martes_miércoles_jueves_viernes_sábado'.split('_'),
  		    weekdaysShort: 'dom._lun._mar._mié._jue._vie._sáb.'.split('_'),
  		    weekdaysMin: 'do_lu_ma_mi_ju_vi_sá'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'h:mm A',
  		      LTS: 'h:mm:ss A',
  		      L: 'MM/DD/YYYY',
  		      LL: 'D [de] MMMM [de] YYYY',
  		      LLL: 'D [de] MMMM [de] YYYY h:mm A',
  		      LLLL: 'dddd, D [de] MMMM [de] YYYY h:mm A'
  		    },
  		    calendar: {
  		      sameDay: function () {
  		        return '[hoy a la' + (this.hours() !== 1 ? 's' : '') + '] LT';
  		      },
  		      nextDay: function () {
  		        return '[mañana a la' + (this.hours() !== 1 ? 's' : '') + '] LT';
  		      },
  		      nextWeek: function () {
  		        return 'dddd [a la' + (this.hours() !== 1 ? 's' : '') + '] LT';
  		      },
  		      lastDay: function () {
  		        return '[ayer a la' + (this.hours() !== 1 ? 's' : '') + '] LT';
  		      },
  		      lastWeek: function () {
  		        return '[el] dddd [pasado a la' + (this.hours() !== 1 ? 's' : '') + '] LT';
  		      },
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'en %s',
  		      past: 'hace %s',
  		      s: 'unos segundos',
  		      ss: '%d segundos',
  		      m: 'un minuto',
  		      mm: '%d minutos',
  		      h: 'una hora',
  		      hh: '%d horas',
  		      d: 'un día',
  		      dd: '%d días',
  		      w: 'una semana',
  		      ww: '%d semanas',
  		      M: 'un mes',
  		      MM: '%d meses',
  		      y: 'un año',
  		      yy: '%d años'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}º/,
  		    ordinal: '%dº',
  		    week: {
  		      dow: 0,
  		      // Sunday is the first day of the week.
  		      doy: 6 // The week that contains Jan 6th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var monthsShortDot$3 = 'ene._feb._mar._abr._may._jun._jul._ago._sep._oct._nov._dic.'.split('_'),
  		    monthsShort$4 = 'ene_feb_mar_abr_may_jun_jul_ago_sep_oct_nov_dic'.split('_'),
  		    monthsParse$5 = [/^ene/i, /^feb/i, /^mar/i, /^abr/i, /^may/i, /^jun/i, /^jul/i, /^ago/i, /^sep/i, /^oct/i, /^nov/i, /^dic/i],
  		    monthsRegex$6 = /^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre|ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i;
  		  hooks.defineLocale('es', {
  		    months: 'enero_febrero_marzo_abril_mayo_junio_julio_agosto_septiembre_octubre_noviembre_diciembre'.split('_'),
  		    monthsShort: function (m, format) {
  		      if (!m) {
  		        return monthsShortDot$3;
  		      } else if (/-MMM-/.test(format)) {
  		        return monthsShort$4[m.month()];
  		      } else {
  		        return monthsShortDot$3[m.month()];
  		      }
  		    },
  		    monthsRegex: monthsRegex$6,
  		    monthsShortRegex: monthsRegex$6,
  		    monthsStrictRegex: /^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre)/i,
  		    monthsShortStrictRegex: /^(ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,
  		    monthsParse: monthsParse$5,
  		    longMonthsParse: monthsParse$5,
  		    shortMonthsParse: monthsParse$5,
  		    weekdays: 'domingo_lunes_martes_miércoles_jueves_viernes_sábado'.split('_'),
  		    weekdaysShort: 'dom._lun._mar._mié._jue._vie._sáb.'.split('_'),
  		    weekdaysMin: 'do_lu_ma_mi_ju_vi_sá'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'H:mm',
  		      LTS: 'H:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D [de] MMMM [de] YYYY',
  		      LLL: 'D [de] MMMM [de] YYYY H:mm',
  		      LLLL: 'dddd, D [de] MMMM [de] YYYY H:mm'
  		    },
  		    calendar: {
  		      sameDay: function () {
  		        return '[hoy a la' + (this.hours() !== 1 ? 's' : '') + '] LT';
  		      },
  		      nextDay: function () {
  		        return '[mañana a la' + (this.hours() !== 1 ? 's' : '') + '] LT';
  		      },
  		      nextWeek: function () {
  		        return 'dddd [a la' + (this.hours() !== 1 ? 's' : '') + '] LT';
  		      },
  		      lastDay: function () {
  		        return '[ayer a la' + (this.hours() !== 1 ? 's' : '') + '] LT';
  		      },
  		      lastWeek: function () {
  		        return '[el] dddd [pasado a la' + (this.hours() !== 1 ? 's' : '') + '] LT';
  		      },
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'en %s',
  		      past: 'hace %s',
  		      s: 'unos segundos',
  		      ss: '%d segundos',
  		      m: 'un minuto',
  		      mm: '%d minutos',
  		      h: 'una hora',
  		      hh: '%d horas',
  		      d: 'un día',
  		      dd: '%d días',
  		      w: 'una semana',
  		      ww: '%d semanas',
  		      M: 'un mes',
  		      MM: '%d meses',
  		      y: 'un año',
  		      yy: '%d años'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}º/,
  		    ordinal: '%dº',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    },
  		    invalidDate: 'Fecha inválida'
  		  });

  		  //! moment.js locale configuration

  		  function processRelativeTime$4(number, withoutSuffix, key, isFuture) {
  		    var format = {
  		      s: ['mõne sekundi', 'mõni sekund', 'paar sekundit'],
  		      ss: [number + 'sekundi', number + 'sekundit'],
  		      m: ['ühe minuti', 'üks minut'],
  		      mm: [number + ' minuti', number + ' minutit'],
  		      h: ['ühe tunni', 'tund aega', 'üks tund'],
  		      hh: [number + ' tunni', number + ' tundi'],
  		      d: ['ühe päeva', 'üks päev'],
  		      M: ['kuu aja', 'kuu aega', 'üks kuu'],
  		      MM: [number + ' kuu', number + ' kuud'],
  		      y: ['ühe aasta', 'aasta', 'üks aasta'],
  		      yy: [number + ' aasta', number + ' aastat']
  		    };
  		    if (withoutSuffix) {
  		      return format[key][2] ? format[key][2] : format[key][1];
  		    }
  		    return isFuture ? format[key][0] : format[key][1];
  		  }
  		  hooks.defineLocale('et', {
  		    months: 'jaanuar_veebruar_märts_aprill_mai_juuni_juuli_august_september_oktoober_november_detsember'.split('_'),
  		    monthsShort: 'jaan_veebr_märts_apr_mai_juuni_juuli_aug_sept_okt_nov_dets'.split('_'),
  		    weekdays: 'pühapäev_esmaspäev_teisipäev_kolmapäev_neljapäev_reede_laupäev'.split('_'),
  		    weekdaysShort: 'P_E_T_K_N_R_L'.split('_'),
  		    weekdaysMin: 'P_E_T_K_N_R_L'.split('_'),
  		    longDateFormat: {
  		      LT: 'H:mm',
  		      LTS: 'H:mm:ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'D. MMMM YYYY',
  		      LLL: 'D. MMMM YYYY H:mm',
  		      LLLL: 'dddd, D. MMMM YYYY H:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Täna,] LT',
  		      nextDay: '[Homme,] LT',
  		      nextWeek: '[Järgmine] dddd LT',
  		      lastDay: '[Eile,] LT',
  		      lastWeek: '[Eelmine] dddd LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%s pärast',
  		      past: '%s tagasi',
  		      s: processRelativeTime$4,
  		      ss: processRelativeTime$4,
  		      m: processRelativeTime$4,
  		      mm: processRelativeTime$4,
  		      h: processRelativeTime$4,
  		      hh: processRelativeTime$4,
  		      d: processRelativeTime$4,
  		      dd: '%d päeva',
  		      M: processRelativeTime$4,
  		      MM: processRelativeTime$4,
  		      y: processRelativeTime$4,
  		      yy: processRelativeTime$4
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}\./,
  		    ordinal: '%d.',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('eu', {
  		    months: 'urtarrila_otsaila_martxoa_apirila_maiatza_ekaina_uztaila_abuztua_iraila_urria_azaroa_abendua'.split('_'),
  		    monthsShort: 'urt._ots._mar._api._mai._eka._uzt._abu._ira._urr._aza._abe.'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'igandea_astelehena_asteartea_asteazkena_osteguna_ostirala_larunbata'.split('_'),
  		    weekdaysShort: 'ig._al._ar._az._og._ol._lr.'.split('_'),
  		    weekdaysMin: 'ig_al_ar_az_og_ol_lr'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'YYYY-MM-DD',
  		      LL: 'YYYY[ko] MMMM[ren] D[a]',
  		      LLL: 'YYYY[ko] MMMM[ren] D[a] HH:mm',
  		      LLLL: 'dddd, YYYY[ko] MMMM[ren] D[a] HH:mm',
  		      l: 'YYYY-M-D',
  		      ll: 'YYYY[ko] MMM D[a]',
  		      lll: 'YYYY[ko] MMM D[a] HH:mm',
  		      llll: 'ddd, YYYY[ko] MMM D[a] HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[gaur] LT[etan]',
  		      nextDay: '[bihar] LT[etan]',
  		      nextWeek: 'dddd LT[etan]',
  		      lastDay: '[atzo] LT[etan]',
  		      lastWeek: '[aurreko] dddd LT[etan]',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%s barru',
  		      past: 'duela %s',
  		      s: 'segundo batzuk',
  		      ss: '%d segundo',
  		      m: 'minutu bat',
  		      mm: '%d minutu',
  		      h: 'ordu bat',
  		      hh: '%d ordu',
  		      d: 'egun bat',
  		      dd: '%d egun',
  		      M: 'hilabete bat',
  		      MM: '%d hilabete',
  		      y: 'urte bat',
  		      yy: '%d urte'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}\./,
  		    ordinal: '%d.',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 7 // The week that contains Jan 7th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var symbolMap$7 = {
  		      1: '۱',
  		      2: '۲',
  		      3: '۳',
  		      4: '۴',
  		      5: '۵',
  		      6: '۶',
  		      7: '۷',
  		      8: '۸',
  		      9: '۹',
  		      0: '۰'
  		    },
  		    numberMap$6 = {
  		      '۱': '1',
  		      '۲': '2',
  		      '۳': '3',
  		      '۴': '4',
  		      '۵': '5',
  		      '۶': '6',
  		      '۷': '7',
  		      '۸': '8',
  		      '۹': '9',
  		      '۰': '0'
  		    };
  		  hooks.defineLocale('fa', {
  		    months: 'ژانویه_فوریه_مارس_آوریل_مه_ژوئن_ژوئیه_اوت_سپتامبر_اکتبر_نوامبر_دسامبر'.split('_'),
  		    monthsShort: 'ژانویه_فوریه_مارس_آوریل_مه_ژوئن_ژوئیه_اوت_سپتامبر_اکتبر_نوامبر_دسامبر'.split('_'),
  		    weekdays: 'یک\u200cشنبه_دوشنبه_سه\u200cشنبه_چهارشنبه_پنج\u200cشنبه_جمعه_شنبه'.split('_'),
  		    weekdaysShort: 'یک\u200cشنبه_دوشنبه_سه\u200cشنبه_چهارشنبه_پنج\u200cشنبه_جمعه_شنبه'.split('_'),
  		    weekdaysMin: 'ی_د_س_چ_پ_ج_ش'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd, D MMMM YYYY HH:mm'
  		    },
  		    meridiemParse: /قبل از ظهر|بعد از ظهر/,
  		    isPM: function (input) {
  		      return /بعد از ظهر/.test(input);
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 12) {
  		        return 'قبل از ظهر';
  		      } else {
  		        return 'بعد از ظهر';
  		      }
  		    },
  		    calendar: {
  		      sameDay: '[امروز ساعت] LT',
  		      nextDay: '[فردا ساعت] LT',
  		      nextWeek: 'dddd [ساعت] LT',
  		      lastDay: '[دیروز ساعت] LT',
  		      lastWeek: 'dddd [پیش] [ساعت] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'در %s',
  		      past: '%s پیش',
  		      s: 'چند ثانیه',
  		      ss: '%d ثانیه',
  		      m: 'یک دقیقه',
  		      mm: '%d دقیقه',
  		      h: 'یک ساعت',
  		      hh: '%d ساعت',
  		      d: 'یک روز',
  		      dd: '%d روز',
  		      M: 'یک ماه',
  		      MM: '%d ماه',
  		      y: 'یک سال',
  		      yy: '%d سال'
  		    },
  		    preparse: function (string) {
  		      return string.replace(/[۰-۹]/g, function (match) {
  		        return numberMap$6[match];
  		      }).replace(/،/g, ',');
  		    },
  		    postformat: function (string) {
  		      return string.replace(/\d/g, function (match) {
  		        return symbolMap$7[match];
  		      }).replace(/,/g, '،');
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}م/,
  		    ordinal: '%dم',
  		    week: {
  		      dow: 6,
  		      // Saturday is the first day of the week.
  		      doy: 12 // The week that contains Jan 12th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var numbersPast = 'nolla yksi kaksi kolme neljä viisi kuusi seitsemän kahdeksan yhdeksän'.split(' '),
  		    numbersFuture = ['nolla', 'yhden', 'kahden', 'kolmen', 'neljän', 'viiden', 'kuuden', numbersPast[7], numbersPast[8], numbersPast[9]];
  		  function translate$2(number, withoutSuffix, key, isFuture) {
  		    var result = '';
  		    switch (key) {
  		      case 's':
  		        return isFuture ? 'muutaman sekunnin' : 'muutama sekunti';
  		      case 'ss':
  		        result = isFuture ? 'sekunnin' : 'sekuntia';
  		        break;
  		      case 'm':
  		        return isFuture ? 'minuutin' : 'minuutti';
  		      case 'mm':
  		        result = isFuture ? 'minuutin' : 'minuuttia';
  		        break;
  		      case 'h':
  		        return isFuture ? 'tunnin' : 'tunti';
  		      case 'hh':
  		        result = isFuture ? 'tunnin' : 'tuntia';
  		        break;
  		      case 'd':
  		        return isFuture ? 'päivän' : 'päivä';
  		      case 'dd':
  		        result = isFuture ? 'päivän' : 'päivää';
  		        break;
  		      case 'M':
  		        return isFuture ? 'kuukauden' : 'kuukausi';
  		      case 'MM':
  		        result = isFuture ? 'kuukauden' : 'kuukautta';
  		        break;
  		      case 'y':
  		        return isFuture ? 'vuoden' : 'vuosi';
  		      case 'yy':
  		        result = isFuture ? 'vuoden' : 'vuotta';
  		        break;
  		    }
  		    result = verbalNumber(number, isFuture) + ' ' + result;
  		    return result;
  		  }
  		  function verbalNumber(number, isFuture) {
  		    return number < 10 ? isFuture ? numbersFuture[number] : numbersPast[number] : number;
  		  }
  		  hooks.defineLocale('fi', {
  		    months: 'tammikuu_helmikuu_maaliskuu_huhtikuu_toukokuu_kesäkuu_heinäkuu_elokuu_syyskuu_lokakuu_marraskuu_joulukuu'.split('_'),
  		    monthsShort: 'tammi_helmi_maalis_huhti_touko_kesä_heinä_elo_syys_loka_marras_joulu'.split('_'),
  		    weekdays: 'sunnuntai_maanantai_tiistai_keskiviikko_torstai_perjantai_lauantai'.split('_'),
  		    weekdaysShort: 'su_ma_ti_ke_to_pe_la'.split('_'),
  		    weekdaysMin: 'su_ma_ti_ke_to_pe_la'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH.mm',
  		      LTS: 'HH.mm.ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'Do MMMM[ta] YYYY',
  		      LLL: 'Do MMMM[ta] YYYY, [klo] HH.mm',
  		      LLLL: 'dddd, Do MMMM[ta] YYYY, [klo] HH.mm',
  		      l: 'D.M.YYYY',
  		      ll: 'Do MMM YYYY',
  		      lll: 'Do MMM YYYY, [klo] HH.mm',
  		      llll: 'ddd, Do MMM YYYY, [klo] HH.mm'
  		    },
  		    calendar: {
  		      sameDay: '[tänään] [klo] LT',
  		      nextDay: '[huomenna] [klo] LT',
  		      nextWeek: 'dddd [klo] LT',
  		      lastDay: '[eilen] [klo] LT',
  		      lastWeek: '[viime] dddd[na] [klo] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%s päästä',
  		      past: '%s sitten',
  		      s: translate$2,
  		      ss: translate$2,
  		      m: translate$2,
  		      mm: translate$2,
  		      h: translate$2,
  		      hh: translate$2,
  		      d: translate$2,
  		      dd: translate$2,
  		      M: translate$2,
  		      MM: translate$2,
  		      y: translate$2,
  		      yy: translate$2
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}\./,
  		    ordinal: '%d.',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('fil', {
  		    months: 'Enero_Pebrero_Marso_Abril_Mayo_Hunyo_Hulyo_Agosto_Setyembre_Oktubre_Nobyembre_Disyembre'.split('_'),
  		    monthsShort: 'Ene_Peb_Mar_Abr_May_Hun_Hul_Ago_Set_Okt_Nob_Dis'.split('_'),
  		    weekdays: 'Linggo_Lunes_Martes_Miyerkules_Huwebes_Biyernes_Sabado'.split('_'),
  		    weekdaysShort: 'Lin_Lun_Mar_Miy_Huw_Biy_Sab'.split('_'),
  		    weekdaysMin: 'Li_Lu_Ma_Mi_Hu_Bi_Sab'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'MM/D/YYYY',
  		      LL: 'MMMM D, YYYY',
  		      LLL: 'MMMM D, YYYY HH:mm',
  		      LLLL: 'dddd, MMMM DD, YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: 'LT [ngayong araw]',
  		      nextDay: '[Bukas ng] LT',
  		      nextWeek: 'LT [sa susunod na] dddd',
  		      lastDay: 'LT [kahapon]',
  		      lastWeek: 'LT [noong nakaraang] dddd',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'sa loob ng %s',
  		      past: '%s ang nakalipas',
  		      s: 'ilang segundo',
  		      ss: '%d segundo',
  		      m: 'isang minuto',
  		      mm: '%d minuto',
  		      h: 'isang oras',
  		      hh: '%d oras',
  		      d: 'isang araw',
  		      dd: '%d araw',
  		      M: 'isang buwan',
  		      MM: '%d buwan',
  		      y: 'isang taon',
  		      yy: '%d taon'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}/,
  		    ordinal: function (number) {
  		      return number;
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('fo', {
  		    months: 'januar_februar_mars_apríl_mai_juni_juli_august_september_oktober_november_desember'.split('_'),
  		    monthsShort: 'jan_feb_mar_apr_mai_jun_jul_aug_sep_okt_nov_des'.split('_'),
  		    weekdays: 'sunnudagur_mánadagur_týsdagur_mikudagur_hósdagur_fríggjadagur_leygardagur'.split('_'),
  		    weekdaysShort: 'sun_mán_týs_mik_hós_frí_ley'.split('_'),
  		    weekdaysMin: 'su_má_tý_mi_hó_fr_le'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd D. MMMM, YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Í dag kl.] LT',
  		      nextDay: '[Í morgin kl.] LT',
  		      nextWeek: 'dddd [kl.] LT',
  		      lastDay: '[Í gjár kl.] LT',
  		      lastWeek: '[síðstu] dddd [kl] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'um %s',
  		      past: '%s síðani',
  		      s: 'fá sekund',
  		      ss: '%d sekundir',
  		      m: 'ein minuttur',
  		      mm: '%d minuttir',
  		      h: 'ein tími',
  		      hh: '%d tímar',
  		      d: 'ein dagur',
  		      dd: '%d dagar',
  		      M: 'ein mánaður',
  		      MM: '%d mánaðir',
  		      y: 'eitt ár',
  		      yy: '%d ár'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}\./,
  		    ordinal: '%d.',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('fr-ca', {
  		    months: 'janvier_février_mars_avril_mai_juin_juillet_août_septembre_octobre_novembre_décembre'.split('_'),
  		    monthsShort: 'janv._févr._mars_avr._mai_juin_juil._août_sept._oct._nov._déc.'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'dimanche_lundi_mardi_mercredi_jeudi_vendredi_samedi'.split('_'),
  		    weekdaysShort: 'dim._lun._mar._mer._jeu._ven._sam.'.split('_'),
  		    weekdaysMin: 'di_lu_ma_me_je_ve_sa'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'YYYY-MM-DD',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Aujourd’hui à] LT',
  		      nextDay: '[Demain à] LT',
  		      nextWeek: 'dddd [à] LT',
  		      lastDay: '[Hier à] LT',
  		      lastWeek: 'dddd [dernier à] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'dans %s',
  		      past: 'il y a %s',
  		      s: 'quelques secondes',
  		      ss: '%d secondes',
  		      m: 'une minute',
  		      mm: '%d minutes',
  		      h: 'une heure',
  		      hh: '%d heures',
  		      d: 'un jour',
  		      dd: '%d jours',
  		      M: 'un mois',
  		      MM: '%d mois',
  		      y: 'un an',
  		      yy: '%d ans'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(er|e)/,
  		    ordinal: function (number, period) {
  		      switch (period) {
  		        // Words with masculine grammatical gender: mois, trimestre, jour
  		        default:
  		        case 'M':
  		        case 'Q':
  		        case 'D':
  		        case 'DDD':
  		        case 'd':
  		          return number + (number === 1 ? 'er' : 'e');

  		        // Words with feminine grammatical gender: semaine
  		        case 'w':
  		        case 'W':
  		          return number + (number === 1 ? 're' : 'e');
  		      }
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('fr-ch', {
  		    months: 'janvier_février_mars_avril_mai_juin_juillet_août_septembre_octobre_novembre_décembre'.split('_'),
  		    monthsShort: 'janv._févr._mars_avr._mai_juin_juil._août_sept._oct._nov._déc.'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'dimanche_lundi_mardi_mercredi_jeudi_vendredi_samedi'.split('_'),
  		    weekdaysShort: 'dim._lun._mar._mer._jeu._ven._sam.'.split('_'),
  		    weekdaysMin: 'di_lu_ma_me_je_ve_sa'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Aujourd’hui à] LT',
  		      nextDay: '[Demain à] LT',
  		      nextWeek: 'dddd [à] LT',
  		      lastDay: '[Hier à] LT',
  		      lastWeek: 'dddd [dernier à] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'dans %s',
  		      past: 'il y a %s',
  		      s: 'quelques secondes',
  		      ss: '%d secondes',
  		      m: 'une minute',
  		      mm: '%d minutes',
  		      h: 'une heure',
  		      hh: '%d heures',
  		      d: 'un jour',
  		      dd: '%d jours',
  		      M: 'un mois',
  		      MM: '%d mois',
  		      y: 'un an',
  		      yy: '%d ans'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(er|e)/,
  		    ordinal: function (number, period) {
  		      switch (period) {
  		        // Words with masculine grammatical gender: mois, trimestre, jour
  		        default:
  		        case 'M':
  		        case 'Q':
  		        case 'D':
  		        case 'DDD':
  		        case 'd':
  		          return number + (number === 1 ? 'er' : 'e');

  		        // Words with feminine grammatical gender: semaine
  		        case 'w':
  		        case 'W':
  		          return number + (number === 1 ? 're' : 'e');
  		      }
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var monthsStrictRegex$1 = /^(janvier|février|mars|avril|mai|juin|juillet|août|septembre|octobre|novembre|décembre)/i,
  		    monthsShortStrictRegex$1 = /(janv\.?|févr\.?|mars|avr\.?|mai|juin|juil\.?|août|sept\.?|oct\.?|nov\.?|déc\.?)/i,
  		    monthsRegex$7 = /(janv\.?|févr\.?|mars|avr\.?|mai|juin|juil\.?|août|sept\.?|oct\.?|nov\.?|déc\.?|janvier|février|mars|avril|mai|juin|juillet|août|septembre|octobre|novembre|décembre)/i,
  		    monthsParse$6 = [/^janv/i, /^févr/i, /^mars/i, /^avr/i, /^mai/i, /^juin/i, /^juil/i, /^août/i, /^sept/i, /^oct/i, /^nov/i, /^déc/i];
  		  hooks.defineLocale('fr', {
  		    months: 'janvier_février_mars_avril_mai_juin_juillet_août_septembre_octobre_novembre_décembre'.split('_'),
  		    monthsShort: 'janv._févr._mars_avr._mai_juin_juil._août_sept._oct._nov._déc.'.split('_'),
  		    monthsRegex: monthsRegex$7,
  		    monthsShortRegex: monthsRegex$7,
  		    monthsStrictRegex: monthsStrictRegex$1,
  		    monthsShortStrictRegex: monthsShortStrictRegex$1,
  		    monthsParse: monthsParse$6,
  		    longMonthsParse: monthsParse$6,
  		    shortMonthsParse: monthsParse$6,
  		    weekdays: 'dimanche_lundi_mardi_mercredi_jeudi_vendredi_samedi'.split('_'),
  		    weekdaysShort: 'dim._lun._mar._mer._jeu._ven._sam.'.split('_'),
  		    weekdaysMin: 'di_lu_ma_me_je_ve_sa'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Aujourd’hui à] LT',
  		      nextDay: '[Demain à] LT',
  		      nextWeek: 'dddd [à] LT',
  		      lastDay: '[Hier à] LT',
  		      lastWeek: 'dddd [dernier à] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'dans %s',
  		      past: 'il y a %s',
  		      s: 'quelques secondes',
  		      ss: '%d secondes',
  		      m: 'une minute',
  		      mm: '%d minutes',
  		      h: 'une heure',
  		      hh: '%d heures',
  		      d: 'un jour',
  		      dd: '%d jours',
  		      w: 'une semaine',
  		      ww: '%d semaines',
  		      M: 'un mois',
  		      MM: '%d mois',
  		      y: 'un an',
  		      yy: '%d ans'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(er|)/,
  		    ordinal: function (number, period) {
  		      switch (period) {
  		        // TODO: Return 'e' when day of month > 1. Move this case inside
  		        // block for masculine words below.
  		        // See https://github.com/moment/moment/issues/3375
  		        case 'D':
  		          return number + (number === 1 ? 'er' : '');

  		        // Words with masculine grammatical gender: mois, trimestre, jour
  		        default:
  		        case 'M':
  		        case 'Q':
  		        case 'DDD':
  		        case 'd':
  		          return number + (number === 1 ? 'er' : 'e');

  		        // Words with feminine grammatical gender: semaine
  		        case 'w':
  		        case 'W':
  		          return number + (number === 1 ? 're' : 'e');
  		      }
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var monthsShortWithDots = 'jan._feb._mrt._apr._mai_jun._jul._aug._sep._okt._nov._des.'.split('_'),
  		    monthsShortWithoutDots = 'jan_feb_mrt_apr_mai_jun_jul_aug_sep_okt_nov_des'.split('_');
  		  hooks.defineLocale('fy', {
  		    months: 'jannewaris_febrewaris_maart_april_maaie_juny_july_augustus_septimber_oktober_novimber_desimber'.split('_'),
  		    monthsShort: function (m, format) {
  		      if (!m) {
  		        return monthsShortWithDots;
  		      } else if (/-MMM-/.test(format)) {
  		        return monthsShortWithoutDots[m.month()];
  		      } else {
  		        return monthsShortWithDots[m.month()];
  		      }
  		    },
  		    monthsParseExact: true,
  		    weekdays: 'snein_moandei_tiisdei_woansdei_tongersdei_freed_sneon'.split('_'),
  		    weekdaysShort: 'si._mo._ti._wo._to._fr._so.'.split('_'),
  		    weekdaysMin: 'Si_Mo_Ti_Wo_To_Fr_So'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD-MM-YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[hjoed om] LT',
  		      nextDay: '[moarn om] LT',
  		      nextWeek: 'dddd [om] LT',
  		      lastDay: '[juster om] LT',
  		      lastWeek: '[ôfrûne] dddd [om] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'oer %s',
  		      past: '%s lyn',
  		      s: 'in pear sekonden',
  		      ss: '%d sekonden',
  		      m: 'ien minút',
  		      mm: '%d minuten',
  		      h: 'ien oere',
  		      hh: '%d oeren',
  		      d: 'ien dei',
  		      dd: '%d dagen',
  		      M: 'ien moanne',
  		      MM: '%d moannen',
  		      y: 'ien jier',
  		      yy: '%d jierren'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(ste|de)/,
  		    ordinal: function (number) {
  		      return number + (number === 1 || number === 8 || number >= 20 ? 'ste' : 'de');
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var months$6 = ['Eanáir', 'Feabhra', 'Márta', 'Aibreán', 'Bealtaine', 'Meitheamh', 'Iúil', 'Lúnasa', 'Meán Fómhair', 'Deireadh Fómhair', 'Samhain', 'Nollaig'],
  		    monthsShort$5 = ['Ean', 'Feabh', 'Márt', 'Aib', 'Beal', 'Meith', 'Iúil', 'Lún', 'M.F.', 'D.F.', 'Samh', 'Noll'],
  		    weekdays$1 = ['Dé Domhnaigh', 'Dé Luain', 'Dé Máirt', 'Dé Céadaoin', 'Déardaoin', 'Dé hAoine', 'Dé Sathairn'],
  		    weekdaysShort = ['Domh', 'Luan', 'Máirt', 'Céad', 'Déar', 'Aoine', 'Sath'],
  		    weekdaysMin = ['Do', 'Lu', 'Má', 'Cé', 'Dé', 'A', 'Sa'];
  		  hooks.defineLocale('ga', {
  		    months: months$6,
  		    monthsShort: monthsShort$5,
  		    monthsParseExact: true,
  		    weekdays: weekdays$1,
  		    weekdaysShort: weekdaysShort,
  		    weekdaysMin: weekdaysMin,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd, D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Inniu ag] LT',
  		      nextDay: '[Amárach ag] LT',
  		      nextWeek: 'dddd [ag] LT',
  		      lastDay: '[Inné ag] LT',
  		      lastWeek: 'dddd [seo caite] [ag] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'i %s',
  		      past: '%s ó shin',
  		      s: 'cúpla soicind',
  		      ss: '%d soicind',
  		      m: 'nóiméad',
  		      mm: '%d nóiméad',
  		      h: 'uair an chloig',
  		      hh: '%d uair an chloig',
  		      d: 'lá',
  		      dd: '%d lá',
  		      M: 'mí',
  		      MM: '%d míonna',
  		      y: 'bliain',
  		      yy: '%d bliain'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(d|na|mh)/,
  		    ordinal: function (number) {
  		      var output = number === 1 ? 'd' : number % 10 === 2 ? 'na' : 'mh';
  		      return number + output;
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var months$7 = ['Am Faoilleach', 'An Gearran', 'Am Màrt', 'An Giblean', 'An Cèitean', 'An t-Ògmhios', 'An t-Iuchar', 'An Lùnastal', 'An t-Sultain', 'An Dàmhair', 'An t-Samhain', 'An Dùbhlachd'],
  		    monthsShort$6 = ['Faoi', 'Gear', 'Màrt', 'Gibl', 'Cèit', 'Ògmh', 'Iuch', 'Lùn', 'Sult', 'Dàmh', 'Samh', 'Dùbh'],
  		    weekdays$2 = ['Didòmhnaich', 'Diluain', 'Dimàirt', 'Diciadain', 'Diardaoin', 'Dihaoine', 'Disathairne'],
  		    weekdaysShort$1 = ['Did', 'Dil', 'Dim', 'Dic', 'Dia', 'Dih', 'Dis'],
  		    weekdaysMin$1 = ['Dò', 'Lu', 'Mà', 'Ci', 'Ar', 'Ha', 'Sa'];
  		  hooks.defineLocale('gd', {
  		    months: months$7,
  		    monthsShort: monthsShort$6,
  		    monthsParseExact: true,
  		    weekdays: weekdays$2,
  		    weekdaysShort: weekdaysShort$1,
  		    weekdaysMin: weekdaysMin$1,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd, D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[An-diugh aig] LT',
  		      nextDay: '[A-màireach aig] LT',
  		      nextWeek: 'dddd [aig] LT',
  		      lastDay: '[An-dè aig] LT',
  		      lastWeek: 'dddd [seo chaidh] [aig] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'ann an %s',
  		      past: 'bho chionn %s',
  		      s: 'beagan diogan',
  		      ss: '%d diogan',
  		      m: 'mionaid',
  		      mm: '%d mionaidean',
  		      h: 'uair',
  		      hh: '%d uairean',
  		      d: 'latha',
  		      dd: '%d latha',
  		      M: 'mìos',
  		      MM: '%d mìosan',
  		      y: 'bliadhna',
  		      yy: '%d bliadhna'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(d|na|mh)/,
  		    ordinal: function (number) {
  		      var output = number === 1 ? 'd' : number % 10 === 2 ? 'na' : 'mh';
  		      return number + output;
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('gl', {
  		    months: 'xaneiro_febreiro_marzo_abril_maio_xuño_xullo_agosto_setembro_outubro_novembro_decembro'.split('_'),
  		    monthsShort: 'xan._feb._mar._abr._mai._xuñ._xul._ago._set._out._nov._dec.'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'domingo_luns_martes_mércores_xoves_venres_sábado'.split('_'),
  		    weekdaysShort: 'dom._lun._mar._mér._xov._ven._sáb.'.split('_'),
  		    weekdaysMin: 'do_lu_ma_mé_xo_ve_sá'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'H:mm',
  		      LTS: 'H:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D [de] MMMM [de] YYYY',
  		      LLL: 'D [de] MMMM [de] YYYY H:mm',
  		      LLLL: 'dddd, D [de] MMMM [de] YYYY H:mm'
  		    },
  		    calendar: {
  		      sameDay: function () {
  		        return '[hoxe ' + (this.hours() !== 1 ? 'ás' : 'á') + '] LT';
  		      },
  		      nextDay: function () {
  		        return '[mañá ' + (this.hours() !== 1 ? 'ás' : 'á') + '] LT';
  		      },
  		      nextWeek: function () {
  		        return 'dddd [' + (this.hours() !== 1 ? 'ás' : 'a') + '] LT';
  		      },
  		      lastDay: function () {
  		        return '[onte ' + (this.hours() !== 1 ? 'á' : 'a') + '] LT';
  		      },
  		      lastWeek: function () {
  		        return '[o] dddd [pasado ' + (this.hours() !== 1 ? 'ás' : 'a') + '] LT';
  		      },
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: function (str) {
  		        if (str.indexOf('un') === 0) {
  		          return 'n' + str;
  		        }
  		        return 'en ' + str;
  		      },
  		      past: 'hai %s',
  		      s: 'uns segundos',
  		      ss: '%d segundos',
  		      m: 'un minuto',
  		      mm: '%d minutos',
  		      h: 'unha hora',
  		      hh: '%d horas',
  		      d: 'un día',
  		      dd: '%d días',
  		      M: 'un mes',
  		      MM: '%d meses',
  		      y: 'un ano',
  		      yy: '%d anos'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}º/,
  		    ordinal: '%dº',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  function processRelativeTime$5(number, withoutSuffix, key, isFuture) {
  		    var format = {
  		      s: ['थोडया सॅकंडांनी', 'थोडे सॅकंड'],
  		      ss: [number + ' सॅकंडांनी', number + ' सॅकंड'],
  		      m: ['एका मिणटान', 'एक मिनूट'],
  		      mm: [number + ' मिणटांनी', number + ' मिणटां'],
  		      h: ['एका वरान', 'एक वर'],
  		      hh: [number + ' वरांनी', number + ' वरां'],
  		      d: ['एका दिसान', 'एक दीस'],
  		      dd: [number + ' दिसांनी', number + ' दीस'],
  		      M: ['एका म्हयन्यान', 'एक म्हयनो'],
  		      MM: [number + ' म्हयन्यानी', number + ' म्हयने'],
  		      y: ['एका वर्सान', 'एक वर्स'],
  		      yy: [number + ' वर्सांनी', number + ' वर्सां']
  		    };
  		    return isFuture ? format[key][0] : format[key][1];
  		  }
  		  hooks.defineLocale('gom-deva', {
  		    months: {
  		      standalone: 'जानेवारी_फेब्रुवारी_मार्च_एप्रील_मे_जून_जुलय_ऑगस्ट_सप्टेंबर_ऑक्टोबर_नोव्हेंबर_डिसेंबर'.split('_'),
  		      format: 'जानेवारीच्या_फेब्रुवारीच्या_मार्चाच्या_एप्रीलाच्या_मेयाच्या_जूनाच्या_जुलयाच्या_ऑगस्टाच्या_सप्टेंबराच्या_ऑक्टोबराच्या_नोव्हेंबराच्या_डिसेंबराच्या'.split('_'),
  		      isFormat: /MMMM(\s)+D[oD]?/
  		    },
  		    monthsShort: 'जाने._फेब्रु._मार्च_एप्री._मे_जून_जुल._ऑग._सप्टें._ऑक्टो._नोव्हें._डिसें.'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'आयतार_सोमार_मंगळार_बुधवार_बिरेस्तार_सुक्रार_शेनवार'.split('_'),
  		    weekdaysShort: 'आयत._सोम._मंगळ._बुध._ब्रेस्त._सुक्र._शेन.'.split('_'),
  		    weekdaysMin: 'आ_सो_मं_बु_ब्रे_सु_शे'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'A h:mm [वाजतां]',
  		      LTS: 'A h:mm:ss [वाजतां]',
  		      L: 'DD-MM-YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY A h:mm [वाजतां]',
  		      LLLL: 'dddd, MMMM Do, YYYY, A h:mm [वाजतां]',
  		      llll: 'ddd, D MMM YYYY, A h:mm [वाजतां]'
  		    },
  		    calendar: {
  		      sameDay: '[आयज] LT',
  		      nextDay: '[फाल्यां] LT',
  		      nextWeek: '[फुडलो] dddd[,] LT',
  		      lastDay: '[काल] LT',
  		      lastWeek: '[फाटलो] dddd[,] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%s',
  		      past: '%s आदीं',
  		      s: processRelativeTime$5,
  		      ss: processRelativeTime$5,
  		      m: processRelativeTime$5,
  		      mm: processRelativeTime$5,
  		      h: processRelativeTime$5,
  		      hh: processRelativeTime$5,
  		      d: processRelativeTime$5,
  		      dd: processRelativeTime$5,
  		      M: processRelativeTime$5,
  		      MM: processRelativeTime$5,
  		      y: processRelativeTime$5,
  		      yy: processRelativeTime$5
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(वेर)/,
  		    ordinal: function (number, period) {
  		      switch (period) {
  		        // the ordinal 'वेर' only applies to day of the month
  		        case 'D':
  		          return number + 'वेर';
  		        default:
  		        case 'M':
  		        case 'Q':
  		        case 'DDD':
  		        case 'd':
  		        case 'w':
  		        case 'W':
  		          return number;
  		      }
  		    },
  		    week: {
  		      dow: 0,
  		      // Sunday is the first day of the week
  		      doy: 3 // The week that contains Jan 4th is the first week of the year (7 + 0 - 4)
  		    },
  		    meridiemParse: /राती|सकाळीं|दनपारां|सांजे/,
  		    meridiemHour: function (hour, meridiem) {
  		      if (hour === 12) {
  		        hour = 0;
  		      }
  		      if (meridiem === 'राती') {
  		        return hour < 4 ? hour : hour + 12;
  		      } else if (meridiem === 'सकाळीं') {
  		        return hour;
  		      } else if (meridiem === 'दनपारां') {
  		        return hour > 12 ? hour : hour + 12;
  		      } else if (meridiem === 'सांजे') {
  		        return hour + 12;
  		      }
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 4) {
  		        return 'राती';
  		      } else if (hour < 12) {
  		        return 'सकाळीं';
  		      } else if (hour < 16) {
  		        return 'दनपारां';
  		      } else if (hour < 20) {
  		        return 'सांजे';
  		      } else {
  		        return 'राती';
  		      }
  		    }
  		  });

  		  //! moment.js locale configuration

  		  function processRelativeTime$6(number, withoutSuffix, key, isFuture) {
  		    var format = {
  		      s: ['thoddea sekondamni', 'thodde sekond'],
  		      ss: [number + ' sekondamni', number + ' sekond'],
  		      m: ['eka mintan', 'ek minut'],
  		      mm: [number + ' mintamni', number + ' mintam'],
  		      h: ['eka voran', 'ek vor'],
  		      hh: [number + ' voramni', number + ' voram'],
  		      d: ['eka disan', 'ek dis'],
  		      dd: [number + ' disamni', number + ' dis'],
  		      M: ['eka mhoinean', 'ek mhoino'],
  		      MM: [number + ' mhoineamni', number + ' mhoine'],
  		      y: ['eka vorsan', 'ek voros'],
  		      yy: [number + ' vorsamni', number + ' vorsam']
  		    };
  		    return isFuture ? format[key][0] : format[key][1];
  		  }
  		  hooks.defineLocale('gom-latn', {
  		    months: {
  		      standalone: 'Janer_Febrer_Mars_Abril_Mai_Jun_Julai_Agost_Setembr_Otubr_Novembr_Dezembr'.split('_'),
  		      format: 'Janerachea_Febrerachea_Marsachea_Abrilachea_Maiachea_Junachea_Julaiachea_Agostachea_Setembrachea_Otubrachea_Novembrachea_Dezembrachea'.split('_'),
  		      isFormat: /MMMM(\s)+D[oD]?/
  		    },
  		    monthsShort: 'Jan._Feb._Mars_Abr._Mai_Jun_Jul._Ago._Set._Otu._Nov._Dez.'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: "Aitar_Somar_Mongllar_Budhvar_Birestar_Sukrar_Son'var".split('_'),
  		    weekdaysShort: 'Ait._Som._Mon._Bud._Bre._Suk._Son.'.split('_'),
  		    weekdaysMin: 'Ai_Sm_Mo_Bu_Br_Su_Sn'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'A h:mm [vazta]',
  		      LTS: 'A h:mm:ss [vazta]',
  		      L: 'DD-MM-YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY A h:mm [vazta]',
  		      LLLL: 'dddd, MMMM Do, YYYY, A h:mm [vazta]',
  		      llll: 'ddd, D MMM YYYY, A h:mm [vazta]'
  		    },
  		    calendar: {
  		      sameDay: '[Aiz] LT',
  		      nextDay: '[Faleam] LT',
  		      nextWeek: '[Fuddlo] dddd[,] LT',
  		      lastDay: '[Kal] LT',
  		      lastWeek: '[Fattlo] dddd[,] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%s',
  		      past: '%s adim',
  		      s: processRelativeTime$6,
  		      ss: processRelativeTime$6,
  		      m: processRelativeTime$6,
  		      mm: processRelativeTime$6,
  		      h: processRelativeTime$6,
  		      hh: processRelativeTime$6,
  		      d: processRelativeTime$6,
  		      dd: processRelativeTime$6,
  		      M: processRelativeTime$6,
  		      MM: processRelativeTime$6,
  		      y: processRelativeTime$6,
  		      yy: processRelativeTime$6
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(er)/,
  		    ordinal: function (number, period) {
  		      switch (period) {
  		        // the ordinal 'er' only applies to day of the month
  		        case 'D':
  		          return number + 'er';
  		        default:
  		        case 'M':
  		        case 'Q':
  		        case 'DDD':
  		        case 'd':
  		        case 'w':
  		        case 'W':
  		          return number;
  		      }
  		    },
  		    week: {
  		      dow: 0,
  		      // Sunday is the first day of the week
  		      doy: 3 // The week that contains Jan 4th is the first week of the year (7 + 0 - 4)
  		    },
  		    meridiemParse: /rati|sokallim|donparam|sanje/,
  		    meridiemHour: function (hour, meridiem) {
  		      if (hour === 12) {
  		        hour = 0;
  		      }
  		      if (meridiem === 'rati') {
  		        return hour < 4 ? hour : hour + 12;
  		      } else if (meridiem === 'sokallim') {
  		        return hour;
  		      } else if (meridiem === 'donparam') {
  		        return hour > 12 ? hour : hour + 12;
  		      } else if (meridiem === 'sanje') {
  		        return hour + 12;
  		      }
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 4) {
  		        return 'rati';
  		      } else if (hour < 12) {
  		        return 'sokallim';
  		      } else if (hour < 16) {
  		        return 'donparam';
  		      } else if (hour < 20) {
  		        return 'sanje';
  		      } else {
  		        return 'rati';
  		      }
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var symbolMap$8 = {
  		      1: '૧',
  		      2: '૨',
  		      3: '૩',
  		      4: '૪',
  		      5: '૫',
  		      6: '૬',
  		      7: '૭',
  		      8: '૮',
  		      9: '૯',
  		      0: '૦'
  		    },
  		    numberMap$7 = {
  		      '૧': '1',
  		      '૨': '2',
  		      '૩': '3',
  		      '૪': '4',
  		      '૫': '5',
  		      '૬': '6',
  		      '૭': '7',
  		      '૮': '8',
  		      '૯': '9',
  		      '૦': '0'
  		    };
  		  hooks.defineLocale('gu', {
  		    months: 'જાન્યુઆરી_ફેબ્રુઆરી_માર્ચ_એપ્રિલ_મે_જૂન_જુલાઈ_ઑગસ્ટ_સપ્ટેમ્બર_ઑક્ટ્બર_નવેમ્બર_ડિસેમ્બર'.split('_'),
  		    monthsShort: 'જાન્યુ._ફેબ્રુ._માર્ચ_એપ્રિ._મે_જૂન_જુલા._ઑગ._સપ્ટે._ઑક્ટ્._નવે._ડિસે.'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'રવિવાર_સોમવાર_મંગળવાર_બુધ્વાર_ગુરુવાર_શુક્રવાર_શનિવાર'.split('_'),
  		    weekdaysShort: 'રવિ_સોમ_મંગળ_બુધ્_ગુરુ_શુક્ર_શનિ'.split('_'),
  		    weekdaysMin: 'ર_સો_મં_બુ_ગુ_શુ_શ'.split('_'),
  		    longDateFormat: {
  		      LT: 'A h:mm વાગ્યે',
  		      LTS: 'A h:mm:ss વાગ્યે',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY, A h:mm વાગ્યે',
  		      LLLL: 'dddd, D MMMM YYYY, A h:mm વાગ્યે'
  		    },
  		    calendar: {
  		      sameDay: '[આજ] LT',
  		      nextDay: '[કાલે] LT',
  		      nextWeek: 'dddd, LT',
  		      lastDay: '[ગઇકાલે] LT',
  		      lastWeek: '[પાછલા] dddd, LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%s મા',
  		      past: '%s પહેલા',
  		      s: 'અમુક પળો',
  		      ss: '%d સેકંડ',
  		      m: 'એક મિનિટ',
  		      mm: '%d મિનિટ',
  		      h: 'એક કલાક',
  		      hh: '%d કલાક',
  		      d: 'એક દિવસ',
  		      dd: '%d દિવસ',
  		      M: 'એક મહિનો',
  		      MM: '%d મહિનો',
  		      y: 'એક વર્ષ',
  		      yy: '%d વર્ષ'
  		    },
  		    preparse: function (string) {
  		      return string.replace(/[૧૨૩૪૫૬૭૮૯૦]/g, function (match) {
  		        return numberMap$7[match];
  		      });
  		    },
  		    postformat: function (string) {
  		      return string.replace(/\d/g, function (match) {
  		        return symbolMap$8[match];
  		      });
  		    },
  		    // Gujarati notation for meridiems are quite fuzzy in practice. While there exists
  		    // a rigid notion of a 'Pahar' it is not used as rigidly in modern Gujarati.
  		    meridiemParse: /રાત|બપોર|સવાર|સાંજ/,
  		    meridiemHour: function (hour, meridiem) {
  		      if (hour === 12) {
  		        hour = 0;
  		      }
  		      if (meridiem === 'રાત') {
  		        return hour < 4 ? hour : hour + 12;
  		      } else if (meridiem === 'સવાર') {
  		        return hour;
  		      } else if (meridiem === 'બપોર') {
  		        return hour >= 10 ? hour : hour + 12;
  		      } else if (meridiem === 'સાંજ') {
  		        return hour + 12;
  		      }
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 4) {
  		        return 'રાત';
  		      } else if (hour < 10) {
  		        return 'સવાર';
  		      } else if (hour < 17) {
  		        return 'બપોર';
  		      } else if (hour < 20) {
  		        return 'સાંજ';
  		      } else {
  		        return 'રાત';
  		      }
  		    },
  		    week: {
  		      dow: 0,
  		      // Sunday is the first day of the week.
  		      doy: 6 // The week that contains Jan 6th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('he', {
  		    months: 'ינואר_פברואר_מרץ_אפריל_מאי_יוני_יולי_אוגוסט_ספטמבר_אוקטובר_נובמבר_דצמבר'.split('_'),
  		    monthsShort: 'ינו׳_פבר׳_מרץ_אפר׳_מאי_יוני_יולי_אוג׳_ספט׳_אוק׳_נוב׳_דצמ׳'.split('_'),
  		    weekdays: 'ראשון_שני_שלישי_רביעי_חמישי_שישי_שבת'.split('_'),
  		    weekdaysShort: 'א׳_ב׳_ג׳_ד׳_ה׳_ו׳_ש׳'.split('_'),
  		    weekdaysMin: 'א_ב_ג_ד_ה_ו_ש'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D [ב]MMMM YYYY',
  		      LLL: 'D [ב]MMMM YYYY HH:mm',
  		      LLLL: 'dddd, D [ב]MMMM YYYY HH:mm',
  		      l: 'D/M/YYYY',
  		      ll: 'D MMM YYYY',
  		      lll: 'D MMM YYYY HH:mm',
  		      llll: 'ddd, D MMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[היום ב־]LT',
  		      nextDay: '[מחר ב־]LT',
  		      nextWeek: 'dddd [בשעה] LT',
  		      lastDay: '[אתמול ב־]LT',
  		      lastWeek: '[ביום] dddd [האחרון בשעה] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'בעוד %s',
  		      past: 'לפני %s',
  		      s: 'מספר שניות',
  		      ss: '%d שניות',
  		      m: 'דקה',
  		      mm: '%d דקות',
  		      h: 'שעה',
  		      hh: function (number) {
  		        if (number === 2) {
  		          return 'שעתיים';
  		        }
  		        return number + ' שעות';
  		      },
  		      d: 'יום',
  		      dd: function (number) {
  		        if (number === 2) {
  		          return 'יומיים';
  		        }
  		        return number + ' ימים';
  		      },
  		      M: 'חודש',
  		      MM: function (number) {
  		        if (number === 2) {
  		          return 'חודשיים';
  		        }
  		        return number + ' חודשים';
  		      },
  		      y: 'שנה',
  		      yy: function (number) {
  		        if (number === 2) {
  		          return 'שנתיים';
  		        } else if (number % 10 === 0 && number !== 10) {
  		          return number + ' שנה';
  		        }
  		        return number + ' שנים';
  		      }
  		    },
  		    meridiemParse: /אחה"צ|לפנה"צ|אחרי הצהריים|לפני הצהריים|לפנות בוקר|בבוקר|בערב/i,
  		    isPM: function (input) {
  		      return /^(אחה"צ|אחרי הצהריים|בערב)$/.test(input);
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 5) {
  		        return 'לפנות בוקר';
  		      } else if (hour < 10) {
  		        return 'בבוקר';
  		      } else if (hour < 12) {
  		        return isLower ? 'לפנה"צ' : 'לפני הצהריים';
  		      } else if (hour < 18) {
  		        return isLower ? 'אחה"צ' : 'אחרי הצהריים';
  		      } else {
  		        return 'בערב';
  		      }
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var symbolMap$9 = {
  		      1: '१',
  		      2: '२',
  		      3: '३',
  		      4: '४',
  		      5: '५',
  		      6: '६',
  		      7: '७',
  		      8: '८',
  		      9: '९',
  		      0: '०'
  		    },
  		    numberMap$8 = {
  		      '१': '1',
  		      '२': '2',
  		      '३': '3',
  		      '४': '4',
  		      '५': '5',
  		      '६': '6',
  		      '७': '7',
  		      '८': '8',
  		      '९': '9',
  		      '०': '0'
  		    },
  		    monthsParse$7 = [/^जन/i, /^फ़र|फर/i, /^मार्च/i, /^अप्रै/i, /^मई/i, /^जून/i, /^जुल/i, /^अग/i, /^सितं|सित/i, /^अक्टू/i, /^नव|नवं/i, /^दिसं|दिस/i],
  		    shortMonthsParse = [/^जन/i, /^फ़र/i, /^मार्च/i, /^अप्रै/i, /^मई/i, /^जून/i, /^जुल/i, /^अग/i, /^सित/i, /^अक्टू/i, /^नव/i, /^दिस/i];
  		  hooks.defineLocale('hi', {
  		    months: {
  		      format: 'जनवरी_फ़रवरी_मार्च_अप्रैल_मई_जून_जुलाई_अगस्त_सितम्बर_अक्टूबर_नवम्बर_दिसम्बर'.split('_'),
  		      standalone: 'जनवरी_फरवरी_मार्च_अप्रैल_मई_जून_जुलाई_अगस्त_सितंबर_अक्टूबर_नवंबर_दिसंबर'.split('_')
  		    },
  		    monthsShort: 'जन._फ़र._मार्च_अप्रै._मई_जून_जुल._अग._सित._अक्टू._नव._दिस.'.split('_'),
  		    weekdays: 'रविवार_सोमवार_मंगलवार_बुधवार_गुरूवार_शुक्रवार_शनिवार'.split('_'),
  		    weekdaysShort: 'रवि_सोम_मंगल_बुध_गुरू_शुक्र_शनि'.split('_'),
  		    weekdaysMin: 'र_सो_मं_बु_गु_शु_श'.split('_'),
  		    longDateFormat: {
  		      LT: 'A h:mm बजे',
  		      LTS: 'A h:mm:ss बजे',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY, A h:mm बजे',
  		      LLLL: 'dddd, D MMMM YYYY, A h:mm बजे'
  		    },
  		    monthsParse: monthsParse$7,
  		    longMonthsParse: monthsParse$7,
  		    shortMonthsParse: shortMonthsParse,
  		    monthsRegex: /^(जनवरी|जन\.?|फ़रवरी|फरवरी|फ़र\.?|मार्च?|अप्रैल|अप्रै\.?|मई?|जून?|जुलाई|जुल\.?|अगस्त|अग\.?|सितम्बर|सितंबर|सित\.?|अक्टूबर|अक्टू\.?|नवम्बर|नवंबर|नव\.?|दिसम्बर|दिसंबर|दिस\.?)/i,
  		    monthsShortRegex: /^(जनवरी|जन\.?|फ़रवरी|फरवरी|फ़र\.?|मार्च?|अप्रैल|अप्रै\.?|मई?|जून?|जुलाई|जुल\.?|अगस्त|अग\.?|सितम्बर|सितंबर|सित\.?|अक्टूबर|अक्टू\.?|नवम्बर|नवंबर|नव\.?|दिसम्बर|दिसंबर|दिस\.?)/i,
  		    monthsStrictRegex: /^(जनवरी?|फ़रवरी|फरवरी?|मार्च?|अप्रैल?|मई?|जून?|जुलाई?|अगस्त?|सितम्बर|सितंबर|सित?\.?|अक्टूबर|अक्टू\.?|नवम्बर|नवंबर?|दिसम्बर|दिसंबर?)/i,
  		    monthsShortStrictRegex: /^(जन\.?|फ़र\.?|मार्च?|अप्रै\.?|मई?|जून?|जुल\.?|अग\.?|सित\.?|अक्टू\.?|नव\.?|दिस\.?)/i,
  		    calendar: {
  		      sameDay: '[आज] LT',
  		      nextDay: '[कल] LT',
  		      nextWeek: 'dddd, LT',
  		      lastDay: '[कल] LT',
  		      lastWeek: '[पिछले] dddd, LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%s में',
  		      past: '%s पहले',
  		      s: 'कुछ ही क्षण',
  		      ss: '%d सेकंड',
  		      m: 'एक मिनट',
  		      mm: '%d मिनट',
  		      h: 'एक घंटा',
  		      hh: '%d घंटे',
  		      d: 'एक दिन',
  		      dd: '%d दिन',
  		      M: 'एक महीने',
  		      MM: '%d महीने',
  		      y: 'एक वर्ष',
  		      yy: '%d वर्ष'
  		    },
  		    preparse: function (string) {
  		      return string.replace(/[१२३४५६७८९०]/g, function (match) {
  		        return numberMap$8[match];
  		      });
  		    },
  		    postformat: function (string) {
  		      return string.replace(/\d/g, function (match) {
  		        return symbolMap$9[match];
  		      });
  		    },
  		    // Hindi notation for meridiems are quite fuzzy in practice. While there exists
  		    // a rigid notion of a 'Pahar' it is not used as rigidly in modern Hindi.
  		    meridiemParse: /रात|सुबह|दोपहर|शाम/,
  		    meridiemHour: function (hour, meridiem) {
  		      if (hour === 12) {
  		        hour = 0;
  		      }
  		      if (meridiem === 'रात') {
  		        return hour < 4 ? hour : hour + 12;
  		      } else if (meridiem === 'सुबह') {
  		        return hour;
  		      } else if (meridiem === 'दोपहर') {
  		        return hour >= 10 ? hour : hour + 12;
  		      } else if (meridiem === 'शाम') {
  		        return hour + 12;
  		      }
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 4) {
  		        return 'रात';
  		      } else if (hour < 10) {
  		        return 'सुबह';
  		      } else if (hour < 17) {
  		        return 'दोपहर';
  		      } else if (hour < 20) {
  		        return 'शाम';
  		      } else {
  		        return 'रात';
  		      }
  		    },
  		    week: {
  		      dow: 0,
  		      // Sunday is the first day of the week.
  		      doy: 6 // The week that contains Jan 6th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  function translate$3(number, withoutSuffix, key) {
  		    var result = number + ' ';
  		    switch (key) {
  		      case 'ss':
  		        if (number === 1) {
  		          result += 'sekunda';
  		        } else if (number === 2 || number === 3 || number === 4) {
  		          result += 'sekunde';
  		        } else {
  		          result += 'sekundi';
  		        }
  		        return result;
  		      case 'm':
  		        return withoutSuffix ? 'jedna minuta' : 'jedne minute';
  		      case 'mm':
  		        if (number === 1) {
  		          result += 'minuta';
  		        } else if (number === 2 || number === 3 || number === 4) {
  		          result += 'minute';
  		        } else {
  		          result += 'minuta';
  		        }
  		        return result;
  		      case 'h':
  		        return withoutSuffix ? 'jedan sat' : 'jednog sata';
  		      case 'hh':
  		        if (number === 1) {
  		          result += 'sat';
  		        } else if (number === 2 || number === 3 || number === 4) {
  		          result += 'sata';
  		        } else {
  		          result += 'sati';
  		        }
  		        return result;
  		      case 'dd':
  		        if (number === 1) {
  		          result += 'dan';
  		        } else {
  		          result += 'dana';
  		        }
  		        return result;
  		      case 'MM':
  		        if (number === 1) {
  		          result += 'mjesec';
  		        } else if (number === 2 || number === 3 || number === 4) {
  		          result += 'mjeseca';
  		        } else {
  		          result += 'mjeseci';
  		        }
  		        return result;
  		      case 'yy':
  		        if (number === 1) {
  		          result += 'godina';
  		        } else if (number === 2 || number === 3 || number === 4) {
  		          result += 'godine';
  		        } else {
  		          result += 'godina';
  		        }
  		        return result;
  		    }
  		  }
  		  hooks.defineLocale('hr', {
  		    months: {
  		      format: 'siječnja_veljače_ožujka_travnja_svibnja_lipnja_srpnja_kolovoza_rujna_listopada_studenoga_prosinca'.split('_'),
  		      standalone: 'siječanj_veljača_ožujak_travanj_svibanj_lipanj_srpanj_kolovoz_rujan_listopad_studeni_prosinac'.split('_')
  		    },
  		    monthsShort: 'sij._velj._ožu._tra._svi._lip._srp._kol._ruj._lis._stu._pro.'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'nedjelja_ponedjeljak_utorak_srijeda_četvrtak_petak_subota'.split('_'),
  		    weekdaysShort: 'ned._pon._uto._sri._čet._pet._sub.'.split('_'),
  		    weekdaysMin: 'ne_po_ut_sr_če_pe_su'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'H:mm',
  		      LTS: 'H:mm:ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'Do MMMM YYYY',
  		      LLL: 'Do MMMM YYYY H:mm',
  		      LLLL: 'dddd, Do MMMM YYYY H:mm'
  		    },
  		    calendar: {
  		      sameDay: '[danas u] LT',
  		      nextDay: '[sutra u] LT',
  		      nextWeek: function () {
  		        switch (this.day()) {
  		          case 0:
  		            return '[u] [nedjelju] [u] LT';
  		          case 3:
  		            return '[u] [srijedu] [u] LT';
  		          case 6:
  		            return '[u] [subotu] [u] LT';
  		          case 1:
  		          case 2:
  		          case 4:
  		          case 5:
  		            return '[u] dddd [u] LT';
  		        }
  		      },
  		      lastDay: '[jučer u] LT',
  		      lastWeek: function () {
  		        switch (this.day()) {
  		          case 0:
  		            return '[prošlu] [nedjelju] [u] LT';
  		          case 3:
  		            return '[prošlu] [srijedu] [u] LT';
  		          case 6:
  		            return '[prošle] [subote] [u] LT';
  		          case 1:
  		          case 2:
  		          case 4:
  		          case 5:
  		            return '[prošli] dddd [u] LT';
  		        }
  		      },
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'za %s',
  		      past: 'prije %s',
  		      s: 'par sekundi',
  		      ss: translate$3,
  		      m: translate$3,
  		      mm: translate$3,
  		      h: translate$3,
  		      hh: translate$3,
  		      d: 'dan',
  		      dd: translate$3,
  		      M: 'mjesec',
  		      MM: translate$3,
  		      y: 'godinu',
  		      yy: translate$3
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}\./,
  		    ordinal: '%d.',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 7 // The week that contains Jan 7th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var weekEndings = 'vasárnap hétfőn kedden szerdán csütörtökön pénteken szombaton'.split(' ');
  		  function translate$4(number, withoutSuffix, key, isFuture) {
  		    var num = number;
  		    switch (key) {
  		      case 's':
  		        return isFuture || withoutSuffix ? 'néhány másodperc' : 'néhány másodperce';
  		      case 'ss':
  		        return num + (isFuture || withoutSuffix) ? ' másodperc' : ' másodperce';
  		      case 'm':
  		        return 'egy' + (isFuture || withoutSuffix ? ' perc' : ' perce');
  		      case 'mm':
  		        return num + (isFuture || withoutSuffix ? ' perc' : ' perce');
  		      case 'h':
  		        return 'egy' + (isFuture || withoutSuffix ? ' óra' : ' órája');
  		      case 'hh':
  		        return num + (isFuture || withoutSuffix ? ' óra' : ' órája');
  		      case 'd':
  		        return 'egy' + (isFuture || withoutSuffix ? ' nap' : ' napja');
  		      case 'dd':
  		        return num + (isFuture || withoutSuffix ? ' nap' : ' napja');
  		      case 'M':
  		        return 'egy' + (isFuture || withoutSuffix ? ' hónap' : ' hónapja');
  		      case 'MM':
  		        return num + (isFuture || withoutSuffix ? ' hónap' : ' hónapja');
  		      case 'y':
  		        return 'egy' + (isFuture || withoutSuffix ? ' év' : ' éve');
  		      case 'yy':
  		        return num + (isFuture || withoutSuffix ? ' év' : ' éve');
  		    }
  		    return '';
  		  }
  		  function week(isFuture) {
  		    return (isFuture ? '' : '[múlt] ') + '[' + weekEndings[this.day()] + '] LT[-kor]';
  		  }
  		  hooks.defineLocale('hu', {
  		    months: 'január_február_március_április_május_június_július_augusztus_szeptember_október_november_december'.split('_'),
  		    monthsShort: 'jan._feb._márc._ápr._máj._jún._júl._aug._szept._okt._nov._dec.'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'vasárnap_hétfő_kedd_szerda_csütörtök_péntek_szombat'.split('_'),
  		    weekdaysShort: 'vas_hét_kedd_sze_csüt_pén_szo'.split('_'),
  		    weekdaysMin: 'v_h_k_sze_cs_p_szo'.split('_'),
  		    longDateFormat: {
  		      LT: 'H:mm',
  		      LTS: 'H:mm:ss',
  		      L: 'YYYY.MM.DD.',
  		      LL: 'YYYY. MMMM D.',
  		      LLL: 'YYYY. MMMM D. H:mm',
  		      LLLL: 'YYYY. MMMM D., dddd H:mm'
  		    },
  		    meridiemParse: /de|du/i,
  		    isPM: function (input) {
  		      return input.charAt(1).toLowerCase() === 'u';
  		    },
  		    meridiem: function (hours, minutes, isLower) {
  		      if (hours < 12) {
  		        return isLower === true ? 'de' : 'DE';
  		      } else {
  		        return isLower === true ? 'du' : 'DU';
  		      }
  		    },
  		    calendar: {
  		      sameDay: '[ma] LT[-kor]',
  		      nextDay: '[holnap] LT[-kor]',
  		      nextWeek: function () {
  		        return week.call(this, true);
  		      },
  		      lastDay: '[tegnap] LT[-kor]',
  		      lastWeek: function () {
  		        return week.call(this, false);
  		      },
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%s múlva',
  		      past: '%s',
  		      s: translate$4,
  		      ss: translate$4,
  		      m: translate$4,
  		      mm: translate$4,
  		      h: translate$4,
  		      hh: translate$4,
  		      d: translate$4,
  		      dd: translate$4,
  		      M: translate$4,
  		      MM: translate$4,
  		      y: translate$4,
  		      yy: translate$4
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}\./,
  		    ordinal: '%d.',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('hy-am', {
  		    months: {
  		      format: 'հունվարի_փետրվարի_մարտի_ապրիլի_մայիսի_հունիսի_հուլիսի_օգոստոսի_սեպտեմբերի_հոկտեմբերի_նոյեմբերի_դեկտեմբերի'.split('_'),
  		      standalone: 'հունվար_փետրվար_մարտ_ապրիլ_մայիս_հունիս_հուլիս_օգոստոս_սեպտեմբեր_հոկտեմբեր_նոյեմբեր_դեկտեմբեր'.split('_')
  		    },
  		    monthsShort: 'հնվ_փտր_մրտ_ապր_մյս_հնս_հլս_օգս_սպտ_հկտ_նմբ_դկտ'.split('_'),
  		    weekdays: 'կիրակի_երկուշաբթի_երեքշաբթի_չորեքշաբթի_հինգշաբթի_ուրբաթ_շաբաթ'.split('_'),
  		    weekdaysShort: 'կրկ_երկ_երք_չրք_հնգ_ուրբ_շբթ'.split('_'),
  		    weekdaysMin: 'կրկ_երկ_երք_չրք_հնգ_ուրբ_շբթ'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'D MMMM YYYY թ.',
  		      LLL: 'D MMMM YYYY թ., HH:mm',
  		      LLLL: 'dddd, D MMMM YYYY թ., HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[այսօր] LT',
  		      nextDay: '[վաղը] LT',
  		      lastDay: '[երեկ] LT',
  		      nextWeek: function () {
  		        return 'dddd [օրը ժամը] LT';
  		      },
  		      lastWeek: function () {
  		        return '[անցած] dddd [օրը ժամը] LT';
  		      },
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%s հետո',
  		      past: '%s առաջ',
  		      s: 'մի քանի վայրկյան',
  		      ss: '%d վայրկյան',
  		      m: 'րոպե',
  		      mm: '%d րոպե',
  		      h: 'ժամ',
  		      hh: '%d ժամ',
  		      d: 'օր',
  		      dd: '%d օր',
  		      M: 'ամիս',
  		      MM: '%d ամիս',
  		      y: 'տարի',
  		      yy: '%d տարի'
  		    },
  		    meridiemParse: /գիշերվա|առավոտվա|ցերեկվա|երեկոյան/,
  		    isPM: function (input) {
  		      return /^(ցերեկվա|երեկոյան)$/.test(input);
  		    },
  		    meridiem: function (hour) {
  		      if (hour < 4) {
  		        return 'գիշերվա';
  		      } else if (hour < 12) {
  		        return 'առավոտվա';
  		      } else if (hour < 17) {
  		        return 'ցերեկվա';
  		      } else {
  		        return 'երեկոյան';
  		      }
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}|\d{1,2}-(ին|րդ)/,
  		    ordinal: function (number, period) {
  		      switch (period) {
  		        case 'DDD':
  		        case 'w':
  		        case 'W':
  		        case 'DDDo':
  		          if (number === 1) {
  		            return number + '-ին';
  		          }
  		          return number + '-րդ';
  		        default:
  		          return number;
  		      }
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 7 // The week that contains Jan 7th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('id', {
  		    months: 'Januari_Februari_Maret_April_Mei_Juni_Juli_Agustus_September_Oktober_November_Desember'.split('_'),
  		    monthsShort: 'Jan_Feb_Mar_Apr_Mei_Jun_Jul_Agt_Sep_Okt_Nov_Des'.split('_'),
  		    weekdays: 'Minggu_Senin_Selasa_Rabu_Kamis_Jumat_Sabtu'.split('_'),
  		    weekdaysShort: 'Min_Sen_Sel_Rab_Kam_Jum_Sab'.split('_'),
  		    weekdaysMin: 'Mg_Sn_Sl_Rb_Km_Jm_Sb'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH.mm',
  		      LTS: 'HH.mm.ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY [pukul] HH.mm',
  		      LLLL: 'dddd, D MMMM YYYY [pukul] HH.mm'
  		    },
  		    meridiemParse: /pagi|siang|sore|malam/,
  		    meridiemHour: function (hour, meridiem) {
  		      if (hour === 12) {
  		        hour = 0;
  		      }
  		      if (meridiem === 'pagi') {
  		        return hour;
  		      } else if (meridiem === 'siang') {
  		        return hour >= 11 ? hour : hour + 12;
  		      } else if (meridiem === 'sore' || meridiem === 'malam') {
  		        return hour + 12;
  		      }
  		    },
  		    meridiem: function (hours, minutes, isLower) {
  		      if (hours < 11) {
  		        return 'pagi';
  		      } else if (hours < 15) {
  		        return 'siang';
  		      } else if (hours < 19) {
  		        return 'sore';
  		      } else {
  		        return 'malam';
  		      }
  		    },
  		    calendar: {
  		      sameDay: '[Hari ini pukul] LT',
  		      nextDay: '[Besok pukul] LT',
  		      nextWeek: 'dddd [pukul] LT',
  		      lastDay: '[Kemarin pukul] LT',
  		      lastWeek: 'dddd [lalu pukul] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'dalam %s',
  		      past: '%s yang lalu',
  		      s: 'beberapa detik',
  		      ss: '%d detik',
  		      m: 'semenit',
  		      mm: '%d menit',
  		      h: 'sejam',
  		      hh: '%d jam',
  		      d: 'sehari',
  		      dd: '%d hari',
  		      M: 'sebulan',
  		      MM: '%d bulan',
  		      y: 'setahun',
  		      yy: '%d tahun'
  		    },
  		    week: {
  		      dow: 0,
  		      // Sunday is the first day of the week.
  		      doy: 6 // The week that contains Jan 6th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  function plural$2(n) {
  		    if (n % 100 === 11) {
  		      return true;
  		    } else if (n % 10 === 1) {
  		      return false;
  		    }
  		    return true;
  		  }
  		  function translate$5(number, withoutSuffix, key, isFuture) {
  		    var result = number + ' ';
  		    switch (key) {
  		      case 's':
  		        return withoutSuffix || isFuture ? 'nokkrar sekúndur' : 'nokkrum sekúndum';
  		      case 'ss':
  		        if (plural$2(number)) {
  		          return result + (withoutSuffix || isFuture ? 'sekúndur' : 'sekúndum');
  		        }
  		        return result + 'sekúnda';
  		      case 'm':
  		        return withoutSuffix ? 'mínúta' : 'mínútu';
  		      case 'mm':
  		        if (plural$2(number)) {
  		          return result + (withoutSuffix || isFuture ? 'mínútur' : 'mínútum');
  		        } else if (withoutSuffix) {
  		          return result + 'mínúta';
  		        }
  		        return result + 'mínútu';
  		      case 'hh':
  		        if (plural$2(number)) {
  		          return result + (withoutSuffix || isFuture ? 'klukkustundir' : 'klukkustundum');
  		        }
  		        return result + 'klukkustund';
  		      case 'd':
  		        if (withoutSuffix) {
  		          return 'dagur';
  		        }
  		        return isFuture ? 'dag' : 'degi';
  		      case 'dd':
  		        if (plural$2(number)) {
  		          if (withoutSuffix) {
  		            return result + 'dagar';
  		          }
  		          return result + (isFuture ? 'daga' : 'dögum');
  		        } else if (withoutSuffix) {
  		          return result + 'dagur';
  		        }
  		        return result + (isFuture ? 'dag' : 'degi');
  		      case 'M':
  		        if (withoutSuffix) {
  		          return 'mánuður';
  		        }
  		        return isFuture ? 'mánuð' : 'mánuði';
  		      case 'MM':
  		        if (plural$2(number)) {
  		          if (withoutSuffix) {
  		            return result + 'mánuðir';
  		          }
  		          return result + (isFuture ? 'mánuði' : 'mánuðum');
  		        } else if (withoutSuffix) {
  		          return result + 'mánuður';
  		        }
  		        return result + (isFuture ? 'mánuð' : 'mánuði');
  		      case 'y':
  		        return withoutSuffix || isFuture ? 'ár' : 'ári';
  		      case 'yy':
  		        if (plural$2(number)) {
  		          return result + (withoutSuffix || isFuture ? 'ár' : 'árum');
  		        }
  		        return result + (withoutSuffix || isFuture ? 'ár' : 'ári');
  		    }
  		  }
  		  hooks.defineLocale('is', {
  		    months: 'janúar_febrúar_mars_apríl_maí_júní_júlí_ágúst_september_október_nóvember_desember'.split('_'),
  		    monthsShort: 'jan_feb_mar_apr_maí_jún_júl_ágú_sep_okt_nóv_des'.split('_'),
  		    weekdays: 'sunnudagur_mánudagur_þriðjudagur_miðvikudagur_fimmtudagur_föstudagur_laugardagur'.split('_'),
  		    weekdaysShort: 'sun_mán_þri_mið_fim_fös_lau'.split('_'),
  		    weekdaysMin: 'Su_Má_Þr_Mi_Fi_Fö_La'.split('_'),
  		    longDateFormat: {
  		      LT: 'H:mm',
  		      LTS: 'H:mm:ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'D. MMMM YYYY',
  		      LLL: 'D. MMMM YYYY [kl.] H:mm',
  		      LLLL: 'dddd, D. MMMM YYYY [kl.] H:mm'
  		    },
  		    calendar: {
  		      sameDay: '[í dag kl.] LT',
  		      nextDay: '[á morgun kl.] LT',
  		      nextWeek: 'dddd [kl.] LT',
  		      lastDay: '[í gær kl.] LT',
  		      lastWeek: '[síðasta] dddd [kl.] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'eftir %s',
  		      past: 'fyrir %s síðan',
  		      s: translate$5,
  		      ss: translate$5,
  		      m: translate$5,
  		      mm: translate$5,
  		      h: 'klukkustund',
  		      hh: translate$5,
  		      d: translate$5,
  		      dd: translate$5,
  		      M: translate$5,
  		      MM: translate$5,
  		      y: translate$5,
  		      yy: translate$5
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}\./,
  		    ordinal: '%d.',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('it-ch', {
  		    months: 'gennaio_febbraio_marzo_aprile_maggio_giugno_luglio_agosto_settembre_ottobre_novembre_dicembre'.split('_'),
  		    monthsShort: 'gen_feb_mar_apr_mag_giu_lug_ago_set_ott_nov_dic'.split('_'),
  		    weekdays: 'domenica_lunedì_martedì_mercoledì_giovedì_venerdì_sabato'.split('_'),
  		    weekdaysShort: 'dom_lun_mar_mer_gio_ven_sab'.split('_'),
  		    weekdaysMin: 'do_lu_ma_me_gi_ve_sa'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Oggi alle] LT',
  		      nextDay: '[Domani alle] LT',
  		      nextWeek: 'dddd [alle] LT',
  		      lastDay: '[Ieri alle] LT',
  		      lastWeek: function () {
  		        switch (this.day()) {
  		          case 0:
  		            return '[la scorsa] dddd [alle] LT';
  		          default:
  		            return '[lo scorso] dddd [alle] LT';
  		        }
  		      },
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: function (s) {
  		        return (/^[0-9].+$/.test(s) ? 'tra' : 'in') + ' ' + s;
  		      },
  		      past: '%s fa',
  		      s: 'alcuni secondi',
  		      ss: '%d secondi',
  		      m: 'un minuto',
  		      mm: '%d minuti',
  		      h: "un'ora",
  		      hh: '%d ore',
  		      d: 'un giorno',
  		      dd: '%d giorni',
  		      M: 'un mese',
  		      MM: '%d mesi',
  		      y: 'un anno',
  		      yy: '%d anni'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}º/,
  		    ordinal: '%dº',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('it', {
  		    months: 'gennaio_febbraio_marzo_aprile_maggio_giugno_luglio_agosto_settembre_ottobre_novembre_dicembre'.split('_'),
  		    monthsShort: 'gen_feb_mar_apr_mag_giu_lug_ago_set_ott_nov_dic'.split('_'),
  		    weekdays: 'domenica_lunedì_martedì_mercoledì_giovedì_venerdì_sabato'.split('_'),
  		    weekdaysShort: 'dom_lun_mar_mer_gio_ven_sab'.split('_'),
  		    weekdaysMin: 'do_lu_ma_me_gi_ve_sa'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: function () {
  		        return '[Oggi a' + (this.hours() > 1 ? 'lle ' : this.hours() === 0 ? ' ' : "ll'") + ']LT';
  		      },
  		      nextDay: function () {
  		        return '[Domani a' + (this.hours() > 1 ? 'lle ' : this.hours() === 0 ? ' ' : "ll'") + ']LT';
  		      },
  		      nextWeek: function () {
  		        return 'dddd [a' + (this.hours() > 1 ? 'lle ' : this.hours() === 0 ? ' ' : "ll'") + ']LT';
  		      },
  		      lastDay: function () {
  		        return '[Ieri a' + (this.hours() > 1 ? 'lle ' : this.hours() === 0 ? ' ' : "ll'") + ']LT';
  		      },
  		      lastWeek: function () {
  		        switch (this.day()) {
  		          case 0:
  		            return '[La scorsa] dddd [a' + (this.hours() > 1 ? 'lle ' : this.hours() === 0 ? ' ' : "ll'") + ']LT';
  		          default:
  		            return '[Lo scorso] dddd [a' + (this.hours() > 1 ? 'lle ' : this.hours() === 0 ? ' ' : "ll'") + ']LT';
  		        }
  		      },
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'tra %s',
  		      past: '%s fa',
  		      s: 'alcuni secondi',
  		      ss: '%d secondi',
  		      m: 'un minuto',
  		      mm: '%d minuti',
  		      h: "un'ora",
  		      hh: '%d ore',
  		      d: 'un giorno',
  		      dd: '%d giorni',
  		      w: 'una settimana',
  		      ww: '%d settimane',
  		      M: 'un mese',
  		      MM: '%d mesi',
  		      y: 'un anno',
  		      yy: '%d anni'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}º/,
  		    ordinal: '%dº',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('ja', {
  		    eras: [{
  		      since: '2019-05-01',
  		      offset: 1,
  		      name: '令和',
  		      narrow: '㋿',
  		      abbr: 'R'
  		    }, {
  		      since: '1989-01-08',
  		      until: '2019-04-30',
  		      offset: 1,
  		      name: '平成',
  		      narrow: '㍻',
  		      abbr: 'H'
  		    }, {
  		      since: '1926-12-25',
  		      until: '1989-01-07',
  		      offset: 1,
  		      name: '昭和',
  		      narrow: '㍼',
  		      abbr: 'S'
  		    }, {
  		      since: '1912-07-30',
  		      until: '1926-12-24',
  		      offset: 1,
  		      name: '大正',
  		      narrow: '㍽',
  		      abbr: 'T'
  		    }, {
  		      since: '1873-01-01',
  		      until: '1912-07-29',
  		      offset: 6,
  		      name: '明治',
  		      narrow: '㍾',
  		      abbr: 'M'
  		    }, {
  		      since: '0001-01-01',
  		      until: '1873-12-31',
  		      offset: 1,
  		      name: '西暦',
  		      narrow: 'AD',
  		      abbr: 'AD'
  		    }, {
  		      since: '0000-12-31',
  		      until: -Infinity,
  		      offset: 1,
  		      name: '紀元前',
  		      narrow: 'BC',
  		      abbr: 'BC'
  		    }],
  		    eraYearOrdinalRegex: /(元|\d+)年/,
  		    eraYearOrdinalParse: function (input, match) {
  		      return match[1] === '元' ? 1 : parseInt(match[1] || input, 10);
  		    },
  		    months: '1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月'.split('_'),
  		    monthsShort: '1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月'.split('_'),
  		    weekdays: '日曜日_月曜日_火曜日_水曜日_木曜日_金曜日_土曜日'.split('_'),
  		    weekdaysShort: '日_月_火_水_木_金_土'.split('_'),
  		    weekdaysMin: '日_月_火_水_木_金_土'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'YYYY/MM/DD',
  		      LL: 'YYYY年M月D日',
  		      LLL: 'YYYY年M月D日 HH:mm',
  		      LLLL: 'YYYY年M月D日 dddd HH:mm',
  		      l: 'YYYY/MM/DD',
  		      ll: 'YYYY年M月D日',
  		      lll: 'YYYY年M月D日 HH:mm',
  		      llll: 'YYYY年M月D日(ddd) HH:mm'
  		    },
  		    meridiemParse: /午前|午後/i,
  		    isPM: function (input) {
  		      return input === '午後';
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 12) {
  		        return '午前';
  		      } else {
  		        return '午後';
  		      }
  		    },
  		    calendar: {
  		      sameDay: '[今日] LT',
  		      nextDay: '[明日] LT',
  		      nextWeek: function (now) {
  		        if (now.week() !== this.week()) {
  		          return '[来週]dddd LT';
  		        } else {
  		          return 'dddd LT';
  		        }
  		      },
  		      lastDay: '[昨日] LT',
  		      lastWeek: function (now) {
  		        if (this.week() !== now.week()) {
  		          return '[先週]dddd LT';
  		        } else {
  		          return 'dddd LT';
  		        }
  		      },
  		      sameElse: 'L'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}日/,
  		    ordinal: function (number, period) {
  		      switch (period) {
  		        case 'y':
  		          return number === 1 ? '元年' : number + '年';
  		        case 'd':
  		        case 'D':
  		        case 'DDD':
  		          return number + '日';
  		        default:
  		          return number;
  		      }
  		    },
  		    relativeTime: {
  		      future: '%s後',
  		      past: '%s前',
  		      s: '数秒',
  		      ss: '%d秒',
  		      m: '1分',
  		      mm: '%d分',
  		      h: '1時間',
  		      hh: '%d時間',
  		      d: '1日',
  		      dd: '%d日',
  		      M: '1ヶ月',
  		      MM: '%dヶ月',
  		      y: '1年',
  		      yy: '%d年'
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('jv', {
  		    months: 'Januari_Februari_Maret_April_Mei_Juni_Juli_Agustus_September_Oktober_Nopember_Desember'.split('_'),
  		    monthsShort: 'Jan_Feb_Mar_Apr_Mei_Jun_Jul_Ags_Sep_Okt_Nop_Des'.split('_'),
  		    weekdays: 'Minggu_Senen_Seloso_Rebu_Kemis_Jemuwah_Septu'.split('_'),
  		    weekdaysShort: 'Min_Sen_Sel_Reb_Kem_Jem_Sep'.split('_'),
  		    weekdaysMin: 'Mg_Sn_Sl_Rb_Km_Jm_Sp'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH.mm',
  		      LTS: 'HH.mm.ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY [pukul] HH.mm',
  		      LLLL: 'dddd, D MMMM YYYY [pukul] HH.mm'
  		    },
  		    meridiemParse: /enjing|siyang|sonten|ndalu/,
  		    meridiemHour: function (hour, meridiem) {
  		      if (hour === 12) {
  		        hour = 0;
  		      }
  		      if (meridiem === 'enjing') {
  		        return hour;
  		      } else if (meridiem === 'siyang') {
  		        return hour >= 11 ? hour : hour + 12;
  		      } else if (meridiem === 'sonten' || meridiem === 'ndalu') {
  		        return hour + 12;
  		      }
  		    },
  		    meridiem: function (hours, minutes, isLower) {
  		      if (hours < 11) {
  		        return 'enjing';
  		      } else if (hours < 15) {
  		        return 'siyang';
  		      } else if (hours < 19) {
  		        return 'sonten';
  		      } else {
  		        return 'ndalu';
  		      }
  		    },
  		    calendar: {
  		      sameDay: '[Dinten puniko pukul] LT',
  		      nextDay: '[Mbenjang pukul] LT',
  		      nextWeek: 'dddd [pukul] LT',
  		      lastDay: '[Kala wingi pukul] LT',
  		      lastWeek: 'dddd [kepengker pukul] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'wonten ing %s',
  		      past: '%s ingkang kepengker',
  		      s: 'sawetawis detik',
  		      ss: '%d detik',
  		      m: 'setunggal menit',
  		      mm: '%d menit',
  		      h: 'setunggal jam',
  		      hh: '%d jam',
  		      d: 'sedinten',
  		      dd: '%d dinten',
  		      M: 'sewulan',
  		      MM: '%d wulan',
  		      y: 'setaun',
  		      yy: '%d taun'
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 7 // The week that contains Jan 7th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('ka', {
  		    months: 'იანვარი_თებერვალი_მარტი_აპრილი_მაისი_ივნისი_ივლისი_აგვისტო_სექტემბერი_ოქტომბერი_ნოემბერი_დეკემბერი'.split('_'),
  		    monthsShort: 'იან_თებ_მარ_აპრ_მაი_ივნ_ივლ_აგვ_სექ_ოქტ_ნოე_დეკ'.split('_'),
  		    weekdays: {
  		      standalone: 'კვირა_ორშაბათი_სამშაბათი_ოთხშაბათი_ხუთშაბათი_პარასკევი_შაბათი'.split('_'),
  		      format: 'კვირას_ორშაბათს_სამშაბათს_ოთხშაბათს_ხუთშაბათს_პარასკევს_შაბათს'.split('_'),
  		      isFormat: /(წინა|შემდეგ)/
  		    },
  		    weekdaysShort: 'კვი_ორშ_სამ_ოთხ_ხუთ_პარ_შაბ'.split('_'),
  		    weekdaysMin: 'კვ_ორ_სა_ოთ_ხუ_პა_შა'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd, D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[დღეს] LT[-ზე]',
  		      nextDay: '[ხვალ] LT[-ზე]',
  		      lastDay: '[გუშინ] LT[-ზე]',
  		      nextWeek: '[შემდეგ] dddd LT[-ზე]',
  		      lastWeek: '[წინა] dddd LT-ზე',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: function (s) {
  		        return s.replace(/(წამ|წუთ|საათ|წელ|დღ|თვ)(ი|ე)/, function ($0, $1, $2) {
  		          return $2 === 'ი' ? $1 + 'ში' : $1 + $2 + 'ში';
  		        });
  		      },
  		      past: function (s) {
  		        if (/(წამი|წუთი|საათი|დღე|თვე)/.test(s)) {
  		          return s.replace(/(ი|ე)$/, 'ის წინ');
  		        }
  		        if (/წელი/.test(s)) {
  		          return s.replace(/წელი$/, 'წლის წინ');
  		        }
  		        return s;
  		      },
  		      s: 'რამდენიმე წამი',
  		      ss: '%d წამი',
  		      m: 'წუთი',
  		      mm: '%d წუთი',
  		      h: 'საათი',
  		      hh: '%d საათი',
  		      d: 'დღე',
  		      dd: '%d დღე',
  		      M: 'თვე',
  		      MM: '%d თვე',
  		      y: 'წელი',
  		      yy: '%d წელი'
  		    },
  		    dayOfMonthOrdinalParse: /0|1-ლი|მე-\d{1,2}|\d{1,2}-ე/,
  		    ordinal: function (number) {
  		      if (number === 0) {
  		        return number;
  		      }
  		      if (number === 1) {
  		        return number + '-ლი';
  		      }
  		      if (number < 20 || number <= 100 && number % 20 === 0 || number % 100 === 0) {
  		        return 'მე-' + number;
  		      }
  		      return number + '-ე';
  		    },
  		    week: {
  		      dow: 1,
  		      doy: 7
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var suffixes$1 = {
  		    0: '-ші',
  		    1: '-ші',
  		    2: '-ші',
  		    3: '-ші',
  		    4: '-ші',
  		    5: '-ші',
  		    6: '-шы',
  		    7: '-ші',
  		    8: '-ші',
  		    9: '-шы',
  		    10: '-шы',
  		    20: '-шы',
  		    30: '-шы',
  		    40: '-шы',
  		    50: '-ші',
  		    60: '-шы',
  		    70: '-ші',
  		    80: '-ші',
  		    90: '-шы',
  		    100: '-ші'
  		  };
  		  hooks.defineLocale('kk', {
  		    months: 'қаңтар_ақпан_наурыз_сәуір_мамыр_маусым_шілде_тамыз_қыркүйек_қазан_қараша_желтоқсан'.split('_'),
  		    monthsShort: 'қаң_ақп_нау_сәу_мам_мау_шіл_там_қыр_қаз_қар_жел'.split('_'),
  		    weekdays: 'жексенбі_дүйсенбі_сейсенбі_сәрсенбі_бейсенбі_жұма_сенбі'.split('_'),
  		    weekdaysShort: 'жек_дүй_сей_сәр_бей_жұм_сен'.split('_'),
  		    weekdaysMin: 'жк_дй_сй_ср_бй_жм_сн'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd, D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Бүгін сағат] LT',
  		      nextDay: '[Ертең сағат] LT',
  		      nextWeek: 'dddd [сағат] LT',
  		      lastDay: '[Кеше сағат] LT',
  		      lastWeek: '[Өткен аптаның] dddd [сағат] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%s ішінде',
  		      past: '%s бұрын',
  		      s: 'бірнеше секунд',
  		      ss: '%d секунд',
  		      m: 'бір минут',
  		      mm: '%d минут',
  		      h: 'бір сағат',
  		      hh: '%d сағат',
  		      d: 'бір күн',
  		      dd: '%d күн',
  		      M: 'бір ай',
  		      MM: '%d ай',
  		      y: 'бір жыл',
  		      yy: '%d жыл'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}-(ші|шы)/,
  		    ordinal: function (number) {
  		      var a = number % 10,
  		        b = number >= 100 ? 100 : null;
  		      return number + (suffixes$1[number] || suffixes$1[a] || suffixes$1[b]);
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 7 // The week that contains Jan 7th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var symbolMap$a = {
  		      1: '១',
  		      2: '២',
  		      3: '៣',
  		      4: '៤',
  		      5: '៥',
  		      6: '៦',
  		      7: '៧',
  		      8: '៨',
  		      9: '៩',
  		      0: '០'
  		    },
  		    numberMap$9 = {
  		      '១': '1',
  		      '២': '2',
  		      '៣': '3',
  		      '៤': '4',
  		      '៥': '5',
  		      '៦': '6',
  		      '៧': '7',
  		      '៨': '8',
  		      '៩': '9',
  		      '០': '0'
  		    };
  		  hooks.defineLocale('km', {
  		    months: 'មករា_កុម្ភៈ_មីនា_មេសា_ឧសភា_មិថុនា_កក្កដា_សីហា_កញ្ញា_តុលា_វិច្ឆិកា_ធ្នូ'.split('_'),
  		    monthsShort: 'មករា_កុម្ភៈ_មីនា_មេសា_ឧសភា_មិថុនា_កក្កដា_សីហា_កញ្ញា_តុលា_វិច្ឆិកា_ធ្នូ'.split('_'),
  		    weekdays: 'អាទិត្យ_ច័ន្ទ_អង្គារ_ពុធ_ព្រហស្បតិ៍_សុក្រ_សៅរ៍'.split('_'),
  		    weekdaysShort: 'អា_ច_អ_ព_ព្រ_សុ_ស'.split('_'),
  		    weekdaysMin: 'អា_ច_អ_ព_ព្រ_សុ_ស'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd, D MMMM YYYY HH:mm'
  		    },
  		    meridiemParse: /ព្រឹក|ល្ងាច/,
  		    isPM: function (input) {
  		      return input === 'ល្ងាច';
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 12) {
  		        return 'ព្រឹក';
  		      } else {
  		        return 'ល្ងាច';
  		      }
  		    },
  		    calendar: {
  		      sameDay: '[ថ្ងៃនេះ ម៉ោង] LT',
  		      nextDay: '[ស្អែក ម៉ោង] LT',
  		      nextWeek: 'dddd [ម៉ោង] LT',
  		      lastDay: '[ម្សិលមិញ ម៉ោង] LT',
  		      lastWeek: 'dddd [សប្តាហ៍មុន] [ម៉ោង] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%sទៀត',
  		      past: '%sមុន',
  		      s: 'ប៉ុន្មានវិនាទី',
  		      ss: '%d វិនាទី',
  		      m: 'មួយនាទី',
  		      mm: '%d នាទី',
  		      h: 'មួយម៉ោង',
  		      hh: '%d ម៉ោង',
  		      d: 'មួយថ្ងៃ',
  		      dd: '%d ថ្ងៃ',
  		      M: 'មួយខែ',
  		      MM: '%d ខែ',
  		      y: 'មួយឆ្នាំ',
  		      yy: '%d ឆ្នាំ'
  		    },
  		    dayOfMonthOrdinalParse: /ទី\d{1,2}/,
  		    ordinal: 'ទី%d',
  		    preparse: function (string) {
  		      return string.replace(/[១២៣៤៥៦៧៨៩០]/g, function (match) {
  		        return numberMap$9[match];
  		      });
  		    },
  		    postformat: function (string) {
  		      return string.replace(/\d/g, function (match) {
  		        return symbolMap$a[match];
  		      });
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var symbolMap$b = {
  		      1: '೧',
  		      2: '೨',
  		      3: '೩',
  		      4: '೪',
  		      5: '೫',
  		      6: '೬',
  		      7: '೭',
  		      8: '೮',
  		      9: '೯',
  		      0: '೦'
  		    },
  		    numberMap$a = {
  		      '೧': '1',
  		      '೨': '2',
  		      '೩': '3',
  		      '೪': '4',
  		      '೫': '5',
  		      '೬': '6',
  		      '೭': '7',
  		      '೮': '8',
  		      '೯': '9',
  		      '೦': '0'
  		    };
  		  hooks.defineLocale('kn', {
  		    months: 'ಜನವರಿ_ಫೆಬ್ರವರಿ_ಮಾರ್ಚ್_ಏಪ್ರಿಲ್_ಮೇ_ಜೂನ್_ಜುಲೈ_ಆಗಸ್ಟ್_ಸೆಪ್ಟೆಂಬರ್_ಅಕ್ಟೋಬರ್_ನವೆಂಬರ್_ಡಿಸೆಂಬರ್'.split('_'),
  		    monthsShort: 'ಜನ_ಫೆಬ್ರ_ಮಾರ್ಚ್_ಏಪ್ರಿಲ್_ಮೇ_ಜೂನ್_ಜುಲೈ_ಆಗಸ್ಟ್_ಸೆಪ್ಟೆಂ_ಅಕ್ಟೋ_ನವೆಂ_ಡಿಸೆಂ'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'ಭಾನುವಾರ_ಸೋಮವಾರ_ಮಂಗಳವಾರ_ಬುಧವಾರ_ಗುರುವಾರ_ಶುಕ್ರವಾರ_ಶನಿವಾರ'.split('_'),
  		    weekdaysShort: 'ಭಾನು_ಸೋಮ_ಮಂಗಳ_ಬುಧ_ಗುರು_ಶುಕ್ರ_ಶನಿ'.split('_'),
  		    weekdaysMin: 'ಭಾ_ಸೋ_ಮಂ_ಬು_ಗು_ಶು_ಶ'.split('_'),
  		    longDateFormat: {
  		      LT: 'A h:mm',
  		      LTS: 'A h:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY, A h:mm',
  		      LLLL: 'dddd, D MMMM YYYY, A h:mm'
  		    },
  		    calendar: {
  		      sameDay: '[ಇಂದು] LT',
  		      nextDay: '[ನಾಳೆ] LT',
  		      nextWeek: 'dddd, LT',
  		      lastDay: '[ನಿನ್ನೆ] LT',
  		      lastWeek: '[ಕೊನೆಯ] dddd, LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%s ನಂತರ',
  		      past: '%s ಹಿಂದೆ',
  		      s: 'ಕೆಲವು ಕ್ಷಣಗಳು',
  		      ss: '%d ಸೆಕೆಂಡುಗಳು',
  		      m: 'ಒಂದು ನಿಮಿಷ',
  		      mm: '%d ನಿಮಿಷ',
  		      h: 'ಒಂದು ಗಂಟೆ',
  		      hh: '%d ಗಂಟೆ',
  		      d: 'ಒಂದು ದಿನ',
  		      dd: '%d ದಿನ',
  		      M: 'ಒಂದು ತಿಂಗಳು',
  		      MM: '%d ತಿಂಗಳು',
  		      y: 'ಒಂದು ವರ್ಷ',
  		      yy: '%d ವರ್ಷ'
  		    },
  		    preparse: function (string) {
  		      return string.replace(/[೧೨೩೪೫೬೭೮೯೦]/g, function (match) {
  		        return numberMap$a[match];
  		      });
  		    },
  		    postformat: function (string) {
  		      return string.replace(/\d/g, function (match) {
  		        return symbolMap$b[match];
  		      });
  		    },
  		    meridiemParse: /ರಾತ್ರಿ|ಬೆಳಿಗ್ಗೆ|ಮಧ್ಯಾಹ್ನ|ಸಂಜೆ/,
  		    meridiemHour: function (hour, meridiem) {
  		      if (hour === 12) {
  		        hour = 0;
  		      }
  		      if (meridiem === 'ರಾತ್ರಿ') {
  		        return hour < 4 ? hour : hour + 12;
  		      } else if (meridiem === 'ಬೆಳಿಗ್ಗೆ') {
  		        return hour;
  		      } else if (meridiem === 'ಮಧ್ಯಾಹ್ನ') {
  		        return hour >= 10 ? hour : hour + 12;
  		      } else if (meridiem === 'ಸಂಜೆ') {
  		        return hour + 12;
  		      }
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 4) {
  		        return 'ರಾತ್ರಿ';
  		      } else if (hour < 10) {
  		        return 'ಬೆಳಿಗ್ಗೆ';
  		      } else if (hour < 17) {
  		        return 'ಮಧ್ಯಾಹ್ನ';
  		      } else if (hour < 20) {
  		        return 'ಸಂಜೆ';
  		      } else {
  		        return 'ರಾತ್ರಿ';
  		      }
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(ನೇ)/,
  		    ordinal: function (number) {
  		      return number + 'ನೇ';
  		    },
  		    week: {
  		      dow: 0,
  		      // Sunday is the first day of the week.
  		      doy: 6 // The week that contains Jan 6th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('ko', {
  		    months: '1월_2월_3월_4월_5월_6월_7월_8월_9월_10월_11월_12월'.split('_'),
  		    monthsShort: '1월_2월_3월_4월_5월_6월_7월_8월_9월_10월_11월_12월'.split('_'),
  		    weekdays: '일요일_월요일_화요일_수요일_목요일_금요일_토요일'.split('_'),
  		    weekdaysShort: '일_월_화_수_목_금_토'.split('_'),
  		    weekdaysMin: '일_월_화_수_목_금_토'.split('_'),
  		    longDateFormat: {
  		      LT: 'A h:mm',
  		      LTS: 'A h:mm:ss',
  		      L: 'YYYY.MM.DD.',
  		      LL: 'YYYY년 MMMM D일',
  		      LLL: 'YYYY년 MMMM D일 A h:mm',
  		      LLLL: 'YYYY년 MMMM D일 dddd A h:mm',
  		      l: 'YYYY.MM.DD.',
  		      ll: 'YYYY년 MMMM D일',
  		      lll: 'YYYY년 MMMM D일 A h:mm',
  		      llll: 'YYYY년 MMMM D일 dddd A h:mm'
  		    },
  		    calendar: {
  		      sameDay: '오늘 LT',
  		      nextDay: '내일 LT',
  		      nextWeek: 'dddd LT',
  		      lastDay: '어제 LT',
  		      lastWeek: '지난주 dddd LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%s 후',
  		      past: '%s 전',
  		      s: '몇 초',
  		      ss: '%d초',
  		      m: '1분',
  		      mm: '%d분',
  		      h: '한 시간',
  		      hh: '%d시간',
  		      d: '하루',
  		      dd: '%d일',
  		      M: '한 달',
  		      MM: '%d달',
  		      y: '일 년',
  		      yy: '%d년'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(일|월|주)/,
  		    ordinal: function (number, period) {
  		      switch (period) {
  		        case 'd':
  		        case 'D':
  		        case 'DDD':
  		          return number + '일';
  		        case 'M':
  		          return number + '월';
  		        case 'w':
  		        case 'W':
  		          return number + '주';
  		        default:
  		          return number;
  		      }
  		    },
  		    meridiemParse: /오전|오후/,
  		    isPM: function (token) {
  		      return token === '오후';
  		    },
  		    meridiem: function (hour, minute, isUpper) {
  		      return hour < 12 ? '오전' : '오후';
  		    }
  		  });

  		  //! moment.js locale configuration

  		  function processRelativeTime$7(num, withoutSuffix, key, isFuture) {
  		    var format = {
  		      s: ['çend sanîye', 'çend sanîyeyan'],
  		      ss: [num + ' sanîye', num + ' sanîyeyan'],
  		      m: ['deqîqeyek', 'deqîqeyekê'],
  		      mm: [num + ' deqîqe', num + ' deqîqeyan'],
  		      h: ['saetek', 'saetekê'],
  		      hh: [num + ' saet', num + ' saetan'],
  		      d: ['rojek', 'rojekê'],
  		      dd: [num + ' roj', num + ' rojan'],
  		      w: ['hefteyek', 'hefteyekê'],
  		      ww: [num + ' hefte', num + ' hefteyan'],
  		      M: ['mehek', 'mehekê'],
  		      MM: [num + ' meh', num + ' mehan'],
  		      y: ['salek', 'salekê'],
  		      yy: [num + ' sal', num + ' salan']
  		    };
  		    return withoutSuffix ? format[key][0] : format[key][1];
  		  }
  		  // function obliqueNumSuffix(num) {
  		  //     if(num.includes(':'))
  		  //         num = parseInt(num.split(':')[0]);
  		  //     else
  		  //         num = parseInt(num);
  		  //     return num == 0 || num % 10 == 1 ? 'ê'
  		  //                         : (num > 10 && num % 10 == 0 ? 'î' : 'an');
  		  // }
  		  function ezafeNumSuffix(num) {
  		    num = '' + num;
  		    var l = num.substring(num.length - 1),
  		      ll = num.length > 1 ? num.substring(num.length - 2) : '';
  		    if (!(ll == 12 || ll == 13) && (l == '2' || l == '3' || ll == '50' || l == '70' || l == '80')) return 'yê';
  		    return 'ê';
  		  }
  		  hooks.defineLocale('ku-kmr', {
  		    // According to the spelling rules defined by the work group of Weqfa Mezopotamyayê (Mesopotamia Foundation)
  		    // this should be: 'Kanûna Paşîn_Sibat_Adar_Nîsan_Gulan_Hezîran_Tîrmeh_Tebax_Îlon_Çirîya Pêşîn_Çirîya Paşîn_Kanûna Pêşîn'
  		    // But the names below are more well known and handy
  		    months: 'Rêbendan_Sibat_Adar_Nîsan_Gulan_Hezîran_Tîrmeh_Tebax_Îlon_Cotmeh_Mijdar_Berfanbar'.split('_'),
  		    monthsShort: 'Rêb_Sib_Ada_Nîs_Gul_Hez_Tîr_Teb_Îlo_Cot_Mij_Ber'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'Yekşem_Duşem_Sêşem_Çarşem_Pêncşem_În_Şemî'.split('_'),
  		    weekdaysShort: 'Yek_Du_Sê_Çar_Pên_În_Şem'.split('_'),
  		    weekdaysMin: 'Ye_Du_Sê_Ça_Pê_În_Şe'.split('_'),
  		    meridiem: function (hours, minutes, isLower) {
  		      if (hours < 12) {
  		        return isLower ? 'bn' : 'BN';
  		      } else {
  		        return isLower ? 'pn' : 'PN';
  		      }
  		    },
  		    meridiemParse: /bn|BN|pn|PN/,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'Do MMMM[a] YYYY[an]',
  		      LLL: 'Do MMMM[a] YYYY[an] HH:mm',
  		      LLLL: 'dddd, Do MMMM[a] YYYY[an] HH:mm',
  		      ll: 'Do MMM[.] YYYY[an]',
  		      lll: 'Do MMM[.] YYYY[an] HH:mm',
  		      llll: 'ddd[.], Do MMM[.] YYYY[an] HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Îro di saet] LT [de]',
  		      nextDay: '[Sibê di saet] LT [de]',
  		      nextWeek: 'dddd [di saet] LT [de]',
  		      lastDay: '[Duh di saet] LT [de]',
  		      lastWeek: 'dddd[a borî di saet] LT [de]',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'di %s de',
  		      past: 'berî %s',
  		      s: processRelativeTime$7,
  		      ss: processRelativeTime$7,
  		      m: processRelativeTime$7,
  		      mm: processRelativeTime$7,
  		      h: processRelativeTime$7,
  		      hh: processRelativeTime$7,
  		      d: processRelativeTime$7,
  		      dd: processRelativeTime$7,
  		      w: processRelativeTime$7,
  		      ww: processRelativeTime$7,
  		      M: processRelativeTime$7,
  		      MM: processRelativeTime$7,
  		      y: processRelativeTime$7,
  		      yy: processRelativeTime$7
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(?:yê|ê|\.)/,
  		    ordinal: function (num, period) {
  		      var p = period.toLowerCase();
  		      if (p.includes('w') || p.includes('m')) return num + '.';
  		      return num + ezafeNumSuffix(num);
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var symbolMap$c = {
  		      1: '١',
  		      2: '٢',
  		      3: '٣',
  		      4: '٤',
  		      5: '٥',
  		      6: '٦',
  		      7: '٧',
  		      8: '٨',
  		      9: '٩',
  		      0: '٠'
  		    },
  		    numberMap$b = {
  		      '١': '1',
  		      '٢': '2',
  		      '٣': '3',
  		      '٤': '4',
  		      '٥': '5',
  		      '٦': '6',
  		      '٧': '7',
  		      '٨': '8',
  		      '٩': '9',
  		      '٠': '0'
  		    },
  		    months$8 = ['کانونی دووەم', 'شوبات', 'ئازار', 'نیسان', 'ئایار', 'حوزەیران', 'تەمموز', 'ئاب', 'ئەیلوول', 'تشرینی یەكەم', 'تشرینی دووەم', 'كانونی یەکەم'];
  		  hooks.defineLocale('ku', {
  		    months: months$8,
  		    monthsShort: months$8,
  		    weekdays: 'یه‌كشه‌ممه‌_دووشه‌ممه‌_سێشه‌ممه‌_چوارشه‌ممه‌_پێنجشه‌ممه‌_هه‌ینی_شه‌ممه‌'.split('_'),
  		    weekdaysShort: 'یه‌كشه‌م_دووشه‌م_سێشه‌م_چوارشه‌م_پێنجشه‌م_هه‌ینی_شه‌ممه‌'.split('_'),
  		    weekdaysMin: 'ی_د_س_چ_پ_ه_ش'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd, D MMMM YYYY HH:mm'
  		    },
  		    meridiemParse: /ئێواره‌|به‌یانی/,
  		    isPM: function (input) {
  		      return /ئێواره‌/.test(input);
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 12) {
  		        return 'به‌یانی';
  		      } else {
  		        return 'ئێواره‌';
  		      }
  		    },
  		    calendar: {
  		      sameDay: '[ئه‌مرۆ كاتژمێر] LT',
  		      nextDay: '[به‌یانی كاتژمێر] LT',
  		      nextWeek: 'dddd [كاتژمێر] LT',
  		      lastDay: '[دوێنێ كاتژمێر] LT',
  		      lastWeek: 'dddd [كاتژمێر] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'له‌ %s',
  		      past: '%s',
  		      s: 'چه‌ند چركه‌یه‌ك',
  		      ss: 'چركه‌ %d',
  		      m: 'یه‌ك خوله‌ك',
  		      mm: '%d خوله‌ك',
  		      h: 'یه‌ك كاتژمێر',
  		      hh: '%d كاتژمێر',
  		      d: 'یه‌ك ڕۆژ',
  		      dd: '%d ڕۆژ',
  		      M: 'یه‌ك مانگ',
  		      MM: '%d مانگ',
  		      y: 'یه‌ك ساڵ',
  		      yy: '%d ساڵ'
  		    },
  		    preparse: function (string) {
  		      return string.replace(/[١٢٣٤٥٦٧٨٩٠]/g, function (match) {
  		        return numberMap$b[match];
  		      }).replace(/،/g, ',');
  		    },
  		    postformat: function (string) {
  		      return string.replace(/\d/g, function (match) {
  		        return symbolMap$c[match];
  		      }).replace(/,/g, '،');
  		    },
  		    week: {
  		      dow: 6,
  		      // Saturday is the first day of the week.
  		      doy: 12 // The week that contains Jan 12th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var suffixes$2 = {
  		    0: '-чү',
  		    1: '-чи',
  		    2: '-чи',
  		    3: '-чү',
  		    4: '-чү',
  		    5: '-чи',
  		    6: '-чы',
  		    7: '-чи',
  		    8: '-чи',
  		    9: '-чу',
  		    10: '-чу',
  		    20: '-чы',
  		    30: '-чу',
  		    40: '-чы',
  		    50: '-чү',
  		    60: '-чы',
  		    70: '-чи',
  		    80: '-чи',
  		    90: '-чу',
  		    100: '-чү'
  		  };
  		  hooks.defineLocale('ky', {
  		    months: 'январь_февраль_март_апрель_май_июнь_июль_август_сентябрь_октябрь_ноябрь_декабрь'.split('_'),
  		    monthsShort: 'янв_фев_март_апр_май_июнь_июль_авг_сен_окт_ноя_дек'.split('_'),
  		    weekdays: 'Жекшемби_Дүйшөмбү_Шейшемби_Шаршемби_Бейшемби_Жума_Ишемби'.split('_'),
  		    weekdaysShort: 'Жек_Дүй_Шей_Шар_Бей_Жум_Ише'.split('_'),
  		    weekdaysMin: 'Жк_Дй_Шй_Шр_Бй_Жм_Иш'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd, D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Бүгүн саат] LT',
  		      nextDay: '[Эртең саат] LT',
  		      nextWeek: 'dddd [саат] LT',
  		      lastDay: '[Кечээ саат] LT',
  		      lastWeek: '[Өткөн аптанын] dddd [күнү] [саат] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%s ичинде',
  		      past: '%s мурун',
  		      s: 'бирнече секунд',
  		      ss: '%d секунд',
  		      m: 'бир мүнөт',
  		      mm: '%d мүнөт',
  		      h: 'бир саат',
  		      hh: '%d саат',
  		      d: 'бир күн',
  		      dd: '%d күн',
  		      M: 'бир ай',
  		      MM: '%d ай',
  		      y: 'бир жыл',
  		      yy: '%d жыл'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}-(чи|чы|чү|чу)/,
  		    ordinal: function (number) {
  		      var a = number % 10,
  		        b = number >= 100 ? 100 : null;
  		      return number + (suffixes$2[number] || suffixes$2[a] || suffixes$2[b]);
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 7 // The week that contains Jan 7th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  function processRelativeTime$8(number, withoutSuffix, key, isFuture) {
  		    var format = {
  		      m: ['eng Minutt', 'enger Minutt'],
  		      h: ['eng Stonn', 'enger Stonn'],
  		      d: ['een Dag', 'engem Dag'],
  		      M: ['ee Mount', 'engem Mount'],
  		      y: ['ee Joer', 'engem Joer']
  		    };
  		    return withoutSuffix ? format[key][0] : format[key][1];
  		  }
  		  function processFutureTime(string) {
  		    var number = string.substr(0, string.indexOf(' '));
  		    if (eifelerRegelAppliesToNumber(number)) {
  		      return 'a ' + string;
  		    }
  		    return 'an ' + string;
  		  }
  		  function processPastTime(string) {
  		    var number = string.substr(0, string.indexOf(' '));
  		    if (eifelerRegelAppliesToNumber(number)) {
  		      return 'viru ' + string;
  		    }
  		    return 'virun ' + string;
  		  }
  		  /**
  		   * Returns true if the word before the given number loses the '-n' ending.
  		   * e.g. 'an 10 Deeg' but 'a 5 Deeg'
  		   *
  		   * @param number {integer}
  		   * @returns {boolean}
  		   */
  		  function eifelerRegelAppliesToNumber(number) {
  		    number = parseInt(number, 10);
  		    if (isNaN(number)) {
  		      return false;
  		    }
  		    if (number < 0) {
  		      // Negative Number --> always true
  		      return true;
  		    } else if (number < 10) {
  		      // Only 1 digit
  		      if (4 <= number && number <= 7) {
  		        return true;
  		      }
  		      return false;
  		    } else if (number < 100) {
  		      // 2 digits
  		      var lastDigit = number % 10,
  		        firstDigit = number / 10;
  		      if (lastDigit === 0) {
  		        return eifelerRegelAppliesToNumber(firstDigit);
  		      }
  		      return eifelerRegelAppliesToNumber(lastDigit);
  		    } else if (number < 10000) {
  		      // 3 or 4 digits --> recursively check first digit
  		      while (number >= 10) {
  		        number = number / 10;
  		      }
  		      return eifelerRegelAppliesToNumber(number);
  		    } else {
  		      // Anything larger than 4 digits: recursively check first n-3 digits
  		      number = number / 1000;
  		      return eifelerRegelAppliesToNumber(number);
  		    }
  		  }
  		  hooks.defineLocale('lb', {
  		    months: 'Januar_Februar_Mäerz_Abrëll_Mee_Juni_Juli_August_September_Oktober_November_Dezember'.split('_'),
  		    monthsShort: 'Jan._Febr._Mrz._Abr._Mee_Jun._Jul._Aug._Sept._Okt._Nov._Dez.'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'Sonndeg_Méindeg_Dënschdeg_Mëttwoch_Donneschdeg_Freideg_Samschdeg'.split('_'),
  		    weekdaysShort: 'So._Mé._Dë._Më._Do._Fr._Sa.'.split('_'),
  		    weekdaysMin: 'So_Mé_Dë_Më_Do_Fr_Sa'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'H:mm [Auer]',
  		      LTS: 'H:mm:ss [Auer]',
  		      L: 'DD.MM.YYYY',
  		      LL: 'D. MMMM YYYY',
  		      LLL: 'D. MMMM YYYY H:mm [Auer]',
  		      LLLL: 'dddd, D. MMMM YYYY H:mm [Auer]'
  		    },
  		    calendar: {
  		      sameDay: '[Haut um] LT',
  		      sameElse: 'L',
  		      nextDay: '[Muer um] LT',
  		      nextWeek: 'dddd [um] LT',
  		      lastDay: '[Gëschter um] LT',
  		      lastWeek: function () {
  		        // Different date string for 'Dënschdeg' (Tuesday) and 'Donneschdeg' (Thursday) due to phonological rule
  		        switch (this.day()) {
  		          case 2:
  		          case 4:
  		            return '[Leschten] dddd [um] LT';
  		          default:
  		            return '[Leschte] dddd [um] LT';
  		        }
  		      }
  		    },
  		    relativeTime: {
  		      future: processFutureTime,
  		      past: processPastTime,
  		      s: 'e puer Sekonnen',
  		      ss: '%d Sekonnen',
  		      m: processRelativeTime$8,
  		      mm: '%d Minutten',
  		      h: processRelativeTime$8,
  		      hh: '%d Stonnen',
  		      d: processRelativeTime$8,
  		      dd: '%d Deeg',
  		      M: processRelativeTime$8,
  		      MM: '%d Méint',
  		      y: processRelativeTime$8,
  		      yy: '%d Joer'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}\./,
  		    ordinal: '%d.',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('lo', {
  		    months: 'ມັງກອນ_ກຸມພາ_ມີນາ_ເມສາ_ພຶດສະພາ_ມິຖຸນາ_ກໍລະກົດ_ສິງຫາ_ກັນຍາ_ຕຸລາ_ພະຈິກ_ທັນວາ'.split('_'),
  		    monthsShort: 'ມັງກອນ_ກຸມພາ_ມີນາ_ເມສາ_ພຶດສະພາ_ມິຖຸນາ_ກໍລະກົດ_ສິງຫາ_ກັນຍາ_ຕຸລາ_ພະຈິກ_ທັນວາ'.split('_'),
  		    weekdays: 'ອາທິດ_ຈັນ_ອັງຄານ_ພຸດ_ພະຫັດ_ສຸກ_ເສົາ'.split('_'),
  		    weekdaysShort: 'ທິດ_ຈັນ_ອັງຄານ_ພຸດ_ພະຫັດ_ສຸກ_ເສົາ'.split('_'),
  		    weekdaysMin: 'ທ_ຈ_ອຄ_ພ_ພຫ_ສກ_ສ'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'ວັນdddd D MMMM YYYY HH:mm'
  		    },
  		    meridiemParse: /ຕອນເຊົ້າ|ຕອນແລງ/,
  		    isPM: function (input) {
  		      return input === 'ຕອນແລງ';
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 12) {
  		        return 'ຕອນເຊົ້າ';
  		      } else {
  		        return 'ຕອນແລງ';
  		      }
  		    },
  		    calendar: {
  		      sameDay: '[ມື້ນີ້ເວລາ] LT',
  		      nextDay: '[ມື້ອື່ນເວລາ] LT',
  		      nextWeek: '[ວັນ]dddd[ໜ້າເວລາ] LT',
  		      lastDay: '[ມື້ວານນີ້ເວລາ] LT',
  		      lastWeek: '[ວັນ]dddd[ແລ້ວນີ້ເວລາ] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'ອີກ %s',
  		      past: '%sຜ່ານມາ',
  		      s: 'ບໍ່ເທົ່າໃດວິນາທີ',
  		      ss: '%d ວິນາທີ',
  		      m: '1 ນາທີ',
  		      mm: '%d ນາທີ',
  		      h: '1 ຊົ່ວໂມງ',
  		      hh: '%d ຊົ່ວໂມງ',
  		      d: '1 ມື້',
  		      dd: '%d ມື້',
  		      M: '1 ເດືອນ',
  		      MM: '%d ເດືອນ',
  		      y: '1 ປີ',
  		      yy: '%d ປີ'
  		    },
  		    dayOfMonthOrdinalParse: /(ທີ່)\d{1,2}/,
  		    ordinal: function (number) {
  		      return 'ທີ່' + number;
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var units = {
  		    ss: 'sekundė_sekundžių_sekundes',
  		    m: 'minutė_minutės_minutę',
  		    mm: 'minutės_minučių_minutes',
  		    h: 'valanda_valandos_valandą',
  		    hh: 'valandos_valandų_valandas',
  		    d: 'diena_dienos_dieną',
  		    dd: 'dienos_dienų_dienas',
  		    M: 'mėnuo_mėnesio_mėnesį',
  		    MM: 'mėnesiai_mėnesių_mėnesius',
  		    y: 'metai_metų_metus',
  		    yy: 'metai_metų_metus'
  		  };
  		  function translateSeconds(number, withoutSuffix, key, isFuture) {
  		    if (withoutSuffix) {
  		      return 'kelios sekundės';
  		    } else {
  		      return isFuture ? 'kelių sekundžių' : 'kelias sekundes';
  		    }
  		  }
  		  function translateSingular(number, withoutSuffix, key, isFuture) {
  		    return withoutSuffix ? forms(key)[0] : isFuture ? forms(key)[1] : forms(key)[2];
  		  }
  		  function special(number) {
  		    return number % 10 === 0 || number > 10 && number < 20;
  		  }
  		  function forms(key) {
  		    return units[key].split('_');
  		  }
  		  function translate$6(number, withoutSuffix, key, isFuture) {
  		    var result = number + ' ';
  		    if (number === 1) {
  		      return result + translateSingular(number, withoutSuffix, key[0], isFuture);
  		    } else if (withoutSuffix) {
  		      return result + (special(number) ? forms(key)[1] : forms(key)[0]);
  		    } else {
  		      if (isFuture) {
  		        return result + forms(key)[1];
  		      } else {
  		        return result + (special(number) ? forms(key)[1] : forms(key)[2]);
  		      }
  		    }
  		  }
  		  hooks.defineLocale('lt', {
  		    months: {
  		      format: 'sausio_vasario_kovo_balandžio_gegužės_birželio_liepos_rugpjūčio_rugsėjo_spalio_lapkričio_gruodžio'.split('_'),
  		      standalone: 'sausis_vasaris_kovas_balandis_gegužė_birželis_liepa_rugpjūtis_rugsėjis_spalis_lapkritis_gruodis'.split('_'),
  		      isFormat: /D[oD]?(\[[^\[\]]*\]|\s)+MMMM?|MMMM?(\[[^\[\]]*\]|\s)+D[oD]?/
  		    },
  		    monthsShort: 'sau_vas_kov_bal_geg_bir_lie_rgp_rgs_spa_lap_grd'.split('_'),
  		    weekdays: {
  		      format: 'sekmadienį_pirmadienį_antradienį_trečiadienį_ketvirtadienį_penktadienį_šeštadienį'.split('_'),
  		      standalone: 'sekmadienis_pirmadienis_antradienis_trečiadienis_ketvirtadienis_penktadienis_šeštadienis'.split('_'),
  		      isFormat: /dddd HH:mm/
  		    },
  		    weekdaysShort: 'Sek_Pir_Ant_Tre_Ket_Pen_Šeš'.split('_'),
  		    weekdaysMin: 'S_P_A_T_K_Pn_Š'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'YYYY-MM-DD',
  		      LL: 'YYYY [m.] MMMM D [d.]',
  		      LLL: 'YYYY [m.] MMMM D [d.], HH:mm [val.]',
  		      LLLL: 'YYYY [m.] MMMM D [d.], dddd, HH:mm [val.]',
  		      l: 'YYYY-MM-DD',
  		      ll: 'YYYY [m.] MMMM D [d.]',
  		      lll: 'YYYY [m.] MMMM D [d.], HH:mm [val.]',
  		      llll: 'YYYY [m.] MMMM D [d.], ddd, HH:mm [val.]'
  		    },
  		    calendar: {
  		      sameDay: '[Šiandien] LT',
  		      nextDay: '[Rytoj] LT',
  		      nextWeek: 'dddd LT',
  		      lastDay: '[Vakar] LT',
  		      lastWeek: '[Praėjusį] dddd LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'po %s',
  		      past: 'prieš %s',
  		      s: translateSeconds,
  		      ss: translate$6,
  		      m: translateSingular,
  		      mm: translate$6,
  		      h: translateSingular,
  		      hh: translate$6,
  		      d: translateSingular,
  		      dd: translate$6,
  		      M: translateSingular,
  		      MM: translate$6,
  		      y: translateSingular,
  		      yy: translate$6
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}-oji/,
  		    ordinal: function (number) {
  		      return number + '-oji';
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var units$1 = {
  		    ss: 'sekundes_sekundēm_sekunde_sekundes'.split('_'),
  		    m: 'minūtes_minūtēm_minūte_minūtes'.split('_'),
  		    mm: 'minūtes_minūtēm_minūte_minūtes'.split('_'),
  		    h: 'stundas_stundām_stunda_stundas'.split('_'),
  		    hh: 'stundas_stundām_stunda_stundas'.split('_'),
  		    d: 'dienas_dienām_diena_dienas'.split('_'),
  		    dd: 'dienas_dienām_diena_dienas'.split('_'),
  		    M: 'mēneša_mēnešiem_mēnesis_mēneši'.split('_'),
  		    MM: 'mēneša_mēnešiem_mēnesis_mēneši'.split('_'),
  		    y: 'gada_gadiem_gads_gadi'.split('_'),
  		    yy: 'gada_gadiem_gads_gadi'.split('_')
  		  };
  		  /**
  		   * @param withoutSuffix boolean true = a length of time; false = before/after a period of time.
  		   */
  		  function format$1(forms, number, withoutSuffix) {
  		    if (withoutSuffix) {
  		      // E.g. "21 minūte", "3 minūtes".
  		      return number % 10 === 1 && number % 100 !== 11 ? forms[2] : forms[3];
  		    } else {
  		      // E.g. "21 minūtes" as in "pēc 21 minūtes".
  		      // E.g. "3 minūtēm" as in "pēc 3 minūtēm".
  		      return number % 10 === 1 && number % 100 !== 11 ? forms[0] : forms[1];
  		    }
  		  }
  		  function relativeTimeWithPlural$1(number, withoutSuffix, key) {
  		    return number + ' ' + format$1(units$1[key], number, withoutSuffix);
  		  }
  		  function relativeTimeWithSingular(number, withoutSuffix, key) {
  		    return format$1(units$1[key], number, withoutSuffix);
  		  }
  		  function relativeSeconds(number, withoutSuffix) {
  		    return withoutSuffix ? 'dažas sekundes' : 'dažām sekundēm';
  		  }
  		  hooks.defineLocale('lv', {
  		    months: 'janvāris_februāris_marts_aprīlis_maijs_jūnijs_jūlijs_augusts_septembris_oktobris_novembris_decembris'.split('_'),
  		    monthsShort: 'jan_feb_mar_apr_mai_jūn_jūl_aug_sep_okt_nov_dec'.split('_'),
  		    weekdays: 'svētdiena_pirmdiena_otrdiena_trešdiena_ceturtdiena_piektdiena_sestdiena'.split('_'),
  		    weekdaysShort: 'Sv_P_O_T_C_Pk_S'.split('_'),
  		    weekdaysMin: 'Sv_P_O_T_C_Pk_S'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD.MM.YYYY.',
  		      LL: 'YYYY. [gada] D. MMMM',
  		      LLL: 'YYYY. [gada] D. MMMM, HH:mm',
  		      LLLL: 'YYYY. [gada] D. MMMM, dddd, HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Šodien pulksten] LT',
  		      nextDay: '[Rīt pulksten] LT',
  		      nextWeek: 'dddd [pulksten] LT',
  		      lastDay: '[Vakar pulksten] LT',
  		      lastWeek: '[Pagājušā] dddd [pulksten] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'pēc %s',
  		      past: 'pirms %s',
  		      s: relativeSeconds,
  		      ss: relativeTimeWithPlural$1,
  		      m: relativeTimeWithSingular,
  		      mm: relativeTimeWithPlural$1,
  		      h: relativeTimeWithSingular,
  		      hh: relativeTimeWithPlural$1,
  		      d: relativeTimeWithSingular,
  		      dd: relativeTimeWithPlural$1,
  		      M: relativeTimeWithSingular,
  		      MM: relativeTimeWithPlural$1,
  		      y: relativeTimeWithSingular,
  		      yy: relativeTimeWithPlural$1
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}\./,
  		    ordinal: '%d.',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var translator = {
  		    words: {
  		      //Different grammatical cases
  		      ss: ['sekund', 'sekunda', 'sekundi'],
  		      m: ['jedan minut', 'jednog minuta'],
  		      mm: ['minut', 'minuta', 'minuta'],
  		      h: ['jedan sat', 'jednog sata'],
  		      hh: ['sat', 'sata', 'sati'],
  		      dd: ['dan', 'dana', 'dana'],
  		      MM: ['mjesec', 'mjeseca', 'mjeseci'],
  		      yy: ['godina', 'godine', 'godina']
  		    },
  		    correctGrammaticalCase: function (number, wordKey) {
  		      return number === 1 ? wordKey[0] : number >= 2 && number <= 4 ? wordKey[1] : wordKey[2];
  		    },
  		    translate: function (number, withoutSuffix, key) {
  		      var wordKey = translator.words[key];
  		      if (key.length === 1) {
  		        return withoutSuffix ? wordKey[0] : wordKey[1];
  		      } else {
  		        return number + ' ' + translator.correctGrammaticalCase(number, wordKey);
  		      }
  		    }
  		  };
  		  hooks.defineLocale('me', {
  		    months: 'januar_februar_mart_april_maj_jun_jul_avgust_septembar_oktobar_novembar_decembar'.split('_'),
  		    monthsShort: 'jan._feb._mar._apr._maj_jun_jul_avg._sep._okt._nov._dec.'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'nedjelja_ponedjeljak_utorak_srijeda_četvrtak_petak_subota'.split('_'),
  		    weekdaysShort: 'ned._pon._uto._sri._čet._pet._sub.'.split('_'),
  		    weekdaysMin: 'ne_po_ut_sr_če_pe_su'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'H:mm',
  		      LTS: 'H:mm:ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'D. MMMM YYYY',
  		      LLL: 'D. MMMM YYYY H:mm',
  		      LLLL: 'dddd, D. MMMM YYYY H:mm'
  		    },
  		    calendar: {
  		      sameDay: '[danas u] LT',
  		      nextDay: '[sjutra u] LT',
  		      nextWeek: function () {
  		        switch (this.day()) {
  		          case 0:
  		            return '[u] [nedjelju] [u] LT';
  		          case 3:
  		            return '[u] [srijedu] [u] LT';
  		          case 6:
  		            return '[u] [subotu] [u] LT';
  		          case 1:
  		          case 2:
  		          case 4:
  		          case 5:
  		            return '[u] dddd [u] LT';
  		        }
  		      },
  		      lastDay: '[juče u] LT',
  		      lastWeek: function () {
  		        var lastWeekDays = ['[prošle] [nedjelje] [u] LT', '[prošlog] [ponedjeljka] [u] LT', '[prošlog] [utorka] [u] LT', '[prošle] [srijede] [u] LT', '[prošlog] [četvrtka] [u] LT', '[prošlog] [petka] [u] LT', '[prošle] [subote] [u] LT'];
  		        return lastWeekDays[this.day()];
  		      },
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'za %s',
  		      past: 'prije %s',
  		      s: 'nekoliko sekundi',
  		      ss: translator.translate,
  		      m: translator.translate,
  		      mm: translator.translate,
  		      h: translator.translate,
  		      hh: translator.translate,
  		      d: 'dan',
  		      dd: translator.translate,
  		      M: 'mjesec',
  		      MM: translator.translate,
  		      y: 'godinu',
  		      yy: translator.translate
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}\./,
  		    ordinal: '%d.',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 7 // The week that contains Jan 7th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('mi', {
  		    months: 'Kohi-tāte_Hui-tanguru_Poutū-te-rangi_Paenga-whāwhā_Haratua_Pipiri_Hōngoingoi_Here-turi-kōkā_Mahuru_Whiringa-ā-nuku_Whiringa-ā-rangi_Hakihea'.split('_'),
  		    monthsShort: 'Kohi_Hui_Pou_Pae_Hara_Pipi_Hōngoi_Here_Mahu_Whi-nu_Whi-ra_Haki'.split('_'),
  		    monthsRegex: /(?:['a-z\u0101\u014D\u016B]+\-?){1,3}/i,
  		    monthsStrictRegex: /(?:['a-z\u0101\u014D\u016B]+\-?){1,3}/i,
  		    monthsShortRegex: /(?:['a-z\u0101\u014D\u016B]+\-?){1,3}/i,
  		    monthsShortStrictRegex: /(?:['a-z\u0101\u014D\u016B]+\-?){1,2}/i,
  		    weekdays: 'Rātapu_Mane_Tūrei_Wenerei_Tāite_Paraire_Hātarei'.split('_'),
  		    weekdaysShort: 'Ta_Ma_Tū_We_Tāi_Pa_Hā'.split('_'),
  		    weekdaysMin: 'Ta_Ma_Tū_We_Tāi_Pa_Hā'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY [i] HH:mm',
  		      LLLL: 'dddd, D MMMM YYYY [i] HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[i teie mahana, i] LT',
  		      nextDay: '[apopo i] LT',
  		      nextWeek: 'dddd [i] LT',
  		      lastDay: '[inanahi i] LT',
  		      lastWeek: 'dddd [whakamutunga i] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'i roto i %s',
  		      past: '%s i mua',
  		      s: 'te hēkona ruarua',
  		      ss: '%d hēkona',
  		      m: 'he meneti',
  		      mm: '%d meneti',
  		      h: 'te haora',
  		      hh: '%d haora',
  		      d: 'he ra',
  		      dd: '%d ra',
  		      M: 'he marama',
  		      MM: '%d marama',
  		      y: 'he tau',
  		      yy: '%d tau'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}º/,
  		    ordinal: '%dº',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('mk', {
  		    months: 'јануари_февруари_март_април_мај_јуни_јули_август_септември_октомври_ноември_декември'.split('_'),
  		    monthsShort: 'јан_фев_мар_апр_мај_јун_јул_авг_сеп_окт_ное_дек'.split('_'),
  		    weekdays: 'недела_понеделник_вторник_среда_четврток_петок_сабота'.split('_'),
  		    weekdaysShort: 'нед_пон_вто_сре_чет_пет_саб'.split('_'),
  		    weekdaysMin: 'нe_пo_вт_ср_че_пе_сa'.split('_'),
  		    longDateFormat: {
  		      LT: 'H:mm',
  		      LTS: 'H:mm:ss',
  		      L: 'D.MM.YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY H:mm',
  		      LLLL: 'dddd, D MMMM YYYY H:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Денес во] LT',
  		      nextDay: '[Утре во] LT',
  		      nextWeek: '[Во] dddd [во] LT',
  		      lastDay: '[Вчера во] LT',
  		      lastWeek: function () {
  		        switch (this.day()) {
  		          case 0:
  		          case 3:
  		          case 6:
  		            return '[Изминатата] dddd [во] LT';
  		          case 1:
  		          case 2:
  		          case 4:
  		          case 5:
  		            return '[Изминатиот] dddd [во] LT';
  		        }
  		      },
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'за %s',
  		      past: 'пред %s',
  		      s: 'неколку секунди',
  		      ss: '%d секунди',
  		      m: 'една минута',
  		      mm: '%d минути',
  		      h: 'еден час',
  		      hh: '%d часа',
  		      d: 'еден ден',
  		      dd: '%d дена',
  		      M: 'еден месец',
  		      MM: '%d месеци',
  		      y: 'една година',
  		      yy: '%d години'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}-(ев|ен|ти|ви|ри|ми)/,
  		    ordinal: function (number) {
  		      var lastDigit = number % 10,
  		        last2Digits = number % 100;
  		      if (number === 0) {
  		        return number + '-ев';
  		      } else if (last2Digits === 0) {
  		        return number + '-ен';
  		      } else if (last2Digits > 10 && last2Digits < 20) {
  		        return number + '-ти';
  		      } else if (lastDigit === 1) {
  		        return number + '-ви';
  		      } else if (lastDigit === 2) {
  		        return number + '-ри';
  		      } else if (lastDigit === 7 || lastDigit === 8) {
  		        return number + '-ми';
  		      } else {
  		        return number + '-ти';
  		      }
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 7 // The week that contains Jan 7th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('ml', {
  		    months: 'ജനുവരി_ഫെബ്രുവരി_മാർച്ച്_ഏപ്രിൽ_മേയ്_ജൂൺ_ജൂലൈ_ഓഗസ്റ്റ്_സെപ്റ്റംബർ_ഒക്ടോബർ_നവംബർ_ഡിസംബർ'.split('_'),
  		    monthsShort: 'ജനു._ഫെബ്രു._മാർ._ഏപ്രി._മേയ്_ജൂൺ_ജൂലൈ._ഓഗ._സെപ്റ്റ._ഒക്ടോ._നവം._ഡിസം.'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'ഞായറാഴ്ച_തിങ്കളാഴ്ച_ചൊവ്വാഴ്ച_ബുധനാഴ്ച_വ്യാഴാഴ്ച_വെള്ളിയാഴ്ച_ശനിയാഴ്ച'.split('_'),
  		    weekdaysShort: 'ഞായർ_തിങ്കൾ_ചൊവ്വ_ബുധൻ_വ്യാഴം_വെള്ളി_ശനി'.split('_'),
  		    weekdaysMin: 'ഞാ_തി_ചൊ_ബു_വ്യാ_വെ_ശ'.split('_'),
  		    longDateFormat: {
  		      LT: 'A h:mm -നു',
  		      LTS: 'A h:mm:ss -നു',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY, A h:mm -നു',
  		      LLLL: 'dddd, D MMMM YYYY, A h:mm -നു'
  		    },
  		    calendar: {
  		      sameDay: '[ഇന്ന്] LT',
  		      nextDay: '[നാളെ] LT',
  		      nextWeek: 'dddd, LT',
  		      lastDay: '[ഇന്നലെ] LT',
  		      lastWeek: '[കഴിഞ്ഞ] dddd, LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%s കഴിഞ്ഞ്',
  		      past: '%s മുൻപ്',
  		      s: 'അൽപ നിമിഷങ്ങൾ',
  		      ss: '%d സെക്കൻഡ്',
  		      m: 'ഒരു മിനിറ്റ്',
  		      mm: '%d മിനിറ്റ്',
  		      h: 'ഒരു മണിക്കൂർ',
  		      hh: '%d മണിക്കൂർ',
  		      d: 'ഒരു ദിവസം',
  		      dd: '%d ദിവസം',
  		      M: 'ഒരു മാസം',
  		      MM: '%d മാസം',
  		      y: 'ഒരു വർഷം',
  		      yy: '%d വർഷം'
  		    },
  		    meridiemParse: /രാത്രി|രാവിലെ|ഉച്ച കഴിഞ്ഞ്|വൈകുന്നേരം|രാത്രി/i,
  		    meridiemHour: function (hour, meridiem) {
  		      if (hour === 12) {
  		        hour = 0;
  		      }
  		      if (meridiem === 'രാത്രി' && hour >= 4 || meridiem === 'ഉച്ച കഴിഞ്ഞ്' || meridiem === 'വൈകുന്നേരം') {
  		        return hour + 12;
  		      } else {
  		        return hour;
  		      }
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 4) {
  		        return 'രാത്രി';
  		      } else if (hour < 12) {
  		        return 'രാവിലെ';
  		      } else if (hour < 17) {
  		        return 'ഉച്ച കഴിഞ്ഞ്';
  		      } else if (hour < 20) {
  		        return 'വൈകുന്നേരം';
  		      } else {
  		        return 'രാത്രി';
  		      }
  		    }
  		  });

  		  //! moment.js locale configuration

  		  function translate$7(number, withoutSuffix, key, isFuture) {
  		    switch (key) {
  		      case 's':
  		        return withoutSuffix ? 'хэдхэн секунд' : 'хэдхэн секундын';
  		      case 'ss':
  		        return number + (withoutSuffix ? ' секунд' : ' секундын');
  		      case 'm':
  		      case 'mm':
  		        return number + (withoutSuffix ? ' минут' : ' минутын');
  		      case 'h':
  		      case 'hh':
  		        return number + (withoutSuffix ? ' цаг' : ' цагийн');
  		      case 'd':
  		      case 'dd':
  		        return number + (withoutSuffix ? ' өдөр' : ' өдрийн');
  		      case 'M':
  		      case 'MM':
  		        return number + (withoutSuffix ? ' сар' : ' сарын');
  		      case 'y':
  		      case 'yy':
  		        return number + (withoutSuffix ? ' жил' : ' жилийн');
  		      default:
  		        return number;
  		    }
  		  }
  		  hooks.defineLocale('mn', {
  		    months: 'Нэгдүгээр сар_Хоёрдугаар сар_Гуравдугаар сар_Дөрөвдүгээр сар_Тавдугаар сар_Зургадугаар сар_Долдугаар сар_Наймдугаар сар_Есдүгээр сар_Аравдугаар сар_Арван нэгдүгээр сар_Арван хоёрдугаар сар'.split('_'),
  		    monthsShort: '1 сар_2 сар_3 сар_4 сар_5 сар_6 сар_7 сар_8 сар_9 сар_10 сар_11 сар_12 сар'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'Ням_Даваа_Мягмар_Лхагва_Пүрэв_Баасан_Бямба'.split('_'),
  		    weekdaysShort: 'Ням_Дав_Мяг_Лха_Пүр_Баа_Бям'.split('_'),
  		    weekdaysMin: 'Ня_Да_Мя_Лх_Пү_Ба_Бя'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'YYYY-MM-DD',
  		      LL: 'YYYY оны MMMMын D',
  		      LLL: 'YYYY оны MMMMын D HH:mm',
  		      LLLL: 'dddd, YYYY оны MMMMын D HH:mm'
  		    },
  		    meridiemParse: /ҮӨ|ҮХ/i,
  		    isPM: function (input) {
  		      return input === 'ҮХ';
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 12) {
  		        return 'ҮӨ';
  		      } else {
  		        return 'ҮХ';
  		      }
  		    },
  		    calendar: {
  		      sameDay: '[Өнөөдөр] LT',
  		      nextDay: '[Маргааш] LT',
  		      nextWeek: '[Ирэх] dddd LT',
  		      lastDay: '[Өчигдөр] LT',
  		      lastWeek: '[Өнгөрсөн] dddd LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%s дараа',
  		      past: '%s өмнө',
  		      s: translate$7,
  		      ss: translate$7,
  		      m: translate$7,
  		      mm: translate$7,
  		      h: translate$7,
  		      hh: translate$7,
  		      d: translate$7,
  		      dd: translate$7,
  		      M: translate$7,
  		      MM: translate$7,
  		      y: translate$7,
  		      yy: translate$7
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2} өдөр/,
  		    ordinal: function (number, period) {
  		      switch (period) {
  		        case 'd':
  		        case 'D':
  		        case 'DDD':
  		          return number + ' өдөр';
  		        default:
  		          return number;
  		      }
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var symbolMap$d = {
  		      1: '१',
  		      2: '२',
  		      3: '३',
  		      4: '४',
  		      5: '५',
  		      6: '६',
  		      7: '७',
  		      8: '८',
  		      9: '९',
  		      0: '०'
  		    },
  		    numberMap$c = {
  		      '१': '1',
  		      '२': '2',
  		      '३': '3',
  		      '४': '4',
  		      '५': '5',
  		      '६': '6',
  		      '७': '7',
  		      '८': '8',
  		      '९': '9',
  		      '०': '0'
  		    };
  		  function relativeTimeMr(number, withoutSuffix, string, isFuture) {
  		    var output = '';
  		    if (withoutSuffix) {
  		      switch (string) {
  		        case 's':
  		          output = 'काही सेकंद';
  		          break;
  		        case 'ss':
  		          output = '%d सेकंद';
  		          break;
  		        case 'm':
  		          output = 'एक मिनिट';
  		          break;
  		        case 'mm':
  		          output = '%d मिनिटे';
  		          break;
  		        case 'h':
  		          output = 'एक तास';
  		          break;
  		        case 'hh':
  		          output = '%d तास';
  		          break;
  		        case 'd':
  		          output = 'एक दिवस';
  		          break;
  		        case 'dd':
  		          output = '%d दिवस';
  		          break;
  		        case 'M':
  		          output = 'एक महिना';
  		          break;
  		        case 'MM':
  		          output = '%d महिने';
  		          break;
  		        case 'y':
  		          output = 'एक वर्ष';
  		          break;
  		        case 'yy':
  		          output = '%d वर्षे';
  		          break;
  		      }
  		    } else {
  		      switch (string) {
  		        case 's':
  		          output = 'काही सेकंदां';
  		          break;
  		        case 'ss':
  		          output = '%d सेकंदां';
  		          break;
  		        case 'm':
  		          output = 'एका मिनिटा';
  		          break;
  		        case 'mm':
  		          output = '%d मिनिटां';
  		          break;
  		        case 'h':
  		          output = 'एका तासा';
  		          break;
  		        case 'hh':
  		          output = '%d तासां';
  		          break;
  		        case 'd':
  		          output = 'एका दिवसा';
  		          break;
  		        case 'dd':
  		          output = '%d दिवसां';
  		          break;
  		        case 'M':
  		          output = 'एका महिन्या';
  		          break;
  		        case 'MM':
  		          output = '%d महिन्यां';
  		          break;
  		        case 'y':
  		          output = 'एका वर्षा';
  		          break;
  		        case 'yy':
  		          output = '%d वर्षां';
  		          break;
  		      }
  		    }
  		    return output.replace(/%d/i, number);
  		  }
  		  hooks.defineLocale('mr', {
  		    months: 'जानेवारी_फेब्रुवारी_मार्च_एप्रिल_मे_जून_जुलै_ऑगस्ट_सप्टेंबर_ऑक्टोबर_नोव्हेंबर_डिसेंबर'.split('_'),
  		    monthsShort: 'जाने._फेब्रु._मार्च._एप्रि._मे._जून._जुलै._ऑग._सप्टें._ऑक्टो._नोव्हें._डिसें.'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'रविवार_सोमवार_मंगळवार_बुधवार_गुरूवार_शुक्रवार_शनिवार'.split('_'),
  		    weekdaysShort: 'रवि_सोम_मंगळ_बुध_गुरू_शुक्र_शनि'.split('_'),
  		    weekdaysMin: 'र_सो_मं_बु_गु_शु_श'.split('_'),
  		    longDateFormat: {
  		      LT: 'A h:mm वाजता',
  		      LTS: 'A h:mm:ss वाजता',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY, A h:mm वाजता',
  		      LLLL: 'dddd, D MMMM YYYY, A h:mm वाजता'
  		    },
  		    calendar: {
  		      sameDay: '[आज] LT',
  		      nextDay: '[उद्या] LT',
  		      nextWeek: 'dddd, LT',
  		      lastDay: '[काल] LT',
  		      lastWeek: '[मागील] dddd, LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%sमध्ये',
  		      past: '%sपूर्वी',
  		      s: relativeTimeMr,
  		      ss: relativeTimeMr,
  		      m: relativeTimeMr,
  		      mm: relativeTimeMr,
  		      h: relativeTimeMr,
  		      hh: relativeTimeMr,
  		      d: relativeTimeMr,
  		      dd: relativeTimeMr,
  		      M: relativeTimeMr,
  		      MM: relativeTimeMr,
  		      y: relativeTimeMr,
  		      yy: relativeTimeMr
  		    },
  		    preparse: function (string) {
  		      return string.replace(/[१२३४५६७८९०]/g, function (match) {
  		        return numberMap$c[match];
  		      });
  		    },
  		    postformat: function (string) {
  		      return string.replace(/\d/g, function (match) {
  		        return symbolMap$d[match];
  		      });
  		    },
  		    meridiemParse: /पहाटे|सकाळी|दुपारी|सायंकाळी|रात्री/,
  		    meridiemHour: function (hour, meridiem) {
  		      if (hour === 12) {
  		        hour = 0;
  		      }
  		      if (meridiem === 'पहाटे' || meridiem === 'सकाळी') {
  		        return hour;
  		      } else if (meridiem === 'दुपारी' || meridiem === 'सायंकाळी' || meridiem === 'रात्री') {
  		        return hour >= 12 ? hour : hour + 12;
  		      }
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour >= 0 && hour < 6) {
  		        return 'पहाटे';
  		      } else if (hour < 12) {
  		        return 'सकाळी';
  		      } else if (hour < 17) {
  		        return 'दुपारी';
  		      } else if (hour < 20) {
  		        return 'सायंकाळी';
  		      } else {
  		        return 'रात्री';
  		      }
  		    },
  		    week: {
  		      dow: 0,
  		      // Sunday is the first day of the week.
  		      doy: 6 // The week that contains Jan 6th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('ms-my', {
  		    months: 'Januari_Februari_Mac_April_Mei_Jun_Julai_Ogos_September_Oktober_November_Disember'.split('_'),
  		    monthsShort: 'Jan_Feb_Mac_Apr_Mei_Jun_Jul_Ogs_Sep_Okt_Nov_Dis'.split('_'),
  		    weekdays: 'Ahad_Isnin_Selasa_Rabu_Khamis_Jumaat_Sabtu'.split('_'),
  		    weekdaysShort: 'Ahd_Isn_Sel_Rab_Kha_Jum_Sab'.split('_'),
  		    weekdaysMin: 'Ah_Is_Sl_Rb_Km_Jm_Sb'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH.mm',
  		      LTS: 'HH.mm.ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY [pukul] HH.mm',
  		      LLLL: 'dddd, D MMMM YYYY [pukul] HH.mm'
  		    },
  		    meridiemParse: /pagi|tengahari|petang|malam/,
  		    meridiemHour: function (hour, meridiem) {
  		      if (hour === 12) {
  		        hour = 0;
  		      }
  		      if (meridiem === 'pagi') {
  		        return hour;
  		      } else if (meridiem === 'tengahari') {
  		        return hour >= 11 ? hour : hour + 12;
  		      } else if (meridiem === 'petang' || meridiem === 'malam') {
  		        return hour + 12;
  		      }
  		    },
  		    meridiem: function (hours, minutes, isLower) {
  		      if (hours < 11) {
  		        return 'pagi';
  		      } else if (hours < 15) {
  		        return 'tengahari';
  		      } else if (hours < 19) {
  		        return 'petang';
  		      } else {
  		        return 'malam';
  		      }
  		    },
  		    calendar: {
  		      sameDay: '[Hari ini pukul] LT',
  		      nextDay: '[Esok pukul] LT',
  		      nextWeek: 'dddd [pukul] LT',
  		      lastDay: '[Kelmarin pukul] LT',
  		      lastWeek: 'dddd [lepas pukul] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'dalam %s',
  		      past: '%s yang lepas',
  		      s: 'beberapa saat',
  		      ss: '%d saat',
  		      m: 'seminit',
  		      mm: '%d minit',
  		      h: 'sejam',
  		      hh: '%d jam',
  		      d: 'sehari',
  		      dd: '%d hari',
  		      M: 'sebulan',
  		      MM: '%d bulan',
  		      y: 'setahun',
  		      yy: '%d tahun'
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 7 // The week that contains Jan 7th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('ms', {
  		    months: 'Januari_Februari_Mac_April_Mei_Jun_Julai_Ogos_September_Oktober_November_Disember'.split('_'),
  		    monthsShort: 'Jan_Feb_Mac_Apr_Mei_Jun_Jul_Ogs_Sep_Okt_Nov_Dis'.split('_'),
  		    weekdays: 'Ahad_Isnin_Selasa_Rabu_Khamis_Jumaat_Sabtu'.split('_'),
  		    weekdaysShort: 'Ahd_Isn_Sel_Rab_Kha_Jum_Sab'.split('_'),
  		    weekdaysMin: 'Ah_Is_Sl_Rb_Km_Jm_Sb'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH.mm',
  		      LTS: 'HH.mm.ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY [pukul] HH.mm',
  		      LLLL: 'dddd, D MMMM YYYY [pukul] HH.mm'
  		    },
  		    meridiemParse: /pagi|tengahari|petang|malam/,
  		    meridiemHour: function (hour, meridiem) {
  		      if (hour === 12) {
  		        hour = 0;
  		      }
  		      if (meridiem === 'pagi') {
  		        return hour;
  		      } else if (meridiem === 'tengahari') {
  		        return hour >= 11 ? hour : hour + 12;
  		      } else if (meridiem === 'petang' || meridiem === 'malam') {
  		        return hour + 12;
  		      }
  		    },
  		    meridiem: function (hours, minutes, isLower) {
  		      if (hours < 11) {
  		        return 'pagi';
  		      } else if (hours < 15) {
  		        return 'tengahari';
  		      } else if (hours < 19) {
  		        return 'petang';
  		      } else {
  		        return 'malam';
  		      }
  		    },
  		    calendar: {
  		      sameDay: '[Hari ini pukul] LT',
  		      nextDay: '[Esok pukul] LT',
  		      nextWeek: 'dddd [pukul] LT',
  		      lastDay: '[Kelmarin pukul] LT',
  		      lastWeek: 'dddd [lepas pukul] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'dalam %s',
  		      past: '%s yang lepas',
  		      s: 'beberapa saat',
  		      ss: '%d saat',
  		      m: 'seminit',
  		      mm: '%d minit',
  		      h: 'sejam',
  		      hh: '%d jam',
  		      d: 'sehari',
  		      dd: '%d hari',
  		      M: 'sebulan',
  		      MM: '%d bulan',
  		      y: 'setahun',
  		      yy: '%d tahun'
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 7 // The week that contains Jan 7th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('mt', {
  		    months: 'Jannar_Frar_Marzu_April_Mejju_Ġunju_Lulju_Awwissu_Settembru_Ottubru_Novembru_Diċembru'.split('_'),
  		    monthsShort: 'Jan_Fra_Mar_Apr_Mej_Ġun_Lul_Aww_Set_Ott_Nov_Diċ'.split('_'),
  		    weekdays: 'Il-Ħadd_It-Tnejn_It-Tlieta_L-Erbgħa_Il-Ħamis_Il-Ġimgħa_Is-Sibt'.split('_'),
  		    weekdaysShort: 'Ħad_Tne_Tli_Erb_Ħam_Ġim_Sib'.split('_'),
  		    weekdaysMin: 'Ħa_Tn_Tl_Er_Ħa_Ġi_Si'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd, D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Illum fil-]LT',
  		      nextDay: '[Għada fil-]LT',
  		      nextWeek: 'dddd [fil-]LT',
  		      lastDay: '[Il-bieraħ fil-]LT',
  		      lastWeek: 'dddd [li għadda] [fil-]LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'f’ %s',
  		      past: '%s ilu',
  		      s: 'ftit sekondi',
  		      ss: '%d sekondi',
  		      m: 'minuta',
  		      mm: '%d minuti',
  		      h: 'siegħa',
  		      hh: '%d siegħat',
  		      d: 'ġurnata',
  		      dd: '%d ġranet',
  		      M: 'xahar',
  		      MM: '%d xhur',
  		      y: 'sena',
  		      yy: '%d sni'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}º/,
  		    ordinal: '%dº',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var symbolMap$e = {
  		      1: '၁',
  		      2: '၂',
  		      3: '၃',
  		      4: '၄',
  		      5: '၅',
  		      6: '၆',
  		      7: '၇',
  		      8: '၈',
  		      9: '၉',
  		      0: '၀'
  		    },
  		    numberMap$d = {
  		      '၁': '1',
  		      '၂': '2',
  		      '၃': '3',
  		      '၄': '4',
  		      '၅': '5',
  		      '၆': '6',
  		      '၇': '7',
  		      '၈': '8',
  		      '၉': '9',
  		      '၀': '0'
  		    };
  		  hooks.defineLocale('my', {
  		    months: 'ဇန်နဝါရီ_ဖေဖော်ဝါရီ_မတ်_ဧပြီ_မေ_ဇွန်_ဇူလိုင်_သြဂုတ်_စက်တင်ဘာ_အောက်တိုဘာ_နိုဝင်ဘာ_ဒီဇင်ဘာ'.split('_'),
  		    monthsShort: 'ဇန်_ဖေ_မတ်_ပြီ_မေ_ဇွန်_လိုင်_သြ_စက်_အောက်_နို_ဒီ'.split('_'),
  		    weekdays: 'တနင်္ဂနွေ_တနင်္လာ_အင်္ဂါ_ဗုဒ္ဓဟူး_ကြာသပတေး_သောကြာ_စနေ'.split('_'),
  		    weekdaysShort: 'နွေ_လာ_ဂါ_ဟူး_ကြာ_သော_နေ'.split('_'),
  		    weekdaysMin: 'နွေ_လာ_ဂါ_ဟူး_ကြာ_သော_နေ'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[ယနေ.] LT [မှာ]',
  		      nextDay: '[မနက်ဖြန်] LT [မှာ]',
  		      nextWeek: 'dddd LT [မှာ]',
  		      lastDay: '[မနေ.က] LT [မှာ]',
  		      lastWeek: '[ပြီးခဲ့သော] dddd LT [မှာ]',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'လာမည့် %s မှာ',
  		      past: 'လွန်ခဲ့သော %s က',
  		      s: 'စက္ကန်.အနည်းငယ်',
  		      ss: '%d စက္ကန့်',
  		      m: 'တစ်မိနစ်',
  		      mm: '%d မိနစ်',
  		      h: 'တစ်နာရီ',
  		      hh: '%d နာရီ',
  		      d: 'တစ်ရက်',
  		      dd: '%d ရက်',
  		      M: 'တစ်လ',
  		      MM: '%d လ',
  		      y: 'တစ်နှစ်',
  		      yy: '%d နှစ်'
  		    },
  		    preparse: function (string) {
  		      return string.replace(/[၁၂၃၄၅၆၇၈၉၀]/g, function (match) {
  		        return numberMap$d[match];
  		      });
  		    },
  		    postformat: function (string) {
  		      return string.replace(/\d/g, function (match) {
  		        return symbolMap$e[match];
  		      });
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('nb', {
  		    months: 'januar_februar_mars_april_mai_juni_juli_august_september_oktober_november_desember'.split('_'),
  		    monthsShort: 'jan._feb._mars_apr._mai_juni_juli_aug._sep._okt._nov._des.'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'søndag_mandag_tirsdag_onsdag_torsdag_fredag_lørdag'.split('_'),
  		    weekdaysShort: 'sø._ma._ti._on._to._fr._lø.'.split('_'),
  		    weekdaysMin: 'sø_ma_ti_on_to_fr_lø'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'D. MMMM YYYY',
  		      LLL: 'D. MMMM YYYY [kl.] HH:mm',
  		      LLLL: 'dddd D. MMMM YYYY [kl.] HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[i dag kl.] LT',
  		      nextDay: '[i morgen kl.] LT',
  		      nextWeek: 'dddd [kl.] LT',
  		      lastDay: '[i går kl.] LT',
  		      lastWeek: '[forrige] dddd [kl.] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'om %s',
  		      past: '%s siden',
  		      s: 'noen sekunder',
  		      ss: '%d sekunder',
  		      m: 'ett minutt',
  		      mm: '%d minutter',
  		      h: 'én time',
  		      hh: '%d timer',
  		      d: 'én dag',
  		      dd: '%d dager',
  		      w: 'én uke',
  		      ww: '%d uker',
  		      M: 'én måned',
  		      MM: '%d måneder',
  		      y: 'ett år',
  		      yy: '%d år'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}\./,
  		    ordinal: '%d.',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var symbolMap$f = {
  		      1: '१',
  		      2: '२',
  		      3: '३',
  		      4: '४',
  		      5: '५',
  		      6: '६',
  		      7: '७',
  		      8: '८',
  		      9: '९',
  		      0: '०'
  		    },
  		    numberMap$e = {
  		      '१': '1',
  		      '२': '2',
  		      '३': '3',
  		      '४': '4',
  		      '५': '5',
  		      '६': '6',
  		      '७': '7',
  		      '८': '8',
  		      '९': '9',
  		      '०': '0'
  		    };
  		  hooks.defineLocale('ne', {
  		    months: 'जनवरी_फेब्रुवरी_मार्च_अप्रिल_मई_जुन_जुलाई_अगष्ट_सेप्टेम्बर_अक्टोबर_नोभेम्बर_डिसेम्बर'.split('_'),
  		    monthsShort: 'जन._फेब्रु._मार्च_अप्रि._मई_जुन_जुलाई._अग._सेप्ट._अक्टो._नोभे._डिसे.'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'आइतबार_सोमबार_मङ्गलबार_बुधबार_बिहिबार_शुक्रबार_शनिबार'.split('_'),
  		    weekdaysShort: 'आइत._सोम._मङ्गल._बुध._बिहि._शुक्र._शनि.'.split('_'),
  		    weekdaysMin: 'आ._सो._मं._बु._बि._शु._श.'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'Aको h:mm बजे',
  		      LTS: 'Aको h:mm:ss बजे',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY, Aको h:mm बजे',
  		      LLLL: 'dddd, D MMMM YYYY, Aको h:mm बजे'
  		    },
  		    preparse: function (string) {
  		      return string.replace(/[१२३४५६७८९०]/g, function (match) {
  		        return numberMap$e[match];
  		      });
  		    },
  		    postformat: function (string) {
  		      return string.replace(/\d/g, function (match) {
  		        return symbolMap$f[match];
  		      });
  		    },
  		    meridiemParse: /राति|बिहान|दिउँसो|साँझ/,
  		    meridiemHour: function (hour, meridiem) {
  		      if (hour === 12) {
  		        hour = 0;
  		      }
  		      if (meridiem === 'राति') {
  		        return hour < 4 ? hour : hour + 12;
  		      } else if (meridiem === 'बिहान') {
  		        return hour;
  		      } else if (meridiem === 'दिउँसो') {
  		        return hour >= 10 ? hour : hour + 12;
  		      } else if (meridiem === 'साँझ') {
  		        return hour + 12;
  		      }
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 3) {
  		        return 'राति';
  		      } else if (hour < 12) {
  		        return 'बिहान';
  		      } else if (hour < 16) {
  		        return 'दिउँसो';
  		      } else if (hour < 20) {
  		        return 'साँझ';
  		      } else {
  		        return 'राति';
  		      }
  		    },
  		    calendar: {
  		      sameDay: '[आज] LT',
  		      nextDay: '[भोलि] LT',
  		      nextWeek: '[आउँदो] dddd[,] LT',
  		      lastDay: '[हिजो] LT',
  		      lastWeek: '[गएको] dddd[,] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%sमा',
  		      past: '%s अगाडि',
  		      s: 'केही क्षण',
  		      ss: '%d सेकेण्ड',
  		      m: 'एक मिनेट',
  		      mm: '%d मिनेट',
  		      h: 'एक घण्टा',
  		      hh: '%d घण्टा',
  		      d: 'एक दिन',
  		      dd: '%d दिन',
  		      M: 'एक महिना',
  		      MM: '%d महिना',
  		      y: 'एक बर्ष',
  		      yy: '%d बर्ष'
  		    },
  		    week: {
  		      dow: 0,
  		      // Sunday is the first day of the week.
  		      doy: 6 // The week that contains Jan 6th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var monthsShortWithDots$1 = 'jan._feb._mrt._apr._mei_jun._jul._aug._sep._okt._nov._dec.'.split('_'),
  		    monthsShortWithoutDots$1 = 'jan_feb_mrt_apr_mei_jun_jul_aug_sep_okt_nov_dec'.split('_'),
  		    monthsParse$8 = [/^jan/i, /^feb/i, /^(maart|mrt\.?)$/i, /^apr/i, /^mei$/i, /^jun[i.]?$/i, /^jul[i.]?$/i, /^aug/i, /^sep/i, /^okt/i, /^nov/i, /^dec/i],
  		    monthsRegex$8 = /^(januari|februari|maart|april|mei|ju[nl]i|augustus|september|oktober|november|december|jan\.?|feb\.?|mrt\.?|apr\.?|ju[nl]\.?|aug\.?|sep\.?|okt\.?|nov\.?|dec\.?)/i;
  		  hooks.defineLocale('nl-be', {
  		    months: 'januari_februari_maart_april_mei_juni_juli_augustus_september_oktober_november_december'.split('_'),
  		    monthsShort: function (m, format) {
  		      if (!m) {
  		        return monthsShortWithDots$1;
  		      } else if (/-MMM-/.test(format)) {
  		        return monthsShortWithoutDots$1[m.month()];
  		      } else {
  		        return monthsShortWithDots$1[m.month()];
  		      }
  		    },
  		    monthsRegex: monthsRegex$8,
  		    monthsShortRegex: monthsRegex$8,
  		    monthsStrictRegex: /^(januari|februari|maart|april|mei|ju[nl]i|augustus|september|oktober|november|december)/i,
  		    monthsShortStrictRegex: /^(jan\.?|feb\.?|mrt\.?|apr\.?|mei|ju[nl]\.?|aug\.?|sep\.?|okt\.?|nov\.?|dec\.?)/i,
  		    monthsParse: monthsParse$8,
  		    longMonthsParse: monthsParse$8,
  		    shortMonthsParse: monthsParse$8,
  		    weekdays: 'zondag_maandag_dinsdag_woensdag_donderdag_vrijdag_zaterdag'.split('_'),
  		    weekdaysShort: 'zo._ma._di._wo._do._vr._za.'.split('_'),
  		    weekdaysMin: 'zo_ma_di_wo_do_vr_za'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[vandaag om] LT',
  		      nextDay: '[morgen om] LT',
  		      nextWeek: 'dddd [om] LT',
  		      lastDay: '[gisteren om] LT',
  		      lastWeek: '[afgelopen] dddd [om] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'over %s',
  		      past: '%s geleden',
  		      s: 'een paar seconden',
  		      ss: '%d seconden',
  		      m: 'één minuut',
  		      mm: '%d minuten',
  		      h: 'één uur',
  		      hh: '%d uur',
  		      d: 'één dag',
  		      dd: '%d dagen',
  		      M: 'één maand',
  		      MM: '%d maanden',
  		      y: 'één jaar',
  		      yy: '%d jaar'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(ste|de)/,
  		    ordinal: function (number) {
  		      return number + (number === 1 || number === 8 || number >= 20 ? 'ste' : 'de');
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var monthsShortWithDots$2 = 'jan._feb._mrt._apr._mei_jun._jul._aug._sep._okt._nov._dec.'.split('_'),
  		    monthsShortWithoutDots$2 = 'jan_feb_mrt_apr_mei_jun_jul_aug_sep_okt_nov_dec'.split('_'),
  		    monthsParse$9 = [/^jan/i, /^feb/i, /^(maart|mrt\.?)$/i, /^apr/i, /^mei$/i, /^jun[i.]?$/i, /^jul[i.]?$/i, /^aug/i, /^sep/i, /^okt/i, /^nov/i, /^dec/i],
  		    monthsRegex$9 = /^(januari|februari|maart|april|mei|ju[nl]i|augustus|september|oktober|november|december|jan\.?|feb\.?|mrt\.?|apr\.?|ju[nl]\.?|aug\.?|sep\.?|okt\.?|nov\.?|dec\.?)/i;
  		  hooks.defineLocale('nl', {
  		    months: 'januari_februari_maart_april_mei_juni_juli_augustus_september_oktober_november_december'.split('_'),
  		    monthsShort: function (m, format) {
  		      if (!m) {
  		        return monthsShortWithDots$2;
  		      } else if (/-MMM-/.test(format)) {
  		        return monthsShortWithoutDots$2[m.month()];
  		      } else {
  		        return monthsShortWithDots$2[m.month()];
  		      }
  		    },
  		    monthsRegex: monthsRegex$9,
  		    monthsShortRegex: monthsRegex$9,
  		    monthsStrictRegex: /^(januari|februari|maart|april|mei|ju[nl]i|augustus|september|oktober|november|december)/i,
  		    monthsShortStrictRegex: /^(jan\.?|feb\.?|mrt\.?|apr\.?|mei|ju[nl]\.?|aug\.?|sep\.?|okt\.?|nov\.?|dec\.?)/i,
  		    monthsParse: monthsParse$9,
  		    longMonthsParse: monthsParse$9,
  		    shortMonthsParse: monthsParse$9,
  		    weekdays: 'zondag_maandag_dinsdag_woensdag_donderdag_vrijdag_zaterdag'.split('_'),
  		    weekdaysShort: 'zo._ma._di._wo._do._vr._za.'.split('_'),
  		    weekdaysMin: 'zo_ma_di_wo_do_vr_za'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD-MM-YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[vandaag om] LT',
  		      nextDay: '[morgen om] LT',
  		      nextWeek: 'dddd [om] LT',
  		      lastDay: '[gisteren om] LT',
  		      lastWeek: '[afgelopen] dddd [om] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'over %s',
  		      past: '%s geleden',
  		      s: 'een paar seconden',
  		      ss: '%d seconden',
  		      m: 'één minuut',
  		      mm: '%d minuten',
  		      h: 'één uur',
  		      hh: '%d uur',
  		      d: 'één dag',
  		      dd: '%d dagen',
  		      w: 'één week',
  		      ww: '%d weken',
  		      M: 'één maand',
  		      MM: '%d maanden',
  		      y: 'één jaar',
  		      yy: '%d jaar'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(ste|de)/,
  		    ordinal: function (number) {
  		      return number + (number === 1 || number === 8 || number >= 20 ? 'ste' : 'de');
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('nn', {
  		    months: 'januar_februar_mars_april_mai_juni_juli_august_september_oktober_november_desember'.split('_'),
  		    monthsShort: 'jan._feb._mars_apr._mai_juni_juli_aug._sep._okt._nov._des.'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'sundag_måndag_tysdag_onsdag_torsdag_fredag_laurdag'.split('_'),
  		    weekdaysShort: 'su._må._ty._on._to._fr._lau.'.split('_'),
  		    weekdaysMin: 'su_må_ty_on_to_fr_la'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'D. MMMM YYYY',
  		      LLL: 'D. MMMM YYYY [kl.] H:mm',
  		      LLLL: 'dddd D. MMMM YYYY [kl.] HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[I dag klokka] LT',
  		      nextDay: '[I morgon klokka] LT',
  		      nextWeek: 'dddd [klokka] LT',
  		      lastDay: '[I går klokka] LT',
  		      lastWeek: '[Føregåande] dddd [klokka] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'om %s',
  		      past: '%s sidan',
  		      s: 'nokre sekund',
  		      ss: '%d sekund',
  		      m: 'eit minutt',
  		      mm: '%d minutt',
  		      h: 'ein time',
  		      hh: '%d timar',
  		      d: 'ein dag',
  		      dd: '%d dagar',
  		      w: 'ei veke',
  		      ww: '%d veker',
  		      M: 'ein månad',
  		      MM: '%d månader',
  		      y: 'eit år',
  		      yy: '%d år'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}\./,
  		    ordinal: '%d.',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('oc-lnc', {
  		    months: {
  		      standalone: 'genièr_febrièr_març_abril_mai_junh_julhet_agost_setembre_octòbre_novembre_decembre'.split('_'),
  		      format: "de genièr_de febrièr_de març_d'abril_de mai_de junh_de julhet_d'agost_de setembre_d'octòbre_de novembre_de decembre".split('_'),
  		      isFormat: /D[oD]?(\s)+MMMM/
  		    },
  		    monthsShort: 'gen._febr._març_abr._mai_junh_julh._ago._set._oct._nov._dec.'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'dimenge_diluns_dimars_dimècres_dijòus_divendres_dissabte'.split('_'),
  		    weekdaysShort: 'dg._dl._dm._dc._dj._dv._ds.'.split('_'),
  		    weekdaysMin: 'dg_dl_dm_dc_dj_dv_ds'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'H:mm',
  		      LTS: 'H:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM [de] YYYY',
  		      ll: 'D MMM YYYY',
  		      LLL: 'D MMMM [de] YYYY [a] H:mm',
  		      lll: 'D MMM YYYY, H:mm',
  		      LLLL: 'dddd D MMMM [de] YYYY [a] H:mm',
  		      llll: 'ddd D MMM YYYY, H:mm'
  		    },
  		    calendar: {
  		      sameDay: '[uèi a] LT',
  		      nextDay: '[deman a] LT',
  		      nextWeek: 'dddd [a] LT',
  		      lastDay: '[ièr a] LT',
  		      lastWeek: 'dddd [passat a] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: "d'aquí %s",
  		      past: 'fa %s',
  		      s: 'unas segondas',
  		      ss: '%d segondas',
  		      m: 'una minuta',
  		      mm: '%d minutas',
  		      h: 'una ora',
  		      hh: '%d oras',
  		      d: 'un jorn',
  		      dd: '%d jorns',
  		      M: 'un mes',
  		      MM: '%d meses',
  		      y: 'un an',
  		      yy: '%d ans'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(r|n|t|è|a)/,
  		    ordinal: function (number, period) {
  		      var output = number === 1 ? 'r' : number === 2 ? 'n' : number === 3 ? 'r' : number === 4 ? 't' : 'è';
  		      if (period === 'w' || period === 'W') {
  		        output = 'a';
  		      }
  		      return number + output;
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var symbolMap$g = {
  		      1: '੧',
  		      2: '੨',
  		      3: '੩',
  		      4: '੪',
  		      5: '੫',
  		      6: '੬',
  		      7: '੭',
  		      8: '੮',
  		      9: '੯',
  		      0: '੦'
  		    },
  		    numberMap$f = {
  		      '੧': '1',
  		      '੨': '2',
  		      '੩': '3',
  		      '੪': '4',
  		      '੫': '5',
  		      '੬': '6',
  		      '੭': '7',
  		      '੮': '8',
  		      '੯': '9',
  		      '੦': '0'
  		    };
  		  hooks.defineLocale('pa-in', {
  		    // There are months name as per Nanakshahi Calendar but they are not used as rigidly in modern Punjabi.
  		    months: 'ਜਨਵਰੀ_ਫ਼ਰਵਰੀ_ਮਾਰਚ_ਅਪ੍ਰੈਲ_ਮਈ_ਜੂਨ_ਜੁਲਾਈ_ਅਗਸਤ_ਸਤੰਬਰ_ਅਕਤੂਬਰ_ਨਵੰਬਰ_ਦਸੰਬਰ'.split('_'),
  		    monthsShort: 'ਜਨਵਰੀ_ਫ਼ਰਵਰੀ_ਮਾਰਚ_ਅਪ੍ਰੈਲ_ਮਈ_ਜੂਨ_ਜੁਲਾਈ_ਅਗਸਤ_ਸਤੰਬਰ_ਅਕਤੂਬਰ_ਨਵੰਬਰ_ਦਸੰਬਰ'.split('_'),
  		    weekdays: 'ਐਤਵਾਰ_ਸੋਮਵਾਰ_ਮੰਗਲਵਾਰ_ਬੁਧਵਾਰ_ਵੀਰਵਾਰ_ਸ਼ੁੱਕਰਵਾਰ_ਸ਼ਨੀਚਰਵਾਰ'.split('_'),
  		    weekdaysShort: 'ਐਤ_ਸੋਮ_ਮੰਗਲ_ਬੁਧ_ਵੀਰ_ਸ਼ੁਕਰ_ਸ਼ਨੀ'.split('_'),
  		    weekdaysMin: 'ਐਤ_ਸੋਮ_ਮੰਗਲ_ਬੁਧ_ਵੀਰ_ਸ਼ੁਕਰ_ਸ਼ਨੀ'.split('_'),
  		    longDateFormat: {
  		      LT: 'A h:mm ਵਜੇ',
  		      LTS: 'A h:mm:ss ਵਜੇ',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY, A h:mm ਵਜੇ',
  		      LLLL: 'dddd, D MMMM YYYY, A h:mm ਵਜੇ'
  		    },
  		    calendar: {
  		      sameDay: '[ਅਜ] LT',
  		      nextDay: '[ਕਲ] LT',
  		      nextWeek: '[ਅਗਲਾ] dddd, LT',
  		      lastDay: '[ਕਲ] LT',
  		      lastWeek: '[ਪਿਛਲੇ] dddd, LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%s ਵਿੱਚ',
  		      past: '%s ਪਿਛਲੇ',
  		      s: 'ਕੁਝ ਸਕਿੰਟ',
  		      ss: '%d ਸਕਿੰਟ',
  		      m: 'ਇਕ ਮਿੰਟ',
  		      mm: '%d ਮਿੰਟ',
  		      h: 'ਇੱਕ ਘੰਟਾ',
  		      hh: '%d ਘੰਟੇ',
  		      d: 'ਇੱਕ ਦਿਨ',
  		      dd: '%d ਦਿਨ',
  		      M: 'ਇੱਕ ਮਹੀਨਾ',
  		      MM: '%d ਮਹੀਨੇ',
  		      y: 'ਇੱਕ ਸਾਲ',
  		      yy: '%d ਸਾਲ'
  		    },
  		    preparse: function (string) {
  		      return string.replace(/[੧੨੩੪੫੬੭੮੯੦]/g, function (match) {
  		        return numberMap$f[match];
  		      });
  		    },
  		    postformat: function (string) {
  		      return string.replace(/\d/g, function (match) {
  		        return symbolMap$g[match];
  		      });
  		    },
  		    // Punjabi notation for meridiems are quite fuzzy in practice. While there exists
  		    // a rigid notion of a 'Pahar' it is not used as rigidly in modern Punjabi.
  		    meridiemParse: /ਰਾਤ|ਸਵੇਰ|ਦੁਪਹਿਰ|ਸ਼ਾਮ/,
  		    meridiemHour: function (hour, meridiem) {
  		      if (hour === 12) {
  		        hour = 0;
  		      }
  		      if (meridiem === 'ਰਾਤ') {
  		        return hour < 4 ? hour : hour + 12;
  		      } else if (meridiem === 'ਸਵੇਰ') {
  		        return hour;
  		      } else if (meridiem === 'ਦੁਪਹਿਰ') {
  		        return hour >= 10 ? hour : hour + 12;
  		      } else if (meridiem === 'ਸ਼ਾਮ') {
  		        return hour + 12;
  		      }
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 4) {
  		        return 'ਰਾਤ';
  		      } else if (hour < 10) {
  		        return 'ਸਵੇਰ';
  		      } else if (hour < 17) {
  		        return 'ਦੁਪਹਿਰ';
  		      } else if (hour < 20) {
  		        return 'ਸ਼ਾਮ';
  		      } else {
  		        return 'ਰਾਤ';
  		      }
  		    },
  		    week: {
  		      dow: 0,
  		      // Sunday is the first day of the week.
  		      doy: 6 // The week that contains Jan 6th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var monthsNominative = 'styczeń_luty_marzec_kwiecień_maj_czerwiec_lipiec_sierpień_wrzesień_październik_listopad_grudzień'.split('_'),
  		    monthsSubjective = 'stycznia_lutego_marca_kwietnia_maja_czerwca_lipca_sierpnia_września_października_listopada_grudnia'.split('_'),
  		    monthsParse$a = [/^sty/i, /^lut/i, /^mar/i, /^kwi/i, /^maj/i, /^cze/i, /^lip/i, /^sie/i, /^wrz/i, /^paź/i, /^lis/i, /^gru/i];
  		  function plural$3(n) {
  		    return n % 10 < 5 && n % 10 > 1 && ~~(n / 10) % 10 !== 1;
  		  }
  		  function translate$8(number, withoutSuffix, key) {
  		    var result = number + ' ';
  		    switch (key) {
  		      case 'ss':
  		        return result + (plural$3(number) ? 'sekundy' : 'sekund');
  		      case 'm':
  		        return withoutSuffix ? 'minuta' : 'minutę';
  		      case 'mm':
  		        return result + (plural$3(number) ? 'minuty' : 'minut');
  		      case 'h':
  		        return withoutSuffix ? 'godzina' : 'godzinę';
  		      case 'hh':
  		        return result + (plural$3(number) ? 'godziny' : 'godzin');
  		      case 'ww':
  		        return result + (plural$3(number) ? 'tygodnie' : 'tygodni');
  		      case 'MM':
  		        return result + (plural$3(number) ? 'miesiące' : 'miesięcy');
  		      case 'yy':
  		        return result + (plural$3(number) ? 'lata' : 'lat');
  		    }
  		  }
  		  hooks.defineLocale('pl', {
  		    months: function (momentToFormat, format) {
  		      if (!momentToFormat) {
  		        return monthsNominative;
  		      } else if (/D MMMM/.test(format)) {
  		        return monthsSubjective[momentToFormat.month()];
  		      } else {
  		        return monthsNominative[momentToFormat.month()];
  		      }
  		    },
  		    monthsShort: 'sty_lut_mar_kwi_maj_cze_lip_sie_wrz_paź_lis_gru'.split('_'),
  		    monthsParse: monthsParse$a,
  		    longMonthsParse: monthsParse$a,
  		    shortMonthsParse: monthsParse$a,
  		    weekdays: 'niedziela_poniedziałek_wtorek_środa_czwartek_piątek_sobota'.split('_'),
  		    weekdaysShort: 'ndz_pon_wt_śr_czw_pt_sob'.split('_'),
  		    weekdaysMin: 'Nd_Pn_Wt_Śr_Cz_Pt_So'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd, D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Dziś o] LT',
  		      nextDay: '[Jutro o] LT',
  		      nextWeek: function () {
  		        switch (this.day()) {
  		          case 0:
  		            return '[W niedzielę o] LT';
  		          case 2:
  		            return '[We wtorek o] LT';
  		          case 3:
  		            return '[W środę o] LT';
  		          case 6:
  		            return '[W sobotę o] LT';
  		          default:
  		            return '[W] dddd [o] LT';
  		        }
  		      },
  		      lastDay: '[Wczoraj o] LT',
  		      lastWeek: function () {
  		        switch (this.day()) {
  		          case 0:
  		            return '[W zeszłą niedzielę o] LT';
  		          case 3:
  		            return '[W zeszłą środę o] LT';
  		          case 6:
  		            return '[W zeszłą sobotę o] LT';
  		          default:
  		            return '[W zeszły] dddd [o] LT';
  		        }
  		      },
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'za %s',
  		      past: '%s temu',
  		      s: 'kilka sekund',
  		      ss: translate$8,
  		      m: translate$8,
  		      mm: translate$8,
  		      h: translate$8,
  		      hh: translate$8,
  		      d: '1 dzień',
  		      dd: '%d dni',
  		      w: 'tydzień',
  		      ww: translate$8,
  		      M: 'miesiąc',
  		      MM: translate$8,
  		      y: 'rok',
  		      yy: translate$8
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}\./,
  		    ordinal: '%d.',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('pt-br', {
  		    months: 'janeiro_fevereiro_março_abril_maio_junho_julho_agosto_setembro_outubro_novembro_dezembro'.split('_'),
  		    monthsShort: 'jan_fev_mar_abr_mai_jun_jul_ago_set_out_nov_dez'.split('_'),
  		    weekdays: 'domingo_segunda-feira_terça-feira_quarta-feira_quinta-feira_sexta-feira_sábado'.split('_'),
  		    weekdaysShort: 'dom_seg_ter_qua_qui_sex_sáb'.split('_'),
  		    weekdaysMin: 'do_2ª_3ª_4ª_5ª_6ª_sá'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D [de] MMMM [de] YYYY',
  		      LLL: 'D [de] MMMM [de] YYYY [às] HH:mm',
  		      LLLL: 'dddd, D [de] MMMM [de] YYYY [às] HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Hoje às] LT',
  		      nextDay: '[Amanhã às] LT',
  		      nextWeek: 'dddd [às] LT',
  		      lastDay: '[Ontem às] LT',
  		      lastWeek: function () {
  		        return this.day() === 0 || this.day() === 6 ? '[Último] dddd [às] LT' // Saturday + Sunday
  		        : '[Última] dddd [às] LT'; // Monday - Friday
  		      },
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'em %s',
  		      past: 'há %s',
  		      s: 'poucos segundos',
  		      ss: '%d segundos',
  		      m: 'um minuto',
  		      mm: '%d minutos',
  		      h: 'uma hora',
  		      hh: '%d horas',
  		      d: 'um dia',
  		      dd: '%d dias',
  		      M: 'um mês',
  		      MM: '%d meses',
  		      y: 'um ano',
  		      yy: '%d anos'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}º/,
  		    ordinal: '%dº',
  		    invalidDate: 'Data inválida'
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('pt', {
  		    months: 'janeiro_fevereiro_março_abril_maio_junho_julho_agosto_setembro_outubro_novembro_dezembro'.split('_'),
  		    monthsShort: 'jan_fev_mar_abr_mai_jun_jul_ago_set_out_nov_dez'.split('_'),
  		    weekdays: 'Domingo_Segunda-feira_Terça-feira_Quarta-feira_Quinta-feira_Sexta-feira_Sábado'.split('_'),
  		    weekdaysShort: 'Dom_Seg_Ter_Qua_Qui_Sex_Sáb'.split('_'),
  		    weekdaysMin: 'Do_2ª_3ª_4ª_5ª_6ª_Sá'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D [de] MMMM [de] YYYY',
  		      LLL: 'D [de] MMMM [de] YYYY HH:mm',
  		      LLLL: 'dddd, D [de] MMMM [de] YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Hoje às] LT',
  		      nextDay: '[Amanhã às] LT',
  		      nextWeek: 'dddd [às] LT',
  		      lastDay: '[Ontem às] LT',
  		      lastWeek: function () {
  		        return this.day() === 0 || this.day() === 6 ? '[Último] dddd [às] LT' // Saturday + Sunday
  		        : '[Última] dddd [às] LT'; // Monday - Friday
  		      },
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'em %s',
  		      past: 'há %s',
  		      s: 'segundos',
  		      ss: '%d segundos',
  		      m: 'um minuto',
  		      mm: '%d minutos',
  		      h: 'uma hora',
  		      hh: '%d horas',
  		      d: 'um dia',
  		      dd: '%d dias',
  		      w: 'uma semana',
  		      ww: '%d semanas',
  		      M: 'um mês',
  		      MM: '%d meses',
  		      y: 'um ano',
  		      yy: '%d anos'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}º/,
  		    ordinal: '%dº',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  function relativeTimeWithPlural$2(number, withoutSuffix, key) {
  		    var format = {
  		        ss: 'secunde',
  		        mm: 'minute',
  		        hh: 'ore',
  		        dd: 'zile',
  		        ww: 'săptămâni',
  		        MM: 'luni',
  		        yy: 'ani'
  		      },
  		      separator = ' ';
  		    if (number % 100 >= 20 || number >= 100 && number % 100 === 0) {
  		      separator = ' de ';
  		    }
  		    return number + separator + format[key];
  		  }
  		  hooks.defineLocale('ro', {
  		    months: 'ianuarie_februarie_martie_aprilie_mai_iunie_iulie_august_septembrie_octombrie_noiembrie_decembrie'.split('_'),
  		    monthsShort: 'ian._feb._mart._apr._mai_iun._iul._aug._sept._oct._nov._dec.'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'duminică_luni_marți_miercuri_joi_vineri_sâmbătă'.split('_'),
  		    weekdaysShort: 'Dum_Lun_Mar_Mie_Joi_Vin_Sâm'.split('_'),
  		    weekdaysMin: 'Du_Lu_Ma_Mi_Jo_Vi_Sâ'.split('_'),
  		    longDateFormat: {
  		      LT: 'H:mm',
  		      LTS: 'H:mm:ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY H:mm',
  		      LLLL: 'dddd, D MMMM YYYY H:mm'
  		    },
  		    calendar: {
  		      sameDay: '[azi la] LT',
  		      nextDay: '[mâine la] LT',
  		      nextWeek: 'dddd [la] LT',
  		      lastDay: '[ieri la] LT',
  		      lastWeek: '[fosta] dddd [la] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'peste %s',
  		      past: '%s în urmă',
  		      s: 'câteva secunde',
  		      ss: relativeTimeWithPlural$2,
  		      m: 'un minut',
  		      mm: relativeTimeWithPlural$2,
  		      h: 'o oră',
  		      hh: relativeTimeWithPlural$2,
  		      d: 'o zi',
  		      dd: relativeTimeWithPlural$2,
  		      w: 'o săptămână',
  		      ww: relativeTimeWithPlural$2,
  		      M: 'o lună',
  		      MM: relativeTimeWithPlural$2,
  		      y: 'un an',
  		      yy: relativeTimeWithPlural$2
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 7 // The week that contains Jan 7th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  function plural$4(word, num) {
  		    var forms = word.split('_');
  		    return num % 10 === 1 && num % 100 !== 11 ? forms[0] : num % 10 >= 2 && num % 10 <= 4 && (num % 100 < 10 || num % 100 >= 20) ? forms[1] : forms[2];
  		  }
  		  function relativeTimeWithPlural$3(number, withoutSuffix, key) {
  		    var format = {
  		      ss: withoutSuffix ? 'секунда_секунды_секунд' : 'секунду_секунды_секунд',
  		      mm: withoutSuffix ? 'минута_минуты_минут' : 'минуту_минуты_минут',
  		      hh: 'час_часа_часов',
  		      dd: 'день_дня_дней',
  		      ww: 'неделя_недели_недель',
  		      MM: 'месяц_месяца_месяцев',
  		      yy: 'год_года_лет'
  		    };
  		    if (key === 'm') {
  		      return withoutSuffix ? 'минута' : 'минуту';
  		    } else {
  		      return number + ' ' + plural$4(format[key], +number);
  		    }
  		  }
  		  var monthsParse$b = [/^янв/i, /^фев/i, /^мар/i, /^апр/i, /^ма[йя]/i, /^июн/i, /^июл/i, /^авг/i, /^сен/i, /^окт/i, /^ноя/i, /^дек/i];

  		  // http://new.gramota.ru/spravka/rules/139-prop : § 103
  		  // Сокращения месяцев: http://new.gramota.ru/spravka/buro/search-answer?s=242637
  		  // CLDR data:          http://www.unicode.org/cldr/charts/28/summary/ru.html#1753
  		  hooks.defineLocale('ru', {
  		    months: {
  		      format: 'января_февраля_марта_апреля_мая_июня_июля_августа_сентября_октября_ноября_декабря'.split('_'),
  		      standalone: 'январь_февраль_март_апрель_май_июнь_июль_август_сентябрь_октябрь_ноябрь_декабрь'.split('_')
  		    },
  		    monthsShort: {
  		      // по CLDR именно "июл." и "июн.", но какой смысл менять букву на точку?
  		      format: 'янв._февр._мар._апр._мая_июня_июля_авг._сент._окт._нояб._дек.'.split('_'),
  		      standalone: 'янв._февр._март_апр._май_июнь_июль_авг._сент._окт._нояб._дек.'.split('_')
  		    },
  		    weekdays: {
  		      standalone: 'воскресенье_понедельник_вторник_среда_четверг_пятница_суббота'.split('_'),
  		      format: 'воскресенье_понедельник_вторник_среду_четверг_пятницу_субботу'.split('_'),
  		      isFormat: /\[ ?[Вв] ?(?:прошлую|следующую|эту)? ?] ?dddd/
  		    },
  		    weekdaysShort: 'вс_пн_вт_ср_чт_пт_сб'.split('_'),
  		    weekdaysMin: 'вс_пн_вт_ср_чт_пт_сб'.split('_'),
  		    monthsParse: monthsParse$b,
  		    longMonthsParse: monthsParse$b,
  		    shortMonthsParse: monthsParse$b,
  		    // полные названия с падежами, по три буквы, для некоторых, по 4 буквы, сокращения с точкой и без точки
  		    monthsRegex: /^(январ[ья]|янв\.?|феврал[ья]|февр?\.?|марта?|мар\.?|апрел[ья]|апр\.?|ма[йя]|июн[ья]|июн\.?|июл[ья]|июл\.?|августа?|авг\.?|сентябр[ья]|сент?\.?|октябр[ья]|окт\.?|ноябр[ья]|нояб?\.?|декабр[ья]|дек\.?)/i,
  		    // копия предыдущего
  		    monthsShortRegex: /^(январ[ья]|янв\.?|феврал[ья]|февр?\.?|марта?|мар\.?|апрел[ья]|апр\.?|ма[йя]|июн[ья]|июн\.?|июл[ья]|июл\.?|августа?|авг\.?|сентябр[ья]|сент?\.?|октябр[ья]|окт\.?|ноябр[ья]|нояб?\.?|декабр[ья]|дек\.?)/i,
  		    // полные названия с падежами
  		    monthsStrictRegex: /^(январ[яь]|феврал[яь]|марта?|апрел[яь]|ма[яй]|июн[яь]|июл[яь]|августа?|сентябр[яь]|октябр[яь]|ноябр[яь]|декабр[яь])/i,
  		    // Выражение, которое соответствует только сокращённым формам
  		    monthsShortStrictRegex: /^(янв\.|февр?\.|мар[т.]|апр\.|ма[яй]|июн[ья.]|июл[ья.]|авг\.|сент?\.|окт\.|нояб?\.|дек\.)/i,
  		    longDateFormat: {
  		      LT: 'H:mm',
  		      LTS: 'H:mm:ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'D MMMM YYYY г.',
  		      LLL: 'D MMMM YYYY г., H:mm',
  		      LLLL: 'dddd, D MMMM YYYY г., H:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Сегодня, в] LT',
  		      nextDay: '[Завтра, в] LT',
  		      lastDay: '[Вчера, в] LT',
  		      nextWeek: function (now) {
  		        if (now.week() !== this.week()) {
  		          switch (this.day()) {
  		            case 0:
  		              return '[В следующее] dddd, [в] LT';
  		            case 1:
  		            case 2:
  		            case 4:
  		              return '[В следующий] dddd, [в] LT';
  		            case 3:
  		            case 5:
  		            case 6:
  		              return '[В следующую] dddd, [в] LT';
  		          }
  		        } else {
  		          if (this.day() === 2) {
  		            return '[Во] dddd, [в] LT';
  		          } else {
  		            return '[В] dddd, [в] LT';
  		          }
  		        }
  		      },
  		      lastWeek: function (now) {
  		        if (now.week() !== this.week()) {
  		          switch (this.day()) {
  		            case 0:
  		              return '[В прошлое] dddd, [в] LT';
  		            case 1:
  		            case 2:
  		            case 4:
  		              return '[В прошлый] dddd, [в] LT';
  		            case 3:
  		            case 5:
  		            case 6:
  		              return '[В прошлую] dddd, [в] LT';
  		          }
  		        } else {
  		          if (this.day() === 2) {
  		            return '[Во] dddd, [в] LT';
  		          } else {
  		            return '[В] dddd, [в] LT';
  		          }
  		        }
  		      },
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'через %s',
  		      past: '%s назад',
  		      s: 'несколько секунд',
  		      ss: relativeTimeWithPlural$3,
  		      m: relativeTimeWithPlural$3,
  		      mm: relativeTimeWithPlural$3,
  		      h: 'час',
  		      hh: relativeTimeWithPlural$3,
  		      d: 'день',
  		      dd: relativeTimeWithPlural$3,
  		      w: 'неделя',
  		      ww: relativeTimeWithPlural$3,
  		      M: 'месяц',
  		      MM: relativeTimeWithPlural$3,
  		      y: 'год',
  		      yy: relativeTimeWithPlural$3
  		    },
  		    meridiemParse: /ночи|утра|дня|вечера/i,
  		    isPM: function (input) {
  		      return /^(дня|вечера)$/.test(input);
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 4) {
  		        return 'ночи';
  		      } else if (hour < 12) {
  		        return 'утра';
  		      } else if (hour < 17) {
  		        return 'дня';
  		      } else {
  		        return 'вечера';
  		      }
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}-(й|го|я)/,
  		    ordinal: function (number, period) {
  		      switch (period) {
  		        case 'M':
  		        case 'd':
  		        case 'DDD':
  		          return number + '-й';
  		        case 'D':
  		          return number + '-го';
  		        case 'w':
  		        case 'W':
  		          return number + '-я';
  		        default:
  		          return number;
  		      }
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var months$9 = ['جنوري', 'فيبروري', 'مارچ', 'اپريل', 'مئي', 'جون', 'جولاءِ', 'آگسٽ', 'سيپٽمبر', 'آڪٽوبر', 'نومبر', 'ڊسمبر'],
  		    days$1 = ['آچر', 'سومر', 'اڱارو', 'اربع', 'خميس', 'جمع', 'ڇنڇر'];
  		  hooks.defineLocale('sd', {
  		    months: months$9,
  		    monthsShort: months$9,
  		    weekdays: days$1,
  		    weekdaysShort: days$1,
  		    weekdaysMin: days$1,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd، D MMMM YYYY HH:mm'
  		    },
  		    meridiemParse: /صبح|شام/,
  		    isPM: function (input) {
  		      return 'شام' === input;
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 12) {
  		        return 'صبح';
  		      }
  		      return 'شام';
  		    },
  		    calendar: {
  		      sameDay: '[اڄ] LT',
  		      nextDay: '[سڀاڻي] LT',
  		      nextWeek: 'dddd [اڳين هفتي تي] LT',
  		      lastDay: '[ڪالهه] LT',
  		      lastWeek: '[گزريل هفتي] dddd [تي] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%s پوء',
  		      past: '%s اڳ',
  		      s: 'چند سيڪنڊ',
  		      ss: '%d سيڪنڊ',
  		      m: 'هڪ منٽ',
  		      mm: '%d منٽ',
  		      h: 'هڪ ڪلاڪ',
  		      hh: '%d ڪلاڪ',
  		      d: 'هڪ ڏينهن',
  		      dd: '%d ڏينهن',
  		      M: 'هڪ مهينو',
  		      MM: '%d مهينا',
  		      y: 'هڪ سال',
  		      yy: '%d سال'
  		    },
  		    preparse: function (string) {
  		      return string.replace(/،/g, ',');
  		    },
  		    postformat: function (string) {
  		      return string.replace(/,/g, '،');
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('se', {
  		    months: 'ođđajagemánnu_guovvamánnu_njukčamánnu_cuoŋománnu_miessemánnu_geassemánnu_suoidnemánnu_borgemánnu_čakčamánnu_golggotmánnu_skábmamánnu_juovlamánnu'.split('_'),
  		    monthsShort: 'ođđj_guov_njuk_cuo_mies_geas_suoi_borg_čakč_golg_skáb_juov'.split('_'),
  		    weekdays: 'sotnabeaivi_vuossárga_maŋŋebárga_gaskavahkku_duorastat_bearjadat_lávvardat'.split('_'),
  		    weekdaysShort: 'sotn_vuos_maŋ_gask_duor_bear_láv'.split('_'),
  		    weekdaysMin: 's_v_m_g_d_b_L'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'MMMM D. [b.] YYYY',
  		      LLL: 'MMMM D. [b.] YYYY [ti.] HH:mm',
  		      LLLL: 'dddd, MMMM D. [b.] YYYY [ti.] HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[otne ti] LT',
  		      nextDay: '[ihttin ti] LT',
  		      nextWeek: 'dddd [ti] LT',
  		      lastDay: '[ikte ti] LT',
  		      lastWeek: '[ovddit] dddd [ti] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%s geažes',
  		      past: 'maŋit %s',
  		      s: 'moadde sekunddat',
  		      ss: '%d sekunddat',
  		      m: 'okta minuhta',
  		      mm: '%d minuhtat',
  		      h: 'okta diimmu',
  		      hh: '%d diimmut',
  		      d: 'okta beaivi',
  		      dd: '%d beaivvit',
  		      M: 'okta mánnu',
  		      MM: '%d mánut',
  		      y: 'okta jahki',
  		      yy: '%d jagit'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}\./,
  		    ordinal: '%d.',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  /*jshint -W100*/
  		  hooks.defineLocale('si', {
  		    months: 'ජනවාරි_පෙබරවාරි_මාර්තු_අප්‍රේල්_මැයි_ජූනි_ජූලි_අගෝස්තු_සැප්තැම්බර්_ඔක්තෝබර්_නොවැම්බර්_දෙසැම්බර්'.split('_'),
  		    monthsShort: 'ජන_පෙබ_මාර්_අප්_මැයි_ජූනි_ජූලි_අගෝ_සැප්_ඔක්_නොවැ_දෙසැ'.split('_'),
  		    weekdays: 'ඉරිදා_සඳුදා_අඟහරුවාදා_බදාදා_බ්‍රහස්පතින්දා_සිකුරාදා_සෙනසුරාදා'.split('_'),
  		    weekdaysShort: 'ඉරි_සඳු_අඟ_බදා_බ්‍රහ_සිකු_සෙන'.split('_'),
  		    weekdaysMin: 'ඉ_ස_අ_බ_බ්‍ර_සි_සෙ'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'a h:mm',
  		      LTS: 'a h:mm:ss',
  		      L: 'YYYY/MM/DD',
  		      LL: 'YYYY MMMM D',
  		      LLL: 'YYYY MMMM D, a h:mm',
  		      LLLL: 'YYYY MMMM D [වැනි] dddd, a h:mm:ss'
  		    },
  		    calendar: {
  		      sameDay: '[අද] LT[ට]',
  		      nextDay: '[හෙට] LT[ට]',
  		      nextWeek: 'dddd LT[ට]',
  		      lastDay: '[ඊයේ] LT[ට]',
  		      lastWeek: '[පසුගිය] dddd LT[ට]',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%sකින්',
  		      past: '%sකට පෙර',
  		      s: 'තත්පර කිහිපය',
  		      ss: 'තත්පර %d',
  		      m: 'මිනිත්තුව',
  		      mm: 'මිනිත්තු %d',
  		      h: 'පැය',
  		      hh: 'පැය %d',
  		      d: 'දිනය',
  		      dd: 'දින %d',
  		      M: 'මාසය',
  		      MM: 'මාස %d',
  		      y: 'වසර',
  		      yy: 'වසර %d'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2} වැනි/,
  		    ordinal: function (number) {
  		      return number + ' වැනි';
  		    },
  		    meridiemParse: /පෙර වරු|පස් වරු|පෙ.ව|ප.ව./,
  		    isPM: function (input) {
  		      return input === 'ප.ව.' || input === 'පස් වරු';
  		    },
  		    meridiem: function (hours, minutes, isLower) {
  		      if (hours > 11) {
  		        return isLower ? 'ප.ව.' : 'පස් වරු';
  		      } else {
  		        return isLower ? 'පෙ.ව.' : 'පෙර වරු';
  		      }
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var months$a = 'január_február_marec_apríl_máj_jún_júl_august_september_október_november_december'.split('_'),
  		    monthsShort$7 = 'jan_feb_mar_apr_máj_jún_júl_aug_sep_okt_nov_dec'.split('_');
  		  function plural$5(n) {
  		    return n > 1 && n < 5;
  		  }
  		  function translate$9(number, withoutSuffix, key, isFuture) {
  		    var result = number + ' ';
  		    switch (key) {
  		      case 's':
  		        // a few seconds / in a few seconds / a few seconds ago
  		        return withoutSuffix || isFuture ? 'pár sekúnd' : 'pár sekundami';
  		      case 'ss':
  		        // 9 seconds / in 9 seconds / 9 seconds ago
  		        if (withoutSuffix || isFuture) {
  		          return result + (plural$5(number) ? 'sekundy' : 'sekúnd');
  		        } else {
  		          return result + 'sekundami';
  		        }
  		      case 'm':
  		        // a minute / in a minute / a minute ago
  		        return withoutSuffix ? 'minúta' : isFuture ? 'minútu' : 'minútou';
  		      case 'mm':
  		        // 9 minutes / in 9 minutes / 9 minutes ago
  		        if (withoutSuffix || isFuture) {
  		          return result + (plural$5(number) ? 'minúty' : 'minút');
  		        } else {
  		          return result + 'minútami';
  		        }
  		      case 'h':
  		        // an hour / in an hour / an hour ago
  		        return withoutSuffix ? 'hodina' : isFuture ? 'hodinu' : 'hodinou';
  		      case 'hh':
  		        // 9 hours / in 9 hours / 9 hours ago
  		        if (withoutSuffix || isFuture) {
  		          return result + (plural$5(number) ? 'hodiny' : 'hodín');
  		        } else {
  		          return result + 'hodinami';
  		        }
  		      case 'd':
  		        // a day / in a day / a day ago
  		        return withoutSuffix || isFuture ? 'deň' : 'dňom';
  		      case 'dd':
  		        // 9 days / in 9 days / 9 days ago
  		        if (withoutSuffix || isFuture) {
  		          return result + (plural$5(number) ? 'dni' : 'dní');
  		        } else {
  		          return result + 'dňami';
  		        }
  		      case 'M':
  		        // a month / in a month / a month ago
  		        return withoutSuffix || isFuture ? 'mesiac' : 'mesiacom';
  		      case 'MM':
  		        // 9 months / in 9 months / 9 months ago
  		        if (withoutSuffix || isFuture) {
  		          return result + (plural$5(number) ? 'mesiace' : 'mesiacov');
  		        } else {
  		          return result + 'mesiacmi';
  		        }
  		      case 'y':
  		        // a year / in a year / a year ago
  		        return withoutSuffix || isFuture ? 'rok' : 'rokom';
  		      case 'yy':
  		        // 9 years / in 9 years / 9 years ago
  		        if (withoutSuffix || isFuture) {
  		          return result + (plural$5(number) ? 'roky' : 'rokov');
  		        } else {
  		          return result + 'rokmi';
  		        }
  		    }
  		  }
  		  hooks.defineLocale('sk', {
  		    months: months$a,
  		    monthsShort: monthsShort$7,
  		    weekdays: 'nedeľa_pondelok_utorok_streda_štvrtok_piatok_sobota'.split('_'),
  		    weekdaysShort: 'ne_po_ut_st_št_pi_so'.split('_'),
  		    weekdaysMin: 'ne_po_ut_st_št_pi_so'.split('_'),
  		    longDateFormat: {
  		      LT: 'H:mm',
  		      LTS: 'H:mm:ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'D. MMMM YYYY',
  		      LLL: 'D. MMMM YYYY H:mm',
  		      LLLL: 'dddd D. MMMM YYYY H:mm'
  		    },
  		    calendar: {
  		      sameDay: '[dnes o] LT',
  		      nextDay: '[zajtra o] LT',
  		      nextWeek: function () {
  		        switch (this.day()) {
  		          case 0:
  		            return '[v nedeľu o] LT';
  		          case 1:
  		          case 2:
  		            return '[v] dddd [o] LT';
  		          case 3:
  		            return '[v stredu o] LT';
  		          case 4:
  		            return '[vo štvrtok o] LT';
  		          case 5:
  		            return '[v piatok o] LT';
  		          case 6:
  		            return '[v sobotu o] LT';
  		        }
  		      },
  		      lastDay: '[včera o] LT',
  		      lastWeek: function () {
  		        switch (this.day()) {
  		          case 0:
  		            return '[minulú nedeľu o] LT';
  		          case 1:
  		          case 2:
  		            return '[minulý] dddd [o] LT';
  		          case 3:
  		            return '[minulú stredu o] LT';
  		          case 4:
  		          case 5:
  		            return '[minulý] dddd [o] LT';
  		          case 6:
  		            return '[minulú sobotu o] LT';
  		        }
  		      },
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'za %s',
  		      past: 'pred %s',
  		      s: translate$9,
  		      ss: translate$9,
  		      m: translate$9,
  		      mm: translate$9,
  		      h: translate$9,
  		      hh: translate$9,
  		      d: translate$9,
  		      dd: translate$9,
  		      M: translate$9,
  		      MM: translate$9,
  		      y: translate$9,
  		      yy: translate$9
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}\./,
  		    ordinal: '%d.',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  function processRelativeTime$9(number, withoutSuffix, key, isFuture) {
  		    var result = number + ' ';
  		    switch (key) {
  		      case 's':
  		        return withoutSuffix || isFuture ? 'nekaj sekund' : 'nekaj sekundami';
  		      case 'ss':
  		        if (number === 1) {
  		          result += withoutSuffix ? 'sekundo' : 'sekundi';
  		        } else if (number === 2) {
  		          result += withoutSuffix || isFuture ? 'sekundi' : 'sekundah';
  		        } else if (number < 5) {
  		          result += withoutSuffix || isFuture ? 'sekunde' : 'sekundah';
  		        } else {
  		          result += 'sekund';
  		        }
  		        return result;
  		      case 'm':
  		        return withoutSuffix ? 'ena minuta' : 'eno minuto';
  		      case 'mm':
  		        if (number === 1) {
  		          result += withoutSuffix ? 'minuta' : 'minuto';
  		        } else if (number === 2) {
  		          result += withoutSuffix || isFuture ? 'minuti' : 'minutama';
  		        } else if (number < 5) {
  		          result += withoutSuffix || isFuture ? 'minute' : 'minutami';
  		        } else {
  		          result += withoutSuffix || isFuture ? 'minut' : 'minutami';
  		        }
  		        return result;
  		      case 'h':
  		        return withoutSuffix ? 'ena ura' : 'eno uro';
  		      case 'hh':
  		        if (number === 1) {
  		          result += withoutSuffix ? 'ura' : 'uro';
  		        } else if (number === 2) {
  		          result += withoutSuffix || isFuture ? 'uri' : 'urama';
  		        } else if (number < 5) {
  		          result += withoutSuffix || isFuture ? 'ure' : 'urami';
  		        } else {
  		          result += withoutSuffix || isFuture ? 'ur' : 'urami';
  		        }
  		        return result;
  		      case 'd':
  		        return withoutSuffix || isFuture ? 'en dan' : 'enim dnem';
  		      case 'dd':
  		        if (number === 1) {
  		          result += withoutSuffix || isFuture ? 'dan' : 'dnem';
  		        } else if (number === 2) {
  		          result += withoutSuffix || isFuture ? 'dni' : 'dnevoma';
  		        } else {
  		          result += withoutSuffix || isFuture ? 'dni' : 'dnevi';
  		        }
  		        return result;
  		      case 'M':
  		        return withoutSuffix || isFuture ? 'en mesec' : 'enim mesecem';
  		      case 'MM':
  		        if (number === 1) {
  		          result += withoutSuffix || isFuture ? 'mesec' : 'mesecem';
  		        } else if (number === 2) {
  		          result += withoutSuffix || isFuture ? 'meseca' : 'mesecema';
  		        } else if (number < 5) {
  		          result += withoutSuffix || isFuture ? 'mesece' : 'meseci';
  		        } else {
  		          result += withoutSuffix || isFuture ? 'mesecev' : 'meseci';
  		        }
  		        return result;
  		      case 'y':
  		        return withoutSuffix || isFuture ? 'eno leto' : 'enim letom';
  		      case 'yy':
  		        if (number === 1) {
  		          result += withoutSuffix || isFuture ? 'leto' : 'letom';
  		        } else if (number === 2) {
  		          result += withoutSuffix || isFuture ? 'leti' : 'letoma';
  		        } else if (number < 5) {
  		          result += withoutSuffix || isFuture ? 'leta' : 'leti';
  		        } else {
  		          result += withoutSuffix || isFuture ? 'let' : 'leti';
  		        }
  		        return result;
  		    }
  		  }
  		  hooks.defineLocale('sl', {
  		    months: 'januar_februar_marec_april_maj_junij_julij_avgust_september_oktober_november_december'.split('_'),
  		    monthsShort: 'jan._feb._mar._apr._maj._jun._jul._avg._sep._okt._nov._dec.'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'nedelja_ponedeljek_torek_sreda_četrtek_petek_sobota'.split('_'),
  		    weekdaysShort: 'ned._pon._tor._sre._čet._pet._sob.'.split('_'),
  		    weekdaysMin: 'ne_po_to_sr_če_pe_so'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'H:mm',
  		      LTS: 'H:mm:ss',
  		      L: 'DD. MM. YYYY',
  		      LL: 'D. MMMM YYYY',
  		      LLL: 'D. MMMM YYYY H:mm',
  		      LLLL: 'dddd, D. MMMM YYYY H:mm'
  		    },
  		    calendar: {
  		      sameDay: '[danes ob] LT',
  		      nextDay: '[jutri ob] LT',
  		      nextWeek: function () {
  		        switch (this.day()) {
  		          case 0:
  		            return '[v] [nedeljo] [ob] LT';
  		          case 3:
  		            return '[v] [sredo] [ob] LT';
  		          case 6:
  		            return '[v] [soboto] [ob] LT';
  		          case 1:
  		          case 2:
  		          case 4:
  		          case 5:
  		            return '[v] dddd [ob] LT';
  		        }
  		      },
  		      lastDay: '[včeraj ob] LT',
  		      lastWeek: function () {
  		        switch (this.day()) {
  		          case 0:
  		            return '[prejšnjo] [nedeljo] [ob] LT';
  		          case 3:
  		            return '[prejšnjo] [sredo] [ob] LT';
  		          case 6:
  		            return '[prejšnjo] [soboto] [ob] LT';
  		          case 1:
  		          case 2:
  		          case 4:
  		          case 5:
  		            return '[prejšnji] dddd [ob] LT';
  		        }
  		      },
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'čez %s',
  		      past: 'pred %s',
  		      s: processRelativeTime$9,
  		      ss: processRelativeTime$9,
  		      m: processRelativeTime$9,
  		      mm: processRelativeTime$9,
  		      h: processRelativeTime$9,
  		      hh: processRelativeTime$9,
  		      d: processRelativeTime$9,
  		      dd: processRelativeTime$9,
  		      M: processRelativeTime$9,
  		      MM: processRelativeTime$9,
  		      y: processRelativeTime$9,
  		      yy: processRelativeTime$9
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}\./,
  		    ordinal: '%d.',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 7 // The week that contains Jan 7th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('sq', {
  		    months: 'Janar_Shkurt_Mars_Prill_Maj_Qershor_Korrik_Gusht_Shtator_Tetor_Nëntor_Dhjetor'.split('_'),
  		    monthsShort: 'Jan_Shk_Mar_Pri_Maj_Qer_Kor_Gus_Sht_Tet_Nën_Dhj'.split('_'),
  		    weekdays: 'E Diel_E Hënë_E Martë_E Mërkurë_E Enjte_E Premte_E Shtunë'.split('_'),
  		    weekdaysShort: 'Die_Hën_Mar_Mër_Enj_Pre_Sht'.split('_'),
  		    weekdaysMin: 'D_H_Ma_Më_E_P_Sh'.split('_'),
  		    weekdaysParseExact: true,
  		    meridiemParse: /PD|MD/,
  		    isPM: function (input) {
  		      return input.charAt(0) === 'M';
  		    },
  		    meridiem: function (hours, minutes, isLower) {
  		      return hours < 12 ? 'PD' : 'MD';
  		    },
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd, D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Sot në] LT',
  		      nextDay: '[Nesër në] LT',
  		      nextWeek: 'dddd [në] LT',
  		      lastDay: '[Dje në] LT',
  		      lastWeek: 'dddd [e kaluar në] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'në %s',
  		      past: '%s më parë',
  		      s: 'disa sekonda',
  		      ss: '%d sekonda',
  		      m: 'një minutë',
  		      mm: '%d minuta',
  		      h: 'një orë',
  		      hh: '%d orë',
  		      d: 'një ditë',
  		      dd: '%d ditë',
  		      M: 'një muaj',
  		      MM: '%d muaj',
  		      y: 'një vit',
  		      yy: '%d vite'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}\./,
  		    ordinal: '%d.',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var translator$1 = {
  		    words: {
  		      //Different grammatical cases
  		      ss: ['секунда', 'секунде', 'секунди'],
  		      m: ['један минут', 'једног минута'],
  		      mm: ['минут', 'минута', 'минута'],
  		      h: ['један сат', 'једног сата'],
  		      hh: ['сат', 'сата', 'сати'],
  		      d: ['један дан', 'једног дана'],
  		      dd: ['дан', 'дана', 'дана'],
  		      M: ['један месец', 'једног месеца'],
  		      MM: ['месец', 'месеца', 'месеци'],
  		      y: ['једну годину', 'једне године'],
  		      yy: ['годину', 'године', 'година']
  		    },
  		    correctGrammaticalCase: function (number, wordKey) {
  		      if (number % 10 >= 1 && number % 10 <= 4 && (number % 100 < 10 || number % 100 >= 20)) {
  		        return number % 10 === 1 ? wordKey[0] : wordKey[1];
  		      }
  		      return wordKey[2];
  		    },
  		    translate: function (number, withoutSuffix, key, isFuture) {
  		      var wordKey = translator$1.words[key],
  		        word;
  		      if (key.length === 1) {
  		        // Nominativ
  		        if (key === 'y' && withoutSuffix) return 'једна година';
  		        return isFuture || withoutSuffix ? wordKey[0] : wordKey[1];
  		      }
  		      word = translator$1.correctGrammaticalCase(number, wordKey);
  		      // Nominativ
  		      if (key === 'yy' && withoutSuffix && word === 'годину') {
  		        return number + ' година';
  		      }
  		      return number + ' ' + word;
  		    }
  		  };
  		  hooks.defineLocale('sr-cyrl', {
  		    months: 'јануар_фебруар_март_април_мај_јун_јул_август_септембар_октобар_новембар_децембар'.split('_'),
  		    monthsShort: 'јан._феб._мар._апр._мај_јун_јул_авг._сеп._окт._нов._дец.'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'недеља_понедељак_уторак_среда_четвртак_петак_субота'.split('_'),
  		    weekdaysShort: 'нед._пон._уто._сре._чет._пет._суб.'.split('_'),
  		    weekdaysMin: 'не_по_ут_ср_че_пе_су'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'H:mm',
  		      LTS: 'H:mm:ss',
  		      L: 'D. M. YYYY.',
  		      LL: 'D. MMMM YYYY.',
  		      LLL: 'D. MMMM YYYY. H:mm',
  		      LLLL: 'dddd, D. MMMM YYYY. H:mm'
  		    },
  		    calendar: {
  		      sameDay: '[данас у] LT',
  		      nextDay: '[сутра у] LT',
  		      nextWeek: function () {
  		        switch (this.day()) {
  		          case 0:
  		            return '[у] [недељу] [у] LT';
  		          case 3:
  		            return '[у] [среду] [у] LT';
  		          case 6:
  		            return '[у] [суботу] [у] LT';
  		          case 1:
  		          case 2:
  		          case 4:
  		          case 5:
  		            return '[у] dddd [у] LT';
  		        }
  		      },
  		      lastDay: '[јуче у] LT',
  		      lastWeek: function () {
  		        var lastWeekDays = ['[прошле] [недеље] [у] LT', '[прошлог] [понедељка] [у] LT', '[прошлог] [уторка] [у] LT', '[прошле] [среде] [у] LT', '[прошлог] [четвртка] [у] LT', '[прошлог] [петка] [у] LT', '[прошле] [суботе] [у] LT'];
  		        return lastWeekDays[this.day()];
  		      },
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'за %s',
  		      past: 'пре %s',
  		      s: 'неколико секунди',
  		      ss: translator$1.translate,
  		      m: translator$1.translate,
  		      mm: translator$1.translate,
  		      h: translator$1.translate,
  		      hh: translator$1.translate,
  		      d: translator$1.translate,
  		      dd: translator$1.translate,
  		      M: translator$1.translate,
  		      MM: translator$1.translate,
  		      y: translator$1.translate,
  		      yy: translator$1.translate
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}\./,
  		    ordinal: '%d.',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 7 // The week that contains Jan 1st is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var translator$2 = {
  		    words: {
  		      //Different grammatical cases
  		      ss: ['sekunda', 'sekunde', 'sekundi'],
  		      m: ['jedan minut', 'jednog minuta'],
  		      mm: ['minut', 'minuta', 'minuta'],
  		      h: ['jedan sat', 'jednog sata'],
  		      hh: ['sat', 'sata', 'sati'],
  		      d: ['jedan dan', 'jednog dana'],
  		      dd: ['dan', 'dana', 'dana'],
  		      M: ['jedan mesec', 'jednog meseca'],
  		      MM: ['mesec', 'meseca', 'meseci'],
  		      y: ['jednu godinu', 'jedne godine'],
  		      yy: ['godinu', 'godine', 'godina']
  		    },
  		    correctGrammaticalCase: function (number, wordKey) {
  		      if (number % 10 >= 1 && number % 10 <= 4 && (number % 100 < 10 || number % 100 >= 20)) {
  		        return number % 10 === 1 ? wordKey[0] : wordKey[1];
  		      }
  		      return wordKey[2];
  		    },
  		    translate: function (number, withoutSuffix, key, isFuture) {
  		      var wordKey = translator$2.words[key],
  		        word;
  		      if (key.length === 1) {
  		        // Nominativ
  		        if (key === 'y' && withoutSuffix) return 'jedna godina';
  		        return isFuture || withoutSuffix ? wordKey[0] : wordKey[1];
  		      }
  		      word = translator$2.correctGrammaticalCase(number, wordKey);
  		      // Nominativ
  		      if (key === 'yy' && withoutSuffix && word === 'godinu') {
  		        return number + ' godina';
  		      }
  		      return number + ' ' + word;
  		    }
  		  };
  		  hooks.defineLocale('sr', {
  		    months: 'januar_februar_mart_april_maj_jun_jul_avgust_septembar_oktobar_novembar_decembar'.split('_'),
  		    monthsShort: 'jan._feb._mar._apr._maj_jun_jul_avg._sep._okt._nov._dec.'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'nedelja_ponedeljak_utorak_sreda_četvrtak_petak_subota'.split('_'),
  		    weekdaysShort: 'ned._pon._uto._sre._čet._pet._sub.'.split('_'),
  		    weekdaysMin: 'ne_po_ut_sr_če_pe_su'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'H:mm',
  		      LTS: 'H:mm:ss',
  		      L: 'D. M. YYYY.',
  		      LL: 'D. MMMM YYYY.',
  		      LLL: 'D. MMMM YYYY. H:mm',
  		      LLLL: 'dddd, D. MMMM YYYY. H:mm'
  		    },
  		    calendar: {
  		      sameDay: '[danas u] LT',
  		      nextDay: '[sutra u] LT',
  		      nextWeek: function () {
  		        switch (this.day()) {
  		          case 0:
  		            return '[u] [nedelju] [u] LT';
  		          case 3:
  		            return '[u] [sredu] [u] LT';
  		          case 6:
  		            return '[u] [subotu] [u] LT';
  		          case 1:
  		          case 2:
  		          case 4:
  		          case 5:
  		            return '[u] dddd [u] LT';
  		        }
  		      },
  		      lastDay: '[juče u] LT',
  		      lastWeek: function () {
  		        var lastWeekDays = ['[prošle] [nedelje] [u] LT', '[prošlog] [ponedeljka] [u] LT', '[prošlog] [utorka] [u] LT', '[prošle] [srede] [u] LT', '[prošlog] [četvrtka] [u] LT', '[prošlog] [petka] [u] LT', '[prošle] [subote] [u] LT'];
  		        return lastWeekDays[this.day()];
  		      },
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'za %s',
  		      past: 'pre %s',
  		      s: 'nekoliko sekundi',
  		      ss: translator$2.translate,
  		      m: translator$2.translate,
  		      mm: translator$2.translate,
  		      h: translator$2.translate,
  		      hh: translator$2.translate,
  		      d: translator$2.translate,
  		      dd: translator$2.translate,
  		      M: translator$2.translate,
  		      MM: translator$2.translate,
  		      y: translator$2.translate,
  		      yy: translator$2.translate
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}\./,
  		    ordinal: '%d.',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 7 // The week that contains Jan 7th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('ss', {
  		    months: "Bhimbidvwane_Indlovana_Indlov'lenkhulu_Mabasa_Inkhwekhweti_Inhlaba_Kholwane_Ingci_Inyoni_Imphala_Lweti_Ingongoni".split('_'),
  		    monthsShort: 'Bhi_Ina_Inu_Mab_Ink_Inh_Kho_Igc_Iny_Imp_Lwe_Igo'.split('_'),
  		    weekdays: 'Lisontfo_Umsombuluko_Lesibili_Lesitsatfu_Lesine_Lesihlanu_Umgcibelo'.split('_'),
  		    weekdaysShort: 'Lis_Umb_Lsb_Les_Lsi_Lsh_Umg'.split('_'),
  		    weekdaysMin: 'Li_Us_Lb_Lt_Ls_Lh_Ug'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'h:mm A',
  		      LTS: 'h:mm:ss A',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY h:mm A',
  		      LLLL: 'dddd, D MMMM YYYY h:mm A'
  		    },
  		    calendar: {
  		      sameDay: '[Namuhla nga] LT',
  		      nextDay: '[Kusasa nga] LT',
  		      nextWeek: 'dddd [nga] LT',
  		      lastDay: '[Itolo nga] LT',
  		      lastWeek: 'dddd [leliphelile] [nga] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'nga %s',
  		      past: 'wenteka nga %s',
  		      s: 'emizuzwana lomcane',
  		      ss: '%d mzuzwana',
  		      m: 'umzuzu',
  		      mm: '%d emizuzu',
  		      h: 'lihora',
  		      hh: '%d emahora',
  		      d: 'lilanga',
  		      dd: '%d emalanga',
  		      M: 'inyanga',
  		      MM: '%d tinyanga',
  		      y: 'umnyaka',
  		      yy: '%d iminyaka'
  		    },
  		    meridiemParse: /ekuseni|emini|entsambama|ebusuku/,
  		    meridiem: function (hours, minutes, isLower) {
  		      if (hours < 11) {
  		        return 'ekuseni';
  		      } else if (hours < 15) {
  		        return 'emini';
  		      } else if (hours < 19) {
  		        return 'entsambama';
  		      } else {
  		        return 'ebusuku';
  		      }
  		    },
  		    meridiemHour: function (hour, meridiem) {
  		      if (hour === 12) {
  		        hour = 0;
  		      }
  		      if (meridiem === 'ekuseni') {
  		        return hour;
  		      } else if (meridiem === 'emini') {
  		        return hour >= 11 ? hour : hour + 12;
  		      } else if (meridiem === 'entsambama' || meridiem === 'ebusuku') {
  		        if (hour === 0) {
  		          return 0;
  		        }
  		        return hour + 12;
  		      }
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}/,
  		    ordinal: '%d',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('sv', {
  		    months: 'januari_februari_mars_april_maj_juni_juli_augusti_september_oktober_november_december'.split('_'),
  		    monthsShort: 'jan_feb_mar_apr_maj_jun_jul_aug_sep_okt_nov_dec'.split('_'),
  		    weekdays: 'söndag_måndag_tisdag_onsdag_torsdag_fredag_lördag'.split('_'),
  		    weekdaysShort: 'sön_mån_tis_ons_tor_fre_lör'.split('_'),
  		    weekdaysMin: 'sö_må_ti_on_to_fr_lö'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'YYYY-MM-DD',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY [kl.] HH:mm',
  		      LLLL: 'dddd D MMMM YYYY [kl.] HH:mm',
  		      lll: 'D MMM YYYY HH:mm',
  		      llll: 'ddd D MMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Idag] LT',
  		      nextDay: '[Imorgon] LT',
  		      lastDay: '[Igår] LT',
  		      nextWeek: '[På] dddd LT',
  		      lastWeek: '[I] dddd[s] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'om %s',
  		      past: 'för %s sedan',
  		      s: 'några sekunder',
  		      ss: '%d sekunder',
  		      m: 'en minut',
  		      mm: '%d minuter',
  		      h: 'en timme',
  		      hh: '%d timmar',
  		      d: 'en dag',
  		      dd: '%d dagar',
  		      M: 'en månad',
  		      MM: '%d månader',
  		      y: 'ett år',
  		      yy: '%d år'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(\:e|\:a)/,
  		    ordinal: function (number) {
  		      var b = number % 10,
  		        output = ~~(number % 100 / 10) === 1 ? ':e' : b === 1 ? ':a' : b === 2 ? ':a' : b === 3 ? ':e' : ':e';
  		      return number + output;
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('sw', {
  		    months: 'Januari_Februari_Machi_Aprili_Mei_Juni_Julai_Agosti_Septemba_Oktoba_Novemba_Desemba'.split('_'),
  		    monthsShort: 'Jan_Feb_Mac_Apr_Mei_Jun_Jul_Ago_Sep_Okt_Nov_Des'.split('_'),
  		    weekdays: 'Jumapili_Jumatatu_Jumanne_Jumatano_Alhamisi_Ijumaa_Jumamosi'.split('_'),
  		    weekdaysShort: 'Jpl_Jtat_Jnne_Jtan_Alh_Ijm_Jmos'.split('_'),
  		    weekdaysMin: 'J2_J3_J4_J5_Al_Ij_J1'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'hh:mm A',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd, D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[leo saa] LT',
  		      nextDay: '[kesho saa] LT',
  		      nextWeek: '[wiki ijayo] dddd [saat] LT',
  		      lastDay: '[jana] LT',
  		      lastWeek: '[wiki iliyopita] dddd [saat] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%s baadaye',
  		      past: 'tokea %s',
  		      s: 'hivi punde',
  		      ss: 'sekunde %d',
  		      m: 'dakika moja',
  		      mm: 'dakika %d',
  		      h: 'saa limoja',
  		      hh: 'masaa %d',
  		      d: 'siku moja',
  		      dd: 'siku %d',
  		      M: 'mwezi mmoja',
  		      MM: 'miezi %d',
  		      y: 'mwaka mmoja',
  		      yy: 'miaka %d'
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 7 // The week that contains Jan 7th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var symbolMap$h = {
  		      1: '௧',
  		      2: '௨',
  		      3: '௩',
  		      4: '௪',
  		      5: '௫',
  		      6: '௬',
  		      7: '௭',
  		      8: '௮',
  		      9: '௯',
  		      0: '௦'
  		    },
  		    numberMap$g = {
  		      '௧': '1',
  		      '௨': '2',
  		      '௩': '3',
  		      '௪': '4',
  		      '௫': '5',
  		      '௬': '6',
  		      '௭': '7',
  		      '௮': '8',
  		      '௯': '9',
  		      '௦': '0'
  		    };
  		  hooks.defineLocale('ta', {
  		    months: 'ஜனவரி_பிப்ரவரி_மார்ச்_ஏப்ரல்_மே_ஜூன்_ஜூலை_ஆகஸ்ட்_செப்டெம்பர்_அக்டோபர்_நவம்பர்_டிசம்பர்'.split('_'),
  		    monthsShort: 'ஜனவரி_பிப்ரவரி_மார்ச்_ஏப்ரல்_மே_ஜூன்_ஜூலை_ஆகஸ்ட்_செப்டெம்பர்_அக்டோபர்_நவம்பர்_டிசம்பர்'.split('_'),
  		    weekdays: 'ஞாயிற்றுக்கிழமை_திங்கட்கிழமை_செவ்வாய்கிழமை_புதன்கிழமை_வியாழக்கிழமை_வெள்ளிக்கிழமை_சனிக்கிழமை'.split('_'),
  		    weekdaysShort: 'ஞாயிறு_திங்கள்_செவ்வாய்_புதன்_வியாழன்_வெள்ளி_சனி'.split('_'),
  		    weekdaysMin: 'ஞா_தி_செ_பு_வி_வெ_ச'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY, HH:mm',
  		      LLLL: 'dddd, D MMMM YYYY, HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[இன்று] LT',
  		      nextDay: '[நாளை] LT',
  		      nextWeek: 'dddd, LT',
  		      lastDay: '[நேற்று] LT',
  		      lastWeek: '[கடந்த வாரம்] dddd, LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%s இல்',
  		      past: '%s முன்',
  		      s: 'ஒரு சில விநாடிகள்',
  		      ss: '%d விநாடிகள்',
  		      m: 'ஒரு நிமிடம்',
  		      mm: '%d நிமிடங்கள்',
  		      h: 'ஒரு மணி நேரம்',
  		      hh: '%d மணி நேரம்',
  		      d: 'ஒரு நாள்',
  		      dd: '%d நாட்கள்',
  		      M: 'ஒரு மாதம்',
  		      MM: '%d மாதங்கள்',
  		      y: 'ஒரு வருடம்',
  		      yy: '%d ஆண்டுகள்'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}வது/,
  		    ordinal: function (number) {
  		      return number + 'வது';
  		    },
  		    preparse: function (string) {
  		      return string.replace(/[௧௨௩௪௫௬௭௮௯௦]/g, function (match) {
  		        return numberMap$g[match];
  		      });
  		    },
  		    postformat: function (string) {
  		      return string.replace(/\d/g, function (match) {
  		        return symbolMap$h[match];
  		      });
  		    },
  		    // refer http://ta.wikipedia.org/s/1er1
  		    meridiemParse: /யாமம்|வைகறை|காலை|நண்பகல்|எற்பாடு|மாலை/,
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 2) {
  		        return ' யாமம்';
  		      } else if (hour < 6) {
  		        return ' வைகறை'; // வைகறை
  		      } else if (hour < 10) {
  		        return ' காலை'; // காலை
  		      } else if (hour < 14) {
  		        return ' நண்பகல்'; // நண்பகல்
  		      } else if (hour < 18) {
  		        return ' எற்பாடு'; // எற்பாடு
  		      } else if (hour < 22) {
  		        return ' மாலை'; // மாலை
  		      } else {
  		        return ' யாமம்';
  		      }
  		    },
  		    meridiemHour: function (hour, meridiem) {
  		      if (hour === 12) {
  		        hour = 0;
  		      }
  		      if (meridiem === 'யாமம்') {
  		        return hour < 2 ? hour : hour + 12;
  		      } else if (meridiem === 'வைகறை' || meridiem === 'காலை') {
  		        return hour;
  		      } else if (meridiem === 'நண்பகல்') {
  		        return hour >= 10 ? hour : hour + 12;
  		      } else {
  		        return hour + 12;
  		      }
  		    },
  		    week: {
  		      dow: 0,
  		      // Sunday is the first day of the week.
  		      doy: 6 // The week that contains Jan 6th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('te', {
  		    months: 'జనవరి_ఫిబ్రవరి_మార్చి_ఏప్రిల్_మే_జూన్_జులై_ఆగస్టు_సెప్టెంబర్_అక్టోబర్_నవంబర్_డిసెంబర్'.split('_'),
  		    monthsShort: 'జన._ఫిబ్ర._మార్చి_ఏప్రి._మే_జూన్_జులై_ఆగ._సెప్._అక్టో._నవ._డిసె.'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'ఆదివారం_సోమవారం_మంగళవారం_బుధవారం_గురువారం_శుక్రవారం_శనివారం'.split('_'),
  		    weekdaysShort: 'ఆది_సోమ_మంగళ_బుధ_గురు_శుక్ర_శని'.split('_'),
  		    weekdaysMin: 'ఆ_సో_మం_బు_గు_శు_శ'.split('_'),
  		    longDateFormat: {
  		      LT: 'A h:mm',
  		      LTS: 'A h:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY, A h:mm',
  		      LLLL: 'dddd, D MMMM YYYY, A h:mm'
  		    },
  		    calendar: {
  		      sameDay: '[నేడు] LT',
  		      nextDay: '[రేపు] LT',
  		      nextWeek: 'dddd, LT',
  		      lastDay: '[నిన్న] LT',
  		      lastWeek: '[గత] dddd, LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%s లో',
  		      past: '%s క్రితం',
  		      s: 'కొన్ని క్షణాలు',
  		      ss: '%d సెకన్లు',
  		      m: 'ఒక నిమిషం',
  		      mm: '%d నిమిషాలు',
  		      h: 'ఒక గంట',
  		      hh: '%d గంటలు',
  		      d: 'ఒక రోజు',
  		      dd: '%d రోజులు',
  		      M: 'ఒక నెల',
  		      MM: '%d నెలలు',
  		      y: 'ఒక సంవత్సరం',
  		      yy: '%d సంవత్సరాలు'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}వ/,
  		    ordinal: '%dవ',
  		    meridiemParse: /రాత్రి|ఉదయం|మధ్యాహ్నం|సాయంత్రం/,
  		    meridiemHour: function (hour, meridiem) {
  		      if (hour === 12) {
  		        hour = 0;
  		      }
  		      if (meridiem === 'రాత్రి') {
  		        return hour < 4 ? hour : hour + 12;
  		      } else if (meridiem === 'ఉదయం') {
  		        return hour;
  		      } else if (meridiem === 'మధ్యాహ్నం') {
  		        return hour >= 10 ? hour : hour + 12;
  		      } else if (meridiem === 'సాయంత్రం') {
  		        return hour + 12;
  		      }
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 4) {
  		        return 'రాత్రి';
  		      } else if (hour < 10) {
  		        return 'ఉదయం';
  		      } else if (hour < 17) {
  		        return 'మధ్యాహ్నం';
  		      } else if (hour < 20) {
  		        return 'సాయంత్రం';
  		      } else {
  		        return 'రాత్రి';
  		      }
  		    },
  		    week: {
  		      dow: 0,
  		      // Sunday is the first day of the week.
  		      doy: 6 // The week that contains Jan 6th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('tet', {
  		    months: 'Janeiru_Fevereiru_Marsu_Abril_Maiu_Juñu_Jullu_Agustu_Setembru_Outubru_Novembru_Dezembru'.split('_'),
  		    monthsShort: 'Jan_Fev_Mar_Abr_Mai_Jun_Jul_Ago_Set_Out_Nov_Dez'.split('_'),
  		    weekdays: 'Domingu_Segunda_Tersa_Kuarta_Kinta_Sesta_Sabadu'.split('_'),
  		    weekdaysShort: 'Dom_Seg_Ters_Kua_Kint_Sest_Sab'.split('_'),
  		    weekdaysMin: 'Do_Seg_Te_Ku_Ki_Ses_Sa'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd, D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Ohin iha] LT',
  		      nextDay: '[Aban iha] LT',
  		      nextWeek: 'dddd [iha] LT',
  		      lastDay: '[Horiseik iha] LT',
  		      lastWeek: 'dddd [semana kotuk] [iha] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'iha %s',
  		      past: '%s liuba',
  		      s: 'segundu balun',
  		      ss: 'segundu %d',
  		      m: 'minutu ida',
  		      mm: 'minutu %d',
  		      h: 'oras ida',
  		      hh: 'oras %d',
  		      d: 'loron ida',
  		      dd: 'loron %d',
  		      M: 'fulan ida',
  		      MM: 'fulan %d',
  		      y: 'tinan ida',
  		      yy: 'tinan %d'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(st|nd|rd|th)/,
  		    ordinal: function (number) {
  		      var b = number % 10,
  		        output = ~~(number % 100 / 10) === 1 ? 'th' : b === 1 ? 'st' : b === 2 ? 'nd' : b === 3 ? 'rd' : 'th';
  		      return number + output;
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var suffixes$3 = {
  		    0: '-ум',
  		    1: '-ум',
  		    2: '-юм',
  		    3: '-юм',
  		    4: '-ум',
  		    5: '-ум',
  		    6: '-ум',
  		    7: '-ум',
  		    8: '-ум',
  		    9: '-ум',
  		    10: '-ум',
  		    12: '-ум',
  		    13: '-ум',
  		    20: '-ум',
  		    30: '-юм',
  		    40: '-ум',
  		    50: '-ум',
  		    60: '-ум',
  		    70: '-ум',
  		    80: '-ум',
  		    90: '-ум',
  		    100: '-ум'
  		  };
  		  hooks.defineLocale('tg', {
  		    months: {
  		      format: 'январи_феврали_марти_апрели_майи_июни_июли_августи_сентябри_октябри_ноябри_декабри'.split('_'),
  		      standalone: 'январ_феврал_март_апрел_май_июн_июл_август_сентябр_октябр_ноябр_декабр'.split('_')
  		    },
  		    monthsShort: 'янв_фев_мар_апр_май_июн_июл_авг_сен_окт_ноя_дек'.split('_'),
  		    weekdays: 'якшанбе_душанбе_сешанбе_чоршанбе_панҷшанбе_ҷумъа_шанбе'.split('_'),
  		    weekdaysShort: 'яшб_дшб_сшб_чшб_пшб_ҷум_шнб'.split('_'),
  		    weekdaysMin: 'яш_дш_сш_чш_пш_ҷм_шб'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd, D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Имрӯз соати] LT',
  		      nextDay: '[Фардо соати] LT',
  		      lastDay: '[Дирӯз соати] LT',
  		      nextWeek: 'dddd[и] [ҳафтаи оянда соати] LT',
  		      lastWeek: 'dddd[и] [ҳафтаи гузашта соати] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'баъди %s',
  		      past: '%s пеш',
  		      s: 'якчанд сония',
  		      m: 'як дақиқа',
  		      mm: '%d дақиқа',
  		      h: 'як соат',
  		      hh: '%d соат',
  		      d: 'як рӯз',
  		      dd: '%d рӯз',
  		      M: 'як моҳ',
  		      MM: '%d моҳ',
  		      y: 'як сол',
  		      yy: '%d сол'
  		    },
  		    meridiemParse: /шаб|субҳ|рӯз|бегоҳ/,
  		    meridiemHour: function (hour, meridiem) {
  		      if (hour === 12) {
  		        hour = 0;
  		      }
  		      if (meridiem === 'шаб') {
  		        return hour < 4 ? hour : hour + 12;
  		      } else if (meridiem === 'субҳ') {
  		        return hour;
  		      } else if (meridiem === 'рӯз') {
  		        return hour >= 11 ? hour : hour + 12;
  		      } else if (meridiem === 'бегоҳ') {
  		        return hour + 12;
  		      }
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 4) {
  		        return 'шаб';
  		      } else if (hour < 11) {
  		        return 'субҳ';
  		      } else if (hour < 16) {
  		        return 'рӯз';
  		      } else if (hour < 19) {
  		        return 'бегоҳ';
  		      } else {
  		        return 'шаб';
  		      }
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}-(ум|юм)/,
  		    ordinal: function (number) {
  		      var a = number % 10,
  		        b = number >= 100 ? 100 : null;
  		      return number + (suffixes$3[number] || suffixes$3[a] || suffixes$3[b]);
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 7 // The week that contains Jan 1th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('th', {
  		    months: 'มกราคม_กุมภาพันธ์_มีนาคม_เมษายน_พฤษภาคม_มิถุนายน_กรกฎาคม_สิงหาคม_กันยายน_ตุลาคม_พฤศจิกายน_ธันวาคม'.split('_'),
  		    monthsShort: 'ม.ค._ก.พ._มี.ค._เม.ย._พ.ค._มิ.ย._ก.ค._ส.ค._ก.ย._ต.ค._พ.ย._ธ.ค.'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'อาทิตย์_จันทร์_อังคาร_พุธ_พฤหัสบดี_ศุกร์_เสาร์'.split('_'),
  		    weekdaysShort: 'อาทิตย์_จันทร์_อังคาร_พุธ_พฤหัส_ศุกร์_เสาร์'.split('_'),
  		    // yes, three characters difference
  		    weekdaysMin: 'อา._จ._อ._พ._พฤ._ศ._ส.'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'H:mm',
  		      LTS: 'H:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY เวลา H:mm',
  		      LLLL: 'วันddddที่ D MMMM YYYY เวลา H:mm'
  		    },
  		    meridiemParse: /ก่อนเที่ยง|หลังเที่ยง/,
  		    isPM: function (input) {
  		      return input === 'หลังเที่ยง';
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 12) {
  		        return 'ก่อนเที่ยง';
  		      } else {
  		        return 'หลังเที่ยง';
  		      }
  		    },
  		    calendar: {
  		      sameDay: '[วันนี้ เวลา] LT',
  		      nextDay: '[พรุ่งนี้ เวลา] LT',
  		      nextWeek: 'dddd[หน้า เวลา] LT',
  		      lastDay: '[เมื่อวานนี้ เวลา] LT',
  		      lastWeek: '[วัน]dddd[ที่แล้ว เวลา] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'อีก %s',
  		      past: '%sที่แล้ว',
  		      s: 'ไม่กี่วินาที',
  		      ss: '%d วินาที',
  		      m: '1 นาที',
  		      mm: '%d นาที',
  		      h: '1 ชั่วโมง',
  		      hh: '%d ชั่วโมง',
  		      d: '1 วัน',
  		      dd: '%d วัน',
  		      w: '1 สัปดาห์',
  		      ww: '%d สัปดาห์',
  		      M: '1 เดือน',
  		      MM: '%d เดือน',
  		      y: '1 ปี',
  		      yy: '%d ปี'
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var suffixes$4 = {
  		    1: "'inji",
  		    5: "'inji",
  		    8: "'inji",
  		    70: "'inji",
  		    80: "'inji",
  		    2: "'nji",
  		    7: "'nji",
  		    20: "'nji",
  		    50: "'nji",
  		    3: "'ünji",
  		    4: "'ünji",
  		    100: "'ünji",
  		    6: "'njy",
  		    9: "'unjy",
  		    10: "'unjy",
  		    30: "'unjy",
  		    60: "'ynjy",
  		    90: "'ynjy"
  		  };
  		  hooks.defineLocale('tk', {
  		    months: 'Ýanwar_Fewral_Mart_Aprel_Maý_Iýun_Iýul_Awgust_Sentýabr_Oktýabr_Noýabr_Dekabr'.split('_'),
  		    monthsShort: 'Ýan_Few_Mar_Apr_Maý_Iýn_Iýl_Awg_Sen_Okt_Noý_Dek'.split('_'),
  		    weekdays: 'Ýekşenbe_Duşenbe_Sişenbe_Çarşenbe_Penşenbe_Anna_Şenbe'.split('_'),
  		    weekdaysShort: 'Ýek_Duş_Siş_Çar_Pen_Ann_Şen'.split('_'),
  		    weekdaysMin: 'Ýk_Dş_Sş_Çr_Pn_An_Şn'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd, D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[bugün sagat] LT',
  		      nextDay: '[ertir sagat] LT',
  		      nextWeek: '[indiki] dddd [sagat] LT',
  		      lastDay: '[düýn] LT',
  		      lastWeek: '[geçen] dddd [sagat] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%s soň',
  		      past: '%s öň',
  		      s: 'birnäçe sekunt',
  		      m: 'bir minut',
  		      mm: '%d minut',
  		      h: 'bir sagat',
  		      hh: '%d sagat',
  		      d: 'bir gün',
  		      dd: '%d gün',
  		      M: 'bir aý',
  		      MM: '%d aý',
  		      y: 'bir ýyl',
  		      yy: '%d ýyl'
  		    },
  		    ordinal: function (number, period) {
  		      switch (period) {
  		        case 'd':
  		        case 'D':
  		        case 'Do':
  		        case 'DD':
  		          return number;
  		        default:
  		          if (number === 0) {
  		            // special case for zero
  		            return number + "'unjy";
  		          }
  		          var a = number % 10,
  		            b = number % 100 - a,
  		            c = number >= 100 ? 100 : null;
  		          return number + (suffixes$4[a] || suffixes$4[b] || suffixes$4[c]);
  		      }
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 7 // The week that contains Jan 7th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('tl-ph', {
  		    months: 'Enero_Pebrero_Marso_Abril_Mayo_Hunyo_Hulyo_Agosto_Setyembre_Oktubre_Nobyembre_Disyembre'.split('_'),
  		    monthsShort: 'Ene_Peb_Mar_Abr_May_Hun_Hul_Ago_Set_Okt_Nob_Dis'.split('_'),
  		    weekdays: 'Linggo_Lunes_Martes_Miyerkules_Huwebes_Biyernes_Sabado'.split('_'),
  		    weekdaysShort: 'Lin_Lun_Mar_Miy_Huw_Biy_Sab'.split('_'),
  		    weekdaysMin: 'Li_Lu_Ma_Mi_Hu_Bi_Sab'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'MM/D/YYYY',
  		      LL: 'MMMM D, YYYY',
  		      LLL: 'MMMM D, YYYY HH:mm',
  		      LLLL: 'dddd, MMMM DD, YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: 'LT [ngayong araw]',
  		      nextDay: '[Bukas ng] LT',
  		      nextWeek: 'LT [sa susunod na] dddd',
  		      lastDay: 'LT [kahapon]',
  		      lastWeek: 'LT [noong nakaraang] dddd',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'sa loob ng %s',
  		      past: '%s ang nakalipas',
  		      s: 'ilang segundo',
  		      ss: '%d segundo',
  		      m: 'isang minuto',
  		      mm: '%d minuto',
  		      h: 'isang oras',
  		      hh: '%d oras',
  		      d: 'isang araw',
  		      dd: '%d araw',
  		      M: 'isang buwan',
  		      MM: '%d buwan',
  		      y: 'isang taon',
  		      yy: '%d taon'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}/,
  		    ordinal: function (number) {
  		      return number;
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var numbersNouns = 'pagh_wa’_cha’_wej_loS_vagh_jav_Soch_chorgh_Hut'.split('_');
  		  function translateFuture(output) {
  		    var time = output;
  		    time = output.indexOf('jaj') !== -1 ? time.slice(0, -3) + 'leS' : output.indexOf('jar') !== -1 ? time.slice(0, -3) + 'waQ' : output.indexOf('DIS') !== -1 ? time.slice(0, -3) + 'nem' : time + ' pIq';
  		    return time;
  		  }
  		  function translatePast(output) {
  		    var time = output;
  		    time = output.indexOf('jaj') !== -1 ? time.slice(0, -3) + 'Hu’' : output.indexOf('jar') !== -1 ? time.slice(0, -3) + 'wen' : output.indexOf('DIS') !== -1 ? time.slice(0, -3) + 'ben' : time + ' ret';
  		    return time;
  		  }
  		  function translate$a(number, withoutSuffix, string, isFuture) {
  		    var numberNoun = numberAsNoun(number);
  		    switch (string) {
  		      case 'ss':
  		        return numberNoun + ' lup';
  		      case 'mm':
  		        return numberNoun + ' tup';
  		      case 'hh':
  		        return numberNoun + ' rep';
  		      case 'dd':
  		        return numberNoun + ' jaj';
  		      case 'MM':
  		        return numberNoun + ' jar';
  		      case 'yy':
  		        return numberNoun + ' DIS';
  		    }
  		  }
  		  function numberAsNoun(number) {
  		    var hundred = Math.floor(number % 1000 / 100),
  		      ten = Math.floor(number % 100 / 10),
  		      one = number % 10,
  		      word = '';
  		    if (hundred > 0) {
  		      word += numbersNouns[hundred] + 'vatlh';
  		    }
  		    if (ten > 0) {
  		      word += (word !== '' ? ' ' : '') + numbersNouns[ten] + 'maH';
  		    }
  		    if (one > 0) {
  		      word += (word !== '' ? ' ' : '') + numbersNouns[one];
  		    }
  		    return word === '' ? 'pagh' : word;
  		  }
  		  hooks.defineLocale('tlh', {
  		    months: 'tera’ jar wa’_tera’ jar cha’_tera’ jar wej_tera’ jar loS_tera’ jar vagh_tera’ jar jav_tera’ jar Soch_tera’ jar chorgh_tera’ jar Hut_tera’ jar wa’maH_tera’ jar wa’maH wa’_tera’ jar wa’maH cha’'.split('_'),
  		    monthsShort: 'jar wa’_jar cha’_jar wej_jar loS_jar vagh_jar jav_jar Soch_jar chorgh_jar Hut_jar wa’maH_jar wa’maH wa’_jar wa’maH cha’'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'lojmItjaj_DaSjaj_povjaj_ghItlhjaj_loghjaj_buqjaj_ghInjaj'.split('_'),
  		    weekdaysShort: 'lojmItjaj_DaSjaj_povjaj_ghItlhjaj_loghjaj_buqjaj_ghInjaj'.split('_'),
  		    weekdaysMin: 'lojmItjaj_DaSjaj_povjaj_ghItlhjaj_loghjaj_buqjaj_ghInjaj'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd, D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[DaHjaj] LT',
  		      nextDay: '[wa’leS] LT',
  		      nextWeek: 'LLL',
  		      lastDay: '[wa’Hu’] LT',
  		      lastWeek: 'LLL',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: translateFuture,
  		      past: translatePast,
  		      s: 'puS lup',
  		      ss: translate$a,
  		      m: 'wa’ tup',
  		      mm: translate$a,
  		      h: 'wa’ rep',
  		      hh: translate$a,
  		      d: 'wa’ jaj',
  		      dd: translate$a,
  		      M: 'wa’ jar',
  		      MM: translate$a,
  		      y: 'wa’ DIS',
  		      yy: translate$a
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}\./,
  		    ordinal: '%d.',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var suffixes$5 = {
  		    1: "'inci",
  		    5: "'inci",
  		    8: "'inci",
  		    70: "'inci",
  		    80: "'inci",
  		    2: "'nci",
  		    7: "'nci",
  		    20: "'nci",
  		    50: "'nci",
  		    3: "'üncü",
  		    4: "'üncü",
  		    100: "'üncü",
  		    6: "'ncı",
  		    9: "'uncu",
  		    10: "'uncu",
  		    30: "'uncu",
  		    60: "'ıncı",
  		    90: "'ıncı"
  		  };
  		  hooks.defineLocale('tr', {
  		    months: 'Ocak_Şubat_Mart_Nisan_Mayıs_Haziran_Temmuz_Ağustos_Eylül_Ekim_Kasım_Aralık'.split('_'),
  		    monthsShort: 'Oca_Şub_Mar_Nis_May_Haz_Tem_Ağu_Eyl_Eki_Kas_Ara'.split('_'),
  		    weekdays: 'Pazar_Pazartesi_Salı_Çarşamba_Perşembe_Cuma_Cumartesi'.split('_'),
  		    weekdaysShort: 'Paz_Pzt_Sal_Çar_Per_Cum_Cmt'.split('_'),
  		    weekdaysMin: 'Pz_Pt_Sa_Ça_Pe_Cu_Ct'.split('_'),
  		    meridiem: function (hours, minutes, isLower) {
  		      if (hours < 12) {
  		        return isLower ? 'öö' : 'ÖÖ';
  		      } else {
  		        return isLower ? 'ös' : 'ÖS';
  		      }
  		    },
  		    meridiemParse: /öö|ÖÖ|ös|ÖS/,
  		    isPM: function (input) {
  		      return input === 'ös' || input === 'ÖS';
  		    },
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd, D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[bugün saat] LT',
  		      nextDay: '[yarın saat] LT',
  		      nextWeek: '[gelecek] dddd [saat] LT',
  		      lastDay: '[dün] LT',
  		      lastWeek: '[geçen] dddd [saat] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%s sonra',
  		      past: '%s önce',
  		      s: 'birkaç saniye',
  		      ss: '%d saniye',
  		      m: 'bir dakika',
  		      mm: '%d dakika',
  		      h: 'bir saat',
  		      hh: '%d saat',
  		      d: 'bir gün',
  		      dd: '%d gün',
  		      w: 'bir hafta',
  		      ww: '%d hafta',
  		      M: 'bir ay',
  		      MM: '%d ay',
  		      y: 'bir yıl',
  		      yy: '%d yıl'
  		    },
  		    ordinal: function (number, period) {
  		      switch (period) {
  		        case 'd':
  		        case 'D':
  		        case 'Do':
  		        case 'DD':
  		          return number;
  		        default:
  		          if (number === 0) {
  		            // special case for zero
  		            return number + "'ıncı";
  		          }
  		          var a = number % 10,
  		            b = number % 100 - a,
  		            c = number >= 100 ? 100 : null;
  		          return number + (suffixes$5[a] || suffixes$5[b] || suffixes$5[c]);
  		      }
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 7 // The week that contains Jan 7th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  // After the year there should be a slash and the amount of years since December 26, 1979 in Roman numerals.
  		  // This is currently too difficult (maybe even impossible) to add.
  		  hooks.defineLocale('tzl', {
  		    months: 'Januar_Fevraglh_Març_Avrïu_Mai_Gün_Julia_Guscht_Setemvar_Listopäts_Noemvar_Zecemvar'.split('_'),
  		    monthsShort: 'Jan_Fev_Mar_Avr_Mai_Gün_Jul_Gus_Set_Lis_Noe_Zec'.split('_'),
  		    weekdays: 'Súladi_Lúneçi_Maitzi_Márcuri_Xhúadi_Viénerçi_Sáturi'.split('_'),
  		    weekdaysShort: 'Súl_Lún_Mai_Már_Xhú_Vié_Sát'.split('_'),
  		    weekdaysMin: 'Sú_Lú_Ma_Má_Xh_Vi_Sá'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH.mm',
  		      LTS: 'HH.mm.ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'D. MMMM [dallas] YYYY',
  		      LLL: 'D. MMMM [dallas] YYYY HH.mm',
  		      LLLL: 'dddd, [li] D. MMMM [dallas] YYYY HH.mm'
  		    },
  		    meridiemParse: /d\'o|d\'a/i,
  		    isPM: function (input) {
  		      return "d'o" === input.toLowerCase();
  		    },
  		    meridiem: function (hours, minutes, isLower) {
  		      if (hours > 11) {
  		        return isLower ? "d'o" : "D'O";
  		      } else {
  		        return isLower ? "d'a" : "D'A";
  		      }
  		    },
  		    calendar: {
  		      sameDay: '[oxhi à] LT',
  		      nextDay: '[demà à] LT',
  		      nextWeek: 'dddd [à] LT',
  		      lastDay: '[ieiri à] LT',
  		      lastWeek: '[sür el] dddd [lasteu à] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'osprei %s',
  		      past: 'ja%s',
  		      s: processRelativeTime$a,
  		      ss: processRelativeTime$a,
  		      m: processRelativeTime$a,
  		      mm: processRelativeTime$a,
  		      h: processRelativeTime$a,
  		      hh: processRelativeTime$a,
  		      d: processRelativeTime$a,
  		      dd: processRelativeTime$a,
  		      M: processRelativeTime$a,
  		      MM: processRelativeTime$a,
  		      y: processRelativeTime$a,
  		      yy: processRelativeTime$a
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}\./,
  		    ordinal: '%d.',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });
  		  function processRelativeTime$a(number, withoutSuffix, key, isFuture) {
  		    var format = {
  		      s: ['viensas secunds', "'iensas secunds"],
  		      ss: [number + ' secunds', '' + number + ' secunds'],
  		      m: ["'n míut", "'iens míut"],
  		      mm: [number + ' míuts', '' + number + ' míuts'],
  		      h: ["'n þora", "'iensa þora"],
  		      hh: [number + ' þoras', '' + number + ' þoras'],
  		      d: ["'n ziua", "'iensa ziua"],
  		      dd: [number + ' ziuas', '' + number + ' ziuas'],
  		      M: ["'n mes", "'iens mes"],
  		      MM: [number + ' mesen', '' + number + ' mesen'],
  		      y: ["'n ar", "'iens ar"],
  		      yy: [number + ' ars', '' + number + ' ars']
  		    };
  		    return isFuture ? format[key][0] : withoutSuffix ? format[key][0] : format[key][1];
  		  }

  		  //! moment.js locale configuration

  		  hooks.defineLocale('tzm-latn', {
  		    months: 'innayr_brˤayrˤ_marˤsˤ_ibrir_mayyw_ywnyw_ywlywz_ɣwšt_šwtanbir_ktˤwbrˤ_nwwanbir_dwjnbir'.split('_'),
  		    monthsShort: 'innayr_brˤayrˤ_marˤsˤ_ibrir_mayyw_ywnyw_ywlywz_ɣwšt_šwtanbir_ktˤwbrˤ_nwwanbir_dwjnbir'.split('_'),
  		    weekdays: 'asamas_aynas_asinas_akras_akwas_asimwas_asiḍyas'.split('_'),
  		    weekdaysShort: 'asamas_aynas_asinas_akras_akwas_asimwas_asiḍyas'.split('_'),
  		    weekdaysMin: 'asamas_aynas_asinas_akras_akwas_asimwas_asiḍyas'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[asdkh g] LT',
  		      nextDay: '[aska g] LT',
  		      nextWeek: 'dddd [g] LT',
  		      lastDay: '[assant g] LT',
  		      lastWeek: 'dddd [g] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'dadkh s yan %s',
  		      past: 'yan %s',
  		      s: 'imik',
  		      ss: '%d imik',
  		      m: 'minuḍ',
  		      mm: '%d minuḍ',
  		      h: 'saɛa',
  		      hh: '%d tassaɛin',
  		      d: 'ass',
  		      dd: '%d ossan',
  		      M: 'ayowr',
  		      MM: '%d iyyirn',
  		      y: 'asgas',
  		      yy: '%d isgasn'
  		    },
  		    week: {
  		      dow: 6,
  		      // Saturday is the first day of the week.
  		      doy: 12 // The week that contains Jan 12th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('tzm', {
  		    months: 'ⵉⵏⵏⴰⵢⵔ_ⴱⵕⴰⵢⵕ_ⵎⴰⵕⵚ_ⵉⴱⵔⵉⵔ_ⵎⴰⵢⵢⵓ_ⵢⵓⵏⵢⵓ_ⵢⵓⵍⵢⵓⵣ_ⵖⵓⵛⵜ_ⵛⵓⵜⴰⵏⴱⵉⵔ_ⴽⵟⵓⴱⵕ_ⵏⵓⵡⴰⵏⴱⵉⵔ_ⴷⵓⵊⵏⴱⵉⵔ'.split('_'),
  		    monthsShort: 'ⵉⵏⵏⴰⵢⵔ_ⴱⵕⴰⵢⵕ_ⵎⴰⵕⵚ_ⵉⴱⵔⵉⵔ_ⵎⴰⵢⵢⵓ_ⵢⵓⵏⵢⵓ_ⵢⵓⵍⵢⵓⵣ_ⵖⵓⵛⵜ_ⵛⵓⵜⴰⵏⴱⵉⵔ_ⴽⵟⵓⴱⵕ_ⵏⵓⵡⴰⵏⴱⵉⵔ_ⴷⵓⵊⵏⴱⵉⵔ'.split('_'),
  		    weekdays: 'ⴰⵙⴰⵎⴰⵙ_ⴰⵢⵏⴰⵙ_ⴰⵙⵉⵏⴰⵙ_ⴰⴽⵔⴰⵙ_ⴰⴽⵡⴰⵙ_ⴰⵙⵉⵎⵡⴰⵙ_ⴰⵙⵉⴹⵢⴰⵙ'.split('_'),
  		    weekdaysShort: 'ⴰⵙⴰⵎⴰⵙ_ⴰⵢⵏⴰⵙ_ⴰⵙⵉⵏⴰⵙ_ⴰⴽⵔⴰⵙ_ⴰⴽⵡⴰⵙ_ⴰⵙⵉⵎⵡⴰⵙ_ⴰⵙⵉⴹⵢⴰⵙ'.split('_'),
  		    weekdaysMin: 'ⴰⵙⴰⵎⴰⵙ_ⴰⵢⵏⴰⵙ_ⴰⵙⵉⵏⴰⵙ_ⴰⴽⵔⴰⵙ_ⴰⴽⵡⴰⵙ_ⴰⵙⵉⵎⵡⴰⵙ_ⴰⵙⵉⴹⵢⴰⵙ'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[ⴰⵙⴷⵅ ⴴ] LT',
  		      nextDay: '[ⴰⵙⴽⴰ ⴴ] LT',
  		      nextWeek: 'dddd [ⴴ] LT',
  		      lastDay: '[ⴰⵚⴰⵏⵜ ⴴ] LT',
  		      lastWeek: 'dddd [ⴴ] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'ⴷⴰⴷⵅ ⵙ ⵢⴰⵏ %s',
  		      past: 'ⵢⴰⵏ %s',
  		      s: 'ⵉⵎⵉⴽ',
  		      ss: '%d ⵉⵎⵉⴽ',
  		      m: 'ⵎⵉⵏⵓⴺ',
  		      mm: '%d ⵎⵉⵏⵓⴺ',
  		      h: 'ⵙⴰⵄⴰ',
  		      hh: '%d ⵜⴰⵙⵙⴰⵄⵉⵏ',
  		      d: 'ⴰⵙⵙ',
  		      dd: '%d oⵙⵙⴰⵏ',
  		      M: 'ⴰⵢoⵓⵔ',
  		      MM: '%d ⵉⵢⵢⵉⵔⵏ',
  		      y: 'ⴰⵙⴳⴰⵙ',
  		      yy: '%d ⵉⵙⴳⴰⵙⵏ'
  		    },
  		    week: {
  		      dow: 6,
  		      // Saturday is the first day of the week.
  		      doy: 12 // The week that contains Jan 12th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('ug-cn', {
  		    months: 'يانۋار_فېۋرال_مارت_ئاپرېل_ماي_ئىيۇن_ئىيۇل_ئاۋغۇست_سېنتەبىر_ئۆكتەبىر_نويابىر_دېكابىر'.split('_'),
  		    monthsShort: 'يانۋار_فېۋرال_مارت_ئاپرېل_ماي_ئىيۇن_ئىيۇل_ئاۋغۇست_سېنتەبىر_ئۆكتەبىر_نويابىر_دېكابىر'.split('_'),
  		    weekdays: 'يەكشەنبە_دۈشەنبە_سەيشەنبە_چارشەنبە_پەيشەنبە_جۈمە_شەنبە'.split('_'),
  		    weekdaysShort: 'يە_دۈ_سە_چا_پە_جۈ_شە'.split('_'),
  		    weekdaysMin: 'يە_دۈ_سە_چا_پە_جۈ_شە'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'YYYY-MM-DD',
  		      LL: 'YYYY-يىلىM-ئاينىڭD-كۈنى',
  		      LLL: 'YYYY-يىلىM-ئاينىڭD-كۈنى، HH:mm',
  		      LLLL: 'dddd، YYYY-يىلىM-ئاينىڭD-كۈنى، HH:mm'
  		    },
  		    meridiemParse: /يېرىم كېچە|سەھەر|چۈشتىن بۇرۇن|چۈش|چۈشتىن كېيىن|كەچ/,
  		    meridiemHour: function (hour, meridiem) {
  		      if (hour === 12) {
  		        hour = 0;
  		      }
  		      if (meridiem === 'يېرىم كېچە' || meridiem === 'سەھەر' || meridiem === 'چۈشتىن بۇرۇن') {
  		        return hour;
  		      } else if (meridiem === 'چۈشتىن كېيىن' || meridiem === 'كەچ') {
  		        return hour + 12;
  		      } else {
  		        return hour >= 11 ? hour : hour + 12;
  		      }
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      var hm = hour * 100 + minute;
  		      if (hm < 600) {
  		        return 'يېرىم كېچە';
  		      } else if (hm < 900) {
  		        return 'سەھەر';
  		      } else if (hm < 1130) {
  		        return 'چۈشتىن بۇرۇن';
  		      } else if (hm < 1230) {
  		        return 'چۈش';
  		      } else if (hm < 1800) {
  		        return 'چۈشتىن كېيىن';
  		      } else {
  		        return 'كەچ';
  		      }
  		    },
  		    calendar: {
  		      sameDay: '[بۈگۈن سائەت] LT',
  		      nextDay: '[ئەتە سائەت] LT',
  		      nextWeek: '[كېلەركى] dddd [سائەت] LT',
  		      lastDay: '[تۆنۈگۈن] LT',
  		      lastWeek: '[ئالدىنقى] dddd [سائەت] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%s كېيىن',
  		      past: '%s بۇرۇن',
  		      s: 'نەچچە سېكونت',
  		      ss: '%d سېكونت',
  		      m: 'بىر مىنۇت',
  		      mm: '%d مىنۇت',
  		      h: 'بىر سائەت',
  		      hh: '%d سائەت',
  		      d: 'بىر كۈن',
  		      dd: '%d كۈن',
  		      M: 'بىر ئاي',
  		      MM: '%d ئاي',
  		      y: 'بىر يىل',
  		      yy: '%d يىل'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(-كۈنى|-ئاي|-ھەپتە)/,
  		    ordinal: function (number, period) {
  		      switch (period) {
  		        case 'd':
  		        case 'D':
  		        case 'DDD':
  		          return number + '-كۈنى';
  		        case 'w':
  		        case 'W':
  		          return number + '-ھەپتە';
  		        default:
  		          return number;
  		      }
  		    },
  		    preparse: function (string) {
  		      return string.replace(/،/g, ',');
  		    },
  		    postformat: function (string) {
  		      return string.replace(/,/g, '،');
  		    },
  		    week: {
  		      // GB/T 7408-1994《数据元和交换格式·信息交换·日期和时间表示法》与ISO 8601:1988等效
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 7 // The week that contains Jan 1st is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  function plural$6(word, num) {
  		    var forms = word.split('_');
  		    return num % 10 === 1 && num % 100 !== 11 ? forms[0] : num % 10 >= 2 && num % 10 <= 4 && (num % 100 < 10 || num % 100 >= 20) ? forms[1] : forms[2];
  		  }
  		  function relativeTimeWithPlural$4(number, withoutSuffix, key) {
  		    var format = {
  		      ss: withoutSuffix ? 'секунда_секунди_секунд' : 'секунду_секунди_секунд',
  		      mm: withoutSuffix ? 'хвилина_хвилини_хвилин' : 'хвилину_хвилини_хвилин',
  		      hh: withoutSuffix ? 'година_години_годин' : 'годину_години_годин',
  		      dd: 'день_дні_днів',
  		      MM: 'місяць_місяці_місяців',
  		      yy: 'рік_роки_років'
  		    };
  		    if (key === 'm') {
  		      return withoutSuffix ? 'хвилина' : 'хвилину';
  		    } else if (key === 'h') {
  		      return withoutSuffix ? 'година' : 'годину';
  		    } else {
  		      return number + ' ' + plural$6(format[key], +number);
  		    }
  		  }
  		  function weekdaysCaseReplace(m, format) {
  		    var weekdays = {
  		        nominative: 'неділя_понеділок_вівторок_середа_четвер_п’ятниця_субота'.split('_'),
  		        accusative: 'неділю_понеділок_вівторок_середу_четвер_п’ятницю_суботу'.split('_'),
  		        genitive: 'неділі_понеділка_вівторка_середи_четверга_п’ятниці_суботи'.split('_')
  		      },
  		      nounCase;
  		    if (m === true) {
  		      return weekdays['nominative'].slice(1, 7).concat(weekdays['nominative'].slice(0, 1));
  		    }
  		    if (!m) {
  		      return weekdays['nominative'];
  		    }
  		    nounCase = /(\[[ВвУу]\]) ?dddd/.test(format) ? 'accusative' : /\[?(?:минулої|наступної)? ?\] ?dddd/.test(format) ? 'genitive' : 'nominative';
  		    return weekdays[nounCase][m.day()];
  		  }
  		  function processHoursFunction(str) {
  		    return function () {
  		      return str + 'о' + (this.hours() === 11 ? 'б' : '') + '] LT';
  		    };
  		  }
  		  hooks.defineLocale('uk', {
  		    months: {
  		      format: 'січня_лютого_березня_квітня_травня_червня_липня_серпня_вересня_жовтня_листопада_грудня'.split('_'),
  		      standalone: 'січень_лютий_березень_квітень_травень_червень_липень_серпень_вересень_жовтень_листопад_грудень'.split('_')
  		    },
  		    monthsShort: 'січ_лют_бер_квіт_трав_черв_лип_серп_вер_жовт_лист_груд'.split('_'),
  		    weekdays: weekdaysCaseReplace,
  		    weekdaysShort: 'нд_пн_вт_ср_чт_пт_сб'.split('_'),
  		    weekdaysMin: 'нд_пн_вт_ср_чт_пт_сб'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD.MM.YYYY',
  		      LL: 'D MMMM YYYY р.',
  		      LLL: 'D MMMM YYYY р., HH:mm',
  		      LLLL: 'dddd, D MMMM YYYY р., HH:mm'
  		    },
  		    calendar: {
  		      sameDay: processHoursFunction('[Сьогодні '),
  		      nextDay: processHoursFunction('[Завтра '),
  		      lastDay: processHoursFunction('[Вчора '),
  		      nextWeek: processHoursFunction('[У] dddd ['),
  		      lastWeek: function () {
  		        switch (this.day()) {
  		          case 0:
  		          case 3:
  		          case 5:
  		          case 6:
  		            return processHoursFunction('[Минулої] dddd [').call(this);
  		          case 1:
  		          case 2:
  		          case 4:
  		            return processHoursFunction('[Минулого] dddd [').call(this);
  		        }
  		      },
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'за %s',
  		      past: '%s тому',
  		      s: 'декілька секунд',
  		      ss: relativeTimeWithPlural$4,
  		      m: relativeTimeWithPlural$4,
  		      mm: relativeTimeWithPlural$4,
  		      h: 'годину',
  		      hh: relativeTimeWithPlural$4,
  		      d: 'день',
  		      dd: relativeTimeWithPlural$4,
  		      M: 'місяць',
  		      MM: relativeTimeWithPlural$4,
  		      y: 'рік',
  		      yy: relativeTimeWithPlural$4
  		    },
  		    // M. E.: those two are virtually unused but a user might want to implement them for his/her website for some reason
  		    meridiemParse: /ночі|ранку|дня|вечора/,
  		    isPM: function (input) {
  		      return /^(дня|вечора)$/.test(input);
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 4) {
  		        return 'ночі';
  		      } else if (hour < 12) {
  		        return 'ранку';
  		      } else if (hour < 17) {
  		        return 'дня';
  		      } else {
  		        return 'вечора';
  		      }
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}-(й|го)/,
  		    ordinal: function (number, period) {
  		      switch (period) {
  		        case 'M':
  		        case 'd':
  		        case 'DDD':
  		        case 'w':
  		        case 'W':
  		          return number + '-й';
  		        case 'D':
  		          return number + '-го';
  		        default:
  		          return number;
  		      }
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 7 // The week that contains Jan 7th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  var months$b = ['جنوری', 'فروری', 'مارچ', 'اپریل', 'مئی', 'جون', 'جولائی', 'اگست', 'ستمبر', 'اکتوبر', 'نومبر', 'دسمبر'],
  		    days$2 = ['اتوار', 'پیر', 'منگل', 'بدھ', 'جمعرات', 'جمعہ', 'ہفتہ'];
  		  hooks.defineLocale('ur', {
  		    months: months$b,
  		    monthsShort: months$b,
  		    weekdays: days$2,
  		    weekdaysShort: days$2,
  		    weekdaysMin: days$2,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd، D MMMM YYYY HH:mm'
  		    },
  		    meridiemParse: /صبح|شام/,
  		    isPM: function (input) {
  		      return 'شام' === input;
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      if (hour < 12) {
  		        return 'صبح';
  		      }
  		      return 'شام';
  		    },
  		    calendar: {
  		      sameDay: '[آج بوقت] LT',
  		      nextDay: '[کل بوقت] LT',
  		      nextWeek: 'dddd [بوقت] LT',
  		      lastDay: '[گذشتہ روز بوقت] LT',
  		      lastWeek: '[گذشتہ] dddd [بوقت] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%s بعد',
  		      past: '%s قبل',
  		      s: 'چند سیکنڈ',
  		      ss: '%d سیکنڈ',
  		      m: 'ایک منٹ',
  		      mm: '%d منٹ',
  		      h: 'ایک گھنٹہ',
  		      hh: '%d گھنٹے',
  		      d: 'ایک دن',
  		      dd: '%d دن',
  		      M: 'ایک ماہ',
  		      MM: '%d ماہ',
  		      y: 'ایک سال',
  		      yy: '%d سال'
  		    },
  		    preparse: function (string) {
  		      return string.replace(/،/g, ',');
  		    },
  		    postformat: function (string) {
  		      return string.replace(/,/g, '،');
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('uz-latn', {
  		    months: 'Yanvar_Fevral_Mart_Aprel_May_Iyun_Iyul_Avgust_Sentabr_Oktabr_Noyabr_Dekabr'.split('_'),
  		    monthsShort: 'Yan_Fev_Mar_Apr_May_Iyun_Iyul_Avg_Sen_Okt_Noy_Dek'.split('_'),
  		    weekdays: 'Yakshanba_Dushanba_Seshanba_Chorshanba_Payshanba_Juma_Shanba'.split('_'),
  		    weekdaysShort: 'Yak_Dush_Sesh_Chor_Pay_Jum_Shan'.split('_'),
  		    weekdaysMin: 'Ya_Du_Se_Cho_Pa_Ju_Sha'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'D MMMM YYYY, dddd HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Bugun soat] LT [da]',
  		      nextDay: '[Ertaga] LT [da]',
  		      nextWeek: 'dddd [kuni soat] LT [da]',
  		      lastDay: '[Kecha soat] LT [da]',
  		      lastWeek: "[O'tgan] dddd [kuni soat] LT [da]",
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'Yaqin %s ichida',
  		      past: 'Bir necha %s oldin',
  		      s: 'soniya',
  		      ss: '%d soniya',
  		      m: 'bir daqiqa',
  		      mm: '%d daqiqa',
  		      h: 'bir soat',
  		      hh: '%d soat',
  		      d: 'bir kun',
  		      dd: '%d kun',
  		      M: 'bir oy',
  		      MM: '%d oy',
  		      y: 'bir yil',
  		      yy: '%d yil'
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 7 // The week that contains Jan 7th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('uz', {
  		    months: 'январ_феврал_март_апрел_май_июн_июл_август_сентябр_октябр_ноябр_декабр'.split('_'),
  		    monthsShort: 'янв_фев_мар_апр_май_июн_июл_авг_сен_окт_ноя_дек'.split('_'),
  		    weekdays: 'Якшанба_Душанба_Сешанба_Чоршанба_Пайшанба_Жума_Шанба'.split('_'),
  		    weekdaysShort: 'Якш_Душ_Сеш_Чор_Пай_Жум_Шан'.split('_'),
  		    weekdaysMin: 'Як_Ду_Се_Чо_Па_Жу_Ша'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'D MMMM YYYY, dddd HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Бугун соат] LT [да]',
  		      nextDay: '[Эртага] LT [да]',
  		      nextWeek: 'dddd [куни соат] LT [да]',
  		      lastDay: '[Кеча соат] LT [да]',
  		      lastWeek: '[Утган] dddd [куни соат] LT [да]',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'Якин %s ичида',
  		      past: 'Бир неча %s олдин',
  		      s: 'фурсат',
  		      ss: '%d фурсат',
  		      m: 'бир дакика',
  		      mm: '%d дакика',
  		      h: 'бир соат',
  		      hh: '%d соат',
  		      d: 'бир кун',
  		      dd: '%d кун',
  		      M: 'бир ой',
  		      MM: '%d ой',
  		      y: 'бир йил',
  		      yy: '%d йил'
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 7 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('vi', {
  		    months: 'tháng 1_tháng 2_tháng 3_tháng 4_tháng 5_tháng 6_tháng 7_tháng 8_tháng 9_tháng 10_tháng 11_tháng 12'.split('_'),
  		    monthsShort: 'Thg 01_Thg 02_Thg 03_Thg 04_Thg 05_Thg 06_Thg 07_Thg 08_Thg 09_Thg 10_Thg 11_Thg 12'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'chủ nhật_thứ hai_thứ ba_thứ tư_thứ năm_thứ sáu_thứ bảy'.split('_'),
  		    weekdaysShort: 'CN_T2_T3_T4_T5_T6_T7'.split('_'),
  		    weekdaysMin: 'CN_T2_T3_T4_T5_T6_T7'.split('_'),
  		    weekdaysParseExact: true,
  		    meridiemParse: /sa|ch/i,
  		    isPM: function (input) {
  		      return /^ch$/i.test(input);
  		    },
  		    meridiem: function (hours, minutes, isLower) {
  		      if (hours < 12) {
  		        return isLower ? 'sa' : 'SA';
  		      } else {
  		        return isLower ? 'ch' : 'CH';
  		      }
  		    },
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM [năm] YYYY',
  		      LLL: 'D MMMM [năm] YYYY HH:mm',
  		      LLLL: 'dddd, D MMMM [năm] YYYY HH:mm',
  		      l: 'DD/M/YYYY',
  		      ll: 'D MMM YYYY',
  		      lll: 'D MMM YYYY HH:mm',
  		      llll: 'ddd, D MMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[Hôm nay lúc] LT',
  		      nextDay: '[Ngày mai lúc] LT',
  		      nextWeek: 'dddd [tuần tới lúc] LT',
  		      lastDay: '[Hôm qua lúc] LT',
  		      lastWeek: 'dddd [tuần trước lúc] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: '%s tới',
  		      past: '%s trước',
  		      s: 'vài giây',
  		      ss: '%d giây',
  		      m: 'một phút',
  		      mm: '%d phút',
  		      h: 'một giờ',
  		      hh: '%d giờ',
  		      d: 'một ngày',
  		      dd: '%d ngày',
  		      w: 'một tuần',
  		      ww: '%d tuần',
  		      M: 'một tháng',
  		      MM: '%d tháng',
  		      y: 'một năm',
  		      yy: '%d năm'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}/,
  		    ordinal: function (number) {
  		      return number;
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('x-pseudo', {
  		    months: 'J~áñúá~rý_F~ébrú~árý_~Márc~h_Áp~ríl_~Máý_~Júñé~_Júl~ý_Áú~gúst~_Sép~témb~ér_Ó~ctób~ér_Ñ~óvém~bér_~Décé~mbér'.split('_'),
  		    monthsShort: 'J~áñ_~Féb_~Már_~Ápr_~Máý_~Júñ_~Júl_~Áúg_~Sép_~Óct_~Ñóv_~Déc'.split('_'),
  		    monthsParseExact: true,
  		    weekdays: 'S~úñdá~ý_Mó~ñdáý~_Túé~sdáý~_Wéd~ñésd~áý_T~húrs~dáý_~Fríd~áý_S~átúr~dáý'.split('_'),
  		    weekdaysShort: 'S~úñ_~Móñ_~Túé_~Wéd_~Thú_~Frí_~Sát'.split('_'),
  		    weekdaysMin: 'S~ú_Mó~_Tú_~Wé_T~h_Fr~_Sá'.split('_'),
  		    weekdaysParseExact: true,
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY HH:mm',
  		      LLLL: 'dddd, D MMMM YYYY HH:mm'
  		    },
  		    calendar: {
  		      sameDay: '[T~ódá~ý át] LT',
  		      nextDay: '[T~ómó~rró~w át] LT',
  		      nextWeek: 'dddd [át] LT',
  		      lastDay: '[Ý~ést~érdá~ý át] LT',
  		      lastWeek: '[L~ást] dddd [át] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'í~ñ %s',
  		      past: '%s á~gó',
  		      s: 'á ~féw ~sécó~ñds',
  		      ss: '%d s~écóñ~ds',
  		      m: 'á ~míñ~úté',
  		      mm: '%d m~íñú~tés',
  		      h: 'á~ñ hó~úr',
  		      hh: '%d h~óúrs',
  		      d: 'á ~dáý',
  		      dd: '%d d~áýs',
  		      M: 'á ~móñ~th',
  		      MM: '%d m~óñt~hs',
  		      y: 'á ~ýéár',
  		      yy: '%d ý~éárs'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(th|st|nd|rd)/,
  		    ordinal: function (number) {
  		      var b = number % 10,
  		        output = ~~(number % 100 / 10) === 1 ? 'th' : b === 1 ? 'st' : b === 2 ? 'nd' : b === 3 ? 'rd' : 'th';
  		      return number + output;
  		    },
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('yo', {
  		    months: 'Sẹ́rẹ́_Èrèlè_Ẹrẹ̀nà_Ìgbé_Èbibi_Òkùdu_Agẹmo_Ògún_Owewe_Ọ̀wàrà_Bélú_Ọ̀pẹ̀̀'.split('_'),
  		    monthsShort: 'Sẹ́r_Èrl_Ẹrn_Ìgb_Èbi_Òkù_Agẹ_Ògú_Owe_Ọ̀wà_Bél_Ọ̀pẹ̀̀'.split('_'),
  		    weekdays: 'Àìkú_Ajé_Ìsẹ́gun_Ọjọ́rú_Ọjọ́bọ_Ẹtì_Àbámẹ́ta'.split('_'),
  		    weekdaysShort: 'Àìk_Ajé_Ìsẹ́_Ọjr_Ọjb_Ẹtì_Àbá'.split('_'),
  		    weekdaysMin: 'Àì_Aj_Ìs_Ọr_Ọb_Ẹt_Àb'.split('_'),
  		    longDateFormat: {
  		      LT: 'h:mm A',
  		      LTS: 'h:mm:ss A',
  		      L: 'DD/MM/YYYY',
  		      LL: 'D MMMM YYYY',
  		      LLL: 'D MMMM YYYY h:mm A',
  		      LLLL: 'dddd, D MMMM YYYY h:mm A'
  		    },
  		    calendar: {
  		      sameDay: '[Ònì ni] LT',
  		      nextDay: '[Ọ̀la ni] LT',
  		      nextWeek: "dddd [Ọsẹ̀ tón'bọ] [ni] LT",
  		      lastDay: '[Àna ni] LT',
  		      lastWeek: 'dddd [Ọsẹ̀ tólọ́] [ni] LT',
  		      sameElse: 'L'
  		    },
  		    relativeTime: {
  		      future: 'ní %s',
  		      past: '%s kọjá',
  		      s: 'ìsẹjú aayá die',
  		      ss: 'aayá %d',
  		      m: 'ìsẹjú kan',
  		      mm: 'ìsẹjú %d',
  		      h: 'wákati kan',
  		      hh: 'wákati %d',
  		      d: 'ọjọ́ kan',
  		      dd: 'ọjọ́ %d',
  		      M: 'osù kan',
  		      MM: 'osù %d',
  		      y: 'ọdún kan',
  		      yy: 'ọdún %d'
  		    },
  		    dayOfMonthOrdinalParse: /ọjọ́\s\d{1,2}/,
  		    ordinal: 'ọjọ́ %d',
  		    week: {
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('zh-cn', {
  		    months: '一月_二月_三月_四月_五月_六月_七月_八月_九月_十月_十一月_十二月'.split('_'),
  		    monthsShort: '1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月'.split('_'),
  		    weekdays: '星期日_星期一_星期二_星期三_星期四_星期五_星期六'.split('_'),
  		    weekdaysShort: '周日_周一_周二_周三_周四_周五_周六'.split('_'),
  		    weekdaysMin: '日_一_二_三_四_五_六'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'YYYY/MM/DD',
  		      LL: 'YYYY年M月D日',
  		      LLL: 'YYYY年M月D日Ah点mm分',
  		      LLLL: 'YYYY年M月D日ddddAh点mm分',
  		      l: 'YYYY/M/D',
  		      ll: 'YYYY年M月D日',
  		      lll: 'YYYY年M月D日 HH:mm',
  		      llll: 'YYYY年M月D日dddd HH:mm'
  		    },
  		    meridiemParse: /凌晨|早上|上午|中午|下午|晚上/,
  		    meridiemHour: function (hour, meridiem) {
  		      if (hour === 12) {
  		        hour = 0;
  		      }
  		      if (meridiem === '凌晨' || meridiem === '早上' || meridiem === '上午') {
  		        return hour;
  		      } else if (meridiem === '下午' || meridiem === '晚上') {
  		        return hour + 12;
  		      } else {
  		        // '中午'
  		        return hour >= 11 ? hour : hour + 12;
  		      }
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      var hm = hour * 100 + minute;
  		      if (hm < 600) {
  		        return '凌晨';
  		      } else if (hm < 900) {
  		        return '早上';
  		      } else if (hm < 1130) {
  		        return '上午';
  		      } else if (hm < 1230) {
  		        return '中午';
  		      } else if (hm < 1800) {
  		        return '下午';
  		      } else {
  		        return '晚上';
  		      }
  		    },
  		    calendar: {
  		      sameDay: '[今天]LT',
  		      nextDay: '[明天]LT',
  		      nextWeek: function (now) {
  		        if (now.week() !== this.week()) {
  		          return '[下]dddLT';
  		        } else {
  		          return '[本]dddLT';
  		        }
  		      },
  		      lastDay: '[昨天]LT',
  		      lastWeek: function (now) {
  		        if (this.week() !== now.week()) {
  		          return '[上]dddLT';
  		        } else {
  		          return '[本]dddLT';
  		        }
  		      },
  		      sameElse: 'L'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(日|月|周)/,
  		    ordinal: function (number, period) {
  		      switch (period) {
  		        case 'd':
  		        case 'D':
  		        case 'DDD':
  		          return number + '日';
  		        case 'M':
  		          return number + '月';
  		        case 'w':
  		        case 'W':
  		          return number + '周';
  		        default:
  		          return number;
  		      }
  		    },
  		    relativeTime: {
  		      future: '%s后',
  		      past: '%s前',
  		      s: '几秒',
  		      ss: '%d 秒',
  		      m: '1 分钟',
  		      mm: '%d 分钟',
  		      h: '1 小时',
  		      hh: '%d 小时',
  		      d: '1 天',
  		      dd: '%d 天',
  		      w: '1 周',
  		      ww: '%d 周',
  		      M: '1 个月',
  		      MM: '%d 个月',
  		      y: '1 年',
  		      yy: '%d 年'
  		    },
  		    week: {
  		      // GB/T 7408-1994《数据元和交换格式·信息交换·日期和时间表示法》与ISO 8601:1988等效
  		      dow: 1,
  		      // Monday is the first day of the week.
  		      doy: 4 // The week that contains Jan 4th is the first week of the year.
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('zh-hk', {
  		    months: '一月_二月_三月_四月_五月_六月_七月_八月_九月_十月_十一月_十二月'.split('_'),
  		    monthsShort: '1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月'.split('_'),
  		    weekdays: '星期日_星期一_星期二_星期三_星期四_星期五_星期六'.split('_'),
  		    weekdaysShort: '週日_週一_週二_週三_週四_週五_週六'.split('_'),
  		    weekdaysMin: '日_一_二_三_四_五_六'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'YYYY/MM/DD',
  		      LL: 'YYYY年M月D日',
  		      LLL: 'YYYY年M月D日 HH:mm',
  		      LLLL: 'YYYY年M月D日dddd HH:mm',
  		      l: 'YYYY/M/D',
  		      ll: 'YYYY年M月D日',
  		      lll: 'YYYY年M月D日 HH:mm',
  		      llll: 'YYYY年M月D日dddd HH:mm'
  		    },
  		    meridiemParse: /凌晨|早上|上午|中午|下午|晚上/,
  		    meridiemHour: function (hour, meridiem) {
  		      if (hour === 12) {
  		        hour = 0;
  		      }
  		      if (meridiem === '凌晨' || meridiem === '早上' || meridiem === '上午') {
  		        return hour;
  		      } else if (meridiem === '中午') {
  		        return hour >= 11 ? hour : hour + 12;
  		      } else if (meridiem === '下午' || meridiem === '晚上') {
  		        return hour + 12;
  		      }
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      var hm = hour * 100 + minute;
  		      if (hm < 600) {
  		        return '凌晨';
  		      } else if (hm < 900) {
  		        return '早上';
  		      } else if (hm < 1200) {
  		        return '上午';
  		      } else if (hm === 1200) {
  		        return '中午';
  		      } else if (hm < 1800) {
  		        return '下午';
  		      } else {
  		        return '晚上';
  		      }
  		    },
  		    calendar: {
  		      sameDay: '[今天]LT',
  		      nextDay: '[明天]LT',
  		      nextWeek: '[下]ddddLT',
  		      lastDay: '[昨天]LT',
  		      lastWeek: '[上]ddddLT',
  		      sameElse: 'L'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(日|月|週)/,
  		    ordinal: function (number, period) {
  		      switch (period) {
  		        case 'd':
  		        case 'D':
  		        case 'DDD':
  		          return number + '日';
  		        case 'M':
  		          return number + '月';
  		        case 'w':
  		        case 'W':
  		          return number + '週';
  		        default:
  		          return number;
  		      }
  		    },
  		    relativeTime: {
  		      future: '%s後',
  		      past: '%s前',
  		      s: '幾秒',
  		      ss: '%d 秒',
  		      m: '1 分鐘',
  		      mm: '%d 分鐘',
  		      h: '1 小時',
  		      hh: '%d 小時',
  		      d: '1 天',
  		      dd: '%d 天',
  		      M: '1 個月',
  		      MM: '%d 個月',
  		      y: '1 年',
  		      yy: '%d 年'
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('zh-mo', {
  		    months: '一月_二月_三月_四月_五月_六月_七月_八月_九月_十月_十一月_十二月'.split('_'),
  		    monthsShort: '1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月'.split('_'),
  		    weekdays: '星期日_星期一_星期二_星期三_星期四_星期五_星期六'.split('_'),
  		    weekdaysShort: '週日_週一_週二_週三_週四_週五_週六'.split('_'),
  		    weekdaysMin: '日_一_二_三_四_五_六'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'DD/MM/YYYY',
  		      LL: 'YYYY年M月D日',
  		      LLL: 'YYYY年M月D日 HH:mm',
  		      LLLL: 'YYYY年M月D日dddd HH:mm',
  		      l: 'D/M/YYYY',
  		      ll: 'YYYY年M月D日',
  		      lll: 'YYYY年M月D日 HH:mm',
  		      llll: 'YYYY年M月D日dddd HH:mm'
  		    },
  		    meridiemParse: /凌晨|早上|上午|中午|下午|晚上/,
  		    meridiemHour: function (hour, meridiem) {
  		      if (hour === 12) {
  		        hour = 0;
  		      }
  		      if (meridiem === '凌晨' || meridiem === '早上' || meridiem === '上午') {
  		        return hour;
  		      } else if (meridiem === '中午') {
  		        return hour >= 11 ? hour : hour + 12;
  		      } else if (meridiem === '下午' || meridiem === '晚上') {
  		        return hour + 12;
  		      }
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      var hm = hour * 100 + minute;
  		      if (hm < 600) {
  		        return '凌晨';
  		      } else if (hm < 900) {
  		        return '早上';
  		      } else if (hm < 1130) {
  		        return '上午';
  		      } else if (hm < 1230) {
  		        return '中午';
  		      } else if (hm < 1800) {
  		        return '下午';
  		      } else {
  		        return '晚上';
  		      }
  		    },
  		    calendar: {
  		      sameDay: '[今天] LT',
  		      nextDay: '[明天] LT',
  		      nextWeek: '[下]dddd LT',
  		      lastDay: '[昨天] LT',
  		      lastWeek: '[上]dddd LT',
  		      sameElse: 'L'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(日|月|週)/,
  		    ordinal: function (number, period) {
  		      switch (period) {
  		        case 'd':
  		        case 'D':
  		        case 'DDD':
  		          return number + '日';
  		        case 'M':
  		          return number + '月';
  		        case 'w':
  		        case 'W':
  		          return number + '週';
  		        default:
  		          return number;
  		      }
  		    },
  		    relativeTime: {
  		      future: '%s內',
  		      past: '%s前',
  		      s: '幾秒',
  		      ss: '%d 秒',
  		      m: '1 分鐘',
  		      mm: '%d 分鐘',
  		      h: '1 小時',
  		      hh: '%d 小時',
  		      d: '1 天',
  		      dd: '%d 天',
  		      M: '1 個月',
  		      MM: '%d 個月',
  		      y: '1 年',
  		      yy: '%d 年'
  		    }
  		  });

  		  //! moment.js locale configuration

  		  hooks.defineLocale('zh-tw', {
  		    months: '一月_二月_三月_四月_五月_六月_七月_八月_九月_十月_十一月_十二月'.split('_'),
  		    monthsShort: '1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月'.split('_'),
  		    weekdays: '星期日_星期一_星期二_星期三_星期四_星期五_星期六'.split('_'),
  		    weekdaysShort: '週日_週一_週二_週三_週四_週五_週六'.split('_'),
  		    weekdaysMin: '日_一_二_三_四_五_六'.split('_'),
  		    longDateFormat: {
  		      LT: 'HH:mm',
  		      LTS: 'HH:mm:ss',
  		      L: 'YYYY/MM/DD',
  		      LL: 'YYYY年M月D日',
  		      LLL: 'YYYY年M月D日 HH:mm',
  		      LLLL: 'YYYY年M月D日dddd HH:mm',
  		      l: 'YYYY/M/D',
  		      ll: 'YYYY年M月D日',
  		      lll: 'YYYY年M月D日 HH:mm',
  		      llll: 'YYYY年M月D日dddd HH:mm'
  		    },
  		    meridiemParse: /凌晨|早上|上午|中午|下午|晚上/,
  		    meridiemHour: function (hour, meridiem) {
  		      if (hour === 12) {
  		        hour = 0;
  		      }
  		      if (meridiem === '凌晨' || meridiem === '早上' || meridiem === '上午') {
  		        return hour;
  		      } else if (meridiem === '中午') {
  		        return hour >= 11 ? hour : hour + 12;
  		      } else if (meridiem === '下午' || meridiem === '晚上') {
  		        return hour + 12;
  		      }
  		    },
  		    meridiem: function (hour, minute, isLower) {
  		      var hm = hour * 100 + minute;
  		      if (hm < 600) {
  		        return '凌晨';
  		      } else if (hm < 900) {
  		        return '早上';
  		      } else if (hm < 1130) {
  		        return '上午';
  		      } else if (hm < 1230) {
  		        return '中午';
  		      } else if (hm < 1800) {
  		        return '下午';
  		      } else {
  		        return '晚上';
  		      }
  		    },
  		    calendar: {
  		      sameDay: '[今天] LT',
  		      nextDay: '[明天] LT',
  		      nextWeek: '[下]dddd LT',
  		      lastDay: '[昨天] LT',
  		      lastWeek: '[上]dddd LT',
  		      sameElse: 'L'
  		    },
  		    dayOfMonthOrdinalParse: /\d{1,2}(日|月|週)/,
  		    ordinal: function (number, period) {
  		      switch (period) {
  		        case 'd':
  		        case 'D':
  		        case 'DDD':
  		          return number + '日';
  		        case 'M':
  		          return number + '月';
  		        case 'w':
  		        case 'W':
  		          return number + '週';
  		        default:
  		          return number;
  		      }
  		    },
  		    relativeTime: {
  		      future: '%s後',
  		      past: '%s前',
  		      s: '幾秒',
  		      ss: '%d 秒',
  		      m: '1 分鐘',
  		      mm: '%d 分鐘',
  		      h: '1 小時',
  		      hh: '%d 小時',
  		      d: '1 天',
  		      dd: '%d 天',
  		      M: '1 個月',
  		      MM: '%d 個月',
  		      y: '1 年',
  		      yy: '%d 年'
  		    }
  		  });
  		  hooks.locale('en');
  		  return hooks;
  		}); 
  	} (momentWithLocales$1));
  	return momentWithLocales$1.exports;
  }

  var momentWithLocalesExports = requireMomentWithLocales();
  var moment = /*@__PURE__*/getDefaultExportFromCjs(momentWithLocalesExports);

  var momentBusinessDays = {exports: {}};

  var hasRequiredMomentBusinessDays;

  function requireMomentBusinessDays () {
  	if (hasRequiredMomentBusinessDays) return momentBusinessDays.exports;
  	hasRequiredMomentBusinessDays = 1;
  	(function (module) {

  		if (typeof commonjsRequire === 'function') {
  		  var moment = requireMoment();
  		}
  		moment.fn.isHoliday = function () {
  		  var locale = this.localeData();
  		  if (locale._holidays) {
  		    if (locale._holidays.indexOf(this.format(locale._holidayFormat)) >= 0) return true;
  		  }
  		  if (locale.holiday) {
  		    if (locale.holiday(this)) {
  		      return true;
  		    }
  		    return false;
  		  }
  		  return false;
  		};
  		moment.fn.isBusinessDay = function () {
  		  var locale = this.localeData();
  		  var defaultWorkingWeekdays = [1, 2, 3, 4, 5];
  		  var workingWeekdays = locale._workingWeekdays || defaultWorkingWeekdays;
  		  if (this.isHoliday()) return false;
  		  if (workingWeekdays.indexOf(this.day()) >= 0) return true;
  		  return false;
  		};
  		moment.fn.businessDaysIntoMonth = function () {
  		  if (!this.isValid()) {
  		    return NaN;
  		  }
  		  var businessDay = this.isBusinessDay() ? this : this.prevBusinessDay();
  		  var monthBusinessDays = businessDay.monthBusinessDays();
  		  var businessDaysIntoMonth;
  		  monthBusinessDays.map(function (day, index) {
  		    if (day.format('M/DD/YY') === businessDay.format('M/DD/YY')) {
  		      businessDaysIntoMonth = index + 1;
  		    }
  		  });
  		  return businessDaysIntoMonth;
  		};
  		moment.fn.businessDiff = function (param, relative) {
  		  var d1 = this.clone();
  		  var d2 = param.clone();
  		  var positive = d1 >= d2;
  		  var start = d1 < d2 ? d1 : d2;
  		  var end = d2 > d1 ? d2 : d1;
  		  var daysBetween = 0;
  		  if (start.format('DD/MM/YYYY') === end.format('DD/MM/YYYY')) {
  		    return daysBetween;
  		  }
  		  while (start < end) {
  		    if (start.isBusinessDay()) {
  		      daysBetween++;
  		    }
  		    start.add(1, 'd');
  		  }
  		  if (relative) {
  		    return positive ? daysBetween : -daysBetween;
  		  }
  		  return daysBetween;
  		};
  		moment.fn.businessAdd = function (number, period) {
  		  var day = this.clone();
  		  if (!day.isValid()) {
  		    return day;
  		  }
  		  if (number < 0) {
  		    number = Math.round(-1 * number) * -1;
  		  } else {
  		    number = Math.round(number);
  		  }
  		  var signal = number < 0 ? -1 : 1;
  		  period = typeof period !== 'undefined' ? period : 'days';
  		  var remaining = Math.abs(number);
  		  while (remaining > 0) {
  		    day.add(signal, period);
  		    if (day.isBusinessDay()) {
  		      remaining--;
  		    }
  		  }
  		  return day;
  		};
  		moment.fn.businessSubtract = function (number, period) {
  		  return this.businessAdd(-number, period);
  		};
  		moment.fn.nextBusinessDay = function () {
  		  var locale = this.localeData();
  		  var loop = 1;
  		  var defaultNextBusinessDayLimit = 7;
  		  var limit = locale._nextBusinessDayLimit || defaultNextBusinessDayLimit;
  		  while (loop < limit) {
  		    if (this.add(1, 'd').isBusinessDay()) {
  		      break;
  		    }
  		    loop++;
  		  }
  		  return this;
  		};
  		moment.fn.prevBusinessDay = function () {
  		  var locale = this.localeData();
  		  var loop = 1;
  		  var defaultPrevBusinessDayLimit = 7;
  		  var limit = locale._prevBusinessDayLimit || defaultPrevBusinessDayLimit;
  		  while (loop < limit) {
  		    if (this.subtract(1, 'd').isBusinessDay()) {
  		      break;
  		    }
  		    loop++;
  		  }
  		  return this;
  		};
  		moment.fn.monthBusinessDays = function (partialEndDate) {
  		  if (!this.isValid()) {
  		    return [];
  		  }
  		  var me = this.clone();
  		  var day = me.clone().startOf('month');
  		  var end = partialEndDate ? partialEndDate : me.clone().endOf('month');
  		  var daysArr = [];
  		  var done = false;
  		  while (!done) {
  		    if (day.isBusinessDay()) {
  		      daysArr.push(day.clone());
  		    }
  		    if (end.diff(day.add(1, 'd')) < 0) {
  		      done = true;
  		    }
  		  }
  		  return daysArr;
  		};
  		moment.fn.monthNaturalDays = function (fromToday) {
  		  if (!this.isValid()) {
  		    return [];
  		  }
  		  var me = this.clone();
  		  var day = fromToday ? me.clone() : me.clone().startOf('month');
  		  var end = me.clone().endOf('month');
  		  var daysArr = [];
  		  var done = false;
  		  while (!done) {
  		    daysArr.push(day.clone());
  		    if (end.diff(day.add(1, 'd')) < 0) {
  		      done = true;
  		    }
  		  }
  		  return daysArr;
  		};
  		moment.fn.monthBusinessWeeks = function (fromToday) {
  		  fromToday = fromToday || false;
  		  var me = this.clone();
  		  var startDate = fromToday ? me.clone() : me.clone().startOf('month');
  		  return getBusinessWeeks(this, fromToday, null, startDate);
  		};
  		moment.fn.businessWeeksBetween = function (endDate) {
  		  var me = this.clone();
  		  var startDate = me.clone();
  		  return getBusinessWeeks(this, false, endDate, startDate);
  		};
  		var getBusinessWeeks = function (self, fromToday, endDate, startDate) {
  		  if (!self.isValid()) {
  		    return [];
  		  }
  		  var me = self.clone();
  		  var day = startDate;
  		  var end = endDate ? moment(endDate).clone() : me.clone().endOf('month');
  		  var weeksArr = [];
  		  var daysArr = [];
  		  var done = false;
  		  while (!done) {
  		    if (day.day() >= 1 && day.day() < 6) {
  		      daysArr.push(day.clone());
  		    }
  		    if (day.day() === 5) {
  		      weeksArr.push(daysArr);
  		      daysArr = [];
  		    }
  		    if (end.diff(day.add(1, 'd')) < 0) {
  		      if (daysArr.length < 5) {
  		        weeksArr.push(daysArr);
  		      }
  		      done = true;
  		    }
  		  }
  		  return weeksArr;
  		};
  		moment.fn.monthNaturalWeeks = function (fromToday) {
  		  if (!this.isValid()) {
  		    return [];
  		  }
  		  var me = this.clone();
  		  var day = fromToday ? me.clone() : me.clone().startOf('month');
  		  var end = me.clone().endOf('month');
  		  var weeksArr = [];
  		  var daysArr = [];
  		  var done = false;
  		  while (!done) {
  		    daysArr.push(day.clone());
  		    if (day.day() === 6) {
  		      weeksArr.push(daysArr);
  		      daysArr = [];
  		    }
  		    if (end.diff(day.add(1, 'd')) < 0) {
  		      if (daysArr.length < 7) {
  		        weeksArr.push(daysArr);
  		      }
  		      done = true;
  		    }
  		  }
  		  return weeksArr;
  		};
  		if (module.exports) {
  		  module.exports = moment;
  		} 
  	} (momentBusinessDays));
  	return momentBusinessDays.exports;
  }

  requireMomentBusinessDays();

  //-------------------------------------------------------------------------------
  // Date functions, that don't rely on NotePlan functions/types
  // @jgclark except where shown

  //-----------------------------------------------------------
  // CONSTANTS
  const MOMENT_FORMAT_NP_ISO = 'YYYY-MM-DD';
  const MOMENT_FORMAT_NP_DAY = 'YYYYMMDD';
  const MOMENT_FORMAT_NP_MONTH = 'YYYY-MM';
  const MOMENT_FORMAT_NP_QUARTER = 'YYYY-[Q]Q';
  const RE_DATE = '\\d{4}-[01]\\d-[0123]\\d'; // find ISO dates of form YYYY-MM-DD (stricter than before)
  const RE_YYYYMMDD_DATE = '\\d{4}[01]\\d[0123]\\d'; // version of above that finds dates of form YYYYMMDD
  const RE_FILE_EXTENSIONS_GROUP = `\\.(md|txt)$`; // and tie to end of string
  const RE_NP_DAY_SPEC = RE_YYYYMMDD_DATE;
  const RE_DAILY_NOTE_FILENAME = `(^|\\/)${RE_YYYYMMDD_DATE}${RE_FILE_EXTENSIONS_GROUP}`;

  // Week regex strings
  const RE_NP_WEEK_SPEC = '\\d{4}\\-W[0-5]\\d'; // find dates of form YYYY-Wnn
  const RE_WEEKLY_NOTE_FILENAME = `(^|\\/)${RE_NP_WEEK_SPEC}${RE_FILE_EXTENSIONS_GROUP}`;

  // Months
  // export const RE_NP_MONTH_SPEC = '(?<![\\d-])\\d{4}-[01]\\d(?![\\d-])' // find dates of form YYYY-mm not following or followed by digit or - [doesn't work because it has a lookbehind]
  const RE_NP_MONTH_SPEC = '\\d{4}-[01]\\d(?![\\d-])'; // find dates of form YYYY-mm not followed by digit or - [fails if I add negative start or negative lookbehinds]
  const RE_MONTHLY_NOTE_FILENAME = `(^|\\/)${RE_NP_MONTH_SPEC}${RE_FILE_EXTENSIONS_GROUP}`;

  // Quarters
  const RE_NP_QUARTER_SPEC = '\\d{4}\\-Q[1-4](?!\\d)'; // find dates of form YYYY-Qn not followed by digit
  const RE_QUARTERLY_NOTE_FILENAME = `(^|\\/)${RE_NP_QUARTER_SPEC}${RE_FILE_EXTENSIONS_GROUP}`;

  // Years
  const RE_NP_YEAR_SPEC = '\\d{4}(?![\\d-])'; // find years of form YYYY without leading or trailing - or digit [fails if I add negative start or negative lookbehinds]
  const RE_YEARLY_NOTE_FILENAME = `(^|\\/)${RE_NP_YEAR_SPEC}${RE_FILE_EXTENSIONS_GROUP}`;
  const RE_IS_SCHEDULED = new RegExp(`>(${RE_DATE}|${RE_NP_WEEK_SPEC}|${RE_NP_MONTH_SPEC}|${RE_NP_QUARTER_SPEC}|${RE_NP_YEAR_SPEC}|today)`);
  /**
   * Constant version of getTodaysDateHyphenated()
   * This uses local time, so shouldn't get TZ problems.
   * @author @jgclark
   */
  moment().format('YYYY-MM-DD');

  // Note: there are others in NPdateTime.js that use locale settings

  // return datetime in UTC ISO format
  moment().toISOString().replace('T', ' ').slice(0, 16);

  /**
   * Returns the NP string representation of a Calendar note's date, from its filename. Covers daily to yearly notes.
   * @param {string} filename
   * @param {boolean} returnISODate - returns ISO daily note YYYY-MM-DD not actual filename YYYYMMDD
   * @returns {string} YYYYMMDD or YYYY-MM-DD depending on 2nd parameter / YYYY-Wnn / YYYY-mm / YYYY-Qn / YYYY date (some only from NP v3.7.2)
   * @tests in jest file
   */
  function getDateStringFromCalendarFilename(filename, returnISODate = false) {
    try {
      // logDebug('gDSFCF', `for ${filename} ...`)
      if (filename.match(RE_DAILY_NOTE_FILENAME)) {
        // logDebug('gDSFCF', `= daily`)
        if (returnISODate) {
          return getISODateStringFromYYYYMMDD(filename);
        } else {
          return filename.slice(0, 8);
        }
      } else if (filename.match(RE_WEEKLY_NOTE_FILENAME)) {
        //TEST:
        // logDebug('gDSFCF', `${filename} = weekly`)
        return filename.slice(0, 8);
      } else if (filename.match(RE_MONTHLY_NOTE_FILENAME)) {
        // logDebug('gDSFCF', `${filename} = monthly`)
        return filename.slice(0, 7);
      } else if (filename.match(RE_QUARTERLY_NOTE_FILENAME)) {
        // logDebug('gDSFCF', `${filename} = quarterly`)
        return filename.slice(0, 7);
      } else if (filename.match(RE_YEARLY_NOTE_FILENAME)) {
        // logDebug('gDSFCF', `${filename} = yearly`)
        return filename.slice(0, 4);
      } else {
        throw new Error(`Invalid calendar filename: ${filename}`);
      }
    } catch (err) {
      logError('dateTime / getDateStringFromCalendarFilename', err.message);
      return '(invalid date)'; // for completeness
    }
  }

  /**
   * Change a YYYYMMDD date string to YYYY-MM-DD
   * @param {string} dateStr without hyphens
   * @returns {string} ISO hyphenated string
   */
  function getISODateStringFromYYYYMMDD(filename) {
    if (filename.match(/^\d{8}/)) {
      return `${filename.slice(0, 4)}-${filename.slice(4, 6)}-${filename.slice(6, 8)}`;
    } else {
      return '(not a YYYYMMDD date)';
    }
  }

  /**
   * Return difference between start and end dates (by default ignoring any time components)
   * if returnFractionalDays is true, then use time components and return a fractional number of days (e.g. 1.5 for 36 hours)
   * Note: It is highly recommended that if you have an ISO string (e.g. '2022-01-01') you send the string
   * rather than trying to convert it to a Date object, as the Date object may be converted to local time, which may not be what you want.
   * Note: v2 uses a.moment(b).diff(moment().startOf('day'), 'days') instead
   * @author @jgclark
   * @tests in jest file
   * @param {string|Date} startDate - if string, must be in ISO format (e.g. '2022-01-01')
   * @param {string|Date} endDate - if string, must be in ISO format (e.g. '2022-01-01')
   * @param {boolean} returnFractionalDays (default: false) - if true, return a fractional number of days (e.g. 1.5 for 36 hours)
   * otherwise, it truncates the decimal part (e.g. 1 for 36 hours)
   * @return {number} - number of days between startDate and endDate (truncated to integer if returnFractionalDays is false)
   */
  function daysBetween(startDate, endDate, returnFractionalDays = false) {
    const reISODATE = new RegExp(RE_DATE);
    if (typeof startDate === 'string' && !startDate.match(reISODATE) || typeof endDate === 'string' && !endDate.match(reISODATE)) {
      throw new Error('Invalid date format');
    }
    // v1 method:
    // const res = Math.round((endDate - startDate) / 1000 / 60 / 60 / 24) // i.e. milliseconds -> days
    // return (res === -0) ? 0 : res // handle weird edge case

    // v2 method:
    // moment's a.diff(b, 'days') gives the different in days between a and b, with the answer truncated (not rounded)
    if (returnFractionalDays) {
      return moment(endDate).diff(moment(startDate), 'days', returnFractionalDays);
    } else {
      return moment(endDate).startOf('day').diff(moment(startDate).startOf('day'), 'days', returnFractionalDays);
    }
  }

  /**
   * Return rough relative string version of difference between date and today.
   * Don't return all the detail, but just the most significant unit (year, month, week, day)
   * If date is in the past then adds 'ago'.
   * Note: there is a newer locale-aware version of this function, using moment library at NPdateTime::localeRelativeDateFromNumber
   * @author @jgclark
   * @param {number} diffIn - number of days difference (positive or negative)
   * @param {boolean?} shortStyle?
   * @returns {string} - relative date string (e.g. today, 3w ago, 2m, 4y ago.)
   */
  function relativeDateFromNumber(diffIn, useShortStyle = false) {
    let output = '';
    let diff = diffIn;
    if (diffIn == null || diffIn === undefined || isNaN(diffIn)) {
      logWarn('dateTime / relativeDateFromNumber', `diffIn param is undefined`);
      return 'unknown date';
    }
    let isPast = false;
    // logDebug('dateTime / relativeDateFromNumber', `original diff = ${diff}`)
    if (diff < 0) {
      diff = Math.abs(diff);
      isPast = true;
    }
    if (useShortStyle) {
      if (diff === 1) {
        output = `${diff}d`; // day
      } else if (diff < 9) {
        output = `${diff}d`; // days
      } else if (diff < 12) {
        output = `${Math.round(diff / 7.0)}w`; // wk
      } else if (diff < 29) {
        output = `${Math.round(diff / 7.0)}w`; // wks
      } else if (diff < 550) {
        output = `${Math.round(diff / 30.4)}m`; // mon
      } else {
        output = `${Math.round(diff / 365.0)}y`; // yrs
      }
    } else {
      if (diff === 1) {
        output = `${diff} day`;
      } else if (diff < 9) {
        output = `${diff} days`;
      } else if (diff < 12) {
        output = `${Math.round(diff / 7.0)} wk`;
      } else if (diff < 29) {
        output = `${Math.round(diff / 7.0)} wks`;
      } else if (diff < 550) {
        output = `${Math.round(diff / 30.4)} mon`;
      } else {
        output = `${Math.round(diff / 365.0)} yrs`;
      }
    }
    if (diff === 0) {
      output = `today`;
    } else if (isPast) {
      output += ` ago`;
    } else {
      output = `in ${output}`;
    }
    // logDebug('dateTime / relativeDateFromNumber', `--> ${output}`)
    return output;
  }

  /**
   * Validate if a filename could be used to pull up any calendar note (of all NP allowed calendar note durations)
   * Note: This is just a regex test: it doesn't test if such a note actually exists.
   * @param {string} text
   * @returns {boolean} whether it passes the @jgclark RegEx texts for day (note YYYYMMDD not ISO), week, month, quarter or year.
   */
  function isValidCalendarNoteFilename(text) {
    const combinedRE = new RegExp(`^(${RE_NP_DAY_SPEC}|${RE_NP_WEEK_SPEC}|${RE_NP_MONTH_SPEC}|${RE_NP_QUARTER_SPEC}|${RE_NP_YEAR_SPEC})\.(md|txt)$`);
    return combinedRE.test(text);
  }

  //-------------------------------------------------------------------------------
  // Date functions that rely on NotePlan functions/types
  // @jgclark except where shown
  //-------------------------------------------------------------------------------

  //--------------------------------------------------------------------------------

  // TODO: work out how to test these next few functions
  function setMomentLocaleFromEnvironment() {
    // logDebug('NPdateTime', `NP reports languageCode = ${NotePlan.environment.languageCode ?? '<not set>'}`)
    // logDebug('NPdateTime', `NP reports regionCode   = ${NotePlan.environment.regionCode ?? '<not set>'}`)
    // Locale-specific date + time formats
    // Set locale for moment library
    const userLocaleSetting = `${NotePlan.environment.languageCode}${NotePlan.environment.regionCode ? `-${NotePlan.environment.regionCode}` : ''}`;
    moment.locale(userLocaleSetting);
    // logDebug('NPdateTime', `locale for moment library is now ${moment.locale()}`)
  }
  function nowLocaleShortDateTime() {
    setMomentLocaleFromEnvironment();
    return moment().format('l LT'); // format('L LT')
  }
  moment().toDate().toLocaleString();

  // TODO: Finish moving references to this file from dateTime.js
  // TODO: Or can this now be deprecated in favour of newer functions above?
  function toNPLocaleDateString(dateObj, dateStyle = "short") {
    /**
     * TODO: use details from NotePlan.environment...
     *  "languageCode": "en",
     *   "regionCode": "GB",
     *   "preferredLanguages": [
     *     "en-GB"
     *   ],
     */
    const shortDateFmt = new Intl.DateTimeFormat(NotePlan.environment.preferredLanguages[0], {
      dateStyle: dateStyle
    });
    return shortDateFmt.format(dateObj);
  }
  function pad(n) {
    return n < 10 ? `0${n}` : String(n);
  }

  /**
   * Get all the week details for a given unhyphenated|hyphenated(ISO8601) date string, a week string (YYYY-Wnn) or a Date object
   * Week info is offset depending on the NotePlan setting for the first day of the week
   * Note: requires NP API calls introduced in v3.7.0.
   * Note: gracefully falls back to using moment calls if NP API calls not available
   * @param {string} dateIn - date string in format YYYY-MM-DD, YYYYMMDD, YYYY-Wnn OR a Date object (default = today).
   * Note: Make sure that if you send in a date object that it's a date in the correct time/timezone you want.
   * If you create a new date of your own without a time (e.g. new Date("2022-01-01")) it could produce a date
   * in a previous or next day depending on your timezone. So if you are creating the date, just send through the date string rather than a date object.
   * @param {number} offsetIncrement - number of days|weeks|month to add (or negative=subtract) to date (default: 0)
   * @param {string} offsetType - the increment to add/subtract: 'day'|'week'|'month'|'year' (default: 'week'). Note: not quarters!
   * @returns { NotePlanWeekInfo | null } - an object with all the week details, or null if there's an error
   * getNPWeekData: alias weekInfo, weekData, getWeek, weeklyNote
   * {
   *   weekNumber: number, // e.g. 1
   *   weekYear: number, // e.g. 2022
   *   weekString: string, // e.g. 2022-W01
   *   startDate: Date,
   *   endDate: Date,
   *   date: Date,
   * }
   * @author @dwertheimer, extended by @jgclark
   * @tests in jest file
   */
  function getNPWeekData(dateIn = new Date(), offsetIncrement = 0, offsetType = 'week') {
    try {
      // if (NotePlan?.environment?.buildVersion < 876 ?? true) {
      //   // to allow NPDateTime.test.js to run
      //   throw new Error('Sorry; week API calls requires NotePlan v3.7 or newer.')
      // }

      let dateStrFormat = 'YYYY-MM-DD',
        newMom;
      if (typeof dateIn === 'string') {
        if (new RegExp(RE_YYYYMMDD_DATE).test(dateIn)) dateStrFormat = 'YYYYMMDD';
        if (new RegExp(RE_NP_WEEK_SPEC).test(dateIn)) dateStrFormat = 'YYYY-[W]WW';
        newMom = moment(dateIn, dateStrFormat).add(offsetIncrement, offsetType);
      } else {
        newMom = moment(dateIn).add(offsetIncrement, offsetType);
      }
      if (!newMom) {
        throw new Error(`Cannot get newMom from dateIn '${String(dateIn)}'`);
      }
      const date = newMom.toDate();
      if (!date) {
        throw new Error(`Cannot get date from dateIn '${String(dateIn)}'`);
      }
      let weekNumber;
      let startDate;
      let endDate;
      // This might be run from React side, where Calendar.* is not available.
      // If this happens, then instead offer the ISO week number.
      if (!Calendar || typeof Calendar !== 'object') {
        logInfo('NPdateTime::getNPWeekData', `NP's Calendar API functions are not available, so I will use moment instead. This doesn't know what your chosen first day of week is.`);
        weekNumber = newMom.week(); // uses moment locale
        startDate = newMom.startOf('week').toDate();
        endDate = newMom.endOf('week').toDate();
      } else {
        weekNumber = Calendar.weekNumber(date);
        startDate = Calendar.startOfWeek(date);
        endDate = Calendar.endOfWeek(date);
      }
      const weekStartYear = startDate.getFullYear();
      const weekEndYear = endDate.getFullYear();
      const weekYear = weekStartYear === weekEndYear ? weekStartYear : weekNumber === 1 ? weekEndYear : weekStartYear;
      const weekString = `${weekYear}-W${pad(weekNumber)}`;
      return {
        weekNumber,
        startDate,
        endDate,
        weekYear,
        date,
        weekString
      };
    } catch (err) {
      logError('NPdateTime::getNPWeekData', err.message);
      return null;
    }
  }

  /**
   * Get array of dates relative to today for day, week and month.
   * Note: now tests to see if NP API calls are available, and if not returns an empty array
   * @author @jgclark
   * @returns {Array<Object>} relative date name, relative date string, TNote for that relative date
   */
  function getRelativeDates() {
    try {
      const relativeDates = [];
      const todayMom = moment();
      if (!DataStore || typeof DataStore !== 'object') {
        logDebug('NPdateTime::getRelativeDates', `NP DataStore functions are not available, so returning an empty set.`);
        return [{}];
      }

      // Calculate relative dates. Remember to clone todayMom first as moments aren't immutable
      const thisDateStrDisplay = moment(todayMom).format(MOMENT_FORMAT_NP_ISO);
      const todayDateStr = moment(todayMom).format(MOMENT_FORMAT_NP_DAY);
      relativeDates.push({
        relName: 'today',
        dateStr: thisDateStrDisplay,
        note: DataStore.calendarNoteByDateString(todayDateStr)
      });
      const yesterdayDateStr = moment(todayMom).subtract(1, 'days').startOf('day').format(MOMENT_FORMAT_NP_DAY);
      relativeDates.push({
        relName: 'yesterday',
        dateStr: thisDateStrDisplay,
        note: DataStore.calendarNoteByDateString(yesterdayDateStr)
      });
      const tomorrowDateStr = moment(todayMom).add(1, 'days').startOf('day').format(MOMENT_FORMAT_NP_DAY);
      relativeDates.push({
        relName: 'tomorrow',
        dateStr: thisDateStrDisplay,
        note: DataStore.calendarNoteByDateString(tomorrowDateStr)
      });

      // can't start with moment as NP weeks count differently
      // $FlowIgnore[incompatible-type]
      let thisNPWeekInfo = getNPWeekData(new Date());
      let thisDateStr = thisNPWeekInfo.weekString;
      relativeDates.push({
        relName: 'this week',
        dateStr: thisDateStr,
        note: DataStore.calendarNoteByDateString(thisDateStr)
      });
      // $FlowIgnore[incompatible-type]
      thisNPWeekInfo = getNPWeekData(new Date(), -1);
      // $FlowIgnore[incompatible-use]
      thisDateStr = thisNPWeekInfo.weekString;
      relativeDates.push({
        relName: 'last week',
        dateStr: thisDateStr,
        note: DataStore.calendarNoteByDateString(thisDateStr)
      });
      // $FlowIgnore[incompatible-type]
      thisNPWeekInfo = getNPWeekData(new Date(), 1);
      // $FlowIgnore[incompatible-use]
      thisDateStr = thisNPWeekInfo.weekString;
      relativeDates.push({
        relName: 'next week',
        dateStr: thisDateStr,
        note: DataStore.calendarNoteByDateString(thisDateStr)
      });
      thisDateStr = moment(todayMom).startOf('month').format(MOMENT_FORMAT_NP_MONTH);
      relativeDates.push({
        relName: 'this month',
        dateStr: thisDateStr,
        note: DataStore.calendarNoteByDateString(thisDateStr)
      });
      thisDateStr = moment(todayMom).subtract(1, 'month').startOf('month').format(MOMENT_FORMAT_NP_MONTH);
      relativeDates.push({
        relName: 'last month',
        dateStr: thisDateStr,
        note: DataStore.calendarNoteByDateString(thisDateStr)
      });
      thisDateStr = moment(todayMom).add(1, 'month').startOf('month').format(MOMENT_FORMAT_NP_MONTH);
      relativeDates.push({
        relName: 'next month',
        dateStr: thisDateStr,
        note: DataStore.calendarNoteByDateString(thisDateStr)
      });
      thisDateStr = moment(todayMom).startOf('quarter').format(MOMENT_FORMAT_NP_QUARTER);
      relativeDates.push({
        relName: 'this quarter',
        dateStr: thisDateStr,
        note: DataStore.calendarNoteByDateString(thisDateStr)
      });
      thisDateStr = moment(todayMom).subtract(1, 'quarter').startOf('quarter').format(MOMENT_FORMAT_NP_QUARTER);
      relativeDates.push({
        relName: 'last quarter',
        dateStr: thisDateStr,
        note: DataStore.calendarNoteByDateString(thisDateStr)
      });
      thisDateStr = moment(todayMom).add(1, 'quarter').startOf('quarter').format(MOMENT_FORMAT_NP_QUARTER);
      relativeDates.push({
        relName: 'next quarter',
        dateStr: thisDateStr,
        note: DataStore.calendarNoteByDateString(thisDateStr)
      });

      // for (const rd of relativeDates) {
      //   const noteTitle = (rd.note) ? displayTitle(rd.note) : '(error)'
      //   logDebug('getRelativeDates', `${rd.name ?? ''}: ${rd.dateStr ?? ''} / ${noteTitle}`)
      // }
      return relativeDates;
    } catch (err) {
      logError('getRelativeDates', `${err.name}: ${err.message}`);
      return [{}]; // for completeness
    }
  }

  /**
   * Checks if `value` is classified as an `Array` object.
   *
   * @static
   * @memberOf _
   * @since 0.1.0
   * @category Lang
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is an array, else `false`.
   * @example
   *
   * _.isArray([1, 2, 3]);
   * // => true
   *
   * _.isArray(document.body.children);
   * // => false
   *
   * _.isArray('abc');
   * // => false
   *
   * _.isArray(_.noop);
   * // => false
   */

  var isArray_1;
  var hasRequiredIsArray;

  function requireIsArray () {
  	if (hasRequiredIsArray) return isArray_1;
  	hasRequiredIsArray = 1;
  	var isArray = Array.isArray;
  	isArray_1 = isArray;
  	return isArray_1;
  }

  /** Detect free variable `global` from Node.js. */

  var _freeGlobal;
  var hasRequired_freeGlobal;

  function require_freeGlobal () {
  	if (hasRequired_freeGlobal) return _freeGlobal;
  	hasRequired_freeGlobal = 1;
  	var freeGlobal = typeof commonjsGlobal == 'object' && commonjsGlobal && commonjsGlobal.Object === Object && commonjsGlobal;
  	_freeGlobal = freeGlobal;
  	return _freeGlobal;
  }

  var _root;
  var hasRequired_root;

  function require_root () {
  	if (hasRequired_root) return _root;
  	hasRequired_root = 1;
  	var freeGlobal = require_freeGlobal();

  	/** Detect free variable `self`. */
  	var freeSelf = typeof self == 'object' && self && self.Object === Object && self;

  	/** Used as a reference to the global object. */
  	var root = freeGlobal || freeSelf || Function('return this')();
  	_root = root;
  	return _root;
  }

  var _Symbol;
  var hasRequired_Symbol;

  function require_Symbol () {
  	if (hasRequired_Symbol) return _Symbol;
  	hasRequired_Symbol = 1;
  	var root = require_root();

  	/** Built-in value references. */
  	var Symbol = root.Symbol;
  	_Symbol = Symbol;
  	return _Symbol;
  }

  var _getRawTag;
  var hasRequired_getRawTag;

  function require_getRawTag () {
  	if (hasRequired_getRawTag) return _getRawTag;
  	hasRequired_getRawTag = 1;
  	var Symbol = require_Symbol();

  	/** Used for built-in method references. */
  	var objectProto = Object.prototype;

  	/** Used to check objects for own properties. */
  	var hasOwnProperty = objectProto.hasOwnProperty;

  	/**
  	 * Used to resolve the
  	 * [`toStringTag`](http://ecma-international.org/ecma-262/7.0/#sec-object.prototype.tostring)
  	 * of values.
  	 */
  	var nativeObjectToString = objectProto.toString;

  	/** Built-in value references. */
  	var symToStringTag = Symbol ? Symbol.toStringTag : undefined;

  	/**
  	 * A specialized version of `baseGetTag` which ignores `Symbol.toStringTag` values.
  	 *
  	 * @private
  	 * @param {*} value The value to query.
  	 * @returns {string} Returns the raw `toStringTag`.
  	 */
  	function getRawTag(value) {
  	  var isOwn = hasOwnProperty.call(value, symToStringTag),
  	    tag = value[symToStringTag];
  	  try {
  	    value[symToStringTag] = undefined;
  	    var unmasked = true;
  	  } catch (e) {}
  	  var result = nativeObjectToString.call(value);
  	  if (unmasked) {
  	    if (isOwn) {
  	      value[symToStringTag] = tag;
  	    } else {
  	      delete value[symToStringTag];
  	    }
  	  }
  	  return result;
  	}
  	_getRawTag = getRawTag;
  	return _getRawTag;
  }

  /** Used for built-in method references. */

  var _objectToString;
  var hasRequired_objectToString;

  function require_objectToString () {
  	if (hasRequired_objectToString) return _objectToString;
  	hasRequired_objectToString = 1;
  	var objectProto = Object.prototype;

  	/**
  	 * Used to resolve the
  	 * [`toStringTag`](http://ecma-international.org/ecma-262/7.0/#sec-object.prototype.tostring)
  	 * of values.
  	 */
  	var nativeObjectToString = objectProto.toString;

  	/**
  	 * Converts `value` to a string using `Object.prototype.toString`.
  	 *
  	 * @private
  	 * @param {*} value The value to convert.
  	 * @returns {string} Returns the converted string.
  	 */
  	function objectToString(value) {
  	  return nativeObjectToString.call(value);
  	}
  	_objectToString = objectToString;
  	return _objectToString;
  }

  var _baseGetTag;
  var hasRequired_baseGetTag;

  function require_baseGetTag () {
  	if (hasRequired_baseGetTag) return _baseGetTag;
  	hasRequired_baseGetTag = 1;
  	var Symbol = require_Symbol(),
  	  getRawTag = require_getRawTag(),
  	  objectToString = require_objectToString();

  	/** `Object#toString` result references. */
  	var nullTag = '[object Null]',
  	  undefinedTag = '[object Undefined]';

  	/** Built-in value references. */
  	var symToStringTag = Symbol ? Symbol.toStringTag : undefined;

  	/**
  	 * The base implementation of `getTag` without fallbacks for buggy environments.
  	 *
  	 * @private
  	 * @param {*} value The value to query.
  	 * @returns {string} Returns the `toStringTag`.
  	 */
  	function baseGetTag(value) {
  	  if (value == null) {
  	    return value === undefined ? undefinedTag : nullTag;
  	  }
  	  return symToStringTag && symToStringTag in Object(value) ? getRawTag(value) : objectToString(value);
  	}
  	_baseGetTag = baseGetTag;
  	return _baseGetTag;
  }

  /**
   * Checks if `value` is object-like. A value is object-like if it's not `null`
   * and has a `typeof` result of "object".
   *
   * @static
   * @memberOf _
   * @since 4.0.0
   * @category Lang
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is object-like, else `false`.
   * @example
   *
   * _.isObjectLike({});
   * // => true
   *
   * _.isObjectLike([1, 2, 3]);
   * // => true
   *
   * _.isObjectLike(_.noop);
   * // => false
   *
   * _.isObjectLike(null);
   * // => false
   */

  var isObjectLike_1;
  var hasRequiredIsObjectLike;

  function requireIsObjectLike () {
  	if (hasRequiredIsObjectLike) return isObjectLike_1;
  	hasRequiredIsObjectLike = 1;
  	function isObjectLike(value) {
  	  return value != null && typeof value == 'object';
  	}
  	isObjectLike_1 = isObjectLike;
  	return isObjectLike_1;
  }

  var isSymbol_1;
  var hasRequiredIsSymbol;

  function requireIsSymbol () {
  	if (hasRequiredIsSymbol) return isSymbol_1;
  	hasRequiredIsSymbol = 1;
  	var baseGetTag = require_baseGetTag(),
  	  isObjectLike = requireIsObjectLike();

  	/** `Object#toString` result references. */
  	var symbolTag = '[object Symbol]';

  	/**
  	 * Checks if `value` is classified as a `Symbol` primitive or object.
  	 *
  	 * @static
  	 * @memberOf _
  	 * @since 4.0.0
  	 * @category Lang
  	 * @param {*} value The value to check.
  	 * @returns {boolean} Returns `true` if `value` is a symbol, else `false`.
  	 * @example
  	 *
  	 * _.isSymbol(Symbol.iterator);
  	 * // => true
  	 *
  	 * _.isSymbol('abc');
  	 * // => false
  	 */
  	function isSymbol(value) {
  	  return typeof value == 'symbol' || isObjectLike(value) && baseGetTag(value) == symbolTag;
  	}
  	isSymbol_1 = isSymbol;
  	return isSymbol_1;
  }

  var _isKey;
  var hasRequired_isKey;

  function require_isKey () {
  	if (hasRequired_isKey) return _isKey;
  	hasRequired_isKey = 1;
  	var isArray = requireIsArray(),
  	  isSymbol = requireIsSymbol();

  	/** Used to match property names within property paths. */
  	var reIsDeepProp = /\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/,
  	  reIsPlainProp = /^\w*$/;

  	/**
  	 * Checks if `value` is a property name and not a property path.
  	 *
  	 * @private
  	 * @param {*} value The value to check.
  	 * @param {Object} [object] The object to query keys on.
  	 * @returns {boolean} Returns `true` if `value` is a property name, else `false`.
  	 */
  	function isKey(value, object) {
  	  if (isArray(value)) {
  	    return false;
  	  }
  	  var type = typeof value;
  	  if (type == 'number' || type == 'symbol' || type == 'boolean' || value == null || isSymbol(value)) {
  	    return true;
  	  }
  	  return reIsPlainProp.test(value) || !reIsDeepProp.test(value) || object != null && value in Object(object);
  	}
  	_isKey = isKey;
  	return _isKey;
  }

  /**
   * Checks if `value` is the
   * [language type](http://www.ecma-international.org/ecma-262/7.0/#sec-ecmascript-language-types)
   * of `Object`. (e.g. arrays, functions, objects, regexes, `new Number(0)`, and `new String('')`)
   *
   * @static
   * @memberOf _
   * @since 0.1.0
   * @category Lang
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is an object, else `false`.
   * @example
   *
   * _.isObject({});
   * // => true
   *
   * _.isObject([1, 2, 3]);
   * // => true
   *
   * _.isObject(_.noop);
   * // => true
   *
   * _.isObject(null);
   * // => false
   */

  var isObject_1;
  var hasRequiredIsObject;

  function requireIsObject () {
  	if (hasRequiredIsObject) return isObject_1;
  	hasRequiredIsObject = 1;
  	function isObject(value) {
  	  var type = typeof value;
  	  return value != null && (type == 'object' || type == 'function');
  	}
  	isObject_1 = isObject;
  	return isObject_1;
  }

  var isFunction_1;
  var hasRequiredIsFunction;

  function requireIsFunction () {
  	if (hasRequiredIsFunction) return isFunction_1;
  	hasRequiredIsFunction = 1;
  	var baseGetTag = require_baseGetTag(),
  	  isObject = requireIsObject();

  	/** `Object#toString` result references. */
  	var asyncTag = '[object AsyncFunction]',
  	  funcTag = '[object Function]',
  	  genTag = '[object GeneratorFunction]',
  	  proxyTag = '[object Proxy]';

  	/**
  	 * Checks if `value` is classified as a `Function` object.
  	 *
  	 * @static
  	 * @memberOf _
  	 * @since 0.1.0
  	 * @category Lang
  	 * @param {*} value The value to check.
  	 * @returns {boolean} Returns `true` if `value` is a function, else `false`.
  	 * @example
  	 *
  	 * _.isFunction(_);
  	 * // => true
  	 *
  	 * _.isFunction(/abc/);
  	 * // => false
  	 */
  	function isFunction(value) {
  	  if (!isObject(value)) {
  	    return false;
  	  }
  	  // The use of `Object#toString` avoids issues with the `typeof` operator
  	  // in Safari 9 which returns 'object' for typed arrays and other constructors.
  	  var tag = baseGetTag(value);
  	  return tag == funcTag || tag == genTag || tag == asyncTag || tag == proxyTag;
  	}
  	isFunction_1 = isFunction;
  	return isFunction_1;
  }

  var _coreJsData;
  var hasRequired_coreJsData;

  function require_coreJsData () {
  	if (hasRequired_coreJsData) return _coreJsData;
  	hasRequired_coreJsData = 1;
  	var root = require_root();

  	/** Used to detect overreaching core-js shims. */
  	var coreJsData = root['__core-js_shared__'];
  	_coreJsData = coreJsData;
  	return _coreJsData;
  }

  var _isMasked;
  var hasRequired_isMasked;

  function require_isMasked () {
  	if (hasRequired_isMasked) return _isMasked;
  	hasRequired_isMasked = 1;
  	var coreJsData = require_coreJsData();

  	/** Used to detect methods masquerading as native. */
  	var maskSrcKey = function () {
  	  var uid = /[^.]+$/.exec(coreJsData && coreJsData.keys && coreJsData.keys.IE_PROTO || '');
  	  return uid ? 'Symbol(src)_1.' + uid : '';
  	}();

  	/**
  	 * Checks if `func` has its source masked.
  	 *
  	 * @private
  	 * @param {Function} func The function to check.
  	 * @returns {boolean} Returns `true` if `func` is masked, else `false`.
  	 */
  	function isMasked(func) {
  	  return !!maskSrcKey && maskSrcKey in func;
  	}
  	_isMasked = isMasked;
  	return _isMasked;
  }

  /** Used for built-in method references. */

  var _toSource;
  var hasRequired_toSource;

  function require_toSource () {
  	if (hasRequired_toSource) return _toSource;
  	hasRequired_toSource = 1;
  	var funcProto = Function.prototype;

  	/** Used to resolve the decompiled source of functions. */
  	var funcToString = funcProto.toString;

  	/**
  	 * Converts `func` to its source code.
  	 *
  	 * @private
  	 * @param {Function} func The function to convert.
  	 * @returns {string} Returns the source code.
  	 */
  	function toSource(func) {
  	  if (func != null) {
  	    try {
  	      return funcToString.call(func);
  	    } catch (e) {}
  	    try {
  	      return func + '';
  	    } catch (e) {}
  	  }
  	  return '';
  	}
  	_toSource = toSource;
  	return _toSource;
  }

  var _baseIsNative;
  var hasRequired_baseIsNative;

  function require_baseIsNative () {
  	if (hasRequired_baseIsNative) return _baseIsNative;
  	hasRequired_baseIsNative = 1;
  	var isFunction = requireIsFunction(),
  	  isMasked = require_isMasked(),
  	  isObject = requireIsObject(),
  	  toSource = require_toSource();

  	/**
  	 * Used to match `RegExp`
  	 * [syntax characters](http://ecma-international.org/ecma-262/7.0/#sec-patterns).
  	 */
  	var reRegExpChar = /[\\^$.*+?()[\]{}|]/g;

  	/** Used to detect host constructors (Safari). */
  	var reIsHostCtor = /^\[object .+?Constructor\]$/;

  	/** Used for built-in method references. */
  	var funcProto = Function.prototype,
  	  objectProto = Object.prototype;

  	/** Used to resolve the decompiled source of functions. */
  	var funcToString = funcProto.toString;

  	/** Used to check objects for own properties. */
  	var hasOwnProperty = objectProto.hasOwnProperty;

  	/** Used to detect if a method is native. */
  	var reIsNative = RegExp('^' + funcToString.call(hasOwnProperty).replace(reRegExpChar, '\\$&').replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g, '$1.*?') + '$');

  	/**
  	 * The base implementation of `_.isNative` without bad shim checks.
  	 *
  	 * @private
  	 * @param {*} value The value to check.
  	 * @returns {boolean} Returns `true` if `value` is a native function,
  	 *  else `false`.
  	 */
  	function baseIsNative(value) {
  	  if (!isObject(value) || isMasked(value)) {
  	    return false;
  	  }
  	  var pattern = isFunction(value) ? reIsNative : reIsHostCtor;
  	  return pattern.test(toSource(value));
  	}
  	_baseIsNative = baseIsNative;
  	return _baseIsNative;
  }

  /**
   * Gets the value at `key` of `object`.
   *
   * @private
   * @param {Object} [object] The object to query.
   * @param {string} key The key of the property to get.
   * @returns {*} Returns the property value.
   */

  var _getValue;
  var hasRequired_getValue;

  function require_getValue () {
  	if (hasRequired_getValue) return _getValue;
  	hasRequired_getValue = 1;
  	function getValue(object, key) {
  	  return object == null ? undefined : object[key];
  	}
  	_getValue = getValue;
  	return _getValue;
  }

  var _getNative;
  var hasRequired_getNative;

  function require_getNative () {
  	if (hasRequired_getNative) return _getNative;
  	hasRequired_getNative = 1;
  	var baseIsNative = require_baseIsNative(),
  	  getValue = require_getValue();

  	/**
  	 * Gets the native function at `key` of `object`.
  	 *
  	 * @private
  	 * @param {Object} object The object to query.
  	 * @param {string} key The key of the method to get.
  	 * @returns {*} Returns the function if it's native, else `undefined`.
  	 */
  	function getNative(object, key) {
  	  var value = getValue(object, key);
  	  return baseIsNative(value) ? value : undefined;
  	}
  	_getNative = getNative;
  	return _getNative;
  }

  var _nativeCreate;
  var hasRequired_nativeCreate;

  function require_nativeCreate () {
  	if (hasRequired_nativeCreate) return _nativeCreate;
  	hasRequired_nativeCreate = 1;
  	var getNative = require_getNative();

  	/* Built-in method references that are verified to be native. */
  	var nativeCreate = getNative(Object, 'create');
  	_nativeCreate = nativeCreate;
  	return _nativeCreate;
  }

  var _hashClear;
  var hasRequired_hashClear;

  function require_hashClear () {
  	if (hasRequired_hashClear) return _hashClear;
  	hasRequired_hashClear = 1;
  	var nativeCreate = require_nativeCreate();

  	/**
  	 * Removes all key-value entries from the hash.
  	 *
  	 * @private
  	 * @name clear
  	 * @memberOf Hash
  	 */
  	function hashClear() {
  	  this.__data__ = nativeCreate ? nativeCreate(null) : {};
  	  this.size = 0;
  	}
  	_hashClear = hashClear;
  	return _hashClear;
  }

  /**
   * Removes `key` and its value from the hash.
   *
   * @private
   * @name delete
   * @memberOf Hash
   * @param {Object} hash The hash to modify.
   * @param {string} key The key of the value to remove.
   * @returns {boolean} Returns `true` if the entry was removed, else `false`.
   */

  var _hashDelete;
  var hasRequired_hashDelete;

  function require_hashDelete () {
  	if (hasRequired_hashDelete) return _hashDelete;
  	hasRequired_hashDelete = 1;
  	function hashDelete(key) {
  	  var result = this.has(key) && delete this.__data__[key];
  	  this.size -= result ? 1 : 0;
  	  return result;
  	}
  	_hashDelete = hashDelete;
  	return _hashDelete;
  }

  var _hashGet;
  var hasRequired_hashGet;

  function require_hashGet () {
  	if (hasRequired_hashGet) return _hashGet;
  	hasRequired_hashGet = 1;
  	var nativeCreate = require_nativeCreate();

  	/** Used to stand-in for `undefined` hash values. */
  	var HASH_UNDEFINED = '__lodash_hash_undefined__';

  	/** Used for built-in method references. */
  	var objectProto = Object.prototype;

  	/** Used to check objects for own properties. */
  	var hasOwnProperty = objectProto.hasOwnProperty;

  	/**
  	 * Gets the hash value for `key`.
  	 *
  	 * @private
  	 * @name get
  	 * @memberOf Hash
  	 * @param {string} key The key of the value to get.
  	 * @returns {*} Returns the entry value.
  	 */
  	function hashGet(key) {
  	  var data = this.__data__;
  	  if (nativeCreate) {
  	    var result = data[key];
  	    return result === HASH_UNDEFINED ? undefined : result;
  	  }
  	  return hasOwnProperty.call(data, key) ? data[key] : undefined;
  	}
  	_hashGet = hashGet;
  	return _hashGet;
  }

  var _hashHas;
  var hasRequired_hashHas;

  function require_hashHas () {
  	if (hasRequired_hashHas) return _hashHas;
  	hasRequired_hashHas = 1;
  	var nativeCreate = require_nativeCreate();

  	/** Used for built-in method references. */
  	var objectProto = Object.prototype;

  	/** Used to check objects for own properties. */
  	var hasOwnProperty = objectProto.hasOwnProperty;

  	/**
  	 * Checks if a hash value for `key` exists.
  	 *
  	 * @private
  	 * @name has
  	 * @memberOf Hash
  	 * @param {string} key The key of the entry to check.
  	 * @returns {boolean} Returns `true` if an entry for `key` exists, else `false`.
  	 */
  	function hashHas(key) {
  	  var data = this.__data__;
  	  return nativeCreate ? data[key] !== undefined : hasOwnProperty.call(data, key);
  	}
  	_hashHas = hashHas;
  	return _hashHas;
  }

  var _hashSet;
  var hasRequired_hashSet;

  function require_hashSet () {
  	if (hasRequired_hashSet) return _hashSet;
  	hasRequired_hashSet = 1;
  	var nativeCreate = require_nativeCreate();

  	/** Used to stand-in for `undefined` hash values. */
  	var HASH_UNDEFINED = '__lodash_hash_undefined__';

  	/**
  	 * Sets the hash `key` to `value`.
  	 *
  	 * @private
  	 * @name set
  	 * @memberOf Hash
  	 * @param {string} key The key of the value to set.
  	 * @param {*} value The value to set.
  	 * @returns {Object} Returns the hash instance.
  	 */
  	function hashSet(key, value) {
  	  var data = this.__data__;
  	  this.size += this.has(key) ? 0 : 1;
  	  data[key] = nativeCreate && value === undefined ? HASH_UNDEFINED : value;
  	  return this;
  	}
  	_hashSet = hashSet;
  	return _hashSet;
  }

  var _Hash;
  var hasRequired_Hash;

  function require_Hash () {
  	if (hasRequired_Hash) return _Hash;
  	hasRequired_Hash = 1;
  	var hashClear = require_hashClear(),
  	  hashDelete = require_hashDelete(),
  	  hashGet = require_hashGet(),
  	  hashHas = require_hashHas(),
  	  hashSet = require_hashSet();

  	/**
  	 * Creates a hash object.
  	 *
  	 * @private
  	 * @constructor
  	 * @param {Array} [entries] The key-value pairs to cache.
  	 */
  	function Hash(entries) {
  	  var index = -1,
  	    length = entries == null ? 0 : entries.length;
  	  this.clear();
  	  while (++index < length) {
  	    var entry = entries[index];
  	    this.set(entry[0], entry[1]);
  	  }
  	}

  	// Add methods to `Hash`.
  	Hash.prototype.clear = hashClear;
  	Hash.prototype['delete'] = hashDelete;
  	Hash.prototype.get = hashGet;
  	Hash.prototype.has = hashHas;
  	Hash.prototype.set = hashSet;
  	_Hash = Hash;
  	return _Hash;
  }

  /**
   * Removes all key-value entries from the list cache.
   *
   * @private
   * @name clear
   * @memberOf ListCache
   */

  var _listCacheClear;
  var hasRequired_listCacheClear;

  function require_listCacheClear () {
  	if (hasRequired_listCacheClear) return _listCacheClear;
  	hasRequired_listCacheClear = 1;
  	function listCacheClear() {
  	  this.__data__ = [];
  	  this.size = 0;
  	}
  	_listCacheClear = listCacheClear;
  	return _listCacheClear;
  }

  /**
   * Performs a
   * [`SameValueZero`](http://ecma-international.org/ecma-262/7.0/#sec-samevaluezero)
   * comparison between two values to determine if they are equivalent.
   *
   * @static
   * @memberOf _
   * @since 4.0.0
   * @category Lang
   * @param {*} value The value to compare.
   * @param {*} other The other value to compare.
   * @returns {boolean} Returns `true` if the values are equivalent, else `false`.
   * @example
   *
   * var object = { 'a': 1 };
   * var other = { 'a': 1 };
   *
   * _.eq(object, object);
   * // => true
   *
   * _.eq(object, other);
   * // => false
   *
   * _.eq('a', 'a');
   * // => true
   *
   * _.eq('a', Object('a'));
   * // => false
   *
   * _.eq(NaN, NaN);
   * // => true
   */

  var eq_1;
  var hasRequiredEq;

  function requireEq () {
  	if (hasRequiredEq) return eq_1;
  	hasRequiredEq = 1;
  	function eq(value, other) {
  	  return value === other || value !== value && other !== other;
  	}
  	eq_1 = eq;
  	return eq_1;
  }

  var _assocIndexOf;
  var hasRequired_assocIndexOf;

  function require_assocIndexOf () {
  	if (hasRequired_assocIndexOf) return _assocIndexOf;
  	hasRequired_assocIndexOf = 1;
  	var eq = requireEq();

  	/**
  	 * Gets the index at which the `key` is found in `array` of key-value pairs.
  	 *
  	 * @private
  	 * @param {Array} array The array to inspect.
  	 * @param {*} key The key to search for.
  	 * @returns {number} Returns the index of the matched value, else `-1`.
  	 */
  	function assocIndexOf(array, key) {
  	  var length = array.length;
  	  while (length--) {
  	    if (eq(array[length][0], key)) {
  	      return length;
  	    }
  	  }
  	  return -1;
  	}
  	_assocIndexOf = assocIndexOf;
  	return _assocIndexOf;
  }

  var _listCacheDelete;
  var hasRequired_listCacheDelete;

  function require_listCacheDelete () {
  	if (hasRequired_listCacheDelete) return _listCacheDelete;
  	hasRequired_listCacheDelete = 1;
  	var assocIndexOf = require_assocIndexOf();

  	/** Used for built-in method references. */
  	var arrayProto = Array.prototype;

  	/** Built-in value references. */
  	var splice = arrayProto.splice;

  	/**
  	 * Removes `key` and its value from the list cache.
  	 *
  	 * @private
  	 * @name delete
  	 * @memberOf ListCache
  	 * @param {string} key The key of the value to remove.
  	 * @returns {boolean} Returns `true` if the entry was removed, else `false`.
  	 */
  	function listCacheDelete(key) {
  	  var data = this.__data__,
  	    index = assocIndexOf(data, key);
  	  if (index < 0) {
  	    return false;
  	  }
  	  var lastIndex = data.length - 1;
  	  if (index == lastIndex) {
  	    data.pop();
  	  } else {
  	    splice.call(data, index, 1);
  	  }
  	  --this.size;
  	  return true;
  	}
  	_listCacheDelete = listCacheDelete;
  	return _listCacheDelete;
  }

  var _listCacheGet;
  var hasRequired_listCacheGet;

  function require_listCacheGet () {
  	if (hasRequired_listCacheGet) return _listCacheGet;
  	hasRequired_listCacheGet = 1;
  	var assocIndexOf = require_assocIndexOf();

  	/**
  	 * Gets the list cache value for `key`.
  	 *
  	 * @private
  	 * @name get
  	 * @memberOf ListCache
  	 * @param {string} key The key of the value to get.
  	 * @returns {*} Returns the entry value.
  	 */
  	function listCacheGet(key) {
  	  var data = this.__data__,
  	    index = assocIndexOf(data, key);
  	  return index < 0 ? undefined : data[index][1];
  	}
  	_listCacheGet = listCacheGet;
  	return _listCacheGet;
  }

  var _listCacheHas;
  var hasRequired_listCacheHas;

  function require_listCacheHas () {
  	if (hasRequired_listCacheHas) return _listCacheHas;
  	hasRequired_listCacheHas = 1;
  	var assocIndexOf = require_assocIndexOf();

  	/**
  	 * Checks if a list cache value for `key` exists.
  	 *
  	 * @private
  	 * @name has
  	 * @memberOf ListCache
  	 * @param {string} key The key of the entry to check.
  	 * @returns {boolean} Returns `true` if an entry for `key` exists, else `false`.
  	 */
  	function listCacheHas(key) {
  	  return assocIndexOf(this.__data__, key) > -1;
  	}
  	_listCacheHas = listCacheHas;
  	return _listCacheHas;
  }

  var _listCacheSet;
  var hasRequired_listCacheSet;

  function require_listCacheSet () {
  	if (hasRequired_listCacheSet) return _listCacheSet;
  	hasRequired_listCacheSet = 1;
  	var assocIndexOf = require_assocIndexOf();

  	/**
  	 * Sets the list cache `key` to `value`.
  	 *
  	 * @private
  	 * @name set
  	 * @memberOf ListCache
  	 * @param {string} key The key of the value to set.
  	 * @param {*} value The value to set.
  	 * @returns {Object} Returns the list cache instance.
  	 */
  	function listCacheSet(key, value) {
  	  var data = this.__data__,
  	    index = assocIndexOf(data, key);
  	  if (index < 0) {
  	    ++this.size;
  	    data.push([key, value]);
  	  } else {
  	    data[index][1] = value;
  	  }
  	  return this;
  	}
  	_listCacheSet = listCacheSet;
  	return _listCacheSet;
  }

  var _ListCache;
  var hasRequired_ListCache;

  function require_ListCache () {
  	if (hasRequired_ListCache) return _ListCache;
  	hasRequired_ListCache = 1;
  	var listCacheClear = require_listCacheClear(),
  	  listCacheDelete = require_listCacheDelete(),
  	  listCacheGet = require_listCacheGet(),
  	  listCacheHas = require_listCacheHas(),
  	  listCacheSet = require_listCacheSet();

  	/**
  	 * Creates an list cache object.
  	 *
  	 * @private
  	 * @constructor
  	 * @param {Array} [entries] The key-value pairs to cache.
  	 */
  	function ListCache(entries) {
  	  var index = -1,
  	    length = entries == null ? 0 : entries.length;
  	  this.clear();
  	  while (++index < length) {
  	    var entry = entries[index];
  	    this.set(entry[0], entry[1]);
  	  }
  	}

  	// Add methods to `ListCache`.
  	ListCache.prototype.clear = listCacheClear;
  	ListCache.prototype['delete'] = listCacheDelete;
  	ListCache.prototype.get = listCacheGet;
  	ListCache.prototype.has = listCacheHas;
  	ListCache.prototype.set = listCacheSet;
  	_ListCache = ListCache;
  	return _ListCache;
  }

  var _Map;
  var hasRequired_Map;

  function require_Map () {
  	if (hasRequired_Map) return _Map;
  	hasRequired_Map = 1;
  	var getNative = require_getNative(),
  	  root = require_root();

  	/* Built-in method references that are verified to be native. */
  	var Map = getNative(root, 'Map');
  	_Map = Map;
  	return _Map;
  }

  var _mapCacheClear;
  var hasRequired_mapCacheClear;

  function require_mapCacheClear () {
  	if (hasRequired_mapCacheClear) return _mapCacheClear;
  	hasRequired_mapCacheClear = 1;
  	var Hash = require_Hash(),
  	  ListCache = require_ListCache(),
  	  Map = require_Map();

  	/**
  	 * Removes all key-value entries from the map.
  	 *
  	 * @private
  	 * @name clear
  	 * @memberOf MapCache
  	 */
  	function mapCacheClear() {
  	  this.size = 0;
  	  this.__data__ = {
  	    'hash': new Hash(),
  	    'map': new (Map || ListCache)(),
  	    'string': new Hash()
  	  };
  	}
  	_mapCacheClear = mapCacheClear;
  	return _mapCacheClear;
  }

  /**
   * Checks if `value` is suitable for use as unique object key.
   *
   * @private
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is suitable, else `false`.
   */

  var _isKeyable;
  var hasRequired_isKeyable;

  function require_isKeyable () {
  	if (hasRequired_isKeyable) return _isKeyable;
  	hasRequired_isKeyable = 1;
  	function isKeyable(value) {
  	  var type = typeof value;
  	  return type == 'string' || type == 'number' || type == 'symbol' || type == 'boolean' ? value !== '__proto__' : value === null;
  	}
  	_isKeyable = isKeyable;
  	return _isKeyable;
  }

  var _getMapData;
  var hasRequired_getMapData;

  function require_getMapData () {
  	if (hasRequired_getMapData) return _getMapData;
  	hasRequired_getMapData = 1;
  	var isKeyable = require_isKeyable();

  	/**
  	 * Gets the data for `map`.
  	 *
  	 * @private
  	 * @param {Object} map The map to query.
  	 * @param {string} key The reference key.
  	 * @returns {*} Returns the map data.
  	 */
  	function getMapData(map, key) {
  	  var data = map.__data__;
  	  return isKeyable(key) ? data[typeof key == 'string' ? 'string' : 'hash'] : data.map;
  	}
  	_getMapData = getMapData;
  	return _getMapData;
  }

  var _mapCacheDelete;
  var hasRequired_mapCacheDelete;

  function require_mapCacheDelete () {
  	if (hasRequired_mapCacheDelete) return _mapCacheDelete;
  	hasRequired_mapCacheDelete = 1;
  	var getMapData = require_getMapData();

  	/**
  	 * Removes `key` and its value from the map.
  	 *
  	 * @private
  	 * @name delete
  	 * @memberOf MapCache
  	 * @param {string} key The key of the value to remove.
  	 * @returns {boolean} Returns `true` if the entry was removed, else `false`.
  	 */
  	function mapCacheDelete(key) {
  	  var result = getMapData(this, key)['delete'](key);
  	  this.size -= result ? 1 : 0;
  	  return result;
  	}
  	_mapCacheDelete = mapCacheDelete;
  	return _mapCacheDelete;
  }

  var _mapCacheGet;
  var hasRequired_mapCacheGet;

  function require_mapCacheGet () {
  	if (hasRequired_mapCacheGet) return _mapCacheGet;
  	hasRequired_mapCacheGet = 1;
  	var getMapData = require_getMapData();

  	/**
  	 * Gets the map value for `key`.
  	 *
  	 * @private
  	 * @name get
  	 * @memberOf MapCache
  	 * @param {string} key The key of the value to get.
  	 * @returns {*} Returns the entry value.
  	 */
  	function mapCacheGet(key) {
  	  return getMapData(this, key).get(key);
  	}
  	_mapCacheGet = mapCacheGet;
  	return _mapCacheGet;
  }

  var _mapCacheHas;
  var hasRequired_mapCacheHas;

  function require_mapCacheHas () {
  	if (hasRequired_mapCacheHas) return _mapCacheHas;
  	hasRequired_mapCacheHas = 1;
  	var getMapData = require_getMapData();

  	/**
  	 * Checks if a map value for `key` exists.
  	 *
  	 * @private
  	 * @name has
  	 * @memberOf MapCache
  	 * @param {string} key The key of the entry to check.
  	 * @returns {boolean} Returns `true` if an entry for `key` exists, else `false`.
  	 */
  	function mapCacheHas(key) {
  	  return getMapData(this, key).has(key);
  	}
  	_mapCacheHas = mapCacheHas;
  	return _mapCacheHas;
  }

  var _mapCacheSet;
  var hasRequired_mapCacheSet;

  function require_mapCacheSet () {
  	if (hasRequired_mapCacheSet) return _mapCacheSet;
  	hasRequired_mapCacheSet = 1;
  	var getMapData = require_getMapData();

  	/**
  	 * Sets the map `key` to `value`.
  	 *
  	 * @private
  	 * @name set
  	 * @memberOf MapCache
  	 * @param {string} key The key of the value to set.
  	 * @param {*} value The value to set.
  	 * @returns {Object} Returns the map cache instance.
  	 */
  	function mapCacheSet(key, value) {
  	  var data = getMapData(this, key),
  	    size = data.size;
  	  data.set(key, value);
  	  this.size += data.size == size ? 0 : 1;
  	  return this;
  	}
  	_mapCacheSet = mapCacheSet;
  	return _mapCacheSet;
  }

  var _MapCache;
  var hasRequired_MapCache;

  function require_MapCache () {
  	if (hasRequired_MapCache) return _MapCache;
  	hasRequired_MapCache = 1;
  	var mapCacheClear = require_mapCacheClear(),
  	  mapCacheDelete = require_mapCacheDelete(),
  	  mapCacheGet = require_mapCacheGet(),
  	  mapCacheHas = require_mapCacheHas(),
  	  mapCacheSet = require_mapCacheSet();

  	/**
  	 * Creates a map cache object to store key-value pairs.
  	 *
  	 * @private
  	 * @constructor
  	 * @param {Array} [entries] The key-value pairs to cache.
  	 */
  	function MapCache(entries) {
  	  var index = -1,
  	    length = entries == null ? 0 : entries.length;
  	  this.clear();
  	  while (++index < length) {
  	    var entry = entries[index];
  	    this.set(entry[0], entry[1]);
  	  }
  	}

  	// Add methods to `MapCache`.
  	MapCache.prototype.clear = mapCacheClear;
  	MapCache.prototype['delete'] = mapCacheDelete;
  	MapCache.prototype.get = mapCacheGet;
  	MapCache.prototype.has = mapCacheHas;
  	MapCache.prototype.set = mapCacheSet;
  	_MapCache = MapCache;
  	return _MapCache;
  }

  var memoize_1;
  var hasRequiredMemoize;

  function requireMemoize () {
  	if (hasRequiredMemoize) return memoize_1;
  	hasRequiredMemoize = 1;
  	var MapCache = require_MapCache();

  	/** Error message constants. */
  	var FUNC_ERROR_TEXT = 'Expected a function';

  	/**
  	 * Creates a function that memoizes the result of `func`. If `resolver` is
  	 * provided, it determines the cache key for storing the result based on the
  	 * arguments provided to the memoized function. By default, the first argument
  	 * provided to the memoized function is used as the map cache key. The `func`
  	 * is invoked with the `this` binding of the memoized function.
  	 *
  	 * **Note:** The cache is exposed as the `cache` property on the memoized
  	 * function. Its creation may be customized by replacing the `_.memoize.Cache`
  	 * constructor with one whose instances implement the
  	 * [`Map`](http://ecma-international.org/ecma-262/7.0/#sec-properties-of-the-map-prototype-object)
  	 * method interface of `clear`, `delete`, `get`, `has`, and `set`.
  	 *
  	 * @static
  	 * @memberOf _
  	 * @since 0.1.0
  	 * @category Function
  	 * @param {Function} func The function to have its output memoized.
  	 * @param {Function} [resolver] The function to resolve the cache key.
  	 * @returns {Function} Returns the new memoized function.
  	 * @example
  	 *
  	 * var object = { 'a': 1, 'b': 2 };
  	 * var other = { 'c': 3, 'd': 4 };
  	 *
  	 * var values = _.memoize(_.values);
  	 * values(object);
  	 * // => [1, 2]
  	 *
  	 * values(other);
  	 * // => [3, 4]
  	 *
  	 * object.a = 2;
  	 * values(object);
  	 * // => [1, 2]
  	 *
  	 * // Modify the result cache.
  	 * values.cache.set(object, ['a', 'b']);
  	 * values(object);
  	 * // => ['a', 'b']
  	 *
  	 * // Replace `_.memoize.Cache`.
  	 * _.memoize.Cache = WeakMap;
  	 */
  	function memoize(func, resolver) {
  	  if (typeof func != 'function' || resolver != null && typeof resolver != 'function') {
  	    throw new TypeError(FUNC_ERROR_TEXT);
  	  }
  	  var memoized = function () {
  	    var args = arguments,
  	      key = resolver ? resolver.apply(this, args) : args[0],
  	      cache = memoized.cache;
  	    if (cache.has(key)) {
  	      return cache.get(key);
  	    }
  	    var result = func.apply(this, args);
  	    memoized.cache = cache.set(key, result) || cache;
  	    return result;
  	  };
  	  memoized.cache = new (memoize.Cache || MapCache)();
  	  return memoized;
  	}

  	// Expose `MapCache`.
  	memoize.Cache = MapCache;
  	memoize_1 = memoize;
  	return memoize_1;
  }

  var _memoizeCapped;
  var hasRequired_memoizeCapped;

  function require_memoizeCapped () {
  	if (hasRequired_memoizeCapped) return _memoizeCapped;
  	hasRequired_memoizeCapped = 1;
  	var memoize = requireMemoize();

  	/** Used as the maximum memoize cache size. */
  	var MAX_MEMOIZE_SIZE = 500;

  	/**
  	 * A specialized version of `_.memoize` which clears the memoized function's
  	 * cache when it exceeds `MAX_MEMOIZE_SIZE`.
  	 *
  	 * @private
  	 * @param {Function} func The function to have its output memoized.
  	 * @returns {Function} Returns the new memoized function.
  	 */
  	function memoizeCapped(func) {
  	  var result = memoize(func, function (key) {
  	    if (cache.size === MAX_MEMOIZE_SIZE) {
  	      cache.clear();
  	    }
  	    return key;
  	  });
  	  var cache = result.cache;
  	  return result;
  	}
  	_memoizeCapped = memoizeCapped;
  	return _memoizeCapped;
  }

  var _stringToPath;
  var hasRequired_stringToPath;

  function require_stringToPath () {
  	if (hasRequired_stringToPath) return _stringToPath;
  	hasRequired_stringToPath = 1;
  	var memoizeCapped = require_memoizeCapped();

  	/** Used to match property names within property paths. */
  	var rePropName = /[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g;

  	/** Used to match backslashes in property paths. */
  	var reEscapeChar = /\\(\\)?/g;

  	/**
  	 * Converts `string` to a property path array.
  	 *
  	 * @private
  	 * @param {string} string The string to convert.
  	 * @returns {Array} Returns the property path array.
  	 */
  	var stringToPath = memoizeCapped(function (string) {
  	  var result = [];
  	  if (string.charCodeAt(0) === 46 /* . */) {
  	    result.push('');
  	  }
  	  string.replace(rePropName, function (match, number, quote, subString) {
  	    result.push(quote ? subString.replace(reEscapeChar, '$1') : number || match);
  	  });
  	  return result;
  	});
  	_stringToPath = stringToPath;
  	return _stringToPath;
  }

  /**
   * A specialized version of `_.map` for arrays without support for iteratee
   * shorthands.
   *
   * @private
   * @param {Array} [array] The array to iterate over.
   * @param {Function} iteratee The function invoked per iteration.
   * @returns {Array} Returns the new mapped array.
   */

  var _arrayMap;
  var hasRequired_arrayMap;

  function require_arrayMap () {
  	if (hasRequired_arrayMap) return _arrayMap;
  	hasRequired_arrayMap = 1;
  	function arrayMap(array, iteratee) {
  	  var index = -1,
  	    length = array == null ? 0 : array.length,
  	    result = Array(length);
  	  while (++index < length) {
  	    result[index] = iteratee(array[index], index, array);
  	  }
  	  return result;
  	}
  	_arrayMap = arrayMap;
  	return _arrayMap;
  }

  var _baseToString;
  var hasRequired_baseToString;

  function require_baseToString () {
  	if (hasRequired_baseToString) return _baseToString;
  	hasRequired_baseToString = 1;
  	var Symbol = require_Symbol(),
  	  arrayMap = require_arrayMap(),
  	  isArray = requireIsArray(),
  	  isSymbol = requireIsSymbol();

  	/** Used as references for various `Number` constants. */
  	var INFINITY = 1 / 0;

  	/** Used to convert symbols to primitives and strings. */
  	var symbolProto = Symbol ? Symbol.prototype : undefined,
  	  symbolToString = symbolProto ? symbolProto.toString : undefined;

  	/**
  	 * The base implementation of `_.toString` which doesn't convert nullish
  	 * values to empty strings.
  	 *
  	 * @private
  	 * @param {*} value The value to process.
  	 * @returns {string} Returns the string.
  	 */
  	function baseToString(value) {
  	  // Exit early for strings to avoid a performance hit in some environments.
  	  if (typeof value == 'string') {
  	    return value;
  	  }
  	  if (isArray(value)) {
  	    // Recursively convert values (susceptible to call stack limits).
  	    return arrayMap(value, baseToString) + '';
  	  }
  	  if (isSymbol(value)) {
  	    return symbolToString ? symbolToString.call(value) : '';
  	  }
  	  var result = value + '';
  	  return result == '0' && 1 / value == -INFINITY ? '-0' : result;
  	}
  	_baseToString = baseToString;
  	return _baseToString;
  }

  var toString_1;
  var hasRequiredToString;

  function requireToString () {
  	if (hasRequiredToString) return toString_1;
  	hasRequiredToString = 1;
  	var baseToString = require_baseToString();

  	/**
  	 * Converts `value` to a string. An empty string is returned for `null`
  	 * and `undefined` values. The sign of `-0` is preserved.
  	 *
  	 * @static
  	 * @memberOf _
  	 * @since 4.0.0
  	 * @category Lang
  	 * @param {*} value The value to convert.
  	 * @returns {string} Returns the converted string.
  	 * @example
  	 *
  	 * _.toString(null);
  	 * // => ''
  	 *
  	 * _.toString(-0);
  	 * // => '-0'
  	 *
  	 * _.toString([1, 2, 3]);
  	 * // => '1,2,3'
  	 */
  	function toString(value) {
  	  return value == null ? '' : baseToString(value);
  	}
  	toString_1 = toString;
  	return toString_1;
  }

  var _castPath;
  var hasRequired_castPath;

  function require_castPath () {
  	if (hasRequired_castPath) return _castPath;
  	hasRequired_castPath = 1;
  	var isArray = requireIsArray(),
  	  isKey = require_isKey(),
  	  stringToPath = require_stringToPath(),
  	  toString = requireToString();

  	/**
  	 * Casts `value` to a path array if it's not one.
  	 *
  	 * @private
  	 * @param {*} value The value to inspect.
  	 * @param {Object} [object] The object to query keys on.
  	 * @returns {Array} Returns the cast property path array.
  	 */
  	function castPath(value, object) {
  	  if (isArray(value)) {
  	    return value;
  	  }
  	  return isKey(value, object) ? [value] : stringToPath(toString(value));
  	}
  	_castPath = castPath;
  	return _castPath;
  }

  var _toKey;
  var hasRequired_toKey;

  function require_toKey () {
  	if (hasRequired_toKey) return _toKey;
  	hasRequired_toKey = 1;
  	var isSymbol = requireIsSymbol();

  	/** Used as references for various `Number` constants. */
  	var INFINITY = 1 / 0;

  	/**
  	 * Converts `value` to a string key if it's not a string or symbol.
  	 *
  	 * @private
  	 * @param {*} value The value to inspect.
  	 * @returns {string|symbol} Returns the key.
  	 */
  	function toKey(value) {
  	  if (typeof value == 'string' || isSymbol(value)) {
  	    return value;
  	  }
  	  var result = value + '';
  	  return result == '0' && 1 / value == -INFINITY ? '-0' : result;
  	}
  	_toKey = toKey;
  	return _toKey;
  }

  var _baseGet;
  var hasRequired_baseGet;

  function require_baseGet () {
  	if (hasRequired_baseGet) return _baseGet;
  	hasRequired_baseGet = 1;
  	var castPath = require_castPath(),
  	  toKey = require_toKey();

  	/**
  	 * The base implementation of `_.get` without support for default values.
  	 *
  	 * @private
  	 * @param {Object} object The object to query.
  	 * @param {Array|string} path The path of the property to get.
  	 * @returns {*} Returns the resolved value.
  	 */
  	function baseGet(object, path) {
  	  path = castPath(path, object);
  	  var index = 0,
  	    length = path.length;
  	  while (object != null && index < length) {
  	    object = object[toKey(path[index++])];
  	  }
  	  return index && index == length ? object : undefined;
  	}
  	_baseGet = baseGet;
  	return _baseGet;
  }

  var get_1;
  var hasRequiredGet;

  function requireGet () {
  	if (hasRequiredGet) return get_1;
  	hasRequiredGet = 1;
  	var baseGet = require_baseGet();

  	/**
  	 * Gets the value at `path` of `object`. If the resolved value is
  	 * `undefined`, the `defaultValue` is returned in its place.
  	 *
  	 * @static
  	 * @memberOf _
  	 * @since 3.7.0
  	 * @category Object
  	 * @param {Object} object The object to query.
  	 * @param {Array|string} path The path of the property to get.
  	 * @param {*} [defaultValue] The value returned for `undefined` resolved values.
  	 * @returns {*} Returns the resolved value.
  	 * @example
  	 *
  	 * var object = { 'a': [{ 'b': { 'c': 3 } }] };
  	 *
  	 * _.get(object, 'a[0].b.c');
  	 * // => 3
  	 *
  	 * _.get(object, ['a', '0', 'b', 'c']);
  	 * // => 3
  	 *
  	 * _.get(object, 'a.b.c', 'default');
  	 * // => 'default'
  	 */
  	function get(object, path, defaultValue) {
  	  var result = object == null ? undefined : baseGet(object, path);
  	  return result === undefined ? defaultValue : result;
  	}
  	get_1 = get;
  	return get_1;
  }

  requireGet();

  var frontMatter = {exports: {}};

  var jsYaml$1 = {};

  var loader = {};

  var common = {};

  var hasRequiredCommon;

  function requireCommon () {
  	if (hasRequiredCommon) return common;
  	hasRequiredCommon = 1;

  	function isNothing(subject) {
  	  return typeof subject === 'undefined' || subject === null;
  	}
  	function isObject(subject) {
  	  return typeof subject === 'object' && subject !== null;
  	}
  	function toArray(sequence) {
  	  if (Array.isArray(sequence)) return sequence;else if (isNothing(sequence)) return [];
  	  return [sequence];
  	}
  	function extend(target, source) {
  	  var index, length, key, sourceKeys;
  	  if (source) {
  	    sourceKeys = Object.keys(source);
  	    for (index = 0, length = sourceKeys.length; index < length; index += 1) {
  	      key = sourceKeys[index];
  	      target[key] = source[key];
  	    }
  	  }
  	  return target;
  	}
  	function repeat(string, count) {
  	  var result = '',
  	    cycle;
  	  for (cycle = 0; cycle < count; cycle += 1) {
  	    result += string;
  	  }
  	  return result;
  	}
  	function isNegativeZero(number) {
  	  return number === 0 && Number.NEGATIVE_INFINITY === 1 / number;
  	}
  	common.isNothing = isNothing;
  	common.isObject = isObject;
  	common.toArray = toArray;
  	common.repeat = repeat;
  	common.isNegativeZero = isNegativeZero;
  	common.extend = extend;
  	return common;
  }

  var exception;
  var hasRequiredException;

  function requireException () {
  	if (hasRequiredException) return exception;
  	hasRequiredException = 1;

  	function YAMLException(reason, mark) {
  	  // Super constructor
  	  Error.call(this);
  	  this.name = 'YAMLException';
  	  this.reason = reason;
  	  this.mark = mark;
  	  this.message = (this.reason || '(unknown reason)') + (this.mark ? ' ' + this.mark.toString() : '');

  	  // Include stack trace in error object
  	  if (Error.captureStackTrace) {
  	    // Chrome and NodeJS
  	    Error.captureStackTrace(this, this.constructor);
  	  } else {
  	    // FF, IE 10+ and Safari 6+. Fallback for others
  	    this.stack = new Error().stack || '';
  	  }
  	}

  	// Inherit from Error
  	YAMLException.prototype = Object.create(Error.prototype);
  	YAMLException.prototype.constructor = YAMLException;
  	YAMLException.prototype.toString = function toString(compact) {
  	  var result = this.name + ': ';
  	  result += this.reason || '(unknown reason)';
  	  if (!compact && this.mark) {
  	    result += ' ' + this.mark.toString();
  	  }
  	  return result;
  	};
  	exception = YAMLException;
  	return exception;
  }

  var mark;
  var hasRequiredMark;

  function requireMark () {
  	if (hasRequiredMark) return mark;
  	hasRequiredMark = 1;

  	var common = requireCommon();
  	function Mark(name, buffer, position, line, column) {
  	  this.name = name;
  	  this.buffer = buffer;
  	  this.position = position;
  	  this.line = line;
  	  this.column = column;
  	}
  	Mark.prototype.getSnippet = function getSnippet(indent, maxLength) {
  	  var head, start, tail, end, snippet;
  	  if (!this.buffer) return null;
  	  indent = indent || 4;
  	  maxLength = maxLength || 75;
  	  head = '';
  	  start = this.position;
  	  while (start > 0 && '\x00\r\n\x85\u2028\u2029'.indexOf(this.buffer.charAt(start - 1)) === -1) {
  	    start -= 1;
  	    if (this.position - start > maxLength / 2 - 1) {
  	      head = ' ... ';
  	      start += 5;
  	      break;
  	    }
  	  }
  	  tail = '';
  	  end = this.position;
  	  while (end < this.buffer.length && '\x00\r\n\x85\u2028\u2029'.indexOf(this.buffer.charAt(end)) === -1) {
  	    end += 1;
  	    if (end - this.position > maxLength / 2 - 1) {
  	      tail = ' ... ';
  	      end -= 5;
  	      break;
  	    }
  	  }
  	  snippet = this.buffer.slice(start, end);
  	  return common.repeat(' ', indent) + head + snippet + tail + '\n' + common.repeat(' ', indent + this.position - start + head.length) + '^';
  	};
  	Mark.prototype.toString = function toString(compact) {
  	  var snippet,
  	    where = '';
  	  if (this.name) {
  	    where += 'in "' + this.name + '" ';
  	  }
  	  where += 'at line ' + (this.line + 1) + ', column ' + (this.column + 1);
  	  if (!compact) {
  	    snippet = this.getSnippet();
  	    if (snippet) {
  	      where += ':\n' + snippet;
  	    }
  	  }
  	  return where;
  	};
  	mark = Mark;
  	return mark;
  }

  var type;
  var hasRequiredType;

  function requireType () {
  	if (hasRequiredType) return type;
  	hasRequiredType = 1;

  	var YAMLException = requireException();
  	var TYPE_CONSTRUCTOR_OPTIONS = ['kind', 'resolve', 'construct', 'instanceOf', 'predicate', 'represent', 'defaultStyle', 'styleAliases'];
  	var YAML_NODE_KINDS = ['scalar', 'sequence', 'mapping'];
  	function compileStyleAliases(map) {
  	  var result = {};
  	  if (map !== null) {
  	    Object.keys(map).forEach(function (style) {
  	      map[style].forEach(function (alias) {
  	        result[String(alias)] = style;
  	      });
  	    });
  	  }
  	  return result;
  	}
  	function Type(tag, options) {
  	  options = options || {};
  	  Object.keys(options).forEach(function (name) {
  	    if (TYPE_CONSTRUCTOR_OPTIONS.indexOf(name) === -1) {
  	      throw new YAMLException('Unknown option "' + name + '" is met in definition of "' + tag + '" YAML type.');
  	    }
  	  });

  	  // TODO: Add tag format check.
  	  this.tag = tag;
  	  this.kind = options['kind'] || null;
  	  this.resolve = options['resolve'] || function () {
  	    return true;
  	  };
  	  this.construct = options['construct'] || function (data) {
  	    return data;
  	  };
  	  this.instanceOf = options['instanceOf'] || null;
  	  this.predicate = options['predicate'] || null;
  	  this.represent = options['represent'] || null;
  	  this.defaultStyle = options['defaultStyle'] || null;
  	  this.styleAliases = compileStyleAliases(options['styleAliases'] || null);
  	  if (YAML_NODE_KINDS.indexOf(this.kind) === -1) {
  	    throw new YAMLException('Unknown kind "' + this.kind + '" is specified for "' + tag + '" YAML type.');
  	  }
  	}
  	type = Type;
  	return type;
  }

  var schema;
  var hasRequiredSchema;

  function requireSchema () {
  	if (hasRequiredSchema) return schema;
  	hasRequiredSchema = 1;

  	/*eslint-disable max-len*/
  	var common = requireCommon();
  	var YAMLException = requireException();
  	var Type = requireType();
  	function compileList(schema, name, result) {
  	  var exclude = [];
  	  schema.include.forEach(function (includedSchema) {
  	    result = compileList(includedSchema, name, result);
  	  });
  	  schema[name].forEach(function (currentType) {
  	    result.forEach(function (previousType, previousIndex) {
  	      if (previousType.tag === currentType.tag && previousType.kind === currentType.kind) {
  	        exclude.push(previousIndex);
  	      }
  	    });
  	    result.push(currentType);
  	  });
  	  return result.filter(function (type, index) {
  	    return exclude.indexOf(index) === -1;
  	  });
  	}
  	function compileMap(/* lists... */
  	) {
  	  var result = {
  	      scalar: {},
  	      sequence: {},
  	      mapping: {},
  	      fallback: {}
  	    },
  	    index,
  	    length;
  	  function collectType(type) {
  	    result[type.kind][type.tag] = result['fallback'][type.tag] = type;
  	  }
  	  for (index = 0, length = arguments.length; index < length; index += 1) {
  	    arguments[index].forEach(collectType);
  	  }
  	  return result;
  	}
  	function Schema(definition) {
  	  this.include = definition.include || [];
  	  this.implicit = definition.implicit || [];
  	  this.explicit = definition.explicit || [];
  	  this.implicit.forEach(function (type) {
  	    if (type.loadKind && type.loadKind !== 'scalar') {
  	      throw new YAMLException('There is a non-scalar type in the implicit list of a schema. Implicit resolving of such types is not supported.');
  	    }
  	  });
  	  this.compiledImplicit = compileList(this, 'implicit', []);
  	  this.compiledExplicit = compileList(this, 'explicit', []);
  	  this.compiledTypeMap = compileMap(this.compiledImplicit, this.compiledExplicit);
  	}
  	Schema.DEFAULT = null;
  	Schema.create = function createSchema() {
  	  var schemas, types;
  	  switch (arguments.length) {
  	    case 1:
  	      schemas = Schema.DEFAULT;
  	      types = arguments[0];
  	      break;
  	    case 2:
  	      schemas = arguments[0];
  	      types = arguments[1];
  	      break;
  	    default:
  	      throw new YAMLException('Wrong number of arguments for Schema.create function');
  	  }
  	  schemas = common.toArray(schemas);
  	  types = common.toArray(types);
  	  if (!schemas.every(function (schema) {
  	    return schema instanceof Schema;
  	  })) {
  	    throw new YAMLException('Specified list of super schemas (or a single Schema object) contains a non-Schema object.');
  	  }
  	  if (!types.every(function (type) {
  	    return type instanceof Type;
  	  })) {
  	    throw new YAMLException('Specified list of YAML types (or a single Type object) contains a non-Type object.');
  	  }
  	  return new Schema({
  	    include: schemas,
  	    explicit: types
  	  });
  	};
  	schema = Schema;
  	return schema;
  }

  var str;
  var hasRequiredStr;

  function requireStr () {
  	if (hasRequiredStr) return str;
  	hasRequiredStr = 1;

  	var Type = requireType();
  	str = new Type('tag:yaml.org,2002:str', {
  	  kind: 'scalar',
  	  construct: function (data) {
  	    return data !== null ? data : '';
  	  }
  	});
  	return str;
  }

  var seq;
  var hasRequiredSeq;

  function requireSeq () {
  	if (hasRequiredSeq) return seq;
  	hasRequiredSeq = 1;

  	var Type = requireType();
  	seq = new Type('tag:yaml.org,2002:seq', {
  	  kind: 'sequence',
  	  construct: function (data) {
  	    return data !== null ? data : [];
  	  }
  	});
  	return seq;
  }

  var map;
  var hasRequiredMap;

  function requireMap () {
  	if (hasRequiredMap) return map;
  	hasRequiredMap = 1;

  	var Type = requireType();
  	map = new Type('tag:yaml.org,2002:map', {
  	  kind: 'mapping',
  	  construct: function (data) {
  	    return data !== null ? data : {};
  	  }
  	});
  	return map;
  }

  var failsafe;
  var hasRequiredFailsafe;

  function requireFailsafe () {
  	if (hasRequiredFailsafe) return failsafe;
  	hasRequiredFailsafe = 1;

  	var Schema = requireSchema();
  	failsafe = new Schema({
  	  explicit: [requireStr(), requireSeq(), requireMap()]
  	});
  	return failsafe;
  }

  var _null;
  var hasRequired_null;

  function require_null () {
  	if (hasRequired_null) return _null;
  	hasRequired_null = 1;

  	var Type = requireType();
  	function resolveYamlNull(data) {
  	  if (data === null) return true;
  	  var max = data.length;
  	  return max === 1 && data === '~' || max === 4 && (data === 'null' || data === 'Null' || data === 'NULL');
  	}
  	function constructYamlNull() {
  	  return null;
  	}
  	function isNull(object) {
  	  return object === null;
  	}
  	_null = new Type('tag:yaml.org,2002:null', {
  	  kind: 'scalar',
  	  resolve: resolveYamlNull,
  	  construct: constructYamlNull,
  	  predicate: isNull,
  	  represent: {
  	    canonical: function () {
  	      return '~';
  	    },
  	    lowercase: function () {
  	      return 'null';
  	    },
  	    uppercase: function () {
  	      return 'NULL';
  	    },
  	    camelcase: function () {
  	      return 'Null';
  	    }
  	  },
  	  defaultStyle: 'lowercase'
  	});
  	return _null;
  }

  var bool;
  var hasRequiredBool;

  function requireBool () {
  	if (hasRequiredBool) return bool;
  	hasRequiredBool = 1;

  	var Type = requireType();
  	function resolveYamlBoolean(data) {
  	  if (data === null) return false;
  	  var max = data.length;
  	  return max === 4 && (data === 'true' || data === 'True' || data === 'TRUE') || max === 5 && (data === 'false' || data === 'False' || data === 'FALSE');
  	}
  	function constructYamlBoolean(data) {
  	  return data === 'true' || data === 'True' || data === 'TRUE';
  	}
  	function isBoolean(object) {
  	  return Object.prototype.toString.call(object) === '[object Boolean]';
  	}
  	bool = new Type('tag:yaml.org,2002:bool', {
  	  kind: 'scalar',
  	  resolve: resolveYamlBoolean,
  	  construct: constructYamlBoolean,
  	  predicate: isBoolean,
  	  represent: {
  	    lowercase: function (object) {
  	      return object ? 'true' : 'false';
  	    },
  	    uppercase: function (object) {
  	      return object ? 'TRUE' : 'FALSE';
  	    },
  	    camelcase: function (object) {
  	      return object ? 'True' : 'False';
  	    }
  	  },
  	  defaultStyle: 'lowercase'
  	});
  	return bool;
  }

  var int;
  var hasRequiredInt;

  function requireInt () {
  	if (hasRequiredInt) return int;
  	hasRequiredInt = 1;

  	var common = requireCommon();
  	var Type = requireType();
  	function isHexCode(c) {
  	  return 0x30 /* 0 */ <= c && c <= 0x39 /* 9 */ || 0x41 /* A */ <= c && c <= 0x46 /* F */ || 0x61 /* a */ <= c && c <= 0x66 /* f */;
  	}
  	function isOctCode(c) {
  	  return 0x30 /* 0 */ <= c && c <= 0x37 /* 7 */;
  	}
  	function isDecCode(c) {
  	  return 0x30 /* 0 */ <= c && c <= 0x39 /* 9 */;
  	}
  	function resolveYamlInteger(data) {
  	  if (data === null) return false;
  	  var max = data.length,
  	    index = 0,
  	    hasDigits = false,
  	    ch;
  	  if (!max) return false;
  	  ch = data[index];

  	  // sign
  	  if (ch === '-' || ch === '+') {
  	    ch = data[++index];
  	  }
  	  if (ch === '0') {
  	    // 0
  	    if (index + 1 === max) return true;
  	    ch = data[++index];

  	    // base 2, base 8, base 16

  	    if (ch === 'b') {
  	      // base 2
  	      index++;
  	      for (; index < max; index++) {
  	        ch = data[index];
  	        if (ch === '_') continue;
  	        if (ch !== '0' && ch !== '1') return false;
  	        hasDigits = true;
  	      }
  	      return hasDigits && ch !== '_';
  	    }
  	    if (ch === 'x') {
  	      // base 16
  	      index++;
  	      for (; index < max; index++) {
  	        ch = data[index];
  	        if (ch === '_') continue;
  	        if (!isHexCode(data.charCodeAt(index))) return false;
  	        hasDigits = true;
  	      }
  	      return hasDigits && ch !== '_';
  	    }

  	    // base 8
  	    for (; index < max; index++) {
  	      ch = data[index];
  	      if (ch === '_') continue;
  	      if (!isOctCode(data.charCodeAt(index))) return false;
  	      hasDigits = true;
  	    }
  	    return hasDigits && ch !== '_';
  	  }

  	  // base 10 (except 0) or base 60

  	  // value should not start with `_`;
  	  if (ch === '_') return false;
  	  for (; index < max; index++) {
  	    ch = data[index];
  	    if (ch === '_') continue;
  	    if (ch === ':') break;
  	    if (!isDecCode(data.charCodeAt(index))) {
  	      return false;
  	    }
  	    hasDigits = true;
  	  }

  	  // Should have digits and should not end with `_`
  	  if (!hasDigits || ch === '_') return false;

  	  // if !base60 - done;
  	  if (ch !== ':') return true;

  	  // base60 almost not used, no needs to optimize
  	  return /^(:[0-5]?[0-9])+$/.test(data.slice(index));
  	}
  	function constructYamlInteger(data) {
  	  var value = data,
  	    sign = 1,
  	    ch,
  	    base,
  	    digits = [];
  	  if (value.indexOf('_') !== -1) {
  	    value = value.replace(/_/g, '');
  	  }
  	  ch = value[0];
  	  if (ch === '-' || ch === '+') {
  	    if (ch === '-') sign = -1;
  	    value = value.slice(1);
  	    ch = value[0];
  	  }
  	  if (value === '0') return 0;
  	  if (ch === '0') {
  	    if (value[1] === 'b') return sign * parseInt(value.slice(2), 2);
  	    if (value[1] === 'x') return sign * parseInt(value, 16);
  	    return sign * parseInt(value, 8);
  	  }
  	  if (value.indexOf(':') !== -1) {
  	    value.split(':').forEach(function (v) {
  	      digits.unshift(parseInt(v, 10));
  	    });
  	    value = 0;
  	    base = 1;
  	    digits.forEach(function (d) {
  	      value += d * base;
  	      base *= 60;
  	    });
  	    return sign * value;
  	  }
  	  return sign * parseInt(value, 10);
  	}
  	function isInteger(object) {
  	  return Object.prototype.toString.call(object) === '[object Number]' && object % 1 === 0 && !common.isNegativeZero(object);
  	}
  	int = new Type('tag:yaml.org,2002:int', {
  	  kind: 'scalar',
  	  resolve: resolveYamlInteger,
  	  construct: constructYamlInteger,
  	  predicate: isInteger,
  	  represent: {
  	    binary: function (obj) {
  	      return obj >= 0 ? '0b' + obj.toString(2) : '-0b' + obj.toString(2).slice(1);
  	    },
  	    octal: function (obj) {
  	      return obj >= 0 ? '0' + obj.toString(8) : '-0' + obj.toString(8).slice(1);
  	    },
  	    decimal: function (obj) {
  	      return obj.toString(10);
  	    },
  	    /* eslint-disable max-len */
  	    hexadecimal: function (obj) {
  	      return obj >= 0 ? '0x' + obj.toString(16).toUpperCase() : '-0x' + obj.toString(16).toUpperCase().slice(1);
  	    }
  	  },
  	  defaultStyle: 'decimal',
  	  styleAliases: {
  	    binary: [2, 'bin'],
  	    octal: [8, 'oct'],
  	    decimal: [10, 'dec'],
  	    hexadecimal: [16, 'hex']
  	  }
  	});
  	return int;
  }

  var float;
  var hasRequiredFloat;

  function requireFloat () {
  	if (hasRequiredFloat) return float;
  	hasRequiredFloat = 1;

  	var common = requireCommon();
  	var Type = requireType();
  	var YAML_FLOAT_PATTERN = new RegExp(
  	// 2.5e4, 2.5 and integers
  	'^(?:[-+]?(?:0|[1-9][0-9_]*)(?:\\.[0-9_]*)?(?:[eE][-+]?[0-9]+)?' +
  	// .2e4, .2
  	// special case, seems not from spec
  	'|\\.[0-9_]+(?:[eE][-+]?[0-9]+)?' +
  	// 20:59
  	'|[-+]?[0-9][0-9_]*(?::[0-5]?[0-9])+\\.[0-9_]*' +
  	// .inf
  	'|[-+]?\\.(?:inf|Inf|INF)' +
  	// .nan
  	'|\\.(?:nan|NaN|NAN))$');
  	function resolveYamlFloat(data) {
  	  if (data === null) return false;
  	  if (!YAML_FLOAT_PATTERN.test(data) ||
  	  // Quick hack to not allow integers end with `_`
  	  // Probably should update regexp & check speed
  	  data[data.length - 1] === '_') {
  	    return false;
  	  }
  	  return true;
  	}
  	function constructYamlFloat(data) {
  	  var value, sign, base, digits;
  	  value = data.replace(/_/g, '').toLowerCase();
  	  sign = value[0] === '-' ? -1 : 1;
  	  digits = [];
  	  if ('+-'.indexOf(value[0]) >= 0) {
  	    value = value.slice(1);
  	  }
  	  if (value === '.inf') {
  	    return sign === 1 ? Number.POSITIVE_INFINITY : Number.NEGATIVE_INFINITY;
  	  } else if (value === '.nan') {
  	    return NaN;
  	  } else if (value.indexOf(':') >= 0) {
  	    value.split(':').forEach(function (v) {
  	      digits.unshift(parseFloat(v, 10));
  	    });
  	    value = 0.0;
  	    base = 1;
  	    digits.forEach(function (d) {
  	      value += d * base;
  	      base *= 60;
  	    });
  	    return sign * value;
  	  }
  	  return sign * parseFloat(value, 10);
  	}
  	var SCIENTIFIC_WITHOUT_DOT = /^[-+]?[0-9]+e/;
  	function representYamlFloat(object, style) {
  	  var res;
  	  if (isNaN(object)) {
  	    switch (style) {
  	      case 'lowercase':
  	        return '.nan';
  	      case 'uppercase':
  	        return '.NAN';
  	      case 'camelcase':
  	        return '.NaN';
  	    }
  	  } else if (Number.POSITIVE_INFINITY === object) {
  	    switch (style) {
  	      case 'lowercase':
  	        return '.inf';
  	      case 'uppercase':
  	        return '.INF';
  	      case 'camelcase':
  	        return '.Inf';
  	    }
  	  } else if (Number.NEGATIVE_INFINITY === object) {
  	    switch (style) {
  	      case 'lowercase':
  	        return '-.inf';
  	      case 'uppercase':
  	        return '-.INF';
  	      case 'camelcase':
  	        return '-.Inf';
  	    }
  	  } else if (common.isNegativeZero(object)) {
  	    return '-0.0';
  	  }
  	  res = object.toString(10);

  	  // JS stringifier can build scientific format without dots: 5e-100,
  	  // while YAML requres dot: 5.e-100. Fix it with simple hack

  	  return SCIENTIFIC_WITHOUT_DOT.test(res) ? res.replace('e', '.e') : res;
  	}
  	function isFloat(object) {
  	  return Object.prototype.toString.call(object) === '[object Number]' && (object % 1 !== 0 || common.isNegativeZero(object));
  	}
  	float = new Type('tag:yaml.org,2002:float', {
  	  kind: 'scalar',
  	  resolve: resolveYamlFloat,
  	  construct: constructYamlFloat,
  	  predicate: isFloat,
  	  represent: representYamlFloat,
  	  defaultStyle: 'lowercase'
  	});
  	return float;
  }

  var json;
  var hasRequiredJson;

  function requireJson () {
  	if (hasRequiredJson) return json;
  	hasRequiredJson = 1;

  	var Schema = requireSchema();
  	json = new Schema({
  	  include: [requireFailsafe()],
  	  implicit: [require_null(), requireBool(), requireInt(), requireFloat()]
  	});
  	return json;
  }

  var core;
  var hasRequiredCore;

  function requireCore () {
  	if (hasRequiredCore) return core;
  	hasRequiredCore = 1;

  	var Schema = requireSchema();
  	core = new Schema({
  	  include: [requireJson()]
  	});
  	return core;
  }

  var timestamp;
  var hasRequiredTimestamp;

  function requireTimestamp () {
  	if (hasRequiredTimestamp) return timestamp;
  	hasRequiredTimestamp = 1;

  	var Type = requireType();
  	var YAML_DATE_REGEXP = new RegExp('^([0-9][0-9][0-9][0-9])' +
  	// [1] year
  	'-([0-9][0-9])' +
  	// [2] month
  	'-([0-9][0-9])$'); // [3] day

  	var YAML_TIMESTAMP_REGEXP = new RegExp('^([0-9][0-9][0-9][0-9])' +
  	// [1] year
  	'-([0-9][0-9]?)' +
  	// [2] month
  	'-([0-9][0-9]?)' +
  	// [3] day
  	'(?:[Tt]|[ \\t]+)' +
  	// ...
  	'([0-9][0-9]?)' +
  	// [4] hour
  	':([0-9][0-9])' +
  	// [5] minute
  	':([0-9][0-9])' +
  	// [6] second
  	'(?:\\.([0-9]*))?' +
  	// [7] fraction
  	'(?:[ \\t]*(Z|([-+])([0-9][0-9]?)' +
  	// [8] tz [9] tz_sign [10] tz_hour
  	'(?::([0-9][0-9]))?))?$'); // [11] tz_minute

  	function resolveYamlTimestamp(data) {
  	  if (data === null) return false;
  	  if (YAML_DATE_REGEXP.exec(data) !== null) return true;
  	  if (YAML_TIMESTAMP_REGEXP.exec(data) !== null) return true;
  	  return false;
  	}
  	function constructYamlTimestamp(data) {
  	  var match,
  	    year,
  	    month,
  	    day,
  	    hour,
  	    minute,
  	    second,
  	    fraction = 0,
  	    delta = null,
  	    tz_hour,
  	    tz_minute,
  	    date;
  	  match = YAML_DATE_REGEXP.exec(data);
  	  if (match === null) match = YAML_TIMESTAMP_REGEXP.exec(data);
  	  if (match === null) throw new Error('Date resolve error');

  	  // match: [1] year [2] month [3] day

  	  year = +match[1];
  	  month = +match[2] - 1; // JS month starts with 0
  	  day = +match[3];
  	  if (!match[4]) {
  	    // no hour
  	    return new Date(Date.UTC(year, month, day));
  	  }

  	  // match: [4] hour [5] minute [6] second [7] fraction

  	  hour = +match[4];
  	  minute = +match[5];
  	  second = +match[6];
  	  if (match[7]) {
  	    fraction = match[7].slice(0, 3);
  	    while (fraction.length < 3) {
  	      // milli-seconds
  	      fraction += '0';
  	    }
  	    fraction = +fraction;
  	  }

  	  // match: [8] tz [9] tz_sign [10] tz_hour [11] tz_minute

  	  if (match[9]) {
  	    tz_hour = +match[10];
  	    tz_minute = +(match[11] || 0);
  	    delta = (tz_hour * 60 + tz_minute) * 60000; // delta in mili-seconds
  	    if (match[9] === '-') delta = -delta;
  	  }
  	  date = new Date(Date.UTC(year, month, day, hour, minute, second, fraction));
  	  if (delta) date.setTime(date.getTime() - delta);
  	  return date;
  	}
  	function representYamlTimestamp(object /*, style*/) {
  	  return object.toISOString();
  	}
  	timestamp = new Type('tag:yaml.org,2002:timestamp', {
  	  kind: 'scalar',
  	  resolve: resolveYamlTimestamp,
  	  construct: constructYamlTimestamp,
  	  instanceOf: Date,
  	  represent: representYamlTimestamp
  	});
  	return timestamp;
  }

  var merge;
  var hasRequiredMerge;

  function requireMerge () {
  	if (hasRequiredMerge) return merge;
  	hasRequiredMerge = 1;

  	var Type = requireType();
  	function resolveYamlMerge(data) {
  	  return data === '<<' || data === null;
  	}
  	merge = new Type('tag:yaml.org,2002:merge', {
  	  kind: 'scalar',
  	  resolve: resolveYamlMerge
  	});
  	return merge;
  }

  var binary;
  var hasRequiredBinary;

  function requireBinary () {
  	if (hasRequiredBinary) return binary;
  	hasRequiredBinary = 1;

  	/*eslint-disable no-bitwise*/
  	var NodeBuffer;
  	try {
  	  // A trick for browserified version, to not include `Buffer` shim
  	  var _require = commonjsRequire;
  	  NodeBuffer = _require('buffer').Buffer;
  	} catch (__) {}
  	var Type = requireType();

  	// [ 64, 65, 66 ] -> [ padding, CR, LF ]
  	var BASE64_MAP = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=\n\r';
  	function resolveYamlBinary(data) {
  	  if (data === null) return false;
  	  var code,
  	    idx,
  	    bitlen = 0,
  	    max = data.length,
  	    map = BASE64_MAP;

  	  // Convert one by one.
  	  for (idx = 0; idx < max; idx++) {
  	    code = map.indexOf(data.charAt(idx));

  	    // Skip CR/LF
  	    if (code > 64) continue;

  	    // Fail on illegal characters
  	    if (code < 0) return false;
  	    bitlen += 6;
  	  }

  	  // If there are any bits left, source was corrupted
  	  return bitlen % 8 === 0;
  	}
  	function constructYamlBinary(data) {
  	  var idx,
  	    tailbits,
  	    input = data.replace(/[\r\n=]/g, ''),
  	    // remove CR/LF & padding to simplify scan
  	    max = input.length,
  	    map = BASE64_MAP,
  	    bits = 0,
  	    result = [];

  	  // Collect by 6*4 bits (3 bytes)

  	  for (idx = 0; idx < max; idx++) {
  	    if (idx % 4 === 0 && idx) {
  	      result.push(bits >> 16 & 0xFF);
  	      result.push(bits >> 8 & 0xFF);
  	      result.push(bits & 0xFF);
  	    }
  	    bits = bits << 6 | map.indexOf(input.charAt(idx));
  	  }

  	  // Dump tail

  	  tailbits = max % 4 * 6;
  	  if (tailbits === 0) {
  	    result.push(bits >> 16 & 0xFF);
  	    result.push(bits >> 8 & 0xFF);
  	    result.push(bits & 0xFF);
  	  } else if (tailbits === 18) {
  	    result.push(bits >> 10 & 0xFF);
  	    result.push(bits >> 2 & 0xFF);
  	  } else if (tailbits === 12) {
  	    result.push(bits >> 4 & 0xFF);
  	  }

  	  // Wrap into Buffer for NodeJS and leave Array for browser
  	  if (NodeBuffer) {
  	    // Support node 6.+ Buffer API when available
  	    return NodeBuffer.from ? NodeBuffer.from(result) : new NodeBuffer(result);
  	  }
  	  return result;
  	}
  	function representYamlBinary(object /*, style*/) {
  	  var result = '',
  	    bits = 0,
  	    idx,
  	    tail,
  	    max = object.length,
  	    map = BASE64_MAP;

  	  // Convert every three bytes to 4 ASCII characters.

  	  for (idx = 0; idx < max; idx++) {
  	    if (idx % 3 === 0 && idx) {
  	      result += map[bits >> 18 & 0x3F];
  	      result += map[bits >> 12 & 0x3F];
  	      result += map[bits >> 6 & 0x3F];
  	      result += map[bits & 0x3F];
  	    }
  	    bits = (bits << 8) + object[idx];
  	  }

  	  // Dump tail

  	  tail = max % 3;
  	  if (tail === 0) {
  	    result += map[bits >> 18 & 0x3F];
  	    result += map[bits >> 12 & 0x3F];
  	    result += map[bits >> 6 & 0x3F];
  	    result += map[bits & 0x3F];
  	  } else if (tail === 2) {
  	    result += map[bits >> 10 & 0x3F];
  	    result += map[bits >> 4 & 0x3F];
  	    result += map[bits << 2 & 0x3F];
  	    result += map[64];
  	  } else if (tail === 1) {
  	    result += map[bits >> 2 & 0x3F];
  	    result += map[bits << 4 & 0x3F];
  	    result += map[64];
  	    result += map[64];
  	  }
  	  return result;
  	}
  	function isBinary(object) {
  	  return NodeBuffer && NodeBuffer.isBuffer(object);
  	}
  	binary = new Type('tag:yaml.org,2002:binary', {
  	  kind: 'scalar',
  	  resolve: resolveYamlBinary,
  	  construct: constructYamlBinary,
  	  predicate: isBinary,
  	  represent: representYamlBinary
  	});
  	return binary;
  }

  var omap;
  var hasRequiredOmap;

  function requireOmap () {
  	if (hasRequiredOmap) return omap;
  	hasRequiredOmap = 1;

  	var Type = requireType();
  	var _hasOwnProperty = Object.prototype.hasOwnProperty;
  	var _toString = Object.prototype.toString;
  	function resolveYamlOmap(data) {
  	  if (data === null) return true;
  	  var objectKeys = [],
  	    index,
  	    length,
  	    pair,
  	    pairKey,
  	    pairHasKey,
  	    object = data;
  	  for (index = 0, length = object.length; index < length; index += 1) {
  	    pair = object[index];
  	    pairHasKey = false;
  	    if (_toString.call(pair) !== '[object Object]') return false;
  	    for (pairKey in pair) {
  	      if (_hasOwnProperty.call(pair, pairKey)) {
  	        if (!pairHasKey) pairHasKey = true;else return false;
  	      }
  	    }
  	    if (!pairHasKey) return false;
  	    if (objectKeys.indexOf(pairKey) === -1) objectKeys.push(pairKey);else return false;
  	  }
  	  return true;
  	}
  	function constructYamlOmap(data) {
  	  return data !== null ? data : [];
  	}
  	omap = new Type('tag:yaml.org,2002:omap', {
  	  kind: 'sequence',
  	  resolve: resolveYamlOmap,
  	  construct: constructYamlOmap
  	});
  	return omap;
  }

  var pairs;
  var hasRequiredPairs;

  function requirePairs () {
  	if (hasRequiredPairs) return pairs;
  	hasRequiredPairs = 1;

  	var Type = requireType();
  	var _toString = Object.prototype.toString;
  	function resolveYamlPairs(data) {
  	  if (data === null) return true;
  	  var index,
  	    length,
  	    pair,
  	    keys,
  	    result,
  	    object = data;
  	  result = new Array(object.length);
  	  for (index = 0, length = object.length; index < length; index += 1) {
  	    pair = object[index];
  	    if (_toString.call(pair) !== '[object Object]') return false;
  	    keys = Object.keys(pair);
  	    if (keys.length !== 1) return false;
  	    result[index] = [keys[0], pair[keys[0]]];
  	  }
  	  return true;
  	}
  	function constructYamlPairs(data) {
  	  if (data === null) return [];
  	  var index,
  	    length,
  	    pair,
  	    keys,
  	    result,
  	    object = data;
  	  result = new Array(object.length);
  	  for (index = 0, length = object.length; index < length; index += 1) {
  	    pair = object[index];
  	    keys = Object.keys(pair);
  	    result[index] = [keys[0], pair[keys[0]]];
  	  }
  	  return result;
  	}
  	pairs = new Type('tag:yaml.org,2002:pairs', {
  	  kind: 'sequence',
  	  resolve: resolveYamlPairs,
  	  construct: constructYamlPairs
  	});
  	return pairs;
  }

  var set;
  var hasRequiredSet;

  function requireSet () {
  	if (hasRequiredSet) return set;
  	hasRequiredSet = 1;

  	var Type = requireType();
  	var _hasOwnProperty = Object.prototype.hasOwnProperty;
  	function resolveYamlSet(data) {
  	  if (data === null) return true;
  	  var key,
  	    object = data;
  	  for (key in object) {
  	    if (_hasOwnProperty.call(object, key)) {
  	      if (object[key] !== null) return false;
  	    }
  	  }
  	  return true;
  	}
  	function constructYamlSet(data) {
  	  return data !== null ? data : {};
  	}
  	set = new Type('tag:yaml.org,2002:set', {
  	  kind: 'mapping',
  	  resolve: resolveYamlSet,
  	  construct: constructYamlSet
  	});
  	return set;
  }

  var default_safe;
  var hasRequiredDefault_safe;

  function requireDefault_safe () {
  	if (hasRequiredDefault_safe) return default_safe;
  	hasRequiredDefault_safe = 1;

  	var Schema = requireSchema();
  	default_safe = new Schema({
  	  include: [requireCore()],
  	  implicit: [requireTimestamp(), requireMerge()],
  	  explicit: [requireBinary(), requireOmap(), requirePairs(), requireSet()]
  	});
  	return default_safe;
  }

  var _undefined;
  var hasRequired_undefined;

  function require_undefined () {
  	if (hasRequired_undefined) return _undefined;
  	hasRequired_undefined = 1;

  	var Type = requireType();
  	function resolveJavascriptUndefined() {
  	  return true;
  	}
  	function constructJavascriptUndefined() {
  	  /*eslint-disable no-undefined*/
  	  return undefined;
  	}
  	function representJavascriptUndefined() {
  	  return '';
  	}
  	function isUndefined(object) {
  	  return typeof object === 'undefined';
  	}
  	_undefined = new Type('tag:yaml.org,2002:js/undefined', {
  	  kind: 'scalar',
  	  resolve: resolveJavascriptUndefined,
  	  construct: constructJavascriptUndefined,
  	  predicate: isUndefined,
  	  represent: representJavascriptUndefined
  	});
  	return _undefined;
  }

  var regexp;
  var hasRequiredRegexp;

  function requireRegexp () {
  	if (hasRequiredRegexp) return regexp;
  	hasRequiredRegexp = 1;

  	var Type = requireType();
  	function resolveJavascriptRegExp(data) {
  	  if (data === null) return false;
  	  if (data.length === 0) return false;
  	  var regexp = data,
  	    tail = /\/([gim]*)$/.exec(data),
  	    modifiers = '';

  	  // if regexp starts with '/' it can have modifiers and must be properly closed
  	  // `/foo/gim` - modifiers tail can be maximum 3 chars
  	  if (regexp[0] === '/') {
  	    if (tail) modifiers = tail[1];
  	    if (modifiers.length > 3) return false;
  	    // if expression starts with /, is should be properly terminated
  	    if (regexp[regexp.length - modifiers.length - 1] !== '/') return false;
  	  }
  	  return true;
  	}
  	function constructJavascriptRegExp(data) {
  	  var regexp = data,
  	    tail = /\/([gim]*)$/.exec(data),
  	    modifiers = '';

  	  // `/foo/gim` - tail can be maximum 4 chars
  	  if (regexp[0] === '/') {
  	    if (tail) modifiers = tail[1];
  	    regexp = regexp.slice(1, regexp.length - modifiers.length - 1);
  	  }
  	  return new RegExp(regexp, modifiers);
  	}
  	function representJavascriptRegExp(object /*, style*/) {
  	  var result = '/' + object.source + '/';
  	  if (object.global) result += 'g';
  	  if (object.multiline) result += 'm';
  	  if (object.ignoreCase) result += 'i';
  	  return result;
  	}
  	function isRegExp(object) {
  	  return Object.prototype.toString.call(object) === '[object RegExp]';
  	}
  	regexp = new Type('tag:yaml.org,2002:js/regexp', {
  	  kind: 'scalar',
  	  resolve: resolveJavascriptRegExp,
  	  construct: constructJavascriptRegExp,
  	  predicate: isRegExp,
  	  represent: representJavascriptRegExp
  	});
  	return regexp;
  }

  var _function;
  var hasRequired_function;

  function require_function () {
  	if (hasRequired_function) return _function;
  	hasRequired_function = 1;

  	var esprima;

  	// Browserified version does not have esprima
  	//
  	// 1. For node.js just require module as deps
  	// 2. For browser try to require mudule via external AMD system.
  	//    If not found - try to fallback to window.esprima. If not
  	//    found too - then fail to parse.
  	//
  	try {
  	  // workaround to exclude package from browserify list.
  	  var _require = commonjsRequire;
  	  esprima = _require('esprima');
  	} catch (_) {
  	  /* eslint-disable no-redeclare */
  	  /* global window */
  	  if (typeof window !== 'undefined') esprima = window.esprima;
  	}
  	var Type = requireType();
  	function resolveJavascriptFunction(data) {
  	  if (data === null) return false;
  	  try {
  	    var source = '(' + data + ')',
  	      ast = esprima.parse(source, {
  	        range: true
  	      });
  	    if (ast.type !== 'Program' || ast.body.length !== 1 || ast.body[0].type !== 'ExpressionStatement' || ast.body[0].expression.type !== 'ArrowFunctionExpression' && ast.body[0].expression.type !== 'FunctionExpression') {
  	      return false;
  	    }
  	    return true;
  	  } catch (err) {
  	    return false;
  	  }
  	}
  	function constructJavascriptFunction(data) {
  	  /*jslint evil:true*/

  	  var source = '(' + data + ')',
  	    ast = esprima.parse(source, {
  	      range: true
  	    }),
  	    params = [],
  	    body;
  	  if (ast.type !== 'Program' || ast.body.length !== 1 || ast.body[0].type !== 'ExpressionStatement' || ast.body[0].expression.type !== 'ArrowFunctionExpression' && ast.body[0].expression.type !== 'FunctionExpression') {
  	    throw new Error('Failed to resolve function');
  	  }
  	  ast.body[0].expression.params.forEach(function (param) {
  	    params.push(param.name);
  	  });
  	  body = ast.body[0].expression.body.range;

  	  // Esprima's ranges include the first '{' and the last '}' characters on
  	  // function expressions. So cut them out.
  	  if (ast.body[0].expression.body.type === 'BlockStatement') {
  	    /*eslint-disable no-new-func*/
  	    return new Function(params, source.slice(body[0] + 1, body[1] - 1));
  	  }
  	  // ES6 arrow functions can omit the BlockStatement. In that case, just return
  	  // the body.
  	  /*eslint-disable no-new-func*/
  	  return new Function(params, 'return ' + source.slice(body[0], body[1]));
  	}
  	function representJavascriptFunction(object /*, style*/) {
  	  return object.toString();
  	}
  	function isFunction(object) {
  	  return Object.prototype.toString.call(object) === '[object Function]';
  	}
  	_function = new Type('tag:yaml.org,2002:js/function', {
  	  kind: 'scalar',
  	  resolve: resolveJavascriptFunction,
  	  construct: constructJavascriptFunction,
  	  predicate: isFunction,
  	  represent: representJavascriptFunction
  	});
  	return _function;
  }

  var default_full;
  var hasRequiredDefault_full;

  function requireDefault_full () {
  	if (hasRequiredDefault_full) return default_full;
  	hasRequiredDefault_full = 1;

  	var Schema = requireSchema();
  	default_full = Schema.DEFAULT = new Schema({
  	  include: [requireDefault_safe()],
  	  explicit: [require_undefined(), requireRegexp(), require_function()]
  	});
  	return default_full;
  }

  var hasRequiredLoader;

  function requireLoader () {
  	if (hasRequiredLoader) return loader;
  	hasRequiredLoader = 1;

  	/*eslint-disable max-len,no-use-before-define*/
  	var common = requireCommon();
  	var YAMLException = requireException();
  	var Mark = requireMark();
  	var DEFAULT_SAFE_SCHEMA = requireDefault_safe();
  	var DEFAULT_FULL_SCHEMA = requireDefault_full();
  	var _hasOwnProperty = Object.prototype.hasOwnProperty;
  	var CONTEXT_FLOW_IN = 1;
  	var CONTEXT_FLOW_OUT = 2;
  	var CONTEXT_BLOCK_IN = 3;
  	var CONTEXT_BLOCK_OUT = 4;
  	var CHOMPING_CLIP = 1;
  	var CHOMPING_STRIP = 2;
  	var CHOMPING_KEEP = 3;
  	var PATTERN_NON_PRINTABLE = /[\x00-\x08\x0B\x0C\x0E-\x1F\x7F-\x84\x86-\x9F\uFFFE\uFFFF]|[\uD800-\uDBFF](?![\uDC00-\uDFFF])|(?:[^\uD800-\uDBFF]|^)[\uDC00-\uDFFF]/;
  	var PATTERN_NON_ASCII_LINE_BREAKS = /[\x85\u2028\u2029]/;
  	var PATTERN_FLOW_INDICATORS = /[,\[\]\{\}]/;
  	var PATTERN_TAG_HANDLE = /^(?:!|!!|![a-z\-]+!)$/i;
  	var PATTERN_TAG_URI = /^(?:!|[^,\[\]\{\}])(?:%[0-9a-f]{2}|[0-9a-z\-#;\/\?:@&=\+\$,_\.!~\*'\(\)\[\]])*$/i;
  	function _class(obj) {
  	  return Object.prototype.toString.call(obj);
  	}
  	function is_EOL(c) {
  	  return c === 0x0A /* LF */ || c === 0x0D /* CR */;
  	}
  	function is_WHITE_SPACE(c) {
  	  return c === 0x09 /* Tab */ || c === 0x20 /* Space */;
  	}
  	function is_WS_OR_EOL(c) {
  	  return c === 0x09 /* Tab */ || c === 0x20 /* Space */ || c === 0x0A /* LF */ || c === 0x0D /* CR */;
  	}
  	function is_FLOW_INDICATOR(c) {
  	  return c === 0x2C /* , */ || c === 0x5B /* [ */ || c === 0x5D /* ] */ || c === 0x7B /* { */ || c === 0x7D /* } */;
  	}
  	function fromHexCode(c) {
  	  var lc;
  	  if (0x30 /* 0 */ <= c && c <= 0x39 /* 9 */) {
  	    return c - 0x30;
  	  }

  	  /*eslint-disable no-bitwise*/
  	  lc = c | 0x20;
  	  if (0x61 /* a */ <= lc && lc <= 0x66 /* f */) {
  	    return lc - 0x61 + 10;
  	  }
  	  return -1;
  	}
  	function escapedHexLen(c) {
  	  if (c === 0x78 /* x */) {
  	    return 2;
  	  }
  	  if (c === 0x75 /* u */) {
  	    return 4;
  	  }
  	  if (c === 0x55 /* U */) {
  	    return 8;
  	  }
  	  return 0;
  	}
  	function fromDecimalCode(c) {
  	  if (0x30 /* 0 */ <= c && c <= 0x39 /* 9 */) {
  	    return c - 0x30;
  	  }
  	  return -1;
  	}
  	function simpleEscapeSequence(c) {
  	  /* eslint-disable indent */
  	  return c === 0x30 /* 0 */ ? '\x00' : c === 0x61 /* a */ ? '\x07' : c === 0x62 /* b */ ? '\x08' : c === 0x74 /* t */ ? '\x09' : c === 0x09 /* Tab */ ? '\x09' : c === 0x6E /* n */ ? '\x0A' : c === 0x76 /* v */ ? '\x0B' : c === 0x66 /* f */ ? '\x0C' : c === 0x72 /* r */ ? '\x0D' : c === 0x65 /* e */ ? '\x1B' : c === 0x20 /* Space */ ? ' ' : c === 0x22 /* " */ ? '\x22' : c === 0x2F /* / */ ? '/' : c === 0x5C /* \ */ ? '\x5C' : c === 0x4E /* N */ ? '\x85' : c === 0x5F /* _ */ ? '\xA0' : c === 0x4C /* L */ ? '\u2028' : c === 0x50 /* P */ ? '\u2029' : '';
  	}
  	function charFromCodepoint(c) {
  	  if (c <= 0xFFFF) {
  	    return String.fromCharCode(c);
  	  }
  	  // Encode UTF-16 surrogate pair
  	  // https://en.wikipedia.org/wiki/UTF-16#Code_points_U.2B010000_to_U.2B10FFFF
  	  return String.fromCharCode((c - 0x010000 >> 10) + 0xD800, (c - 0x010000 & 0x03FF) + 0xDC00);
  	}
  	var simpleEscapeCheck = new Array(256); // integer, for fast access
  	var simpleEscapeMap = new Array(256);
  	for (var i = 0; i < 256; i++) {
  	  simpleEscapeCheck[i] = simpleEscapeSequence(i) ? 1 : 0;
  	  simpleEscapeMap[i] = simpleEscapeSequence(i);
  	}
  	function State(input, options) {
  	  this.input = input;
  	  this.filename = options['filename'] || null;
  	  this.schema = options['schema'] || DEFAULT_FULL_SCHEMA;
  	  this.onWarning = options['onWarning'] || null;
  	  this.legacy = options['legacy'] || false;
  	  this.json = options['json'] || false;
  	  this.listener = options['listener'] || null;
  	  this.implicitTypes = this.schema.compiledImplicit;
  	  this.typeMap = this.schema.compiledTypeMap;
  	  this.length = input.length;
  	  this.position = 0;
  	  this.line = 0;
  	  this.lineStart = 0;
  	  this.lineIndent = 0;
  	  this.documents = [];

  	  /*
  	  this.version;
  	  this.checkLineBreaks;
  	  this.tagMap;
  	  this.anchorMap;
  	  this.tag;
  	  this.anchor;
  	  this.kind;
  	  this.result;*/
  	}
  	function generateError(state, message) {
  	  return new YAMLException(message, new Mark(state.filename, state.input, state.position, state.line, state.position - state.lineStart));
  	}
  	function throwError(state, message) {
  	  throw generateError(state, message);
  	}
  	function throwWarning(state, message) {
  	  if (state.onWarning) {
  	    state.onWarning.call(null, generateError(state, message));
  	  }
  	}
  	var directiveHandlers = {
  	  YAML: function handleYamlDirective(state, name, args) {
  	    var match, major, minor;
  	    if (state.version !== null) {
  	      throwError(state, 'duplication of %YAML directive');
  	    }
  	    if (args.length !== 1) {
  	      throwError(state, 'YAML directive accepts exactly one argument');
  	    }
  	    match = /^([0-9]+)\.([0-9]+)$/.exec(args[0]);
  	    if (match === null) {
  	      throwError(state, 'ill-formed argument of the YAML directive');
  	    }
  	    major = parseInt(match[1], 10);
  	    minor = parseInt(match[2], 10);
  	    if (major !== 1) {
  	      throwError(state, 'unacceptable YAML version of the document');
  	    }
  	    state.version = args[0];
  	    state.checkLineBreaks = minor < 2;
  	    if (minor !== 1 && minor !== 2) {
  	      throwWarning(state, 'unsupported YAML version of the document');
  	    }
  	  },
  	  TAG: function handleTagDirective(state, name, args) {
  	    var handle, prefix;
  	    if (args.length !== 2) {
  	      throwError(state, 'TAG directive accepts exactly two arguments');
  	    }
  	    handle = args[0];
  	    prefix = args[1];
  	    if (!PATTERN_TAG_HANDLE.test(handle)) {
  	      throwError(state, 'ill-formed tag handle (first argument) of the TAG directive');
  	    }
  	    if (_hasOwnProperty.call(state.tagMap, handle)) {
  	      throwError(state, 'there is a previously declared suffix for "' + handle + '" tag handle');
  	    }
  	    if (!PATTERN_TAG_URI.test(prefix)) {
  	      throwError(state, 'ill-formed tag prefix (second argument) of the TAG directive');
  	    }
  	    state.tagMap[handle] = prefix;
  	  }
  	};
  	function captureSegment(state, start, end, checkJson) {
  	  var _position, _length, _character, _result;
  	  if (start < end) {
  	    _result = state.input.slice(start, end);
  	    if (checkJson) {
  	      for (_position = 0, _length = _result.length; _position < _length; _position += 1) {
  	        _character = _result.charCodeAt(_position);
  	        if (!(_character === 0x09 || 0x20 <= _character && _character <= 0x10FFFF)) {
  	          throwError(state, 'expected valid JSON character');
  	        }
  	      }
  	    } else if (PATTERN_NON_PRINTABLE.test(_result)) {
  	      throwError(state, 'the stream contains non-printable characters');
  	    }
  	    state.result += _result;
  	  }
  	}
  	function mergeMappings(state, destination, source, overridableKeys) {
  	  var sourceKeys, key, index, quantity;
  	  if (!common.isObject(source)) {
  	    throwError(state, 'cannot merge mappings; the provided source object is unacceptable');
  	  }
  	  sourceKeys = Object.keys(source);
  	  for (index = 0, quantity = sourceKeys.length; index < quantity; index += 1) {
  	    key = sourceKeys[index];
  	    if (!_hasOwnProperty.call(destination, key)) {
  	      destination[key] = source[key];
  	      overridableKeys[key] = true;
  	    }
  	  }
  	}
  	function storeMappingPair(state, _result, overridableKeys, keyTag, keyNode, valueNode, startLine, startPos) {
  	  var index, quantity;

  	  // The output is a plain object here, so keys can only be strings.
  	  // We need to convert keyNode to a string, but doing so can hang the process
  	  // (deeply nested arrays that explode exponentially using aliases).
  	  if (Array.isArray(keyNode)) {
  	    keyNode = Array.prototype.slice.call(keyNode);
  	    for (index = 0, quantity = keyNode.length; index < quantity; index += 1) {
  	      if (Array.isArray(keyNode[index])) {
  	        throwError(state, 'nested arrays are not supported inside keys');
  	      }
  	      if (typeof keyNode === 'object' && _class(keyNode[index]) === '[object Object]') {
  	        keyNode[index] = '[object Object]';
  	      }
  	    }
  	  }

  	  // Avoid code execution in load() via toString property
  	  // (still use its own toString for arrays, timestamps,
  	  // and whatever user schema extensions happen to have @@toStringTag)
  	  if (typeof keyNode === 'object' && _class(keyNode) === '[object Object]') {
  	    keyNode = '[object Object]';
  	  }
  	  keyNode = String(keyNode);
  	  if (_result === null) {
  	    _result = {};
  	  }
  	  if (keyTag === 'tag:yaml.org,2002:merge') {
  	    if (Array.isArray(valueNode)) {
  	      for (index = 0, quantity = valueNode.length; index < quantity; index += 1) {
  	        mergeMappings(state, _result, valueNode[index], overridableKeys);
  	      }
  	    } else {
  	      mergeMappings(state, _result, valueNode, overridableKeys);
  	    }
  	  } else {
  	    if (!state.json && !_hasOwnProperty.call(overridableKeys, keyNode) && _hasOwnProperty.call(_result, keyNode)) {
  	      state.line = startLine || state.line;
  	      state.position = startPos || state.position;
  	      throwError(state, 'duplicated mapping key');
  	    }
  	    _result[keyNode] = valueNode;
  	    delete overridableKeys[keyNode];
  	  }
  	  return _result;
  	}
  	function readLineBreak(state) {
  	  var ch;
  	  ch = state.input.charCodeAt(state.position);
  	  if (ch === 0x0A /* LF */) {
  	    state.position++;
  	  } else if (ch === 0x0D /* CR */) {
  	    state.position++;
  	    if (state.input.charCodeAt(state.position) === 0x0A /* LF */) {
  	      state.position++;
  	    }
  	  } else {
  	    throwError(state, 'a line break is expected');
  	  }
  	  state.line += 1;
  	  state.lineStart = state.position;
  	}
  	function skipSeparationSpace(state, allowComments, checkIndent) {
  	  var lineBreaks = 0,
  	    ch = state.input.charCodeAt(state.position);
  	  while (ch !== 0) {
  	    while (is_WHITE_SPACE(ch)) {
  	      ch = state.input.charCodeAt(++state.position);
  	    }
  	    if (allowComments && ch === 0x23 /* # */) {
  	      do {
  	        ch = state.input.charCodeAt(++state.position);
  	      } while (ch !== 0x0A /* LF */ && ch !== 0x0D /* CR */ && ch !== 0);
  	    }
  	    if (is_EOL(ch)) {
  	      readLineBreak(state);
  	      ch = state.input.charCodeAt(state.position);
  	      lineBreaks++;
  	      state.lineIndent = 0;
  	      while (ch === 0x20 /* Space */) {
  	        state.lineIndent++;
  	        ch = state.input.charCodeAt(++state.position);
  	      }
  	    } else {
  	      break;
  	    }
  	  }
  	  if (checkIndent !== -1 && lineBreaks !== 0 && state.lineIndent < checkIndent) {
  	    throwWarning(state, 'deficient indentation');
  	  }
  	  return lineBreaks;
  	}
  	function testDocumentSeparator(state) {
  	  var _position = state.position,
  	    ch;
  	  ch = state.input.charCodeAt(_position);

  	  // Condition state.position === state.lineStart is tested
  	  // in parent on each call, for efficiency. No needs to test here again.
  	  if ((ch === 0x2D /* - */ || ch === 0x2E /* . */) && ch === state.input.charCodeAt(_position + 1) && ch === state.input.charCodeAt(_position + 2)) {
  	    _position += 3;
  	    ch = state.input.charCodeAt(_position);
  	    if (ch === 0 || is_WS_OR_EOL(ch)) {
  	      return true;
  	    }
  	  }
  	  return false;
  	}
  	function writeFoldedLines(state, count) {
  	  if (count === 1) {
  	    state.result += ' ';
  	  } else if (count > 1) {
  	    state.result += common.repeat('\n', count - 1);
  	  }
  	}
  	function readPlainScalar(state, nodeIndent, withinFlowCollection) {
  	  var preceding,
  	    following,
  	    captureStart,
  	    captureEnd,
  	    hasPendingContent,
  	    _line,
  	    _lineStart,
  	    _lineIndent,
  	    _kind = state.kind,
  	    _result = state.result,
  	    ch;
  	  ch = state.input.charCodeAt(state.position);
  	  if (is_WS_OR_EOL(ch) || is_FLOW_INDICATOR(ch) || ch === 0x23 /* # */ || ch === 0x26 /* & */ || ch === 0x2A /* * */ || ch === 0x21 /* ! */ || ch === 0x7C /* | */ || ch === 0x3E /* > */ || ch === 0x27 /* ' */ || ch === 0x22 /* " */ || ch === 0x25 /* % */ || ch === 0x40 /* @ */ || ch === 0x60 /* ` */) {
  	    return false;
  	  }
  	  if (ch === 0x3F /* ? */ || ch === 0x2D /* - */) {
  	    following = state.input.charCodeAt(state.position + 1);
  	    if (is_WS_OR_EOL(following) || withinFlowCollection && is_FLOW_INDICATOR(following)) {
  	      return false;
  	    }
  	  }
  	  state.kind = 'scalar';
  	  state.result = '';
  	  captureStart = captureEnd = state.position;
  	  hasPendingContent = false;
  	  while (ch !== 0) {
  	    if (ch === 0x3A /* : */) {
  	      following = state.input.charCodeAt(state.position + 1);
  	      if (is_WS_OR_EOL(following) || withinFlowCollection && is_FLOW_INDICATOR(following)) {
  	        break;
  	      }
  	    } else if (ch === 0x23 /* # */) {
  	      preceding = state.input.charCodeAt(state.position - 1);
  	      if (is_WS_OR_EOL(preceding)) {
  	        break;
  	      }
  	    } else if (state.position === state.lineStart && testDocumentSeparator(state) || withinFlowCollection && is_FLOW_INDICATOR(ch)) {
  	      break;
  	    } else if (is_EOL(ch)) {
  	      _line = state.line;
  	      _lineStart = state.lineStart;
  	      _lineIndent = state.lineIndent;
  	      skipSeparationSpace(state, false, -1);
  	      if (state.lineIndent >= nodeIndent) {
  	        hasPendingContent = true;
  	        ch = state.input.charCodeAt(state.position);
  	        continue;
  	      } else {
  	        state.position = captureEnd;
  	        state.line = _line;
  	        state.lineStart = _lineStart;
  	        state.lineIndent = _lineIndent;
  	        break;
  	      }
  	    }
  	    if (hasPendingContent) {
  	      captureSegment(state, captureStart, captureEnd, false);
  	      writeFoldedLines(state, state.line - _line);
  	      captureStart = captureEnd = state.position;
  	      hasPendingContent = false;
  	    }
  	    if (!is_WHITE_SPACE(ch)) {
  	      captureEnd = state.position + 1;
  	    }
  	    ch = state.input.charCodeAt(++state.position);
  	  }
  	  captureSegment(state, captureStart, captureEnd, false);
  	  if (state.result) {
  	    return true;
  	  }
  	  state.kind = _kind;
  	  state.result = _result;
  	  return false;
  	}
  	function readSingleQuotedScalar(state, nodeIndent) {
  	  var ch, captureStart, captureEnd;
  	  ch = state.input.charCodeAt(state.position);
  	  if (ch !== 0x27 /* ' */) {
  	    return false;
  	  }
  	  state.kind = 'scalar';
  	  state.result = '';
  	  state.position++;
  	  captureStart = captureEnd = state.position;
  	  while ((ch = state.input.charCodeAt(state.position)) !== 0) {
  	    if (ch === 0x27 /* ' */) {
  	      captureSegment(state, captureStart, state.position, true);
  	      ch = state.input.charCodeAt(++state.position);
  	      if (ch === 0x27 /* ' */) {
  	        captureStart = state.position;
  	        state.position++;
  	        captureEnd = state.position;
  	      } else {
  	        return true;
  	      }
  	    } else if (is_EOL(ch)) {
  	      captureSegment(state, captureStart, captureEnd, true);
  	      writeFoldedLines(state, skipSeparationSpace(state, false, nodeIndent));
  	      captureStart = captureEnd = state.position;
  	    } else if (state.position === state.lineStart && testDocumentSeparator(state)) {
  	      throwError(state, 'unexpected end of the document within a single quoted scalar');
  	    } else {
  	      state.position++;
  	      captureEnd = state.position;
  	    }
  	  }
  	  throwError(state, 'unexpected end of the stream within a single quoted scalar');
  	}
  	function readDoubleQuotedScalar(state, nodeIndent) {
  	  var captureStart, captureEnd, hexLength, hexResult, tmp, ch;
  	  ch = state.input.charCodeAt(state.position);
  	  if (ch !== 0x22 /* " */) {
  	    return false;
  	  }
  	  state.kind = 'scalar';
  	  state.result = '';
  	  state.position++;
  	  captureStart = captureEnd = state.position;
  	  while ((ch = state.input.charCodeAt(state.position)) !== 0) {
  	    if (ch === 0x22 /* " */) {
  	      captureSegment(state, captureStart, state.position, true);
  	      state.position++;
  	      return true;
  	    } else if (ch === 0x5C /* \ */) {
  	      captureSegment(state, captureStart, state.position, true);
  	      ch = state.input.charCodeAt(++state.position);
  	      if (is_EOL(ch)) {
  	        skipSeparationSpace(state, false, nodeIndent);

  	        // TODO: rework to inline fn with no type cast?
  	      } else if (ch < 256 && simpleEscapeCheck[ch]) {
  	        state.result += simpleEscapeMap[ch];
  	        state.position++;
  	      } else if ((tmp = escapedHexLen(ch)) > 0) {
  	        hexLength = tmp;
  	        hexResult = 0;
  	        for (; hexLength > 0; hexLength--) {
  	          ch = state.input.charCodeAt(++state.position);
  	          if ((tmp = fromHexCode(ch)) >= 0) {
  	            hexResult = (hexResult << 4) + tmp;
  	          } else {
  	            throwError(state, 'expected hexadecimal character');
  	          }
  	        }
  	        state.result += charFromCodepoint(hexResult);
  	        state.position++;
  	      } else {
  	        throwError(state, 'unknown escape sequence');
  	      }
  	      captureStart = captureEnd = state.position;
  	    } else if (is_EOL(ch)) {
  	      captureSegment(state, captureStart, captureEnd, true);
  	      writeFoldedLines(state, skipSeparationSpace(state, false, nodeIndent));
  	      captureStart = captureEnd = state.position;
  	    } else if (state.position === state.lineStart && testDocumentSeparator(state)) {
  	      throwError(state, 'unexpected end of the document within a double quoted scalar');
  	    } else {
  	      state.position++;
  	      captureEnd = state.position;
  	    }
  	  }
  	  throwError(state, 'unexpected end of the stream within a double quoted scalar');
  	}
  	function readFlowCollection(state, nodeIndent) {
  	  var readNext = true,
  	    _line,
  	    _tag = state.tag,
  	    _result,
  	    _anchor = state.anchor,
  	    following,
  	    terminator,
  	    isPair,
  	    isExplicitPair,
  	    isMapping,
  	    overridableKeys = {},
  	    keyNode,
  	    keyTag,
  	    valueNode,
  	    ch;
  	  ch = state.input.charCodeAt(state.position);
  	  if (ch === 0x5B /* [ */) {
  	    terminator = 0x5D; /* ] */
  	    isMapping = false;
  	    _result = [];
  	  } else if (ch === 0x7B /* { */) {
  	    terminator = 0x7D; /* } */
  	    isMapping = true;
  	    _result = {};
  	  } else {
  	    return false;
  	  }
  	  if (state.anchor !== null) {
  	    state.anchorMap[state.anchor] = _result;
  	  }
  	  ch = state.input.charCodeAt(++state.position);
  	  while (ch !== 0) {
  	    skipSeparationSpace(state, true, nodeIndent);
  	    ch = state.input.charCodeAt(state.position);
  	    if (ch === terminator) {
  	      state.position++;
  	      state.tag = _tag;
  	      state.anchor = _anchor;
  	      state.kind = isMapping ? 'mapping' : 'sequence';
  	      state.result = _result;
  	      return true;
  	    } else if (!readNext) {
  	      throwError(state, 'missed comma between flow collection entries');
  	    }
  	    keyTag = keyNode = valueNode = null;
  	    isPair = isExplicitPair = false;
  	    if (ch === 0x3F /* ? */) {
  	      following = state.input.charCodeAt(state.position + 1);
  	      if (is_WS_OR_EOL(following)) {
  	        isPair = isExplicitPair = true;
  	        state.position++;
  	        skipSeparationSpace(state, true, nodeIndent);
  	      }
  	    }
  	    _line = state.line;
  	    composeNode(state, nodeIndent, CONTEXT_FLOW_IN, false, true);
  	    keyTag = state.tag;
  	    keyNode = state.result;
  	    skipSeparationSpace(state, true, nodeIndent);
  	    ch = state.input.charCodeAt(state.position);
  	    if ((isExplicitPair || state.line === _line) && ch === 0x3A /* : */) {
  	      isPair = true;
  	      ch = state.input.charCodeAt(++state.position);
  	      skipSeparationSpace(state, true, nodeIndent);
  	      composeNode(state, nodeIndent, CONTEXT_FLOW_IN, false, true);
  	      valueNode = state.result;
  	    }
  	    if (isMapping) {
  	      storeMappingPair(state, _result, overridableKeys, keyTag, keyNode, valueNode);
  	    } else if (isPair) {
  	      _result.push(storeMappingPair(state, null, overridableKeys, keyTag, keyNode, valueNode));
  	    } else {
  	      _result.push(keyNode);
  	    }
  	    skipSeparationSpace(state, true, nodeIndent);
  	    ch = state.input.charCodeAt(state.position);
  	    if (ch === 0x2C /* , */) {
  	      readNext = true;
  	      ch = state.input.charCodeAt(++state.position);
  	    } else {
  	      readNext = false;
  	    }
  	  }
  	  throwError(state, 'unexpected end of the stream within a flow collection');
  	}
  	function readBlockScalar(state, nodeIndent) {
  	  var captureStart,
  	    folding,
  	    chomping = CHOMPING_CLIP,
  	    didReadContent = false,
  	    detectedIndent = false,
  	    textIndent = nodeIndent,
  	    emptyLines = 0,
  	    atMoreIndented = false,
  	    tmp,
  	    ch;
  	  ch = state.input.charCodeAt(state.position);
  	  if (ch === 0x7C /* | */) {
  	    folding = false;
  	  } else if (ch === 0x3E /* > */) {
  	    folding = true;
  	  } else {
  	    return false;
  	  }
  	  state.kind = 'scalar';
  	  state.result = '';
  	  while (ch !== 0) {
  	    ch = state.input.charCodeAt(++state.position);
  	    if (ch === 0x2B /* + */ || ch === 0x2D /* - */) {
  	      if (CHOMPING_CLIP === chomping) {
  	        chomping = ch === 0x2B /* + */ ? CHOMPING_KEEP : CHOMPING_STRIP;
  	      } else {
  	        throwError(state, 'repeat of a chomping mode identifier');
  	      }
  	    } else if ((tmp = fromDecimalCode(ch)) >= 0) {
  	      if (tmp === 0) {
  	        throwError(state, 'bad explicit indentation width of a block scalar; it cannot be less than one');
  	      } else if (!detectedIndent) {
  	        textIndent = nodeIndent + tmp - 1;
  	        detectedIndent = true;
  	      } else {
  	        throwError(state, 'repeat of an indentation width identifier');
  	      }
  	    } else {
  	      break;
  	    }
  	  }
  	  if (is_WHITE_SPACE(ch)) {
  	    do {
  	      ch = state.input.charCodeAt(++state.position);
  	    } while (is_WHITE_SPACE(ch));
  	    if (ch === 0x23 /* # */) {
  	      do {
  	        ch = state.input.charCodeAt(++state.position);
  	      } while (!is_EOL(ch) && ch !== 0);
  	    }
  	  }
  	  while (ch !== 0) {
  	    readLineBreak(state);
  	    state.lineIndent = 0;
  	    ch = state.input.charCodeAt(state.position);
  	    while ((!detectedIndent || state.lineIndent < textIndent) && ch === 0x20 /* Space */) {
  	      state.lineIndent++;
  	      ch = state.input.charCodeAt(++state.position);
  	    }
  	    if (!detectedIndent && state.lineIndent > textIndent) {
  	      textIndent = state.lineIndent;
  	    }
  	    if (is_EOL(ch)) {
  	      emptyLines++;
  	      continue;
  	    }

  	    // End of the scalar.
  	    if (state.lineIndent < textIndent) {
  	      // Perform the chomping.
  	      if (chomping === CHOMPING_KEEP) {
  	        state.result += common.repeat('\n', didReadContent ? 1 + emptyLines : emptyLines);
  	      } else if (chomping === CHOMPING_CLIP) {
  	        if (didReadContent) {
  	          // i.e. only if the scalar is not empty.
  	          state.result += '\n';
  	        }
  	      }

  	      // Break this `while` cycle and go to the funciton's epilogue.
  	      break;
  	    }

  	    // Folded style: use fancy rules to handle line breaks.
  	    if (folding) {
  	      // Lines starting with white space characters (more-indented lines) are not folded.
  	      if (is_WHITE_SPACE(ch)) {
  	        atMoreIndented = true;
  	        // except for the first content line (cf. Example 8.1)
  	        state.result += common.repeat('\n', didReadContent ? 1 + emptyLines : emptyLines);

  	        // End of more-indented block.
  	      } else if (atMoreIndented) {
  	        atMoreIndented = false;
  	        state.result += common.repeat('\n', emptyLines + 1);

  	        // Just one line break - perceive as the same line.
  	      } else if (emptyLines === 0) {
  	        if (didReadContent) {
  	          // i.e. only if we have already read some scalar content.
  	          state.result += ' ';
  	        }

  	        // Several line breaks - perceive as different lines.
  	      } else {
  	        state.result += common.repeat('\n', emptyLines);
  	      }

  	      // Literal style: just add exact number of line breaks between content lines.
  	    } else {
  	      // Keep all line breaks except the header line break.
  	      state.result += common.repeat('\n', didReadContent ? 1 + emptyLines : emptyLines);
  	    }
  	    didReadContent = true;
  	    detectedIndent = true;
  	    emptyLines = 0;
  	    captureStart = state.position;
  	    while (!is_EOL(ch) && ch !== 0) {
  	      ch = state.input.charCodeAt(++state.position);
  	    }
  	    captureSegment(state, captureStart, state.position, false);
  	  }
  	  return true;
  	}
  	function readBlockSequence(state, nodeIndent) {
  	  var _line,
  	    _tag = state.tag,
  	    _anchor = state.anchor,
  	    _result = [],
  	    following,
  	    detected = false,
  	    ch;
  	  if (state.anchor !== null) {
  	    state.anchorMap[state.anchor] = _result;
  	  }
  	  ch = state.input.charCodeAt(state.position);
  	  while (ch !== 0) {
  	    if (ch !== 0x2D /* - */) {
  	      break;
  	    }
  	    following = state.input.charCodeAt(state.position + 1);
  	    if (!is_WS_OR_EOL(following)) {
  	      break;
  	    }
  	    detected = true;
  	    state.position++;
  	    if (skipSeparationSpace(state, true, -1)) {
  	      if (state.lineIndent <= nodeIndent) {
  	        _result.push(null);
  	        ch = state.input.charCodeAt(state.position);
  	        continue;
  	      }
  	    }
  	    _line = state.line;
  	    composeNode(state, nodeIndent, CONTEXT_BLOCK_IN, false, true);
  	    _result.push(state.result);
  	    skipSeparationSpace(state, true, -1);
  	    ch = state.input.charCodeAt(state.position);
  	    if ((state.line === _line || state.lineIndent > nodeIndent) && ch !== 0) {
  	      throwError(state, 'bad indentation of a sequence entry');
  	    } else if (state.lineIndent < nodeIndent) {
  	      break;
  	    }
  	  }
  	  if (detected) {
  	    state.tag = _tag;
  	    state.anchor = _anchor;
  	    state.kind = 'sequence';
  	    state.result = _result;
  	    return true;
  	  }
  	  return false;
  	}
  	function readBlockMapping(state, nodeIndent, flowIndent) {
  	  var following,
  	    allowCompact,
  	    _line,
  	    _pos,
  	    _tag = state.tag,
  	    _anchor = state.anchor,
  	    _result = {},
  	    overridableKeys = {},
  	    keyTag = null,
  	    keyNode = null,
  	    valueNode = null,
  	    atExplicitKey = false,
  	    detected = false,
  	    ch;
  	  if (state.anchor !== null) {
  	    state.anchorMap[state.anchor] = _result;
  	  }
  	  ch = state.input.charCodeAt(state.position);
  	  while (ch !== 0) {
  	    following = state.input.charCodeAt(state.position + 1);
  	    _line = state.line; // Save the current line.
  	    _pos = state.position;

  	    //
  	    // Explicit notation case. There are two separate blocks:
  	    // first for the key (denoted by "?") and second for the value (denoted by ":")
  	    //
  	    if ((ch === 0x3F /* ? */ || ch === 0x3A /* : */) && is_WS_OR_EOL(following)) {
  	      if (ch === 0x3F /* ? */) {
  	        if (atExplicitKey) {
  	          storeMappingPair(state, _result, overridableKeys, keyTag, keyNode, null);
  	          keyTag = keyNode = valueNode = null;
  	        }
  	        detected = true;
  	        atExplicitKey = true;
  	        allowCompact = true;
  	      } else if (atExplicitKey) {
  	        // i.e. 0x3A/* : */ === character after the explicit key.
  	        atExplicitKey = false;
  	        allowCompact = true;
  	      } else {
  	        throwError(state, 'incomplete explicit mapping pair; a key node is missed; or followed by a non-tabulated empty line');
  	      }
  	      state.position += 1;
  	      ch = following;

  	      //
  	      // Implicit notation case. Flow-style node as the key first, then ":", and the value.
  	      //
  	    } else if (composeNode(state, flowIndent, CONTEXT_FLOW_OUT, false, true)) {
  	      if (state.line === _line) {
  	        ch = state.input.charCodeAt(state.position);
  	        while (is_WHITE_SPACE(ch)) {
  	          ch = state.input.charCodeAt(++state.position);
  	        }
  	        if (ch === 0x3A /* : */) {
  	          ch = state.input.charCodeAt(++state.position);
  	          if (!is_WS_OR_EOL(ch)) {
  	            throwError(state, 'a whitespace character is expected after the key-value separator within a block mapping');
  	          }
  	          if (atExplicitKey) {
  	            storeMappingPair(state, _result, overridableKeys, keyTag, keyNode, null);
  	            keyTag = keyNode = valueNode = null;
  	          }
  	          detected = true;
  	          atExplicitKey = false;
  	          allowCompact = false;
  	          keyTag = state.tag;
  	          keyNode = state.result;
  	        } else if (detected) {
  	          throwError(state, 'can not read an implicit mapping pair; a colon is missed');
  	        } else {
  	          state.tag = _tag;
  	          state.anchor = _anchor;
  	          return true; // Keep the result of `composeNode`.
  	        }
  	      } else if (detected) {
  	        throwError(state, 'can not read a block mapping entry; a multiline key may not be an implicit key');
  	      } else {
  	        state.tag = _tag;
  	        state.anchor = _anchor;
  	        return true; // Keep the result of `composeNode`.
  	      }
  	    } else {
  	      break; // Reading is done. Go to the epilogue.
  	    }

  	    //
  	    // Common reading code for both explicit and implicit notations.
  	    //
  	    if (state.line === _line || state.lineIndent > nodeIndent) {
  	      if (composeNode(state, nodeIndent, CONTEXT_BLOCK_OUT, true, allowCompact)) {
  	        if (atExplicitKey) {
  	          keyNode = state.result;
  	        } else {
  	          valueNode = state.result;
  	        }
  	      }
  	      if (!atExplicitKey) {
  	        storeMappingPair(state, _result, overridableKeys, keyTag, keyNode, valueNode, _line, _pos);
  	        keyTag = keyNode = valueNode = null;
  	      }
  	      skipSeparationSpace(state, true, -1);
  	      ch = state.input.charCodeAt(state.position);
  	    }
  	    if (state.lineIndent > nodeIndent && ch !== 0) {
  	      throwError(state, 'bad indentation of a mapping entry');
  	    } else if (state.lineIndent < nodeIndent) {
  	      break;
  	    }
  	  }

  	  //
  	  // Epilogue.
  	  //

  	  // Special case: last mapping's node contains only the key in explicit notation.
  	  if (atExplicitKey) {
  	    storeMappingPair(state, _result, overridableKeys, keyTag, keyNode, null);
  	  }

  	  // Expose the resulting mapping.
  	  if (detected) {
  	    state.tag = _tag;
  	    state.anchor = _anchor;
  	    state.kind = 'mapping';
  	    state.result = _result;
  	  }
  	  return detected;
  	}
  	function readTagProperty(state) {
  	  var _position,
  	    isVerbatim = false,
  	    isNamed = false,
  	    tagHandle,
  	    tagName,
  	    ch;
  	  ch = state.input.charCodeAt(state.position);
  	  if (ch !== 0x21 /* ! */) return false;
  	  if (state.tag !== null) {
  	    throwError(state, 'duplication of a tag property');
  	  }
  	  ch = state.input.charCodeAt(++state.position);
  	  if (ch === 0x3C /* < */) {
  	    isVerbatim = true;
  	    ch = state.input.charCodeAt(++state.position);
  	  } else if (ch === 0x21 /* ! */) {
  	    isNamed = true;
  	    tagHandle = '!!';
  	    ch = state.input.charCodeAt(++state.position);
  	  } else {
  	    tagHandle = '!';
  	  }
  	  _position = state.position;
  	  if (isVerbatim) {
  	    do {
  	      ch = state.input.charCodeAt(++state.position);
  	    } while (ch !== 0 && ch !== 0x3E /* > */);
  	    if (state.position < state.length) {
  	      tagName = state.input.slice(_position, state.position);
  	      ch = state.input.charCodeAt(++state.position);
  	    } else {
  	      throwError(state, 'unexpected end of the stream within a verbatim tag');
  	    }
  	  } else {
  	    while (ch !== 0 && !is_WS_OR_EOL(ch)) {
  	      if (ch === 0x21 /* ! */) {
  	        if (!isNamed) {
  	          tagHandle = state.input.slice(_position - 1, state.position + 1);
  	          if (!PATTERN_TAG_HANDLE.test(tagHandle)) {
  	            throwError(state, 'named tag handle cannot contain such characters');
  	          }
  	          isNamed = true;
  	          _position = state.position + 1;
  	        } else {
  	          throwError(state, 'tag suffix cannot contain exclamation marks');
  	        }
  	      }
  	      ch = state.input.charCodeAt(++state.position);
  	    }
  	    tagName = state.input.slice(_position, state.position);
  	    if (PATTERN_FLOW_INDICATORS.test(tagName)) {
  	      throwError(state, 'tag suffix cannot contain flow indicator characters');
  	    }
  	  }
  	  if (tagName && !PATTERN_TAG_URI.test(tagName)) {
  	    throwError(state, 'tag name cannot contain such characters: ' + tagName);
  	  }
  	  if (isVerbatim) {
  	    state.tag = tagName;
  	  } else if (_hasOwnProperty.call(state.tagMap, tagHandle)) {
  	    state.tag = state.tagMap[tagHandle] + tagName;
  	  } else if (tagHandle === '!') {
  	    state.tag = '!' + tagName;
  	  } else if (tagHandle === '!!') {
  	    state.tag = 'tag:yaml.org,2002:' + tagName;
  	  } else {
  	    throwError(state, 'undeclared tag handle "' + tagHandle + '"');
  	  }
  	  return true;
  	}
  	function readAnchorProperty(state) {
  	  var _position, ch;
  	  ch = state.input.charCodeAt(state.position);
  	  if (ch !== 0x26 /* & */) return false;
  	  if (state.anchor !== null) {
  	    throwError(state, 'duplication of an anchor property');
  	  }
  	  ch = state.input.charCodeAt(++state.position);
  	  _position = state.position;
  	  while (ch !== 0 && !is_WS_OR_EOL(ch) && !is_FLOW_INDICATOR(ch)) {
  	    ch = state.input.charCodeAt(++state.position);
  	  }
  	  if (state.position === _position) {
  	    throwError(state, 'name of an anchor node must contain at least one character');
  	  }
  	  state.anchor = state.input.slice(_position, state.position);
  	  return true;
  	}
  	function readAlias(state) {
  	  var _position, alias, ch;
  	  ch = state.input.charCodeAt(state.position);
  	  if (ch !== 0x2A /* * */) return false;
  	  ch = state.input.charCodeAt(++state.position);
  	  _position = state.position;
  	  while (ch !== 0 && !is_WS_OR_EOL(ch) && !is_FLOW_INDICATOR(ch)) {
  	    ch = state.input.charCodeAt(++state.position);
  	  }
  	  if (state.position === _position) {
  	    throwError(state, 'name of an alias node must contain at least one character');
  	  }
  	  alias = state.input.slice(_position, state.position);
  	  if (!_hasOwnProperty.call(state.anchorMap, alias)) {
  	    throwError(state, 'unidentified alias "' + alias + '"');
  	  }
  	  state.result = state.anchorMap[alias];
  	  skipSeparationSpace(state, true, -1);
  	  return true;
  	}
  	function composeNode(state, parentIndent, nodeContext, allowToSeek, allowCompact) {
  	  var allowBlockStyles,
  	    allowBlockScalars,
  	    allowBlockCollections,
  	    indentStatus = 1,
  	    // 1: this>parent, 0: this=parent, -1: this<parent
  	    atNewLine = false,
  	    hasContent = false,
  	    typeIndex,
  	    typeQuantity,
  	    type,
  	    flowIndent,
  	    blockIndent;
  	  if (state.listener !== null) {
  	    state.listener('open', state);
  	  }
  	  state.tag = null;
  	  state.anchor = null;
  	  state.kind = null;
  	  state.result = null;
  	  allowBlockStyles = allowBlockScalars = allowBlockCollections = CONTEXT_BLOCK_OUT === nodeContext || CONTEXT_BLOCK_IN === nodeContext;
  	  if (allowToSeek) {
  	    if (skipSeparationSpace(state, true, -1)) {
  	      atNewLine = true;
  	      if (state.lineIndent > parentIndent) {
  	        indentStatus = 1;
  	      } else if (state.lineIndent === parentIndent) {
  	        indentStatus = 0;
  	      } else if (state.lineIndent < parentIndent) {
  	        indentStatus = -1;
  	      }
  	    }
  	  }
  	  if (indentStatus === 1) {
  	    while (readTagProperty(state) || readAnchorProperty(state)) {
  	      if (skipSeparationSpace(state, true, -1)) {
  	        atNewLine = true;
  	        allowBlockCollections = allowBlockStyles;
  	        if (state.lineIndent > parentIndent) {
  	          indentStatus = 1;
  	        } else if (state.lineIndent === parentIndent) {
  	          indentStatus = 0;
  	        } else if (state.lineIndent < parentIndent) {
  	          indentStatus = -1;
  	        }
  	      } else {
  	        allowBlockCollections = false;
  	      }
  	    }
  	  }
  	  if (allowBlockCollections) {
  	    allowBlockCollections = atNewLine || allowCompact;
  	  }
  	  if (indentStatus === 1 || CONTEXT_BLOCK_OUT === nodeContext) {
  	    if (CONTEXT_FLOW_IN === nodeContext || CONTEXT_FLOW_OUT === nodeContext) {
  	      flowIndent = parentIndent;
  	    } else {
  	      flowIndent = parentIndent + 1;
  	    }
  	    blockIndent = state.position - state.lineStart;
  	    if (indentStatus === 1) {
  	      if (allowBlockCollections && (readBlockSequence(state, blockIndent) || readBlockMapping(state, blockIndent, flowIndent)) || readFlowCollection(state, flowIndent)) {
  	        hasContent = true;
  	      } else {
  	        if (allowBlockScalars && readBlockScalar(state, flowIndent) || readSingleQuotedScalar(state, flowIndent) || readDoubleQuotedScalar(state, flowIndent)) {
  	          hasContent = true;
  	        } else if (readAlias(state)) {
  	          hasContent = true;
  	          if (state.tag !== null || state.anchor !== null) {
  	            throwError(state, 'alias node should not have any properties');
  	          }
  	        } else if (readPlainScalar(state, flowIndent, CONTEXT_FLOW_IN === nodeContext)) {
  	          hasContent = true;
  	          if (state.tag === null) {
  	            state.tag = '?';
  	          }
  	        }
  	        if (state.anchor !== null) {
  	          state.anchorMap[state.anchor] = state.result;
  	        }
  	      }
  	    } else if (indentStatus === 0) {
  	      // Special case: block sequences are allowed to have same indentation level as the parent.
  	      // http://www.yaml.org/spec/1.2/spec.html#id2799784
  	      hasContent = allowBlockCollections && readBlockSequence(state, blockIndent);
  	    }
  	  }
  	  if (state.tag !== null && state.tag !== '!') {
  	    if (state.tag === '?') {
  	      // Implicit resolving is not allowed for non-scalar types, and '?'
  	      // non-specific tag is only automatically assigned to plain scalars.
  	      //
  	      // We only need to check kind conformity in case user explicitly assigns '?'
  	      // tag, for example like this: "!<?> [0]"
  	      //
  	      if (state.result !== null && state.kind !== 'scalar') {
  	        throwError(state, 'unacceptable node kind for !<?> tag; it should be "scalar", not "' + state.kind + '"');
  	      }
  	      for (typeIndex = 0, typeQuantity = state.implicitTypes.length; typeIndex < typeQuantity; typeIndex += 1) {
  	        type = state.implicitTypes[typeIndex];
  	        if (type.resolve(state.result)) {
  	          // `state.result` updated in resolver if matched
  	          state.result = type.construct(state.result);
  	          state.tag = type.tag;
  	          if (state.anchor !== null) {
  	            state.anchorMap[state.anchor] = state.result;
  	          }
  	          break;
  	        }
  	      }
  	    } else if (_hasOwnProperty.call(state.typeMap[state.kind || 'fallback'], state.tag)) {
  	      type = state.typeMap[state.kind || 'fallback'][state.tag];
  	      if (state.result !== null && type.kind !== state.kind) {
  	        throwError(state, 'unacceptable node kind for !<' + state.tag + '> tag; it should be "' + type.kind + '", not "' + state.kind + '"');
  	      }
  	      if (!type.resolve(state.result)) {
  	        // `state.result` updated in resolver if matched
  	        throwError(state, 'cannot resolve a node with !<' + state.tag + '> explicit tag');
  	      } else {
  	        state.result = type.construct(state.result);
  	        if (state.anchor !== null) {
  	          state.anchorMap[state.anchor] = state.result;
  	        }
  	      }
  	    } else {
  	      throwError(state, 'unknown tag !<' + state.tag + '>');
  	    }
  	  }
  	  if (state.listener !== null) {
  	    state.listener('close', state);
  	  }
  	  return state.tag !== null || state.anchor !== null || hasContent;
  	}
  	function readDocument(state) {
  	  var documentStart = state.position,
  	    _position,
  	    directiveName,
  	    directiveArgs,
  	    hasDirectives = false,
  	    ch;
  	  state.version = null;
  	  state.checkLineBreaks = state.legacy;
  	  state.tagMap = {};
  	  state.anchorMap = {};
  	  while ((ch = state.input.charCodeAt(state.position)) !== 0) {
  	    skipSeparationSpace(state, true, -1);
  	    ch = state.input.charCodeAt(state.position);
  	    if (state.lineIndent > 0 || ch !== 0x25 /* % */) {
  	      break;
  	    }
  	    hasDirectives = true;
  	    ch = state.input.charCodeAt(++state.position);
  	    _position = state.position;
  	    while (ch !== 0 && !is_WS_OR_EOL(ch)) {
  	      ch = state.input.charCodeAt(++state.position);
  	    }
  	    directiveName = state.input.slice(_position, state.position);
  	    directiveArgs = [];
  	    if (directiveName.length < 1) {
  	      throwError(state, 'directive name must not be less than one character in length');
  	    }
  	    while (ch !== 0) {
  	      while (is_WHITE_SPACE(ch)) {
  	        ch = state.input.charCodeAt(++state.position);
  	      }
  	      if (ch === 0x23 /* # */) {
  	        do {
  	          ch = state.input.charCodeAt(++state.position);
  	        } while (ch !== 0 && !is_EOL(ch));
  	        break;
  	      }
  	      if (is_EOL(ch)) break;
  	      _position = state.position;
  	      while (ch !== 0 && !is_WS_OR_EOL(ch)) {
  	        ch = state.input.charCodeAt(++state.position);
  	      }
  	      directiveArgs.push(state.input.slice(_position, state.position));
  	    }
  	    if (ch !== 0) readLineBreak(state);
  	    if (_hasOwnProperty.call(directiveHandlers, directiveName)) {
  	      directiveHandlers[directiveName](state, directiveName, directiveArgs);
  	    } else {
  	      throwWarning(state, 'unknown document directive "' + directiveName + '"');
  	    }
  	  }
  	  skipSeparationSpace(state, true, -1);
  	  if (state.lineIndent === 0 && state.input.charCodeAt(state.position) === 0x2D /* - */ && state.input.charCodeAt(state.position + 1) === 0x2D /* - */ && state.input.charCodeAt(state.position + 2) === 0x2D /* - */) {
  	    state.position += 3;
  	    skipSeparationSpace(state, true, -1);
  	  } else if (hasDirectives) {
  	    throwError(state, 'directives end mark is expected');
  	  }
  	  composeNode(state, state.lineIndent - 1, CONTEXT_BLOCK_OUT, false, true);
  	  skipSeparationSpace(state, true, -1);
  	  if (state.checkLineBreaks && PATTERN_NON_ASCII_LINE_BREAKS.test(state.input.slice(documentStart, state.position))) {
  	    throwWarning(state, 'non-ASCII line breaks are interpreted as content');
  	  }
  	  state.documents.push(state.result);
  	  if (state.position === state.lineStart && testDocumentSeparator(state)) {
  	    if (state.input.charCodeAt(state.position) === 0x2E /* . */) {
  	      state.position += 3;
  	      skipSeparationSpace(state, true, -1);
  	    }
  	    return;
  	  }
  	  if (state.position < state.length - 1) {
  	    throwError(state, 'end of the stream or a document separator is expected');
  	  } else {
  	    return;
  	  }
  	}
  	function loadDocuments(input, options) {
  	  input = String(input);
  	  options = options || {};
  	  if (input.length !== 0) {
  	    // Add tailing `\n` if not exists
  	    if (input.charCodeAt(input.length - 1) !== 0x0A /* LF */ && input.charCodeAt(input.length - 1) !== 0x0D /* CR */) {
  	      input += '\n';
  	    }

  	    // Strip BOM
  	    if (input.charCodeAt(0) === 0xFEFF) {
  	      input = input.slice(1);
  	    }
  	  }
  	  var state = new State(input, options);
  	  var nullpos = input.indexOf('\0');
  	  if (nullpos !== -1) {
  	    state.position = nullpos;
  	    throwError(state, 'null byte is not allowed in input');
  	  }

  	  // Use 0 as string terminator. That significantly simplifies bounds check.
  	  state.input += '\0';
  	  while (state.input.charCodeAt(state.position) === 0x20 /* Space */) {
  	    state.lineIndent += 1;
  	    state.position += 1;
  	  }
  	  while (state.position < state.length - 1) {
  	    readDocument(state);
  	  }
  	  return state.documents;
  	}
  	function loadAll(input, iterator, options) {
  	  if (iterator !== null && typeof iterator === 'object' && typeof options === 'undefined') {
  	    options = iterator;
  	    iterator = null;
  	  }
  	  var documents = loadDocuments(input, options);
  	  if (typeof iterator !== 'function') {
  	    return documents;
  	  }
  	  for (var index = 0, length = documents.length; index < length; index += 1) {
  	    iterator(documents[index]);
  	  }
  	}
  	function load(input, options) {
  	  var documents = loadDocuments(input, options);
  	  if (documents.length === 0) {
  	    /*eslint-disable no-undefined*/
  	    return undefined;
  	  } else if (documents.length === 1) {
  	    return documents[0];
  	  }
  	  throw new YAMLException('expected a single document in the stream, but found more');
  	}
  	function safeLoadAll(input, iterator, options) {
  	  if (typeof iterator === 'object' && iterator !== null && typeof options === 'undefined') {
  	    options = iterator;
  	    iterator = null;
  	  }
  	  return loadAll(input, iterator, common.extend({
  	    schema: DEFAULT_SAFE_SCHEMA
  	  }, options));
  	}
  	function safeLoad(input, options) {
  	  return load(input, common.extend({
  	    schema: DEFAULT_SAFE_SCHEMA
  	  }, options));
  	}
  	loader.loadAll = loadAll;
  	loader.load = load;
  	loader.safeLoadAll = safeLoadAll;
  	loader.safeLoad = safeLoad;
  	return loader;
  }

  var dumper = {};

  var hasRequiredDumper;

  function requireDumper () {
  	if (hasRequiredDumper) return dumper;
  	hasRequiredDumper = 1;

  	/*eslint-disable no-use-before-define*/
  	var common = requireCommon();
  	var YAMLException = requireException();
  	var DEFAULT_FULL_SCHEMA = requireDefault_full();
  	var DEFAULT_SAFE_SCHEMA = requireDefault_safe();
  	var _toString = Object.prototype.toString;
  	var _hasOwnProperty = Object.prototype.hasOwnProperty;
  	var CHAR_TAB = 0x09; /* Tab */
  	var CHAR_LINE_FEED = 0x0A; /* LF */
  	var CHAR_CARRIAGE_RETURN = 0x0D; /* CR */
  	var CHAR_SPACE = 0x20; /* Space */
  	var CHAR_EXCLAMATION = 0x21; /* ! */
  	var CHAR_DOUBLE_QUOTE = 0x22; /* " */
  	var CHAR_SHARP = 0x23; /* # */
  	var CHAR_PERCENT = 0x25; /* % */
  	var CHAR_AMPERSAND = 0x26; /* & */
  	var CHAR_SINGLE_QUOTE = 0x27; /* ' */
  	var CHAR_ASTERISK = 0x2A; /* * */
  	var CHAR_COMMA = 0x2C; /* , */
  	var CHAR_MINUS = 0x2D; /* - */
  	var CHAR_COLON = 0x3A; /* : */
  	var CHAR_EQUALS = 0x3D; /* = */
  	var CHAR_GREATER_THAN = 0x3E; /* > */
  	var CHAR_QUESTION = 0x3F; /* ? */
  	var CHAR_COMMERCIAL_AT = 0x40; /* @ */
  	var CHAR_LEFT_SQUARE_BRACKET = 0x5B; /* [ */
  	var CHAR_RIGHT_SQUARE_BRACKET = 0x5D; /* ] */
  	var CHAR_GRAVE_ACCENT = 0x60; /* ` */
  	var CHAR_LEFT_CURLY_BRACKET = 0x7B; /* { */
  	var CHAR_VERTICAL_LINE = 0x7C; /* | */
  	var CHAR_RIGHT_CURLY_BRACKET = 0x7D; /* } */

  	var ESCAPE_SEQUENCES = {};
  	ESCAPE_SEQUENCES[0x00] = '\\0';
  	ESCAPE_SEQUENCES[0x07] = '\\a';
  	ESCAPE_SEQUENCES[0x08] = '\\b';
  	ESCAPE_SEQUENCES[0x09] = '\\t';
  	ESCAPE_SEQUENCES[0x0A] = '\\n';
  	ESCAPE_SEQUENCES[0x0B] = '\\v';
  	ESCAPE_SEQUENCES[0x0C] = '\\f';
  	ESCAPE_SEQUENCES[0x0D] = '\\r';
  	ESCAPE_SEQUENCES[0x1B] = '\\e';
  	ESCAPE_SEQUENCES[0x22] = '\\"';
  	ESCAPE_SEQUENCES[0x5C] = '\\\\';
  	ESCAPE_SEQUENCES[0x85] = '\\N';
  	ESCAPE_SEQUENCES[0xA0] = '\\_';
  	ESCAPE_SEQUENCES[0x2028] = '\\L';
  	ESCAPE_SEQUENCES[0x2029] = '\\P';
  	var DEPRECATED_BOOLEANS_SYNTAX = ['y', 'Y', 'yes', 'Yes', 'YES', 'on', 'On', 'ON', 'n', 'N', 'no', 'No', 'NO', 'off', 'Off', 'OFF'];
  	function compileStyleMap(schema, map) {
  	  var result, keys, index, length, tag, style, type;
  	  if (map === null) return {};
  	  result = {};
  	  keys = Object.keys(map);
  	  for (index = 0, length = keys.length; index < length; index += 1) {
  	    tag = keys[index];
  	    style = String(map[tag]);
  	    if (tag.slice(0, 2) === '!!') {
  	      tag = 'tag:yaml.org,2002:' + tag.slice(2);
  	    }
  	    type = schema.compiledTypeMap['fallback'][tag];
  	    if (type && _hasOwnProperty.call(type.styleAliases, style)) {
  	      style = type.styleAliases[style];
  	    }
  	    result[tag] = style;
  	  }
  	  return result;
  	}
  	function encodeHex(character) {
  	  var string, handle, length;
  	  string = character.toString(16).toUpperCase();
  	  if (character <= 0xFF) {
  	    handle = 'x';
  	    length = 2;
  	  } else if (character <= 0xFFFF) {
  	    handle = 'u';
  	    length = 4;
  	  } else if (character <= 0xFFFFFFFF) {
  	    handle = 'U';
  	    length = 8;
  	  } else {
  	    throw new YAMLException('code point within a string may not be greater than 0xFFFFFFFF');
  	  }
  	  return '\\' + handle + common.repeat('0', length - string.length) + string;
  	}
  	function State(options) {
  	  this.schema = options['schema'] || DEFAULT_FULL_SCHEMA;
  	  this.indent = Math.max(1, options['indent'] || 2);
  	  this.noArrayIndent = options['noArrayIndent'] || false;
  	  this.skipInvalid = options['skipInvalid'] || false;
  	  this.flowLevel = common.isNothing(options['flowLevel']) ? -1 : options['flowLevel'];
  	  this.styleMap = compileStyleMap(this.schema, options['styles'] || null);
  	  this.sortKeys = options['sortKeys'] || false;
  	  this.lineWidth = options['lineWidth'] || 80;
  	  this.noRefs = options['noRefs'] || false;
  	  this.noCompatMode = options['noCompatMode'] || false;
  	  this.condenseFlow = options['condenseFlow'] || false;
  	  this.implicitTypes = this.schema.compiledImplicit;
  	  this.explicitTypes = this.schema.compiledExplicit;
  	  this.tag = null;
  	  this.result = '';
  	  this.duplicates = [];
  	  this.usedDuplicates = null;
  	}

  	// Indents every line in a string. Empty lines (\n only) are not indented.
  	function indentString(string, spaces) {
  	  var ind = common.repeat(' ', spaces),
  	    position = 0,
  	    next = -1,
  	    result = '',
  	    line,
  	    length = string.length;
  	  while (position < length) {
  	    next = string.indexOf('\n', position);
  	    if (next === -1) {
  	      line = string.slice(position);
  	      position = length;
  	    } else {
  	      line = string.slice(position, next + 1);
  	      position = next + 1;
  	    }
  	    if (line.length && line !== '\n') result += ind;
  	    result += line;
  	  }
  	  return result;
  	}
  	function generateNextLine(state, level) {
  	  return '\n' + common.repeat(' ', state.indent * level);
  	}
  	function testImplicitResolving(state, str) {
  	  var index, length, type;
  	  for (index = 0, length = state.implicitTypes.length; index < length; index += 1) {
  	    type = state.implicitTypes[index];
  	    if (type.resolve(str)) {
  	      return true;
  	    }
  	  }
  	  return false;
  	}

  	// [33] s-white ::= s-space | s-tab
  	function isWhitespace(c) {
  	  return c === CHAR_SPACE || c === CHAR_TAB;
  	}

  	// Returns true if the character can be printed without escaping.
  	// From YAML 1.2: "any allowed characters known to be non-printable
  	// should also be escaped. [However,] This isn’t mandatory"
  	// Derived from nb-char - \t - #x85 - #xA0 - #x2028 - #x2029.
  	function isPrintable(c) {
  	  return 0x00020 <= c && c <= 0x00007E || 0x000A1 <= c && c <= 0x00D7FF && c !== 0x2028 && c !== 0x2029 || 0x0E000 <= c && c <= 0x00FFFD && c !== 0xFEFF /* BOM */ || 0x10000 <= c && c <= 0x10FFFF;
  	}

  	// [34] ns-char ::= nb-char - s-white
  	// [27] nb-char ::= c-printable - b-char - c-byte-order-mark
  	// [26] b-char  ::= b-line-feed | b-carriage-return
  	// [24] b-line-feed       ::=     #xA    /* LF */
  	// [25] b-carriage-return ::=     #xD    /* CR */
  	// [3]  c-byte-order-mark ::=     #xFEFF
  	function isNsChar(c) {
  	  return isPrintable(c) && !isWhitespace(c)
  	  // byte-order-mark
  	  && c !== 0xFEFF
  	  // b-char
  	  && c !== CHAR_CARRIAGE_RETURN && c !== CHAR_LINE_FEED;
  	}

  	// Simplified test for values allowed after the first character in plain style.
  	function isPlainSafe(c, prev) {
  	  // Uses a subset of nb-char - c-flow-indicator - ":" - "#"
  	  // where nb-char ::= c-printable - b-char - c-byte-order-mark.
  	  return isPrintable(c) && c !== 0xFEFF
  	  // - c-flow-indicator
  	  && c !== CHAR_COMMA && c !== CHAR_LEFT_SQUARE_BRACKET && c !== CHAR_RIGHT_SQUARE_BRACKET && c !== CHAR_LEFT_CURLY_BRACKET && c !== CHAR_RIGHT_CURLY_BRACKET
  	  // - ":" - "#"
  	  // /* An ns-char preceding */ "#"
  	  && c !== CHAR_COLON && (c !== CHAR_SHARP || prev && isNsChar(prev));
  	}

  	// Simplified test for values allowed as the first character in plain style.
  	function isPlainSafeFirst(c) {
  	  // Uses a subset of ns-char - c-indicator
  	  // where ns-char = nb-char - s-white.
  	  return isPrintable(c) && c !== 0xFEFF && !isWhitespace(c) // - s-white
  	  // - (c-indicator ::=
  	  // “-” | “?” | “:” | “,” | “[” | “]” | “{” | “}”
  	  && c !== CHAR_MINUS && c !== CHAR_QUESTION && c !== CHAR_COLON && c !== CHAR_COMMA && c !== CHAR_LEFT_SQUARE_BRACKET && c !== CHAR_RIGHT_SQUARE_BRACKET && c !== CHAR_LEFT_CURLY_BRACKET && c !== CHAR_RIGHT_CURLY_BRACKET
  	  // | “#” | “&” | “*” | “!” | “|” | “=” | “>” | “'” | “"”
  	  && c !== CHAR_SHARP && c !== CHAR_AMPERSAND && c !== CHAR_ASTERISK && c !== CHAR_EXCLAMATION && c !== CHAR_VERTICAL_LINE && c !== CHAR_EQUALS && c !== CHAR_GREATER_THAN && c !== CHAR_SINGLE_QUOTE && c !== CHAR_DOUBLE_QUOTE
  	  // | “%” | “@” | “`”)
  	  && c !== CHAR_PERCENT && c !== CHAR_COMMERCIAL_AT && c !== CHAR_GRAVE_ACCENT;
  	}

  	// Determines whether block indentation indicator is required.
  	function needIndentIndicator(string) {
  	  var leadingSpaceRe = /^\n* /;
  	  return leadingSpaceRe.test(string);
  	}
  	var STYLE_PLAIN = 1,
  	  STYLE_SINGLE = 2,
  	  STYLE_LITERAL = 3,
  	  STYLE_FOLDED = 4,
  	  STYLE_DOUBLE = 5;

  	// Determines which scalar styles are possible and returns the preferred style.
  	// lineWidth = -1 => no limit.
  	// Pre-conditions: str.length > 0.
  	// Post-conditions:
  	//    STYLE_PLAIN or STYLE_SINGLE => no \n are in the string.
  	//    STYLE_LITERAL => no lines are suitable for folding (or lineWidth is -1).
  	//    STYLE_FOLDED => a line > lineWidth and can be folded (and lineWidth != -1).
  	function chooseScalarStyle(string, singleLineOnly, indentPerLevel, lineWidth, testAmbiguousType) {
  	  var i;
  	  var char, prev_char;
  	  var hasLineBreak = false;
  	  var hasFoldableLine = false; // only checked if shouldTrackWidth
  	  var shouldTrackWidth = lineWidth !== -1;
  	  var previousLineBreak = -1; // count the first line correctly
  	  var plain = isPlainSafeFirst(string.charCodeAt(0)) && !isWhitespace(string.charCodeAt(string.length - 1));
  	  if (singleLineOnly) {
  	    // Case: no block styles.
  	    // Check for disallowed characters to rule out plain and single.
  	    for (i = 0; i < string.length; i++) {
  	      char = string.charCodeAt(i);
  	      if (!isPrintable(char)) {
  	        return STYLE_DOUBLE;
  	      }
  	      prev_char = i > 0 ? string.charCodeAt(i - 1) : null;
  	      plain = plain && isPlainSafe(char, prev_char);
  	    }
  	  } else {
  	    // Case: block styles permitted.
  	    for (i = 0; i < string.length; i++) {
  	      char = string.charCodeAt(i);
  	      if (char === CHAR_LINE_FEED) {
  	        hasLineBreak = true;
  	        // Check if any line can be folded.
  	        if (shouldTrackWidth) {
  	          hasFoldableLine = hasFoldableLine ||
  	          // Foldable line = too long, and not more-indented.
  	          i - previousLineBreak - 1 > lineWidth && string[previousLineBreak + 1] !== ' ';
  	          previousLineBreak = i;
  	        }
  	      } else if (!isPrintable(char)) {
  	        return STYLE_DOUBLE;
  	      }
  	      prev_char = i > 0 ? string.charCodeAt(i - 1) : null;
  	      plain = plain && isPlainSafe(char, prev_char);
  	    }
  	    // in case the end is missing a \n
  	    hasFoldableLine = hasFoldableLine || shouldTrackWidth && i - previousLineBreak - 1 > lineWidth && string[previousLineBreak + 1] !== ' ';
  	  }
  	  // Although every style can represent \n without escaping, prefer block styles
  	  // for multiline, since they're more readable and they don't add empty lines.
  	  // Also prefer folding a super-long line.
  	  if (!hasLineBreak && !hasFoldableLine) {
  	    // Strings interpretable as another type have to be quoted;
  	    // e.g. the string 'true' vs. the boolean true.
  	    return plain && !testAmbiguousType(string) ? STYLE_PLAIN : STYLE_SINGLE;
  	  }
  	  // Edge case: block indentation indicator can only have one digit.
  	  if (indentPerLevel > 9 && needIndentIndicator(string)) {
  	    return STYLE_DOUBLE;
  	  }
  	  // At this point we know block styles are valid.
  	  // Prefer literal style unless we want to fold.
  	  return hasFoldableLine ? STYLE_FOLDED : STYLE_LITERAL;
  	}

  	// Note: line breaking/folding is implemented for only the folded style.
  	// NB. We drop the last trailing newline (if any) of a returned block scalar
  	//  since the dumper adds its own newline. This always works:
  	//    • No ending newline => unaffected; already using strip "-" chomping.
  	//    • Ending newline    => removed then restored.
  	//  Importantly, this keeps the "+" chomp indicator from gaining an extra line.
  	function writeScalar(state, string, level, iskey) {
  	  state.dump = function () {
  	    if (string.length === 0) {
  	      return "''";
  	    }
  	    if (!state.noCompatMode && DEPRECATED_BOOLEANS_SYNTAX.indexOf(string) !== -1) {
  	      return "'" + string + "'";
  	    }
  	    var indent = state.indent * Math.max(1, level); // no 0-indent scalars
  	    // As indentation gets deeper, let the width decrease monotonically
  	    // to the lower bound min(state.lineWidth, 40).
  	    // Note that this implies
  	    //  state.lineWidth ≤ 40 + state.indent: width is fixed at the lower bound.
  	    //  state.lineWidth > 40 + state.indent: width decreases until the lower bound.
  	    // This behaves better than a constant minimum width which disallows narrower options,
  	    // or an indent threshold which causes the width to suddenly increase.
  	    var lineWidth = state.lineWidth === -1 ? -1 : Math.max(Math.min(state.lineWidth, 40), state.lineWidth - indent);

  	    // Without knowing if keys are implicit/explicit, assume implicit for safety.
  	    var singleLineOnly = iskey
  	    // No block styles in flow mode.
  	    || state.flowLevel > -1 && level >= state.flowLevel;
  	    function testAmbiguity(string) {
  	      return testImplicitResolving(state, string);
  	    }
  	    switch (chooseScalarStyle(string, singleLineOnly, state.indent, lineWidth, testAmbiguity)) {
  	      case STYLE_PLAIN:
  	        return string;
  	      case STYLE_SINGLE:
  	        return "'" + string.replace(/'/g, "''") + "'";
  	      case STYLE_LITERAL:
  	        return '|' + blockHeader(string, state.indent) + dropEndingNewline(indentString(string, indent));
  	      case STYLE_FOLDED:
  	        return '>' + blockHeader(string, state.indent) + dropEndingNewline(indentString(foldString(string, lineWidth), indent));
  	      case STYLE_DOUBLE:
  	        return '"' + escapeString(string) + '"';
  	      default:
  	        throw new YAMLException('impossible error: invalid scalar style');
  	    }
  	  }();
  	}

  	// Pre-conditions: string is valid for a block scalar, 1 <= indentPerLevel <= 9.
  	function blockHeader(string, indentPerLevel) {
  	  var indentIndicator = needIndentIndicator(string) ? String(indentPerLevel) : '';

  	  // note the special case: the string '\n' counts as a "trailing" empty line.
  	  var clip = string[string.length - 1] === '\n';
  	  var keep = clip && (string[string.length - 2] === '\n' || string === '\n');
  	  var chomp = keep ? '+' : clip ? '' : '-';
  	  return indentIndicator + chomp + '\n';
  	}

  	// (See the note for writeScalar.)
  	function dropEndingNewline(string) {
  	  return string[string.length - 1] === '\n' ? string.slice(0, -1) : string;
  	}

  	// Note: a long line without a suitable break point will exceed the width limit.
  	// Pre-conditions: every char in str isPrintable, str.length > 0, width > 0.
  	function foldString(string, width) {
  	  // In folded style, $k$ consecutive newlines output as $k+1$ newlines—
  	  // unless they're before or after a more-indented line, or at the very
  	  // beginning or end, in which case $k$ maps to $k$.
  	  // Therefore, parse each chunk as newline(s) followed by a content line.
  	  var lineRe = /(\n+)([^\n]*)/g;

  	  // first line (possibly an empty line)
  	  var result = function () {
  	    var nextLF = string.indexOf('\n');
  	    nextLF = nextLF !== -1 ? nextLF : string.length;
  	    lineRe.lastIndex = nextLF;
  	    return foldLine(string.slice(0, nextLF), width);
  	  }();
  	  // If we haven't reached the first content line yet, don't add an extra \n.
  	  var prevMoreIndented = string[0] === '\n' || string[0] === ' ';
  	  var moreIndented;

  	  // rest of the lines
  	  var match;
  	  while (match = lineRe.exec(string)) {
  	    var prefix = match[1],
  	      line = match[2];
  	    moreIndented = line[0] === ' ';
  	    result += prefix + (!prevMoreIndented && !moreIndented && line !== '' ? '\n' : '') + foldLine(line, width);
  	    prevMoreIndented = moreIndented;
  	  }
  	  return result;
  	}

  	// Greedy line breaking.
  	// Picks the longest line under the limit each time,
  	// otherwise settles for the shortest line over the limit.
  	// NB. More-indented lines *cannot* be folded, as that would add an extra \n.
  	function foldLine(line, width) {
  	  if (line === '' || line[0] === ' ') return line;

  	  // Since a more-indented line adds a \n, breaks can't be followed by a space.
  	  var breakRe = / [^ ]/g; // note: the match index will always be <= length-2.
  	  var match;
  	  // start is an inclusive index. end, curr, and next are exclusive.
  	  var start = 0,
  	    end,
  	    curr = 0,
  	    next = 0;
  	  var result = '';

  	  // Invariants: 0 <= start <= length-1.
  	  //   0 <= curr <= next <= max(0, length-2). curr - start <= width.
  	  // Inside the loop:
  	  //   A match implies length >= 2, so curr and next are <= length-2.
  	  while (match = breakRe.exec(line)) {
  	    next = match.index;
  	    // maintain invariant: curr - start <= width
  	    if (next - start > width) {
  	      end = curr > start ? curr : next; // derive end <= length-2
  	      result += '\n' + line.slice(start, end);
  	      // skip the space that was output as \n
  	      start = end + 1; // derive start <= length-1
  	    }
  	    curr = next;
  	  }

  	  // By the invariants, start <= length-1, so there is something left over.
  	  // It is either the whole string or a part starting from non-whitespace.
  	  result += '\n';
  	  // Insert a break if the remainder is too long and there is a break available.
  	  if (line.length - start > width && curr > start) {
  	    result += line.slice(start, curr) + '\n' + line.slice(curr + 1);
  	  } else {
  	    result += line.slice(start);
  	  }
  	  return result.slice(1); // drop extra \n joiner
  	}

  	// Escapes a double-quoted string.
  	function escapeString(string) {
  	  var result = '';
  	  var char, nextChar;
  	  var escapeSeq;
  	  for (var i = 0; i < string.length; i++) {
  	    char = string.charCodeAt(i);
  	    // Check for surrogate pairs (reference Unicode 3.0 section "3.7 Surrogates").
  	    if (char >= 0xD800 && char <= 0xDBFF /* high surrogate */) {
  	      nextChar = string.charCodeAt(i + 1);
  	      if (nextChar >= 0xDC00 && nextChar <= 0xDFFF /* low surrogate */) {
  	        // Combine the surrogate pair and store it escaped.
  	        result += encodeHex((char - 0xD800) * 0x400 + nextChar - 0xDC00 + 0x10000);
  	        // Advance index one extra since we already used that char here.
  	        i++;
  	        continue;
  	      }
  	    }
  	    escapeSeq = ESCAPE_SEQUENCES[char];
  	    result += !escapeSeq && isPrintable(char) ? string[i] : escapeSeq || encodeHex(char);
  	  }
  	  return result;
  	}
  	function writeFlowSequence(state, level, object) {
  	  var _result = '',
  	    _tag = state.tag,
  	    index,
  	    length;
  	  for (index = 0, length = object.length; index < length; index += 1) {
  	    // Write only valid elements.
  	    if (writeNode(state, level, object[index], false, false)) {
  	      if (index !== 0) _result += ',' + (!state.condenseFlow ? ' ' : '');
  	      _result += state.dump;
  	    }
  	  }
  	  state.tag = _tag;
  	  state.dump = '[' + _result + ']';
  	}
  	function writeBlockSequence(state, level, object, compact) {
  	  var _result = '',
  	    _tag = state.tag,
  	    index,
  	    length;
  	  for (index = 0, length = object.length; index < length; index += 1) {
  	    // Write only valid elements.
  	    if (writeNode(state, level + 1, object[index], true, true)) {
  	      if (!compact || index !== 0) {
  	        _result += generateNextLine(state, level);
  	      }
  	      if (state.dump && CHAR_LINE_FEED === state.dump.charCodeAt(0)) {
  	        _result += '-';
  	      } else {
  	        _result += '- ';
  	      }
  	      _result += state.dump;
  	    }
  	  }
  	  state.tag = _tag;
  	  state.dump = _result || '[]'; // Empty sequence if no valid values.
  	}
  	function writeFlowMapping(state, level, object) {
  	  var _result = '',
  	    _tag = state.tag,
  	    objectKeyList = Object.keys(object),
  	    index,
  	    length,
  	    objectKey,
  	    objectValue,
  	    pairBuffer;
  	  for (index = 0, length = objectKeyList.length; index < length; index += 1) {
  	    pairBuffer = '';
  	    if (index !== 0) pairBuffer += ', ';
  	    if (state.condenseFlow) pairBuffer += '"';
  	    objectKey = objectKeyList[index];
  	    objectValue = object[objectKey];
  	    if (!writeNode(state, level, objectKey, false, false)) {
  	      continue; // Skip this pair because of invalid key;
  	    }
  	    if (state.dump.length > 1024) pairBuffer += '? ';
  	    pairBuffer += state.dump + (state.condenseFlow ? '"' : '') + ':' + (state.condenseFlow ? '' : ' ');
  	    if (!writeNode(state, level, objectValue, false, false)) {
  	      continue; // Skip this pair because of invalid value.
  	    }
  	    pairBuffer += state.dump;

  	    // Both key and value are valid.
  	    _result += pairBuffer;
  	  }
  	  state.tag = _tag;
  	  state.dump = '{' + _result + '}';
  	}
  	function writeBlockMapping(state, level, object, compact) {
  	  var _result = '',
  	    _tag = state.tag,
  	    objectKeyList = Object.keys(object),
  	    index,
  	    length,
  	    objectKey,
  	    objectValue,
  	    explicitPair,
  	    pairBuffer;

  	  // Allow sorting keys so that the output file is deterministic
  	  if (state.sortKeys === true) {
  	    // Default sorting
  	    objectKeyList.sort();
  	  } else if (typeof state.sortKeys === 'function') {
  	    // Custom sort function
  	    objectKeyList.sort(state.sortKeys);
  	  } else if (state.sortKeys) {
  	    // Something is wrong
  	    throw new YAMLException('sortKeys must be a boolean or a function');
  	  }
  	  for (index = 0, length = objectKeyList.length; index < length; index += 1) {
  	    pairBuffer = '';
  	    if (!compact || index !== 0) {
  	      pairBuffer += generateNextLine(state, level);
  	    }
  	    objectKey = objectKeyList[index];
  	    objectValue = object[objectKey];
  	    if (!writeNode(state, level + 1, objectKey, true, true, true)) {
  	      continue; // Skip this pair because of invalid key.
  	    }
  	    explicitPair = state.tag !== null && state.tag !== '?' || state.dump && state.dump.length > 1024;
  	    if (explicitPair) {
  	      if (state.dump && CHAR_LINE_FEED === state.dump.charCodeAt(0)) {
  	        pairBuffer += '?';
  	      } else {
  	        pairBuffer += '? ';
  	      }
  	    }
  	    pairBuffer += state.dump;
  	    if (explicitPair) {
  	      pairBuffer += generateNextLine(state, level);
  	    }
  	    if (!writeNode(state, level + 1, objectValue, true, explicitPair)) {
  	      continue; // Skip this pair because of invalid value.
  	    }
  	    if (state.dump && CHAR_LINE_FEED === state.dump.charCodeAt(0)) {
  	      pairBuffer += ':';
  	    } else {
  	      pairBuffer += ': ';
  	    }
  	    pairBuffer += state.dump;

  	    // Both key and value are valid.
  	    _result += pairBuffer;
  	  }
  	  state.tag = _tag;
  	  state.dump = _result || '{}'; // Empty mapping if no valid pairs.
  	}
  	function detectType(state, object, explicit) {
  	  var _result, typeList, index, length, type, style;
  	  typeList = explicit ? state.explicitTypes : state.implicitTypes;
  	  for (index = 0, length = typeList.length; index < length; index += 1) {
  	    type = typeList[index];
  	    if ((type.instanceOf || type.predicate) && (!type.instanceOf || typeof object === 'object' && object instanceof type.instanceOf) && (!type.predicate || type.predicate(object))) {
  	      state.tag = explicit ? type.tag : '?';
  	      if (type.represent) {
  	        style = state.styleMap[type.tag] || type.defaultStyle;
  	        if (_toString.call(type.represent) === '[object Function]') {
  	          _result = type.represent(object, style);
  	        } else if (_hasOwnProperty.call(type.represent, style)) {
  	          _result = type.represent[style](object, style);
  	        } else {
  	          throw new YAMLException('!<' + type.tag + '> tag resolver accepts not "' + style + '" style');
  	        }
  	        state.dump = _result;
  	      }
  	      return true;
  	    }
  	  }
  	  return false;
  	}

  	// Serializes `object` and writes it to global `result`.
  	// Returns true on success, or false on invalid object.
  	//
  	function writeNode(state, level, object, block, compact, iskey) {
  	  state.tag = null;
  	  state.dump = object;
  	  if (!detectType(state, object, false)) {
  	    detectType(state, object, true);
  	  }
  	  var type = _toString.call(state.dump);
  	  if (block) {
  	    block = state.flowLevel < 0 || state.flowLevel > level;
  	  }
  	  var objectOrArray = type === '[object Object]' || type === '[object Array]',
  	    duplicateIndex,
  	    duplicate;
  	  if (objectOrArray) {
  	    duplicateIndex = state.duplicates.indexOf(object);
  	    duplicate = duplicateIndex !== -1;
  	  }
  	  if (state.tag !== null && state.tag !== '?' || duplicate || state.indent !== 2 && level > 0) {
  	    compact = false;
  	  }
  	  if (duplicate && state.usedDuplicates[duplicateIndex]) {
  	    state.dump = '*ref_' + duplicateIndex;
  	  } else {
  	    if (objectOrArray && duplicate && !state.usedDuplicates[duplicateIndex]) {
  	      state.usedDuplicates[duplicateIndex] = true;
  	    }
  	    if (type === '[object Object]') {
  	      if (block && Object.keys(state.dump).length !== 0) {
  	        writeBlockMapping(state, level, state.dump, compact);
  	        if (duplicate) {
  	          state.dump = '&ref_' + duplicateIndex + state.dump;
  	        }
  	      } else {
  	        writeFlowMapping(state, level, state.dump);
  	        if (duplicate) {
  	          state.dump = '&ref_' + duplicateIndex + ' ' + state.dump;
  	        }
  	      }
  	    } else if (type === '[object Array]') {
  	      var arrayLevel = state.noArrayIndent && level > 0 ? level - 1 : level;
  	      if (block && state.dump.length !== 0) {
  	        writeBlockSequence(state, arrayLevel, state.dump, compact);
  	        if (duplicate) {
  	          state.dump = '&ref_' + duplicateIndex + state.dump;
  	        }
  	      } else {
  	        writeFlowSequence(state, arrayLevel, state.dump);
  	        if (duplicate) {
  	          state.dump = '&ref_' + duplicateIndex + ' ' + state.dump;
  	        }
  	      }
  	    } else if (type === '[object String]') {
  	      if (state.tag !== '?') {
  	        writeScalar(state, state.dump, level, iskey);
  	      }
  	    } else {
  	      if (state.skipInvalid) return false;
  	      throw new YAMLException('unacceptable kind of an object to dump ' + type);
  	    }
  	    if (state.tag !== null && state.tag !== '?') {
  	      state.dump = '!<' + state.tag + '> ' + state.dump;
  	    }
  	  }
  	  return true;
  	}
  	function getDuplicateReferences(object, state) {
  	  var objects = [],
  	    duplicatesIndexes = [],
  	    index,
  	    length;
  	  inspectNode(object, objects, duplicatesIndexes);
  	  for (index = 0, length = duplicatesIndexes.length; index < length; index += 1) {
  	    state.duplicates.push(objects[duplicatesIndexes[index]]);
  	  }
  	  state.usedDuplicates = new Array(length);
  	}
  	function inspectNode(object, objects, duplicatesIndexes) {
  	  var objectKeyList, index, length;
  	  if (object !== null && typeof object === 'object') {
  	    index = objects.indexOf(object);
  	    if (index !== -1) {
  	      if (duplicatesIndexes.indexOf(index) === -1) {
  	        duplicatesIndexes.push(index);
  	      }
  	    } else {
  	      objects.push(object);
  	      if (Array.isArray(object)) {
  	        for (index = 0, length = object.length; index < length; index += 1) {
  	          inspectNode(object[index], objects, duplicatesIndexes);
  	        }
  	      } else {
  	        objectKeyList = Object.keys(object);
  	        for (index = 0, length = objectKeyList.length; index < length; index += 1) {
  	          inspectNode(object[objectKeyList[index]], objects, duplicatesIndexes);
  	        }
  	      }
  	    }
  	  }
  	}
  	function dump(input, options) {
  	  options = options || {};
  	  var state = new State(options);
  	  if (!state.noRefs) getDuplicateReferences(input, state);
  	  if (writeNode(state, 0, input, true, true)) return state.dump + '\n';
  	  return '';
  	}
  	function safeDump(input, options) {
  	  return dump(input, common.extend({
  	    schema: DEFAULT_SAFE_SCHEMA
  	  }, options));
  	}
  	dumper.dump = dump;
  	dumper.safeDump = safeDump;
  	return dumper;
  }

  var hasRequiredJsYaml$1;

  function requireJsYaml$1 () {
  	if (hasRequiredJsYaml$1) return jsYaml$1;
  	hasRequiredJsYaml$1 = 1;

  	var loader = requireLoader();
  	var dumper = requireDumper();
  	function deprecated(name) {
  	  return function () {
  	    throw new Error('Function ' + name + ' is deprecated and cannot be used.');
  	  };
  	}
  	jsYaml$1.Type = requireType();
  	jsYaml$1.Schema = requireSchema();
  	jsYaml$1.FAILSAFE_SCHEMA = requireFailsafe();
  	jsYaml$1.JSON_SCHEMA = requireJson();
  	jsYaml$1.CORE_SCHEMA = requireCore();
  	jsYaml$1.DEFAULT_SAFE_SCHEMA = requireDefault_safe();
  	jsYaml$1.DEFAULT_FULL_SCHEMA = requireDefault_full();
  	jsYaml$1.load = loader.load;
  	jsYaml$1.loadAll = loader.loadAll;
  	jsYaml$1.safeLoad = loader.safeLoad;
  	jsYaml$1.safeLoadAll = loader.safeLoadAll;
  	jsYaml$1.dump = dumper.dump;
  	jsYaml$1.safeDump = dumper.safeDump;
  	jsYaml$1.YAMLException = requireException();

  	// Deprecated schema names from JS-YAML 2.0.x
  	jsYaml$1.MINIMAL_SCHEMA = requireFailsafe();
  	jsYaml$1.SAFE_SCHEMA = requireDefault_safe();
  	jsYaml$1.DEFAULT_SCHEMA = requireDefault_full();

  	// Deprecated functions from JS-YAML 1.x.x
  	jsYaml$1.scan = deprecated('scan');
  	jsYaml$1.parse = deprecated('parse');
  	jsYaml$1.compose = deprecated('compose');
  	jsYaml$1.addConstructor = deprecated('addConstructor');
  	return jsYaml$1;
  }

  var jsYaml;
  var hasRequiredJsYaml;

  function requireJsYaml () {
  	if (hasRequiredJsYaml) return jsYaml;
  	hasRequiredJsYaml = 1;

  	var yaml = requireJsYaml$1();
  	jsYaml = yaml;
  	return jsYaml;
  }

  var hasRequiredFrontMatter;

  function requireFrontMatter () {
  	if (hasRequiredFrontMatter) return frontMatter.exports;
  	hasRequiredFrontMatter = 1;
  	var parser = requireJsYaml();
  	var optionalByteOrderMark = '\\ufeff?';
  	var platform = typeof process !== 'undefined' ? process.platform : '';
  	var pattern = '^(' + optionalByteOrderMark + '(= yaml =|---)' + '$([\\s\\S]*?)' + '^(?:\\2|\\.\\.\\.)\\s*' + '$' + (platform === 'win32' ? '\\r?' : '') + '(?:\\n)?)';
  	// NOTE: If this pattern uses the 'g' flag the `regex` variable definition will
  	// need to be moved down into the functions that use it.
  	var regex = new RegExp(pattern, 'm');
  	frontMatter.exports = extractor;
  	frontMatter.exports.test = test;
  	function extractor(string, options) {
  	  string = string || '';
  	  var defaultOptions = {
  	    allowUnsafe: false
  	  };
  	  options = options instanceof Object ? {
  	    ...defaultOptions,
  	    ...options
  	  } : defaultOptions;
  	  options.allowUnsafe = Boolean(options.allowUnsafe);
  	  var lines = string.split(/(\r?\n)/);
  	  if (lines[0] && /= yaml =|---/.test(lines[0])) {
  	    return parse(string, options.allowUnsafe);
  	  } else {
  	    return {
  	      attributes: {},
  	      body: string,
  	      bodyBegin: 1
  	    };
  	  }
  	}
  	function computeLocation(match, body) {
  	  var line = 1;
  	  var pos = body.indexOf('\n');
  	  var offset = match.index + match[0].length;
  	  while (pos !== -1) {
  	    if (pos >= offset) {
  	      return line;
  	    }
  	    line++;
  	    pos = body.indexOf('\n', pos + 1);
  	  }
  	  return line;
  	}
  	function parse(string, allowUnsafe) {
  	  var match = regex.exec(string);
  	  if (!match) {
  	    return {
  	      attributes: {},
  	      body: string,
  	      bodyBegin: 1
  	    };
  	  }
  	  var loader = allowUnsafe ? parser.load : parser.safeLoad;
  	  var yaml = match[match.length - 1].replace(/^\s+|\s+$/g, '');
  	  var attributes = loader(yaml) || {};
  	  var body = string.replace(match[0], '');
  	  var line = computeLocation(match, string);
  	  return {
  	    attributes: attributes,
  	    body: body,
  	    bodyBegin: line,
  	    frontmatter: yaml
  	  };
  	}
  	function test(string) {
  	  string = string || '';
  	  return regex.test(string);
  	}
  	return frontMatter.exports;
  }

  var frontMatterExports = requireFrontMatter();
  var fm = /*@__PURE__*/getDefaultExportFromCjs(frontMatterExports);

  //-------------------------------------------------------------------------------
  // General helper functions for NotePlan plugins
  //-------------------------------------------------------------------------------

  /**
   * Return title of note useful for display, including for
   * - calendar notes based on the filename
   * Note: local copy of this in helpers/paragraph.js to avoid circular dependency.
   * @author @jgclark
   *
   * @param {?CoreNoteFields} n - note to get title for
   * @return {string}
   */
  function displayTitle$1(n) {
    return !n ? '(error)' : n.type === 'Calendar' ? getDateStringFromCalendarFilename(n.filename) ?? '' // earlier: return n.filename.split('.')[0] // without file extension
    : n.title ?? '(error)';
  }

  /**
   * Create internal link from title string (and optional heading string)
   * @author @dwertheimer
   * @param {string} noteTitle - title of the note
   * @param {string | null} heading - heading inside of note (optional)
   * @returns {string} the [[link#heading]]
   * @tests available
   */
  function returnNoteLink(noteTitle, heading = '') {
    return `[[${noteTitle}${heading && heading !== '' ? `#${heading}` : ''}]]`;
  }

  /**
   * Create xcallback link text for running a plugin
   * @author @dwertheimer
   * @param {string} pluginID - ID of the plugin from plugin.json
   * @param {boolean} commandName - the "name" of the command in plugin.json
   * @param {Array<string> | string} args - either array of arguments to be sent, or JSON string representation
   * @returns {string} the x-callback-url URL string (not the pretty part)
   * @tests available
   */
  function createRunPluginCallbackUrl(pluginID, commandName, args) {
    let xcb = `noteplan://x-callback-url/runPlugin?pluginID=${pluginID}&command=${encodeURIComponent(commandName)}`;
    if (!args || args === undefined) ; else if (typeof args !== 'string') {
      if (args?.length) {
        args.forEach((arg, i) => {
          xcb += `&arg${i}=${encodeURIComponent(arg)}`;
        });
      }
    } else {
      xcb += `&arg0=${encodeURIComponent(args)}`;
    }
    return xcb.replace(/\(/g, '%28').replace(/\)/g, '%29');
  }

  /**
   * Create a pretty/short link hiding an xcallback link text for running a plugin
   * e.g. [linkText](x-callback-url)
   * @param {string} linkText - the text to display for the link
   * @param {string} pluginID - the plugin's ID
   * @param {boolean} command - the "name" field of the plugin command to run
   * @param {Array<string> | string} args - arguments to pass (optional)
   * @returns {string} arguments as strings (or single argument string) to send through to plugin
   * @tests available
   */
  function createPrettyRunPluginLink(linkText, pluginID, command, args = '') {
    return `[${linkText}](${createRunPluginCallbackUrl(pluginID, command, args)})`;
  }

  /* eslint-disable max-len */ //---------------------------------------------------------------------
  // Regex definitions for NotePlan and its plugins
  // Last updated 2024-11-09 by @jgclark
  //---------------------------------------------------------------------
  //
  // This file holds definitions that don't live in more specific helper files, and also lists other files with useful regexes.
  //
  // Note: these are JavaScript RegExp objects.
  // They are then difficult to use in composition to more complex regexes: to do that start with simpler strings.
  //
  // The main ways to use them are:
  // - string.match(RE_NAME) -> array of matches
  // - RE_NAME.test(string) -> boolean
  //
  // Note: some have 'g' (global) or 'i' (case insensitive) flags set
  //---------------------------------------------------------------------
  // Times, Dates
  const RE_ARROW_DATES_G = />(today|tomorrow|yesterday|(([0-9]{4})(-((0[1-9]|1[0-2])(-(0[1-9]|1[0-9]|2[0-9]|3[0-1]))?|Q[1-4]|W0[1-9]|W[1-4]\d|W5[0-3]))?))</g; // as above but with a closing '<'

  // URLs and Links
  const RE_MARKDOWN_LINKS_CAPTURE_G = /\[([^\]]+)\]\(([^\)]+)\)/g;

  /**
   * Key FrontMatter functions:
   * getFrontMatterAttributes() - get the front matter attributes from a note
   * setFrontMatterVars() - set/update the front matter attributes for a note (will create frontmatter if necessary)
   * noteHasFrontMatter() - test whether a Test whether a Note contains front matter
   * ensureFrontMatter() - ensure that a note has front matter (will create frontmatter if necessary)
   * addTrigger() - add a trigger to the front matter (will create frontmatter if necessary)
   */
  const pluginJson$3 = 'helpers/NPFrontMatter.js';

  // Note: update these for each new trigger that gets added

  const TRIGGER_LIST = ['onEditorWillSave', 'onOpen'];

  /**
   * Frontmatter cannot have colons in the content (specifically ": " or ending in colon or values starting in @ or #), so we need to wrap that in quotes
   * If a string is wrapped in double quotes and contains additional double quotes, convert the internal quotes to single quotes.
   * This often happens when people include double quotes in template tags in their frontmatter
   * @param {string} text
   * @returns {string} quotedText (if required)
   */
  function quoteText(text) {
    const needsQuoting = text.includes(': ') || /:$/.test(text) || /^#\S/.test(text) || /^@/.test(text) || text === '' || RE_MARKDOWN_LINKS_CAPTURE_G.test(text) || text.includes('>');
    const isWrappedInQuotes = /^".*"$/.test(text); // Check if already wrapped in quotes

    // Handle the case where text is wrapped in double quotes but contains additional double quotes inside
    if (isWrappedInQuotes) {
      // Replace internal double quotes with escaped double quotes
      return text.replace(/(^")|("$)/g, '') // Remove outer quotes temporarily
      .replace(/"/g, '\\"') // Escape internal double quotes
      .replace(/^/, '"') // Re-add starting double quote
      .replace(/$/, '"'); // Re-add ending double quote
    }

    // Always escape internal double quotes and wrap in quotes if needed
    if (needsQuoting || text.includes('"')) {
      return `"${text.replace(/"/g, '\\"')}"`; // Escape internal double quotes and wrap in quotes
    }

    // No need to quote
    return text;
  }

  /**
   * Test whether a string contains front matter using the front-matter library which has a bug/limitation
   * Note: underlying library doesn't actually check whether the YAML comes at the start of the string. @jgclark has raised an issue to fix that.
   * Will allow nonstandard YAML (e.g. contain colons, value starts with @) by sanitizing it first
   * @param {string} text - the text to test (typically the content of a note -- note.content)
   * @returns {boolean} true if it has front matter
   */
  const hasFrontMatter = text => text.split('\n', 1)[0] === '---' && fm.test(_sanitizeFrontmatterText(text, true));

  /**
   * Test whether a Note contains front matter
   * Note: the underlying library doesn't actually check whether the YAML comes at the start of the string.
   * So @jgclark has added an (imperfect, simple) test to see if it comes at the start, until such a time as the library is updated.
   * @param {CoreNoteFields} note - the note to test
   * @returns {boolean} true if the note has front matter
   */
  function noteHasFrontMatter(note) {
    logDebug('noteHasFrontMatter', `has ${String(note.paragraphs?.length)} lines`);
    if (note.paragraphs.length > 0) logDebug('noteHasFrontMatter', `- para 0 = ${note.paragraphs?.[0].type} "${note.paragraphs?.[0].content}"`);
    if (!note || !note.paragraphs || note.paragraphs?.length < 2) return false;
    return note && hasFrontMatter(note.content || '') && (note.paragraphs[0].type === 'separator' || note.paragraphs[0].content === '---');
  }
  /**
   * get the front matter attributes from a note
   * @param {TNote} note
   * @returns object of attributes or false if the note has no front matter
   */
  const getFrontMatterAttributes = note => hasFrontMatter(note?.content || '') ? getAttributes(note.content) : false;

  /**
   * Get the paragraphs that include the front matter (optionally with the separators)
   * This is a helper function for removeFrontMatter and probably won't need to be called directly
   * @author @dwertheimer
   * @param {CoreNoteFields} note - the note
   * @param {boolean} includeSeparators - whether to include the separator lines (---) in the returned array
   * @returns {Array<TParagraph>} just the paragraphs in the front matter (or false if no frontmatter)
   */
  const getFrontMatterParagraphs = (note, includeSeparators = false) => {
    try {
      const paras = note?.paragraphs || [];
      if (!paras.length || paras[0].content !== '---') return false;
      const startAt = includeSeparators ? 0 : 1;
      for (let i = 1; i < paras.length; i++) {
        const para = paras[i];
        if (para.content === '---') return paras.slice(startAt, includeSeparators ? i + 1 : i);
      }
      return false;
    } catch (err) {
      logError('NPFrontMatter/getFrontMatterParagraphs()', JSP(err));
      return false;
    }
  };

  /**
   * Remove the front matter from a note (optionally including the separators)
   * Note: this is a helper function called by setFrontMatterVars and probably won't need to be called directly
   * @author @dwertheimer
   * @param {CoreNoteFields} note - the note
   * @param {boolean} removeSeparators? - whether to include the separator lines (---) in the deletion. Default: false.
   * @returns {boolean} - whether the front matter was removed or not
   */
  function removeFrontMatter(note, removeSeparators = false) {
    try {
      const fmParas = getFrontMatterParagraphs(note, removeSeparators);
      // clo(fmParas, 'fmParas')
      // clo(note.paragraphs, 'note.paragraphs')
      if (!fmParas) return false;
      const fm = getFrontMatterAttributes(note || '');
      note.removeParagraphs(fmParas);
      if (removeSeparators && fm && fm.title) note.prependParagraph(`# ${fm.title}`, 'text');
      return true;
    } catch (err) {
      logError('NPFrontMatter/removeFrontMatter()', JSP(err));
      return false;
    }
  }

  /**
   * Recursive helper function used to write multi-line-indented frontmatter keys/values
   * @param {any} obj
   * @param {string} indent - level for recursive indent
   * @returns
   */
  function _objectToYaml(obj, indent = ' ') {
    let output = '';
    for (const prop in obj) {
      output += `\n${indent}${prop}:`;
      if (Array.isArray(obj[prop])) {
        obj[prop].forEach(el => {
          output += `\n${indent} - ${el}`;
        });
      } else if (typeof obj[prop] === 'object' && obj[prop] !== null && Object.keys(obj[prop]).length) {
        output += _objectToYaml(obj[prop], `${indent} `);
      } else {
        output += ` ${obj[prop]}`;
      }
    }
    return output;
  }

  /**
   * Write the frontmatter vars to a note which already has frontmatter
   * Note: this is a helper function called by setFrontMatterVars and probably won't need to be called directly
   * You should use setFrontMatterVars instead
   * will add fields for whatever attributes you send in the second argument (could be duplicates)
   * so delete the frontmatter first (using removeFrontMatter()) if you want to add/remove/change fields
   * @param {CoreNoteFields} note
   * @param {Object} attributes - key/value pairs for frontmatter values
   * @param {boolean?} alsoEnsureTitle - ensure that the frontmatter has a title (and set it to the note title if not). Default: true.
   * @param {boolean?} quoteNonStandardYaml - quote any values that are not standard YAML (e.g. contain colons, value starts with @). Default: false.
   * @returns {boolean} was frontmatter written OK?
   * @author @dwertheimer
   */
  function writeFrontMatter(note, attributes, alsoEnsureTitle = true, quoteNonStandardYaml = false) {
    if (!noteHasFrontMatter(note)) {
      logError(pluginJson$3, `writeFrontMatter: no frontmatter already found in note, so stopping.`);
      return false;
    }
    if (ensureFrontmatter(note, alsoEnsureTitle)) {
      const outputArr = [];
      Object.keys(attributes).forEach(key => {
        const value = attributes[key];
        if (value !== null) {
          if (typeof value === 'string') {
            outputArr.push(quoteNonStandardYaml ? `${key}: ${quoteText(value)}` : `${key}: ${value}`);
          } else {
            if (typeof value === 'object') {
              logDebug(pluginJson$3, `writeFrontMatter: value for key '${key}' is an object (e.g. multi-level list). Converting to multi-line string.`);
              const yamlString = _objectToYaml(value);
              outputArr.push(`${key}:${yamlString}`);
            }
          }
        }
      });
      const output = outputArr.join('\n');
      logDebug(pluginJson$3, `writeFrontMatter: writing frontmatter to note '${displayTitle$1(note)}':\n"${output}"`);
      note.insertParagraph(output, 1, 'text');
      return true;
    } else {
      logError(pluginJson$3, `writeFrontMatter: Could not write frontmatter for note "${note.filename || ''}" for some reason.`);
    }
    return false;
  }
  const hasTemplateTagsInFM = fmText => fmText.includes('<%');

  /**
   * Set/update the front matter attributes for a note.
   * Whatever key:value pairs you pass in will be set in the front matter.
   * If the key already exists, it will be set to the new value you passed;
   * If the key does not exist, it will be added.
   * All existing fields you do not explicitly mention in varObj will keep their previous values (including note title).
   * If the value of a key is set to null, the key will be removed from the front matter.
   * @param {CoreNoteFields} note
   * @param {{[string]:string}} varObj - an object with the key:value pairs to set in the front matter (all strings). If the value of a key is set to null, the key will be removed from the front matter.
   * @returns {boolean} - whether the front matter was set or not
   * @author @dwertheimer
   */
  function setFrontMatterVars(note, varObj) {
    try {
      const title = varObj.title || null;
      clo(varObj, `Starting for note ${note.filename} with varObj:`);
      logDebug(`setFrontMatterVars`, `- BEFORE ensureFM: hasFrontmatter:${String(noteHasFrontMatter(note) || '')} note has ${note.paragraphs.length} lines`);
      const hasFM = ensureFrontmatter(note, true, title);
      logDebug('note.paragraphs', `- AFTER ensureFM has ${note.paragraphs.length} lines, that starts:`);
      // console.log(note.paragraphs.slice(0, 7).map(p => p.content).join('\n'))
      if (!hasFM) {
        throw new Error(`setFrontMatterVars: Could not add front matter to note which has no title. Note should have a title, or you should pass in a title in the varObj.`);
      }
      if (hasFrontMatter(note.content || '')) {
        const existingAttributes = getAttributes(note.content);
        const changedAttributes = {
          ...existingAttributes
        };
        Object.keys(varObj).forEach(key => {
          if (varObj[key] === null && existingAttributes.hasOwnProperty(key)) {
            delete changedAttributes[key];
          } else {
            changedAttributes[key] = String(varObj[key]);
          }
        });
        removeFrontMatter(note);
        writeFrontMatter(note, changedAttributes);
        logDebug('setFrontMatterVars', `- ENDING with ${note.paragraphs.length} lines, that starts:`);
        // console.log(note.paragraphs.slice(0, 7).map(p => p.content).join('\n'))
      } else {
        logError('setFrontMatterVars', `- could not change frontmatter for note "${note.filename || ''}" because it has no frontmatter.`);
      }
      return true;
    } catch (error) {
      logError('NPFrontMatter/setFrontMatterVars()', JSP(error));
      return false;
    }
  }

  // /**
  //  * TODO: Decide whether to keep this or the earlier rFMF one
  //  * @param {CoreNoteFields} note
  //  * @param {{[string]:string}} varObj - an object with the key:value pairs to unset (i.e. remove) in the front matter (all strings).
  //  * @returns {boolean} - whether the front matter was unset or not
  //  */
  // export function unsetFrontMatterFields(note: CoreNoteFields, fields: Array<string>): boolean {
  //   const attributes: { [string]: string | null } = {}
  //   // make object with a key for each field to unset
  //   for (let field of fields) {
  //     attributes[field] = null
  //   }
  //   const result = setFrontMatterVars(note, attributes)
  //   return result
  // }

  /**
   * Ensure that a note has front matter (and optionally has a title you specify).
   * WARNING: Failing for @jgclark on calendar notes without existing FM.
   * If the note already has front matter, returns true.
   * If the note does not have front matter, adds it and returns true.
   * If optional title is given, it overrides any existing title in the note for the frontmatter title.
   * @author @dwertheimer based on @jgclark's convertNoteToFrontmatter code
   * @param {TNote} note
   * @param {boolean?} alsoEnsureTitle - if true then fail if a title can't be set. Default: true. For calendar notes this wants to be false.
   * @param {string?} title - optional override text that will be added to the frontmatter as the note title (regardless of whether it already had for a title)
   * @returns {boolean} true if front matter existed or was added, false if failed for some reason
   * @author @dwertheimer
   */
  function ensureFrontmatter(note, alsoEnsureTitle = true, title) {
    try {
      let retVal = false;
      let fm = '';
      if (note == null) {
        // no note - return false
        throw new Error(`No note found. Stopping conversion.`);
      } else if (hasFrontMatter(note.content || '')) {
        // already has frontmatter
        const attr = getAttributes(note.content);
        clo(attr, `ensureFrontmatter: Note '${displayTitle$1(note)}' has frontmatter already: attr =`);
        if (!attr.title && title) {
          logDebug('ensureFrontmatter', `Note '${displayTitle$1(note)}' already has frontmatter but no title. Adding title.`);
          if (note.content) note.content = note.content.replace('---', `---\ntitle: ${title}\n`);
        } else if (title && attr.title !== title) {
          logDebug('ensureFrontmatter', `Note '${displayTitle$1(note)}' already has frontmatter but title is wrong. Updating title.`);
          if (note.content) note.content = note.content.replace(`title: ${attr.title}`, `title: ${title}`);
        }
        retVal = true;
      } else {
        // need to add frontmatter
        let newTitle;
        if (note.type === 'Notes' && alsoEnsureTitle) {
          logDebug('ensureFrontmatter', `'${note.filename}' had no frontmatter or title line, so will now make one:`);
          const firstLine = note.paragraphs.length ? note.paragraphs[0] : {};
          const titleFromFirstLine = firstLine.type === 'title' && firstLine.headingLevel === 1 ? firstLine.content : '';

          // Make title from parameter or note's existing H1 title or note.title respectively
          newTitle = (title || titleFromFirstLine || note.title || '').replace(/`/g, ''); // cover Calendar notes where title is not in the note
          logDebug('ensureFrontmatter', `- newTitle='${newTitle ?? ''}'`);
          if (newTitle === '') {
            logError('ensureFrontmatter', `Cannot find title for '${note.filename}'. Stopping conversion.`);
          }
          if (titleFromFirstLine) note.removeParagraph(note.paragraphs[0]); // remove the heading line now that we set it to fm title
          fm = `---\ntitle: ${quoteText(newTitle)}\n---\n`;
        } else {
          logDebug('ensureFrontmatter', `- just adding empty frontmatter to this calendar note`);
          fm = `---\n---\n`;
        }
        // const newContent = `${front}${note?.content || ''}`
        // logDebug('ensureFrontmatter', `newContent = ${newContent}`)
        // note.content = '' // in reality, we can just set this to newContent, but for the mocks to work, we need to do it the long way
        logDebug('ensureFrontmatter', `front to add: ${fm}`);
        // FIXME: this doesn't do anything for @jgclark
        note.insertParagraph(fm, 0, 'text');
        retVal = true;
        logDebug('ensureFrontmatter', `-> Note '${displayTitle$1(note)}' converted to use frontmatter.`);
      }
      return retVal;
    } catch (error) {
      logError('NPFrontMatter/ensureFrontmattter()', JSP(error));
      return false;
    }
  }

  /**
   * Works out which is the last line of the frontmatter, returning the line index number of the closing separator, or 0 if no frontmatter found.
   * @author @jgclark
   * @param {TNote} note - the note to assess
   * @returns {number | false} - the line index number of the closing separator, or false if no frontmatter found
   */
  function endOfFrontmatterLineIndex(note) {
    try {
      const paras = note.paragraphs;
      const lineCount = paras.length;
      logDebug(`paragraph/endOfFrontmatterLineIndex`, `total paragraphs in note (lineCount) = ${lineCount}`);
      // Can't have frontmatter as less than 2 separators
      if (paras.filter(p => p.type === 'separator').length < 2) {
        return false;
      }
      // No frontmatter if first line isn't ---
      if (note.paragraphs[0].type !== 'separator') {
        return false;
      }
      // No frontmatter if less than 3 lines
      if (note.paragraphs.length < 3) {
        return false;
      }
      // Look for second --- line
      let lineIndex = 1;
      while (lineIndex < lineCount) {
        const p = paras[lineIndex];
        if (p.type === 'separator') {
          logDebug(`paragraph/endOfFrontmatterLineIndex`, `-> line ${lineIndex} of ${lineCount}`);
          return lineIndex;
        }
        lineIndex++;
      }
      // Shouldn't get here ...
      return false;
    } catch (err) {
      logError('paragraph/findEndOfActivePartOfNote', err.message);
      return NaN; // for completeness
    }
  }

  // Triggers in frontmatter: https://help.noteplan.co/article/173-plugin-note-triggers
  // triggers: onEditorWillSave => np.test.onEditorWillSave
  // triggers: onEditorWillSave => plugin.id.commandName, onEditorWillSave => plugin.id2.commandName2

  /**
   * (helper function) Get the triggers from the frontmatter of a note and separate them by trigger, id, and command name
   * @author @dwertheimer
   * @param {Array<string>} triggersArray - the array of triggers from the frontmatter (e.g. ['onEditorWillSave => np.test.onEditorWillSave', 'onEditorWillSave => plugin.id.commandName', 'onEditorWillSave => plugin.id2.commandName2'])
   * @returns {Object.<TriggerTypes, Array<{pluginID: string, commandName: string}>>}
   */
  function getTriggersByCommand(triggersArray) {
    const triggers = {};
    TRIGGER_LIST.forEach(triggerName => triggers[triggerName] = []);
    triggersArray.forEach(trigger => {
      const [triggerName, command] = trigger.split('=>').map(s => s.trim());
      const commandSplit = command.split('.').map(s => s.trim());
      const commandName = commandSplit[commandSplit.length - 1];
      const pluginID = commandSplit.slice(0, commandSplit.length - 1).join('.');
      if (triggerName && pluginID && commandName) {
        triggers[triggerName].push({
          pluginID,
          commandName
        });
      }
    });
    Object.keys(triggers).forEach(triggerName => {
      if (triggers[triggerName].length === 0) delete triggers[triggerName];
    });
    return triggers;
  }

  /**
   * (helper function) Format list of trigger(s) command(s) as a single string to add to frontmatter
   * @author @dwertheimer
   * @param {*} triggerObj
   * @returns {string} - the formatted string
   */
  function formatTriggerString(triggerObj) {
    try {
      clo(triggerObj, `formatTriggerString() starting with triggerObj =`);
      let trigArray = [];
      TRIGGER_LIST.forEach(triggerName => {
        // logDebug('formatTriggerString', triggerName)
        if (triggerObj[triggerName] && triggerObj[triggerName].length) {
          trigArray = trigArray.concat(triggerObj[triggerName].map(trigger => {
            return `${triggerName} => ${trigger.pluginID}.${trigger.commandName}`;
          }));
        }
        logDebug('formatTriggerString', `  - ${trigArray.join(', ')}`);
      });
      logDebug('formatTriggerString', `-> ${trigArray.join(', ')}`);
      return trigArray.join(', ');
    } catch (error) {
      logError('NPFrontMatter/formatTriggerString()', JSP(error));
      return '';
    }
  }

  /**
   * Add a trigger to the frontmatter of a note (will create frontmatter if doesn't exist). Will append onto any existing list of trigger(s).
   * @author @dwertheimer
   * @param {CoreNoteFields} note
   * @param {string} trigger 1 from the TriggerTypes
   * @param {string} pluginID - the ID of the plugin
   * @param {string} commandName - the name (NOT THE jsFunction) of the command to run
   * @returns {boolean} - true if the trigger already existed or was added succesfully
   */
  function addTrigger(note, trigger, pluginID, commandName) {
    try {
      if (!TRIGGER_LIST.includes(trigger)) {
        throw new Error(`'${trigger}' is not in the TRIGGER_LIST. Stopping.`);
      }
      if (ensureFrontmatter(note) === false) {
        throw new Error(`Failed to convert note '${displayTitle$1(note)}' to have frontmatter. Stopping.`);
      }
      logDebug(pluginJson$3, `addTrigger() starting to add the ${trigger} / ${pluginID} /  ${commandName} to FM:`);
      const attributes = getFrontMatterAttributes(note);
      // clo(attributes, `addTrigger() attributes =`)
      const triggersArray = attributes ? attributes.triggers?.split(',') || [] : [];
      const triggersObj = getTriggersByCommand(triggersArray);
      if (triggersObj[trigger]) {
        const commandExists = triggersObj[trigger].find(t => t.pluginID === pluginID && t.commandName === commandName);
        if (commandExists) {
          logDebug(pluginJson$3, `addTrigger: Trigger already exists in frontmatter for ${trigger}=>${pluginID}.${commandName} in note '${displayTitle$1(note)}'. No need to add it.`);
          return true;
        }
      } else {
        triggersObj[trigger] = [];
      }
      triggersObj[trigger].push({
        pluginID,
        commandName
      });
      // clo(triggersObj, `addTrigger() triggersObj =`)
      const triggerFrontMatter = {
        triggers: formatTriggerString(triggersObj)
      };
      clo(triggerFrontMatter, `addTrigger() triggerFrontMatter setting frontmatter for ${displayTitle$1(note)}`);
      return setFrontMatterVars(note, triggerFrontMatter);
    } catch (error) {
      logError('NPFrontMatter/addTrigger()', JSP(error));
      return false;
    }
  }

  /**
   * [Internal function used by frontmatter sanitization functions -- should not be used directly]
   * For pre-processing illegal characters in frontmatter, we must first get the frontmatter text the long way (instead of calling fm() which will error out)
   * Gets the text that represents frontmatter. The first line must be a "---" separator, and include everything until another "---" separator is reached. Include the separators in the output.
   * Returns empty string if no frontmatter found
   * @param {*} text
   */
  function _getFMText(text) {
    const lines = text.split('\n');
    if (lines.length >= 2 && lines[0] === '---') {
      let fmText = '';
      let i = 0;
      while (i < lines.length) {
        fmText += `${lines[i]}\n`;
        i++;
        if (lines[i] === '---') {
          return `${fmText}---\n`;
        }
      }
    }
    return '';
  }

  /**
   * [Internal function used by frontmatter sanitization functions -- should not be used directly]
   * Fix the frontmatter text by quoting values that need it
   * Should always be run after _getFMText() because assumes there is frontmatter
   * @param {string} fmText - the text of the frontmatter (including the separators but not the note text)
   * @returns {string} - the fixed frontmatter text
   * @example fixFrontmatter('---\nfoo: bar:\nbaz: @qux\n---\n') => '---\nfoo: "bar:"\nbaz: "@qux"\n---\n'
   */
  function _fixFrontmatter(fmText) {
    const varLines = fmText.trim().split('\n').slice(1, -1);
    let output = '',
      isMultiline = false;
    varLines.forEach(line => {
      if (line.trim() === '') {
        output += '\n';
        return;
      }
      if (isMultiline && !line.trim().startsWith('-')) {
        isMultiline = false;
      }
      if (!isMultiline && line.trim().endsWith(':') && line.split(':').length === 2) {
        isMultiline = true;
        output += `${line}\n`;
        return;
      }
      if (isMultiline) {
        output += `${line.trimEnd()}\n`;
        return;
      }
      const [varName, ...varValue] = line.split(':');
      const value = varValue.join(':').trim();
      const fixedValue = quoteText(value);
      output += `${varName}: ${fixedValue}\n`;
    });
    return `---\n${output}---\n`;
  }

  /**
   * [Internal function used by frontmatter sanitization functions -- should not be used directly]
   * Typically is run after a parse error from the frontmatter library
   * Sanitizes the frontmatter text by quoting illegal values that need quoting (e.g. colons, strings that start with: @, #)
   * Returns sanitized text as a string
   * @param {string} originalText
   * @param {boolean} removeTemplateTagsInFM - if true, remove any lines from the template frontmatter that contain template tags themselves (default: false)
   * @returns
   */
  function _sanitizeFrontmatterText(originalText, removeTemplateTagsInFM = false) {
    const unfilteredFmText = _getFMText(originalText);
    const hasTags = hasTemplateTagsInFM(unfilteredFmText);
    // remove any lines in fmText which contain <%
    const fmTextWithoutTags = removeTemplateTagsInFM && hasTags ? unfilteredFmText.split('\n').filter(line => !line.includes('<%')).join('\n') : unfilteredFmText;
    if (fmTextWithoutTags === '') return originalText;
    // needs to return full note after sanitizing frontmatter
    // get the text between the separators
    const fixedText = _fixFrontmatter(fmTextWithoutTags);
    return originalText.replace(unfilteredFmText, fixedText);
  }
  /**
   * Get an object representing the document with or without frontmatter
   * Do pre-processing to ensure that the most obvious user-entered illegal character sequences in frontmatter are avoided
   * @param {string} noteText  - full text of note (perhaps starting with frontmatter)
   * @param {boolean} removeTemplateTagsInFM - if true, remove any lines from the template frontmatter that contain template tags themselves (default: false)
   * @returns {Object} - the frontmatter object (or empty object if none)
   */
  function getSanitizedFmParts(noteText, removeTemplateTagsInFM = false) {
    let fmData = {
      attributes: {},
      body: noteText,
      frontmatter: ''
    }; //default
    // we need to pre-process the text to sanitize it instead of running fm because we need to
    // preserve #hashtags, @mentions etc. and fm will blank those lines  out as comments
    const sanitizedText = _sanitizeFrontmatterText(noteText || '', removeTemplateTagsInFM);
    try {
      fmData = fm(sanitizedText, {
        allowUnsafe: true
      });
    } catch (error) {
      // Expected to fail in certain circumstances due to limitations in fm library
      logWarn(`Frontmatter getAttributes error. fm module COULD NOT SANITIZE CONTENT: "${error.message}".\nSuggestion: Check for items in frontmatter that need to be quoted. If fm values are surrounded by double quotes, makes sure they do not contain template tags that also contain double quotes. Template tags in frontmatter will always be quoted. And so make sure your template tags in frontmatter use single quotes, not double quotes in this note:\n"${noteText}\n\nSanitizedText:\n${sanitizedText}"`);
      // logError(`Frontmatter getAttributes error. COULD NOT SANITIZE CONTENT: "${error.message}". Returning empty values for this note: "${JSON.stringify(noteText)}"`)
    }
    return fmData;
  }

  /**
   * Get the frontmatter attributes from a note, sanitizing the frontmatter text by quoting illegal values that need quoting (e.g. colons, strings that start with: @, #)
   * Note that templates may include templates (<%) in their frontmatter, which will stop the parser, so if you are trying to getAttributes of a note that could have templates in
   * the frontmatter, set the second param to true to strip those tags out
   * (moved from '@templating/support/modules/FrontmatterModule')
   * // import { getAttributes } from '@templating/support/modules/FrontmatterModule'
   * @param {string} templateData
   * @param {boolean} removeTemplateTagsInFM - if true, remove any lines from the template frontmatter that contain template tags themselves (default: false)
   * @returns {Object} - the frontmatter object (or empty object if none)
   */
  function getAttributes(templateData = '', removeTemplateTagsInFM = false) {
    const fmData = getSanitizedFmParts(templateData, removeTemplateTagsInFM);
    Object.keys(fmData?.attributes).forEach(key => {
      fmData.attributes[key] || typeof fmData.attributes[key] === 'boolean' ? fmData.attributes[key] : fmData.attributes[key] = '';
    });
    return fmData && fmData?.attributes ? fmData.attributes : {};
  }

  //-----------------------------------------------------------------------------
  // Search helpers
  // @jgclark
  //-----------------------------------------------------------------------------
  // import { trimString } from '@helpers/dataManipulation'

  /**
   * Perform string exact match, ignoring case
   * @author @jgclark
   * @param {string} searchTerm
   * @param {string} textToSearch
   * @returns {boolean}
   * @tests available in jest file
   */
  function caseInsensitiveMatch(searchTerm, textToSearch) {
    try {
      const re = new RegExp(`^${searchTerm}$`, "i"); // = case insensitive match
      return re.test(textToSearch);
    } catch (error) {
      logError('search/caseInsensitiveMatch', `Error matching '${searchTerm}' to '${textToSearch}': ${error.message}`);
      return false;
    }
  }

  /**
   * Returns true if A is a strict subset of B, starting from the beginning.
   * i.e. won't match if A===B
   * @author @jgclark
   * @param {string} searchTerm
   * @param {string} textToSearch
   * @param {boolean} strictSubset? (default: true)
   * @returns {boolean} matches?
   * @tests available in jest file
   */
  function caseInsensitiveStartsWith(searchTerm, textToSearch, strictSubset = true) {
    try {
      const re = strictSubset ? new RegExp(`^${searchTerm}.+`, "i") // = case insensitive 'starts with' regex
      : new RegExp(`^${searchTerm}`, "i"); // = case insensitive 'starts with' regex
      return re.test(textToSearch);
    } catch (error) {
      logError('search/caseInsensitiveStartsWith', `Error matching '${searchTerm}' to '${textToSearch}': ${error.message}`);
      return false;
    }
  }

  //-----------------------------------------------------------------------------
  // Paragraph and block-level helpers functions
  //-----------------------------------------------------------------------------

  /**
   * Return title of note useful for display, including for
   * - daily calendar notes (the YYYYMMDD)
   * - weekly notes (the YYYY-Wnn)
   * Note: this is a local copy of the main helpers/general.js to avoid a circular dependency
   * @author @jgclark
   *
   * @param {?TNote} n - note to get title for
   * @return {string}
   */
  function displayTitle(n) {
    return !n ? '(error)' : n.type === 'Calendar' ? getDateStringFromCalendarFilename(n.filename) ?? '' // earlier: return n.filename.split('.')[0] // without file extension
    : n.title ?? '(error)';
  }

  /**
   * Works out where the first 'active' line of the note is, following the first paragraph of type 'title', or frontmatter (if present).
   * Additionally, it skips past any front-matter like section in a project note, as used by the Reviews plugin before frontmatter was supported.
   * This is indicated by a #hashtag starting the next line. If there is, run on to next heading or blank line.
   * A task/checklist item marks the end of the frontmatter-like section.
   * Note: given this is a precursor to writing to a note, it first checks if the note is completely empty (0 lines). If so, a first 'empty' line is added, to avoid edge cases in calling code.
   * Note: Really should live in helpers/NPParagraph.js, but that introduces a circular dependency, so leaving here.
   * @author @jgclark
   * @tests in jest file
   * @param {TNote} note - the note to assess
   * @param {boolean} allowPreamble?
   * @returns {number} - the line index number
   */
  function findStartOfActivePartOfNote(note, allowPreamble = false) {
    try {
      let startOfActive = NaN;
      let paras = note.paragraphs;
      // First check there's actually anything at all! If note, add a first empty paragraph
      if (paras.length === 0) {
        logInfo(`paragraph/findStartOfActivePartOfNote`, `Note was empty; adding a blank line to make writing to the note work`);
        note.appendParagraph('', 'empty');
        return 0;
      }
      const endOfFMIndex = endOfFrontmatterLineIndex(note);
      if (endOfFMIndex === 0) {
        // No frontmatter found
        if (paras[0].type === 'title' && paras[0].headingLevel === 1) {
          // logDebug(`paragraph/findStartOfActivePartOfNote`, `No frontmatter, but H1 title found -> next line`)
          startOfActive = 1;
        } else {
          // logDebug(`paragraph/findStartOfActivePartOfNote`, `No frontmatter or H1 title found -> first line`)
          startOfActive = 0;
        }
      } else {
        logDebug(`paragraph/findStartOfActivePartOfNote`, `Frontmatter found, finishing at line ${String(endOfFMIndex)}, so looking at line after it`);
        startOfActive = endOfFMIndex + 1;
      }
      // If there is no line after title or FM, add a blank line to use (NB: length = line index + 1)
      if (paras.length === startOfActive) {
        logDebug('paragraph/findStartOfActivePartOfNote', `Added a blank line after title/frontmatter of '${displayTitle(note)}'`);
        note.appendParagraph('', 'empty');
        paras = note.paragraphs;
        startOfActive = paras.length;
      }

      // logDebug('paragraph/findStartOfActivePartOfNote', `- startOfActive so far = ${String(startOfActive)}. allowPreamble: ${allowPreamble ? 'true' : 'false'}`)
      // Additionally, skip past any front-matter-like section in a project note,
      // if either there's a #hashtag starting the next line,
      // or 'allowPreamble' is true.
      // If there is, run on to next heading or blank line (if found) otherwise, just the next line. Finding a separator or any YouTutype of task or checklist also stops the search.
      if (allowPreamble || paras[startOfActive].type === 'text' && paras[startOfActive].content.match(/^#\w/)) {
        // logDebug('paragraph/findStartOfActivePartOfNote', `- We want to allow preamble, or found a metadata line.`)
        // startOfActive += 1
        for (let i = startOfActive; i < paras.length; i++) {
          const p = paras[i];
          if (['open', 'done', 'scheduled', 'cancelled', 'checklist', 'checklistDone', 'checklistScheduled', 'checklistCancelled', 'title', 'code'].includes(p.type)) {
            // logDebug('paragraph/findStartOfActivePartOfNote', `  - Found task/checklist/title/code line -> this line.`)
            startOfActive = i;
            break;
          } else if (p.type === 'separator' || p.type === 'empty') {
            // logDebug('paragraph/findStartOfActivePartOfNote', `  - Found separator/blank -> next line.`)
            startOfActive = i + 1;
            break;
          }
        }
        logDebug('paragraph/findStartOfActivePartOfNote', `-> ${String(startOfActive)}  (after finding preamble or metadata line)`);
      }
      return startOfActive;
    } catch (err) {
      logError('paragraph/findStartOfActivePartOfNote', err.message);
      return NaN; // for completeness
    }
  }

  /**
   * Works out where the first ## Done or ## Cancelled section starts, if present, and returns the paragraph before that.
   * Works with folded Done or Cancelled sections.
   * If the result is a separator, use the line before that instead
   * If neither Done or Cancelled present, return the last non-empty lineIndex.
   * @author @jgclark
   * @tests in jest file
   *
   * @param {TNote} note - the note to assess
   * @returns {number} - the index number (counting from zero)
   */
  function findEndOfActivePartOfNote(note) {
    try {
      const paras = note.paragraphs;
      let lineCount = paras.length;

      // If no lines, return 0
      if (lineCount === 0) {
        return 0;
      } else {
        // If last line is empty, ignore it.
        if (paras[paras.length - 1].type === 'empty') {
          // logDebug('paragraph/findEndOfActivePartOfNote', `last para is empty so ignoring it`)
          lineCount--;
        }

        // Find first example of ## Done
        const doneHeaderLines = paras.filter(p => p.headingLevel === 2 && p.content.startsWith('Done')) ?? [];
        let doneHeaderLine = doneHeaderLines.length > 0 ? doneHeaderLines[0].lineIndex : 0;
        // Now check to see if previous line was a separator; if so use that line instead
        if (doneHeaderLine > 2 && paras[doneHeaderLine - 1].type === 'separator') {
          doneHeaderLine -= 1;
        }
        // Find first example of ## Cancelled
        const cancelledHeaderLines = paras.filter(p => p.headingLevel === 2 && p.content.startsWith('Cancelled')) ?? [];
        let cancelledHeaderLine = cancelledHeaderLines.length > 0 ? cancelledHeaderLines[0].lineIndex : 0;
        // Now check to see if previous line was a separator; if so use that line instead
        if (cancelledHeaderLine > 2 && paras[cancelledHeaderLine - 1].type === 'separator') {
          cancelledHeaderLine -= 1;
        }
        const endOfActive = doneHeaderLine > 1 ? doneHeaderLine - 1 : cancelledHeaderLine > 1 ? cancelledHeaderLine - 1 : lineCount > 1 ? lineCount - 1 : 0;
        // logDebug('paragraph/findEndOfActivePartOfNote', `doneHeaderLine = ${doneHeaderLine}, cancelledHeaderLine = ${cancelledHeaderLine} endOfActive = ${endOfActive}`)
        return endOfActive;
      }
    } catch (err) {
      logError('paragraph/findEndOfActivePartOfNote', err.message);
      return NaN; // for completeness
    }
  }

  /**
   * Get the paragraph from the passed content (using exact match)
   * @author @jgclark
   *
   * @param {CoreNoteFields} note
   * @param {string} contentToFind
   * @return {TParagraph | void} pargraph object with that content, or null if not found
   */
  function getParaFromContent(note, contentToFind) {
    const {
      paragraphs
    } = note;
    for (const p of paragraphs) {
      if (p.content === contentToFind) {
        return p;
      }
    }
    logWarn('helper/getParaFromContent', `warning couldn't find '${contentToFind}`);
    return;
  }

  //-------------------------------------------------------------------------------
  // Folder-level Functions

  /**
   * Return a list of folders, with those that match the 'exclusions' list (or any of their sub-folders) removed.
   * EITHER
   *   - include only those that start with one of the strings on the inclusions list (so will include any sub-folders) if given. Note: Root folder can be included by '/'; this doesn't include sub-folders.
   * OR
   *   - exclude those that start with those on the 'exclusions' list, and any sub-folders (other than root folder ('/') which would then exclude everything).
   *   - optionally exclude all special @... folders as well [this overrides inclusions and exclusions]
   *   - optionally force exclude root folder. Note: setting this to false does not force include it.
   * If given inclusions, then exclusions will be ignored.
   * @author @jgclark
   * @tests in jest file
   * @param {Array<string>} exclusions - if these (sub)strings match then exclude this folder -- can be empty
   * @param {boolean} excludeSpecialFolders? (default: true)
   * @param {boolean} forceExcludeRootFolder? (default: false)
   * @returns {Array<string>} array of folder names
   */
  function getFolderListMinusExclusions(exclusions, excludeSpecialFolders = true, forceExcludeRootFolder = false) {
    try {
      // if (!inclusions && inclusions.length === 0 && !exclusions && exclusions.length === 0) {
      //   throw new Error('No inclusions or exclusions given.')
      // }
      // Get all folders as array of strings (other than @Trash). Also remove root as a special case
      const fullFolderList = DataStore.folders;
      let excludeRoot = forceExcludeRootFolder;
      // logDebug('folders / filteredFolderList', `Starting to filter the ${fullFolderList.length} DataStore.folders with exclusions [${exclusions.toString()}] and forceExcludeRootFolder ${String(forceExcludeRootFolder)}`)

      // if excludeSpecialFolders, filter fullFolderList to only folders that don't start with the character '@' (special folders)
      const reducedFolderList = excludeSpecialFolders ? fullFolderList.filter(folder => !folder.startsWith('@')) : fullFolderList;

      // To aid partial matching, terminate all folder strings with a trailing /
      let reducedTerminatedWithSlash = [];
      for (const f of reducedFolderList) {
        reducedTerminatedWithSlash.push(f.endsWith('/') ? f : `${f}/`);
      }

      // To aid partial matching, terminate all exclusion strings with a trailing /.
      const exclusionsTerminatedWithSlash = [];
      // Note: Root folder('/') here needs special handling: remove it now if found, but add back later.
      for (const e of exclusions) {
        if (e === '/') {
          excludeRoot = true;
        } else {
          exclusionsTerminatedWithSlash.push(e.endsWith('/') ? e : `${e}/`);
        }
      }
      // logDebug('getFolderListMinusExclusions', `- exclusionsTerminatedWithSlash: ${exclusionsTerminatedWithSlash.toString()}\n`)

      // if exclusions list is not empty, filter reducedTerminatedWithSlash to only folders that don't start with an item in the exclusionsTerminatedWithSlash list
      // reducedTerminatedWithSlash = reducedTerminatedWithSlash.filter((folder) => !exclusionsTerminatedWithSlash.some((ee) => folder.startsWith(ee)))
      reducedTerminatedWithSlash = reducedTerminatedWithSlash.filter(folder => !exclusionsTerminatedWithSlash.some(ef => caseInsensitiveStartsWith(ef, folder, false)));
      // logDebug('getFolderListMinusExclusions', `- after exclusions reducedTerminatedWithSlash: ${reducedTerminatedWithSlash.length} folders: ${reducedTerminatedWithSlash.toString()}\n`)

      // now remove trailing slash characters
      const outputList = reducedTerminatedWithSlash.map(folder => folder !== '/' && folder.endsWith('/') ? folder.slice(0, -1) : folder);

      // remove root folder if wanted
      if (excludeRoot) {
        const itemToRemove = outputList.indexOf('/');
        outputList.splice(itemToRemove, 1);
      }
      // logDebug('folders/getFolderListMinusExclusions', `-> outputList: ${outputList.length} items: [${outputList.toString()}] with excludeRoot? ${String(excludeRoot)}`)
      return outputList;
    } catch (error) {
      logError('folders/getFolderListMinusExclusions', error.message);
      return ['(error)'];
    }
  }

  /**
   * Get the folder name from the full NP (project) note filename, without leading or trailing slash.
   * Except for items in root folder -> '/'.
   * @author @jgclark
   * @param {string} fullFilename - full filename to get folder name part from
   * @returns {string} folder/subfolder name
   */
  function getFolderFromFilename(fullFilename) {
    try {
      // First deal with special case of file in root -> '/'
      if (!fullFilename.includes('/')) {
        return '/';
      }
      // drop first character if it's a slash
      const filename = fullFilename.startsWith('/') ? fullFilename.substr(1) : fullFilename;
      const filenameParts = filename.split('/');
      return filenameParts.slice(0, filenameParts.length - 1).join('/');
    } catch (error) {
      logError('folders/getFolderFromFilename', `Error getting folder from filename '${fullFilename}: ${error.message}`);
      return '(error)';
    }
  }

  // shared functions that can be imported and used in helpers without creating circular dependencies

  // TODO: following short functions should really live in helpers/paragraph.js

  /**
   * Test whether a para is open or not (type: 'open' or 'checklist') and not blank
   * @param {Paragraph} t - the paragraph/task to check
   * @returns {boolean} true if open, false if any other status/type
   */
  const isOpen = t => (t.type === 'open' || t.type === 'checklist') && t.content.trim() !== '';

  /**
   * Test whether a task is closed or not (types: 'done', 'cancelled', 'checklistDone', 'checklistCancelled').
   * Note: not the same as isDone(), which is tighter
   * @param {Paragraph} t - the paragraph/task to check
   * @returns {boolean} true if open, false if any other status/type
   */
  const isClosed = t => (t.type === 'done' || t.type === 'cancelled' || t.type === 'checklistDone' || t.type === 'checklistCancelled') && t.content.trim() !== '';

  /**
   * Test whether a task is closed or not (types: 'done', 'cancelled', 'checklistDone', 'checklistCancelled').
   * Note: tighter check than isClosed()
   * @param {Paragraph} t - the paragraph/task to check
   * @returns {boolean} true if open, false if any other status/type
   */
  const isDone = t => (t.type === 'done' || t.type === 'checklistDone') && t.content.trim() !== '';

  /**
   * Test whether a string has a scheduled date (e.g. >2020-01-01, >2020-01, >2020, >2020-W1, >2020-Q1), and not an arrow date (>date<).
   * @param {string} content
   * @returns {boolean} true if has a scheduled date, false if not
   */
  const hasScheduledDate = content => RE_IS_SCHEDULED.test(content) && !RE_ARROW_DATES_G.test(content);

  /**
   * Test whether a paragraph/task is scheduled (type: 'scheduled' or open with a scheduled date), and not an arrow date (>date<).
   * @param {TParagraph} t
   * @returns {boolean} - true if scheduled, false if not
   */
  const isScheduled = t => t.type === 'scheduled' || t.type === 'open' && hasScheduledDate(t.content) && !RE_ARROW_DATES_G.test(t.content);

  /* eslint-disable max-len */ //-------------------------------------------------------------------------------
  // Note-level Functions

  /**
   * Get a note using whatever method works (open by title, filename, etc.)
   * Note: this function was used to debug/work-around API limitations. Probably not necessary anymore
   * Leaving it here for the moment in case any plugins are still using it
   * @author @dwertheimer
   * @param {string} fullPath
   * @param {string} desc
   * @param {boolean} useProjNoteByFilename (default: true)
   * @returns {any} - the note that was opened
   */
  async function noteOpener(fullPath, desc, useProjNoteByFilename = true) {
    logDebug('note/noteOpener', `  About to open filename: "${fullPath}" (${desc}) using ${useProjNoteByFilename ? 'projectNoteByFilename' : 'noteByFilename'}`);
    const newNote = useProjNoteByFilename ? await DataStore.projectNoteByFilename(fullPath) : await DataStore.noteByFilename(fullPath, 'Notes');
    if (newNote != null) {
      logDebug('note/noteOpener', `    Opened ${fullPath} (${desc} version) `);
      return newNote;
    } else {
      logDebug('note/noteOpener', `    Didn't work! ${useProjNoteByFilename ? 'projectNoteByFilename' : 'noteByFilename'} returned ${newNote}`);
    }
  }

  /**
   * Get all notes in a given folder (or all project notes if no folder given), sorted by note title.
   * Optionally look in sub-folders as well.
   * @author @jgclark
   *
   * @param {string} folder - folder to scan
   * @param {string} alsoSubFolders? - also look in subfolders under the folder name
   * @return {Array<TNote>} - list of notes
   */
  function notesInFolderSortedByTitle(folder, alsoSubFolders = false) {
    try {
      // logDebug('note/notesInFolderSortedByTitle', `Starting for folder '${folder}'`)
      const allNotesInFolder = DataStore.projectNotes.slice();
      let notesInFolder;
      // If folder given (not empty) then filter using it
      if (folder !== '') {
        if (alsoSubFolders) {
          notesInFolder = allNotesInFolder.filter(n => getFolderFromFilename(n.filename).startsWith(folder));
        } else {
          notesInFolder = allNotesInFolder.filter(n => getFolderFromFilename(n.filename) === folder);
        }
      } else {
        // return all project notes
        notesInFolder = allNotesInFolder;
      }
      // Sort alphabetically on note's title
      const notesSortedByTitle = notesInFolder.sort((first, second) => (first.title ?? '').localeCompare(second.title ?? ''));
      return notesSortedByTitle;
    } catch (err) {
      logError('note/notesInFolderSortedByTitle', err.message);
      return [];
    }
  }

  /**
   * Find a unique note title for the given text (e.g. "Title", "Title 01" (if "Title" exists, etc.))
   * Keep adding numbers to the end of a filename (if already taken) until it works
   * @author @dwertheimer
   * @param {string} title - the name of the file
   * @returns {string} the title (not filename) that was created
   */
  function getUniqueNoteTitle(title) {
    try {
      let i = 0;
      let res = [];
      let newTitle = title;
      while (++i === 1 || res.length > 0) {
        newTitle = i === 1 ? title : `${title} ${i}`;
        // $FlowFixMe(incompatible-type)
        res = DataStore.projectNoteByTitle(newTitle, true, false);
      }
      return newTitle;
    } catch (err) {
      logError('note/notesInFolderSortedByTitle', err.message);
      return '';
    }
  }

  /**
   * TODO: finish moving refs for this from NPnote to here.
   * Return array of all project notes, excluding those in list of folders to exclude, and (if requested) from special '@...' folders
   * @author @jgclark
   * @param {Array<string>} foldersToExclude
   * @param {boolean} excludeSpecialFolders?
   * @returns {Array<TNote>} wanted notes
   */
  function projectNotesFromFilteredFolders(foldersToExclude, excludeSpecialFolders) {
    // Get list of wanted folders
    const filteredFolders = getFolderListMinusExclusions(foldersToExclude, excludeSpecialFolders);

    // Iterate over all project notes and keep the notes in the wanted folders ...
    const allProjectNotes = DataStore.projectNotes;
    const projectNotesToInclude = [];
    for (const pn of allProjectNotes) {
      const thisFolder = getFolderFromFilename(pn.filename);
      if (filteredFolders.includes(thisFolder)) {
        projectNotesToInclude.push(pn);
      }
    }
    return projectNotesToInclude;
  }

  /**
   * Return list of all notes, sorted by changed date (newest to oldest)
   * @author @jgclark
   * @param {Array<string>} foldersToExclude? (default: [])
   * @return {Array<TNote>} array of notes
   */
  function allNotesSortedByChanged(foldersToIgnore = []) {
    const projectNotes = projectNotesFromFilteredFolders(foldersToIgnore, true);
    const calendarNotes = DataStore.calendarNotes.slice();
    const allNotes = projectNotes.concat(calendarNotes);
    // $FlowIgnore(unsafe-arithmetic)
    const allNotesSorted = allNotes.sort((first, second) => second.changedDate - first.changedDate); // most recent first
    return allNotesSorted;
  }

  //-------------------------------------------------------------------------------
  // Note-level Functions that require NP API calls
  //-------------------------------------------------------------------------------
  // import moment from 'moment/min/moment-with-locales'
  const pluginJson$2 = 'NPnote.js';

  //-------------------------------------------------------------------------------

  /**
   * Print summary of note details to log.
   * @author @eduardmet
   * @param {?TNote} noteIn
   * @param {boolean?} alsoShowParagraphs? (default: false)
   */
  function printNote(noteIn, alsoShowParagraphs = false) {
    try {
      let note;
      if (noteIn == null) {
        logDebug('note/printNote()', 'No Note passed. Will try Editor note.');
        note = Editor?.note;
      } else {
        note = noteIn;
      }
      if (!note) {
        logWarn('note/printNote()', `No valid note found. Stopping.`);
        return;
      }
      if (note.type === 'Notes') {
        const endOfActive = findEndOfActivePartOfNote(note);
        logInfo('note/printNote', `title: ${note.title ?? ''}\n- filename: ${note.filename ?? ''}\n- created: ${String(note.createdDate) ?? ''}\n- changed: ${String(note.changedDate) ?? ''}\n- paragraphs: ${note.paragraphs.length} (endOfActive: ${String(endOfActive)})\n- hashtags: ${note.hashtags?.join(', ') ?? ''}\n- mentions: ${note.mentions?.join(', ') ?? ''}`);
      } else {
        logInfo('note/printNote', `filename: ${note.filename ?? ''}\n- created: ${String(note.createdDate) ?? ''}\n- changed: ${String(note.changedDate) ?? ''}\n- paragraphs: ${note.paragraphs.length}\n- hashtags: ${note.hashtags?.join(', ') ?? ''}\n- mentions: ${note.mentions?.join(', ') ?? ''}`);
      }
      if (note.paragraphs.length > 0) {
        const open = note.paragraphs.filter(p => isOpen(p)).length;
        const done = note.paragraphs.filter(p => isDone(p)).length;
        const closed = note.paragraphs.filter(p => isClosed(p)).length;
        const scheduled = note.paragraphs.filter(p => isScheduled(p)).length;
        console.log(`- open: ${String(open)}\n- done: ${String(done)}\n- closed: ${String(closed)}\n- scheduled: ${String(scheduled)}`);
        if (alsoShowParagraphs) {
          note.paragraphs.map(p => console.log(`- ${p.lineIndex}: ${p.type} ${p.rawContent}`));
        }
      }
      // Now show .backlinks
      if (note.backlinks.length > 0) {
        console.log(`- ${String(note.backlinks.length)} backlinked notes`);
        const flatBacklinkParas = getFlatListOfBacklinks(note); // Note: this requires DataStore
        console.log(`- ${String(flatBacklinkParas.length)} backlink paras:`);
        for (let i = 0; i < flatBacklinkParas.length; i++) {
          const p = flatBacklinkParas[i];
          console.log(`  - ${p.lineIndex} [${p.type}, ${p.indents}]: ${p.content}`);
        }
      }
    } catch (e) {
      logError('note/printNote', `Error printing note: ${e.message}`);
    }
  }

  /**
   * Convert the note to use frontmatter syntax.
   * If optional default text is given, this is added to the frontmatter.
   * @author @jgclark
   * @param {TNote} note to convert
   * @param {string?} defaultFMText to add to frontmatter if supplied
   * @returns {boolean} success?
   */
  function convertNoteToFrontmatter(note, defaultFMText = '') {
    try {
      if (!note) {
        throw new Error("NPnote/convertNoteToFrontmatter: No valid note supplied.");
      }
      const result = ensureFrontmatter(note);
      if (result) {
        logDebug('NPnote/convertNoteToFrontmatter', `ensureFrontmatter() worked for note ${note.filename}`);
        if (defaultFMText !== '') {
          const endOfFMLineIndex = endOfFrontmatterLineIndex(note); // closing separator line
          if (endOfFMLineIndex !== false) {
            note.insertParagraph(defaultFMText, endOfFMLineIndex, 'text'); // inserts before closing separator line
          } else {
            logWarn('NPnote/convertNoteToFrontmatter', `endOfFrontmatterLineIndex() failed for note ${note.filename}`);
          }
        }
      } else {
        logWarn('NPnote/convertNoteToFrontmatter', `ensureFrontmatter() failed for note ${note.filename}`);
      }
    } catch (error) {
      logError(pluginJson$2, `convertNoteToFrontmatter: ${error.message}`);
    }
  }

  /**
   * note.backlinks is an array of Paragraphs, but its subItems can be nested. The nesting can be multiple levels deep.
   * This function returns an array of TParagraphs, one for each backlink, undoing the nesting.
   */
  // $FlowFixMe[incompatible-return]
  function getFlatListOfBacklinks(note) {
    // Iterate over all backlinks, recursing where necessary to visit all subItems, returning a flat list of lineIndex
    function flattenSubItems(subItems) {
      const items = [];
      subItems.forEach(item => {
        if (item.subItems) {
          items.push(item);
          // logDebug('note/getFlatListOfBacklinks', `+ ${item.lineIndex}: ${item.content} [has ${items.length} saved indexes`)
          // Recursively process any subItems of the current item
          items.push(...flattenSubItems(item.subItems));
        }
      });
      return items;
    }
    try {
      const noteBacklinks = note.backlinks;
      if (noteBacklinks.length === 0) {
        return [];
      }
      // logDebug('note/getFlatListOfBacklinks', `Starting for note ${displayTitle(note)}`)
      const flatBacklinkParas = [];
      for (const noteBacklink of noteBacklinks) {
        // Get the note that this backlink points to
        const thisBacklinkNote = DataStore.noteByFilename(noteBacklink.filename, noteBacklink.noteType);
        const thisBacklinkNoteParas = thisBacklinkNote?.paragraphs;
        if (!thisBacklinkNoteParas) {
          logError('note/getFlatListOfBacklinks', `Error getting paragraphs for ${noteBacklink.filename}`);
        }
        // logDebug('note/getFlatListOfBacklinks', `in ${thisBacklinkNote?.filename ?? '(error)'}`)
        let thisNoteItems = [];
        // thisNoteLineIndexes.push(noteBacklink.lineIndex) // noteBacklink.lineIndex

        if (noteBacklink.subItems && noteBacklink.subItems.length > 0) {
          // logDebug('note/getFlatListOfBacklinks', `- has ${noteBacklink.subItems.length} top-levelsubItems`)
          thisNoteItems = flattenSubItems(noteBacklink.subItems);
        }
        // logDebug('note/getFlatListOfBacklinks', `  => ${thisNoteItems.length} items`)

        // Now find paragraphs from those lineIndexes
        for (const item of thisNoteItems) {
          // logDebug('note/getFlatListOfBacklinks', `+ ${item.lineIndex}`)
          // $FlowIgnore[incompatible-use]
          flatBacklinkParas.push(thisBacklinkNoteParas[item.lineIndex]);
        }
        // logDebug('note/getFlatListOfBacklinks', `  fBLP now has ${String(flatBacklinkParas.length)} paragraphs`)
      }
      return flatBacklinkParas;
    } catch (e) {
      logError('note/printNote', `Error printing note: ${e.message}`);
    }
  }
  /**
   * Convenience Method for Editor.openNoteByFilename, include only the options you care about (requires NP v3.7.2+)
   * Tries to work around NP bug where opening a note that doesn't exist doesn't work
   * If you send the options.content field to force content setting,   it should have a value or undefined (not null)
   * @param {string} filename - Filename of the note file (can be without extension), but has to include the relative folder such as `folder/filename.txt`
   * @param {OpenNoteOptions} options - options for opening the note (all optional -- see fields in type)
   * @returns {Promise<TNote|void>} - the note that was opened
   * @author @dwertheimer
   */
  async function openNoteByFilename(filename, options = {}) {
    const isCalendarNote = isValidCalendarNoteFilename(filename);
    let note = await Editor.openNoteByFilename(filename, options.newWindow || false, options.highlightStart || 0, options.highlightEnd || 0, options.splitView || false, options.createIfNeeded || false, options.content || undefined /* important for this to be undefined or NP creates a note with "null" */);
    if (!note) {
      logDebug(pluginJson$2, `openNoteByFilename could not open note with filename: "${filename}" (probably didn't exist)`);
      // note may not exist yet, so try to create it (if it's a calendar note)
      const dataStoreNote = isCalendarNote ? await DataStore.noteByFilename(filename, 'Calendar') : null;
      if (dataStoreNote) {
        dataStoreNote.content = '';
        // $FlowIgnore[incompatible-call]
        note = await Editor.openNoteByFilename(filename, options.newWindow || false, options.highlightStart || 0, options.highlightEnd || 0, options.splitView || false, options.createIfNeeded || false, options.content || undefined);
      }
    }
    if (!note) {
      logError(pluginJson$2, `openNoteByFilename could not open ${isCalendarNote ? 'Calendar ' : 'Project'} note with filename: "${filename}" ${isCalendarNote ? '' : '. You may need to set "createIfNeeded" to true for this to work'}`);
    }
    return note;
  }
  /**
   * Get list of headings from a note, optionally including markdown markers.
   * Note: If the first 'title' line matches the note title, then skip it (as it's the title itself).
   * Note: There is no right-trimming of the heading text, as this can cause problems with NP API calls which don't do trimming when you expect they would.
   * @author @dwertheimer (adapted from @jgclark)
   *
   * @param {TNote} note - note to get headings from
   * @param {boolean} includeMarkdown - whether to include markdown markers in the headings
   * @param {boolean} optionAddATopAndtBottom - whether to add 'top of note' and 'bottom of note' options. Default: true.
   * @param {boolean} optionCreateNewHeading - whether to offer to create a new heading at top or bottom of note. Default: false.
   * @param {boolean} includeArchive - whether to include headings in the Archive section of the note (i.e. after 'Done'). Default: false.
   * @return {Array<string>}
   */
  function getHeadingsFromNote(note, includeMarkdown = false, optionAddATopAndtBottom = true, optionCreateNewHeading = false, includeArchive = false) {
    let headingStrings = [];
    const spacer = '#';
    let headingParas = [];
    const indexEndOfActive = findEndOfActivePartOfNote(note);
    if (includeArchive) {
      headingParas = note.paragraphs.filter(p => p.type === 'title' && p.lineIndex < indexEndOfActive);
    } else {
      headingParas = note.paragraphs.filter(p => p.type === 'title');
    }

    // If this is the title line, skip it
    if (headingParas.length > 0) {
      if (headingParas[0].content === note.title) {
        headingParas = headingParas.slice(1);
      }
    }
    if (headingParas.length > 0) {
      headingStrings = headingParas.map(p => {
        let prefix = '';
        for (let i = 0; i < p.headingLevel; i++) {
          prefix += spacer;
        }
        return `${prefix} ${p.content.trimLeft()}`;
      });
    }
    if (optionCreateNewHeading) {
      if (note.type === 'Calendar') {
        headingStrings.unshift('➕#️⃣ (first insert new heading at the start of the note)');
      } else {
        headingStrings.unshift(`➕#️⃣ (first insert new heading under the title)`);
      }
      headingStrings.push(`➕#️⃣ (first insert new heading at the end of the note)`);
    }
    if (optionAddATopAndtBottom) {
      headingStrings.unshift('⏫ (top of note)');
      headingStrings.push('⏬ (bottom of note)');
    }
    if (headingStrings.length === 0) {
      return [''];
    }
    if (!includeMarkdown) {
      headingStrings = headingStrings.map(h => h.replace(/^#{1,5}\s*/, '')); // remove any markdown heading markers
    }
    return headingStrings;
  }

  //-----------------------------------------------------------------------------
  // Specialised user input functions

  // (from @nmn / nmn.sweep)

  /**
   * Ask user to choose from a set of options (from nmn.sweep) using CommandBar
   * @author @nmn
   *
   * @param {string} message - text to display to user
   * @param {Array<T>} options - array of label:value options to present to the user
   * @param {TDefault} defaultValue - (optional) default value to use (default: options[0].value)
   * @return {TDefault} - the value attribute of the user-chosen item
   */
  async function chooseOption(message, options, defaultValue = null) {
    const {
      index
    } = await CommandBar.showOptions(options.map(option => option.label), message);
    return options[index]?.value ?? defaultValue ?? options[0].value;
  }

  /**
   * Ask user to choose from a set of options (from nmn.sweep) using CommandBar
   * @author @dwertheimer based on @nmn chooseOption
   *
   * @param {string} message - text to display to user
   * @param {Array<T>} options - array of label:value options to present to the user
   * @return {{ label:string, value:string, index: number, keyModifiers: Array<string> }} - the value attribute of the user-chosen item
   * keyModifiers is an array of 0+ strings, e.g. ["cmd", "opt", "shift", "ctrl"] that were pressed while selecting a result.
   */
  // @nmn we need some $FlowFixMe
  async function chooseOptionWithModifiers(message, options) {
    logDebug('userInput / chooseOptionWithModifiers()', `About to showOptions with ${options.length} options & prompt:"${message}"`);
    // $FlowFixMe[prop-missing]
    const {
      index,
      keyModifiers
    } = await CommandBar.showOptions(options.map(option => option.label), message);
    // $FlowFixMe[incompatible-return]
    return {
      ...options[index],
      index,
      keyModifiers
    };
  }

  /**
   * Ask user to give arbitary input using CommandBar.
   * Will now use newer native dialog if available (from 3.3.2), which gets a title and default, but doesn't allow to customise the button text.
   * @author @jgclark, updating @nmn
   *
   * @param {string} message - request text to display to user
   * @param {?string} okLabel - the "button" (option) text (default: 'OK')
   * @param {?string} dialogTitle - title for the dialog (default: empty)
   * @param {?string} defaultValue - default value to display in text entry (default: empty)
   * @return {Promise<boolean|string>} - string that the user enters. Maybe be the empty string. If the user cancels the operation, it will return false instead.
   */
  async function getInput(message, okLabel = 'OK', dialogTitle = 'Enter value', defaultValue = '') {
    if (typeof CommandBar.textPrompt === 'function') {
      // i.e. do we have .textPrompt available?
      return await CommandBar.textPrompt(dialogTitle, message, defaultValue);
    } else {
      return await CommandBar.showInput(message, okLabel);
    }
  }

  /**
   * Get user input, trimmed at both ends, using CommandBar.
   * Will now use newer native dialog if available (from 3.3.2), which gets a title and default, but doesn't allow to customise the button text.
   * @author @jgclark, updating @m1well
   *
   * @param {string} message - request text to display to user
   * @param {?string} okLabel - the "button" (option) text (default: 'OK')
   * @param {?string} dialogTitle - title for the dialog (default: empty)
   * @param {?string} defaultValue - default value to display in text entry (default: empty)
   * @returns {Promise<boolean|string>} string that the user enters. Maybe be the empty string. If the user cancels the operation, it will return false instead.
   */
  async function getInputTrimmed(message, okLabel = 'OK', dialogTitle = 'Enter value', defaultValue = '') {
    if (typeof CommandBar.textPrompt === 'function') {
      // i.e. do we have .textPrompt available?
      const reply = await CommandBar.textPrompt(dialogTitle, message, defaultValue);
      return typeof reply === 'string' ? reply.trim() : reply;
    } else {
      const reply = await CommandBar.showInput(message, okLabel);
      return reply.trim();
    }
  }

  /**
   * Show a single-button dialog-box like message (modal) using CommandBar.
   * Will now use newer native dialog if available (from 3.3.2), which adds a title.
   * Note: There's a copy in helpersNPParagaph.js to avoid a circular dependency
   * @author @jgclark, updating @dwertheimer, updating @nmn
   *
   * @param {string} message - text to display to user
   * @param {?string} confirmButton - the "button" (option) text (default: 'OK')
   * @param {?string} dialogTitle - title for the dialog (default: empty)
   * @param {?boolean} useCommandBar - force use NP CommandBar instead of native prompt (default: false)
   */
  async function showMessage(message, confirmButton = 'OK', dialogTitle = '', useCommandBar = false) {
    if (typeof CommandBar.prompt === 'function' && !useCommandBar) {
      // i.e. do we have .textPrompt available?
      await CommandBar.prompt(dialogTitle, message, [confirmButton]);
    } else {
      await CommandBar.showOptions([confirmButton], message);
    }
  }

  /**
   * Show a simple Yes/No (could be OK/Cancel, etc.) dialog using CommandBar.
   * Returns the text of the chosen option (by default 'Yes' or 'No')
   * Will now use newer native dialog if available (from 3.3.2), which adds a title.
   * Note: There's a copy in helpers/NPParagaph.js to avoid a circular dependency
   * @author @jgclark, updating @nmn
   *
   * @param {string} message - text to display to user
   * @param {?Array<string>} choicesArray - an array of the choices to give (default: ['Yes', 'No'])
   * @param {?string} dialogTitle - title for the dialog (default: empty)
   * @param {?boolean} useCommandBar - force use NP CommandBar instead of native prompt (default: false)
   * @returns {string} - returns the user's choice - the actual *text* choice from the input array provided (by default 'Yes' or 'No')
   */
  async function showMessageYesNo(message, choicesArray = ['Yes', 'No'], dialogTitle = '', useCommandBar = false) {
    let answer;
    if (typeof CommandBar.prompt === 'function' && !useCommandBar) {
      // i.e. do we have .textPrompt available?
      answer = await CommandBar.prompt(dialogTitle, message, choicesArray);
    } else {
      const answerObj = await CommandBar.showOptions(choicesArray, `${message}`);
      answer = answerObj.index;
    }
    return choicesArray[answer];
  }

  /**
   * Show a simple yes/no/cancel (or OK/No/Cancel, etc.) native dialog.
   * @author @jgclark
   *
   * @param {string} message - text to display to user
   * @param {?Array<string>} choicesArray - an array of the choices to give (default: ['Yes', 'No'])
   * @param {?string} dialogTitle - title for the dialog (default: empty)
   * @param {?boolean} useCommandBar - force use NP CommandBar instead of native prompt (default: false)
   * @returns {string} - returns the user's choice - the actual *text* choice from the input array provided
   */
  async function showMessageYesNoCancel(message, choicesArray = ['Yes', 'No', 'Cancel'], dialogTitle = '') {
    const answer = await CommandBar.prompt(dialogTitle, message, choicesArray);
    return choicesArray[answer];
  }

  /**
   * Let user pick from a nicely-indented list of available folders (or / for root, or optionally create a new folder)
   * @author @jgclark + @dwertheimer
   *
   * @param {string} msg - text to display to user
   * @param {boolean} includeArchive - if true, include the Archive folder in the list of folders; default is false
   * @param {boolean} includeNewFolderOption - if true, add a 'New Folder' option that will allow users to create a new folder and select it; IMPORTANT
   * NOTE: the API does not allow for creation of the folder, so all this does is pass back a path which will be created when the user saves/moves the note
   * If your use case does not include the creation or moving of a note to the chosen path, this option will not work for you
   * @param {string} startFolder - folder to start the list in (e.g. to limit the folders to a specific subfolder) - default is root (/) -- set to "/" to force start at root
   * @returns {string} - returns the user's folder choice (or / for root)
   */
  async function chooseFolder(msg, includeArchive = false, includeNewFolderOption = false, startFolder) {
    const IS_DESKTOP = NotePlan.environment.platform === 'macOS';
    const NEW_FOLDER = `➕ (Add New Folder${IS_DESKTOP ? ' - or opt-click on a parent folder to create new subfolder' : ''})`;
    let folder;
    let folders = [];
    if (includeNewFolderOption) {
      folders.push(NEW_FOLDER);
    }
    folders = [...folders, ...DataStore.folders.slice()]; // excludes Trash
    {
      const archiveFolders = folders.filter(f => f.startsWith('@Archive'));
      const otherSpecialFolders = folders.filter(f => f.startsWith('@') && !f.startsWith('@Archive'));
      // Remove special folders from list
      folders = folders.filter(f => !f.startsWith('@'));
      // Now add them back on at the end, with @Archive going last
      folders = [...folders, ...otherSpecialFolders];
      if (includeArchive) {
        folders = [...folders, ...archiveFolders];
      }
    }
    let value, keyModifiers;
    if (folders.length > 0) {
      // make a slightly fancy list with indented labels, different from plain values
      const folderOptionList = [];
      for (const f of folders) {
        if (f === NEW_FOLDER) {
          folderOptionList.push({
            label: NEW_FOLDER,
            value: NEW_FOLDER
          });
        } else if (f !== '/') {
          const folderParts = f.split('/');
          const icon = folderParts[0] === '@Archive' ? `🗄️` : folderParts[0] === '@Templates' ? '📝' : '📁';
          // Replace earlier parts of the path with indentation spaces
          for (let i = 0; i < folderParts.length - 1; i++) {
            folderParts[i] = '     ';
          }
          folderParts[folderParts.length - 1] = `${icon} ${folderParts[folderParts.length - 1]}`;
          const folderLabel = folderParts.join('');
          folderOptionList.push({
            label: folderLabel,
            value: f
          });
        } else {
          // deal with special case for root folder
          folderOptionList.push({
            label: '📁 /',
            value: '/'
          });
        }
      }
      ({
        value,
        keyModifiers
      } = await chooseOptionWithModifiers(msg, folderOptionList));
      if (keyModifiers?.length && keyModifiers.indexOf('opt') > -1) {
        folder = NEW_FOLDER;
      } else {
        folder = value;
      }
      logDebug(`helpers/userInput`, `chooseFolder folder:${folder} value:${value} keyModifiers:${keyModifiers} keyModifiers.indexOf('opt')=${keyModifiers.indexOf('opt')}`);
    } else {
      // no Folders so go to root
      folder = '/';
    }
    // logDebug('userInput / chooseFolder', `-> ${folder}`)
    if (folder === NEW_FOLDER) {
      const optClicked = value?.length && keyModifiers && keyModifiers.indexOf('opt') > -1;
      const newFolderName = await CommandBar.textPrompt(`Create new folder${optClicked ? ` inside folder:\n"${value || ''}".` : '...\nYou will choose where to create the folder in the next step.'}`, 'Folder name:', '');
      if (newFolderName && newFolderName.length) {
        const inWhichFolder = optClicked && value ? value : await chooseFolder(`Create '${newFolderName}' inside which folder? (${'/'} for root)`, includeArchive, false);
        if (inWhichFolder) {
          folder = inWhichFolder === '/' ? newFolderName : `${inWhichFolder}/${newFolderName}`;
        }
      }
    }
    logDebug(`helpers/userInput`, `chooseFolder folder chosen: "${folder}"`);
    return folder;
  }

  /**
   * Ask user to select a heading from those in a given note (regular or calendar), or optionally create a new heading at top or bottom of note to use, or the top or bottom of the note.
   * Note: Any whitespace on the end of the heading text is left in place, as otherwise this would cause issues with NP API calls that take heading parameter.
   * @author @jgclark
   *
   * @param {TNote} note - note to draw headings from
   * @param {boolean} optionAddATopAndtBottom - whether to add 'top of note' and 'bottom of note' options. Default: true.
   * @param {boolean} optionCreateNewHeading - whether to offer to create a new heading at the top or bottom of the note. Default: false.
   * @param {boolean} includeArchive - whether to include headings in the Archive section of the note (i.e. after 'Done'). Default: false.
   * @param {number} headingLevel - if adding a heading, the H1-H5 level to set (as an integer)
   * @returns {string} - the selected heading as text without any markdown heading markers. Blank string implies no heading selected, and user wishes to write to the end of the note. Special string '<<top of note>>' implies to write to the top (after any preamble or frontmatter). Also <<bottom of note>>
   */
  async function chooseHeading(note, optionAddATopAndtBottom = true, optionCreateNewHeading = false, includeArchive = false, headingLevel = 2) {
    try {
      const headingStrings = getHeadingsFromNote(note, true, optionAddATopAndtBottom, optionCreateNewHeading, includeArchive);

      // Present heading options to user and ask for choice
      const result = await CommandBar.showOptions(headingStrings, `Select a heading from note '${note.title ?? 'Untitled'}'`);
      // Get the underlying heading back by removing added # marks and trimming left. We don't trim right as there can be valid traillng spaces.
      let headingToReturn = headingStrings[result.index].replace(/^#{1,5}\s*/, '');
      headingToReturn = await processChosenHeading(note, headingLevel, headingToReturn);
      return headingToReturn;
    } catch (error) {
      logError('userInput / chooseHeading', error.message);
      return '<error>';
    }
  }
  const relativeDates = getRelativeDates();

  /**
   * Create a new note with a given title, content, and in a specified folder.
   * If title, content, or folder is not provided, it will prompt the user for input.
   *
   * @param {string} [_title] - The title of the new note.
   * @param {string} [_content] - The content of the new note.
   * @param {string} [_folder] - The folder to create the new note in.
   * @returns {Promise<Note | false>} - The newly created note, or false if the operation was cancelled.
   */
  async function createNewNote(_title = '', _content = '', _folder = '') {
    const title = _title || (await getInput('Title of new note', 'OK', 'New Note', ''));
    const content = _content;
    if (title) {
      const folder = _folder || (await chooseFolder('Select folder to add note in:', false, true));
      const noteContent = `# ${title}\n${content}`;
      const filename = await DataStore.newNoteWithContent(noteContent, folder);
      return DataStore.noteByFilename(filename, 'Notes') || null;
    } else {
      return null;
    }
  }

  /**
   * V2 of displayTitle that optionally adds the relative date string after relevant calendar note titles, to make it easier to spot last/this/next D/W/M/Q
   * Note: that this returns ISO title for daily notes (YYYY-MM-DD) not the one from the filename. This is different from the original displayTitle.
   * Note: Forked from helpers/general.js, but needed here anyway to avoid a circular dependency
   * @param {CoreNoteFields} noteIn
   * @param {boolean} showRelativeDates? (default: false)
   * @returns {string}
   */
  function displayTitleWithRelDate(noteIn, showRelativeDates = true) {
    if (noteIn.type === 'Calendar') {
      let calNoteTitle = getDateStringFromCalendarFilename(noteIn.filename, true) ?? '(error)';
      if (showRelativeDates) {
        for (const rd of relativeDates) {
          if (calNoteTitle === rd.dateStr) {
            // console.log(`Found match with ${rd.relName}`)
            calNoteTitle = `${rd.dateStr}\t(📆 ${rd.relName})`;
          }
        }
      }
      return calNoteTitle;
    } else {
      return noteIn.title ?? '(error)';
    }
  }

  /**
   * Choose a particular note from a CommandBar list of notes
   * @author @dwertheimer extended by @jgclark to include 'relative date' indicators in displayed title
   * @param {boolean} includeProjectNotes
   * @param {boolean} includeCalendarNotes
   * @param {Array<string>} foldersToIgnore - a list of folder names to ignore
   * @param {string} promptText - text to display in the CommandBar
   * @param {boolean} currentNoteFirst - add currently open note to the front of the list
   * @param {boolean} allowNewNoteCreation - add option for user to create new note to return instead of choosing existing note
   * @returns {TNote | null} note
   */
  async function chooseNote(includeProjectNotes = true, includeCalendarNotes = false, foldersToIgnore = [], promptText = 'Choose a note', currentNoteFirst = false, allowNewNoteCreation = false) {
    let noteList = [];
    const projectNotes = DataStore.projectNotes;
    const calendarNotes = DataStore.calendarNotes;
    if (includeProjectNotes) {
      noteList = noteList.concat(projectNotes);
    }
    if (includeCalendarNotes) {
      noteList = noteList.concat(calendarNotes);
    }
    const noteListFiltered = noteList.filter(note => {
      // filter out notes that are in folders to ignore
      let isInIgnoredFolder = false;
      foldersToIgnore.forEach(folder => {
        if (note.filename.includes(`${folder}/`)) {
          isInIgnoredFolder = true;
        }
      });
      isInIgnoredFolder = isInIgnoredFolder || !/(\.md|\.txt)$/i.test(note.filename); //do not include non-markdown files
      return !isInIgnoredFolder;
    });
    const sortedNoteListFiltered = noteListFiltered.sort((first, second) => second.changedDate - first.changedDate); // most recent first
    const opts = sortedNoteListFiltered.map(note => {
      return displayTitleWithRelDate(note);
    });
    const {
      note
    } = Editor;
    if (allowNewNoteCreation) {
      opts.unshift('[New note]');
      sortedNoteListFiltered.unshift('[New note]'); // just keep the indexes matching
    }
    if (currentNoteFirst && note) {
      sortedNoteListFiltered.unshift(note);
      opts.unshift(`[Current note: "${displayTitleWithRelDate(Editor)}"]`);
    }
    const {
      index
    } = await CommandBar.showOptions(opts, promptText);
    let noteToReturn = sortedNoteListFiltered[index];
    if (noteToReturn === '[New note]') {
      noteToReturn = await createNewNote();
    }
    return noteToReturn ?? null;
  }

  /**
   * Used as part of chooseHeading (above) and Dashboard, to handle special instructions -- inserting a new heading, or inserting at top or bottom of the note.
   * If there are no special instructions, it just returns the heading as is.
   * @param {TNote} note
   * @param {number} headingLevel - The level of the heading to add (1-5) where requested
   * @param {string} chosenHeading - The text of the new heading to add (where requested)
   * @returns {string} headingToReturn
   */
  async function processChosenHeading(note, headingLevel = 2, chosenHeading) {
    let newHeading,
      headingToReturn = chosenHeading;
    logDebug('userInput / processChosenHeading', `headingLevel: ${headingLevel} chosenHeading: '${chosenHeading}'`);
    switch (headingToReturn) {
      case `➕#️⃣ (first insert new heading at the start of the note)`:
        // ask for new heading, and insert right at top
        newHeading = await getInput(`Enter heading to add at the start of the note`);
        if (newHeading && typeof newHeading === 'string') {
          const startPos = 0;
          // $FlowIgnore
          note.insertHeading(newHeading, startPos, headingLevel);
          logDebug('userInput / processChosenHeading', `prepended new heading '${newHeading}' at line ${startPos} (calendar note)`);
          headingToReturn = newHeading;
        } else {
          throw new Error(`user cancelled operation`);
        }
        break;
      case '\u2795#\ufe0f\u20e3 (first insert new heading under the title)':
        // ask for new heading, find smart insertion position, and insert it
        newHeading = await getInput(`Enter heading to add at the start of the note`);
        if (newHeading && typeof newHeading === 'string') {
          const startPos = findStartOfActivePartOfNote(note);
          // $FlowIgnore
          note.insertHeading(newHeading, startPos, headingLevel);
          logDebug('userInput / processChosenHeading', `prepended new heading '${newHeading}' at line ${startPos} (project note)`);
          headingToReturn = newHeading;
        } else {
          throw new Error(`user cancelled operation`);
        }
        break;
      case `➕#️⃣ (first insert new heading at the end of the note)`:
        // ask for new heading, and then append it
        newHeading = await getInput(`Enter heading to add at the end of the note`);
        if (newHeading && typeof newHeading === 'string') {
          const indexEndOfActive = findEndOfActivePartOfNote(note);
          const newLindeIndex = indexEndOfActive + 1;
          // $FlowIgnore - headingLevel is a union type, and we've already checked it's a number
          note.insertHeading(newHeading, newLindeIndex, headingLevel || 2);
          logDebug('userInput / processChosenHeading', `appended new heading '${newHeading}' at line ${newLindeIndex}`);
          headingToReturn = newHeading;
        } else {
          throw new Error(`user cancelled operation`);
        }
        break;
      case '\u23eb (top of note)':
        logDebug('userInput / processChosenHeading', `selected top of note, rather than a heading`);
        headingToReturn = '<<top of note>>'; // hopefully won't ever be used as an actual title!
        break;
      case '\u23ec (bottom of note)':
        logDebug('userInput / processChosenHeading', `selected end of note, rather than a heading`);
        headingToReturn = '<<bottom of note>>';
        break;
    }
    return headingToReturn;
  }

  /*----------------------------------------------------------------------------------------------------------------------------
   * Configuration Utilities
   * @author @codedungeon unless otherwise noted
   * Requires NotePlan 3.4 or greater (waiting for NotePlan.environment version method to perform proper validation)
   * --------------------------------------------------------------------------------------------------------------------------*/
  async function getPluginJson(pluginId = '') {
    return await DataStore.loadJSON(`../../${pluginId}/plugin.json`);
  }

  const pluginJson$1 = 'helpers/NPSettings';
  function getSettingsFromPluginJson(pluginJson) {
    return pluginJson['plugin.settings'];
  }

  /**
   * Get options array (label/value pairs for popup) for settings
   * @param {*} settingsArray - the [plugin.settings] array from plugin.json
   * @param {boolean} includeHidden - whether to include hidden settings (default is false)
   * @returns {*} settings array
   */
  function getSettingsOptions(settingsArray, includeHidden = false) {
    let settings = settingsArray.filter(setting => Boolean(setting.type));
    settings = settings.filter(setting => setting.type !== 'separator');
    settings = includeHidden ? settings : settings.filter(setting => setting.type !== 'hidden');
    return settings.map(setting => {
      if (setting.type === 'heading') {
        return {
          label: setting.title.toUpperCase(),
          value: '---'
        };
      }
      return {
        label: `    ${setting.title}`,
        value: setting.key
      };
    });
  }

  /**
   * hidden type - this should not ever be called because of the filter
   * @param {*} setting object
   * @param {*} currentValue
   * @returns {string}
   */
  // eslint-disable-next-line no-unused-vars
  function updateSettingType_hidden(setting, currentValue) {
    return currentValue;
  }

  /**
   * bool type
   * @param {*} setting object
   * @param {*} currentValue
   * @returns {boolean}
   */
  async function updateSettingType_bool(setting, currentValue) {
    const newVal = await showMessageYesNo(`"${setting.title}" is currently: '${currentValue}'. Set '${setting.title}' to:`, ['False', 'True']);
    return newVal === 'True';
  }

  /**
   * number type
   * @param {*} setting object
   * @param {*} currentValue
   * @returns {number}
   */
  async function updateSettingType_number(setting, currentValue) {
    const newVal = await getInput(`${setting.title} is currently: '${currentValue}'. Set '${setting.title}' to:`, 'OK', setting.title, currentValue);
    return Number(newVal);
  }

  /**
   * single string type
   * @param {*} setting object
   * @param {*} currentValue
   * @returns {string}
   */
  async function updateSettingType_string(setting, currentValue) {
    const newVal = await getInput(`Enter a value for '${setting.title}'`, 'OK', `${setting.title}`, currentValue);
    if (typeof newVal === 'string') {
      logDebug(pluginJson$1, `updateSettingType_string: newValue: ${newVal}`);
      return newVal;
    } else {
      return currentValue;
    }
  }

  /**
   * [string] type
   * @param {*} setting object
   * @param {*} currentValue
   * @returns {Array<string>}
   */
  async function updateSettingType__string_(setting, currentValue) {
    const newVal = await getInput(`Enter a value for '${setting.title}' as an array of strings separated by commas`, 'OK', `${setting.title}`, currentValue);
    if (typeof newVal === 'string') {
      logDebug(pluginJson$1, `updateSettingType_[string]: newValue: ${newVal}`);
      return newVal.split(',').map(item => item.trim());
    } else {
      return currentValue;
    }
  }

  /**
   * string CHOICES type
   * @param {*} setting object
   * @param {*} currentValue
   * @returns {string}
   */
  async function updateSettingType_stringChoices(setting, currentValue) {
    const choices = setting.choices.map(choice => ({
      label: choice,
      value: choice.trim()
    }));
    const newVal = await chooseOption(`Choose a value for '${setting.title}'`, choices, currentValue);
    if (typeof newVal === 'string') {
      logDebug(pluginJson$1, `updateSettingType_string: newValue: ${newVal}`);
      return newVal;
    } else {
      return currentValue;
    }
  }

  /**
   * date type
   * @param {*} setting
   */
  async function updateSettingType_date(setting, currentValue) {
    const newVal = await getInput(`Enter a value for '${setting.title}' in the form YYYY-MM-DD`, 'OK', `${setting.title}`, currentValue);
    if (typeof newVal === 'string') {
      logDebug(pluginJson$1, `updateSettingType_date: newValue: ${newVal}`);
      return moment$1(newVal).toDate();
    } else {
      return null;
    }
  }

  /**
   * json type
   * @param {*} setting
   */
  async function updateSettingType_json(setting, currentValue) {
    const newVal = await getInput(`Enter a value for '${setting.title}'. We know, this small box is awful for writing JSON. So write it somewhere else and paste it here.`, 'OK', `${setting.title}`, currentValue);
    if (typeof newVal === 'string') {
      logDebug(pluginJson$1, `updateSettingType_string: newValue: ${newVal}`);
      return JSON.parse(newVal);
    } else {
      return currentValue;
    }
  }

  /**
   * all types
   */
  const getNewValue = {
    updateSettingType_string,
    updateSettingType__string_,
    updateSettingType_number,
    updateSettingType_bool,
    updateSettingType_hidden,
    updateSettingType_date,
    updateSettingType_json,
    updateSettingType_stringChoices
  };

  /**
   * After a user has selected a key to update, call the appropriate update function based on key name
   * @param {string} key - the key of the setting to update
   * @param {*} pluginJson - the current settings for this plugin (JSON)
   * @returns {boolean} success?
   */
  async function updateSetting(key, pluginJson) {
    const settings = getSettingsFromPluginJson(pluginJson);
    const setting = settings.find(setting => setting.key === key);
    const currentSettings = DataStore.settings;
    if (setting) {
      logDebug(pluginJson, `updateSetting: ${key}`);
      try {
        const cleanType = setting.type.replace(/-/g, '_').replace(/\[|\]/g, '_');
        const updateFunction = `updateSettingType_${cleanType}${setting.choices ? 'Choices' : ''}`;
        if (typeof getNewValue[updateFunction] === 'undefined') {
          throw `updateSetting: function name ${updateFunction} not specified`;
        }
        if (setting.description) {
          await showMessage(`${setting.description}`, 'Continue', `About the setting: "${setting.title}"`);
        }
        // call the specific updater function for the setting type
        const newVal = await getNewValue[updateFunction](setting, currentSettings[key]);
        // note: can't if (newVal) because newVal could be false
        if (newVal !== undefined && newVal !== null && newVal !== currentSettings[key]) {
          const settings = DataStore.settings;
          settings[key] = newVal;
          DataStore.settings = settings;
          return true;
        }
      } catch (error) {
        logError(pluginJson, `updateSetting: ${key} ${JSP(error)}`);
      }
    } else {
      logError(pluginJson, `No setting found with key ${key}`);
    }
    return false;
  }

  /**
   * Choose from all commands in plugin.json to interactively edit a setting (for plugin settings changes on iOS)
   * Can be run directly as a jsFunction or called from another function.
   * @param {any} _pluginJson - the whole plugin json object (optional. if not provided, will be fetched from pluginID in DataStore.settings)
   * @returns {number} - number of edits made
   * @example 3 steps below. Change <plugin_name> and <plugin_id>:
   * Put in index.js: export { editSettings } from '@helpers/NPSettings'
   * Put in plugin.json/commands:     {
        "name": "<plugin_name>: Update Plugin Settings",
        "description": "Preferences",
        "jsFunction": "editSettings"
      },
   * Put in plugin.json/settings:     {
      {
        "type": "hidden",
        "key": "plugin_ID",
        "default": "<pluginid>"
      },
   */
  async function editSettings(_pluginJson) {
    const {
      pluginID
    } = DataStore.settings;
    const pluginJson = _pluginJson || (await getPluginJson(pluginID));
    if (!pluginJson) throw 'editSettings: no pluginJson or pluginID found. It needs to be passed in or set in DataStore.settings.pluginID';
    // clo(pluginJson, 'editSettings: plugin.json:')
    // clo(DataStore.settings, 'editSettings: starting settings:')
    const settings = getSettingsFromPluginJson(pluginJson);
    let editsMade = 0;
    if (settings && settings.length) {
      let chosenSetting;
      const settingsOptions = await getSettingsOptions(settings);
      while (chosenSetting !== '__done__') {
        logDebug('editSettings', `editSettings: top of while loop: editsMade=${editsMade}`);
        const msg = `Choose a${editsMade > 0 ? 'nother' : ''} setting to edit`;
        logDebug('editSettings', `editSettings: msg="${msg}"`);
        chosenSetting = await chooseOption(msg, settingsOptions);
        logDebug('editSettings', `editSettings: after chooseOption: chosenSetting=${chosenSetting}`);
        if (chosenSetting && chosenSetting !== '__done__' && chosenSetting !== '---') {
          logDebug('editSettings', `editSettings: chosenSetting: "${chosenSetting}"`);
          await updateSetting(chosenSetting, pluginJson);
          logDebug('editSettings', `editSettings: updated: "${chosenSetting}"`);
        }
        if (editsMade === 0) {
          settingsOptions.unshift({
            label: '[ ✅ Finished Editing Settings ]',
            value: '__done__'
          });
        }
        editsMade++;
      }
      clo(DataStore.settings, 'editSettings: final settings:');
    } else {
      logError(pluginJson, 'NPSettings/editSettings(): No settings array found');
    }
    return editsMade;
  }

  /**
   * Append a human readable number of days to/from the date given in the bullet points in a list
   */
  async function countAndAddDays() {
    const note = Editor.note;
    if (!note) {
      await showMessage('No Note Open');
      return;
    }

    //   let insertedCharacters = 0
    let matches = 0;
    const paragraphs = note.paragraphs;
    for (let para of paragraphs) {
      if (para.type !== 'list') {
        continue;
      }
      if (!para.content.trim().match(/\[\[[0-9]{4}\-[0-9]{2}\-[0-9]{2}\]\]:$/)) {
        continue;
      }
      const match = para.content.trim().match(/\[\[([0-9]{4})\-([0-9]{2})\-([0-9]{2})\]\]:$/);
      if (!match) {
        continue;
      }
      const [_, year, month, date] = match;
      const days = daysUntil(parseInt(year, 10), parseInt(month, 10), parseInt(date, 10));
      const insertAtIndex = para.contentRange?.end;
      if (insertAtIndex == null) {
        continue;
      }
      matches++;
      const stringToInsert = days > 0 ? ` **${days}** days to go!` : days < 0 ? ` **${-days}** ago!` : ` **today!**`;
      para.content = para.content + stringToInsert;
      // Editor.insertTextAtCharacterIndex(` **${days}** days to go!`, insertAtIndex + insertedCharacters)
    }
    note.paragraphs = paragraphs;
    await showMessage(`Added ${matches} dates`);
  }

  // function to get msSinceEpoch of given date
  function getMsSinceEpoch(year, month, date) {
    const dateObj = new Date(year, month - 1, date);
    return dateObj.getTime();
  }

  // function to get the year, month, and date of today
  function getToday() {
    const dateObj = new Date();
    return [dateObj.getFullYear(), dateObj.getMonth() + 1, dateObj.getDate()];
  }

  // function to count days until given date
  function daysUntil(year, month, date) {
    const today = getMsSinceEpoch(...getToday());
    const target = getMsSinceEpoch(year, month, date);
    const diff = target - today;
    return Math.ceil(diff / (1000 * 60 * 60 * 24));
  }

  //-----------------------------------------------------------------------------
  // Note Helpers plugin for NotePlan
  // Jonathan Clark & Eduard Metzger
  // Last updated 2024-12-26 for v1.0.0 by @jgclark
  //-----------------------------------------------------------------------------
  //-----------------------------------------------------------------
  // Settings

  const pluginID$1 = 'jgclark.NoteHelpers';
  /**
   * Get config settings
   * @author @jgclark
   */
  async function getSettings() {
    try {
      // Get settings
      const config = await DataStore.loadJSON(`../${pluginID$1}/settings.json`);
      if (config == null || Object.keys(config).length === 0) {
        await showMessage(`Cannot find settings for the 'NoteHelpers' plugin. Please make sure you have installed it from the Plugin Preferences pane.`);
        return;
      } else {
        // clo(config, `settings`)
        return config;
      }
    } catch (err) {
      logError(pluginJson$4, JSP(err));
      await showMessage(err.message);
    }
  }

  //-----------------------------------------------------------------

  /**
   * Command from Eduard to move a note to a different folder
   * @author @eduardme
   */
  async function logEditorNoteDetailed() {
    try {
      if (!Editor || !Editor.note) {
        // No note open, so don't do anything.
        logError('logEditorNoteDetailed()', 'No note open. Stopping.');
        return;
      }
      logDebug('logEditorNoteDetailed()', `Editor: ${Editor.filename})`);
      printNote(Editor.note, true);
    } catch (err) {
      logError(pluginJson$4, `${err.name}: ${err.message}`);
      await showMessage(err.message);
    }
  }

  /**
   * Command from Eduard to move a note to a different folder
   * @author @eduardme
   */
  async function moveNote() {
    try {
      const {
        title,
        filename,
        type
      } = Editor;
      if (title == null || filename == null) {
        // No note open, so don't do anything.
        logError('moveNote()', `No note open. Stopping.`);
        const res = await showMessage(`No note open. Stopping.`);
        return;
      }
      if (type === 'Calendar') {
        // Can't move Calendar notes
        logError('moveNote()', `Can't move a calendar note. Stopping.`);
        const res = await showMessage(`NotePlan doesn't allow moving calendar notes.`);
        return;
      }
      const selectedFolder = await chooseFolder(`Select a folder for '${title}'`, true, true); // include @Archive as an option, and to create a new folder
      logDebug('moveNote()', `move ${title} (filename = ${filename}) to ${selectedFolder}`);
      const newFilename = DataStore.moveNote(filename, selectedFolder);
      if (newFilename != null) {
        await Editor.openNoteByFilename(newFilename);
      } else {
        logError('moveNote()', `Error trying to move note`);
      }
    } catch (err) {
      logError(pluginJson$4, `${err.name}: ${err.message}`);
      await showMessage(err.message);
    }
  }

  //-----------------------------------------------------------------
  /**
   * Delete a note -- by moving to the special @Trash folder
   * @author @jgclark
   */
  async function trashNote() {
    try {
      const {
        title,
        filename
      } = Editor;
      if (title == null || filename == null) {
        // No note open, so don't do anything.
        logError('trashNote()', 'No note open. Stopping.');
        return;
      }
      const newFilename = DataStore.moveNote(filename, '@Trash');
      if (!newFilename) {
        logError('trashNote()', `Error trying to move note to @Trash`);
      }
    } catch (err) {
      logError(pluginJson$4, `${err.name}: ${err.message}`);
      await showMessage(err.message);
    }
  }

  //-----------------------------------------------------------------

  /**
   * Add trigger to the currently open Editor note, with choice offered to user of which trigger to add (if param not given).
   * It decides which are trigger-related functions by:
   * - if plugin command name is on TRIGGER_LIST
   * - if plugin command description is on TRIGGER_LIST
   * - if either contains string 'trigger'
   * @author @jgclark
   * @param {string?} triggerStringArg optional full trigger string, e.g. onEditorWillSave => jgclark.DashboardReact.decideWhetherToUpdateDashboard
   */
  async function addTriggerToNote(triggerStringArg = '') {
    try {
      if (!Editor || !Editor.note) {
        throw new Error("No Editor open, so can't continue");
      }
      logDebug(pluginJson$4, `addTriggerToNote('${triggerStringArg}') starting ...`);

      // Get list of available triggers from looking at installed plugins
      const allVisibleCommands = [];
      const triggerRelatedCommands = [];
      const triggerRelatedStrings = [];
      const installedPlugins = DataStore.installedPlugins();
      // logDebug('addTriggerToNote', "Found following trigger-related plugin commands:")
      for (const p of installedPlugins) {
        for (const pluginCommand of p.commands) {
          // Only include if this command name or description is a trigger type or contains the string 'trigger' (excluding this one!)
          if ((pluginCommand.desc.includes("trigger") || pluginCommand.name.includes("trigger") || TRIGGER_LIST.includes(pluginCommand.name) || TRIGGER_LIST.includes(pluginCommand.desc)) && pluginCommand.name !== "add trigger to note") {
            triggerRelatedCommands.push(pluginCommand);
            const thisTriggerString = `${pluginCommand.name} => ${p.id}.${pluginCommand.name}`;
            // logDebug('addTriggerToNote', `- ${thisTriggerString}`)
            triggerRelatedStrings.push(thisTriggerString);
          }
          if (!pluginCommand.hidden) {
            allVisibleCommands.push(pluginCommand);
          }
        }
      }
      // clo(triggerRelatedCommands, 'triggerRelatedCommands')

      let triggerName = '';
      let triggerPluginID = '';
      let funcName = '';
      if (triggerStringArg !== '') {
        // if (!TRIGGER_LIST.includes(triggerName)) {
        if (!triggerRelatedStrings.includes(triggerStringArg)) {
          logInfo('addTriggerToNote', `Trigger '${triggerStringArg}' not found in the list of triggers I can identify, but will still add it.`);
        }

        // Add to note
        // Note: using Editor, not Editor.note, in case this is used in a Template
        // V1 Note: this can make duplicate frontmatter, as it calls ensureFrontmatter()
        // await convertNoteToFrontmatter(Editor, `triggers: ${triggerStringArg}`)

        // V2 trying to be smarter. Note: setFrontMatterVars also calls ensureFrontmatter() :-(
        const hasFMalready = noteHasFrontMatter(Editor);
        if (hasFMalready) {
          logDebug('addTriggerToNote', `- Editor "${displayTitle$1(Editor)}" already has frontmatter`);
          const res = setFrontMatterVars(Editor, {
            "triggers": triggerStringArg
          });
          logDebug('addTriggerToNote', `- result of setFrontMatterVars = ${String(res)}`);
        } else {
          logDebug('addTriggerToNote', `- Editor "${displayTitle$1(Editor)}" doesn't already have frontmatter`);
          await convertNoteToFrontmatter(Editor, `triggers: ${triggerStringArg}`);
        }
        return;
      } else {
        // Ask user to select one. Examples:
        // let trigger = "onEditorWillSave"
        // let pluginID = "jgclark.RepeatExtensions"
        // let commandName = "generate repeats"
        if (triggerRelatedCommands.length === 0) {
          throw new Error("No triggers are supported in your installed plugins.");
        }
        let commandOptions = triggerRelatedCommands.map(pco => {
          return {
            label: `${pco.pluginName} '${pco.name}'`,
            value: pco
          };
        });
        commandOptions.push({
          label: `Pick from whole list of functions ...`,
          value: "pickFromAll"
        });
        const result = await chooseOption("Pick the trigger to add", commandOptions);
        let choice;
        // If user has chosen pick from whole list, then show that full list and get selection
        if (typeof result === "string" && result === "pickFromAll") {
          commandOptions = allVisibleCommands.map(pco => {
            return {
              label: `${pco.pluginName} '${pco.name}'`,
              value: pco
            };
          });
          choice = await chooseOption("Pick the trigger to add", commandOptions);
        } else if (typeof result !== "string") {
          choice = result; // this check appeases flow from here on
        }
        if (!choice) {
          throw new Error("Couldn't get a valid trigger choice for some reason. Stopping.");
        } else {
          // clo(choice, 'choice')
          // Get trigger type from either name or description
          triggerName = TRIGGER_LIST.includes(choice.name) ? choice.name : TRIGGER_LIST.includes(choice.desc) ? choice.desc : 'onEditorWillSave'; // default to onEditorWillSave if no trigger type is found
          triggerPluginID = choice.pluginID;
          funcName = choice.name;
        }
      }

      // Add trigger to note
      const res = await addTrigger(Editor, triggerName, triggerPluginID, funcName);
      if (res) {
        logDebug('addTriggerToNote', `Trigger ${triggerName} for ${triggerPluginID} func ${funcName} was added to note ${displayTitle$1(Editor)}`);
      } else {
        logError('addTriggerToNote', `Trigger ${triggerName} for ${triggerPluginID} func ${funcName} WASN'T added to note ${displayTitle$1(Editor)}`);
      }
    } catch (err) {
      logError(pluginJson$4, err.message);
      await showMessage(err.message);
    }
  }

  /**
   * Converts all links that start with a `#` symbol, i.e links to headings within a note,
   * to x-callback-urls that call the `jumpToHeading` plugin command to actually jump to that heading.
   * @author @nmn
   */
  function convertLocalLinksToPluginLinks() {
    const note = Editor;
    const paragraphs = note?.paragraphs;
    if (note == null || paragraphs == null) {
      // No note open, or no content
      return;
    }
    // Look for markdown links that are local to the note
    // and convert them to plugin links
    let changed = false;
    for (const para of paragraphs) {
      const content = para.content;
      const newContent = content.replace(/\[(.*?)\]\(\#(.*?)\)/g, (match, label, link) => {
        const newLink = `noteplan://x-callback-url/runPlugin?pluginID=jgclark.NoteHelpers&command=jump%20to%20heading&arg1=${encodeURIComponent(link)}`;
        return `[${label}](${newLink})`;
      });
      if (newContent !== content) {
        para.content = newContent;
        changed = true;
      }
    }
    if (changed) {
      // Force update the note
      note.paragraphs = paragraphs;
    }
  }

  /**
   * Rename the currently open note's file on disk
   * NB: Only available from v3.6.1 build 826+
   * @author @jgclark
   */
  async function renameNoteFile() {
    try {
      const {
        note
      } = Editor;
      // Check for version less than v3.6.1 (828)
      const bvNumber = NotePlan.environment.buildVersion;
      if (bvNumber < 826) {
        logError('renameNoteFile()', 'Will only work on NotePlan v3.6.1 or greater. Stopping.');
        return;
      }
      if (note == null || note.paragraphs.length < 1) {
        // No note open, so don't do anything.
        logError('renameNoteFile()', 'No note open, or no content. Stopping.');
        return;
      }
      if (Editor.type === 'Calendar') {
        // Won't work on calendar notes
        logError('renameNoteFile()', 'This will not work on Calendar notes. Stopping.');
        return;
      }
      const currentFullPath = note.filename;
      // let currentFilename = ''
      // let currentFolder = ''
      // if (currentFullPath.lastIndexOf('/') > -1) {
      //   currentFolder = res.substr(0, res.lastIndexOf('/'))
      //   currentFilename = res.substr(res.lastIndexOf('/') + 1)
      // } else {
      //   currentFolder = '/'
      //   currentFilename = res
      // }

      const requestedPath = await getInput(`Please enter new filename for file (including folder and file extension)`, 'OK', 'Rename file', currentFullPath);
      if (typeof requestedPath === 'string') {
        // let newFolder = ''
        // let newFilename = ''
        // if (requestedPath.lastIndexOf('/') > -1) {
        //   newFolder = requestedPath.substr(0, requestedPath.lastIndexOf('/'))
        //   newFilename = requestedPath.substr(requestedPath.lastIndexOf('/') + 1)
        // } else {
        //   newFolder = '/'
        //   newFilename = requestedPath
        // }
        // logDebug(pluginJson, `${newFolder}  /  ${newFilename}`)
        logDebug(pluginJson$4, `Requested new filepath = ${requestedPath}`);
        const newFilename = note.rename(requestedPath);
        logDebug('renameNoteFile()', `Note file renamed from '${currentFullPath}' to '${newFilename}'`);
      } else {
        // User cancelled operation
        logWarn('renameNoteFile()', `User cancelled operation`);
      }
    } catch (err) {
      logError(pluginJson$4, `${err.name}: ${err.message}`);
      await showMessage(err.message);
    }
  }

  /**
   * Convert the note to using frontmatter Syntax
   * If the plugin settings contains default frontmatter, this is added to the frontmatter.
   * @author @jgclark
   * @param {TNote} note
   */
  async function addFrontmatterToNote(note) {
    try {
      let thisNote;
      if (note == null) {
        if (Editor == null) {
          throw new Error(`No note open to convert.`);
        } else {
          thisNote = Editor;
        }
      } else {
        thisNote = note;
      }
      if (!thisNote) {
        throw new Error(`No note supplied, and can't find Editor either.`);
      }
      const config = await getSettings();
      const res = await convertNoteToFrontmatter(thisNote, config.defaultFMText ?? '');
      logDebug('note/convertNoteToFrontmatter', `ensureFrontmatter() returned ${String(res)}.`);
    } catch (error) {
      logError(pluginJson$4, JSP(error));
      await showMessage(error.message);
      return;
    }
  }

  //-----------------------------------------------------------------------------
  // Jonathan Clark
  // Last updated 30.3.2024 for v0.19.0+ by @jgclark
  //-----------------------------------------------------------------------------
  const pluginID = 'jgclark.NoteHelpers';

  //-----------------------------------------------------------------------------
  /**
   * Private function to generate the index of a specified folder, including
   * each note as a wikilink, with relative time since it was last updated.
   * @author @jgclark
   *
   * @param {string} folder - folder name (without trailling /)
   * @param {any} config - config object
   * @param {boolean} includeSubfolders?
   * @returns {Array<string>} array of strings, one for each output line
  */
  function makeFolderIndex(folder, config, includeSubfolders) {
    try {
      logDebug(pluginJson$4, `makeFolderIndex() starting for '${folder}', displayOrder:${config.displayOrder} / dateDisplayType:${config.dateDisplayType} / ${config.includeSubfolders ? 'with' : 'without'} subfolders`);
      const outputArray = [];
      let folderList = [];
      // if we want a to include any subfolders, create list of folders
      if (config.includeSubfolders) {
        folderList = DataStore.folders.filter(f => f.startsWith(folder));
      } else {
        // otherwise use a single folder
        folderList = [folder];
      }
      logDebug('makeFolderIndex', `- Found ${folderList.length} matching folder(s)`);

      // Prepare output items we need just once in the output
      const sortExplainer = config.displayOrder === "updatedDate" ? "Sorted by most recently updated date" : config.displayOrder === "createdDate" ? "Sorted by most recently created date" : "Sorted by title"; // setting value "alphabetical"
      // const dateDisplayExplainer = (config.dateDisplayType === "timeSince")
      const dateExplainer = config.dateDisplayType === "updatedDate" ? "Dates are when note was last updated." : config.dateDisplayType === "timeSince" ? "Times are since note was last updated." : ""; // setting value "none"
      // const paramsForXCB: Array<string> = [folder, config.displayOrder, config.dateDisplayType, String(includeSubfolders)]
      const argsForXCB = `displayOrder=${config.displayOrder};dateDisplayType=${config.dateDisplayType};includeSubfolders=${String(includeSubfolders)}`;
      const paramsForXCB = [folder, argsForXCB];
      const refreshXCBStr = createPrettyRunPluginLink('🔄 Refresh', pluginID, 'index folders', paramsForXCB);

      // Iterate over any sub-folders
      let isSubFolder = false;
      for (const f of folderList) {
        // Get list of the notes in this folder, but ignore any '_index' notes :-)
        const lastPartOfFolderName = f.split('/').slice(-1)[0];
        const outputTitle = config.indexTitle.replace('{{full_folder_path}}', f).replace('{{folder}}', lastPartOfFolderName);
        let notes = notesInFolderSortedByTitle(f).filter(n => n.title !== outputTitle);
        // logDebug('makeFolderIndex', `- Found ${notes.length} notes in '${f}' before '${config.displayOrder}' sort`)

        // Sort this list by whatever the user's setting says
        // (Need to do this before the gatherMatchingLines, as afterwards we don't have date information.)
        switch (config.displayOrder) {
          case 'updatedDate':
            notes = notes.sort((a, b) => a.changedDate > b.changedDate ? -1 : 1);
            break;
          case 'createdDate':
            // though data is very unreliable at least from NP 3.0.23 to 3.8.0
            notes = notes.sort((a, b) => a.createdDate > b.createdDate ? -1 : 1);
            break;
          default:
            // alphabetical
            notes = notes.sort((a, b) => displayTitle$1(a).toUpperCase() < displayTitle$1(b).toUpperCase() ? -1 : 1);
            break;
        }
        logDebug('makeFolderIndex', `- ${notes.length} notes after sort`);

        // If this is a sub-folder level, then prefix with ### for a 3rd level heading,
        // otherwise leave blank, as a suitable header gets added elsewhere.
        if (isSubFolder) {
          // const folderNameWithoutFirstPart = f.split('/').slice(1).join('/')
          // outputArray.push(`### ${folderNameWithoutFirstPart} (${notes.length})`)
          const lastPartOfFolderName = f.split('/').slice(-1)[0];
          const folderLevel = f.split('/').length;
          outputArray.push(`${'#'.repeat(folderLevel)} ${lastPartOfFolderName} (${notes.length})`);
        } else {
          outputArray.push(outputTitle);
          outputArray.push(`Generated ${nowLocaleShortDateTime()} ${refreshXCBStr}\n${sortExplainer}. ${dateExplainer}`);
        }

        // Add suffix, if wanted
        if (notes.length > 0) {
          // outputArray.push(`${notes.length} notes`)
          // iterate over this folder's notes
          for (const note of notes) {
            // add type of date suffix (if wanted)
            const dateSuffix = config.dateDisplayType === "updatedDate" ? '\t' + toNPLocaleDateString(note.changedDate) : config.dateDisplayType === "timeSince" ? '\t' + relativeDateFromNumber(daysBetween(new Date(), note.changedDate)) : '';
            outputArray.push(`- ${returnNoteLink(note.title ?? 'error')}${dateSuffix}`);
          }
          outputArray.push('');
        } else {
          if (isSubFolder) {
            outputArray.push('_No notes found_');
          }
        }
        isSubFolder = true;
      }
      return outputArray;
    } catch (err) {
      logError(pluginJson$4, JSP(err));
      return ['error running makeFolderIndex']; // for completeness
    }
  }

  /**
   * Command to index folders, creating list of notes.
   * Called by user directly, or via x-callback call.
   * Options:
   * 1. This folder only (insert into current note)
   * 2. This folder only (add/update to _index note)
   * 3. This folder + subfolders (add/update into single _index note)
   * 4. This folder + subfolders (add/update into _index notes in each subfolder)
   * @author @jgclark
   * @param {string?} folder - folder name (without trailling /). If empty, folder of current Editor's note is used
   * @param {string?} args - (optional) other arguments, as semicolon-separated set of key=value. Possible keys:
   * - displayOrder - sort order for index items ('updatedDate'/'createdDate'/'alphabetical')
   * - dateDisplayType - what type of date suffix to add ('none'/'timeSince'/'updateDate')
   * - includeSubfolders? optional 'true'/'false', defaults to whatever the user's settings say.
   */
  async function indexFolders(folder = "", args = '') {
    try {
      let folderToUse = '';
      // let fullFilename = ''

      // Use parameters if passed, otherwise fallback to the settings
      // v2 method
      let config = await getSettings();
      config = overrideSettingsWithStringArgs(config, args);
      logDebug(pluginJson$4, `indexFolders() starting with displayOrder:${config.displayOrder} / dateDisplayType:${config.dateDisplayType} / includeSubfolders ? ${config.includeSubfolders}`);

      // Get folder from param, falling back to current note's folder
      if (folder) {
        folderToUse = folder;
      } else {
        // logDebug('indexFolders', Editor.filename)
        logDebug('indexFolders', NotePlan.selectedSidebarFolder);
        folderToUse = Editor.filename ? getFolderFromFilename(Editor.filename) : NotePlan.selectedSidebarFolder ? NotePlan.selectedSidebarFolder : null;
        logDebug('indexFolders', `folderToUse: ${folderToUse ?? '(undefined still)'}`);
        if (Editor.type === 'Calendar' || folderToUse === undefined) {
          logDebug('indexFolders', `Info: No valid current filename (or folder) found, so will ask instead.`);
          folderToUse = await chooseFolder(`Please pick folder to index`, true, true); // include @Archive as an option, and to create a new folder
        }
      }
      if (!folderToUse) {
        throw new Error(`Could not find folderToUse for some reason`);
      }
      // logDebug('indexFolders', `- values to use: folder:'${folderToUse}' / displayOrderToUse:${config.displayOrder} / dateDisplayTypeToUse:${config.dateDisplayType} / ${config.includeSubfolders ? 'with' : 'without'} subfolders`)

      // If we've been called by x-callback then output will be to relevant folder's Index file.
      let option;
      if (folder) {
        option = config.includeSubfolders ? 'all-to-one-index' : 'one-to-index';
      } else {
        option = await chooseOption('Create index for which folder(s)?', [{
          label: `🖊 This folder only (add/update to Index note)`,
          value: 'one-to-index'
        }, {
          label: `🖊 This folder and sub-folders (add/update to single Index note)`,
          value: 'all-to-one-index'
        }, {
          label: `🖊 This folder only (insert into current note)`,
          value: 'one-to-current'
        }, {
          label: `📋 This folder only (to console log)`,
          value: 'one-to-log'
        }, {
          label: '❌ Cancel',
          value: false
        }], false);
      }
      if (!option) {
        // Cancel selected
        return;
      }
      // logDebug('indexFolders', `- option: ${option}`)

      // Start constructing output
      let outputArray = [];
      if (option.startsWith('one')) {
        outputArray = makeFolderIndex(folderToUse, config, false);
      } else if (option.startsWith('all')) {
        outputArray = makeFolderIndex(folderToUse, config, true);
      }
      const outString = outputArray.join('\n');
      if (option.endsWith('index')) {
        // write out to Index file(s)
        let outputFilename = `${folderToUse}/_index.${DataStore.defaultFileExtension}`;
        // see if we already have an _index file in this folder
        let outputNote = DataStore.projectNoteByFilename(outputFilename);
        if (outputNote == null) {
          // make a new note for this
          outputFilename = (await DataStore.newNote('_index', folderToUse)) ?? '';
          logDebug('indexFolders', `- newNote filename: ${String(outputFilename)}`);
          // outputFilename = `${pref_folderToStore}/${String(outputFilename)}` ?? '(error)'
          // NB: filename here = folder + filename
          if (outputFilename === '') {
            throw new Error(`couldn't make a new note in folder ${folderToUse}' for some reason. Stopping.`);
          }
          logInfo('indexFolders', `Writing index to new note '${outputFilename}'`);
          const options = {
            newWindow: false,
            splitView: true,
            content: `# ${outString}`,
            highlightStart: 0,
            highlightEnd: 0
          };
          outputNote = await openNoteByFilename(outputFilename, options);
        } else {
          logInfo('indexFolders', `Writing index to note '${outputFilename}'`);
        }
        // fresh test to see if we now have the note
        if (outputNote != null) {
          outputNote.content = `# ${outString}`; // overwrite what was there before
          // Note: this setter doesn't seem to be enough in some cases?
        } else {
          throw new Error(`error after newNote(): no valid note ${outputFilename} to write to`);
        }
      } else if (option.endsWith('current')) {
        // write out to the current file
        Editor.insertTextAtCursor(`${outString}`);
      } else {
        // write out to the log
        logDebug('indexFolders', `Output:\n${outString}`);
      }
      logDebug('indexFolders', `Finished indexFolders.`);
    } catch (err) {
      logError('indexFolders', err.message);
    }
  }

  /**
   * Command to update all existing index notes for folders.
   */
  async function updateAllIndexes() {
    try {
      // let config: noteHelpersConfigType = await getSettings()

      // Find all existing index Notes
      const allProjectNotesToCheck = projectNotesFromFilteredFolders([], true);
      const indexNotes = allProjectNotesToCheck.filter(n => n.filename.endsWith(`_index.${DataStore.defaultFileExtension}`));
      logDebug('updateAllIndexes', `Will update .index files in [${indexNotes.length}] folders ...`);

      // Update each in turn
      for (const indexNote of indexNotes) {
        const thisFolder = getFolderFromFilename(indexNote.filename);
        logDebug('updateAllIndexes', `Recreating .index for folder [${thisFolder}]`);
        await indexFolders(thisFolder);
      }
      return;
    } catch (err) {
      logError('indexFolders', JSP(err));
    }
  }

  /**
   * Takes a Note returns a new path for a note.
   * Now copes with notes that use front matter, and substituting for '/' characters in filename (otherwise treated as a new folder in path)
   * @author @Leo, extended by @jgclark
   * @param {note: TNote} note 
   * @returns {string} filepath
   */
  function newFilepathForNote(note) {
    const {
      defaultFileExtension
    } = DataStore;

    // Get new title for note, though with any '/' or ':' replaced
    const title = displayTitle$1(note).replace('/', '_').replace(':', '_');
    if (title !== '') {
      const currentFullPath = note.filename;
      const pathWithoutTitle = getFolderFromFilename(currentFullPath);
      const newName = [pathWithoutTitle, `${title}.${defaultFileExtension}`].filter(Boolean).join('/');
      return newName;
    } else {
      logWarn(pluginJson$4, `newFilepathForNote(): No title found in note ${note.filename}. Returning empty string.`);
      return '';
    }
  }

  // import pluginJson from '../../plugin.json'
  function findInconsistentNames(folder = '') {
    const {
      projectNotes
    } = DataStore;

    // Note: there's a faster way to do this if folder is given ... (which it currently never is)
    return projectNotes.filter(note => {
      const currentFullPath = note.filename;
      if (currentFullPath.substring(0, 1) === '@') {
        // Ignore Notes in reserved folders
        return false;
      }

      // If a folder is specified, only check notes in that folder, ignored if '/' is specified
      if (folder.length > 0 && folder !== '/') {
        // Only check notes in the specified folder
        if (currentFullPath.indexOf(folder) !== 0) {
          // logDebug(pluginJson, `findInconsistentNames(): Ignoring note ${currentFullPath} as not in specified folder ${folder}`)
          return false;
        }
      }
      const newPath = newFilepathForNote(note);
      return currentFullPath !== newPath;
    }).sort();
  }

  // ----------------------------------------------------------------------------
  // Helpers for window management
  // See also HTMLView for specifics of working in HTML
  // ----------------------------------------------------------------------------

  /**
   * Return string version of Rect's x/y/width/height attributes
   * @param {Rect} rect
   * @returns {string}
   */
  function rectToString(rect) {
    return `X${String(rect.x)},Y${String(rect.y)},w${String(rect.width)},h${String(rect.height)}`;
  }

  /**
   * Search open HTML windows and return the window object that matches a given customId (if available).
   * Matches are case-insensitive, and either an exact match or a starts-with-match.
   * @param {string} customId - to look for
   * @returns {string} the matching open HTML window's ID or false if not found
   */
  function getWindowIdFromCustomId(customId) {
    if (NotePlan.environment.platform !== 'macOS') {
      logDebug('isHTMLWindowOpen', `Platform is ${NotePlan.environment.platform}`);
      // return false
    }
    const allHTMLWindows = NotePlan.htmlWindows;
    // clo(allHTMLWindows, 'getWindowIdFromCustomId: allHTMLWindows')
    for (const thisWin of allHTMLWindows) {
      // clo(thisWin, `getWindowIdFromCustomId(): thisWin=`)
      if (caseInsensitiveMatch(customId, thisWin.customId) || caseInsensitiveStartsWith(customId, thisWin.customId)) {
        thisWin.customId = customId;
        return thisWin.id;
      }
    }
    logDebug('isHTMLWindowOpen', `Did not find open window with ID:"${customId}" on platform:"${NotePlan.environment.platform}". This is ok if the window is not open or the platform is not macOS.`);
    return false;
  }

  /**
   * Is a given HTML window open? Matches are case-insensitive, and either an exact match or a starts-with-match on the supplied customId.
   * @author @jgclark
   * @param {string} customId to look for
   * @returns {boolean}
   */
  function isHTMLWindowOpen(customId) {
    return !!getWindowIdFromCustomId(customId);
  }
  function getWindowFromCustomId(windowCustomId) {
    // First loop over all Editor windows
    const allEditorWindows = NotePlan.editors;
    for (const thisWindow of allEditorWindows) {
      if (thisWindow.customId === windowCustomId) {
        return thisWindow;
      }
    }
    // And if not found so far, then all HTML windows
    const allHTMLWindows = NotePlan.htmlWindows;
    for (const thisWindow of allHTMLWindows) {
      if (thisWindow.customId === windowCustomId) {
        return thisWindow;
      }
    }
    logWarn('getWindowFromCustomId', `Couldn't find window matching customId '${windowCustomId}'`);
    return false;
  }

  /**
   * Save the Rect (x/y/w/h) of the given window, given by its ID, to the local device's NP preferences store.
   * @param {string} customId
   */
  function storeWindowRect(customId) {
    if (NotePlan.environment.buildVersion < 1020) {
      logDebug('storeWindowRect', `Cannot save window rect as not running v3.9.1 or later.`);
      return;
    }
    // Find the window by its customId
    const thisWindow = getWindowFromCustomId(customId);
    if (thisWindow) {
      // Get its Rect from the live window
      const windowRect = thisWindow.windowRect;
      const prefName = `WinRect_${customId}`;
      DataStore.setPreference(prefName, windowRect);
      logDebug('storeWindowRect', `Saved Rect ${rectToString(windowRect)} to ${prefName}`);
    } else {
      logWarn('storeWindowRect', `Couldn't save Rect for '${customId}'`);
    }
  }

  /**
   * Get the Rect (x/y/w/h) of the given window, given by its ID, from the local device's NP preferences store
   * @param {string} customId
   * @returns {Rect} the Rect (x/y/w/h)
   */
  function getStoredWindowRect(customId) {
    try {
      if (NotePlan.environment.buildVersion < 1020) {
        logWarn('getWindowRect', `Cannot get window rect as not running v3.9.1 or later.`);
        return false;
      }
      const prefName = `WinRect_${customId}`;
      // $FlowFixMe[incompatible-type]
      const windowRect = DataStore.preference(prefName);
      if (!windowRect) {
        logWarn('getWindowRect', `Couldn't retrieve Rect from saved pref ${prefName}`);
        return false;
      }
      logDebug('getWindowRect', `Retrieved Rect ${rectToString(windowRect)} from saved ${prefName}`);
      return windowRect;
    } catch (error) {
      logError('getStoredWindowRect', error.message);
      return false;
    }
  }

  // ---------------------------------------------------------
  // HTML helper functions to create CSS from NP Themes
  // by @jgclark
  // Last updated 2024-12-27 by @jgclark
  // ---------------------------------------------------------

  // ---------------------------------------------------------
  // Constants and Types

  let userFSPref; // updated later
  let themeBodyFS; // updated later
  let baseFontSize; // updated later

  // ---------------------------------------------------------

  /**
   * Generate CSS as an equivalent to the given theme (or current one if not given, or 'dark' theme if that isn't available) to use as an embedded style sheet.
   * TODO: ideally consult theme to see if Editor's "shouldOverwriteFont" is false before changing font size and family?
   * 
   * @author @jgclark
   * @param {string?} themeNameIn (optional)
   * @returns {string} outputCSS
   */
  function generateCSSFromTheme(themeNameIn = '') {
    try {
      let themeName = '';
      let themeJSON;
      const availableThemeNames = Editor.availableThemes.map(m => m.name.endsWith('.json') ? m.name.slice(0, -5) : m.name);
      let matchingThemeObjs = []; // Eduard hasn't typed the Theme objects
      let currentThemeMode = 'light'; // default; overridden later

      // If we havee a supplied themeName, then attempt to use it
      if (themeNameIn) {
        // get list of available themes
        logDebug('generateCSSFromTheme', String(availableThemeNames));
        matchingThemeObjs = Editor.availableThemes.filter(f => f.name === themeNameIn);
        if (matchingThemeObjs.length > 0) {
          themeName = themeNameIn;
          logDebug('generateCSSFromTheme', `Reading theme '${themeName}'`);
          themeJSON = matchingThemeObjs[0].values;
          currentThemeMode = Editor.currentTheme.mode;
        } else {
          logWarn('generateCSSFromTheme', `Theme '${themeNameIn}' is not in list of available themes. Will try to use current theme instead.`);
        }
      }

      // If that hasn't worked, then currentTheme
      if (!themeName) {
        themeName = Editor.currentTheme.name ?? '';
        themeName = themeName.endsWith('.json') ? themeName.slice(0, -5) : themeName;
        logDebug('generateCSSFromTheme', `Translating your current theme '${themeName}'`);
        if (themeName !== '') {
          themeJSON = Editor.currentTheme.values;
          currentThemeMode = Editor.currentTheme.mode;
        } else {
          logWarn('generateCSSFromTheme', `Cannot get settings for your current theme '${themeName}'`);
        }
      }

      // If that hasn't worked, try dark theme
      if (!themeName) {
        themeName = String(DataStore.preference('themeDark'));
        themeName = themeName.endsWith('.json') ? themeName.slice(0, -5) : themeName;
        matchingThemeObjs = Editor.availableThemes.filter(f => f.name === themeName);
        if (matchingThemeObjs.length > 0) {
          logDebug('generateCSSFromTheme', `Reading your dark theme '${themeName}'`);
          themeJSON = matchingThemeObjs[0].values;
          currentThemeMode = 'dark';
        } else {
          logWarn('generateCSSFromTheme', `Cannot get settings for your dark theme '${themeName}'`);
        }
      }

      // Check we can proceed
      if (themeJSON == null || themeJSON.length === 0) {
        logError('generateCSSFromTheme', `themeJSON is empty. Stopping.`);
        return '';
      }

      //-----------------------------------------------------
      // Calculate the CSS properties for various selectors
      const output = [];
      let tempSel = [];
      const rootSel = []; // for special :root selector which sets variables picked up in several places below
      let styleObj;
      // const isLightTheme = themeJSON.style === 'Light' // used to be used

      // Set 'html':
      // - main font size (as global variable) from:
      //   - theme body, or
      //   - user's NP Editor setting
      //   - or default to 14
      userFSPref = Number(DataStore.preference('fontSize')) ?? NaN;
      themeBodyFS = Number(themeJSON?.styles?.body?.size) ?? 14;
      baseFontSize = userFSPref ?? themeBodyFS;
      logDebug('generateCSSFromTheme', `baseFontSize -> ${String(baseFontSize)}`);
      const bgMainColor = themeJSON?.editor?.backgroundColor ?? '#1D1E1F';
      tempSel.push(`background: var(--bg-main-color)`);
      tempSel.push(`font-size: ${baseFontSize}px`);
      output.push(makeCSSSelector('html', tempSel));
      rootSel.push(`--bg-main-color: ${bgMainColor}`);

      // Set sidebar from NP fixed colours
      if (currentThemeMode === 'light') {
        rootSel.push(`--fg-sidebar-color: #242E32`);
        rootSel.push(`--bg-sidebar-color: #F6F6F6`);
        rootSel.push(`--divider-color: #D6D6D6`);
      } else {
        rootSel.push(`--fg-sidebar-color: #EBEBEB`);
        rootSel.push(`--bg-sidebar-color: #383838`);
        rootSel.push(`--divider-color: #52535B`);
      }

      // Set body:
      // - main font = styles.body.font
      const tempBodyFont = themeJSON.styles.body.font ?? '-apple-system';
      const bodyFont = tempBodyFont === '.AppleSystemUIFont' ? '-apple-system' : tempBodyFont;
      logDebug('generateCSSFromTheme', `bodyFont: ${bodyFont}`);
      // - main foreground colour (styles.body.color)
      // - main background colour (editor.backgroundColor)
      tempSel = [];
      styleObj = themeJSON.styles.body;
      if (styleObj) {
        const thisColor = RGBColourConvert(themeJSON?.editor?.textColor ?? '#CC6666');
        tempSel.push(`color: var(--fg-main-color)`);
        tempSel = tempSel.concat(convertStyleObjectBlock(styleObj));
        tempSel.push(`font-size: ${baseFontSize}px`);
        output.push(makeCSSSelector('body, .body', tempSel));
        rootSel.push(`--fg-main-color: ${thisColor}`);
        if (styleObj?.lineSpacing) {
          // borrowed from convertStyleObjectBlock()
          const lineSpacingRem = (Number(styleObj?.lineSpacing) * 1.5).toPrecision(3); // some fudge factor seems to be needed
          rootSel.push(`--body-line-height: ${String(lineSpacingRem)}rem`);
        }
      }

      // Set H1 from styles.title1
      tempSel = [];
      styleObj = themeJSON.styles.title1;
      if (styleObj) {
        const thisColor = RGBColourConvert(themeJSON.styles.title1.color ?? '#CC6666');
        tempSel.push(`color: ${thisColor}`);
        const thisBackgroundColor = RGBColourConvert(themeJSON.styles.title1.backgroundColor ?? bgMainColor);
        tempSel.push(`background-color: ${thisBackgroundColor}`);
        tempSel = tempSel.concat(convertStyleObjectBlock(styleObj));
        output.push(makeCSSSelector('h1, .h1', tempSel)); // allow this same style to be used as a class too
        rootSel.push(`--h1-color: ${thisColor}`);
      }
      // Set H2 similarly
      tempSel = [];
      styleObj = themeJSON.styles.title2;
      if (styleObj) {
        const thisColor = RGBColourConvert(themeJSON.styles.title2.color ?? '#E9C062');
        tempSel.push(`color: ${thisColor}`);
        const thisBackgroundColor = RGBColourConvert(themeJSON.styles.title2.backgroundColor ?? bgMainColor);
        tempSel.push(`background-color: ${thisBackgroundColor}`);
        tempSel = tempSel.concat(convertStyleObjectBlock(styleObj));
        output.push(makeCSSSelector('h2, .h2', tempSel));
        rootSel.push(`--h2-color: ${thisColor}`);
      }
      // Set H3 similarly
      tempSel = [];
      styleObj = themeJSON.styles.title3;
      if (styleObj) {
        const thisColor = RGBColourConvert(themeJSON.styles.title3.color ?? '#E9C062');
        tempSel.push(`color: ${thisColor}`);
        const thisBackgroundColor = RGBColourConvert(themeJSON.styles.title3.backgroundColor ?? bgMainColor);
        tempSel.push(`background-color: ${thisBackgroundColor}`);
        tempSel = tempSel.concat(convertStyleObjectBlock(styleObj));
        output.push(makeCSSSelector('h3, .h3', tempSel));
        rootSel.push(`--h3-color: ${thisColor}`);
      }
      // Set H4 similarly
      tempSel = [];
      styleObj = themeJSON.styles.title4;
      if (styleObj) {
        tempSel.push(`color: ${RGBColourConvert(themeJSON.styles.title4.color ?? '#E9C062')}`);
        const thisBackgroundColor = RGBColourConvert(themeJSON.styles.title4.backgroundColor ?? bgMainColor);
        tempSel.push(`background-color: ${thisBackgroundColor}`);
        tempSel = tempSel.concat(convertStyleObjectBlock(styleObj));
        output.push(makeCSSSelector('h4, .h4', tempSel));
      }
      // NP doesn't support H5 styling

      // Set core colour features from theme
      const altColor = RGBColourConvert(themeJSON.editor?.altBackgroundColor) ?? '#2E2F30';
      rootSel.push(`--bg-alt-color: ${altColor}`);
      const tintColor = RGBColourConvert(themeJSON.editor?.tintColor) ?? '#E9C0A2';
      rootSel.push(`--tint-color: ${tintColor}`);
      // Following added to mimic what the NP settings screen main background is
      rootSel.push(`--bg-mid-color: ${mixHexColors(bgMainColor, altColor)}`);

      // Set font for native controls (otherwise will go to Apple default)
      output.push(makeCSSSelector('button, input', [`font-family: "${bodyFont}"`]));

      // Set a few styles here that require computed light and dark settings
      // Note: Now found a way to do this just in CSS so moved to plugins
      // if (isLightTheme) {
      //   output.push(makeCSSSelector('button, input', [
      //     'color: var(--fg-main-color)',
      //     'background-color: var(--bg-alt-color)',
      //   ]))
      //   output.push(
      //     makeCSSSelector('.fake-button a', [
      //       'color: var(--fg-main-color)',
      //       'background-color: var(--bg-alt-color)',
      //       'border-color: #DFE0E0',
      //       'box-shadow: 0 1px 1px #CBCBCB',
      //     ]),
      //   )
      // } else {
      //   // dark theme
      //   output.push(makeCSSSelector('button, input', [
      //     'color: var(--fg-main-color)',
      //     'background-color: var(--bg-alt-color)',
      //   ]))
      //   output.push(
      //     makeCSSSelector('.fake-button a', [
      //       'color: var(--fg-main-color)',
      //       'background-color: var(--bg-alt-color)',
      //       'border-color: #5E5E5E',
      //       'box-shadow: 0 -1px 1px #6F6F6F',
      //     ]),
      //   )
      // }

      // Set italic text if present
      tempSel = [];
      styleObj = themeJSON.styles.italic;
      if (styleObj) {
        tempSel.push(`color: ${RGBColourConvert(styleObj.color ?? '#96CBFE')}`);
        tempSel = tempSel.concat(convertStyleObjectBlock(styleObj));
        output.push(makeCSSSelector('p emph', tempSel)); // not 'i' as otherwise it can mess up the fontawesome icons
      }
      // Set bold text if present
      tempSel = [];
      styleObj = themeJSON.styles.bold;
      if (styleObj) {
        tempSel.push(`color: ${RGBColourConvert(styleObj.color ?? '#CC6666')}`);
        tempSel = tempSel.concat(convertStyleObjectBlock(styleObj));
        output.push(makeCSSSelector('p b', tempSel));
      }

      // Can't easily set bold-italic in CSS ...

      // Set class for open tasks ('todo') if present
      tempSel = [];
      styleObj = themeJSON.styles.todo;
      if (styleObj) {
        const todoColor = RGBColourConvert(styleObj.color) ?? 'var(--tint-color)';
        tempSel.push(`color: ${todoColor}`);
        tempSel = tempSel.concat(convertStyleObjectBlock(styleObj, false));
        // hack: easier to add second definition than to undo the last one
        tempSel.push('line-height: var(--body-line-height)');
        output.push(makeCSSSelector('.todo', tempSel));
        rootSel.push(`--item-icon-color: ${todoColor}`);
      }

      // Set class for completed tasks ('checked') if present
      tempSel = [];
      styleObj = themeJSON.styles.checked;
      if (styleObj) {
        tempSel.push(`color: ${RGBColourConvert(styleObj.color ?? '#098308A0')}`);
        tempSel = tempSel.concat(convertStyleObjectBlock(styleObj, false));
        tempSel.push('line-height: var(--body-line-height)');
        output.push(makeCSSSelector('.checked', tempSel));
      }

      // Set class for cancelled tasks ('checked-canceled') if present
      // following is workaround in object handling as 'checked-canceled' JSON property has a dash in it
      tempSel = [];
      styleObj = themeJSON.styles['checked-canceled'];
      if (styleObj) {
        tempSel.push(`color: ${RGBColourConvert(styleObj.color ?? '#E04F57A0')}`);
        tempSel = tempSel.concat(convertStyleObjectBlock(styleObj, false));
        tempSel.push('line-height: var(--body-line-height)');
        output.push(makeCSSSelector('.cancelled', tempSel));
      }

      // Set class for scheduled tasks ('checked-scheduled') if present
      // following is workaround in object handling as 'checked-canceled' JSON property has a dash in it
      tempSel = [];
      styleObj = themeJSON.styles['checked-scheduled'];
      if (styleObj) {
        tempSel.push(`color: ${RGBColourConvert(styleObj.color ?? '#7B7C86A0')}`);
        tempSel = tempSel.concat(convertStyleObjectBlock(styleObj, false));
        tempSel.push('line-height: var(--body-line-height)');
        output.push(makeCSSSelector('.task-scheduled', tempSel));
      }

      // Set class for hashtags ('hashtag') if present
      tempSel = [];
      styleObj = themeJSON.styles.hashtag;
      if (styleObj) {
        tempSel.push(`color: ${RGBColourConvert(styleObj.color ?? 'inherit')}`);
        tempSel.push(`background-color: ${RGBColourConvert(styleObj.backgroundColor ?? 'inherit')}`);
        tempSel.push('border-radius: 5px');
        tempSel.push('padding-inline: 3px');
        tempSel = tempSel.concat(convertStyleObjectBlock(styleObj));
        output.push(makeCSSSelector('.hashtag', tempSel));
        // Also save just the foreground colour to root
        rootSel.push(`--hashtag-color: ${RGBColourConvert(styleObj.color ?? 'inherit')}`);
      }

      // Set class for mentions ('attag') if present
      tempSel = [];
      styleObj = themeJSON.styles.attag;
      if (styleObj) {
        tempSel.push(`color: ${RGBColourConvert(styleObj.color ?? 'inherit')}`);
        tempSel.push(`background-color: ${RGBColourConvert(styleObj.backgroundColor ?? 'inherit')}`);
        tempSel.push('border-radius: 5px');
        tempSel.push('padding-inline: 3px');
        tempSel = tempSel.concat(convertStyleObjectBlock(styleObj));
        output.push(makeCSSSelector('.attag', tempSel));
        // Also save just the foreground colour to root
        rootSel.push(`--attag-color: ${RGBColourConvert(styleObj.color ?? 'inherit')}`);
      }

      // Set class for `pre-formatted text` ('code') if present
      tempSel = [];
      styleObj = themeJSON.styles.code;
      if (styleObj) {
        tempSel.push(`color: ${RGBColourConvert(styleObj.color ?? 'inherit')}`);
        tempSel.push(`background-color: ${RGBColourConvert(styleObj.backgroundColor ?? 'inherit')}`);
        tempSel.push('border-radius: 5px');
        tempSel.push('padding-inline: 3px');
        tempSel = tempSel.concat(convertStyleObjectBlock(styleObj));
        output.push(makeCSSSelector('.code', tempSel));
      }

      // Set class for ==highlights== ('highlighted') if present
      tempSel = [];
      styleObj = themeJSON.styles.highlighted;
      if (styleObj) {
        tempSel.push(`color: ${RGBColourConvert(styleObj.color ?? 'inherit')}`);
        tempSel.push(`background-color: ${RGBColourConvert(styleObj.backgroundColor ?? 'inherit')}`);
        tempSel.push('border-radius: 5px');
        tempSel.push('padding-inline: 3px');
        tempSel = tempSel.concat(convertStyleObjectBlock(styleObj));
        output.push(makeCSSSelector('.highlighted', tempSel));
      }

      // Set class for ~underlined~ ('underline') if present
      tempSel = [];
      styleObj = themeJSON.styles.underline;
      if (styleObj) {
        tempSel = tempSel.concat(convertStyleObjectBlock(styleObj, true));
        output.push(makeCSSSelector('.underlined', tempSel));
      }
      // Set class for ~~strikethrough~~ ('strikethrough') if present
      tempSel = [];
      styleObj = themeJSON.styles.strikethrough;
      if (styleObj) {
        tempSel = tempSel.concat(convertStyleObjectBlock(styleObj, true));
        output.push(makeCSSSelector('.strikethrough', tempSel));
      }

      // Set class for 'flagged-1' (priority 1) if present
      tempSel = [];
      styleObj = themeJSON.styles['flagged-1'];
      if (styleObj) {
        tempSel.push(`color: ${RGBColourConvert(styleObj.color) ?? 'inherit'}`);
        tempSel.push(`background-color: ${RGBColourConvert(styleObj.backgroundColor ?? '#FFE5E5')}`);
        tempSel.push('border-radius: 5px');
        tempSel.push('padding-inline: 3px');
        tempSel = tempSel.concat(convertStyleObjectBlock(styleObj));
        output.push(makeCSSSelector('.priority1', tempSel));
      }

      // Set class for 'flagged-2' (priority 2) if present
      tempSel = [];
      styleObj = themeJSON.styles['flagged-2'];
      if (styleObj) {
        tempSel.push(`color: ${RGBColourConvert(styleObj.color) ?? 'inherit'}`);
        tempSel.push(`background-color: ${RGBColourConvert(styleObj.backgroundColor ?? '#FFC5C5')}`);
        tempSel.push('border-radius: 5px');
        tempSel.push('padding-inline: 3px');
        tempSel = tempSel.concat(convertStyleObjectBlock(styleObj));
        output.push(makeCSSSelector('.priority2', tempSel));
      }

      // Set class for 'flagged-3' (priority 3) if present
      tempSel = [];
      styleObj = themeJSON.styles['flagged-3'];
      if (styleObj) {
        tempSel.push(`color: ${RGBColourConvert(styleObj.color) ?? 'inherit'}`);
        tempSel.push(`background-color: ${RGBColourConvert(styleObj.backgroundColor ?? '#FFA5A5')}`);
        tempSel.push('border-radius: 5px');
        tempSel.push('padding-inline: 3px');
        tempSel = tempSel.concat(convertStyleObjectBlock(styleObj));
        output.push(makeCSSSelector('.priority3', tempSel));
      }

      // Set class for 'working-on' if present
      tempSel = [];
      styleObj = themeJSON.styles['working-on'];
      if (styleObj) {
        tempSel.push(`color: ${RGBColourConvert(styleObj.color) ?? 'inherit'}`);
        tempSel.push(`background-color: ${RGBColourConvert(styleObj.backgroundColor ?? '#FFA5A5')}`);
        tempSel.push('border-radius: 5px');
        tempSel.push('padding-inline: 3px');
        tempSel = tempSel.concat(convertStyleObjectBlock(styleObj));
        output.push(makeCSSSelector('.priority4', tempSel));
      }

      // Set class for Time Blocks if present
      // Note: fudges size down in the samw way that NP seems to under the hood
      tempSel = [];
      tempSel.push(`color: ${RGBColourConvert(themeJSON.editor.timeBlockColor) ?? 'inherit'}`);
      tempSel.push(`font-family: "Menlo-Regular"`);
      const smallerFontSize = Math.round(baseFontSize * 0.84);
      tempSel.push(`font-size: ${smallerFontSize}px;`);
      tempSel = tempSel.concat(convertStyleObjectBlock(styleObj));
      output.push(makeCSSSelector('.timeBlock', tempSel));

      // Now put the important info and rootSel at the start of the output
      output.unshift(makeCSSSelector(':root', rootSel));
      output.unshift(`/* Generated from theme '${themeName}' by @jgclark's generateCSSFromTheme */`);

      // Finally, just a note requested by @dwertheimer, and others who might have non-standard names in their themes
      const themeKeysToCheck = ['flagged-1', 'flagged-2', 'flagged-3', 'working-on'];
      themeKeysToCheck.forEach(key => !themeJSON.styles[key] && logWarn(generateCSSFromTheme, `Your theme does not have the key "${key}" which Dashboard uses. This may be ok if you don't want that style, but if you do, you need to rename your theme style for this type of line`));

      // logDebug('generateCSSFromTheme', `Generated CSS:\n${output.join('\n')}`)
      return output.join('\n');
    } catch (error) {
      logError('generateCSSFromTheme', error.message);
      return '<error>';
    }
  }

  /**
   * Convert NotePlan Theme style information to CSS equivalent(s)
   * Covers attributes within a theme item: size (of font), paragraphSpacingBefore, paragraphSpacing, lineSpacing, font, strikethroughStyle, underlineStyle, underlineColor.
   * @author @jgclark
   * @param {Object} style object from JSON theme
   * @param {boolean} includeFontDetails? (default: false)
   * @returns {Array} CSS elements
   */
  function convertStyleObjectBlock(styleObject, includeFontDetails = true) {
    let cssStyleLinesOutput = [];
    if (styleObject?.size) {
      cssStyleLinesOutput.push(`font-size: ${pxToRem(styleObject?.size, userFSPref)}`);
    }
    if (includeFontDetails) {
      if (styleObject?.font) {
        cssStyleLinesOutput = cssStyleLinesOutput.concat(fontPropertiesFromNP(styleObject?.font));
      }
    }
    if (styleObject?.paragraphSpacingBefore) {
      cssStyleLinesOutput.push(`margin-top: ${pxToRem(styleObject?.paragraphSpacingBefore, userFSPref)}`);
    }
    if (styleObject?.paragraphSpacing) {
      cssStyleLinesOutput.push(`margin-bottom: ${pxToRem(styleObject?.paragraphSpacing, userFSPref)}`);
    }
    if (styleObject?.lineSpacing) {
      const lineSpacingRem = (Number(styleObject?.lineSpacing) * 1.5).toPrecision(3); // this fudge factor seems to be required
      cssStyleLinesOutput.push(`line-height: ${String(lineSpacingRem)}rem`);
    }
    if (styleObject?.strikethroughStyle) {
      const themeStyleNumber = Number(styleObject?.strikethroughStyle);
      /**
       * Values from 1-8 increase the thickness.
       * The next bit values that have an effect are: 1...
       * + 8: double (= 9)
       * + 256: patternDot (= 257)
       * + 512: patternDash (= 513)
       * + 1024: patternDashDotDot (= 1025)
       * + 8192: over line (= 8193)
       * +32768: by Word (= 32769)
       */
      if (themeStyleNumber > 0 && themeStyleNumber <= 8) {
        cssStyleLinesOutput.push('text-decoration: line-through');
        cssStyleLinesOutput.push(`text-decoration-style: solid`);
        cssStyleLinesOutput.push(`text-decoration-thickness: ${String(themeStyleNumber)}px`);
      }
      if (themeStyleNumber > 8 && themeStyleNumber <= 16) {
        cssStyleLinesOutput.push('text-decoration: line-through');
        cssStyleLinesOutput.push(`text-decoration-style: double`);
        cssStyleLinesOutput.push(`text-decoration-thickness: ${String(themeStyleNumber - 8)}px`);
      }
      if (themeStyleNumber > 256 && themeStyleNumber <= 264) {
        cssStyleLinesOutput.push('text-decoration: line-through');
        cssStyleLinesOutput.push(`text-decoration-style: dotted`);
        cssStyleLinesOutput.push(`text-decoration-thickness: ${String(themeStyleNumber - 256)}px`);
      }
      if (themeStyleNumber > 512 && themeStyleNumber <= 520) {
        cssStyleLinesOutput.push('text-decoration: line-through');
        cssStyleLinesOutput.push(`text-decoration-style: dashed`);
        cssStyleLinesOutput.push(`text-decoration-thickness: ${String(themeStyleNumber - 512)}px`);
      }
      if (themeStyleNumber > 8192 && themeStyleNumber <= 8200) {
        cssStyleLinesOutput.push(`text-decoration-style: overline`);
        cssStyleLinesOutput.push(`text-decoration-thickness: ${String(themeStyleNumber - 8192)}px`);
      }
      // cssStyleLinesOutput.push(textDecorationFromNP('strikethroughStyle', Number(styleObject?.strikethroughStyle)))
    }
    if (styleObject?.strikethroughColor) {
      cssStyleLinesOutput.push(`text-decoration-color: ${RGBColourConvert(styleObject.strikethroughColor ?? 'var(--fg-main-color)')}`);
    }
    if (styleObject?.underlineStyle) {
      const themeStyleNumber = Number(styleObject?.underlineStyle);
      /**
       * Values from 1-8 increase the thickness.
       * The next bit values that have an effect are: 1...
       * + 8: double (= 9)
       * + 256: patternDot (= 257)
       * + 512: patternDash (= 513)
       * + 1024: patternDashDotDot (= 1025)
       * + 8192: over line (= 8193)
       * +32768: by Word (= 32769)
       */
      if (themeStyleNumber > 0 && themeStyleNumber <= 8) {
        cssStyleLinesOutput.push('text-decoration: underline');
        cssStyleLinesOutput.push(`text-decoration-style: solid`);
        cssStyleLinesOutput.push(`text-decoration-thickness: ${String(themeStyleNumber)}px`);
      }
      if (themeStyleNumber > 8 && themeStyleNumber <= 16) {
        cssStyleLinesOutput.push('text-decoration: underline');
        cssStyleLinesOutput.push(`text-decoration-style: double`);
        cssStyleLinesOutput.push(`text-decoration-thickness: ${String(themeStyleNumber - 8)}px`);
      }
      if (themeStyleNumber > 256 && themeStyleNumber <= 264) {
        cssStyleLinesOutput.push('text-decoration: underline');
        cssStyleLinesOutput.push(`text-decoration-style: dotted`);
        cssStyleLinesOutput.push(`text-decoration-thickness: ${String(themeStyleNumber - 256)}px`);
      }
      if (themeStyleNumber > 512 && themeStyleNumber <= 520) {
        cssStyleLinesOutput.push('text-decoration: underline');
        cssStyleLinesOutput.push(`text-decoration-style: dashed`);
        cssStyleLinesOutput.push(`text-decoration-thickness: ${String(themeStyleNumber - 512)}px`);
      }
      if (themeStyleNumber > 8192 && themeStyleNumber <= 8200) {
        cssStyleLinesOutput.push(`text-decoration-style: overline`);
        cssStyleLinesOutput.push(`text-decoration-thickness: ${String(themeStyleNumber - 8192)}px`);
      }
    }
    if (styleObject?.underlineColor) {
      cssStyleLinesOutput.push(`text-decoration-color: ${RGBColourConvert(styleObject.underlineColor ?? 'var(--fg-main-color)')}`);
    }
    return cssStyleLinesOutput;
  }

  /**
   * Convert a font size (in px) to rem (as a string).
   * Note: assumes userFSPref (in px) is the basis for 1.0rem.
   * Then uses the ratio between the input size and the body size.
   * e.g. if body = 16, then H1 at 24 => 24/16 = 1.5rem
   * @param {number} thisFontSize
   * @param {number} baseFontSize
   * @returns {string} size including 'rem' units
   */
  function pxToRem(thisFontSize, baseFontSize) {
    let rem = thisFontSize / baseFontSize;
    // Note: Need to apply fudge to get it closer to actual size seen in NP Editor
    rem *= 0.95;
    const output = `${String(rem.toPrecision(2))}rem`;
    // logDebug('pxToRem', `${thisFontSize}px / ${baseFontSize} -> ${output}`)
    return output;
  }

  /**
   * Convert [A]RGB (used by NP) to RGB[A] (CSS)
   * @param {string} #[A]RGB
   * @returns {string} #RGB[A]
   */
  function RGBColourConvert(RGBIn) {
    try {
      // default to just passing the colour through, unless
      // we have ARGB, so need to switch things round
      let output = RGBIn;
      if (RGBIn != null && RGBIn.match(/#[0-9A-Fa-f]{8}/)) {
        output = `#${RGBIn.slice(3, 9)}${RGBIn.slice(1, 3)}`;
      }
      return output;
    } catch (error) {
      logError('RGBColourConvert', `${error.message} for RGBIn '${RGBIn}'`);
      return '#888888'; // for completeness
    }
  }

  /**
   * Note: in future it should be possible to do this in CSS with `color-mix(in srgb, <color-A>, <color-B>)`
   * From https://stackoverflow.com/a/66402402/3238281
   */
  /**
   * Mixes two hex color strings by averaging their RGB components.
   *
   * @param {string} color1 - The first hex color string (e.g., '#ff0000').
   * @param {string} color2 - The second hex color string (e.g., '#0000ff').
   * @returns {string} The resulting hex color string after mixing (e.g., '#800080').
   */
  function mixHexColors(color1, color2) {
    const RE_RGB6 = /^#[0-9a-fA-F]{6}$/;
    if (!RE_RGB6.test(color1) || !RE_RGB6.test(color2)) throw new Error('Invalid hex color format');
    // Remove the '#' and split the hex color into RGB components
    const valuesColor1 = color1.replace('#', '').match(/.{2}/g)?.map(value => parseInt(value, 16)) || [];
    const valuesColor2 = color2.replace('#', '').match(/.{2}/g)?.map(value => parseInt(value, 16)) || [];

    // Ensure both colors have valid RGB components
    if (valuesColor1.length !== 3 || valuesColor2.length !== 3) {
      throw new Error('Invalid hex color format');
    }

    // Mix the RGB components by averaging
    const mixedValues = valuesColor1.map((value, index) => Math.round((value + valuesColor2[index]) / 2).toString(16).padStart(2, '0'));
    return `#${mixedValues.join('')}`;
  }

  /**
   * Translate from the font name, as used in the NP Theme file,
   * to the form CSS is expecting.
   * If no translation is defined, try to use the user's own default font.
   * If that fails, use fallback font 'sans'.
   * Further info at https://help.noteplan.co/article/44-customize-themes#fonts
   * @author @jgclark
   * @param {string} fontNameNP
   * @returns {Array<string>} resulting CSS font properties
   */
  function fontPropertiesFromNP(fontNameNP) {
    // logDebug('fontPropertiesFromNP', `for '${fontNameNP}'`)
    const outputArr = [];

    // Deal with special case of Apple's System font
    // See https://www.webkit.org/blog/3709/using-the-system-font-in-web-content/ for more info
    if (fontNameNP.startsWith('.AppleSystemUIFont')) {
      outputArr.push(`font-family: "-apple-system"`);
      outputArr.push(`line-height: 1.2rem`);
      if (fontNameNP.includes('Bold')) {
        outputArr.push(`font-weight: 700`);
      } else {
        outputArr.push(`font-weight: 400`);
      }
      // logDebug('fontPropertiesFromNP', `special: ${fontNameNP} ->  ${outputArr.toString()}`)
      return outputArr;
    }

    // Then test to see if this is one of the other specials
    const specialFontList = new Map();
    // lookup list of special cases
    specialFontList.set('System', ['sans', 'regular', 'normal']);
    specialFontList.set('', ['sans', 'regular', 'normal']);
    specialFontList.set('noteplanstate', ['noteplanstate', 'regular', 'normal']);
    const specials = specialFontList.get(fontNameNP); // or undefined if none match
    if (specials !== undefined) {
      outputArr.push(`font-family: "${specials[0]}"`);
      outputArr.push(`font-weight: "${specials[1]}"`);
      outputArr.push(`font-style: "${specials[2]}"`);
      // logDebug('fontPropertiesFromNP', `specials: ${fontNameNP} ->  ${outputArr.toString()}`)
      return outputArr;
    }

    // Not a special. So now split input string into parts either side of '-'
    // and then insert spaces before capital letters
    let translatedFamily;
    let translatedWeight = '400';
    let translatedStyle = 'normal';
    const splitParts = fontNameNP.split('-');
    const namePartNoSpaces = splitParts[0];
    let namePartSpaced = '';
    const modifierLC = splitParts.length > 0 ? splitParts[1]?.toLowerCase() : '';
    for (let i = 0; i < namePartNoSpaces.length; i++) {
      const c = namePartNoSpaces[i];
      if (c.match(/[A-Z]/)) {
        namePartSpaced += ` ${c}`;
      } else {
        namePartSpaced += c;
      }
    }
    translatedFamily = namePartSpaced.trim();
    // logDebug('fontPropertiesFromNP', `family -> ${translatedFamily}`)

    // Using the numeric font-weight system
    // With info from https://developer.mozilla.org/en-US/docs/Web/CSS/font-weight#common_weight_name_mapping
    switch (modifierLC) {
      case 'thin':
        {
          translatedWeight = '100';
          break;
        }
      case 'light':
        {
          translatedWeight = '300';
          break;
        }
      case 'book':
        {
          translatedWeight = '500';
          break;
        }
      case 'demi-bold':
        {
          translatedWeight = '600';
          break;
        }
      case 'demibold':
        {
          translatedWeight = '600';
          break;
        }
      case 'semi-bold':
        {
          translatedWeight = '600';
          break;
        }
      case 'semibold':
        {
          translatedWeight = '600';
          break;
        }
      case 'bold':
        {
          translatedWeight = '700';
          break;
        }
      case 'heavy':
        {
          translatedWeight = '900';
          break;
        }
      case 'black':
        {
          translatedWeight = '900';
          break;
        }
      case 'italic':
        {
          translatedStyle = 'italic';
          break;
        }
      case 'bolditalic':
        {
          translatedWeight = '700';
          translatedStyle = 'italic';
          break;
        }
      case 'slant':
        {
          translatedStyle = 'italic';
          break;
        }
      default:
        {
          // including '', 'normal' and 'regular'
          translatedWeight = '400';
          translatedStyle = 'normal';
          break;
        }
    }
    // logDebug('translateFontNameNPToCSS', `  - ${translatedStyle} / ${translatedWeight}`)

    // Finally if we're still working on default 'Sans', then
    // at least try to use the user's default font setting.
    if (translatedFamily === 'Sans') {
      logDebug('fontPropertiesFromNP', `For '${fontNameNP}' trying user's default font setting`);
      const userFont = String(DataStore.preference('fontFamily')) ?? '';
      logDebug('fontPropertiesFromNP', `- userFont = '${userFont}'`);
      translatedFamily = userFont;
    }
    outputArr.push(`font-family: "${translatedFamily}"`);
    outputArr.push(`font-weight: ${translatedWeight}`);
    outputArr.push(`font-style: "${translatedStyle}"`);
    // logDebug('translateFontNameNPToCSS', `${fontNameNP} ->  ${outputArr.toString()}`)
    return outputArr;
  }

  /**
   * Make a CSS selector from an array of parameters
   * @param {string} selector
   * @param {Array<string>} settingsArray
   * @returns {string} CSS selector with its various parameters
   */
  function makeCSSSelector(selector, settingsArray) {
    const outputArray = [];
    outputArray.push(`\t${selector} {`);
    outputArray.push(`\t\t${settingsArray.join(';\n\t\t')}`);
    outputArray.push(`\t}`);
    return outputArray.join('\n');
  }

  // ---------------------------------------------------------
  // HTML helper functions for use with HTMLView API
  // by @jgclark, @dwertheimer
  // Last updated 2024-11-09 by @jgclark
  // ---------------------------------------------------------

  // ---------------------------------------------------------
  // Constants and Types

  const pluginJson = 'helpers/HTMLView';
  // Meta tags to always apply:
  // - to make windows always responsive
  const fixedMetaTags = `
<meta charset="utf-8">
<meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no, maximum-scale=1, viewport-fit=cover">

`;
  /**
   * Generate scripts string from array of strings or objects with code and type
   * Strings are assumed to be javascript
   * Use ScriptObj type to specify type (typically "text/babel" for React/JSX)
   * @author @dwertheimer
   * @param {string|ScriptObj | Array<string|ScriptObj>} scripts
   * @returns {string} the fully formed string with all the scripts
   * @tests exist
   */
  function generateScriptTags(scripts) {
    if (!scripts || !scripts?.length && typeof scripts !== 'object') return '';
    const scriptsArr = Array.isArray(scripts) ? scripts : [scripts];
    const output = [];
    scriptsArr.forEach(script => {
      let hasScriptTag;
      let scriptText = '';
      if (typeof script === 'string') {
        if (script !== '') {
          hasScriptTag = script.includes('<script');
          scriptText = hasScriptTag ? '' : '<script type="text/javascript">\n';
          scriptText += script;
        }
      } else {
        const {
          code,
          type
        } = script || {};
        hasScriptTag = code.includes('<script');
        if (hasScriptTag && type !== 'text/javascript') {
          logError(pluginJson, `generateScriptTags script had <script tag and type:"${type || ''}" and value:"${code}" - this is not supported (send only the code)`);
        }
        scriptText += hasScriptTag ? '' : `<script type="${type ?? 'text/javascript'}">\n`;
        scriptText += code;
      }
      if (script !== '') scriptText += hasScriptTag ? '\n' : '\n</script>\n';
      output.push(scriptText);
    });
    return output.join('\n');
  }

  /**
   * Assemble/collate the HTML to use from its various parts.
   * @author @jgclark
   * @param {string} body
   * @param {HtmlWindowOptions} winOpts
   * @returns
   */
  function assembleHTMLParts(body, winOpts) {
    try {
      const fullHTML = [];
      fullHTML.push('<!DOCTYPE html>'); // needed to let emojis work without special coding
      fullHTML.push('<html>');
      fullHTML.push('<head>');
      fullHTML.push(`<title>${winOpts.windowTitle}</title>`);
      fullHTML.push(fixedMetaTags);
      const preScript = generateScriptTags(winOpts.preBodyScript ?? '');
      if (preScript !== '') {
        fullHTML.push(preScript); // dbw moved to top because we need the logging bridge to be loaded before any content which could have errors
      }
      fullHTML.push(winOpts.headerTags ?? '');
      fullHTML.push('<style type="text/css" title="Original Theme Styles">');
      // If generalCSSIn is empty, then generate it from the current theme. (Note: could extend this to save CSS from theme, and then check if it can be reused.)
      const generalCSS = winOpts.generalCSSIn && winOpts.generalCSSIn !== '' ? winOpts.generalCSSIn : generateCSSFromTheme('');
      fullHTML.push(generalCSS);
      fullHTML.push(winOpts.specificCSS ?? '');
      fullHTML.push('</style>');
      fullHTML.push('</head>');
      fullHTML.push(winOpts.bodyOptions ? `\n<body ${winOpts.bodyOptions}>` : `\n<body>`);
      fullHTML.push(body);
      fullHTML.push('\n</body>');
      const postScript = generateScriptTags(winOpts.postBodyScript ?? '');
      if (postScript !== '') {
        fullHTML.push(postScript);
      }
      fullHTML.push('</html>');
      const fullHTMLStr = fullHTML.join('\n');
      return fullHTMLStr;
    } catch (err) {
      logError(pluginJson, err.message);
      return '';
    }
  }

  /**
   * WARNING: Deprecated. Please use more advanced features in showHTMLV2() instead. This version will also (probably) not allow use of multiple HTML windows from NP3.9.6.
   * Helper function to construct HTML to show in a new window.
   * Note: used up to v3.9.2 before more advanced window handling possible.
   * Note: if customID not passed, it will fall back to using windowTitle
   *
   * @param {string} windowTitle
   * @param {string} headerTags
   * @param {string} body
   * @param {string} generalCSSIn
   * @param {string} specificCSS
   * @param {boolean?} makeModal?
   * @param {string|ScriptObj | Array<string|ScriptObj>?} preBodyScript
   * @param {string|ScriptObj | Array<string|ScriptObj>?} postBodyScript
   * @param {string?} filenameForSavedFileVersion
   * @param {number?} width
   * @param {number?} height
   * @param {string?} customId
   */
  function showHTML(windowTitle, headerTags, body, generalCSSIn, specificCSS, makeModal = false, preBodyScript = '', postBodyScript = '', filenameForSavedFileVersion = '', width, height,
  // eslint-disable-next-line no-unused-vars
  customId = '' // Note: now unused
  ) {
    try {
      const opts = {
        windowTitle: windowTitle,
        headerTags: headerTags,
        generalCSSIn: generalCSSIn,
        specificCSS: specificCSS,
        preBodyScript: preBodyScript,
        postBodyScript: postBodyScript
      };
      // clo(opts)
      const fullHTMLStr = assembleHTMLParts(body, opts);

      // Call the appropriate function, with or without h/w params.
      // Currently non-modal windows only available on macOS and from 3.7 (build 864)
      if (width === undefined || height === undefined) {
        if (makeModal || NotePlan.environment.platform !== 'macOS') {
          logDebug('showHTML', `Using showSheet modal view for b${NotePlan.environment.buildVersion}`);
          HTMLView.showSheet(fullHTMLStr); // available from 3.6.2
        } else {
          logDebug('showHTML', `Using showWindow non-modal view for b${NotePlan.environment.buildVersion}`);
          HTMLView.showWindow(fullHTMLStr, windowTitle); // available from 3.7.0
        }
      } else {
        if (makeModal || NotePlan.environment.platform !== 'macOS' || NotePlan.environment.buildVersion < 863) {
          logDebug('showHTML', `Using showSheet modal view for b${NotePlan.environment.buildVersion}`);
          HTMLView.showSheet(fullHTMLStr, width, height);
        } else {
          logDebug('showHTML', `Using showWindow non-modal view with w+h for b${NotePlan.environment.buildVersion}`);
          HTMLView.showWindow(fullHTMLStr, windowTitle, width, height); // available from 3.7.0
        }
      }

      // TEST: remove from 3.9.6
      // Set customId for this window (with fallback to be windowTitle) Note: requires NP v3.8.1+
      // if (NotePlan.environment.buildVersion >= 976) {
      //   setHTMLWindowId(customId ?? windowTitle)
      // }

      // If wanted, also write this HTML to a file so we can work on it offline.
      // Note: this is saved to the Plugins/Data/<Plugin> folder, not a user-accessible Note.
      if (filenameForSavedFileVersion !== '') {
        const filenameWithoutSpaces = filenameForSavedFileVersion.split(' ').join('');
        // Write to specified file in NP sandbox
        const res = DataStore.saveData(fullHTMLStr, filenameWithoutSpaces, true);
        if (res) {
          logDebug('showHTML', `Saved copy of HTML to '${windowTitle}' to ${filenameForSavedFileVersion}`);
        } else {
          logError('showHTML', `Couldn't save resulting HTML '${windowTitle}' to ${filenameForSavedFileVersion}.`);
        }
      }
    } catch (error) {
      logError('HTMLView / showHTML', error.message);
    }
  }

  /**
   * V2 helper function to construct HTML and decide how and where to show it in a window.
   * Most data comes via an opts object, to ease future expansion.
   * Adds ability to automatically display windows at the last position and size that the user had left them at. To enable this:
   * - set opts.reuseUsersWindowRect to true
   * - supply a opts.customId to distinguish which window this is to the plugin (e.g. 'review-list'). I suggest this is lower-case-with-dashes. (If customId not passed, it will fall back to using opts.windowTitle instead.)
   * - (optional) still supply default opts.width and opts.height to use the first time
   * Under the hood it saves the windowRect to local preference "<plugin.id>.<customId>".
   * Note: Could allow for style file via saving arbitrary data file, and have it triggered on theme change.
   * Note: requires NP v3.9.2 build 1037
   * @author @jgclark
   * @param {string} body
   * @param {HtmlWindowOptions} opts
   */
  async function showHTMLV2(body, opts) {
    try {
      if (NotePlan.environment.buildVersion < 1037) {
        logWarn('HTMLView / showHTMLV2', 'showHTMLV2() is only available on 3.9.2 build 1037 or newer. Will fall back to using older, simpler, showHTML() instead ...');
        await showHTML(opts.windowTitle, fixedMetaTags + (opts.headerTags ?? ''), body, opts.generalCSSIn ?? '', opts.specificCSS ?? '', opts.makeModal, opts.preBodyScript, opts.postBodyScript, opts.savedFilename ?? '', opts.width, opts.height, opts.customId);
        return true; // for completeness
      }
      logDebug('HTMLView / showHTMLV2', `starting with customId ${opts.customId ?? ''} and reuseUsersWindowRect ${String(opts.reuseUsersWindowRect) ?? '??'}`);

      // Assemble the parts of the HTML into a single string
      const fullHTMLStr = assembleHTMLParts(body, opts);

      // Ensure we have a window ID to use
      const cId = opts.customId ?? opts.windowTitle ?? 'fallback';

      // Before showing anything, see if the window is already open, and if so save its x/y/w/h (if requested)
      if (isHTMLWindowOpen(cId)) {
        logDebug('showHTMLV2', `Window is already open, and will save its x/y/w/h`);
        storeWindowRect(cId);
      }

      // Decide which of the appropriate functions to call.
      if (opts.makeModal) {
        // if (opts.makeModal || NotePlan.environment.platform !== 'macOS') {
        logDebug('showHTMLV2', `Using modal 'sheet' view for ${NotePlan.environment.buildVersion} build on ${NotePlan.environment.platform}`);
        // if (opts.width === undefined || opts.height === undefined) {
        //   HTMLView.showSheet(fullHTMLStr)
        // } else {
        HTMLView.showSheet(fullHTMLStr, opts.width, opts.height);
        // }
      } else {
        // Make a normal non-modal window
        let winOptions = {};
        // First set to the default values
        if (NotePlan.environment.buildVersion >= 1087) {
          // From 3.9.6 can set windowId/customId directly through options
          winOptions = {
            x: opts.x,
            y: opts.y,
            width: opts.width,
            height: opts.height,
            shouldFocus: opts.shouldFocus,
            id: cId,
            // don't need both ... but trying to work out which is the current one for the API
            windowId: cId
          };
        } else {
          winOptions = {
            x: opts.x,
            y: opts.y,
            width: opts.width,
            height: opts.height,
            shouldFocus: opts.shouldFocus
          };
        }
        // Now override with saved x/y/w/h for this window if wanted, and if available
        if (opts.reuseUsersWindowRect && cId) {
          // logDebug('showHTMLV2', `- Trying to use user's saved Rect from pref for ${cId}`)
          const storedRect = getStoredWindowRect(cId);
          if (storedRect) {
            winOptions = {
              x: storedRect.x,
              y: storedRect.y,
              width: storedRect.width,
              height: storedRect.height,
              shouldFocus: opts.shouldFocus,
              id: cId,
              // don't need both ... but trying to work out which is the current one for the API
              windowId: cId
            };
            logDebug('showHTMLV2', `- Read user's saved Rect from pref from ${cId}`);
            // if (NotePlan.environment.platform !== 'macOS') {
            // const extraInfo = "<p>OS:" + NotePlan.environment.platform +
            //   " Type: " + (opts.makeModal ? 'Modal' : 'Floating') +
            //   " W: " + opts.width +
            //   " H: " + opts.height + "</p>\n" +
            //   " StoredW: " + storedRect.width +
            //   " StoredH: " + storedRect.height + "</p>\n"
            // fullHTMLStr = fullHTMLStr.replace("<body>", `<body>\n${extraInfo}\n`)
            // }
          } else {
            logDebug('showHTMLV2', `- Couldn't read user's saved Rect from pref from ${cId}`);
            // if (NotePlan.environment.platform !== 'macOS') {
            //   const extraInfo = "<p>OS:" + NotePlan.environment.platform +
            //     " Type: " + (opts.makeModal ? 'Modal' : 'Floating') +
            //     " W: " + opts.width +
            //     " H: " + opts.height + "</p>\n"
            //   fullHTMLStr = fullHTMLStr.replace("<body>", `<body>\n${extraInfo}\n`)
            // }
          }
        }
        // clo(winOptions, 'showHTMLV2 using winOptions:')

        // From v3.9.8 we can test to see if requested window dimensions would exceed screen dimensions; if so reduce them accordingly
        // Note: could also check window will be visible on screen and if not, move accordingly
        if (NotePlan.environment.buildVersion >= 1100) {
          const screenWidth = NotePlan.environment.screenWidth;
          const screenHeight = NotePlan.environment.screenHeight;
          logDebug('showHTMLV2', `- screen dimensions are ${String(screenWidth)} x ${String(screenHeight)} for device ${NotePlan.environment.machineName}`);
          if (winOptions.width > screenWidth) {
            logDebug('showHTMLV2', `- Constrained width from ${String(winOptions.width)} to ${String(screenWidth)}`);
            winOptions.width = screenWidth;
          }
          if (winOptions.height > screenHeight) {
            logDebug('showHTMLV2', `- Constrained height from ${String(winOptions.height)} to ${String(screenHeight)}`);
            winOptions.height = screenHeight;
          }
        }
        const win = await HTMLView.showWindowWithOptions(fullHTMLStr, opts.windowTitle, winOptions); // winOptions available from 3.9.1.
        // clo(win, '-> win:')

        // If wanted, also write this HTML to a file so we can work on it offline.
        // Note: this is saved to the Plugins/Data/<Plugin> folder, not a user-accessible Note.
        if (opts.savedFilename !== '') {
          const thisFilename = opts.savedFilename ?? '';
          const filenameWithoutSpaces = thisFilename.split(' ').join('') ?? '';
          // Write to specified file in NP sandbox
          const res = DataStore.saveData(fullHTMLStr, filenameWithoutSpaces, true);
          if (res) {
            logDebug('showHTMLV2', `- Saved copy of HTML to '${opts.windowTitle}' to ${thisFilename}`);
          } else {
            logError('showHTMLV2', `- Couldn't save resulting HTML '${opts.windowTitle}' to ${thisFilename}.`);
          }
        }

        // Set customId for this window (with fallback to be windowTitle)
        // Note: only required between NP v3.8.1 + and 3.9.5.After that its built in.
        if (NotePlan.environment.buildVersion < 1087) {
          logDebug('showHTMLV2', `- setting the customId to '${cId}'`);
          win.customId = cId;
        }

        // Double-check: read back from the window itself
        logDebug('showHTMLV2', `- Window has customId:'${win?.customId || ''}' / id:"${win?.id || ''}"`);
        return win;
      }
    } catch (error) {
      logError('HTMLView / showHTMLV2', error.message);
      return false;
    }
  }

  /**
   * Shows a list of notes with inconsistent names (i.e. where the note title and filename are different).
   * @returns void
   */
  function listInconsistentNames() {
    try {
      logDebug(pluginJson$4, 'listInconsistentNames(): Checking for inconsistent names in project notes...');
      const inconsistentNames = findInconsistentNames();
      if (inconsistentNames.length > 0) {
        const notesList = inconsistentNames.map(note => {
          const currentFullPath = note.filename;
          const newPath = newFilepathForNote(note);
          return `<li><strong>${displayTitle$1(note)}</strong><br><pre>${currentFullPath}</pre><pre>${newPath}</pre></li>`;
        }).join('\n');
        const htmlBody = `
      <h1>Inconsistent note names</h1>
      <p>Found ${inconsistentNames.length} inconsistent names in project notes (current -> new):</p>
      <ol>
      ${notesList}
      </ol>
      `;
        const res = showHTMLV2(htmlBody, {
          windowTitle: 'Inconsistent note names',
          customId: 'inconsistent-names',
          savedFilename: '../../jgclark.NoteHelpers/inconsistent-names.html',
          shouldFocus: true
        });
      }
    } catch (error) {
      logError(pluginJson$4, `listInconsistentNames() error: ${error.message}`);
    }
  }

  // --------------------------------------------------------------
  // Originally by Leo Melo, with bug fixes by @jgclark
  // Last updated 2024-08-16 for v0.19.3 by @jgclark
  // --------------------------------------------------------------
  async function renameNoteToTitle(note, shouldPromptBeforeRenaming = true) {
    if (note == null || note.paragraphs.length < 1) {
      // No note open, so don't do anything.
      logDebug(pluginJson$4, 'No note open, or no content. Stopping.');
      return false;
    }
    if (note.type === 'Calendar') {
      // Won't work on calendar notes
      await showMessage('This command does not support renaming calendar notes.');
      return false;
    }
    const title = note.title;
    const currentFilepath = note.filename;
    const newFilepath = newFilepathForNote(note);
    if (newFilepath === '' || title === '') {
      // No title found, so don't do anything.
      logWarn(pluginJson$4, 'renameNoteToTitle(): No title found. Stopping.');
      return false;
    }
    if (currentFilepath === newFilepath) {
      // No need to rename
      logDebug(pluginJson$4, 'renameNoteToTitle(): Current path is the same as the new path. Stopping.');
      await showMessage('The filename is already consistent with the note name.');
      return false;
    }
    if (!isValidFilename(newFilepath)) {
      logWarn(pluginJson$4, `renameNoteToTitle(): Invalid filename "${newFilepath}". Stopping.`);
      await showMessage(`The filename "${newFilepath}" is not valid. Please check the title and try again.`);
      return false;
    }
    if (!shouldPromptBeforeRenaming) {
      const newFilename = note.rename(newFilepath);
      logDebug(pluginJson$4, `renameNoteToTitle(): ${currentFilepath} -> ${newFilename}`);
      return true;
    }
    const currentFilename = currentFilepath.split('/').pop();
    // $FlowIgnore[incompatible-type]
    const promptResponse = await showMessageYesNoCancel(`Would you like to rename "${currentFilename}" to match the note title "${title}"?

  Current path: ${currentFilepath}

  New path: ${newFilepath}
  `);
    if (promptResponse === 'Cancel') {
      logDebug(pluginJson$4, `renameNoteToTitle(): User cancelled`);
      return false;
    } else if (promptResponse === 'Yes') {
      const newFilename = note.rename(newFilepath);
      logDebug(pluginJson$4, `renameNoteToTitle(): ${currentFilepath} -> ${newFilename}`);
      // $FlowIgnore[incompatible-type]
      await showMessage(`Renamed note ${title} to ${newFilename}.`);
    } else {
      logDebug(pluginJson$4, 'renameNoteToTitle(): User chose not to rename.');
    }
    return true;
  }
  function isValidFilename(path) {
    // Check for invalid characters in filename
    const invalidChars = /[<>:"/\\|?*]/g;
    if (path.split('/').pop().match(invalidChars)) {
      return false;
    }
    return true;
  }

  /**
   * Renames the current note to match its title.
   * @returns void
   */
  async function titleToFilename() {
    try {
      const {
        note
      } = Editor;
      if (note) {
        await renameNoteToTitle(note);
      }
    } catch (error) {
      logError(error);
    }
  }

  async function makeNoteTitleMatchFilename(note, shouldPromptBeforeRenaming = true) {
    const {
      defaultFileExtension
    } = DataStore;
    if (note == null || note.paragraphs.length < 1) {
      // No note open, so don't do anything.
      logDebug(pluginJson$4, 'No note open, or no content. Stopping.');
      return false;
    }
    if (note.type === 'Calendar') {
      // Won't work on calendar notes
      await showMessage('Sorry: calendar notes cannot be renamed.');
      return false;
    }
    const currentTitle = note.paragraphs[0]?.content ?? '';
    const currentFullPath = note.filename;
    const currentFilename = currentFullPath.split('/').pop();
    const newTitle = currentFilename.replace(`.${defaultFileExtension}`, '');
    if (newTitle === currentTitle) {
      // No need to rename
      logDebug(pluginJson$4, 'makeNoteTitleMatchFilename(): Current title is the same as the filename. Stopping.');
      await showMessage('The note title is already consistent with the filename.');
      return false;
    }
    if (!shouldPromptBeforeRenaming) {
      note.removeParagraphAtIndex(0);
      note.insertHeading(newTitle, 0, 1);
      logDebug(pluginJson$4, `makeNoteTitleMatchFilename(): ${currentTitle} -> ${newTitle}`);
      return true;
    }
    const promptResponse = await showMessageYesNoCancel(`
  Would you like to change the note title "${currentTitle}" to match the filename "${newTitle}"?
  `);
    if (promptResponse === 'Cancel') {
      logDebug(pluginJson$4, `makeNoteTitleMatchFilename(): User cancelled`);
      return false;
    } else if (promptResponse === 'Yes') {
      note.removeParagraphAtIndex(0);
      note.insertHeading(newTitle, 0, 1);
      logDebug(pluginJson$4, `makeNoteTitleMatchFilename(): ${currentTitle} -> ${newTitle}`);
      await showMessage(`Changed note title from ${currentTitle} to ${newTitle}.`);
    } else {
      logDebug(pluginJson$4, 'makeNoteTitleMatchFilename(): User chose not to rename.');
    }
    return true;
  }

  /**
   * Renames the current note to match its title.
   * @returns void
   */
  async function filenameToTitle() {
    try {
      const {
        note
      } = Editor;
      if (note) {
        await makeNoteTitleMatchFilename(note);
      }
    } catch (error) {
      logError(error);
    }
  }

  //-----------------------------------------------------------------------------
  // Functions to identify and fix where note names and their filenames are inconsistent.
  // Leo Melo, readied for the plugin by Jonathan Clark
  // Last updated 15.8.2023 for v0.18.1 by @jgclark
  //-----------------------------------------------------------------------------
  // TODO:
  // - add a 'foldersToIgnore' option.

  /**
   * Renames all project notes with inconsistent names (i.e. where the note title and filename are different).
   * Optionally prompts the user before renaming each note.
   *
   * @returns void
   */
  async function renameInconsistentNames() {
    const directory = await chooseFolder('Choose a folder to rename notes in');
    if (!directory) {
      logWarn(pluginJson$4, 'renameInconsistentNames(): No folder chosen. Stopping.');
      return;
    }
    logDebug(pluginJson$4, `renameInconsistentNames(): Chosen folder: ${directory}`);
    try {
      const inconsistentNames = findInconsistentNames(directory);
      if (!Array.isArray(inconsistentNames) || inconsistentNames.length < 1) {
        logDebug(pluginJson$4, 'renameInconsistentNames(): No inconsistent names found. Stopping.');
        showMessage('No inconsistent names found. Well done!');
        return;
      }
      await showMessage(`Found ${inconsistentNames.length} inconsistent names.
      ${inconsistentNames.map(note => note.filename).join('\n')}
    `);
      const response = await showMessageYesNoCancel(`Would you like to be prompted before renaming each of these note filenames?
      (If you choose 'No', the notes will be renamed automatically. 'Cancel' will stop the process.)`);
      if (response === 'Cancel') {
        logDebug(pluginJson$4, 'renameInconsistentNames(): User chose to cancel.');
        return;
      }
      const shouldPromptBeforeRenaming = response === 'Yes';
      for (const note of inconsistentNames) {
        const shouldContinue = await renameNoteToTitle(note, shouldPromptBeforeRenaming);
        if (!shouldContinue) {
          logDebug(pluginJson$4, 'renameInconsistentNames(): User chose to cancel.');
          return;
        }
      }
    } catch (error) {
      logError(pluginJson$4, `renameInconsistentNames() error: ${error.message}`);
    }
  }

  //-----------------------------------------------------------------------------
  // @dwertheimer based on @jgclark's newNote
  // Create new note from currently selected text
  // and (optionally) leave backlink to it where selection was
  // Note: this was originally in Filer plugin
  // Last updated 2024-12-26 for 1.0.0, @jgclark (originallu)
  //-----------------------------------------------------------------------------

  /**
   * Create new (regular) note.
   * @author @jgclark
   */
  async function newNote() {
    try {
      // Get title for this note
      const title = await getInputTrimmed('Title of new note', 'OK', 'New Note from Clipboard', '');
      if (typeof title === 'string') {
        const currentFolder = await chooseFolder('Select folder to add note in:', false, true); // don't include @Archive as an option, but do allow creation of a new folder
        const content = `# ${title}\n`;
        if (title) {
          // Create new note in the specific folder
          const filename = (await DataStore.newNoteWithContent(content, currentFolder)) ?? '';
          logDebug('newNote', ` -> filename: ${filename}`);

          // Add frontmatter if required
          const config = await getSettings();
          if (config.defaultFrontmatter !== '') {
            // Add frontmatter to the note
            const newNote = await DataStore.noteByFilename(filename, 'Notes');
            if (newNote) {
              await addFrontmatterToNote(newNote);
            }
          }
          if ((await showMessageYesNo('New Note created. Open it now?', ['Yes', 'No'], `New Note`)) === 'Yes') {
            await Editor.openNoteByFilename(filename);
          }
        } else {
          logError('newNote', 'Undefined or empty title');
        }
      } else {
        logWarn('newNote', 'The user cancelled the operation.');
      }
    } catch (err) {
      logError(pluginJson$4, `newNote: ${err}`);
    }
  }

  /**
   * Create new note from the clipboard contents.
   * @author @jgclark
   */
  async function newNoteFromClipboard() {
    const {
      string
    } = Clipboard;
    if (string != null && string.length > 0) {
      logDebug(pluginJson$4, `newNoteFromClipboard() starting: have ${string.length} characters in clipboard`);

      // Get title for this note
      // Offer the first line to use, shorn of any leading # marks
      const firstLineMatches = string.match(/(?:---\n(?:title:\s*)?|[#\s]*)?(.*)\n/) ?? [];
      const titleToOffer = firstLineMatches[1] ?? '';
      let title = await getInput('Title of new note', 'OK', 'New Note from Clipboard', titleToOffer);
      if (typeof title === 'string') {
        const uniqueTitle = getUniqueNoteTitle(title);
        if (title !== uniqueTitle) {
          await showMessage(`  Title exists. Using "${uniqueTitle}" instead`, `OK`, `New Note from Clipboard`);
          title = uniqueTitle;
        }
        const currentFolder = await chooseFolder('Select folder to add note in:', false, true); // don't include @Archive as an option, but do allow creation of a new folder
        const content = `# ${title}\n${string}`;
        if (title) {
          // Create new note in the specific folder
          const filename = (await DataStore.newNoteWithContent(content, currentFolder)) ?? '';
          logDebug(pluginJson$4, ` -> filename: ${filename}`);
          if ((await showMessageYesNo('New Note created. Open it now?', ['Yes', 'No'], `New Note from Clipboard`)) === 'Yes') {
            await Editor.openNoteByFilename(filename);
          }
        } else {
          logError(pluginJson$4, 'Undefined or empty title');
        }
      } else {
        logWarn(pluginJson$4, 'The user cancelled the operation.');
      }
    } else {
      logWarn(pluginJson$4, 'The clipboard was empty, so nothing to do.');
      showMessage("The clipboard was empty, so there's nothing to do.", "OK, I'll try again", `New Note from Clipboard`);
    }
  }

  /**
   * Create new note from currently selected text and (optionally) leave backlink to it where selection was.
   * @author @dwertheimer + @jgclark
   */
  async function newNoteFromSelection() {
    const {
      selectedLinesText,
      selectedText,
      selectedParagraphs,
      note
    } = Editor;
    if (note != null && selectedLinesText.length && selectedText !== '') {
      logDebug(pluginJson$4, `newNoteFromSelection() starting with ${selectedParagraphs.length} selected:`);
      const selectedLinesTextToMutate = selectedLinesText.slice(); // copy that we can change

      // Get title for this note
      const isTextContent = ['title', 'text'].indexOf(selectedParagraphs[0].type) >= 0;
      const strippedFirstLine = selectedParagraphs[0].content;
      let title = await getInput('Title of new note', 'OK', 'New Note from Selection', strippedFirstLine);
      if (typeof title === 'string') {
        // If user selected the first line, then remove it from the body content
        if (title === strippedFirstLine && isTextContent) {
          selectedLinesTextToMutate.shift();
        }
        const movedText = selectedLinesTextToMutate.join('\n');
        const uniqueTitle = getUniqueNoteTitle(title);
        if (title !== uniqueTitle) {
          await showMessage(`Title exists. Using "${uniqueTitle}" instead`, `OK`, `New Note from Selection`);
          title = uniqueTitle;
        }
        const currentFolder = await chooseFolder('Select folder to add note in:', false, true); // don't include @Archive as an option, but do allow creation of a new folder

        if (title) {
          // Create new note in the specific folder
          const origFile = displayTitle$1(note); // Calendar notes have no title, so need to make one
          logDebug(pluginJson$4, `- origFile: ${origFile}`);
          const filename = (await DataStore.newNote(title, currentFolder)) ?? '';
          logDebug(pluginJson$4, `- newNote() -> filename: ${filename}`);

          // This question needs to be here after newNote and before noteOpener to force a cache refresh after newNote.
          // Note: This API bug has probably now been fixed.
          const res = await CommandBar.showOptions(['Yes', 'No'], 'Insert link to new file where selection was?');
          const newNote = await noteOpener(filename, 'using filename');
          if (newNote) {
            logDebug(pluginJson$4, `- newNote's title: ${String(newNote.title)}`);
            logDebug(pluginJson$4, `- newNote's content: ${String(newNote.content)} ...`);
            const insertBackLink = res.index === 0;
            // $FlowFixMe[method-unbinding] - Flow thinks the function is being removed from the object, but it's not
            if (Editor.replaceSelectionWithText) {
              // for compatibility, make sure the function exists
              if (insertBackLink) {
                Editor.replaceSelectionWithText(`[[${title}]]`);
              } else {
                Editor.replaceSelectionWithText(``);
              }
            }
            newNote.appendParagraph(movedText, 'empty');
            if (insertBackLink) {
              newNote.appendParagraph(`^ Moved from [[${origFile}]]:`, 'text');
            }
            if ((await showMessageYesNo('New Note created. Open it now?', ['Yes', 'No'], `New Note from Selection`)) === 'Yes') {
              await Editor.openNoteByFilename(filename);
            }
          } else {
            logWarn(pluginJson$4, `Couldn't open new note: ${filename}`);
            showMessage(`Could not open new note ${filename}`, `OK`, `New Note from Selection`);
          }
        } else {
          logError(pluginJson$4, 'Undefined or empty title');
        }
      } else {
        logWarn(pluginJson$4, 'The user cancelled the operation.');
      }
    } else {
      logDebug(pluginJson$4, '- No text was selected, so nothing to do.');
      showMessage('No text was selected, so nothing to do.', "OK, I'll try again", `New Note from Selection`);
    }
  }

  /**
   * Processes a given URL and returns a LinkObject.
   * @author @dwertheimer
   *
   * @param {string} urlStr - The URL to process.
   * @param {?string} name - The name of the markdown link. If the URL is not a markdown link, this should be null.
   * @param {number} lineIndex - The index of the line the URL was found on.
   * @param {boolean} removeSubdomain - Whether to remove the subdomain (like www) from the URL or not.
   * @returns {LinkObject} The processed LinkObject.
   */
  function processURL(urlStr, name, lineIndex, removeSubdomain) {
    const parts = urlStr.split(/\/+/g);
    const domain = parts.length > 1 ? parts[1] : urlStr;
    const page = parts.slice(2).join('/').split('?')[0];
    const finalDomain = removeSubdomain ? domain.split('.').slice(1, -1).join('.') : domain.split('.').slice(0, -1).join('.');
    return {
      url: urlStr,
      name: name,
      type: name ? 'markdown' : 'bareURL',
      lineIndex: lineIndex,
      domain: finalDomain,
      page: page
    };
  }

  /**
   * Scans multiple lines of text for URLs and returns an array of LinkObjects.
   * @author @jgclark updated by @dwertheimer
   * @tests in jest file
   * @param {string} text - The text to scan for URLs.
   * @param {boolean} [removeSubdomain=false] - Whether to remove the subdomain (like www) from the URLs or not.
   * @returns {LinkObject[]} An array of LinkObjects.
   */
  function findURLsInText(text, removeSubdomain = false) {
    try {
      const markdownURLPattern = /\[([^\]]+)\]\(([^)]+)\)/g; // Match markdown URLs with or without 'http(s)://'
      const bareURLPattern = /(\w+:\/\/[^\s]+)/g;
      const lines = text.split('\n');
      const links = [];
      for (let i = 0; i < lines.length; i++) {
        let line = lines[i];
        let match;

        // Process markdown URLs first and replace them with placeholders in the line.
        while ((match = markdownURLPattern.exec(line)) !== null) {
          // $FlowIgnore[incompatible-use]
          links.push(processURL(match[2], match[1], i, removeSubdomain));
          // $FlowIgnore[incompatible-use]
          line = line.replace(match[0], 'MARKDOWN_LINK_PLACEHOLDER');
        }

        // Process bare URLs.
        while ((match = bareURLPattern.exec(line)) !== null) {
          // $FlowIgnore[incompatible-use]
          links.push(processURL(match[1], null, i, removeSubdomain));
        }
      }
      return links;
    } catch (err) {
      logError('findURLsInText', err.message);
      return [];
    }
  }

  /**
   * Scans a note for URLs and returns an array of LinkObjects.
   * @author @jgclark
   *
   * @param {TNote} note - The note to scan for URLs.
   * @param {boolean} [removeSubdomain=false] - Whether to remove the subdomain (like www) from the URLs or not.
   * @param {boolean} [searchOnlyActivePart=true] - Whether to search the note's archive (Done and Cancelled sections) as well?
   * @param {boolean} [ignoreCompletedItems=false] - Whether to ignore URLs in done/cancelled tasks/checklist items.
   * @returns {LinkObject[]} array of LinkObjects
   */
  function findURLsInNote(note, removeSubdomain = false, searchOnlyActivePart = true, ignoreCompletedItems = false) {
    try {
      const lastLineToSearch = searchOnlyActivePart ? findEndOfActivePartOfNote(note) : note.paragraphs.length - 1;
      let parasToSearch = note.paragraphs.filter(p => p.lineIndex <= lastLineToSearch);
      if (ignoreCompletedItems) {
        parasToSearch = parasToSearch.filter(p => !isClosed(p));
      }
      const textToSearch = parasToSearch.map(p => p.content).join('\n') ?? [];
      return findURLsInText(textToSearch, removeSubdomain);
    } catch (err) {
      logError('findURLsInNote', err.message);
      return [];
    }
  }

  //-----------------------------------------------------------------------------
  // Navigation functions for Note Helpers plugin for NotePlan
  // Jonathan Clark
  // Last updated 2.1.2024 for v0.19.0, @jgclark
  //-----------------------------------------------------------------------------

  //-----------------------------------------------------------------

  /**
   * Jumps the cursor to the heading of the current note that the user selects
   * @author @jgclark
   * @param {?string} heading to jump to
   */
  async function jumpToHeading(heading) {
    try {
      const {
        paragraphs,
        note
      } = Editor;
      if (note == null || paragraphs == null) {
        // No note open, or no content
        return;
      }
      const headingStr = heading ?? (await chooseHeading(note, false, false, true));
      // find out position of this heading, ready to set insertion point
      // (or 0 if it can't be found)
      const startPos = getParaFromContent(note, headingStr)?.contentRange?.start ?? 0;
      logDebug('noteHelpers / jumpToHeading', `for '${headingStr}' at position ${startPos} max ${String(note.content?.length)}`);
      Editor.select(startPos, 0);
    } catch (e) {
      logError('jumpToHeading()', e.message);
    }
  }

  /**
   * Jumps the cursor to the heading of the current note that the user selects
   * NB: need to update to allow this to work with sub-windows, when EM updates API
   * @author @jgclark
   */
  async function jumpToNoteHeading() {
    try {
      // first jump to the note of interest, then to the heading
      // const notesList = allNotesSortedByChanged()
      // const re = await CommandBar.showOptions(
      //   notesList.map((n) => displayTitle(n)),
      //   'Select note to jump to',
      // )
      // const note = notesList[re.index]
      const note = await chooseNote(true, true, [], 'Select note to jump to', false);

      // Open the note in the Editor
      if (note != null && note.title != null) {
        await Editor.openNoteByTitle(note.title);
      } else {
        logError("Couldn't open selected note");
        return;
      }

      // Now jump to the heading
      await jumpToHeading();
    } catch (e) {
      logError('jumpToNoteHeading()', e.message);
    }
  }

  /**
   * Jump cursor to the '## Done' heading in the current file
   * NB: need to update to allow this to work with sub-windows, when EM updates API
   * @author @jgclark
   */
  function jumpToDone() {
    try {
      const paras = Editor?.paragraphs;
      if (paras == null) {
        // No note open
        return;
      }

      // Find the 'Done' heading of interest from all the paragraphs
      const matches = paras.filter(p => p.headingLevel === 2).filter(q => q.content.startsWith('Done')); // startsWith copes with Done section being folded

      if (matches != null) {
        const startPos = matches[0].contentRange?.start ?? 0;
        logDebug('jumpToDone()', `Jumping to '## Done' at position ${startPos}`);
        // Editor.renderedSelect(startPos, 0) // sometimes doesn't work
        Editor.select(startPos, 0);

        // Earlier version
        // Editor.highlight(p)
      } else {
        logWarn('jumpToDone()', "Couldn't find a '## Done' section. Stopping.");
      }
    } catch (e) {
      logError('jumpToDone()', e.message);
    }
  }

  /**
   * Open a URL present in a note. The user is offered a list of all URLs found.
   * @author @jgclark
   */
  async function openURLFromANote() {
    try {
      const config = await getSettings();
      // first jump to the note of interest, then to the heading
      const notesList = allNotesSortedByChanged();
      const re = await CommandBar.showOptions(notesList.map(n => displayTitle$1(n)), 'Select note');
      const note = notesList[re.index];

      // Find all URLs in the Note
      const linkObjects = findURLsInNote(note, false, true, config.ignoreCompletedItems);
      // const linkObjects = findURLsInText(note.content ?? '', false)

      if (linkObjects.length === 0) {
        logWarn(pluginJson$4, `Sorry: I couldn't find any URLs in note '${displayTitle$1(note)}'`);
        await showMessage('Sorry: I couldn\'t find any URLs in note');
        return;
      }

      // Ask user to pick a URL to open
      const foundURLs = linkObjects.map(lo => lo.name ?? lo.url);
      clo(foundURLs);
      const res = await CommandBar.showOptions(foundURLs, 'Select URL to open');
      if (!res) {
        logInfo('openURLFromANote()', 'User cancelled operation');
        return;
      }

      // Now open it
      const chosenURL = linkObjects[res.index].url;
      logDebug('openURLFromANote()', `Opening URL '${chosenURL}'`);
      await NotePlan.openURL(chosenURL);
    } catch (e) {
      logError('openURLFromANote()', e.message);
    }
  }

  /**
   * Open current month calendar note in current Editor.
   */
  async function showMonth() {
    try {
      const res = await Editor.openNoteByDate(new Date(), false, 0, 0, false, "month");
      if (!res) {
        logWarn('showMonth', `Cannot show current month note for some reason`);
      } else {
        logDebug('showMonth', `Opened current month note ${displayTitle$1(res)}`);
      }
    } catch (err) {
      logError('showMonth()', err.message);
    }
  }

  /**
   * Open current quarter calendar note in current Editor.
   */
  async function showQuarter() {
    try {
      const res = await Editor.openNoteByDate(new Date(), false, 0, 0, false, "quarter");
      if (!res) {
        logWarn('showQuarter', `Cannot show current quarter note for some reason`);
      } else {
        logDebug('showQuarter', `Opened current quarter note ${displayTitle$1(res)}`);
      }
    } catch (err) {
      logError('showQuarter()', err.message);
    }
  }

  /**
   * Open current year calendar note in current Editor.
   */
  async function showYear() {
    try {
      const res = await Editor.openNoteByDate(new Date(), false, 0, 0, false, "year");
      if (!res) {
        logWarn('showYear', `Cannot show current year note for some reason`);
      } else {
        logDebug('showYear', `Opened current year note ${displayTitle$1(res)}`);
      }
    } catch (err) {
      logError('showYear()', err.message);
    }
  }

  /**
   * @author @aaronpoweruser
   */
  const CODE_BLOCK_PLACEHOLDER = '8ce08058-d387-4d3a-8043-4f3b7ef63eb7';
  const MARKDOWN_LINK_PLACEHOLDER = '975b7115-5568-4bc6-b6c8-6603350572ea';

  /**
   * Trigger the find unlinked notes command
   */
  async function triggerFindUnlinkedNotes() {
    await findUnlinkedNotesInCurrentNote();
  }

  /**
   * Find all unlinked notes in the current note and create [[links]] to them
   */
  async function findUnlinkedNotesInCurrentNote() {
    const currentNote = Editor.note;
    if (currentNote) {
      await findUnlinkedNotes([currentNote]);
    }
  }

  /**
   * Find all unlinked notes in all notes and create [[links]] to them
   */
  async function findUnlinkedNotesInAllNotes() {
    const runTime = new Date();
    CommandBar.showLoading(true, 'Finding unlinked notes');
    const allNotes = [...DataStore.projectNotes, ...DataStore.calendarNotes];
    const foundLinks = await findUnlinkedNotes(allNotes);
    CommandBar.showLoading(false);
    logInfo(`Found ${foundLinks} unlinked notes in all notes, took: ${timer(runTime)}`);
  }

  /**
   * Find all unlinked notes in the given notes
   * @param {Array<TNote>} notes - the notes to search for unlinked notes in
   * @returns {number} - the number of unlinked notes found
   **/
  async function findUnlinkedNotes(notes) {
    let foundLinks = 0;
    try {
      await CommandBar.onAsyncThread();
      const noteTitlesSortedByLength = getAllNoteTitlesSortedByLength();
      foundLinks = notes.reduce((count, note) => count + findUnlinkedNotesInNote(note, noteTitlesSortedByLength), 0);
      await CommandBar.onMainThread();
    } catch (error) {
      logError(pluginJson$4, JSP(error));
    }
    return foundLinks;
  }

  /**
   * Find all note titles in the current note and replace them with [[note title]]
   * @param {TNote} currentNote - the note to search for links in
   * @param {Array<string>} noteTitlesSortedByLength - the note titles sorted by length
   * @returns {number} - the number of links found
   */
  function findUnlinkedNotesInNote(currentNote, noteTitlesSortedByLength = getAllNoteTitlesSortedByLength()) {
    let foundLinks = 0;
    const overallTime = new Date();
    let content = currentNote.content ?? '';
    logDebug(`Searching for unlinked notes in: ${currentNote.title ?? ''}`);
    const [contentWithCodeBlocksRemoved, codeBlockTracker] = extractCodeBlocks(content);
    content = contentWithCodeBlocksRemoved;
    const [contentWithLinksRemoved, markdownLinkTracker] = extractMarkdownLinks(content);
    content = contentWithLinksRemoved;
    noteTitlesSortedByLength.forEach(note => {
      if (currentNote.title !== note && content.includes(note)) {
        content = content.replaceAll(buildRegex(note), _ => {
          logDebug(`Found link to: ${note}`);
          foundLinks++;
          return `[[${note}]]`;
        });
      }
    });
    content = replaceCodeBlocks(content, codeBlockTracker);
    content = replaceMarkdownLinks(content, markdownLinkTracker);
    if (foundLinks > 0) {
      const startTime = new Date();
      currentNote.content = content;
      logDebug(`Updated note took: ${timer(startTime)}`);
    }
    logInfo(`Linked ${foundLinks} notes in ${currentNote.title ?? ''}, took: ${timer(overallTime)}`);
    return foundLinks;
  }

  /**
   * Builds a regular expression to match a specific note title within a text.
   *
   * @param {string} noteTitle - The title of the note to be matched.
   * @returns {RegExp} - A regular expression object for matching the note title in a given text.
   *
   * The regular expression is constructed dynamically to ensure that the note title:
   * - Is surrounded by word boundaries or specific punctuation marks.
   * - Does not appear within square brackets (like [[this]]).
   * - Is case-insensitive and performs a global search (flags 'gi').
   *
   * Breakdown of the regex pattern:
   * `w*(?<=[\\s,.:;"'])|^)`: Matches any leading whitespace or specific punctuation characters before the note title.
   * `(${sanitizeForRegex(noteTitle)})`: The sanitized note title to be matched.
   * `(?![^[\\]]{2})1: Negative lookahead to ensure the note title is  followed by two closing square brackets (]]).
   */
  function buildRegex(noteTitle) {
    return new RegExp(`(w*(?<=[\\s,.:;"'])|^)(${sanitizeForRegex(noteTitle)})(?![^[\]]{2})`, 'gi');
  }

  /**
   * Extracts code blocks from the content and replaces them with a placeholder
   * @param {string} content - the content to extract code blocks from
   * @returns {[string, Array<string>]} - the content with code blocks replaced and the code blocks
   */
  function extractMarkdownLinks(content) {
    const markdownLinkTracker = [];
    const startTime = new Date();
    const filteredContent = (content ?? '').replaceAll(/\[([^\]]+)\]\(([^)]+)\)/gim, match => {
      markdownLinkTracker.push(match);
      return MARKDOWN_LINK_PLACEHOLDER;
    });
    logDebug(`Replaced ${markdownLinkTracker.length} markdown links, took: ${timer(startTime)}`);
    return [filteredContent, markdownLinkTracker];
  }

  /**
   * Extracts code blocks from the content and replaces them with a placeholder
   * @param {string} content - the content to extract code blocks from
   * @returns {[string, Array<string>]} - the content with code blocks replaced and the code blocks
   */
  function extractCodeBlocks(content) {
    const codeBlockTracker = [];
    const startTime = new Date();
    const filteredContent = (content ?? '').replaceAll(/```([^`]|\n)*```/gim, match => {
      codeBlockTracker.push(match);
      return CODE_BLOCK_PLACEHOLDER;
    });
    logDebug(`Replaced ${codeBlockTracker.length} code blocks, took: ${timer(startTime)}`);
    return [filteredContent, codeBlockTracker];
  }

  /**
   * Replaces code blocks in the content with the original code blocks
   * @param {string} content - the content to replace code blocks in
   * @param {Array<string>} codeblockReversalTracker - the code blocks to replace
   * @returns {string} - the content with code blocks replaced
   */
  function replaceCodeBlocks(content, codeblockReversalTracker) {
    let contentWithCodeBlocks = content;
    codeblockReversalTracker.forEach(value => {
      contentWithCodeBlocks = contentWithCodeBlocks.replace(CODE_BLOCK_PLACEHOLDER, value);
    });
    return contentWithCodeBlocks;
  }

  /**
   * Replaces markdown links in the content with the original markdown links
   * @param {string} content - the content to replace markdown links in
   * @param {Array<string>} markdownLinkTracker - the markdown links to replace
   * @returns {string} - the content with markdown links replaced
   */
  function replaceMarkdownLinks(content, markdownLinkTracker) {
    let contentWithMarkdownLinks = content;
    markdownLinkTracker.forEach(value => {
      contentWithMarkdownLinks = contentWithMarkdownLinks.replace(MARKDOWN_LINK_PLACEHOLDER, value);
    });
    return contentWithMarkdownLinks;
  }

  /**
   * Get all note titles in the project notes sorted by length
   * @returns {Array<string>} - an array of all note titles
   */
  function getAllNoteTitlesSortedByLength() {
    return DataStore.projectNotes.filter(note => note.title !== null && note.title !== '').map(note => note.title ?? '').sort((a, b) => (b.length ?? 0) - (a.length ?? 0)); // sort by length to match longer titles first
  }

  /**
   * Returns a string with regex special characters escaped
   * @param {string} s - the string to parse
   * @returns {string} - the parsed string
   * */
  const sanitizeForRegex = s => {
    return s.replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&');
  };

  // -----------------------------------------------------------------------------
  // Note Helpers plugin for NotePlan
  // Jonathan Clark & Eduard Metzger
  // Last updated 2024-12-26 for v1.0.0 by @jgclark
  // -----------------------------------------------------------------------------
  // allow changes in plugin.json to trigger recompilation
  function resetCaches() {
    NotePlan.resetCaches();
  }
  const configKey = 'noteHelpers';
  function init() {
    // In the background, see if there is an update to the plugin to install, and if so let user know
    DataStore.installOrUpdatePluginsByID([pluginJson$4['plugin.id']], false, false, false);
  }
  async function onSettingsUpdated() {
    // Placeholder only to stop error in logs
  }

  /**
   * Executes when the plugin is updated or installed.
   */
  async function onUpdateOrInstall() {
    try {
      logDebug(pluginJson$4, `${configKey}: onUpdateOrInstall running`);

      // Tell user the plugin has been updated
      if (pluginJson$4['plugin.lastUpdateInfo'] !== undefined) {
        await showMessage(pluginJson$4['plugin.lastUpdateInfo'], 'OK, thanks', `Plugin ${pluginJson$4['plugin.name']}\nupdated to v${pluginJson$4['plugin.version']}`);
      }
    } catch (error) {
      logError(pluginJson$4, error);
    }
    logDebug(pluginJson$4, `${configKey}: onUpdateOrInstall finished`);
  }

  /**
   * Update Settings/Preferences (for iOS/iPadOS)
   * Plugin entrypoint for command: "/<plugin>: Update Plugin Settings/Preferences"
   * @author @dwertheimer
   */
  async function updateSettings() {
    try {
      logDebug(pluginJson$4, `updateSettings running`);
      await editSettings(pluginJson$4);
    } catch (error) {
      logError(pluginJson$4, JSP(error));
    }
  }

  exports.addFrontmatterToNote = addFrontmatterToNote;
  exports.addTriggerToNote = addTriggerToNote;
  exports.convertLocalLinksToPluginLinks = convertLocalLinksToPluginLinks;
  exports.countAndAddDays = countAndAddDays;
  exports.filenameToTitle = filenameToTitle;
  exports.findUnlinkedNotesInAllNotes = findUnlinkedNotesInAllNotes;
  exports.findUnlinkedNotesInCurrentNote = findUnlinkedNotesInCurrentNote;
  exports.indexFolders = indexFolders;
  exports.init = init;
  exports.jumpToDone = jumpToDone;
  exports.jumpToHeading = jumpToHeading;
  exports.jumpToNoteHeading = jumpToNoteHeading;
  exports.listInconsistentNames = listInconsistentNames;
  exports.logEditorNoteDetailed = logEditorNoteDetailed;
  exports.moveNote = moveNote;
  exports.newNote = newNote;
  exports.newNoteFromClipboard = newNoteFromClipboard;
  exports.newNoteFromSelection = newNoteFromSelection;
  exports.onSettingsUpdated = onSettingsUpdated;
  exports.onUpdateOrInstall = onUpdateOrInstall;
  exports.openURLFromANote = openURLFromANote;
  exports.printNote = printNote;
  exports.renameInconsistentNames = renameInconsistentNames;
  exports.renameNoteFile = renameNoteFile;
  exports.resetCaches = resetCaches;
  exports.showMonth = showMonth;
  exports.showQuarter = showQuarter;
  exports.showYear = showYear;
  exports.titleToFilename = titleToFilename;
  exports.trashNote = trashNote;
  exports.triggerFindUnlinkedNotes = triggerFindUnlinkedNotes;
  exports.updateAllIndexes = updateAllIndexes;
  exports.updateSettings = updateSettings;

  return exports;

})({});
Object.assign(typeof(globalThis) == "undefined" ? this : globalThis, exports)
